<?php
namespace Neuron\Attendance;

use Neuron\Generic\Entity\Map;
use Neuron\Generic\Result;
use Neuron\Attendance\Detail;
use Neuron\Attendance\Details;
use Neuron\Application\Framework as NAF;
use Neuron\Attendance\Traits\Mapper;

class Attendance extends \Neuron\Generic\Entity\Mapper {
    use Mapper;

    protected $_storage;

    const REPORTING_TYPE_ATTENDANCE = 1;
    const REPORTING_TYPE_OVERTIME   = 2;
    const REPORTING_TYPE_ABSENT   = 3;
    
    public function __construct(\Neuron\Attendance\Attendance\Storage\Skeleton $storage, $data = array()) {
        
        $this->_storage = $storage;
        
        parent::__construct($data, array(
            new Map('id'),
            new Map('type_id'),
            new Map('start_dtm'),
            new Map('start_description'),
            new Map('end_dtm'),
            new Map('end_description'),
            new Map('ref_code'),
            new Map('ref_id'),
            new Map('source_type_code'),
            new Map('source_reference'),
            new Map('source_dtm'),
            new Map('overtime_start_dtm'),
            new Map('overtime_end_dtm'),
            new Map('longitude'),
            new Map('latitude'),
            new Map('geocode_location',null,true)
        ), parent::CASE_INSENSITIVE);
    }
    public function listAttendance($params) {
        $result = $this->_storage->listAttendance($params, $this);
        $draw = (isset($params['draw']) ? $params['draw'] : null);

        $rows       = $result['data'];
        
        $newRows    = []; 
        foreach($rows as $row){
            /* ambil attd_details */
            $listDetail = [];
            $details    = new Details(Detail\Storage::factory(
                $this->_storage->getAdapter(),$this->_storage->getConfig()
            ));
            $details->load($row['id'],Detail\Storage::LISTBY_ATTENDANCE_ID);
            foreach($details as $detail){
                $listDetail[] = $detail->pull();
            }
            $row['attd_details'] = $listDetail;



            /* Ambil evidence */
            $evidence           = new Evidence(Evidence\Storage::factory(
                $this->_storage->getAdapter(),
                $this->_storage->getConfig()
            ));
            $evidence->load($row['id'],Evidence\Storage::LOADBY_ATTENDANCE);
            if($evidence->attd_evidence_id){
                $dataEvidence               = $evidence->pull();
                $row['attd_evidence']       = $dataEvidence;
            }
            
            $newRows[]          = $row;
        }

        $retval = array(
            'draw' => $draw,
            'recordsTotal' => $result['total'],
            'recordsFiltered' => $result['filtered'],
            'data' => $newRows
        );
        return $retval;
    }
    public function save($update = false) {
        $cek = $this->checkAttendance($this->type_id, $this->ref_id, $this->start_dtm);
        
        if (!$cek || $update) {
            if ($id = $this->_storage->save($this, $update)) {
                $this->id = $id;
                return array(
                    'success' => true,
                    'message' => 'success save attendance'
                );
            } else {
                return array(
                    'success' => false,
                    'message' => 'failed save attendance'
                );
            }
        } else {
            return array(
                'success' => false,
                'message' => 'Sorry, your attendance already recorded'
            );
        }
    }
    public function delete($id) {
        return $this->_storage->remove($id);
    }
    public function checkAttendance($type_id, $ref_id, $start_dtm) {
        $start    = explode(" ", $start_dtm);
        $arrAttd  = array();
        $dataAttd = $this->_storage->checkAttendance($type_id, $ref_id, date('Y-m-d', strtotime($start[0])));
        if ($dataAttd) {
            foreach ($dataAttd as $val) {
                array_push($arrAttd, $val);
            }
        }
        return $arrAttd;
    }
    public function load($attdId) {
        return $this->_storage->load($attdId);
    }
    public function loadList($id = null, $mode = \Neuron\Attendance\Attendance\Storage::LIST_ALL, $params = array(), $timeMode = \Neuron\Attendance\Attendance\Storage::TIME_WEEKLY) {
        $attendance = $this->_storage->loadList($id, $mode, $params, $timeMode);
        
        $result = new Result;
        if ($attendance) {
            $data = array();
            foreach ($attendance as $rows) {
                if($mode != \Neuron\Attendance\Attendance\Storage::COUNT_HOUR){
                    $sDetail = \Neuron\Attendance\Detail\Storage::factory($this->_storage->getAdapter(), $this->_storage->getConfig());
                    $mDetail= new \Neuron\Attendance\Detail($sDetail);
                    $mDetail->load($rows['id'], \Neuron\Attendance\Detail\Storage::LOADBY_ATTENDANCE);
                    $rows['geocode_location'] = $mDetail->geocode_location;

                    $sEvidence = \Neuron\Attendance\Evidence\Storage::factory($this->_storage->getAdapter(), $this->_storage->getConfig());
                    $mEvidence= new \Neuron\Attendance\Evidence($sEvidence);
                    $mEvidence->load($rows['id'], \Neuron\Attendance\Evidence\Storage::LOADBY_ATTENDANCE);
                    $rows['evidence'] = $mEvidence->filename;
                }
                
                $data[] = $rows;
            }
            $result->data = $data;
        } else {
            $result->code = 501;
            $result->info = 'Failed to load attendance';
        }
        return $result;
    }  
    public function countAttendance($id = null, $mode = \Neuron\Attendance\Attendance\Storage::LIST_ALL, $params = array(), $timeMode = \Neuron\Attendance\Attendance\Storage::TIME_TODAY) {
        
        $attendance = $this->_storage->countAttendance($id, $mode, $params, $timeMode);
        
        $result = new Result;
        if ($attendance) {
            $result->data = $attendance['countAttendance'];
        } else {
            $result->code = 501;
            $result->info = 'Failed to load attendance';
            $result->data = 0;
        }
        return $result;
    }
    public function countNotAttend($start_date,$end_date){
        $result             = new Result();
        $data               = $this->_storage->countNotAttend($start_date,$end_date);
        if($data){
            $list           = array();
            foreach($data as $data){
                $list[]     = $data;
            }

            $result->data = array('list' => $list,'total' => count($list));
        }else{
            $result->data   = array('list' => array(),'total' => 0);
        }

        return $result;
    }
    public function fetchLeaveData($timeMode) {
        $result = new Result;
        $fetch  = $this->_storage->fetchLeaveData($timeMode);
        
        $leaveData = array();
        if ($fetch) {
            foreach ($fetch as $data) {
                $leaveData[$data['description']] = $data['totalCount'];
            }
        }
        $result->data = $leaveData;
        
        return $result;
    }
    /* Pengganti save() karena sudah digunakan */
    public function directSave($update = FALSE){
        $save       = $this->_storage->directSave($this,$update);
        if($save){
            return true;
            $this->id   = $save;
        }else{
            return false;
        }
    }
    public function loadRow($id = null, $mode = Attendance\Storage::LOADBY_ID,$params = array()){
        $result = new Result();
        $data   = $this->_storage->loadRow($id,$mode,$params);
        if($data){
            $this->push($data);
            $result->data = $data;
        }else{
            $result->data = array();
        }
        
        return $result;
    }
    public function todayStatus($params = array()){
        $result     = new Result();
        if(!isset($params['start_dtm']) || empty($params['start_dtm'])){
            $params['start_dtm'] = date('Y-m-d');
        }

        $startDate          = date('Y-m-d');
        $listAttendance     = $this->_storage->loadList(null,Attendance\Storage::LISTBY_START_DATE,$params,Attendance\Storage::TIME_NONE);
        $attendances        = array();
        foreach ($listAttendance as $data ) {
            $attendances[]  = $data;
        }

        $persons            = array();
        $employee           = new \Neuron\Employee\Entity(\Neuron\Employee\Entity\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
        $persons            = $employee->listEmployee(array());
        
        $status             = array();

        /* Loop all persons */
        foreach ($persons['data'] as $key => $person) {
            $person_id      = $person['id'];
            $person['attendance_status'] = 0;
            $person['attendance_data'] = array();

            /* Find in attendances */
            foreach ($attendances as $attendance ) {
                $ref_id     = $attendance['ref_id'];
                if($ref_id === $person_id){
                    $person['attendance_status'] = 1;
                    $person['attendance_data'] = $attendance;
                    break;
                }
            }

            $status[]       = $person;

        }

        $result->data   = $status;
        return $result;

    }
    public function convertBase64ToImage($photo = null, $path = null) {
        $status = false;
        $file = null;
        $message = 'File Empty';
        
        if (!empty($photo)) {
            $photo = str_replace('data:image/png;base64,', '', $photo);
            $photo = str_replace(' ', '+', $photo);
            $photo = str_replace('data:image/jpeg;base64,', '', $photo);
            $photo = str_replace('data:image/gif;base64,', '', $photo);
            $entry = base64_decode($photo);
            $image = imagecreatefromstring($entry);

            $fileName = time() . ".jpeg";
            $fileLoc = './public/assets/avatars/';
            $directory = $fileLoc . $fileName;

            if (!empty($path)) {
                if (file_exists($path)) {
                    unlink($path);
                }
            }

            $saveImage = imagejpeg($image, $directory);

            imagedestroy($image);

            if ($saveImage) {
                $status = true;
                $file = $fileName;
                $message = 'File saved!';
            } else {
                $status = false;
                $file = null;
                $message = 'File not saved!';
            }
        }

        return array('status' => $status, 'file' => $file, 'message' => $message);
    }
    public function countReporting($type,$start_date,$end_date,$filter = array()){
        $result         = new Result();
        $data = $this->_storage->countReporting($type,$start_date,$end_date,$filter);
        if($data){
            $totalReporting     = $data['countTotal'];
        }else{
            $totalReporting     = 0;
        }
        $result->data = array(
            'total'     => $totalReporting
        );
        
        return $result;
    }
	public function checkAttendanceRange($type_id, $ref_id, $start_dtm, $end_dtm) {
		$result   = new Result();
        $start    = explode(" ", $start_dtm);
        $end      = explode(" ", $end_dtm);
        $arrAttd  = array();
        if($dataAttd = $this->_storage->checkAttendanceRange($type_id, $ref_id, date('Y-m-d', strtotime($start[0])), date('Y-m-d', strtotime($end[0]))))																																					{
			$result->code = 1;
            $result->info = "Pegawai sudah melakukan absen masuk, silahkan melakukan reject untuk rekomendasi ini";
		}else{
			$result->code = 0;
            $result->info = "Data attendnace not found";
		}
        return $result;
    }
}