<?php

namespace Neuron\Attendance\Attendance\Storage;
use \Neuron\Attendance\Attendance;
use Zend\Db\Sql\Predicate\Expression;

class Mysql extends \Neuron\Db\Storage implements Skeleton {

	public function getTotalRows() {
		try {

			$select = $this->select()
							->columns(array('total' => $this->expression('FOUND_ROWS()')));
														 
			return $this->fetchValue($select);  

		} catch (\Exception $ex) {
			return false;
		}              
	}  
	public function listAttendance($params = null,$attdRequestType) {
        $searchable = false;
        $where = [];

        $select = $this->select()->from(array('t' => $this->_tables['attendance']));
        
        if(class_exists('Neuron\Employee\Entity')){
            $select->join(
                array('emp' => $this->_tables['employee']),
                'emp.id = t.ref_id'
            );

            $select->join(
                ['p' => $this->_tables['person']],
                'p.id = emp.person_id',
                array(
                    'ref_name' => new Expression("CONCAT(p.first_name,' ',p.last_name)")
                )
            );
        }
        if(isset($params['search']['value'])){
            if (!empty($params['search']['value'])) {
                $select->where->nest()
                    ->like($this->_('first_name', 'p.'), '%' . trim($params['search']['value']) . '%')
                    ->or
                    ->like($this->_('last_name', 'p.'), '%' . trim($params['search']['value']) . '%')
                ->unnest();
            }
        }

        if(isset($params['draw'])) {
            $i = $params['order'][0]['column'];
			$dir = $params['order'][0]['dir'];
			$select->order(array($params['columns'][$i]['data'] . " " . $dir));
	
			$offset = ($params->start/$params->length) * $params->length;
			$select->limit(intval($params->length));
			$select->offset(intval($offset));
        }

        if(isset($params['ref_id'])) {
            $searchable = true;
            $where['t.ref_id'] = $params['ref_id'];
        }

        if(isset($params['code'])) {
            $searchable = true;
            $where['emp.code'] = $params['code'];
        }

        if(isset($params['type_id'])) {
            $searchable = true;
            $where['t.type_id'] = $params['type_id'];
        }
        if(isset($params['chckdate'])) {
            $searchable = true;
            $where[] = new Expression('DATE(t.start_dtm) = "'.date("Y-m-d",strtotime($params['chckdate'])).'"');
            $where['t.type_id'] = 1;
        }
        if($params['start_dtm'] != null && $params['end_dtm'] != "") {
            $searchable = true;
            $where[] = new Expression("(DATE(t.start_dtm) BETWEEN '". $params['start_dtm']."' AND '". $params['end_dtm']."')");
        }
        $select->where($where);
        
        $data = $this->fetchAll($select);
		$dataAll = [];
		if($data) {
            $count = 0;
			foreach($data as $row) {
                $selectD = $this->select()
                ->columns(array('geocode_location'))
                ->from(array('d' => $this->_tables['attd_details']))
                ->where(array('attendance_id'=> $row['id']));
                $row['geocode_location'] = $this->fetchValue($selectD);

                $selectE = $this->select()
                ->columns(array('filename'))
                ->from(array('d' => $this->_tables['attd_evidence']))
                ->where(array('action_type' => 1,'attendance_id'=> $row['id']));
                $row['evidence'] = $this->fetchValue($selectE);

                array_push($dataAll, $row);
                $count++;
                
			}
        }
		
		// hide karena reset join column handling error saat count mysql 5.7.29 
		// $selectC = $select;
        // $selectC->columns(array('total_count' => new \Zend\Db\Sql\Expression('COUNT(*)')));
		// $dataCount = $this->fetchRow($selectC);
		
		$total = 0;
		if(isset($count)){
			$total = ($count> 0) ? $count:0;
		}
        
        $filtered = $total;
        
        $result = ['data' => $dataAll, 'total' => $total, 'filtered' => $filtered];
        return $result;
	}
	public function save(\Neuron\Attendance\Attendance $attd, $update = false){
		try{
            if ($update) {
                $update = $this->update()
                            ->table($this->_tables['attendance'])
                            ->set($this->__($attd->pull()))
                            ->where($this->__(array('id' => $attd->id)));

                // echo str_replace('"','',$update->getSqlString()).'<br />'; die();
                if ($this->execute($update)) {
                    return $attd->id;
                }
              
            } else {
                $insert = $this->insert()
                            ->into($this->_tables['attendance'])
                            ->values($this->__($attd->pull()));
                // echo str_replace('"','',$insert->getSqlString()).'<br />'; die();
                
                $save = $this->execute($insert);
                if ($save) {
                    $attd->id = $save;
                    return $attd->id;
                }
            }
        }catch(\Exception $e){
            return $e->getMessage();
        }
	}
	public function remove($id) {
        try {
            $delete = $this->delete()
                    ->from($this->_tables['attendance'])
                    ->where(array('id' => $id));
            return $this->execute($delete);
        }catch (\Exception $ex) {
            return false;
        }  
    }
    public function checkAttendance($type_id, $ref_id, $start_dtm){
        try {
            $select = $this->select()
                ->from(array('attd' => $this->_tables['attendance']))
                ->where(
                    array(
                        'type_id'=> $type_id,
                        'ref_id' => $ref_id, 
                        'DATE_FORMAT(start_dtm, "%Y-%m-%d") = \''.$start_dtm.'\''
                    )
                );
                // echo str_replace('"','',$select->getSqlString()).'<br />'; die();
            return $this->fetchAll($select);
        } catch (\Exception $e) {
            return false;
        }
    }	
    public function load($attdId){
        try {
            $select = $this->select()
                        ->from($this->_tables['attendance']) 
                        ->where(array('id'=>$attdId));
            return $this->fetchRow($select);
        } catch (Exception $e) {
            return false;
        }
    }
    public function loadList($id = null,$mode = Attendance\Storage::LIST_ALL,$params = array(),$timeMode = Attendance\Storage::TIME_WEEKLY){
        
        $select = $this->select()->from(array('attd' => $this->_tables['attendance']));
        $select->join(
            array('p' => $this->_tables['person']),
            'p.id = attd.ref_id',
            array()
        );
        if($mode != Attendance\Storage::LIST_ALL){
            switch ($mode) {
                case Attendance\Storage::LISTBY_REF_ID:
                    if(isset($params['type_id'])){
                        $select->where(array($this->_('type_id','attd.') => $params['type_id']));
                    }
                    $select->where(array('ref_id' => $params['ref_id']));
                    break;
                case Attendance\Storage::COUNT_HOUR:
                    if(isset($params['type_id'])){
                        $select->where(array($this->_('type_id','attd.') => $params['type_id']));
                    }
                    if(isset($params['year'])){
                        $select->where(
                            new Expression("YEAR(DATE(start_dtm)) = '".$params['year']."' ")
                        );
                    }
                    /* Untuk filter by organization */
                    if(class_exists('Neuron\Organization\Entity') && class_exists('Neuron\Employee\Entity')){
                        if(isset($params['organization_id']) && !empty($params['organization_id'])){
                            
                            /* Join ke employee jobs */
                            $select->join(
                                array('ej'    => $this->_tables['employee_jobs']),
                                'ej.employee_id = p.id',
                                array()
                            );
                            /* Join ke organiaztion has position */
                            $select->join(
                                array('ohp'     => $this->_tables['org_has_position']),
                                'ohp.id = ej.ohp_id',
                                array()
                            );
                            $select->where(array(
                                $this->_('organization_id','ohp.')  => $params['organization_id']
                            ));
                        }
                    }

                    $select->columns(array(
                            'month'         => new Expression('month(start_dtm)'),
                            'hour'          => new Expression('hour(timediff(start_dtm ,end_dtm ))'),
                            'minutes'          => new Expression('minute(timediff(start_dtm,end_dtm))')
                        )
                    );

                    // $select->group(new Expression('MONTH(start_dtm)'));
                    
                    break;
                case Attendance\Storage::LISTBY_START_DATE:
                    $select->where(
                        new Expression("DATE(start_dtm) = '".$params['start_dtm']."'")
                    );
                    break;

            }
        }

        if(isset($params['ref_name']) && !empty($params['ref_name'])){
            $nest = $select->where->nest();
            $nest->like(new Expression("CONCAT(p.first_name,' ',p.last_name)"),"%".trim(($params['ref_name'])."%"));
            $nest->or->like('code',"%".trim($params['ref_name'])."%");
			$nest->unnest();
        }

        $select->order(
            $this->_('start_dtm','attd.').' '.'desc'
        );

        // defining time mode 
        switch ($timeMode) {
            case Attendance\Storage::TIME_WEEKLY:
                $thisMonday = date('Y-m-d',strtotime('this week monday'));
                $thisFriday = date('Y-m-d',strtotime('this week friday'));
                $select->where->greaterThanOrEqualTo(new Expression('date(start_dtm)'),$thisMonday);
                $select->where->lessThanOrEqualTo(new Expression('date(start_dtm)'),$thisFriday);

                break;
            case Attendance\Storage::TIME_MONTHLY:
                $select->where(
                    "month(start_dtm) = '".date('m')."'"
                );
                $select->where(
                    'year(start_dtm) = "'.date('Y').'" '
                );
                break;
            case Attendance\Storage::TIME_YEARLY:
                $select->where(
                    "year(start_dtm) = '".date('Y')."'"
                );
                break;
            case Attendance\Storage::TIME_TODAY:
                $select->where(
                        'DATE(start_dtm) = "'.date('Y-m-d').'"'
                );
                break;
            case Attendance\Storage::TIME_NONE:
                break;
        }
        
        try {
            return $this->fetchAll($select);
        } catch (\Exception $e) {
            return false;
        }
    }
    public function countAttendance($id = null,$mode = Attendance\Storage::LIST_ALL,$params = array(),$timeMode = Attendance\Storage::TIME_WEEKLY){
        $select = $this->select()->from(array('attd' => $this->_tables['attendance']));
        $select->columns(array(
            'countAttendance'   => new \Zend\Db\Sql\Expression('COUNT(*)')
        ));
        if($mode != Attendance\Storage::LIST_ALL){
            switch ($mode) {
                case Attendance\Storage::LISTBY_REF_ID:
                    $select->where('ref_id',$params['ref_id']);
                    break;
                case Attendance\Storage::LISTBY_TYPE_REF_ID:
                    $select->where(array(
                        'ref_id' => $params['ref_id'],
                        'type_id' => $params['type_id']
                    ));
            }
        }

        

        // defining time mode 
        switch ($timeMode) {
            case Attendance\Storage::TIME_WEEKLY:
                //fetch data in this feek between monday & friday;
                    $thisMonday = date('Y-m-d',strtotime('this week monday'));
                    $thisFriday = date('Y-m-d',strtotime('this week friday'));
                $select->where(
                    "start_dtm >= '".$thisMonday."' AND start_dtm <= '".$thisFriday."' "
                );
                break;
            case Attendance\Storage::TIME_MONTHLY:
                $select->where(
                    "month(start_dtm) = '".date('m')."'"
                );
                break;
            case Attendance\Storage::TIME_YEARLY:
                $select->where(
                    "year(start_dtm) = '".date('Y')."'"
                );
                break;
            case Attendance\Storage::TIME_TODAY:
                $select->where(
                        'DATE(start_dtm) = "'.date('Y-m-d').'"'
                );
                break;
        }
        try {
            return $this->fetchRow($select);
        } catch (\Exception $e) {
            return false;
        }
    }
    public function fetchLeaveData($timeMode = Attendance\Storage::TIME_TODAY){
        $select         = $this->select();
        $select->from($this->_tables['leave']);
        $select->columns(
            array(
                'totalCount'        => new \Zend\Db\Sql\Expression('COUNT(*)'),
                'description'       => new \Zend\Db\Sql\Expression('lt.description'),
            )
        );

        $select->group(array('lt.code'));

        $select->join(
            array(
                'lt'=> $this->_tables['leave_type']),
                'lt.code = leave.type_code', 
                array(),
                'left'
        );
        
        switch ($timeMode) {
            case Attendance\Storage::TIME_WEEKLY:
                //fetch data in this feek between monday & friday;
                    $thisMonday = date('Y-m-d',strtotime('this week monday'));
                    $thisFriday = date('Y-m-d',strtotime('this week friday'));
                $select->where(
                    "start_dtm >= '".$thisMonday."' AND start_dtm <= '".$thisFriday."' "
                );
                break;
            case Attendance\Storage::TIME_MONTHLY:
                $select->where(
                    "month(start_dtm) = '".date('m')."'"
                );
                break;
            case Attendance\Storage::TIME_YEARLY:
                $select->where(
                    "year(start_dtm) = '".date('Y')."'"
                );
                break;
            case Attendance\Storage::TIME_TODAY:
                $select->where(
                        'DATE(start_dtm) = "'.date('Y-m-d').'"'
                );
                break;
        }

        try {
            // echo('$select=> '.str_replace('"','',$select->getSqlString()));die();
            $result         = $this->fetchAll($select);
        } catch (Exception $e) {
            $result         = false;
        }

        return $result;
    }
    public function directSave(Attendance $attendance, $update = FALSE){
        if($update){
            $update     = $this->update();
            $update->table($this->_tables['attendance']);
            $update->where(array($this->_('id') => $attendance->id));
            $update->set($this->__($attendance->pull()));

            try {
                $save   = $this->execute($update);
                if($save){
                    return $attendance->id;
                }else{
                    return false;
                }
            } catch (Exception $e) {
                error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
                return false;
            }
        }else{
            $attendance->id     = 0;
            $insert             = $this->insert();
            $insert->into($this->_tables['attendance']);
            $insert->values($this->__($attendance->pull()));

            try {
                
                $save = $this->execute($insert);
                return $save;
            } catch (Exception $e) {
                error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
                return false;
            }
        }
    }
    public function loadRow($id = null, $mode = Attendance\Storage::LOADBY_ID,$params = array()){
        $select         = $this->select();
        $select->from($this->_tables['attendance']);

        switch ($mode) {
            case Attendance\Storage::LOADBY_ID:
                $select->where(array('id' => $id));
                break;
            case Attendance\Storage::LOADBY_STARTDATE_REF_ID;
                $select->where("DATE(start_dtm) = '".date('Y-m-d',strtotime($params['start_dtm']))."'");
                $select->where(array('ref_id' => $params['ref_id']));
                break;
        }

        try {
            $data   = $this->fetchRow($select);
            return $data;
        } catch (Exception $e) {
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
            return false;
        }
    }
    public function countReporting($type,$start_date,$end_date,$filter){
        $select         = $this->select();
        $select->from($this->_tables['attendance']);
        $select->where(array($this->_('type_id','attendance.') => $type));
        $select->where->greaterThanOrEqualTo(new Expression("DATE(start_dtm)"),$start_date);
 
        $select->join(
            array('p' => $this->_tables['person']),
            'p.id = attendance.ref_id',
            array()
        );
        
        if(class_exists('Neuron\Organization\Entity') && class_exists('Neuron\Employee\Entity')){
            if(isset($filter['organization_id']) && !empty($filter['organization_id'])){
                /* Join ke employee jobs */
                $select = $select->join(
                    array('ej'    => $this->_tables['emp_jobs']),
                    'ej.employee_id = p.id',
                    array()
                );
                /* Join ke organiaztion has position */
                $select = $select->join(
                    array('ohp'     => $this->_tables['org_has_position']),
                    'ohp.id = ej.ohp_id',
                    array()
                );
                $select = $select->where(array(
                    $this->_('organization_id','ohp.')  => $filter['organization_id']
                ));
            }
        }

        if($filter['ref_name']){
            $select = $select->where(new Expression("concat(p.first_name,' ',p.last_name) LIKE '%{$filter['ref_name']}%' "));
        }
        
        $nest   = $select->where->nest();
        $nest->lessThanOrEqualTo(new Expression('DATE(end_dtm)'),$end_date)->or;
        $nest->isNull('end_dtm');
        $nest->unnest();

        $select->columns(array('countTotal' => new Expression("COUNT(*)")));
        
        try{
            $data   = $this->fetchRow($select);
            if($data){
                return $data;
            }else{
                return false;
            }
        }catch(\Exception $ex){
            error_log("ERROR ::". $ex->getMessage().' File '.$ex->getFile(),' Line : '.$ex->getLine());
            return false;
        }
    }

    /* untuk menghitung jumlah orang yang tidak hadir */
    public function countNotAttend($start_date,$end_date){
        $select         = $this->select();
        $select->from($this->__([
            'p'     => $this->_tables['person']
        ]));

		$select->join(array(
            'emp' => $this->_tables['employee']),
            'emp.person_id = p.id'
        );

        $select->where(new Expression("(emp.id NOT IN (
            SELECT
                attendance.ref_id AS ref_id
            FROM
                attendance
            WHERE
                DATE(start_dtm) ='".$start_date."')
			OR emp.id IN (
			SELECT
                attendance.ref_id AS ref_id
            FROM
                attendance
            WHERE
                DATE(start_dtm) ='".$start_date."'
				AND attendance.type_id = 3)
			)"));
        
        $select->where($this->__([
            'emp.status_id' => 1
        ]));
		
		// echo str_replace('"','',$select->getSqlString()).'<br />'; 
        
        if($data = $this->fetchAll($select)){
            return $data;
        }
        return false;
    }
	
	public function checkAttendanceRange($type_id, $ref_id, $start_dtm, $end_date){
        try {
            $select = $this->select()
                ->from(array('attd' => $this->_tables['attendance']))
                ->where(
                    array(
                        'type_id'=> $type_id,
                        'ref_id' => $ref_id, 
                        '(DATE(start_dtm) BETWEEN "'. $start_dtm.'" AND "'. $end_date.'")'
                    )
                );
                // echo str_replace('"','',$select->getSqlString()).'<br />'; die();
            return $this->fetchAll($select);
        } catch (\Exception $e) {
            return false;
        }
    }	
}