<?php
namespace Neuron\Attendance\Controller\Api;

use Neuron\Application\Framework\Entity;
use Neuron\Generic\Result;
use Neuron\Attendance\Request\Approval as ApprovalM;
use Neuron\Attendance\Request\Approvals;
use Neuron\Attendance\Request;
use Neuron\Addon\Order\SimpleFlow;
use Neuron\Application\Framework as NAF;

class Approval extends Controller{

    CONST CODE_SESSION_EXPIRED = 411;
    const ORDER_TYPE_ATTENDANCE = 2100;
    
    public function loadListAction(){
        $post       = $this->getREST()->getParameters();
        $guid       = $this->getREST()->getGuid();
        $result   = $this->validateGuid($guid);

        if ($result->code != 0) {
            return $this->getREST()->getResponse($result);
        }

        $user_id    = $result->data['user_id'];
        /* Load Map */
        if(class_exists('\Neuron\Employee\Entity')){
            $empStorage     = \Neuron\Employee\Entity\Storage::factory($this->getDb(),$this->getConfig());
            $emp            = new \Neuron\Employee\Entity($empStorage);
            $map            = $emp->loadMap($user_id,'user_id');
            if($map['data']){
                $ref_id     = $map['data']['person_id'];
            }else{
                $result->code   = 500;
                $result->info   = 'Failed loading map';
                return $this->getREST()->getResponse($result);
            }
        }else{
            /* Jika tidak ada module employee */
            $ref_id         = 1;
        }

        /* New */

            
            $params     = [
                'order_type_id'     => self::ORDER_TYPE_ATTENDANCE,
                'flow'              => "inbox",
                'order_status_id'   => $post['status_id'],
                'order_subtype_id'  => $post['request_type']
            ];

            $where['optional']['assigned_to_id'] = $user_id;
            $where['where']['order_subtype_id'] = $post['request_type'];
            $where['where']['order_type_id'] = self::ORDER_TYPE_ATTENDANCE;
            if($params['past'] == 1){
                $where['optional']['order_flow_status_id'] = "2,3";
            }else{
                $where['optional']['order_flow_status_id'] = 1;
            }
            $where['flow_new'] = 'inbox';

            if ($manager = NAF\Event\Manager::get()) {
                $manager->events()->register('order_query',array($this,'onSelectQuery'),0,3);
            }


            $entities = new \Neuron\Approval\Entities(\Neuron\Order\Entity\Storage::factory($this->getDb(),$this->getConfig()));
            $result = $entities->load(null,1, $where);


            $storageE = \Neuron\Employee\Entity\Storage::factory($this->getDb(), $this->getConfig());
            $m_employee = new \Neuron\Employee\Entity($storageE);
            $storageDetail = \Neuron\Employee\Detail\Storage::factory($this->getDb(), $this->getConfig());
            $detail = new \Neuron\Employee\Detail($storageDetail);

            $rows   = $result->data;
            $newRows = [];
            foreach($rows as $row){
                $createUserId   = $row['create_user_id'];
                /* LoadMap */
                // $personMap      = new \Neuron\Person\Map(\Neuron\Person\Map\Storage::factory(
                //     $this->getDb(),
                //     $this->getConfig()
                // ));
                // $personMap->load($createUserId,\Neuron\Person\Map\Storage::LOADBY_USER_ID);
                // $dataMap        = $personMap->pull();

                $dataRequester = $m_employee->loadMap($row['create_user_id'],'user_id');
                
                $detail->loadEmpDet($dataRequester['data']['person_id']);
                $detailRequester  = $detail->pull();

                $dataApproval = $m_employee->loadMap($row['assigned_to_id'],'user_id');
                $detail->loadEmpDet($dataApproval['data']['person_id']);
                $detailApproval  = $detail->pull();
                // $tempFlow       = $row['flow'][0];

                /* Load Employee */
                // $employee       = new \Neuron\Employee\Entity(
                //     \Neuron\Employee\Entity\Storage::factory(
                //         $this->getDb(),$this->getConfig()
                //     )
                // );
                // $employee->load($dataMap['person_id']);
                // $empDetail      = new \Neuron\Employee\Detail(
                //     \Neuron\Employee\Detail\Storage::factory(
                //         $this->getDb(),
                //         $this->getConfig()
                //     )  
                // );

                if($row['xn5'] == 1){
                    $order_status_name = 'Requested';
                }else if($row['xn5'] == 2){
                    $order_status_name = 'Approved';
                }else if($row['xn5'] == 3){
                    $order_status_name = 'Rejected';
                }else{
                    $order_status_name = 'Requested';
                }

                $naf     = \Neuron\Application\Framework\Entity::get();
                $baseURL = $naf->layout()->getHomeURL();
                // $empDetail->load($employee->id);
                $tempRow = [
                    "approval_id" => $row['order_id'],
                    "request_id" => $row['order_id'],
                    "ref_code" => "EMPLOYEE",
                    "ref_id" => $dataApproval['data']['person_id'],
                    "status_id" => $row['xn5'],
                    "approval_dtm" => ($row['update_dtm'] != '0000-00-00 00:00:00') ? $row['update_dtm']:null,
                    "reason" => $row['order_desc'],
                    "ref_name" => $dataApproval['data']['first_name'].' '.$dataApproval['data']['last_name'],
                    "request_type" => $row['order_subtype_id'],
                    "requester_ref_id" => $dataRequester['data']['person_id'],
                    "request_dtm" => $row['create_dtm'],
                    "requester_ref_name" => $dataRequester['data']['first_name'].' '.$dataRequester['data']['last_name'],
                    "avatar" => $baseURL . $detailRequester['pic_official'],
                    "request_type_name" => $row['order_subtype_name'],
                    "status_id_name" => $order_status_name,
                ];
                $newRows[] = $tempRow;
            }
        /* New */

        $result     = new \Neuron\Generic\Result;
        $result->info   = $this->translate($result->info);

        $resData = $result->data;
        $resData['rows'] = $newRows;
        $resData['countTotal'] = count($newRows);

        if($resData['countTotal']){
            $resData['countTotal'] = strval($resData['countTotal']);
        }
        $result->data = $resData;
        $result->guid = $guid;
		
		$requestPost = array(
			'guid' => $guid,
			'code' => '',
			'info' => '',
			'data' => ''
		); 

		$responseResult = array(
			'code' => $result->code,
			'info' => $result->info,
			'data' => $result->data
		); 

		$logparam['subject'] = 'API ATTENDANCE APPROVAL LOADLISTACTION';
		$logparam['identity'] = $user_id ? $user_id : '';
		$logparam['user'] = $user_id ? $user_id : '';
		$logparam['description'] = 'Log proses api attendance approval loadlistaction';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode($responseResult);

		$this->writeLogAction($logparam);
		
        return $this->getREST()->getResponse($result);
    }

    public function countApprovalAction(){
        $guid       = $this->getREST()->getGuid();
        $result     = $this->validateGuid($guid);

        if ($result->code != 0) {
            $result->guid = $guid;
            return $this->getREST()->getResponse($result);
        }
        
        $user_id    = $result->data['user_id'];
        /* Load Map */
        if (class_exists('\Neuron\Employee\Entity')) {
            $empStorage     = \Neuron\Employee\Entity\Storage::factory($this->getDb(), $this->getConfig());
            $emp            = new \Neuron\Employee\Entity($empStorage);
            $map            = $emp->loadMap($user_id, 'user_id');
            if ($map['data']) {
                $ref_id     = $map['data']['person_id'];
            } else {
                $result->code   = 500;
                $result->info   = 'Failed loading map';
                return $this->getREST()->getResponse($result);
            }
        } else {
            /* Jika tidak ada module employee */
            $ref_id         = 1;
        }
        
        $approvals   = new Approvals(ApprovalM\Storage::factory($this->getDb(),$this->getConfig()));
        $result     = $approvals->loadCountByType($ref_id,ApprovalM\Storage::LISTBY_REF_ID,array('ref_id' => $ref_id));


        $result->guid = $guid;
		
		$requestPost = array(
			'guid' => $guid,
			'code' => '',
			'info' => '',
			'data' => ''
		); 

		$responseResult = array(
			'code' => $result->code,
			'info' => $result->info,
			'data' => $result->data
		); 

		$logparam['subject'] = 'API ATTENDANCE APPROVAL COUNTAPPROVALACTION';
		$logparam['identity'] = $user_id ? $user_id : '';
		$logparam['user'] = $user_id ? $user_id : '';
		$logparam['description'] = 'Log proses api attendance approval countApprovalAction';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode($responseResult);

		$this->writeLogAction($logparam);
		
        return $this->getREST()->getREsponse($result);

    }

    /* Ini pake order */
    public function saveAction(){
        $guid           = $this->getREST()->getGuid();
        $validateGuid   = $this->validateGuid($guid);
        if($validateGuid->code != 0){
            return $this->getREST()->getResponse($validateGuid);
        }

        $sessData       = $validateGuid->data;

        $post       = $this->getREST()->getParameters();
        $orderId = $post['approval_id'];
        /* Load Flow */
        // $simpleFlowConfig   = \Neuron\Attendance\Generic::getAddonConfig();
        // $flows              = new SimpleFlow\Flows(SimpleFlow\Flow\Storage::factory($this->getDb(),$simpleFlowConfig));
        // $loadFlow           = $flows->load($approvalId,SimpleFlow\Flow\Storage::LISTBY_ID);
        // $flowData           = $loadFlow->data[0];
        // $orderId            = $flowData['order_id']; 

		$entity = new \Neuron\Approval\Entity(\Neuron\Order\Entity\Storage::factory($this->getDb(), $this->getConfig()));
		$param['order_id'] = $orderId;
		/* load and return */
		$result = $entity->loadEx($param, 3);
        $data = $result->data;
		
		print_r($data);die();
        
		//load data type
        $toOrder['user_id'] = $sessData['user_id'];
        $toOrder['order_id'] = $orderId;
        $toOrder['flow']['order_flow_status_id'] = intval($post['status_id']);
        $toOrder['flow']['remark_out'] = $post['reason'];
        $request   = new \Neuron\Approval\Entity(\Neuron\Order\Entity\Storage::factory($this->getDb(),$this->getConfig()));          
        $result    = $request->update(@$toOrder);
        
	    if($result->code == 0){
			$loadnewData = $entity->loadEx($data['order_id'], 3);
            $newData = $loadnewData->data;
            
			if($data['order_status_id'] != $newData['order_status_id']){
                //jika status berubah
				if($post['status_id'] == 2){
                    //jika approve
                    $template_code 	= 'REQUEST_APPROVE'; //template notif
                    
                    // Create attendance data. changeshift / overtime / attendance;
                    $requestType    = new \Neuron\Attendance\Request(\Neuron\Attendance\Request\Storage::factory(
                        $this->getDb(),
                        $this->getConfig()
                    ));
                    
                    $createRequestData = $requestType->createRequestTypeData($newData);
                    if($createRequestData->code != 0){
                        $result->guid = $guid;
                        $result->info   = $this->translate($createRequestData->info);
                        return $this->getREST()->getResponse($result);
                    }
				}else{
					//jika reject
					$template_code 	= 'REQUEST_REJECT'; //template notif
				}
				
				## NOTIFICATION HERE
				# Kirim notifikasi kalo request di approve / di reject.
				$inboxStorage   = \Neuron\Notification\Inbox\Storage::factory(
					$this->getDb(),
					$this->getConfig(),
					\Neuron\Notification\Profile::TYPE_FCM,
					$newData['create_user_id']
				);
				$inbox      = new  \Neuron\Notification\Inbox($inboxStorage);
				$inbox->setPublicUrl($this->getPublicUrl());

				# check terlebihdahuu apakah sudah ada tabelnya atau belum
				$res = @$inbox->pushNotification(
					// $userId,
					$newData['create_user_id'],
					$template_code ,
					array(
						'request_type' => $newData['order_type_name'],
					),
					array(
						'request_id' 	=> $newData['order_id'],
					)
				);
				
				$array = array(
                    "userId" => $sessData['user_id'],
                    "template_code" => $template_code,
                    "request_type" => $newData['order_status_name'],
                    "request_id" => $newData['order_id'],
				);
				
				if ($res->code == 0) {
					$savedNotif[]   = $sessData['user_id'];
				}
			}
        }

        $result->guid = $guid;
        $result->data = null;

        $requestPost = array(
			'guid' => $guid,
			'code' => '',
			'info' => '',
			'data' => ''
		); 

		$responseResult = array(
			'code' => $result->code,
			'info' => $result->info,
			'data' => $result->data
		); 

		$logparam['subject'] = 'API ATTENDANCE APPROVAL SAVEACTION';
		$logparam['identity'] = '';
		$logparam['user'] = '';
		$logparam['description'] = 'Log proses api attendance approval saveAction';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode($responseResult);

		$this->writeLogAction($logparam);
		
        return $this->getREST()->getResponse($result);

    }

    protected function validateGuid($guid){
        $result     = new Result;
        if(!$guid){
            $result->code   = 401;
            $result->info   = "Invalid token";
            return $result;
        }
        $entity = Entity::get($this, $guid);
        $user = $entity->user();
        if (!$user->id) {
            $result->code = self::CODE_SESSION_EXPIRED;
            $result->info = 'Session Expired';
        } else {
            $result->data   = $user->pull();
        }
        return $result;
    }

    public function onSelectQuery($query,$params,$caller){
        if($params['flow_new'] == 'inbox'){
            $query->join(
                array('of' => 'order_flow'),
                'o.order_id = of.order_id',
                array('*'),
                'left'
            )->where(array(
                'of.assigned_to_id' => $params['optional']['assigned_to_id'],
                'of.order_flow_type_id' => 1
            ));

            if(isset($params['optional']['order_flow_status_id']) && !empty($params['optional']['order_flow_status_id']) ){
                $query->where(array('of.order_flow_status_id IN ('. $params['optional']['order_flow_status_id'].')'));
            }
            // echo str_replace('"','',$query->getSqlString()).'<br />';  die();
            
        }
        return $query;
    }
}
