<?php
namespace Neuron\Attendance\Controller\Api;

use Neuron\Application\Framework\Entity;
use Neuron\Attendance\Attendance as AttendanceModel;
use Neuron\Generic\Result;
use Neuron\Application\Framework as NAF;


class Attendance extends \Neuron\Attendance\Controller\Api\Controller {
    const EVENT_SUBJECT = 'Attendance';
    const SUCCESS_CODE = 0;
    const FAIL_CODE = 1;
    const CODE_FIELD_REQUIRED  = 400;
    const CODE_UNAUTHORIZED = 401;
    const CODE_SESSION_EXPIRED = 403;

    const ATTD_TYPE_LEAVE = 2;
    const ATTD_TYPE_RECOMENDATION = 3;
    const ATTD_TYPE_OVERTIME_WORKDAY = 7;
    const ATTD_TYPE_OVERTIME_HOLIDAY = 10;
    const ATTD_TYPE_SET_ADMIN = 4;

    const TYPE_ID_ATTENDANCE    = 1;
    const TYPE_ID_OVERTIME      = 2;
    const TYPE_ID_ABSENT      = 3;

    const request = 1;
    const minApprove = 2;
    const approved = 2;
    const rejected = 3;
    const entrance = 1;
    const refAttendance = 1;
    const refRecomendation = 5;
    const isRoundRangeDate = 2;


    public function timeAction() {
        $result = new \Neuron\Generic\Result();
        
        $request = $this->getRequest();

        if (!$request->isPost()) {
            $result->code = self::FAIL_CODE;
            $result->info = 'Method not allowed';
            return $this->getREST()->getResponse($result);
            
        }

        $data = $this->getREST()->getParameters();
        $guid = $this->getGuid();

        try {
            if($guid == "" || empty($guid)){
                $result->code = self::CODE_FIELD_REQUIRED;
                $result->info = 'token is null';
                return $this->getREST()->getResponse($result);
            }

            $entity = Entity::get($this, $guid);
            $user = $entity->user();
            if(!$user->id){
                $result->code = self::CODE_SESSION_EXPIRED;
                $result->info = 'Session Expired';
                return $this->getREST()->getResponse($result); 
            }
            
            $userPerson = $this->getUserPerson($user);

            if(count($userPerson['person']) <= 0){
                $result->code = 402;
                $result->info = "This user is not mapped yet.";
                return $this->getREST()->getResponse($result);
            }

            $flag_attd_evidence = "0";
            $flag_geolocation = 0;
            $sPersonDetail  = \Neuron\Employee\Detail\Storage::factory($this->getDb(), $this->getConfig());
            $mPersonDetail = new \Neuron\Employee\Detail($sPersonDetail);
            $mPersonDetail->load($userPerson['person']['ref_id'], \Neuron\Employee\Detail\Storage::LOADBY_EMPID);
            if($mPersonDetail->flag_attd_evidence != null){
                $flag_attd_evidence = $mPersonDetail->flag_attd_evidence;
            }

            if($mPersonDetail->flag_geolocation != null){
                $flag_geolocation = $mPersonDetail->flag_geolocation;
            }


            $storageAttendance = \Neuron\Attendance\Attendance\Storage::factory($this->getDb(), $this->getConfig());
            $modalAttendance  = new \Neuron\Attendance\Attendance($storageAttendance);
            $storageTimettable = \Neuron\Attendance\Shift\Storage::factory($this->getDb(), $this->getConfig());
            $modelTimetable = new \Neuron\Attendance\Shift($storageTimettable);
            
            $cekAttdEntrance  = $modalAttendance->checkAttendance(1, $userPerson['person']['ref_id'], date('Y-m-d'));
            $dataattendance = null;
            $isAttend = true;
            if (!$cekAttdEntrance) {
                $isAttend = false;
                $dataattendance = null;
            }else{
                $dataattendance = $cekAttdEntrance[0];
                $cekAttdEntrance[0]['attendance_id'] = $cekAttdEntrance[0]['id'];
                unset($cekAttdEntrance[0]['id']);
            }
            $config = $this->getConfig();
            $arrLocations = array();
            $entity         = new \Neuron\Attendance\Entity(
                \Neuron\Attendance\Entity\Storage::factory($this->getDb(),$this->getConfig())
            );
            $resloc         = $entity->locationList();

            $dataLoc = null;
            if($resloc->data){
                $dataLoc = $resloc->data;
            }

            $min_distance = 75;
            if(isset($config['attendance']['geolocation']['min_distance'])){
                if($config['attendance']['geolocation']['min_distance'] != '' && $config['attendance']['geolocation']['min_distance'] != NULL){
                    $min_distance = intval($config['attendance']['geolocation']['min_distance']);
                }
            }

            if(!isset($config['attendance'])){
                $dataLoc = null;
            }else{
                $attendanceConfig	= $config['attendance'];
                if(isset($attendanceConfig['geolocation'])){
                    $geolocation 	   	= $attendanceConfig['geolocation'];
                    if($geolocation['is_active'] == 1){
                        if($flag_geolocation == 0){
                            $dataLoc = null;
                        }
                    }else{
                        $dataLoc = null;
                    }
                }else{
                    $dataLoc = null;
                }
            }


            $dayno = date('N');
            $cektimetable = $modelTimetable->loadShiftObject($userPerson['person']['ref_id'], date('Y-m-d')); 
            $shift= null;
            if($cektimetable){
                foreach ($cektimetable as $val) {
                    if($val['day_no'] == $dayno+1){
                        $shift = $val;  
                    }
                }
            }
            $dateTime = date('Y-m-d H:i:s');

            $data =  array('isAttend' => $isAttend, 'attendance' => $dataattendance, 'timeshift' => $shift, "currentDate"=>$dateTime, 'locations'=>$dataLoc, 'min_distance' => $min_distance, 'flag_evidence'=> $flag_attd_evidence);
            $result->guid = $guid;
            $result->code = self::SUCCESS_CODE;
            $result->info = 'OK';
            $result->data = $data;
			
			$requestPost = array(
				'guid' => $guid,
				'code' => '',
				'info' => '',
				'data' => $data
			); 

			$responseResult = array(
				'code' => $result->code,
				'info' => $result->info,
				'data' => $result->data
			); 

			$logparam['subject'] = 'API ATTENDANCE ATTENDANCE TIMEACTION';
			$logparam['identity'] = '';
			$logparam['user'] = $user->id ? $user->id : '';
			$logparam['description'] = 'Log proses api attendance attendance timeAction';
			$logparam['request'] = json_encode($requestPost);
			$logparam['response'] = json_encode($responseResult);

			$this->writeLogAction($logparam);
			
            return $this->getREST()->getResponse($result);
        } catch (Exception $e) {
            $result->code = self::FAIL_CODE;
            $result->info = 'error';
            $result->data = array(
                'message' => $ex->getMessage(),
            );
			
			$requestPost = array(
				'guid' => $guid,
				'code' => '',
				'info' => '',
				'data' => $data
			); 

			$responseResult = array(
				'code' => $result->code,
				'info' => $result->info,
				'data' => $result->data
			); 

			$logparam['subject'] = 'API ATTENDANCE ATTENDANCE TIMEACTION';
			$logparam['identity'] = '';
			$logparam['user'] = $user->id ? $user->id : '';
			$logparam['description'] = 'Log proses api attendance attendance timeAction';
			$logparam['request'] = json_encode($requestPost);
			$logparam['response'] = json_encode($responseResult);

			$this->writeLogAction($logparam);
			
            return $this->getREST()->getResponse($result);
        }
       
    }

    /* masih bikin logic sendiri */
    public function saveOldAction(){
        $postData = $this->getParameters();

        /// API generic result
        $result = new \Neuron\Generic\Result();

        $attd_id = (isset($postData['attendance_id']) && !empty($postData['attendance_id']) && $postData['attendance_id'] != '')? $postData['attendance_id']:null;
        $attd_type_id = isset($postData['attendance_type_id'])?$postData['attendance_type_id']:1;
        $attd_desc = isset($postData['description'])?htmlentities($postData['description']):'';
        $fileevidence = isset($postData['file_evidence'])? $postData['file_evidence']:null;
        $latitude = (isset($postData['latitude']) && !empty($postData['latitude']) && $postData['latitude']!= null)? $postData['latitude']:null;
        $longitude = (isset($postData['longitude']) && !empty($postData['longitude']) && $postData['longitude']!= null)? $postData['longitude']:null;
        $attd_date = date('Y-m-d H:i:s');
        $guid = $this->getGuid();
        try {
            if($guid == ""){
                $result->code = self::CODE_FIELD_REQUIRED;
                $result->info = 'token is null';
                return $this->getREST()->getResponse($result);   
            }

            $entity = Entity::get($this, $guid);
            $user = $entity->user();
            if(!$user->id){
                $result->code = self::CODE_SESSION_EXPIRED;
                $result->info = 'Session Expired';
                return $this->getREST()->getResponse($result); 
            }
            
            $userPerson = $this->getUserPerson($user);
            $ref_id = $userPerson['person']['ref_id'];

            $storageAttendance = \Neuron\Attendance\Attendance\Storage::factory($this->getDb(), $this->getConfig());
            $modelAttendance = new \Neuron\Attendance\Attendance($storageAttendance);
            $storageTimettable = \Neuron\Attendance\Shift\Storage::factory($this->getDb(), $this->getConfig());
            $modelTimetable = new \Neuron\Attendance\Shift($storageTimettable);

            $dayno = date('N');
            $cektimetable = $modelTimetable->loadShiftObject($userPerson['person']['ref_id'], date('Y-m-d')); 
            $shift= null;
            if($cektimetable){
                foreach ($cektimetable as $val) {
                    if($val['day_no'] == $dayno+1){
                        $shift = $val;  
                    }
                }
            }

            $modelAttendance->id = $attd_id;
            $modelAttendance->type_id = $attd_type_id;
            $modelAttendance->ref_id = $ref_id;
            $modelAttendance->ref_code = "EMPLOYEE";
            $modelAttendance->source_type_code = 'WEB';
            $modelAttendance->source_reference = 'MOBILE';
            $modelAttendance->source_dtm = date('Y-m-d H:i:s');
            

            if (empty($userPerson['person']['ref_id'])) {
                $success = false;
                $message = "An employee is required to sign an attendance";
                $data =  array('success'=> $success, 'message'=>$message);
                $result->guid = self::SUCCESS_CODE;
                $result->code = self::FAIL_CODE;
                $result->info = $message;
                $result->data = $data;
				
				$requestPost = array(
					'guid' => $guid,
					'code' => '',
					'info' => '',
					'data' => $postData
				); 

				$responseResult = array(
					'code' => $result->code,
					'info' => $result->info,
					'data' => $result->data
				); 

				$logparam['subject'] = 'API ATTENDANCE ATTENDANCE SAVEACTION';
				$logparam['identity'] = '';
				$logparam['user'] = $user->id ? $user->id : '';
				$logparam['description'] = 'Log proses api attendance attendance saveAction';
				$logparam['request'] = json_encode($requestPost);
				$logparam['response'] = json_encode($responseResult);

				$this->writeLogAction($logparam);
				
                return $this->getREST()->getResponse($result);
            }
            
            if ($attd_id == '' || $attd_id == null) {
                $modelAttendance->start_description = $attd_desc;
                $modelAttendance->start_dtm = $attd_date;
                $modelAttendance->end_description = null;
                $modelAttendance->end_dtm = null;
                $modelAttendance->latitude = $latitude;
                $modelAttendance->longitude = $longitude;
                $save = false;
                if (!$modelAttendance->checkAttendance($modelAttendance->type_id, $modelAttendance->ref_id, $modelAttendance->start_dtm)) {
                    $save = $modelAttendance->save(false);
                }
                if($save){
                    if($shift){
                        $dataDetail['attendance_id'] = $modelAttendance->id;
                        $dataDetail['timetable_id'] = @$shift['timetable_id'];
                        if($latitude && $longitude){
                            $dataDetail['geocode_location'] = $this->geoCoordinates($latitude, $longitude);
                        }else{
                            $dataDetail['geocode_location'] = null;
                        }
                        $dataDetail['attd_in_out'] = 1;
                            
                        $storageDetail = \Neuron\Attendance\Detail\Storage::factory($this->getDb(), $this->getConfig());
                        $modelDetail  = new \Neuron\Attendance\Detail($storageDetail);
                        $saveDetail = $modelDetail->create($dataDetail);
                    }

                    if($fileevidence){
                        $dataAttendace = $modelAttendance->pull();
                        $dataAttendace['file'] = $fileevidence;
                        $dataAttendace['action_type'] = 1;
                        $dataAttendace['action_type_name'] = 'attendance';
                        $storageEvidence = \Neuron\Attendance\Evidence\Storage::factory($this->getDb(), $this->getConfig());
                        $modelEvidence  = new \Neuron\Attendance\Evidence($storageEvidence);
                        $modelEvidence->create($dataAttendace);
                    }
                }
                $event = 'insert';
            }else{
                $currAttendance = $modelAttendance->load($attd_id);
                // print_r($currAttendance); die();
                if($currAttendance['end_description'] != ''){
                    $success = false;
                    $msg = 'Sorry, You already attended today';
                    $save = false;
                }else{
                    $modelAttendance->id = $currAttendance['id'];
                    $modelAttendance->type_id = $currAttendance['type_id'];
                    $modelAttendance->ref_id = $currAttendance['ref_id'];
                    $modelAttendance->ref_code = $currAttendance['ref_code'];
                    $modelAttendance->source_type_code = $currAttendance['source_type_code'];
                    $modelAttendance->source_reference = $currAttendance['source_reference'];
                    $modelAttendance->source_dtm = $currAttendance['source_dtm'];
                    $modelAttendance->start_description = $currAttendance['start_description'];
                    $modelAttendance->start_dtm = $currAttendance['start_dtm'];

                    $modelAttendance->end_description = $attd_desc;
                    //$attd->start_dtm = $dateTime;
                    $modelAttendance->end_dtm = $attd_date;

                    $save = $modelAttendance->save(true);
                    if ($save) {
                        
                        $dataDetail['attendance_id'] = $modelAttendance->id;
                        $dataDetail['timetable_id'] = @$shift['timetable_id'];
                        if($latitude && $longitude){
                            $dataDetail['geocode_location'] = $this->geoCoordinates($latitude, $longitude);
                        }else{
                            $dataDetail['geocode_location'] = null;
                        }
                        $dataDetail['attd_in_out'] = 2; 
                            
                        $storageDetail = \Neuron\Attendance\Detail\Storage::factory($this->getDb(), $this->getConfig());
                        $modelDetail  = new \Neuron\Attendance\Detail($storageDetail);
                        $saveDetail = $modelDetail->create($dataDetail);
                    }
                    $event = 'update';
                }
            }

            if ($save) {
                $success = true;
                $message = 'Attendance saved Successfully';
                $data =  array('success'=> $success, 'message'=>$message);

                $result->guid = $guid;
                $result->code = self::SUCCESS_CODE;
                $result->info = 'Attendance saved Successfully';
                $result->data = $data;
				
				$requestPost = array(
					'guid' => $guid,
					'code' => '',
					'info' => '',
					'data' => $postData
				); 

				$responseResult = array(
					'code' => $result->code,
					'info' => $result->info,
					'data' => $result->data
				); 

				$logparam['subject'] = 'API ATTENDANCE ATTENDANCE SAVEACTION';
				$logparam['identity'] = '';
				$logparam['user'] = $user->id ? $user->id : '';
				$logparam['description'] = 'Log proses api attendance attendance saveAction';
				$logparam['request'] = json_encode($requestPost);
				$logparam['response'] = json_encode($responseResult);

				$this->writeLogAction($logparam);
				
                return $this->getREST()->getResponse($result);

            } else {
                $success = false;
                $message = 'Sorry, You already attended today';

                $data =  array('success'=> $success, 'message'=>$message);

                $result->guid = $guid;
                $result->code = self::FAIL_CODE;
                $result->data = $data;
                $result->info = $message;
				
				$requestPost = array(
					'guid' => $guid,
					'code' => '',
					'info' => '',
					'data' => $postData
				); 

				$responseResult = array(
					'code' => $result->code,
					'info' => $result->info,
					'data' => $result->data
				); 

				$logparam['subject'] = 'API ATTENDANCE ATTENDANCE SAVEACTION';
				$logparam['identity'] = '';
				$logparam['user'] = $user->id ? $user->id : '';
				$logparam['description'] = 'Log proses api attendance attendance saveAction';
				$logparam['request'] = json_encode($requestPost);
				$logparam['response'] = json_encode($responseResult);

				$this->writeLogAction($logparam);
				
                return $this->getREST()->getResponse($result);
            }
            
        } catch (\Exception $e) {
            $result->code = self::SUCCESS_CODE;
            $result->info = 'error';
            $result->data = array(
                'message' => $e->getMessage(),
            );
            
            return $this->getREST()->getResponse($result);
        }
       
    }

    /* Pindahin logic ke modelnya */
    public function saveAction(){
        $result      = new \Neuron\Generic\Result;
        /* Validate guid */
        $guid   = $this->getREST()->getGuid();
        $post   = $this->getREST()->getParameters();
        if($guid == ""){
            $result->code = self::CODE_FIELD_REQUIRED;
            $result->info = 'token is null';
            return $this->getREST()->getResponse($result);   
        }
        $entity = Entity::get($this, $guid);
        $user = $entity->user();
        if(!$user->id){
            $result->code = self::CODE_SESSION_EXPIRED;
            $result->info = 'Session Expired';
            return $this->getREST()->getResponse($result); 
        }

        $refId          = $user->id;
        $latitude = (isset($post['latitude']) && !empty($post['latitude']) && $post['latitude']!= null) ? $post['latitude']:null;
        $longitude = (isset($post['longitude']) && !empty($post['longitude']) && $post['longitude']!= null) ? $post['longitude']:null;
        
        /* Check Person Map */
        if(class_exists('Neuron\Employee\Entity')){
            $personMap  = new \Neuron\Employee\Map(\Neuron\Person\Map\Storage::factory(
                $this->getDb(),
                $this->getConfig()
            ));
            $personMap->load($user->id,\Neuron\Person\Map\Storage::LOADBY_USER_ID);
            if($personMap->user_map_id){
                $refId  = $personMap->employee()->id;
            }
        }

        // Attendance Data
        $saveData           = [
            'start_dtm'         => date('Y-m-d H:i:s'),
            'ref_id'            => $refId,
            'type_id'           => \Neuron\Attendance\Entity::TYPE_ATTENDANCE,
            'source_reference'  => 1,
            'source_dtm'        => date('Y-m-d H:i:s'),
            'source_type_code'  => "API",
            'ref_code'          => "EMPLOYEE",
        ];
        
        /* Validate evidence */
        if(isset($post['file_evidence'])){
            $saveData['file_evidence'] = $post['file_evidence'];
        }
        
        /* Validate location */
        if($latitude && $longitude){
            $dataDetail['geocode_location'] = $this->geoCoordinates($latitude, $longitude);
        }else{
            $dataDetail['geocode_location'] = null;
        }
		
		/* Check ada data rekomendasi yang sudah di approve atau tidak, jika ada maka tidak bisa absen */
		$personMap  = new \Neuron\Person\Map(\Neuron\Person\Map\Storage::factory(
						$this->getDb(),
						$this->getConfig()
					));
		$personMap->load($refId,\Neuron\Person\Map\Storage::LOADBY_PERSON_ID);
		$userId 	= $personMap->user_id;
		
		$leave     = new \Neuron\Approval\Entities(\Neuron\Order\Entity\Storage::factory(
						$this->getDb(),
						$this->getConfig()
					));
					
		$where['where']['create_user_id'] = $userId;
		$where['where']['order_type_id'] = 2200;
		$where['where']['order_status_id'] = 2;
		$where['where']['xn5'] = 2;
		$where['where']['sdate'] = date('Y-m-d H:i:s');
		
		if ($manager = NAF\Event\Manager::get()) {
            $manager->events()->register('order_query',array($this,'onCheckQuery'),0,3);
        }
		
		$result = $leave->load(null, \Neuron\Order\Entity\Storage::LIST_ALL, $where, true);
		
		if($result->code == 0){
			$result->code = 1;
			$result->info = 'Cannot attend, your leave recommendation has been approved';
			$result->data = (object) array();
		}else{
			/* Tidak ada rekomendasi selanjutnya, save attendance */
			
			//print_r($saveData);die();
			
			$attendanceEntity   = new \Neuron\Attendance\Entity(\Neuron\Attendance\Entity\Storage::factory(
				$this->getDb(),
				$this->getconfig()
			));
			$result             = $attendanceEntity->saveEx($saveData);
			$result->data       = [
				"success"    => $result->code == 0,
				'message'      => $result->info,
				'data'      => $attendanceEntity->pull()
			];
		}

        $requestPost = array(
            'guid' => $guid,
            'code' => '',
            'info' => '',
            'data' => $post
        ); 

        $responseResult = array(
            'code' => $result->code,
            'info' => $result->info,
            'data' => $result->data
        ); 

        // {'guid': 'm4q340vsub0c711grktacpfv8d', 'code': 0, 'info': 'Attendance saved Successfully', 'data': {'success': True, 'message': 'Attendance saved Successfully'}}

        $logparam['subject'] = 'API ATTENDANCE ATTENDANCE SAVEACTION';
        $logparam['identity'] = '';
        $logparam['user'] = $user->id ? $user->id : '';
        $logparam['description'] = 'Log proses api attendance attendance saveAction';
        $logparam['request'] = json_encode($requestPost);
        $logparam['response'] = json_encode($responseResult);

        $result->guid           = $guid;
        $result->code           = intval($result->code);
        $this->writeLogAction($logparam);
        return $this->getREST()->getResponse($result);

    }

    public function saveAttendanceAction(){
        $result = new \Neuron\Generic\Result();
        
        $request = $this->getRequest();

        if (!$request->isPost()) {
            $result->code = self::FAIL_CODE;
            $result->info = 'Method not allowed';   
            return $this->getREST()->getResponse($result);
            
        }
        $data = $this->getREST()->getParameters();
        $username = isset($data['username'])?$data['username']:'';
        $attd_id = isset($data['attendance_id'])?$data['attendance_id']:'';
        $attd_type_id = isset($data['attendance_type_id'])?$data['attendance_type_id']:'';
        $attd_desc = isset($data['description'])?htmlentities($data['description']):'';
        $attd_date = date('Y-m-d H:i:s');
        $guid = $this->getGuid();
		
		$requestPost = array(
			'guid' => $guid,
			'code' => '',
			'info' => '',
			'data' => $data
		); 
		
        try {
            if($token == ""){
                $result->code = self::CODE_FIELD_REQUIRED;
                $result->info = 'token is null';
                return $this->getREST()->getResponse($result);   
            }
            
            $entity = Entity::get($this, $guid);
            $user = $entity->user();
            if(!$user->id){
                $result->code = self::CODE_SESSION_EXPIRED;
                $result->info = 'Session Expired';
                return $this->getREST()->getResponse($result); 
            }
            
            $userPerson = $this->getUserPerson($user);
            $ref_id = $userPerson['person']['ref_id'];
            $storageAttendance = \Neuron\Attendance\Attendance\Storage::factory($this->getDb(), $this->getConfig());
            $modelAttendance = new \Neuron\Attendance\Attendance($storageAttendance);
            $storageTimettable = \Neuron\Attendance\Shift\Storage::factory($this->getDb(), $this->getConfig());
            $modelTimetable = new \Neuron\Attendance\Shift($storageTimettable);

            $modelAttendance->id = $attd_id;
            $modelAttendance->type_id = $attd_type_id;
            $modelAttendance->ref_id = $ref_id;
            $modelAttendance->ref_code = "EMPLOYEE";
            $modelAttendance->source_type_code = 'WEB';
            $modelAttendance->source_reference = $ref_id;
            $modelAttendance->source_dtm = date('Y-m-d H:i:s');

            if (empty($userPerson['person']['ref_id'])) {
                $success = false;
                $message = "An employee is required to sign an attendance";
                $data =  array('success'=> $success, 'message'=>$message, "token"=>$sessid);
                $result->guid = self::SUCCESS_CODE;
                $result->code = self::FAIL_CODE;
                $result->info = $message;
                $result->data = $data;
				
				$responseResult = array(
					'code' => $result->code,
					'info' => $result->info,
					'data' => $result->data
				); 

				$logparam['subject'] = 'API ATTENDANCE ATTENDANCE SAVEACTION';
				$logparam['identity'] = '';
				$logparam['user'] = $user->id ? $user->id : '';
				$logparam['description'] = 'Log proses api attendance attendance saveAction';
				$logparam['request'] = json_encode($requestPost);
				$logparam['response'] = json_encode($responseResult);

				$this->writeLogAction($logparam);
				
                return $this->getREST()->getResponse($result);
            }

            if ($attd_id == '' || $attd_id == null) {
                $modelAttendance->start_description = $attd_desc;
                $modelAttendance->start_dtm = $attd_date;
                $modelAttendance->end_description = null;
                $modelAttendance->end_dtm = null;
                $save = false;
                if (!$modelAttendance->checkAttendance($modelAttendance->type_id, $modelAttendance->ref_id, $modelAttendance->start_dtm)) {
                    $save = $modelAttendance->save(false);
                }
                // if($save){
                //     //$saveActivity = $this->saveActivity($dateTime, $attd->employee_id, $save);
                //     $saveActivity = $this->saveActivity($attd_date, $userId, $save);
                // }
                $event = 'insert';
            }else{
                $currAttendance = $modelAttendance->load($attd_id);
                // print_r($currAttendance); die();
                if($currAttendance['end_description'] != ''){
                    $success = false;
                    $msg = 'Sorry, You have done this day attendance out';
                    $save = false;
                }else{
                    $modelAttendance->id = $currAttendance['id'];
                    $modelAttendance->type_id = $currAttendance['type_id'];
                    $modelAttendance->ref_id = $currAttendance['ref_id'];
                    $modelAttendance->ref_code = $currAttendance['ref_code'];
                    $modelAttendance->source_type_code = $currAttendance['source_type_code'];
                    $modelAttendance->source_reference = $currAttendance['source_reference'];
                    $modelAttendance->source_dtm = $currAttendance['source_dtm'];
                    $modelAttendance->start_description = $currAttendance['start_description'];
                    $modelAttendance->start_dtm = $currAttendance['start_dtm'];

                    $modelAttendance->end_description = $attd_desc;
                    //$attd->start_dtm = $dateTime;
                    $modelAttendance->end_dtm = $attd_date;

                    $save = $modelAttendance->save(true);
                    // if ($save) {
                    //     //$saveActivity = $this->saveActivity($dateTime, $attd->employee_id, $attd->attendance_id);
                    //     $saveActivity = $this->saveActivity($attd_date, $userId, $attd_id);
                    // }
                    $event = 'update';
                }
            }

            if ($save) {
                $success = true;
                $message = 'Attendance saved Successfully';
                $data =  array('success'=> $success, 'message'=>$message, "token"=>$sessid);

                $result->guid = self::SUCCESS_CODE;
                $result->code = self::SUCCESS_CODE;
                $result->info = 'Attendance saved Successfully';
                $result->data = $data;
				
				$responseResult = array(
					'code' => $result->code,
					'info' => $result->info,
					'data' => $result->data
				); 

				$logparam['subject'] = 'API ATTENDANCE ATTENDANCE SAVEACTION';
				$logparam['identity'] = '';
				$logparam['user'] = $user->id ? $user->id : '';
				$logparam['description'] = 'Log proses api attendance attendance saveAction';
				$logparam['request'] = json_encode($requestPost);
				$logparam['response'] = json_encode($responseResult);

				$this->writeLogAction($logparam);
				
                return $this->getREST()->getResponse($result);

            } else {
                $success = false;
                $message = 'Sorry, You have done this day attendance out';

                $data =  array('success'=> $success, 'message'=>$message, "token"=>$sessid);

                $result->guid = self::SUCCESS_CODE;
                $result->code = self::FAIL_CODE;
                $result->data = $data;
                $result->info = $message;
				
				$responseResult = array(
					'code' => $result->code,
					'info' => $result->info,
					'data' => $result->data
				); 

				$logparam['subject'] = 'API ATTENDANCE ATTENDANCE SAVEACTION';
				$logparam['identity'] = '';
				$logparam['user'] = $user->id ? $user->id : '';
				$logparam['description'] = 'Log proses api attendance attendance saveAction';
				$logparam['request'] = json_encode($requestPost);
				$logparam['response'] = json_encode($responseResult);

				$this->writeLogAction($logparam);
				
                return $this->getREST()->getResponse($result);
            }
          
        } catch (Exception $e) {
            $result->code = self::SUCCESS_CODE;
            $result->info = 'error';
            $result->data = array(
                'message' => $ex->getMessage(),
            );
			
			$responseResult = array(
				'code' => $result->code,
				'info' => $result->info,
				'data' => $result->data
			); 

			$logparam['subject'] = 'API ATTENDANCE ATTENDANCE SAVEACTION';
			$logparam['identity'] = '';
			$logparam['user'] = $user->id ? $user->id : '';
			$logparam['description'] = 'Log proses api attendance attendance saveAction';
			$logparam['request'] = json_encode($requestPost);
			$logparam['response'] = json_encode($responseResult);

			$this->writeLogAction($logparam);
			
            return $this->getREST()->getResponse($result);
        }
    }

    public function saveAttdRecomendationAction(){
        $result = new \Neuron\Generic\Result();
        
        $request = $this->getRequest();

        if (!$request->isPost()) {
            $result->code = self::FAIL_CODE;
            $result->info = 'Method not allowed';
            return $this->getREST()->getResponse($result);
            
        }
        $data = $this->getREST()->getParameters();
        $username = isset($data['username'])?$data['username']:'';
        $attd_type_id = isset($data['attendance_type_id'])? $data['attendance_type_id'] :'';
        $rec_req_message = isset($data['request_message'])? htmlentities($data['request_message']): '';
        $start_dtm = isset($data['start_dtm'])? date('Y-m-d H:i:s', strtotime($data['start_dtm'])): null;
        $end_dtm = isset($data['end_dtm'])?date('Y-m-d H:i:s', strtotime($data['end_dtm'])): null;
        $recTo = isset($data['recomendationTo'])? $data['recomendationTo']: null;
        $inOut = isset($data['inOut'])?$data['inOut']: null;
        $default_shift_id = isset($data['default_shift_id'])? $data['default_shift_id']: '';
        $change_shift_id = isset($data['change_shift_id'])? $data['change_shift_id']: '';
        $isRound = isset($data['is_round'])? $data['is_round']: '';
        $token = isset($data['token'])? $data['token']: '';
       
	   $requestPost = array(
			'guid' => '',
			'code' => '',
			'info' => '',
			'data' => $data
		); 
	   
	    try {
            if($username == ""){
                $result->code = self::CODE_FIELD_REQUIRED;
                $result->info = 'username is null';
                return $this->getREST()->gerResponse($result);
            }
            if($token == ""){
                $result->code = self::CODE_FIELD_REQUIRED;
                $result->info = 'token is null';
                return $this->getREST()->getResponse($result);   
            }
            
            $session =$this->getSession();

            $userPerson = $this->getUserPerson($username);

            if(!$userPerson['person']){
                $result->code = self::CODE_FIELD_REQUIRED;
                $result->info = 'person is null';
                return $this->getREST()->getResponse($result);   
            }

            $session->owner($userPerson['user']['user_id']);

            $sessid = $session->getId();
            if($token != $sessid){
                $result->code = self::CODE_UNAUTHORIZED;
                $result->info = 'Failed authenticate';
                return $this->getREST()->getResponse($result);   
            }

            $islogin = $this->isLoggedIn();
            if(!$islogin){
                $result->code = self::CODE_SESSION_EXPIRED;
                $result->info = 'Session Expired';
                return $this->getREST()->getResponse($result);   
            }

            $storageReq = \Neuron\Attendance\Request\Storage::factory($this->getDb(), $this->getConfig());
            $modelReq = new \Neuron\Attendance\Request($storageReq);

            $modelReq->ref_code = "EMPLOYEE";
            $modelReq->ref_id = $userPerson['person']['ref_id'];
            $modelReq->start_dtm = $start_dtm;
            $modelReq->end_dtm = $end_dtm;
            $modelReq->request_type = $attd_type_id;
            $modelReq->request_dtm = date("Y-m-d H:i:s");
            $modelReq->status_id = 1;
            $modelReq->request_message = $rec_req_message;


            if (count($inOut) == 1) {
                if (in_array("in", $inOut)) {
                    $modelReq->start_dtm = $start_dtm;
                } else {
                    $modelReq->start_dtm = null;
                    $modelReq->end_dtm = $end_dtm;
                }
            } 
        


            if ($isRound == self::isRoundRangeDate) {
                if ($end_dtm == '' || $end_dtm == null) {
                    $CheckOutX = explode("-", date('Y-m-d', strtotime($start_dtm)));
                } else {
                    $CheckOutX = explode("-", date('Y-m-d', strtotime($end_dtm)));
                }
                $CheckInX = explode("-", date('Y-m-d', strtotime($start_dtm)));

                $date1 = mktime(0, 0, 0, $CheckInX[1], $CheckInX[2], $CheckInX[0]);
                $date2 = mktime(0, 0, 0, $CheckOutX[1], $CheckOutX[2], $CheckOutX[0]);
                $interval = ($date2 - $date1) / (3600 * 24);
                //echo $date1.'XX'.$date2.'XXX'.$end_dtm.'<br>';
                //print_r($CheckInX);
                //print_r($CheckOutX);
                for ($x = 0; $x <= $interval; $x++) {
                    $weekend = array('Sat', 'Sun', 'Saturday', 'Sunday', 'Sabtu', 'Minggu', 'Sab', 'Min');
                    if (!in_array(date('D', strtotime($start_dtm . ' +' . $x . ' day')), $weekend)) {
                        //echo 'a :'.date('Y-m-d', strtotime($attd->start_dtm.' +'.$x.' day'))."<br />";
                        $modelReq->start_dtm = date('Y-m-d H:i:s', strtotime($start_dtm . ' +' . $x . ' day'));
                        $modelReq->end_dtm = date('Y-m-d 17:00:00', strtotime($start_dtm . ' +' . $x . ' day'));
                        $save = $modelReq->save(false);
                        //echo date('D', strtotime($start_dtm . ' +' . $x . ' day'));
                        $saveRecToApproval = $this->saveRecToApproval($modelReq->request_id, $recTo, $userId);
                        
                    } else if ($attd_type_id == 5) {
                        $modelReq->start_dtm = date('Y-m-d H:i:s', strtotime($start_dtm . ' +' . $x . ' day'));
                        $modelReq->end_dtm = date('Y-m-d 17:00:00', strtotime($start_dtm . ' +' . $x . ' day'));
                        $save = $modelReq->save(false);
                        $saveRecToApproval = $this->saveRecToApproval($modelReq->request_id, $recTo, $userPerson['person']['ref_id']);
                       
                    } else {
                        $saveRecToApproval = json_encode(array('success' => true, 'msg' => 'Date in weekend not inserted!'));
                        
                    }
                    //echo $this->saveRecToApproval($save, $recTo, $session->owner());
                }
            } else {
                $save = $modelReq->save(false);
                $saveRecToApproval = $this->saveRecToApproval($modelReq->request_id, $recTo,$userPerson['person']['ref_id']);
                
            }
            $hasilsave = json_decode($saveRecToApproval, true);
            if ($hasilsave['success']) {
                $success = $hasilsave['success'];
                $message = $hasilsave['msg'];

                $data =  array('success'=> $success, 'message'=>$message, "token"=>$sessid);

                $result->guid = self::SUCCESS_CODE;
                $result->code = self::SUCCESS_CODE;
                $result->info = $message;
                $result->data = $data;
				
				$responseResult = array(
					'code' => $result->code,
					'info' => $result->info,
					'data' => $result->data
				); 

				$logparam['subject'] = 'API ATTENDANCE ATTENDANCE SAVEATTDRECOMENDATIONACTION';
				$logparam['identity'] = '';
				$logparam['user'] = $userPerson['person'] ? $userPerson['person'] : '';
				$logparam['description'] = 'Log proses api attendance attendance saveAttdRecomendationAction';
				$logparam['request'] = json_encode($requestPost);
				$logparam['response'] = json_encode($responseResult);

				$this->writeLogAction($logparam);
				
                return $this->getREST()->getResponse($result);

            } else {
                $success = $hasilsave['success'];
                $message = $hasilsave['msg'];
                
                $data =  array('success'=> $success, 'message'=>$message, "token"=>$sessid);

                $result->guid = self::SUCCESS_CODE;
                $result->code = self::FAIL_CODE;
                $result->data = $data;
                $result->info = $message;
				
				$responseResult = array(
					'code' => $result->code,
					'info' => $result->info,
					'data' => $result->data
				); 

				$logparam['subject'] = 'API ATTENDANCE ATTENDANCE SAVEATTDRECOMENDATIONACTION';
				$logparam['identity'] = '';
				$logparam['user'] = $userPerson['person'] ? $userPerson['person'] : '';
				$logparam['description'] = 'Log proses api attendance attendance saveAttdRecomendationAction';
				$logparam['request'] = json_encode($requestPost);
				$logparam['response'] = json_encode($responseResult);

				$this->writeLogAction($logparam);
				
                return $this->getREST()->getResponse($result);
            }
          
        } catch (Exception $e) {
            $result->code = self::FAIL_CODE;
            $result->info = 'error';
            $result->data = array(
                'message' => $ex->getMessage(),
            );
			
			$responseResult = array(
				'code' => $result->code,
				'info' => $result->info,
				'data' => $result->data
			); 

			$logparam['subject'] = 'API ATTENDANCE ATTENDANCE SAVEATTDRECOMENDATIONACTION';
			$logparam['identity'] = '';
			$logparam['user'] = $userPerson['person'] ? $userPerson['person'] : '';
			$logparam['description'] = 'Log proses api attendance attendance saveAttdRecomendationAction';
			$logparam['request'] = json_encode($requestPost);
			$logparam['response'] = json_encode($responseResult);

			$this->writeLogAction($logparam);
			
            return $this->getREST()->getResponse($result);
        }
    }
    
    public function saveRecToApproval($save, $recTo, $notifer) {

        $storage = \Neuron\Attendance\Request\Storage::factory($this->getDb(), $this->getConfig());
        $creq = new \Neuron\Attendance\Request($storage);

        $success = false;
        $msg = $save;

        if ($save) {
            if($recTo){
                foreach ($recTo as $appr) {
                    $data['request_id'] = $save;
                    $data['ref_code'] = 'EMPLOYEE';
                    $data['ref_id'] = $appr;
                    $data['status_id'] = self::request;
                    $data['approval_dtm'] = date("Y-m-d H:i:s");
                    $data['reason'] = null;
                    $saveAppr = $creq->saveApproval($data, false);
                    if($saveAppr){
                        $success = true;                        
                        $msg = 'Recomendation saved Successfully';    
                    }else{
                        return json_encode(array('success' => false, 'msg' => 'Error on save approval', 'recomendation_id' => $save));
                    }
                }
            }else{
                $success = true;                        
                $msg = 'Recomendation saved Successfully';
            }
            
        }
		
		$requestPost = array(
			'guid' => '',
			'code' => '',
			'info' => '',
			'data' => array('save' => $save, 'recTo' => $recTo)
		); 
		
		$logparam['subject'] = 'API ATTENDANCE ATTENDANCE SAVERECTOAPPROVAL';
		$logparam['identity'] = '';
		$logparam['user'] = $userPerson['person'] ? $userPerson['person'] : '';
		$logparam['description'] = 'Log proses api attendance attendance saveRecToApproval';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode(array('success' => $success, 'msg' => $msg, 'recomendation_id' => $save));

		$this->writeLogAction($logparam);
		
        return json_encode(array('success' => $success, 'msg' => $msg, 'recomendation_id' => $save));
        //return $this->getResponse();
    }

    public function todayStatusAction(){
        $attendance     = new AttendanceModel(AttendanceModel\Storage::factory($this->getDb(),$this->getConfig()));
        $params         = $this->getREST()->getParameters();
        $result         = $attendance->todayStatus($params);
		
		$requestPost = array(
			'guid' => '',
			'code' => '',
			'info' => '',
			'data' => $params
		); 

		$responseResult = array(
			'code' => $result->code,
			'info' => $result->info,
			'data' => $result->data
		); 

		$logparam['subject'] = 'API ATTENDANCE ATTENDANCE TODAYSTATUSACTION';
		$logparam['identity'] = '';
		$logparam['user'] = '';
		$logparam['description'] = 'Log proses api attendance attendance todaystatusaction';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode($responseResult);

		$this->writeLogAction($logparam);
		
        return $this->getREST()->getResponse($result);
    }

    public function reportingAction(){
        $result     = new Result();

        /* Validating guid */
        $guid = $this->getREST()->getGuid();
        $result = $this->validateGuid($guid);

        if($result->code != 0){
            return $this->getREST()->getResponse($result);
        }

        try {
            /* Fetch user person by user_id */
            $userPerson     = $this->GetMapPerson($result->data['user_id']);
            $personID       = $userPerson['detail']['id'];
            $start_date = date("Y-m-d", mktime(0, 0, 0, date("m"), 1));
            $end_date = date("Y-m-t");
            
            //load data summary
            $filter['start_date'] = $start_date;
            $filter['end_date'] = $end_date;
            $filter['ref_id'] = $personID;
            $entity = new \Neuron\Attendance\Entity(\Neuron\Attendance\Entity\Storage::factory($this->getDb(), $this->getConfig()));
            $retval = $entity->loadSummary($filter);


            $resultData          = array();
            $absent            = new \stdClass;
            $absent->count     = $retval->data['count_absent'];
            $absent->type_name = "Absent";
            $absent->type_code = "3";
            $resultData[]       = $absent;

            $attendance            = new \stdClass;
            $attendance->count     = $retval->data['count_clockin'];
            $attendance->type_name = "Attendance";
            $attendance->type_code = "1";
            $resultData[]       = $attendance;

            $overtime            = new \stdClass;
            $overtime->count     = $retval->data['count_overtime'];
            $overtime->type_name = "Overtime";
            $overtime->type_code = "2";
            $resultData[]       = $overtime;

            $result->data       = $resultData;
            
        } catch (\Exception $th) {
            $result = new Result($guid,500,'Failed fetching data');
        }
        
        $result->guid   = $guid;
		
		$requestPost = array(
			'guid' => $guid,
			'code' => '',
			'info' => '',
			'data' => ''
		); 

		$responseResult = array(
			'code' => $result->code,
			'info' => $result->info,
			'data' => $result->data
		); 

		$logparam['subject'] = 'API ATTENDANCE ATTENDANCE TODAYSTATUSACTION';
		$logparam['identity'] = $personID ? $personID : '';
		$logparam['user'] = $personID ? $personID : '';
		$logparam['description'] = 'Log proses api attendance attendance todaystatusaction';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode($responseResult);

		$this->writeLogAction($logparam);
		
        return $this->getREST()->getResponse($result);
    }

    public function reportingDetailAction(){
        $result     = new Result();

        /* Validating guid */
        $guid = $this->getREST()->getGuid();
        $result = $this->validateGuid($guid);
        $dataAddt  = $this->getREST()->getParameters();

        if($result->code != 0){
            return $this->getREST()->getResponse($result);
        }

        try {
            /* Fetch user person by user_id */
            $userPerson     = $this->GetMapPerson($result->data['user_id']);
            $personID       = $userPerson['detail']['id'];
        
            
            //load data summary
            $dataAddt['start_date'] = isset($dataAddt['start_date'])? date("Y-m-d", strtotime($dataAddt['start_date'])) : date("Y-m-d", mktime(0, 0, 0, date("m"), 1));
            $dataAddt['end_date'] = isset($dataAddt['end_date'])? date("Y-m-d", strtotime($dataAddt['end_date'])) : date("Y-m-t");
            $dataAddt['ref_id'] = $personID;

            if(isset($dataAddt['type_id']) && empty($dataAddt['type_id'])){
                unset($dataAddt['type_id']);
            }

            $mode = \Neuron\Attendance\Entity\Storage::LISTBY_PARAM;
            $entities = new \Neuron\Attendance\Entities(\Neuron\Attendance\Entity\Storage::factory($this->getDb(), $this->getConfig()));
            $retval       = $entities->load(null,$mode,$dataAddt);
            $list = [];
            if($retval->data['list']){
                foreach ($retval->data['list'] as $item) {
                    $val['id'] = $item['id'];
                    $val['ref_id'] = $item['ref_id'];
                    $val['ref_name'] = $item['ref_name'];
                    $val['start_dtm'] = $item['start_dtm'];
                    $val['end_dtm'] = $item['end_dtm'];
                    $val['start_description'] = $item['start_description'];
                    $val['end_description'] = $item['end_description'];
                    $list[] = $val;
                }
            }
            $result->data  = $list;

            
            
        } catch (\Exception $th) {
            $result = new Result($guid,500,'Failed fetching data');
        }
        
        $result->guid   = $guid;
		
		$requestPost = array(
			'guid' => $guid,
			'code' => '',
			'info' => '',
			'data' => ''
		); 

		$responseResult = array(
			'code' => $result->code,
			'info' => $result->info,
			'data' => $result->data
		); 

		$logparam['subject'] = 'API ATTENDANCE ATTENDANCE REPORTINGDETAIL';
		$logparam['identity'] = $personID ? $personID : '';
		$logparam['user'] = $personID ? $personID : '';
		$logparam['description'] = 'Log proses api attendance attendance reportingDetailAction';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode($responseResult);

		$this->writeLogAction($logparam);
		
        return $this->getREST()->getResponse($result);
    }

    public function geoCoordinates($lat, $lng)
    {
        if($lat && $lng) {

            $urLocation = 'https://reverse.geocoder.api.here.com/6.2/reversegeocode.json?prox='.$lat.'%2C'.$lng.'&mode=retrieveAddresses&maxresults=1&gen=9&app_id=eUftlDOAPLvFJW4NoZyv&app_code=0sHXDeWOfdShAqAPGjFInw';

            // Create a stream
            $opts = array('http'=>array('header'=>"User-Agent: StevesCleverAddressScript 3.7.6\r\n"));
            $context = stream_context_create($opts);

            // Open the file using the HTTP headers set above
            $resLocation = file_get_contents($urLocation, false, $context);
            $resLocation = json_decode($resLocation, true);
            
            if(!isset($resLocation['type'])){
                $location = isset($resLocation['Response']['View'][0]['Result'][0]['Location']['Address']['Label']) ? $resLocation['Response']['View'][0]['Result'][0]['Location']['Address']['Label'] : null;
            }else{
                $location = null;
            }
        }else{
            $location = null;
        }
        return $location;
    }
	
	public function onCheckQuery($query,$params,$caller){

        if(isset($params['optional']['start_date'])){
            $query->where(" DATE('". $params['optional']['start_date'] ."') BETWEEN DATE(o.xd1) AND DATE(o.xd2)");
        }
    }
}

