<?php
namespace Neuron\Attendance\Controller\Api;
use Neuron\Access\Session;
use Neuron\Generic\Result;
use Neuron\Application\Framework\Entity;

class Controller extends \Neuron\Mvc\Controller {
    const CODE_SESSION_EXPIRED = 411;
    
	protected $_SESSION;

	public function onDispatch(\Zend\Mvc\MvcEvent $e) {

        /* dispatch NAF - v1 */
        if (class_exists('Neuron\Application\Framework\Hook\Manager')) {
            if ($manager = \Neuron\Application\Framework\Hook\Manager::get()) {
                return $manager->trigger()->onDispatch($this, parent::onDispatch($e));
            }
        }
        return parent::onDispatch($e);

        /* alternative */
        return \Neuron\Application\Framework\Event\Trigger::onDispatch($this, parent::onDispatch($e));
    }

	protected function getPostParams() {

		$request = $this->getRequest();
		if ($request->isPost()) {
			$post = $this->getRequest()->getPost();
		} else {
			$post = $this->getRequest()->getQuery();
		}

		return $post;
	}

	public function getSession($start = true) {

        $config = $this->getConfig();
        $sess_lifetime = $config['session']['gc_maxlifetime'];
        $remember = $config['session']['remember'];
        $tableSession = $config['tables']['sessions'];

        // print_r($tableSession); die();

        if(class_exists('Neuron\Application\Framework\Entity')) {
            $this->_SESSION = \Neuron\Application\Framework\Entity::get($this)->session();

            $lang = $this->_SESSION->get('lang');
            if(!isset($lang)) {
                $this->_SESSION->put(null, ['lang' => 'en'] );
            }

            $ref = ''.$this->_SESSION->get('ref_id');

            if ($this->_SESSION->owner() && (strlen($ref) < 1)) {
                // User Role
                $user = \Neuron\Application\Framework\Entity::get($this)->user();
                $grants = $user->grants->load();
                $roles = $user->roles->load($grants->data);

                // Mapper User
                $user_id =  $this->_SESSION->get('user_id');
                $GetMapperPerson = $this->GetMapPerson($user_id);
                $this->_SESSION->put(null, $GetMapperPerson);

            }

        } else {

            if ($this->_SESSION === null) {
                // $this->_SESSION = Session::factory($this->getConfig(), $this->getDb(), $start);
                $this->_SESSION = new Session('hrmisv2', Session::MODE_DATABASE,
                                                array('adapter'=>$this->getDb(),
                                                        'table'=>$tableSession,
                                                        'lifetime' => $sess_lifetime,
                                                        'remember' => $remember));
                $this->_SESSION->start(false);
                $this->_SESSION->put(null, ['lang' => 'en'] );
            }
            
        }

        //Set Lang
        $s = $this->_SESSION->get();
        $this->getTranslator($s['lang']);

        /* return session */
        return $this->_SESSION;
    }

    protected function isLoggedIn() {

        $session = $this->getSession();
        $owner = $session->owner();
        if (isset($owner) && !empty($owner)) {
            if ($owner != null) {
                $config = $this->getConfig();
                if($session->get('remember')){
                    $sess_lifetime = $config['session']['remember'];
                }else{
                    $sess_lifetime = $config['session']['gc_maxlifetime'];
                }

                
                if ((null !== $session->get('LAST_ACTIVITY')) && (time() - $session->get('LAST_ACTIVITY') > $sess_lifetime)) {
                    $session->stop();
                    return false;
                    //$this->redirect()->toRoute('login');
                    //echo '<script>location.href="/login";</script>';
                }
                $session->put('LAST_ACTIVITY', time());
                
                return $session;
            }
        }
        $session->stop();
        //$this->redirect()->toRoute('login');
        return false;
        //echo '<script>location.href="/login";</script>';
        //return $this->getResponse(); 
    }

    public function getTranslator($lang = 'id') {
        $loc = $this->getServiceLocator();
        $translator = $loc->get('translator');
        $translator->File("phparray",
                     __DIR__ .'/../language/'.strtolower($lang).'.php');
         
        //attach
        $loc->get('ViewHelperManager')->get('translate')
                                      ->setTranslator($translator);
    }

    public function GetMapPerson($user_id = 0) {
        $output = [];

        $employeeMap    = new \Neuron\Employee\Map(\Neuron\Employee\Map\Storage::factory($this->getDb(), $this->getConfig()));
        $employeeMap->load($user_id,\Neuron\Employee\Map\Storage::LOADBY_USER_ID);
        $dataPerson['data'] = array_merge($employeeMap->employee()->pull(), $employeeMap->pull(), $employeeMap->employee()->detail()->pull(), $employeeMap->employee()->job()->pull());

        // $storage = \Neuron\Person\Entity\Storage::factory($this->getDb(), $this->getConfig());
        // $cpersons = new \Neuron\Person\Entity($storage);

        // $dataPerson = $cpersons->loadMap($user_id,'user_id');

        $dataMapper = $dataPerson['data'];
        if($dataMapper) {

            $output['ref_id'] = $dataMapper['id'];
            $output['ohp_id'] = @$dataMapper['ohp_id'];
            $output['request_ref_code'] = "EMPLOYEE";
            $output['detail'] = $dataMapper;
            
        }

        return $output;

    }

    public function getOutput($json) {

        $response = $this->getResponse();
        $response->getHeaders()->addHeaderLine('Content-Type', 'application/json');
        $response->setContent($json);
        return $response;
    }

    public function publish() {

        if (class_exists('Neuron\Application\Framework\Entity')) {
            \Neuron\Application\Framework\Entity::publish();
        }
    }

    public function validateToken($token = null,$code = null){
        $result     = new Result();
        $sessCode   = $this->getSession()->get('code');
        $sessId     = $this->getSession()->getId();


        /* Validate session */
        if(!$sessCode){
            return new Result(0,501,'Invalid code');
        }else if(!$sessId){
            return new Result(0,502,'Invalid Sess ID');
        }

        /* Validate Parameter */
        if(!$token){
            return new Result(0,503,'Invalid Token');
        }else if(!$code){
            return new Result(0,504,'Invalid Code');
        }

        var_dump($token);
        echo '<br>----------------------------------------------<br>';
        var_dump($sessId);die();

        /* Compare Sess & Parameter */
        if($token != $sessId){
            return new Result(0, 505, 'Invalid Token');
        }else if($code != $sessCode){
            return new Result(0, 506, 'Invalid Code');
        }

        $result->data = array(
            'token'     => $token,
            'code'      => $code
        );
        return $result;

    }


    public function getUserPerson($user){
        $user_id = $user->id; 
        $dataPerson = $this->GetMapPerson($user_id);
        $dataUser = $user->pull();

        return array('user'=>$dataUser, 'person'=>$dataPerson);
    }

    protected function validateGuid($guid){
        $result     = new Result;
        if (!$guid) {
            $result->code   = 401;
            $result->info   = "Invalid token";
            return $result;
        }
        $entity = Entity::get($this, $guid);
        $user = $entity->user();
        if (!$user->id) {
            $result->code = self::CODE_SESSION_EXPIRED;
            $result->info = 'Session Expired';
        } else {
            $result->data   = $user->pull();
        }
        return $result;
    }
	
	public function writeLogAction($param) {
	
		$result = new Result();
	
		if(class_exists('Neuron\Event\Log')) { 
			$cfg = $this->getConfig();
			
			//on off log
			$off = array_key_exists('writelog', $cfg) ? $cfg['writelog'] : 'on';
			if ($off != 'off') {
				$log = new \Neuron\Event\Log(\Neuron\Event\Log\Storage::factory(null, $cfg));
				$log->write(array(
					// 'event_type_id' => isset($param['event_type_id']) ? $param['event_type_id'] : '',
					// 'event_status_id' => isset($param['event_status_id']) ? $param['event_status_id'] : '',
					'subject' => isset($param['subject']) ? $param['subject'] : '',
					'identity' => isset($param['identity']) ? $param['identity'] : '',
					'time' => date("Y-m-d H:i:s",time()),
					'name' => isset($param['name']) ? $param['name'] : '',
					'description' => isset($param['description']) ? $param['description'] : '',
					'request' => isset($param['request']) ? $param['request'] : '',
					'response' => isset($param['response']) ? $param['response'] : '',
					'data1' => isset($param['data1']) ? $param['data1'] : '',
					'data2' => isset($param['data2']) ? $param['data2'] : '',
					'data3' => isset($param['data3']) ? $param['data3'] : '',
					/* Optional */
					'details' => array(
						array(
							'event_status_id' => isset($param['event_status_id']) ? $param['event_status_id'] : '',
							'event_status_id_old' => isset($param['event_status_id_old']) ? $param['event_status_id_old'] : '',
							'description' => isset($param['description']) ? $param['description'] : '',
							'data1' => isset($param['data1']) ? $param['data1'] : '',
							'data2' => isset($param['data2']) ? $param['data2'] : ''
						)
					)
				/* Optional */
				));
			}
			
		}
	  return $result;
    }
    
    function GetRefName($param) {

        $outputData = (is_object($param)) ? $param->data : $param;
        
        if(class_exists('Neuron\Employee\Entity')){
            $arrRef_id = [];
            $arrReqBy_id = [];
            foreach($outputData['data'] as $dt) {
                if (! in_array($dt['ref_id'], $arrRef_id)) {
                    array_push($arrRef_id, $dt['ref_id']);
                }
                if (! in_array($dt['ref_id'], $arrRef_id)) {
                    array_push($arrRef_id, $dt['ref_id']);
                }
            }

            $cstorage = \Neuron\Employee\Entity\Storage::factory($this->getDb(), $this->getConfig());
            $cpersons = new \Neuron\Employee\Entity($cstorage);

            $pars['arr_ref_id'] = $arrRef_id;
            $cresult = $cpersons->listEmployee($pars)['data'];

            foreach($outputData['data'] as $key => $val) {
                foreach($cresult as $dt2) {
                    if($val['ref_id'] == $dt2['id']) {
                        $val['ref_name'] = $dt2['first_name']." ".$dt2['last_name'];
                    }
                }
                $outputData['data'][$key] = $val;
            }
        }

        return $outputData;
    }
}
