<?php

namespace Neuron\Attendance\Controller\Api;

use Neuron\Generic\Result;
use Neuron\Attendance\Request as RequestM;
use Neuron\Attendance\Requests;
use Neuron\Attendance\Request\Approval;
use Neuron\Attendance\Request\Approvals;
use Neuron\Person\Map;
use Neuron\Approval\Subtypes as ApprovalSubtypes;
use Neuron\Order\Subtype as OrderSubtype;
use Neuron\Approval\Setting as ApprovalSetting;
use Neuron\Attendance\Entity;
use Neuron\Attendance\Timetable\Changeshift;
use Neuron\Employee\Entity as Employee;
use Neuron\Notification;
use Neuron\Order\Entity as OrderEntity;
use Neuron\Application\Framework as NAF;

use Neuron\Actor\User;
use Neuron\Organization\Entity as OrganizationEntity;
use Neuron\Organization\Position;



class Request extends Controller
{
    const CODE_SESSION_EXPIRED = 411;
    const FAIL_CODE = 1;

    const ORDER_TYPE_ID_ATTENDANCE  = 2100;
    const ORDER_TYPE_ID_LEAVE       = 2200;
    const ORDER_TYPE_ATTENDANCE     = 2101;
    const ORDER_TYPE_OVERTIME       = 2102;
    const ORDER_TYPE_CHANGESHIFT    = 2103;
    const ORDER_TYPE_OVERTIME_WORKDAY    = 2110;
    const ORDER_TYPE_KEMAH    = 2122;

    /* dgunakan untuk load person mapper */
    const LOADBY_USER_ID            = 3;
    const LOADBY_PERSON_ID          = 2;
    
    public function saveRequestAction(){
        \Neuron\Application\Framework\Entity::publish();
        $result = new \Neuron\Generic\Result();
        $request = $this->getRequest();
        if (!$request->isPost()) {
            $result->code = self::FAIL_CODE;
            $result->info = 'Method not allowed';
            return $this->getREST()->getResponse($result);
        }

        $post   = $this->getREST()->getParameters();
        $guid   = $this->getREST()->getGuid();


        $approval_to = array();
        foreach ($post['assigned_to_id'] as $i => $val) {
            $approval_to[] = $val;
        }

        $data = [
            "order_type_id"        => $post['request_parent_type'],
            "order_subtype_id"     => $post['request_type'],
            "order_desc"           => $post['request_message'],
            "customer_desc"        => $post['name'],
            "create_user_id"       => $post['user_id'],
            "xd1"                  => $post['start_dtm'],
            "xd2"                  => $post['end_dtm'],
            "flow"                 => [
                "order_flow_type_id" => 1,
                "assigned_to_id"     => $approval_to,
                "remark_in"          => "Assign to user"

            ]
        ];



        $request    = new \Neuron\Approval\Entity(\Neuron\Order\Entity\Storage::factory($this->getDb(), $this->getConfig()));
        $result     = $request->create($data);

        $requestPost = array(
            'guid' => $guid,
            'code' => '',
            'info' => '',
            'data' => $post
        );

        $responseResult = array(
            'code' => $result->code,
            'info' => $result->info,
            'data' => $result->data
        );

        $logparam['subject']     = 'API ATTENDANCE REQUEST SAVEACTION';
        $logparam['identity']    = '';
        $logparam['user']        = '';
        $logparam['description'] = 'Log proses api attendance request to approval saveAction';
        $logparam['request']     = json_encode($requestPost);
        $logparam['response']    = json_encode($responseResult);

        $this->writeLogAction($logparam);

        return $this->getREST()->getResponse($result);
    }

    public function saveAction(){
        $guid       = $this->getREST()->getGuid();
        $sessmgt       = $this->validateGuid($guid);
        if($sessmgt->code != 0){
            return $this->getREST()->getResponse($sessmgt);
        }
        $post       = $this->getREST()->getParameters();
        $savedNotif = array();
        $result     = new Result();

        $storageTs  = \Neuron\Attendance\Shift\Storage::factory($this->getDb(), $this->getConfig());
        $cts        = new \Neuron\Attendance\Shift($storageTs);

        $request    = new \Neuron\Approval\Entity(\Neuron\Order\Entity\Storage::factory($this->getDb(), $this->getConfig()));

        $requestPost = array(
            'guid' => '',
            'code' => '',
            'info' => '',
            'data' => $post
        );

        $start_dtm  = ($post['start_dtm'] != null) ? $start_dtm = date("Y-m-d H:i:s", strtotime($post['start_dtm'])) : null;
        $end_dtm    = ($post['end_dtm'] != null) ? $end_dtm = date("Y-m-d H:i:s", strtotime($post['end_dtm'])) : null;
        

        if ($post['request_type'] == 2103) {
            $start_dtm = date("Y-m-d H:i:s", strtotime($post['change_date']));
        }

        # Fetching Ref ID [BEGIN]
        $session_owner          = $sessmgt->data['user_id'];
        if (class_exists('\Neuron\Employee\Entity')) {
            $map                = $this->getMapPerson($session_owner);
            $ref_id             = $map['ref_id'];
        } else {
            $ref_id             = $session_owner;
        }
        # Fetching Ref ID [END]

         # Validasi Lembur
		$storageRequest		= \Neuron\Attendance\Request\Storage::factory($this->getDb(), $this->getConfig());
        $requestValidation	= new RequestM($storageRequest);
		$validateRequest 	= $requestValidation->validateRequest($ref_id, $post['request_type'], $start_dtm, $end_dtm);

        if($validateRequest['out_statusCode'] == 0){
             # Cek timetable, apakah waktu request sesuai dengan yang diajukan [BEGIN]
            $cektimetable = $cts->loadObjectTimetable($ref_id, $start_dtm, $post['request_type']);
            if (!$cektimetable['success']) {

                $result->code       = 412;
                $result->info       = $cektimetable['message'];

                $responseResult = array(
                    'code' => $result->code,
                    'info' => $result->info,
                    'data' => $result->data
                );

                $logparam['subject'] = 'ATTENDANCE REQUEST SAVEACTION';
                $logparam['identity'] = '';
                $logparam['user'] = '';
                $logparam['description'] = 'Log proses attendance request saveaction';
                $logparam['request'] = json_encode($requestPost);
                $logparam['response'] = json_encode($responseResult);

                $this->writeLogAction($logparam);

                return $this->getREST()->getResponse($result);
            }
            # Cek timetable, apakah waktu request sesuai dengan yang diajukan [END]
            
            # Buat data untuk dikirim ke "Module Approval"

            $requesterDetail = array();
            # check user
            $employeeMap    = new \Neuron\Employee\Map(\Neuron\Employee\Map\Storage::factory($this->getDb(), $this->getConfig()));
			$employeeMap->load($ref_id,\Neuron\Employee\Map\Storage::LOADBY_EMPID);
			$requesterMap = array_merge(
                $employeeMap->employee()->pull(),
                $employeeMap->pull(),
                $employeeMap->employee()->person()->pull(),
                $employeeMap->employee()->job()->pull(),
                $employeeMap->employee()->job()->ohp()->pull(),
                $employeeMap->employee()->job()->type()->pull()
            );

            # load requester detail

            # get organization
            $orgPosition 	= new Position(Position\Storage::factory($this->getDb(),$this->getConfig()));
            $org 			= new OrganizationEntity(OrganizationEntity\Storage::factory($this->getDb(),$this->getConfig()));
            $orgPosition->load($requesterMap['position_id']);
		    $org->load($requesterMap['organization_id']);
            
            $detailOrg = array_merge($orgPosition->pull(),$org->pull());

            $requesterDetail['position'] =   $detailOrg['title']; 
            $requesterDetail['organization'] =   $detailOrg['name']; 
            $requesterDetail['code'] =   $detailOrg['code']; 
            
            $user = new User(User\Storage::factory($this->getDb(), $this->getConfig()));
            $user->pictures->load($requesterMap['user_id'], \Neuron\Actor\User\Picture\Storage::LISTBY_USER_ID);
            $pic = $user->pictures->find('active', 1);
            if($user->pictures->count() > 0){
                if($pic = $user->pictures->find('active', 1)){
                    $picture = $pic->name;
                }else{
                    $picture =  $this->getPublicURL().'/actor/picture/default';
                }
            }else{
                $picture =  $this->getPublicURL().'/actor/picture/default';
            }
    
            if ($picture) {						
                if (!file_exists(getcwd() .'/public'. $picture)) {
                    $requesterDetail['avatar'] = $this->getPublicURL().'/actor/picture/default';
                } else {
                    $requesterDetail['avatar'] = $this->getPublicURL().$picture;
                }
            } else {
                $requesterDetail['avatar']     = $this->getPublicURL().'/actor/picture/default';
            }

            $requesterDetail['birth_date'] = $requesterMap['birth_date'];
            $requesterDetail['birth_place'] = $requesterMap['birth_place'];

            $approval_to = $post['approval'];
            $data = [
                "order_type_id"        => self::ORDER_TYPE_ID_ATTENDANCE,
                "order_subtype_id"     => $post['request_type'],
                "order_desc"           => $post['request_message'],
                "customer_desc"        => $sessmgt->data['name'],
                "create_user_id"       => $sessmgt->data['user_id'],
                "xd1"                  => $start_dtm,
                "xd2"                  => $end_dtm,
                "xs15"                  => $requesterDetail,
                "flow"                 => [
                    "order_flow_type_id" => 1,
                    "assigned_to_id"     => $approval_to,
                    "remark_in"          => "Assign to user"
                ]
            ];
            
            # cek masing masing tipe request [BEGIN]
            if ($post['request_type'] == self::ORDER_TYPE_OVERTIME || $post['request_type'] == self::ORDER_TYPE_OVERTIME_WORKDAY) {
                # Validasi overtime ==> kalo request overtime harus ada attendance na dulu
                $cektimetable = $cts->loadObjectTimetable($ref_id, $start_dtm, $post['request_type']);
                if (!$cektimetable['success']) {
                    $start_date         = date("Y-m-d", strtotime($post['start_dtm']));
                    $entity             = new Entity(Entity\Storage::factory($this->getDb(), $this->getConfig()));
                    $entity->load($ref_id, Entity\Storage::LOADBY_ID, array('ref_id' => $ref_id, 'start_date' => $start_date));
                    if (!$entity->id) {
                        $result->code   = 414;
                        $result->info   = "You need to attend at $start_date to request overtime!";

                        $responseResult = array(
                            'code' => $result->code,
                            'info' => $result->info,
                            'data' => $result->data
                        );

                        $logparam['subject'] = 'ATTENDANCE REQUEST SAVEACTION';
                        $logparam['identity'] = '';
                        $logparam['user'] = '';
                        $logparam['description'] = 'Log proses attendance request saveaction';
                        $logparam['request'] = json_encode($requestPost);
                        $logparam['response'] = json_encode($responseResult);

                        $this->writeLogAction($logparam);

                        return $this->getREST()->getResponse($result);
                    }
                } else {
                    # Simpan data untuk dikirim ke "Module Approval"
                    $retval = $request->create($data);
                }
            } else if ($post['request_type'] == self::ORDER_TYPE_CHANGESHIFT) {
                $changeShift = new Changeshift(Changeshift\Storage::factory($this->getDb(), $this->getConfig()));

                # cek duplicate shift [BEGIN]
                $parCh['ref_id']             = $sessmgt['ref_id'];
                $parCh['shift_change_date']  = date('Y-m-d', strtotime($start_dtm));
                $checkChShift                = $changeShift->loadEx($id = null, Changeshift\Storage::LOADBY_REFID_CHANGEDATE, $parCh);

                if ($checkChShift->data) {
                    return $this->getREST()->getResponse(new Result(0, 511, $this->translate('You have change shift request on this date ')));
                } else {
                    $retval = $request->create($data);
                    if (isset($post['swap']) && $post['swap'] == "on") {

                        $dayno       = $cts->getDayNumber(date('Y-m-d', strtotime($post['change_date'])))->data;
                        $datachshift = $cts->loadShiftObject($post['target_id'], null);

                        if ($datachshift) {
                            foreach ($datachshift as $val) {
                                if ($val['day_no'] == $dayno) {
                                    $targetId = $val['timetable_shift_id'];
                                }
                            }
                        }

                        $chshift['tt_employee_shift_id'] = null;
                        $chshift['ref_code'] = $post['ref_code'];
                        $chshift['ref_id'] = $post['target_id'];
                        $chshift['shift_change_date'] = date('Y-m-d', strtotime($start_dtm));
                        $chshift['default_shift_id'] = $targetId;
                        $chshift['change_shift_id'] =  $cektimetable['data']['timetable_shift_id'];
                        $chshift['recommendation_id'] = $retval->data;
                        $chshift['change_by'] = $sessmgt['ref_id'];
                    } else {

                        $targetId = $post['target_id'];
                        $chshift['tt_employee_shift_id']    = null;
                        $chshift['ref_code']                = $post['ref_code'];
                        $chshift['ref_id']                  = $sessmgt['ref_id'];
                        $chshift['shift_change_date']       = date('Y-m-d', strtotime($start_dtm));
                        $chshift['default_shift_id']        = $cektimetable['data']['timetable_shift_id'];
                        $chshift['change_shift_id']         = $targetId;
                        $chshift['recommendation_id']       = $retval->data;
                        $chshift['change_by']               = $sessmgt['ref_id'];
                    }

                    $saveChShift                        = $changeShift->saveEx($chshift);
                }
                # cek duplicate shift [END]

            } else {
                $retval = $request->create($data);
            }
            # cek masing masing tipe request [END]

            # Jika sukses lakukan pengecekan untuk action lanjutan
            $employeeMap    = new \Neuron\Employee\Map(\Neuron\Employee\Map\Storage::factory($this->getDb(), $this->getConfig()));
            $employeeMap->load($ref_id,\Neuron\Employee\Map\Storage::LOADBY_EMPID);
            $requesterMap = array_merge($employeeMap->employee()->pull(), $employeeMap->pull(), $employeeMap->employee()->person()->pull());

            $targetId = null;
            if ($retval) {

                # proses kirim notifikasi [BEGIN]
                if ($approval_to) {
                    foreach ($approval_to as $appr) {
                        $employeeMap->load($appr,\Neuron\Employee\Map\Storage::LOADBY_EMPID);
                        $empMap = array_merge($employeeMap->employee()->pull(), $employeeMap->pull(), $employeeMap->employee()->person()->pull());
                        $userId     = $empMap['user_id'];

                        # proses notivikasi [BEGIN]
                        $requestTypeName            = "";
                        switch ($post['request_type']) {
                            case self::ORDER_TYPE_ATTENDANCE:
                                $requestTypeName    = 'Clock in';
                                break;
                            case self::ORDER_TYPE_OVERTIME:
                                $requestTypeName    = 'Overtime';
                                break;
                            case self::ORDER_TYPE_OVERTIME_WORKDAY:
                                $requestTypeName    = 'Overtime Workday';
                                break;
                            case self::ORDER_TYPE_CHANGESHIFT:
                                $requestTypeName    = "Change shift";
                                break;
                            default:
                                $requestTypeName    = "Clock in";
                                break;
                        }

                        $inboxStorage   = Notification\Inbox\Storage::factory(
                            $this->getDb(),
                            $this->getConfig(),
                            Notification\Profile::TYPE_FCM,
                            $userId
                        );
                        $inbox      = new Notification\Inbox($inboxStorage);
                        $inbox->setPublicUrl($this->getPublicUrl());

                        # check terlebihdahuu apakah sudah ada tabelnya atau belum
                        $result = @$inbox->pushNotification(
                            $userId,
                            'REQUEST_NEED_APPROVAL',
                            array(
                                'name' => $requesterMap['first_name'] . ' ' . $requesterMap['last_name'],
                                'request_type' => $requestTypeName,
                            ),
                            array(
                                'request_id'    => $retval->data,
                                'approval_id'   => $retval->data,
                                'assigned_to_id'   => $userId
                            )
                        );

                        if ($result->code == 0) {
                            $savedNotif[]   = $userId;
                        }
                        # proses notivikasi [END]
                    }
                }
            }

            $result         = new Result();
            $result->guid   = $guid;
            $result->data 	= array(
                'savedApproval' 	=> $approval_to,
                'request_id' 		=> $retval->data
            );

            # create log
            $responseResult = array(
                'code' => $result->code,
                'info' => $result->info,
                'data' => $result->data
            );

            $logparam['subject'] = 'ATTENDANCE REQUEST SAVEACTION';
            $logparam['identity'] = '';
            $logparam['user'] = '';
            $logparam['description'] = 'Log proses attendance request saveaction';
            $logparam['request'] = json_encode($requestPost);
            $logparam['response'] = json_encode($responseResult);

            $this->writeLogAction($logparam);
        }else{
			$result->code       = $validateRequest['out_statusCode'];
            $result->info       = $validateRequest['out_statusMsg'];
			
			$responseResult = array(
				'code' => $result->code,
				'info' => $result->info,
				'data' => null
			); 
		}

        return $this->getREST()->getResponse($result);
    }

    public function typeAction()
    {
        $guid   = $this->getREST()->getGuid();
        $result   = $this->validateGuid($guid);
        if ($result->code != 0) {
            return $this->getREST()->getResponse($result);
        }

        $resultData = [];
        $params = $this->getPostParams();
        $sub  = new ApprovalSubtypes(OrderSubtype\Storage::factory($this->getDb(),$this->getConfig()));
        $result = $sub->load(self::ORDER_TYPE_ID_ATTENDANCE, OrderSubtype\Storage::LISTBY_ORDER_TYPE);
        $lists  = $result->data['list'];
        $rows   = [];
        foreach($lists as $row){
            $setting        = new ApprovalSetting(ApprovalSetting\Storage::factory($this->getDb(),$this->getConfig()));
            $loadSetting    = $setting->load($row['setting_id'],ApprovalSetting\Storage::LOADBY_ID);
            $tempSetting    = $loadSetting->data->setting;
            $tempSetting['detail'] = $loadSetting->data->detail;
            $mandatory_end_date = true;
            if($row['order_subtype_id'] == 2101 || $row['order_subtype_id'] == 2122){
                $mandatory_end_date = false;
            }
            $rows[] = [
                "type_id" => $row['order_subtype_id'],
                "type_name" => $row['name'],
                "setting_id" => $row['setting_id'],
                "mandatory_end_date" => $mandatory_end_date,
                'setting' => $tempSetting
            ];
        }

        $resultData['rows'] = $rows;

        $result         = new \Neuron\Generic\Result;
        $result->data   = $resultData;
        $result->guid   = $guid;

        $requestPost = array(
            'guid' => $guid,
            'code' => '',
            'info' => '',
            'data' => ''
        );

        $responseResult = array(
            'code' => $result->code,
            'info' => $result->info,
            'data' => $result->data
        );

        $logparam['subject'] = 'API ATTENDANCE REQUEST TYPEACTION';
        $logparam['identity'] = '';
        $logparam['user'] = '';
        $logparam['description'] = 'Log proses api attendance request typeAction';
        $logparam['request'] = json_encode($requestPost);
        $logparam['response'] = json_encode($responseResult);

        $this->writeLogAction($logparam);

        return $this->getREST()->getResponse($result);
    }

    public function detailAction(){
        $guid   = $this->getREST()->getGuid();
        $validateGuid = $this->validateGuid($guid);
        if($validateGuid->code != 0){
            $validateGuid->info   = $this->translate($validateGuid->info);
            return $this->getREST()->getResponse($validateGuid);
        }

        $post       = $this->getREST()->getParameters();

        $loadData['order_id'] = $post['request_id'];
        $loadData['flow'] = 'list';
        
        $mode = OrderEntity\Storage::LOADBY_ID;
        $entity = new \Neuron\Approval\Entity(\Neuron\Order\Entity\Storage::factory($this->getDb(), $this->getConfig()));
        $result = $entity->loadEx($loadData, $mode);
        $order = $result->data;
        $personMap = $this->loadMap($order['create_user_id']);
        $requesterDetail = $this->loadPersonDetail($personMap['person_id']);
        $refId  = $personMap['person_id'];
        $orderSubtype   = $this->loadSubtype($order['order_subtype_id']);
        $orderStatus    = $this->loadOrderStatus($order['order_status_id']);
        $naf            = NAF\Entity::get();
        $homeURL        = $naf->layout()->getHomeURL();

        $approvals  = $order['flow'];
        $newApprovals = [];
        foreach($approvals as $approval){
            if($approval['order_flow_task_code'] == 'approval'){
                $map    = $this->loadMap($approval['assigned_to_id']);
                $personDetail = $this->loadPersonDetail($map['person_id']);
                $newApprovals[] = [
                    "approval_id" => $approval['order_flow_id'],
                    "request_id" => $approval['order_id'],
                    "ref_code" => "EMPLOYEE",
                    "ref_id" => $map['person_id'],
                    "status_id" => $approval['order_flow_status_id'],
                    "approval_dtm" => $approval['update_dtm'],
                    "reason" => $approval['remark_out'],
                    "ref_name" => $approval['assignee_name'],
                    "request_type" => $order['order_subtype_id'],
                    "requester_ref_id" => $refId,
                    "request_dtm" => $approval['create_dtm'],
                    "requester_ref_name" => $approval['assigner_name'],
                    "avatar" => $homeURL.$personDetail->pic_official,
                    "request_type_name" => $orderSubtype->name,
                    "status_id_name" => $approval['status_name'],
                ];
            }
        }
        if($order['xn5'] == 1){
            $order_status_name = 'Requested';
        }else if($order['xn5'] == 2){
            $order_status_name = 'Approved';
        }else if($order['xn5'] == 3){
            $order_status_name = 'Rejected';
        }else{
            $order_status_name = 'Requested';
        }
        
        $newData    = [
            "request_data" => [
                "request_id" => $order['order_id'],
                "ref_code" => "EMPLOYEE",
                "ref_id" => $refId,
                "start_dtm" => $order['xd1'],
                "end_dtm" => $order['xd2'],
                "request_type" => $order['order_subtype_id'],
                "request_dtm" => $order['create_dtm'],
                "status_id" => $order['xn5'],
                "request_message" => $order['order_desc'],
                "request_type_name" => $orderSubtype->name,
                "status_name" => $order_status_name,
                "avatar" => $homeURL.$requesterDetail->pic_official,
                "ref_name" => $order['customer_desc'],
            ],
            "approval_data"     => $newApprovals
        ];

        $result     = new \Neuron\Generic\Result;
        $result->guid = $guid;
        $result->data   = $newData;

        $responseResult = array(
			'code' => $result->code,
			'info' => $result->info,
			'data' => $result->data
		); 

		$logparam['subject'] = 'API ATTENDANCE REQUEST LOAD DETAIL';
		$logparam['identity'] = $guidData['user_id'] ? $user_id : '';
		$logparam['user'] = $guidData['user_id'] ? $user_id : '';
		$logparam['description'] = 'Log proses API saat load detail request';
		$logparam['request'] = json_encode($this->getREST()->getParameters());
		$logparam['response'] = json_encode($responseResult);

		$this->writeLogAction($logparam);

        return $this->getREST()->getResponse($result);

    }

    protected function validateGuid($guid)
    {
        $result     = new Result;
        if (!$guid) {
            $result->code   = 401;
            $result->info   = "Invalid token";
            return $result;
        }
        $entity = \Neuron\Application\Framework\Entity::get($this, $guid);
        $user = $entity->user();
        if (!$user->id) {
            $result->code = self::CODE_SESSION_EXPIRED;
            $result->info = 'Session Expired';
        } else {
            $result->data   = $user->pull();
        }
        return $result;
    }

    public function countByTypeAction()
    {
        $post       = $this->getREST()->getParameters();
        $guid       = $this->getREST()->getGuid();
        $result   = $this->validateGuid($guid);

        if ($result->code != 0) {
            return $this->getREST()->getResponse($result);
        }

        $user_id    = $result->data['user_id'];
        /* Load Map */
        if (class_exists('\Neuron\Employee\Entity')) {
            $empStorage     = \Neuron\Employee\Entity\Storage::factory($this->getDb(), $this->getConfig());
            $emp            = new \Neuron\Employee\Entity($empStorage);
            $map            = $emp->loadMap($user_id, 'user_id');
            if ($map['data']) {
                $ref_id     = $map['data']['person_id'];
            } else {
                $result->code   = 500;
                $result->info   = 'Failed loading map';
                return $this->getREST()->getResponse($result);
            }
        } else {
            /* Jika tidak ada module employee */
            $ref_id         = 1;
        }

        // $approvalStorage    = RequestM\Storage::factory($this->getDb(), $this->getConfig());
        // $approvals          = new Requests($approvalStorage);
        // $result             = $approvals->loadCountByType($ref_id, RequestM\Storage::LISTBY_REFID, array('ref_id' => $ref_id));
        $sApproval = \Neuron\Order\Entity\Storage::factory($this->getDb(), $this->getConfig());
        $mApproval = new \Neuron\Approval\Entities($sApproval);

        $sub  = new ApprovalSubtypes(OrderSubtype\Storage::factory($this->getDb(),$this->getConfig()));
        $result = $sub->load(self::ORDER_TYPE_ID_ATTENDANCE, OrderSubtype\Storage::LISTBY_ORDER_TYPE);
        $lists  = $result->data['list'];
        $rows   = [];
        foreach($lists as $row){
            $item["type_id"] = $row["order_subtype_id"];
            $item["type_name"] = $row["name"];

            $where['where']['create_user_id'] = $user_id;
            $where['where']['order_subtype_id'] = $row['order_subtype_id'];
            $where['where']['order_type_id'] = self::ORDER_TYPE_ID_ATTENDANCE;
            $where['where']['order_status_id'] = 1;
            $dataListTerkini = $mApproval->load(null, \Neuron\Order\Entity\Storage::LIST_ALL, $where, true);

            $where['where']['create_user_id'] = $user_id;
            $where['where']['order_subtype_id'] = $row['order_subtype_id'];
            $where['where']['order_type_id'] = self::ORDER_TYPE_ID_ATTENDANCE;
            $where['where']['order_status_id'] = 2;
            $dataListTerdahulu = $mApproval->load(null, \Neuron\Order\Entity\Storage::LIST_ALL, $where, true);

            $terkini = ($dataListTerkini->data) ? count($dataListTerkini->data) : 0;
            $terdahulu = ($dataListTerdahulu->data) ? count($dataListTerdahulu->data) : 0;

            $item["terkini"] = strval($terkini);
            $item["terdahulu"] = strval($terdahulu);
            if($row['order_subtype_id'] == 2101 || $row['order_subtype_id'] == 2122){
                $item['mandatory_end_date'] = false;
            }else{
                $item['mandatory_end_date'] = true;
            }

            $rows[] = $item;

        }
        
        
        $result->guid       = $guid;
        $result->data       = $rows;

        $requestPost = array(
            'guid' => $guid,
            'code' => '',
            'info' => '',
            'data' => $post
        );

        $responseResult = array(
            'code' => $result->code,
            'info' => $result->info,
            'data' => $result->data
        );

        $logparam['subject'] = 'API ATTENDANCE REQUEST COUNTBYTYPEACTION';
        $logparam['identity'] = '';
        $logparam['user'] = '';
        $logparam['description'] = 'Log proses api attendance request countByTypeAction';
        $logparam['request'] = json_encode($requestPost);
        $logparam['response'] = json_encode($responseResult);

        $this->writeLogAction($logparam);

        return $this->getREST()->getResponse($result);
    }

    public function loadListAction(){
        $guid   = $this->getREST()->getguid();
        $validateGuid   = $this->validateGuid($guid);
        if($validateGuid->code != 0){
            return $this->getREST()->getResponse($validateGuid);
        }
        $guidData = $validateGuid->data;
        $storageE = \Neuron\Employee\Entity\Storage::factory($this->getDb(), $this->getConfig());
        $m_employee = new \Neuron\Employee\Entity($storageE);
        $dataEmployee = $m_employee->loadMap($guidData['user_id'],'user_id');
        
        $storageDetail = \Neuron\Employee\Detail\Storage::factory($this->getDb(), $this->getConfig());
        $detail = new \Neuron\Employee\Detail($storageDetail);
        $detail->loadEmpDet($dataEmployee['data']['person_id']);
        $detailEmployee = $detail->pull();
    
        $post = $this->getREST()->getParameters();
        $statusId   = isset($post['status_id']) ? $post['status_id'] : 1;
        $userId     = $guidData['user_id'];
        $refId      = $dataEmployee['data']['person_id'];
        /* Load Person Map */

        /* Load orders */
        $filter = [];
        $filter['where']['create_user_id'] = $userId;
        $filter['where']['order_subtype_id'] = $post['request_type'];
        $filter['where']['order_type_id'] = self::ORDER_TYPE_ID_ATTENDANCE;
        $filter['where']['order_status_id'] = $post['status_id'];
        $filter['flow'] = 'list';

        $entities = new \Neuron\Approval\Entities(
            OrderEntity\Storage::factory($this->getDb(),$this->getConfig())
        );
        $load = $entities->load($statusId,OrderEntity\Storage::LISTBY_STATUS,$filter);

        $resultData = [];
        $rows       = [];

        if($load->code == 0){
            $lists  = $load->data;
            foreach($lists as $row){
                // $settings   = new \Neuron\Approval\Setting(\Neuron\Approval\Setting\Storage::factory($this->getDb(),$this->getConfig()));
                // $settings->load($row['xn2']);
                $newFormData    = [
                    "request_type" => $row['order_subtype_id'],
                    "ref_code" => "EMPLOYEE",
                    "ref_id" => $refId,
                    "start_dtm" => $row['xd1'],
                    "end_dtm" => $row['xd2'],
                    "request_message" => $row['order_desc'],
                    "status_id" => $row['xn5'],
                    "request_type_name" => $row['order_subtype_name'],
                    "total_approve" => intval($row['xn3']),
                    "total_reject" => intval($row['xn4']),
                    // "description" => $row['order_subtype_name'],
                    // "include_holiday" => "N",
                    // "num_to_approve" => $settings->num_to_approve,
                    // "min_approved" => $settings->min_approved,
                    // "min_rejected" => $settings->min_rejected,
                    // "sequential_approval" => $settings->sequential,
                    // "direct_reject" => $settings->direct_reject,
                    "status_id_name" => $row['order_status_name'],
                    "status_name" => $row['order_status_name'],
                    "request_id" => $row['order_id'],
                    "ref_name" => $dataEmployee['data']['first_name'].' '.$dataEmployee['data']['last_name'],
                    "avatar" => $this->getPublicURL() . $detailEmployee['pic_official']
                ];

                $mApproval = new \Neuron\Approval\Entity(OrderEntity\Storage::factory($this->getDb(),$this->getConfig()));
                $params['order_id'] = $row['order_id'];
                $params['flow'] = 'list';
                $dataOrder = $mApproval->loadEx($params);
                $flow = $dataOrder->data['flow'];
                if($flow){
                    foreach($flow as $perApproval) {
                        if($perApproval['order_flow_task_code'] == 'approval'){
                            $dataApproval = $m_employee->loadMap($perApproval['assigned_to_id'],'user_id');
                            $detail->loadEmpDet($dataApproval['data']['person_id']);
                            $detailApproval = $detail->pull();

                            $addApproval["approval_id"] = $perApproval['order_id'];
                            $addApproval["request_id"] = $perApproval['order_id'];
                            $addApproval["ref_code"] = "EMPLOYEE";
                            $addApproval["ref_id"] = $dataApproval['data']['person_id'];
                            $addApproval["status_id"] = $perApproval['order_flow_status_id'];
                            $addApproval["approval_status_id"] = $perApproval['order_flow_status_id'];
                            $addApproval["approval_status_name"] = $perApproval['status_name'];
                            $addApproval["approval_dtm"] = ($perApproval['update_dtm'] == '0000-00-00 00:00:00') ? '':$perApproval['update_dtm'];
                            $addApproval["reason"] = $perApproval['remark_out'];
        
                            $addApproval["ref_name"] = $dataApproval['data']['first_name'].' '.$dataApproval['data']['last_name'];
                            $addApproval["start_dtm"] = $row['xd1'];
                            $addApproval["end_dtm"] = $row['xd2'];
        
                            $addApproval["request_type"] = $row['order_subtype_id'];
                            $addApproval["request_dtm"] = $row['create_dtm'];
                            $addApproval["request_message"] = $row['order_desc'];
                            $addApproval["requester_ref_id"] = $dataEmployee['data']['person_id'];
                            $addApproval["requester_ref_name"] = $row['create_user_name'];
                            $addApproval["avatar"] = $this->getPublicURL() . $detailApproval['pic_official'];
                            $addApproval["request_type_name"] = $row['order_subtype_name'];
                            $addApproval["status_id_name"] = $perApproval['status_name'];
                            $addApproval["request_status_id"] = $row['order_status_id'];;
        
                            $newFormData['approval'][] = $addApproval;
                        }
                    }
                }

                $rows[] = $newFormData;
            }
        }
        
    

        $resultData['countTotal'] = strval(count($rows));
        $resultData['rows']     = $rows;
        $resultData['fullname'] = $guidData['name'];
        
        $result     = new \Neuron\Generic\Result;

        $result->data   = $resultData;
        $result->guid   = $guid;
        $result->info   = $this->translate($result->info);
        return $this->getREST()->getResponse($result);
        
    }

    public function loadShiftAction()
    {
        $result     = new Result();
        $post   = $this->getREST()->getParameters();
        $guid   = $this->getREST()->getGuid();


        $result   = $this->validateGuid($guid);
        if ($result->code != 0) {
            return $this->getREST()->getResponse($result);
        }

        $result->guid = $guid;


        $user       = $result->data;
        $map        = new Map(Map\Storage::factory($this->getDb(), $this->getConfig()));
        $map->load($user['user_id'], Map\Storage::LOADBY_USER_ID);
        $dataMap    = $map->pull();
        if ($dataMap['person_id']) {
            $ref_id     = $dataMap['person_id'];
        } else {
            $result->code   = 1;
            $result->info   = "Failed loading Map";
            return $this->getREST()->getResponse($result);
        }

        $storage = \Neuron\Attendance\Shift\Storage::factory($this->getDb(), $this->getConfig());
        $cs = new \Neuron\Attendance\Shift($storage);

        $dayno = $cs->getDayNumber($post['date'])->data;

        $owntimetable = array();
        $arrShift = array();

        if ($post['swap'] == 'true') {
            $dataShift = $cs->loadShiftObject(null, date('Y-m-d', strtotime($post['date'])));
        } else {
            $dataShift = $cs->loadShiftByDate(date('Y-m-d', strtotime($post['date'])), $dayno);
        }

        $own = $cs->loadShiftObject($ref_id, date('Y-m-d', strtotime($post['date'])));
        if ($own) {
            foreach ($own as $val) {
                array_push($owntimetable, $val['timetable_id']);
            }
        }

        if ($dataShift) {
            foreach ($dataShift as $val) {
                // print_r($val);
                if ($val['day_no'] == $dayno) {
                    if ($post['swap'] == 'true') {
                        if ($val['ref_id'] != $ref_id && !in_array($val['timetable_id'], $owntimetable)) {
                            array_push($arrShift, $val);
                        }
                    } else {
                        if (!in_array($val['timetable_id'], $owntimetable)) {
                            array_push($arrShift, $val);
                        }
                    }
                }
            }
        }

        if ($post['swap'] == 'true') {
            $retval = $this->GetRefName(array('data' => $arrShift));
            $result->data   = $retval['data'];
        } else {
            $result->data   = $arrShift;
        }
        // print_r($result['data']); die();
        $requestPost = array(
            'guid' => $guid,
            'code' => '',
            'info' => '',
            'data' => $post
        );

        $responseResult = array(
            'code' => $result->code,
            'info' => $result->info,
            'data' => $result->data
        );

        $logparam['subject'] = 'API ATTENDANCE REQUEST LOADSHIFTACTION';
        $logparam['identity'] = '';
        $logparam['user'] = '';
        $logparam['description'] = 'Log proses api attendance request loadShiftAction';
        $logparam['request'] = json_encode($requestPost);
        $logparam['response'] = json_encode($responseResult);

        $this->writeLogAction($logparam);

        return $this->getREST()->getResponse($result);
    }

    public function cancelOldAction(){
        $post   = $this->getREST()->getParameters();

        $guid   = $this->getREST()->getGuid();
        $result   = $this->validateGuid($guid);
        if ($result->code != 0) {
            return $this->getREST()->getResponse($result);
        }

        $result     = new \Neuron\Generic\Result();

        if (!isset($post['request_id']) || empty($post['request_id'])) {
            $result->code   = 400;
            $result->info   = "request_id required!";
            return $this->getREST()->getResponse($result);
        }

        $storage = \Neuron\Attendance\Request\Storage::factory($this->getDb(), $this->getConfig());
        $modelAttendance = new \Neuron\Attendance\Request($storage);
        $data = $modelAttendance->loadRequest('request_id', $post['request_id']);

        foreach ($data as $row) {
            $attendance = $row;
            $attendance['status_id'] = 4;

            $model = new \Neuron\Attendance\Request($storage, $attendance);
            $attendance_id = $model->save(true);
            if (!$attendance_id) {
                $result->code         = 502;
                $result->info         = "Failed Cancel Request";
                return $this->getREST()->getResponse($result);
            }
            $result->data = $attendance_id;
        }
        
        $result->guid = $guid;
        $result->info = "Success Cancel Request";

        $requestPost = array(
            'guid' => $guid,
            'code' => '',
            'info' => '',
            'data' => $post
        );

        $responseResult = array(
            'code' => $result->code,
            'info' => $result->info,
            'data' => $result->data
        );

        $logparam['subject'] = 'API ATTENDANCE REQUEST DETAILACTION';
        $logparam['identity'] = '';
        $logparam['user'] = '';
        $logparam['description'] = 'Log proses api attendance request detailAction';
        $logparam['request'] = json_encode($requestPost);
        $logparam['response'] = json_encode($responseResult);

        $this->writeLogAction($logparam);

        return $this->getREST()->getResponse($result);
    }


    /** 
     * Ini udah pake order, cuman emang modelnya pake di request,
     * biar web sama API sama logicnya 
     * 
     * */
    public function cancelAction(){
        $guid   = $this->getREST()->getGuid();
        $validateGuid = $this->validateGuid($guid);
        if($validateGuid->code != 0){
            return $this->getREST()->getResponse($validateGuid);
        }
        $post       = $this->getREST()->getParameters();
        $requestId  = $post['request_id'];
        
        $attendanceRequest  = new \Neuron\Attendance\Request(
            \Neuron\Attendance\Request\Storage::factory(
                $this->getDb(),
                $this->getconfig()
            )
        );
        $result = $attendanceRequest->cancel($requestId);
        $result->guid = $guid;
        
        /* Logging */
        $requestPost = array(
            'guid' => $guid,
            'code' => '',
            'info' => '',
            'data' => $post
        );

        $responseResult = array(
            'code' => $result->code,
            'info' => $result->info,
            'data' => $result->data
        );

        $logparam['subject'] = 'API ATTENDANCE REQUEST DETAILACTION';
        $logparam['identity'] = '';
        $logparam['user'] = '';
        $logparam['description'] = 'Log proses api attendance request detailAction';
        $logparam['request'] = json_encode($requestPost);
        $logparam['response'] = json_encode($responseResult);

        $this->writeLogAction($logparam);

        return $this->getREST()->getResponse($result);
    }

    public function loadMap($id, $mode = self::LOADBY_USER_ID){
        $personMap  = new \Neuron\Person\Map(\Neuron\Person\Map\Storage::factory(
            $this->getDb(),
            $this->getConfig()
        ));
        $personMap->load($id,$mode);
        return $personMap->pull();
    }

    public function loadSubtype($subtypeId){
        $orderSubtype   = new \Neuron\Order\Subtype(
            \Neuron\Order\Subtype\Storage::factory(
                $this->getDb(),
                $this->getConfig()
            )
        );
        $orderSubtype->load($subtypeId);
        return $orderSubtype;
    }

    public function loadOrderStatus($statusId){
        $orderStatus    = new \Neuron\Order\Status(\Neuron\Order\Status\Storage::factory(
            $this->getDb(),
            $this->getConfig()
        ));
        $orderStatus->load($statusId);
        return $orderStatus;

    }

    public function loadPersonDetail($personId){
        $personDetail   = new \Neuron\Person\Detail(
            \Neuron\Person\Detail\Storage::factory(
                $this->getDb(),
                $this->getConfig()
            )
        );
        $personDetail->load($personId,\Neuron\Person\Detail\Storage::LOADBY_PERSON);
        return $personDetail;
    }
}
