<?php
namespace Neuron\Attendance\Controller\Api;

use Neuron\Actor;
use Neuron\Composite;
use Neuron\Generic\Result;  

class User extends \Neuron\Attendance\Controller\Api\Controller {
    const EVENT_SUBJECT = 'User';
    const SUCCESS_CODE = 0;
    const FAIL_CODE = 1;
    const FAIL_CODE_SESSION = 11;

    const CODE_FIELD_REQUIRED  = 400;
    const CODE_UNAUTHORIZED = 401;
    const CODE_SESSION_EXPIRED = 403;

    public function listAction() {

        /*
        input: current: nomor halaman
               rowCount: row per halaman
        logic: Actor\Users->load()
        */
    }

    public function authenticateAction() {
        $this->publish();
        
        try {
            $posts = $this->getRequest()->getPost('data');
            $data = json_decode($posts, true);
			
			$requestPost = array(
				'guid' => '',
				'code' => '',
				'info' => '',
				'data' => $posts
			); 

            /* check mandatory parameters */
            $mandatoryData = array(
                'code',
                'password'
            );

            if (!is_array($data)) {
                $result = new Result();
                $result->guid = 0;
                $result->code = 110;
                $result->info = 'mandatory_parameters_error';
				
				$responseResult = array(
					'code' => $result->code,
					'info' => $result->info,
					'data' => $result->data
				); 

				$logparam['subject'] = 'API ATTENDANCE USER AUTHENTICATEACTION';
				$logparam['identity'] = '';
				$logparam['user'] = '';
				$logparam['description'] = 'Log proses api attendance user authenticateAction';
				$logparam['request'] = json_encode($requestPost);
				$logparam['response'] = json_encode($responseResult);

				$this->writeLogAction($logparam);
				
                return $this->getOutput($result->toJson());
            }

            foreach ($mandatoryData as $key) {
                if (!array_key_exists($key, $data)) {
                    $result = new Result();
                    $result->guid = 0;
                    $result->code = 110;
                    $result->info = 'mandatory_parameters_error';
                    $result->data = json_encode(array(
                        'parameter' => $key
                    ));
					
					$responseResult = array(
						'code' => $result->code,
						'info' => $result->info,
						'data' => $result->data
					); 

					$logparam['subject'] = 'API ATTENDANCE USER AUTHENTICATEACTION';
					$logparam['identity'] = '';
					$logparam['user'] = '';
					$logparam['description'] = 'Log proses api attendance user authenticateAction';
					$logparam['request'] = json_encode($requestPost);
					$logparam['response'] = json_encode($responseResult);

					$this->writeLogAction($logparam);
					
                    return $this->getOutput($result->toJson());
                }
            }

            /* Init storage &user object */

            $storage = \Neuron\Composite\User\Storage::factory($this->getDb(), $this->getConfig());
            $user = new \Neuron\Composite\User($storage, $this->getConfig()['user']);

            /* Load user by user code */
            $user->load($data['code'], \Neuron\Actor\User\Storage::LOADBY_CODE);

            /* Do auth */
            $result = $user->authenticate($data['password']);

            /* Open session and set owner */
            if ($result->code == $result::CODE_SUCCESS) {
                $data = $user->pull();
                unset($data['password']);
                unset($data['create_dtm']);
                unset($data['active_dtm']);
                unset($data['terminate_dtm']);
                unset($data['nonce']);
                unset($data['token']);
                unset($data['auth_dtm']);
                $session = $this->getSession();
                $session->start();
                $session->owner($user->id);
                $session->put(null, $data);
                $session->flush();

                $sessid = $session->getId();
                $datares = array(
                    'username' => $data['code'],
                    'fullname'=> $data['name'],
                    'userstatus' => $data['user_status_id'],
                    'role' => 'user',
                    'token' => $sessid
                );
            }

            /* Return result */
//            echo($result->toJson());
//            return false;
            $result->guid = $user->id;
            $result->code = self::SUCCESS_CODE;
            $result->info = 'OK';
            $result->data = $datares;
			
			$responseResult = array(
				'code' => $result->code,
				'info' => $result->info,
				'data' => $result->data
			); 

			$logparam['subject'] = 'API ATTENDANCE USER AUTHENTICATEACTION';
			$logparam['identity'] = '';
			$logparam['user'] = '';
			$logparam['description'] = 'Log proses api attendance user authenticateAction';
			$logparam['request'] = json_encode($requestPost);
			$logparam['response'] = json_encode($responseResult);

			$this->writeLogAction($logparam);

			
            return $this->getREST()->getResponse($result);

            // return $this->getOutput($result->toJson());
        } catch (\Exception $ex) {
            $result = new Result();
            $result->code = self::FAIL_CODE;
            $result->info = 'user_auth_error';
            $result->data = json_encode(array(
                'exception' => $ex->getMessage()
            ));
			
			$responseResult = array(
				'code' => $result->code,
				'info' => $result->info,
				'data' => $result->data
			); 

			$logparam['subject'] = 'API ATTENDANCE USER AUTHENTICATEACTION';
			$logparam['identity'] = '';
			$logparam['user'] = '';
			$logparam['description'] = 'Log proses api attendance user authenticateAction';
			$logparam['request'] = json_encode($requestPost);
			$logparam['response'] = json_encode($responseResult);

			$this->writeLogAction($logparam);
			
            return $this->getOutput($result->toJson());
        }
    }

    public function registerAction() {
        
        /* get parameters (JSON) */
        $data = $this->getREST()->getParameters();
		
		$requestPost = array(
			'guid' => '',
			'code' => '',
			'info' => '',
			'data' => $data
		); 
		
        /* init storage &user object */
        $storage = Composite\User\Storage::factory($this->getDb(), $this->getConfig());
        $user = new Composite\User($storage, $this->getConfig()['actor']['user']);

        /* register */
        $result = $user->register($data['code'], $data['name'], $data['email'], @$data['password'], @$data['language'], @$data['auth_type_id'], array('retype_password' => @$data['retype_password']));
		
		$responseResult = array(
			'code' => $result->code,
			'info' => $result->info,
			'data' => $result->data
		); 

		$logparam['subject'] = 'API ATTENDANCE USER REGISTERACTION';
		$logparam['identity'] = '';
		$logparam['user'] = '';
		$logparam['description'] = 'Log proses api attendance user registerAction';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode($responseResult);

		$this->writeLogAction($logparam);
		
        return $this->getREST()->getResponse($result);
    }
    
    public function forgotPasswordAction(){
        $this->publish();
        
        $result = new Result();
        $data = $this->getREST()->getParameters();
        $session = $this->getSession();
        
        if(class_exists('\Neuron\Utility\Validator')){
            $validator = new \Neuron\Utility\Validator();
            $result = $validator->validate($data, array(
                'csrf' => array('identical' => array('token' => $session->get('csrf'), 'message' => 'Invalid token')),
                'email' => array('email' => array('message' => 'Mohon masukan email dengan format yang benar')),
            ));
            
            if($result->code == $result::CODE_SUCCESS){
                $storage = Composite\User\Storage::factory($this->getDb(), $this->getConfig());
                $user = new Composite\User($storage, $this->getConfig()['actor']['user']);
                $result = $user->forgotPassword($data, $this->getPublicUrl());
            }
        } else {
            $result->code = 201;
            $result->info = 'you need library \Neuron\Utility\Validator to perform this request';
        }
		
		$requestPost = array(
			'guid' => '',
			'code' => '',
			'info' => '',
			'data' => $data
		); 
        
		$responseResult = array(
			'code' => $result->code,
			'info' => $result->info,
			'data' => $result->data
		); 

		$logparam['subject'] = 'API ATTENDANCE USER FORGOTPASSWORDACTION';
		$logparam['identity'] = '';
		$logparam['user'] = '';
		$logparam['description'] = 'Log proses api attendance user forgotPasswordAction';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode($responseResult);

		$this->writeLogAction($logparam);
		
        return $this->getREST()->getResponse($result);
    }
    
    public function changePasswordAction(){
        $this->publish();
        
        $result = new Result();
        $data = $this->getREST()->getParameters();
        $session = $this->getSession();
        
        if(class_exists('\Neuron\Utility\Validator')){
            $validator = new \Neuron\Utility\Validator();
            $result = $validator->validate($data, array(
                'csrf' => array('identical' => array('token' => $session->get('csrf'), 'message' => 'Invalid token')),
                'email' => array('email' => array('message' => 'Mohon masukan email dengan format yang benar')),
                'new_pass' => array('strlen' => array('min' => 8, 'message' => 'Minimal password 8 karakter')),
                'retype_pass' => array('identical' => array('token' => $data['new_pass'], 'message' => 'Password tidak sama')),
                'nonce' => array('notempty' => array('message' => 'Invalid nonce')),
            ));
            
            if($result->code == $result::CODE_SUCCESS){
                $storage = Composite\User\Storage::factory($this->getDb(), $this->getConfig());
                $user = new Composite\User($storage, $this->getConfig()['actor']['user']);
                $result = $user->changePassword($data);
            }
        } else {
            $result->code = 201;
            $result->info = 'you need library \Neuron\Utility\Validator to perform this request';
        }
        
		$requestPost = array(
			'guid' => '',
			'code' => '',
			'info' => '',
			'data' => $data
		); 
        
		$responseResult = array(
			'code' => $result->code,
			'info' => $result->info,
			'data' => $result->data
		); 

		$logparam['subject'] = 'API ATTENDANCE USER CHANGEPASSWORDACTION';
		$logparam['identity'] = '';
		$logparam['user'] = '';
		$logparam['description'] = 'Log proses api attendance user changePasswordAction';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode($responseResult);

		$this->writeLogAction($logparam);
		
        return $this->getREST()->getResponse($result);
    }

}
