<?php
namespace Neuron\Attendance\Controller;

use Neuron\Attendance\Shift;
use Neuron\Attendance\Attendance as AttendanceModel;
use Neuron\Application\Framework\Entity;
use Neuron\Attendance\Entity as AttendanceEntity;
use Neuron\Generic\Result;
use Neuron\Calendar\Event;
use Neuron\Calendar\Events;
use Neuron\Approval\Entities;
use Neuron\Approval\Entity as ApprovalEntity;
use Neuron\Attendance\Traits\Mapper;
use Neuron\Application\Framework as NAF;
use Neuron\Person\Entity as PersonEntity;

class Attendance extends Controller {
    use Mapper;

    const SUCCESS_CODE = 0;
    const FAIL_CODE = 1;
    const CODE_FIELD_REQUIRED  = 400;
    const CODE_UNAUTHORIZED = 401;
    const CODE_SESSION_EXPIRED = 403;

    const ATTEND_TYPE_IN                = 0;
    const ATTEND_TYPE_OUT               = 1;
    CONST ATTEND_TYPE_NOT_IN_RANGE_IN   = 2;
    CONST ATTEND_TYPE_NOT_IN_RANGE_OUT  = 3;
    CONST ATTEND_TYPE_ALREADY_IN        = 4;
    CONST ATTEND_TYPE_ALREADY_OUT       = 5;
    
	protected function getView($name, $variables = array()) {
        $layout = 'neuron/attendance/layout/form';
        $prefix = 'neuron/attendance/attendance';

        /* libraries */
        if(! class_exists('Neuron\Application\Framework\Entity')) {
            $this->using('font-awesome', 10, 'https://use.fontawesome.com/releases/v5.8.1/css/all.css')
                ->using(
                    'bootstrap-4',
                    null,
                    array(
                        'https://ajax.googleapis.com/ajax/libs/jquery/3.3.1/jquery.min.js',
                        'https://cdnjs.cloudflare.com/ajax/libs/popper.js/1.14.7/umd/popper.min.js',
                        'https://maxcdn.bootstrapcdn.com/bootstrap/4.3.1/css/bootstrap.min.css',
                        'https://maxcdn.bootstrapcdn.com/bootstrap/4.3.1/js/bootstrap.min.js'
                    )
                )
                ->using('toast',null)
                ->using('bootbox',null);
        }

        $this->using('select2', 22, array(
            'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.10/js/select2.min.js',
            /* ini load sesuai bahasa dari session */
            'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.10/js/i18n/id.js',
            /**/
            'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.10/css/select2.min.css',
            'https://cdn.jsdelivr.net/npm/@ttskch/select2-bootstrap4-theme@1.3.2/dist/select2-bootstrap4.min.css',
        ));

        /* get layout n view prefix from config */
        $config = $this->getConfig();
        if (isset($config['attendance'])) {
            $config = $config['attendance'];
            $layout = isset($config['formlayout']) ? $config['formlayout'] : $layout;
            $prefix = isset($config['viewprefix']) ? $config['viewprefix'] : $prefix;
        }
        
       	/* set layout and view */
		$this->layout($layout);
		$view = new \Zend\View\Model\ViewModel($variables);
		$view->setTemplate($prefix . '/' . strtolower($name));

        return $view;
    }
    
    public function ManageAction(){
        $this->usingjs('bootbox', 21);
        $this->using('p-loading', 22);
        $this->using('bootstrap-select', 23);
        $this->using('generic', 24);

        $this->using('datatable', 20, array(
            'https://cdn.datatables.net/1.10.19/js/jquery.dataTables.min.js',
            'https://cdn.datatables.net/1.10.19/js/dataTables.bootstrap4.min.js',
            'https://cdn.datatables.net/1.10.19/css/dataTables.bootstrap4.min.css'
        ));

        $this->using('air-datepicker',28);
        $this->using('moment',29);
        $this->using('jquery.validate',30);
    
        $sessmgt = $this->getSession()->get();
        $sess = (isset($sessmgt['ref_id'])) ? true : false;
        $title = $this->translate("Attendance");

        $layout = $this->layout();
		// $layout->setVariables(['previous' => '<a href="'.$this->getPublicURL().'/index/my-request">Leave</a>']);
		$layout->setVariables(['current' => 'Manage Attendance']);
        
        return $this->getView('list', get_defined_vars());
    }
    
    public function loadParamAction() {
        // throw new \Exception("Test Error", 1);

		$refstorage = \Neuron\Attendance\Attendance\Reference\Storage::factory($this->getDb(), $this->getConfig());
		$crefs = new \Neuron\Attendance\Attendance\References($refstorage);

		$sourstorage = \Neuron\Attendance\Attendance\Source\Storage::factory($this->getDb(), $this->getConfig());
		$csour = new \Neuron\Attendance\Attendance\Source($sourstorage);

		$refs = $crefs->listReference(1, 999);
		$sours = $csour->listSource(1, 999);
        
        return $this->getOutput(json_encode(array('references' => $refs, 'source'=>$sours)));
	}

    public function checkAttendanceAction()
    {
        $result     = new Result();

        $attendanceEntity   = new AttendanceEntity(AttendanceEntity\Storage::factory($this->getDb(),$this->getConfig()));
        $getParamsAttd      = $attendanceEntity->checkAttend();
        $paramsAttendance   = $getParamsAttd->data;
        
        $result->data = $paramsAttendance;
        
        return $this->getREST()->getResponse($result);
    }
    
    public function listAction() {
        $sessmgt = $this->getSession()->get();
        $post = $this->getPostParams();
          
        $storage = \Neuron\Attendance\Attendance\Storage::factory($this->getDb(), $this->getConfig());
        $catt = new \Neuron\Attendance\Attendance($storage);  
        $result = $catt->listAttendance($post);

        // echo "<pre>", print_r($result), "</pre>";die();
        
        echo json_encode($result);
        return $this->getResponse();       
    }
    
	public function deleteAction() {
	    $storage = \Neuron\Attendance\Attendance\Storage::factory($this->getDb(), $this->getConfig());
	  	$catt = new \Neuron\Attendance\Attendance($storage);

	    $post = $this->getPostParams();
	    $deltt = $catt->delete($post['id']);
	    $success = false;
		if ($deltt) $success = true;
		
		$requestPost = array(
			'guid' => '',
			'code' => '',
			'info' => '',
			'data' => $post
		); 
		
		$logparam['subject'] = 'ATTENDANCE ATTENDANCE DELETEACTION';
		$logparam['identity'] = '';
		$logparam['user'] = '';
		$logparam['description'] = 'Log proses attendance attendance loadAction';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode(array('success' => $success));

		$this->writeLogAction($logparam);
		
	    echo json_encode(array('success' => $success));
	    return $this->getResponse();
    }
      
    public function generateAttendanceAction(){
        Entity::publish();
        $data = $this->getPostParams();

        $result = new \Neuron\Generic\Result;
        $modelShift = new Shift(Shift\Storage::factory($this->getDb(),$this->getConfig()));
        $errorShift = array();
        $validShift = array();
        
        foreach ($data as $log) {
            $user_id                    = $log['ref_id'];
            /* harus dirubah dulu jadi person_id, karena dari fingerprint ngirimnya user_id */
            $personMap  = new \Neuron\Person\Map(
                \Neuron\Person\Map\Storage::factory(
                    $this->getDb(),$this->getConfig()
                )
            );
            $personMap->load($log['ref_id'],\Neuron\Person\Map\Storage::LOADBY_USER_ID);
            $map        = $personMap->pull();
            if($map['person_id']){
                $user_id = $map['person_id'];
            }

            $dataShift = $modelShift->loadObjectTimetable($user_id, $log['start_dtm'], $log['type_id']);

            if($dataShift['success']){
                $modelAttendance = new AttendanceModel(AttendanceModel\Storage::factory($this->getDb(),$this->getConfig()));
                $modelAttendance->type_id = $dataShift['data']['is_workday'];
                $modelAttendance->ref_code = 'EMPLOYEE';
                $modelAttendance->source_type_code = 'FP';
                $modelAttendance->source_reference = $log['source_reference'];
                $modelAttendance->source_dtm = date('Y-m-d');
                $modelAttendance->ref_id    = $user_id;
                

                if($dataShift['code'] == 1){
                    $cekAttendance = $modelAttendance->checkAttendance($log['type_id'], $user_id, $log['start_dtm']);
                    $update = false;
                    if($cekAttendance){
                        $currentStartDate = $cekAttendance[0]['start_dtm'];
                        $newStartDate = $log['start_dtm'];
                        $log['start_dtm'] = ($newStartDate < $currentStartDate) ? $newStartDate : $currentStartDate;
                        $modelAttendance->id = $cekAttendance[0]['id'];
                        $update = true;
                    }

                    $modelAttendance->start_dtm = $log['start_dtm'];
                    $modelAttendance->start_description = 'Start attendance and ready to work!';
                    $save = $modelAttendance->save($update);

                }else if($dataShift['code'] == 2){

                    $cekAttendance = $modelAttendance->checkAttendance($log['type_id'], $user_id, $log['start_dtm']);
                    $dataAttend = @$cekAttendance[0];
                    $modelAttendance->push($dataAttend);
                    $modelAttendance->end_description = 'Leaving.. may tomorrow better than today..';
                    $modelAttendance->end_dtm = $log['start_dtm'];
                    $save = $modelAttendance->save(true);
                }

                $validShift[] = $save;
            }else{
                $dataShift['data'] = array(
                    'user_id'   => $user_id,
                    'start_dtm' => $log['start_dtm'],
                    'type_id'   => $log['type_id']
                );
                $errorShift[] = $dataShift;
            }
        }
        $result->data = array(
            'errorShift' => $errorShift,
            'validShift' => $validShift);
		
        return $this->getREST()->getResponse($result);
    }		
    
    public function SetByAdminAction() {

        $this->using('font-awesome', 10, 'https://use.fontawesome.com/releases/v5.8.1/css/all.css');
        
        $this->using('naf-inside', 19);

        $this->using('datatable', 20, array(
            'https://cdn.datatables.net/1.10.19/js/jquery.dataTables.min.js',
            'https://cdn.datatables.net/1.10.19/js/dataTables.bootstrap4.min.js',
            'https://cdn.datatables.net/1.10.19/css/dataTables.bootstrap4.min.css'
        ));

        $layout = $this->layout();
        // $layout->setVariables(['previous' => '<a href="' . $this->getPublicURL() . '/attendance/form/request">' . $this->translate("Request") . '</a>']);
        $layout->setVariables(['current' => $this->translate('Set by admin')]);


        $this->usingjs('bootbox', 21);
        $this->using('p-loading', 22);
        $this->usingjs('moment', 23);
        $this->using('bootstrap-select', 26);
        $this->using('toast',27);
        $this->using('generic', 30);
        $this->usingjs('jquery.validate',34);
        $this->using('fullcalendar', 36);
        $this->usingjs('moment-weekdaysin', 37, false);
        $this->using('air-datepicker',40);

        $sessmgt = $this->getSession()->get();
        $sess = (isset($sessmgt['ref_id'])) ? true : false;

        $title = $this->translate("Attendance Set By Admin");

        return $this->getView('setbyadmin', get_defined_vars());
    }
    
    public function loadAttdTimeAction() {
        $session  = $this->getSession();
        $result = new \Neuron\Generic\Result();
        
        $request = $this->getRequest();

        if (!$request->isPost()) {
            $result->code = self::FAIL_CODE;
            $result->info = 'Method not allowed';
            return $this->getREST()->getResponse($result);
            
        }

        $data = $this->getREST()->getParameters();
        $username = isset($data['username'])?$data['username']:'';
        $token = isset($data['token'])? $data['token']: '';
        try {
            if($token == ""){
                $result->code = self::CODE_FIELD_REQUIRED;
                $result->info = 'token is null';
                return $this->getREST()->getResponse($result);   
            }
            
            $userPerson = $this->getUserPerson($username);

            $session->owner($userPerson['user']['user_id']);

            $sessid = $session->getId();
            if($token != $sessid){
                $result->code = self::CODE_UNAUTHORIZED;
                $result->info = 'Failed authenticate';
                return $this->getREST()->getResponse($result);   
            }

            $islogin = $this->isLoggedIn();
            if(!$islogin){
                $result->code = self::CODE_SESSION_EXPIRED;
                $result->info = 'Session Expired';
                return $this->getREST()->getResponse($result);   
            }
            $storageAttendance = \Neuron\Attendance\Attendance\Storage::factory($this->getDb(), $this->getConfig());
            $modalAttendance  = new \Neuron\Attendance\Attendance($storageAttendance);
            $storageTimettable = \Neuron\Attendance\Shift\Storage::factory($this->getDb(), $this->getConfig());
            $modelTimetable = new \Neuron\Attendance\Shift($storageTimettable);
            
            $cekAttdEntrance  = $modalAttendance->checkAttendance(1, $userPerson['person']['ref_id'], date('Y-m-d'));
            $isAttend = true;
            if (!$cekAttdEntrance) {
                $isAttend = false;
            }else{
                $cekAttdEntrance[0]['attendance_id'] = $cekAttdEntrance[0]['id'];
                unset($cekAttdEntrance[0]['id']);
            }
            $config = $this->getConfig();
            $arrLocations = array();
            if($config['geolocation']){ 
                $orgAddress = array();
                if (!empty($orgAddress)) {
                    foreach ($orgAddress as $val) {
                        array_push($arrLocations, $val);
                    }
                }
            }
            $dayno = date('N');
            $cektimetable = $modelTimetable->loadShiftObject($userPerson['person']['ref_id'] , date('Y-m-d')); 
            $shift= null;
            if($cektimetable){
                foreach ($cektimetable as $val) {
                    if($val['day_no'] == $dayno+1){
                        $shift = $val;  
                    }
                }
            }
            $dateTime = date('Y-m-d H:i:s');

            $data =  array('isAttend' => $isAttend, 'attendance' => $cekAttdEntrance, 'timeshift' => $shift, "token"=>$token, "currentDate"=>$dateTime, 'locations'=>$arrLocations);

            $result->guid = self::SUCCESS_CODE;
            $result->code = self::SUCCESS_CODE;
            $result->info = 'OK';
            $result->data = $data;
            return $this->getREST()->getResponse($result);
        } catch (Exception $e) {
            $result->code = self::FAIL_CODE;
            $result->info = 'error';
            $result->data = array(
                'message' => $ex->getMessage(),
            );
            return $this->getREST()->getResponse($result);
        }
    }

    public function testAction(){
        $this->using('jquery.bootgrid',25);
        $this->using('jquery.validate',20);
        $this->using('p-loading',21);
        $this->using('bootbox',22);
        $this->using('moment',23);
        $this->using('bootstrap-datetimepicker',24);

        $sessmgt = $this->getSession()->get();
        $sess = (isset($sessmgt['ref_id'])) ? true : false;
        if($sess){
            $ref_id = $sessmgt['ref_id'];
        }else{
            $ref_id = 2000000;
        }

        return $this->getView('widget-attendance',get_defined_vars());
    }

    public function GetDataOvertimeAction(){
        $params     = $this->getRequest()->getPost()->toArray();
        $attendance  = new AttendanceModel(AttendanceModel\Storage::factory($this->getDb(),$this->getConfig()));
        
        $params['year']     = (isset($params['year']) && !empty($params['year'])) ? $params['year'] : date('Y');
        $params['type_id']  = 2;
        $result = $attendance->loadList(
            null, 
            AttendanceModel\Storage::COUNT_HOUR,
            $params,
            AttendanceModel\Storage::TIME_NONE
        );

        if($result->code != 0){
            $result         = new Result();
            $result->data   = array(
                'month'     => 1,
                'monthname' => "January",
                'hour'      => 0
            );
        }else{
            $data = $result->data;
            $tempArray = [];
            foreach($data as $data){
                $month      = $data['month'];
                if(!array_key_exists($month,$tempArray)){
                    $tempArray[$month] = intval($data['hour']);
                }else{
                    $tempArray[$month] += intval($data['hour']);
                }

                if(intval($data['minutes']) > 30){
                    $tempArray[$month]++;
                }
            }

            $resArray       = []; 
            foreach($tempArray as $key => $ta){
                $resArray[] = array(
                    'month'     => $key,
                    'hour'      => $ta
                );
            }

            $result->data   = $resArray;
        }

        return $this->getREST()->getResponse($result);
    }

    public function CheckBydateAction(){
        $post = $this->getPostParams();

        $storage = \Neuron\Attendance\Attendance\Storage::factory($this->getDb(), $this->getConfig());
        $catt = new \Neuron\Attendance\Attendance($storage);  

        if(class_exists('\Neuron\Employee\Entity')){
            $map    = $this->getMapPerson();
            $ref_id = $map['ref_id'];
        }else{
            $ref_id = $this->getSession()->owner();
        }

        $post['ref_id']     = $ref_id;

        $result = $catt->listAttendance($post);
        
        $output = (count($result['data']) > 0) ? true:false;

        echo json_encode($output);
        return $this->getResponse();
    }

    public function loadListAction(){
        $post       = $this->getRequest()->getPost();
        $keyword         = null;
        
        if(isset($post['draw']) && !empty($post['draw'])){
            $keyword     = $post['search']['value'];   
        }


        $entities   = new \Neuron\Attendance\Entities(\Neuron\Attendance\Entity\Storage::factory($this->getDb(),$this->getConfig()));
        $result       = $entities->load($keyword,\Neuron\Attendance\Entity\Storage::LIST_ALL,$post);

        if(isset($post['draw'])){
            if($result->code == 0){
                $resData    = array(
                    'draw' => $post['draw'],
                    'recordsTotal' => $result->data['countTotal'],
                    'recordsFiltered' => $result->data['countTotal'],
                    'data'  => $result->data['list']
                );
            }else{
                $resData    = array(
                    'draw' => $post['draw'],
                    'recordsTotal' => 0,
                    'recordsFiltered' => 0,
                    'data'  => array()
                );
            }
            
            return $this->getOutput(json_encode($resData));

        }

        return $this->getREST()->getResponse($result);
    }

    public function saveAction(){
        $naf        = NAF\Entity::get();
        $post       = $this->getRequest()->getPost()->toArray();
        $session    = $this->getSession()->get();
        $mapper     = $this->getMapper();
        $employee   = $mapper->data['employee'];
        $activeRole = $session["active_role"]['code'];

        if(
            ($activeRole != "admin" && $activeRole != "hrd") && # Check role yang aktif. kalo bukan admin / hrd, masuk ke pengecekan selanjutnya
            (isset($post['ref_id']) && !empty($post['ref_id'])) && # Check ngepost ref_id apa engga, kalo ngepost, check punya usernya bukan
            ($post['ref_id'] != $employee->id) # Check ref_id yang di post, sama kaya yang dimiliki session gak
        ){
            /* Rolenya bukan Admin / hrd */
            /* usernya ngepost ref_id */
            /* ref_idnya != employee_id user yang login */
            return $this->getREST()->getResponse(new Result(0,401,$this->translate('Authentication Failed, you don\'t have permission')));    
        }
        
        if(!isset($post['ref_id']) || empty($post['ref_id'])){
            if($employee->id){
                $post['ref_id'] = $employee->id;
            }else{
                return $this->getREST()->getResponse(new Result(0,411,$this->translate('Invalid `ref_id` !')));
            }
        }


        /* check ada start_dtm ga, kalo ada, buat key `start_time` di variable $postnya */
        if(isset($post['start_dtm'])){
            $post['start_time'] = date('H:i:s',strtotime($post['start_dtm']));
        }
        
        if(!isset($post['source_reference'])){
            $post['source_reference'] = $session['user_id'];
        }
		
		/* check ada data rekomendasi yang sudah di approve atau tidak, jika ada maka tidak bisa absen */
		$personMap  = new \Neuron\Person\Map(\Neuron\Person\Map\Storage::factory(
						$this->getDb(),
						$this->getConfig()
					));
		$personMap->load($post['ref_id'],\Neuron\Person\Map\Storage::LOADBY_PERSON_ID);
		$userId 	= $personMap->user_id;
		
		$leave     = new \Neuron\Approval\Entities(\Neuron\Order\Entity\Storage::factory(
						$this->getDb(),
						$this->getConfig()
					));
					
		$where['where']['create_user_id'] = $userId;
		$where['where']['order_type_id'] = 2200;
		$where['where']['order_status_id'] = 2;
		$where['where']['xn5'] = 2;
		$where['where']['sdate'] = $post['start_dtm'];
        $where['where']['order_subtype_id'] = ['2105', '2106', '2111', '2114', '2116', '2117', '2118', '2119', '2137'];
		
		if ($manager = NAF\Event\Manager::get()) {
            $manager->events()->register('order_query',array($this,'onCheckQuery'),0,3);
        }
		
		$result = $leave->load(null, \Neuron\Order\Entity\Storage::LIST_ALL, $where, true);
		
		if($result->code == 0){
			$result->code = 1;
			$result->info = 'Cannot attend, your leave recommendation has been approved';
			$result->data = '';
		}else{
			$entity             = new \Neuron\Attendance\Entity(\Neuron\Attendance\Entity\Storage::factory($this->getDb(),$this->getConfig()));
			$post['type_id']    = \Neuron\Attendance\Entity::TYPE_ATTENDANCE;
			$result             = $entity->saveEx($post);
		}
		
		$requestPost = array(
			'guid' => '',
			'code' => '',
			'info' => '',
			'data' => $post
		); 
		
		$responseResult = array(
			'code' => $result->code,
			'info' => $result->info,
			'data' => $result->data
		); 
		
		$logparam['subject'] = 'ATTENDANCE ATTENDANCE SAVEACTION';
		$logparam['identity'] = '';
		$logparam['user'] = '';
		$logparam['description'] = 'Log proses attendance attendance saveAction';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode($responseResult);

		$this->writeLogAction($logparam);
		$result->info = $this->translate($result->info);
        return $this->getREST()->getResponse($result);
    }

    public function locationListAction(){
        $entity         = new \Neuron\Attendance\Entity(
            \Neuron\Attendance\Entity\Storage::factory($this->getDb(),$this->getConfig())
        );
        $result         = $entity->locationList();
        $result->info   = $this->translate($result->info);
        return $this->getREST()->getResponse($result);
        
    }

    public function todayAction(){
        $post       = $this->getRequest()->getPost()->toArray();
        $entity     = new AttendanceEntity(AttendanceEntity\Storage::factory($this->getDb(),$this->getConfig()));
        $result     = $entity->todayStatus($post);
        return $this->getREST()->getResponse($result);
    }

    public function geoCoordinatesAction(){
        $session = $this->getSession()->get();
        $post = $this->getREST()->getParameters();
        $result = new \Neuron\Generic\Result();
    
        $location = null;

        if(isset($post['lat'], $post['lng'])) {
            $lat = $post['lat'];
            $lng = $post['lng'];

            $urLocation = 'https://reverse.geocoder.api.here.com/6.2/reversegeocode.json?prox='.$lat.'%2C'.$lng.'&mode=retrieveAddresses&maxresults=1&gen=9&app_id=eUftlDOAPLvFJW4NoZyv&app_code=0sHXDeWOfdShAqAPGjFInw';

            // Create a stream
            $opts = array('http'=>array('header'=>"User-Agent: StevesCleverAddressScript 3.7.6\r\n"));
            $context = stream_context_create($opts);

            // Open the file using the HTTP headers set above
            $resLocation = file_get_contents($urLocation, false, $context);
            $resLocation = json_decode($resLocation, true);
            
            if(!isset($resLocation['type'])){
                $location = isset($resLocation['Response']['View'][0]['Result'][0]['Location']['Address']['Label']) ? $resLocation['Response']['View'][0]['Result'][0]['Location']['Address']['Label'] : null;

                $result->info = 'OK';
                $result->data = $location;
            }else{
                $result->info = $resLocation['Details'];
                $result->code= 1;
                $result->data = null;
            }
        }else{
            $result->info = 'Coordinate required';
            $result->code= 1;
            $result->data = null;
        }
        return $this->getREST()->getResponse($result);
    }
	
	public function loadAction(){
        $post       = $this->getRequest()->getPost()->toArray();
		
        $attd   	= new \Neuron\Attendance\Attendance(\Neuron\Attendance\Attendance\Storage::factory($this->getDb(),$this->getConfig()));
        $result    	= $attd->load($post['id'],\Neuron\Attendance\Attendance\Storage::LOADBY_ID);
		
		if(class_exists('\Neuron\Employee\Entity')){
			//load data person
			$person 	= new PersonEntity(PersonEntity\Storage::factory($this->getDb(),$this->getConfig()));
			$person		->load($result['ref_id']);
			$dataPerson = $person->pull();

			$first_name = $dataPerson['first_name'];
			$last_name 	= $dataPerson['last_name'];
			
			$name = $first_name. ' ' .$last_name;
        }
		
        echo json_encode(array('data' => $result, 'name' => $name));
        return $this->getResponse(); 
    }

    /**
     * digunakan untuk check apakah sudah absen atau belum
     */
    public function checkAttendAction(){
        $result     = new \Neuron\Generic\Result;
        $attendanceEntity = new AttendanceEntity(AttendanceEntity\Storage::factory(
            $this->getDb(),
            $this->getConfig()
        ));
        
        $sessOwner    = $this->getSession()->owner();
        if(class_exists('Neuron\Employee\Entity')){
            $map    = new \Neuron\Person\Map(
                \Neuron\Person\Map\Storage::factory(
                    $this->getDb(),
                    $this->getConfig()
                )
            );
            $map->load($sessOwner,\Neuron\Person\Map\Storage::LOADBY_USER_ID);
            $refId  = $map->person_id;
        }else{
            $refid  = $sessOwner;
        }
        

        $attendanceEntity->load(
            null,AttendanceEntity\Storage::LOADBY_ID,
            array(
                'ref_id'        => $refId,
                'start_date'    => date("Y-m-d")
            )
        );
        $shift                  = new Shift(Shift\Storage::factory(
            $this->getDb(),$this->getConfig()
        ));
        $data['ref_id']         = $refId;
        $data['start_date']     = date('Y-m-d');
        $data['start_dtm']      = date('Y-m-d H:i:s');
        $data['day_no']         = $shift->getDayNumber($data['start_date'])->data;
        $shift->load(null,Shift\Storage::LOADBY_BETWEENDATE_REF_ID_DAYNO,$data);
        $canAttend              = true;
        $info                   = "Can attend";
        $attendType             = self::ATTEND_TYPE_IN;
        $autoPop                = false;
        
        $isWorkday  = $shift->is_workday;
        if($isWorkday){
            if($attendanceEntity->id && $attendanceEntity->type_id != 3){
                // udah ada absennya 
                $isWithinRange          = $shift->isWithinTimeRange(date('H:i:s'),$shift->out_beginning_time,$shift->out_ending_time);
                if(!$isWithinRange){
                    // Belum jam pulang, berarti gabisa absen
                    $canAttend  = false;
                    $info       = "You're already attended. you cannot clock out at this time.";
                    $attendType = self::ATTEND_TYPE_NOT_IN_RANGE_OUT;
                }else{
                    if($attendanceEntity->end_dtm != null){
                        $canAttend  = false;
                        $info       = "You're already clock out.";
                        $attendType = self::ATTEND_TYPE_ALREADY_OUT;
                    }else{
                        $attendType = self::ATTEND_TYPE_OUT;
                    }
                }
            }else{
                /* Cari dulu, hari ini libur ga */
                $events             = new Events(Event\Storage::factory(
                    $this->getDb(),
                    $this->getConfig()
                ));
                $loadEvents   = $events->load(
                    0, # 0 = hari libur
                    Event\Storage::LISTBY_WORKDAY,
                    array(
                        'start_date'    => date('Y-m-d',strtotime($data['start_dtm'])),
                        'end_date'      => date('Y-m-d',strtotime($data['start_dtm'])),
                        'date_between'  => true
                    )
                );
                if($loadEvents->code == 0){
                    $data   = $loadEvents->data;
                    $eventData = $data['list'][0];
                    $canAttend = false;
                    $attendType = self::ATTEND_TYPE_NOT_IN_RANGE_IN;
                    $info       = "Cannot attend at holiday. [ ".$eventData['event_title']." ]";
                }else{
                    // Check kalo melewati batas waktu absen, gabisa absen;
                    $canAttend      = $shift->isWithinTimeRange(date("H:i"),$shift->in_beginning_time,$shift->in_ending_time);
                    $attendType     = ($canAttend) ? self::ATTEND_TYPE_IN : self::ATTEND_TYPE_NOT_IN_RANGE_IN;
                    $info           = ($canAttend) ? "OK" : "Cannot clock in at this time. Clock in start at " . $shift->in_beginning_time .' Until '.$shift->in_ending_time;
                    $autoPop        = ($canAttend) ? true : false;
                }

            }
        }else{
            $info       = "Cannot Attend at holiday"; 
            $canAttend  = false;
        }

        $result->data = [
            'can_attend'    => $canAttend,
            'attendance'    => $attendanceEntity->pull(),
            'shift'         => $shift->pull(),
            'attend_type'   => $attendType,
            'auto_pop'      => $autoPop
        ];

        $result->info   = $this->translate($info);
        return $this->getREST()->getResponse($result);
    }

    public function todayStatusAction(){
        $this->using('p-loading',99);
        $this->using('moment',100);
        $this->using('air-datepicker',101);
        return $this->getView('today-status');
    }

    public function todayStatusDataAction(){
        $post    = $this->getREST()->getParameters();
        $attendanceEntity = new \Neuron\Attendance\Entity(\Neuron\Attendance\Entity\Storage::factory(
            $this->getDb(),
            $this->getConfig()
        ));		
		
		$publicUrl 	= $this->getPublicUrl();
		
        $date       = $post['date'] ?: date('Y-m-d');
        $name       = $post['name'];
        $org_id     = $post['org_id'];
        $status     = $post['status'];
        $sort       = $post['sort'];
        $length     = $post['length'] ? intval($post['length']) : 30;
        $offset     = $post['offset'] ? intval($post['offset']) : 0;

        $result     = $attendanceEntity->todayStatusEmployees($date,$name,$org_id,$status,$sort,$length,$offset,$publicUrl);
        
        $result->info   = $this->translate($result->info);
        return $this->getREST()->getResponse($result);
    }
	
	public function onCheckQuery($query,$params,$caller){

        if(isset($params['optional']['start_date'])){
            $query->where(" DATE('". $params['optional']['start_date'] ."') BETWEEN DATE(o.xd1) AND DATE(o.xd2)");
        }
    }
}
