<?php
namespace Neuron\Attendance\Controller;
use Neuron\Access\Session;
use Neuron\Notification;
use Neuron\Generic\Result;
use Neuron\Attendance\Traits\Mapper;

class Controller extends \Neuron\Mvc\Controller {
    use Mapper;

	protected $_SESSION;
    const SUCCESS_CODE = 0;
    const FAIL_CODE = 1;
    const CODE_FIELD_REQUIRED  = 400;
    const CODE_UNAUTHORIZED = 401;
    const CODE_SESSION_EXPIRED = 403;
    const IS_DATATABLE = 0;
    const IS_NOT_DATATABLE = 1;

	public function onDispatch(\Zend\Mvc\MvcEvent $e) {
        // $this->using('bootstrap-datetimepicker',22);
        /* dispatch NAF - v1 */
        if (class_exists('Neuron\Application\Framework\Hook\Manager')) {
            if ($manager = \Neuron\Application\Framework\Hook\Manager::get()) {
                return $manager->trigger()->onDispatch($this, parent::onDispatch($e));
            }
        }
        $parent = parent::onDispatch($e);
        $this->getSession();
        return $parent;

        /* alternative */
        return \Neuron\Application\Framework\Event\Trigger::onDispatch($this, parent::onDispatch($e));
    }
	protected function getPostParams() {

		$request = $this->getRequest();
		if ($request->isPost()) {
			$post = $this->getRequest()->getPost();
		} else {
			$post = $this->getRequest()->getQuery();
		}

		return $post;
	}
	public function getSession($start = true) {

        if(class_exists('Neuron\Application\Framework\Entity')) {
            $this->_SESSION = \Neuron\Application\Framework\Entity::get($this)->session();
            $this->_SESSION->put('user_id',$this->_SESSION->owner());
            // User Role
            $user = \Neuron\Application\Framework\Entity::get($this)->user();
            $grants = $user->grants->load();
            $roles = $user->roles->load($grants->data);

            $ref = ''.$this->_SESSION->get('ref_id');

            if ($this->_SESSION->owner() && (strlen($ref) < 1)) {

                // Mapper User
                $user_id =  $this->_SESSION->get('user_id');
                $GetMapperPerson = $this->GetMapPerson($user_id);
                $this->_SESSION->put(null, $GetMapperPerson);

            }
            
            $lang = $this->_SESSION->get('lang');
            $this->_SESSION->put(null, ['lang' => $user->language] );
        } else {
            $config = $this->getConfig();

            $sess_lifetime = $config['session']['gc_maxlifetime'];
            $remember = $config['session']['remember'];
            $tableSession = $config['tables']['sessions'];
            
            if ($this->_SESSION === null) {
                $this->_SESSION = new Session('hrmisv2', Session::MODE_DEFAULT,
                                                array('adapter'=>$this->getDb(),
                                                        'table'=>$tableSession,
                                                        'lifetime' => $sess_lifetime,
                                                        'remember' => $remember));
                $this->_SESSION->start(false);
                $this->_SESSION->put(null, ['lang' => 'en_US'] );
            }
            
        }

        //Set Lang
        $s = $this->_SESSION->get();
        
        $translator = null;
        if (method_exists($this, 'getServiceLocator')) {
            $translator = $this->getServiceLocator()->get('translator');
        } elseif (method_exists($this, 'getTranslator')) {
            $translator = $this->getTranslator();
        }
        if (($translator !== null) && method_exists($translator, 'setLocale')) $translator->setLocale($lang);

        $this->getTranslator($s['lang']);

        /* return session */
        return $this->_SESSION;
    }
    protected function isLoggedIn() {

        $session = $this->getSession();
        $owner = $session->owner();
        if (isset($owner) && !empty($owner)) {
            if ($owner != null) {
                $config = $this->getConfig();
                if($session->get('remember')){
                    $sess_lifetime = $config['session']['remember'];
                }else{
                    $sess_lifetime = $config['session']['gc_maxlifetime'];
                }
                
                if ((null !== $session->get('LAST_ACTIVITY')) && (time() - $session->get('LAST_ACTIVITY') > $sess_lifetime)) {
                    $session->stop();
                    return false;
                }

                $session->put('LAST_ACTIVITY', time());
                
                return $session;
            }
        }
        $session->stop();
        return false;
    }
    
    public function GetMapPerson($user_id = null) {
        // $output = [];

        // $storage = \Neuron\Person\Entity\Storage::factory($this->getDb(), $this->getConfig());
        // $cpersons = new \Neuron\Person\Entity($storage);

        // $dataPerson = $cpersons->loadMap($user_id,'user_id');
        // // print_r($user_id); die();
        // $dataMapper = $dataPerson['data'];
        // if($dataMapper) {

        //     $output['ref_id'] = $dataMapper['person_id'];
        //     $output['ohp_id'] = @$dataMapper['ohp_id'];
        //     $output['request_ref_code'] = "EMPLOYEE";
        //     $output['detail'] = $dataMapper;
            
        // }

        // return $output;

        $output = [];

        $cperson = $this->getMapper();
        $employee = $cperson->data['employee'];
        
        if($employee) {
            $job = $employee->job();
            $ohp_id = $job->ohp_id;

            $output['ref_id'] = $employee->id;
            $output['ohp_id'] = $ohp_id;
            $output['request_ref_code'] = "EMPLOYEE";
            $output['detail'] = $employee->detail()->pull();
            
        }
        
        return $output;
    }

    public function getDataFromApi($url, $param = null) {

        $output = [];

        $config = $this->getConfig();

        if(class_exists('Neuron\Employee\Entity')){
            $url = $this->getPublicURL().'/'.$config['attendance']['api_url'][$url];
        }else{
            $url = $config['attendance']['api_url'][$url];
        }
        $curlSession = curl_init();
        curl_setopt($curlSession, CURLOPT_URL, $url);
        curl_setopt($curlSession, CURLOPT_POSTFIELDS, 'data='.$param);
        curl_setopt($curlSession, CURLOPT_BINARYTRANSFER, true);
        curl_setopt($curlSession, CURLOPT_RETURNTRANSFER, true);

        $jsonData = json_decode(curl_exec($curlSession));
        curl_close($curlSession);
        $result = @$jsonData->data;
        return $result;
    }
    public function getOutput($json) {

        $response = $this->getResponse();
        $response->getHeaders()->addHeaderLine('Content-Type', 'application/json');
        $response->setContent($json);
        return $response;
    }
    public function getUserPerson($username){
        /* Init storage &user object */
        $storage = \Neuron\Composite\User\Storage::factory($this->getDb(), $this->getConfig());
        $user = new \Neuron\Composite\User($storage, $this->getConfig()['user']);

        /* Load user by user code */
        $user->load($username, \Neuron\Actor\User\Storage::LOADBY_CODE);

        $dataPerson = $this->GetMapPerson($user->id);
        $dataUser = $user->pull();

        return array('user'=>$dataUser, 'person'=>$dataPerson);
    }
    public function validasiApi($username, $token){
        $result = new \Neuron\Generic\Result();
        if($token == ""){
            $result->code = self::CODE_FIELD_REQUIRED;
            $result->info = 'token is null';
            return $this->getREST()->getResponse($result);   
        }

        $userPerson = $this->getUserPerson($username);

        $session->owner($userPerson['user']['user_id']);
        
        if (empty($userPerson['person']['ref_id'])) {
            $success = false;
            $message = "username not mapping to any person or employee";
            $data =  array('success'=> $success, 'message'=>$message, "token"=>$sessid);
            $result->guid = self::SUCCESS_CODE;
            $result->code = self::FAIL_CODE;
            $result->info = $message;
            $result->data = $data;
            return $this->getREST()->getResponse($result);
        }
        $sessid = $session->getId();
        if($token != $sessid){
            $result->code = self::CODE_UNAUTHORIZED;
            $result->info = 'Failed authenticate';
            return $this->getREST()->getResponse($result);   
        }

        $islogin = $this->isLoggedIn();
        if(!$islogin){
            $result->code = self::CODE_SESSION_EXPIRED;
            $result->info = 'Session Expired';
            return $this->getREST()->getResponse($result);   
        }

        return true;
    }
    public function resultApi($resData){
        $result = new \Neuron\Generic\Result();
        $success = $resData['success'];
        $message = $resData['message'];
        if(@$resData['data']){
            $data = $resData['data'];
        }else{
            $data =  array('success'=> $success, 'message'=>$message, "token"=>$sessid);
        }
        if($success){
            $result->code = self::SUCCESS_CODE;
        }else{
            $result->code = self::FAIL_CODE;
        }

        $result->guid = self::SUCCESS_CODE;
        $result->info = $message;
        $result->data = $data;
        return $this->getREST()->getResponse($result);
    }
    public function checkGrants($objectCode) {
        if(class_exists('Neuron\Application\Framework\Entity')) {
            $user = \Neuron\Application\Framework\Entity::get($this)->user(true);
            
            return $user->hasGrant($objectCode);
        }
        return true;
    }  
    public function saveNotif($data){
        if(class_exists('Neuron\Notification\Inbox')){

            $cache = new \Neuron\Notification\Inbox\Cache(\Neuron\Notification\Inbox\Cache\Storage::factory($this->getDb(), $this->getConfig()));
        
            $cache->subject = $data['subject'];
            $cache->typeId = 3;
            $cache->token = null;

            if(!$cache->load($data['subject'], \Neuron\Notification\Inbox\Cache\Storage::LOADBY_SUBJECT_TYPE, array('type_id' => 3))){
                $prepare = $cache->prepare();
                $ready = false;
                if($prepare->code ==0){
                    $ready= true;
                }
            }else{
                $ready = true;
            }
            if($ready){
                $storage = \Neuron\Notification\Inbox\Storage::factory($this->getDb(), $this->getConfig(), \Neuron\Notification\Profile::TYPE_FCM, $data['subject']);
                $inbox = new \Neuron\Notification\Inbox($storage);
                
                $result = $inbox->pushNotification(
                    isset($data['subject']) ? $data['subject'] : null,
                    isset($data['template_code']) ? $data['template_code'] : null,
                    isset($data['template_var']) ? $data['template_var'] : array(),
                    isset($data['additional_data']) ? $data['additional_data'] : array()
                );
                
                return $result;
            }

        }
        return false;
    }
    public function readNotif($post){
        if(class_exists('Neuron\Notification\Inbox')){
            $notifInboxID = $post['notif_inbox_id'];

            switch ($post['type']) {
                case \Neuron\Notification\Profile::TYPE_FCM:
                    $type = \Neuron\Notification\Profile::TYPE_FCM;
                    break;
                case \Neuron\Notification\Profile::TYPE_SMS:
                    $type = \Neuron\Notification\Profile::TYPE_SMS;
                    break;
                case \Neuron\Notification\Profile::TYPE_EMAIL:
                    $type = \Neuron\Notification\Profile::TYPE_EMAIL;
                    break;
            }
            $storage = \Neuron\Notification\Inbox\Storage::factory($this->getDb(), $this->getConfig(), $post['type'] , $post['user_id'] );

            // $inboxes = new Inboxes($storage, $type, $post['user_id']);

            $inbox = new \Neuron\Notification\Inbox($storage);
            
            $mode = \Neuron\Notification\Inbox\Storage::LOADBY_ID;
            
            // $row = $inboxes->load($notifInboxID,$mode);
            $row = $inbox->load($notifInboxID);

            $read = $inbox->read();
            
            $result = new \Neuron\Generic\Result;
            if($read){
                $result->data = $inbox->id;
            }else{
                $result->code = 1;
                $result->info = 'Gagal mengupdate';
            }

            return $result;
        }   
        return false;
    }
    public function getRefId(){
        $sessmgt = $this->getSession()->get();
        $result = new \Neuron\Generic\Result;
        if(!isset($sessmgt['detail'])){
            $result->code = 401;
            $result->info = 'Invalid Reference ID';
            return $result;
        }
        $sess = (isset($sessmgt['ref_id'])) ? true : false;
        if(class_exists('Neuron\Application\Framework\Entity')){
            $user_id = $sessmgt['user_id'];
        }else{
            $user_id = $sessmgt['detail']->user_id;
        }

        if(!$user_id){
            $result->code = 402;
            $result->info = 'Invalid Reference ID';
            return $result;
        }

        // Load Map Person
        $mapPerson = $this->getMapPerson($user_id);
        if(!$mapPerson){
            $result->code = 403;
            $result->info = 'Error Loading';
            return $result;
        }
        $ref_id = $mapPerson['ref_id'];
        $sessmgt['ref_id'] = $ref_id;
        $result->data = $ref_id;
        return $result;    
    }
    public function listenFcm(){
        $sessmgt = $this->getSession()->get();
        if(isset($sessmgt['user_id'])){
            if(class_exists('Neuron\Notification\FCM')){
                $config = $this->getConfig();
                $notifConfig = isset($config['notification']) ? $config['notification'] : null;

                $opt = new \Neuron\Notification\FCM\Options();
                $opt->push($notifConfig['fcm']);
                
                $template = new \Neuron\Notification\Template(
                   \Neuron\Notification\Template\Storage::factory($this->getDb(),  $config = $this->getConfig()), 
                    'REQUEST_NEED_APPROVAL'
                );

                $fcm = new \Neuron\Notification\FCM(
                    \Neuron\Notification\Email\Storage::factory($this->getDb(), $this->getConfig()),
                    $template,
                    $opt
                );
                $script = $fcm->listen('callback', $sessmgt['user_id'], $this->getPublicUrl());
                $this->layout()->setVariables(array('script'=>$script));
            }
        }
    }
    function GetRefName($param) {

            $outputData = (is_object($param)) ? $param->data : $param;
            
            if(class_exists('Neuron\Employee\Entity')){
                $arrRef_id = [];
                $arrReqBy_id = [];
                foreach($outputData['data'] as $dt) {
                    if (! in_array($dt['ref_id'], $arrRef_id)) {
                        array_push($arrRef_id, $dt['ref_id']);
                    }
                }

                $pars['arr_ref_id'] = $arrRef_id;
                foreach($outputData['data'] as $key => $val) {
                    $cstorage = \Neuron\Employee\Entity\Storage::factory($this->getDb(), $this->getConfig());
                    $cpersons = new \Neuron\Employee\Entity($cstorage);
                    $cpersons->load($val['ref_id']); 
                    $employee = $cpersons->pull();
                    $person = $cpersons->person()->pull();
                    $val['ref_name'] = $person['first_name']." ".$person['last_name'];
                    $outputData['data'][$key] = $val;
                }
            }

            return $outputData;
    }
	
	public function writeLogAction($param) {
	
		$result = new Result();
	
		if(class_exists('Neuron\Event\Log')) { 
			$cfg = $this->getConfig();
			
			//on off log
			$off = array_key_exists('writelog', $cfg) ? $cfg['writelog'] : 'on';
			if ($off != 'off') {
				$log = new \Neuron\Event\Log(\Neuron\Event\Log\Storage::factory(null, $cfg));
				$log->write(array(
					// 'event_type_id' => isset($param['event_type_id']) ? $param['event_type_id'] : '',
					// 'event_status_id' => isset($param['event_status_id']) ? $param['event_status_id'] : '',
					'subject' => isset($param['subject']) ? $param['subject'] : '',
					'identity' => isset($param['identity']) ? $param['identity'] : '',
					// 'time' => date("Y-m-d H:i:s",time()),
					'name' => isset($param['name']) ? $param['name'] : '',
					'description' => isset($param['description']) ? $param['description'] : '',
					'request' => isset($param['request']) ? $param['request'] : '',
					'response' => isset($param['response']) ? $param['response'] : '',
					'data1' => isset($param['data1']) ? $param['data1'] : '',
					'data2' => isset($param['data2']) ? $param['data2'] : '',
					'data3' => isset($param['data3']) ? $param['data3'] : '',
					/* Optional */
					'details' => array(
						array(
							'event_status_id' => isset($param['event_status_id']) ? $param['event_status_id'] : '',
							'event_status_id_old' => isset($param['event_status_id_old']) ? $param['event_status_id_old'] : '',
							'description' => isset($param['description']) ? $param['description'] : '',
							'data1' => isset($param['data1']) ? $param['data1'] : '',
							'data2' => isset($param['data2']) ? $param['data2'] : ''
						)
					)
				/* Optional */
				));
			}
			
		}
	  return $result;
	}

    protected function getListParamaters($mode){

        $result = new Result();

        $post = $this->getRequest()->getPost();
        if (isset($post['start']) && isset($post['length'])) {
            $params = array(
                'page' => ($post['start'] + $post['length']) / $post['length'],
                'records' => $post['length'],
                'order' => $post['order'],
                'columns' => $post['columns'],
                'params' => $post['params'],
            );
            $keyword = $post['search']['value'];
            $datatable = true;
            $result->code = $this::IS_DATATABLE;
            $result->data = [
                'keyword' => $keyword,
                'mode' => $mode,
                'params' => $params,
            ];
        } else {
            $post = $this->getREST()->getParameters();

            $keyword = @$post['keyword'];
            $mode = @$post['mode'];
            $datatable = false;
            $result->code = $this::IS_NOT_DATATABLE;
            $result->data = [
                'keyword' => $keyword,
                'mode' => $mode,
                'params' => isset($post['params']) ? $post['params'] : $post,
            ];
        }
        return $result;

    }
}
