<?php
namespace Neuron\Attendance\Controller;
use Neuron\Attendance\Entity;
use Neuron\Approval\Entitiy;
use Neuron\Attendance\Attendance;

use Neuron\Order\Subtype as OrderSubtype;
use Neuron\Approval\Subtype as ApprovalSubtype;
use Neuron\Approval\Subtypes as ApprovalSubtypes;
use Neuron\Employee;
use Neuron\Employee\Map;
use Neuron\Attendance\Traits\Mapper;
use Neuron\Attendance\Traits\Model;

class Dashboard extends Controller {
    use Mapper;
    
    const ATTEND_TYPE_IN                = 0;
    const ATTEND_TYPE_OUT               = 1;
    CONST ATTEND_TYPE_NOT_IN_RANGE_IN   = 2;
    CONST ATTEND_TYPE_NOT_IN_RANGE_OUT  = 3;
    CONST ATTEND_TYPE_ALREADY_IN        = 4;
    CONST ATTEND_TYPE_ALREADY_OUT       = 5;

	protected function getView($name, $variables = array()) {

		/* default layout & view prefix */
		$layout = 'neuron/attendance/layout/form';
		$prefix = 'neuron/attendance/v2_1';

		/* libraries */
		// $this->using('jquery', null, false)
		// 	->using('bootstrap', null)
		// 	->using('font-awesome', null, true, false);
		
		/* libraries */;
        $this->using('font-awesome', 10, 'https://use.fontawesome.com/releases/v5.8.1/css/all.css');

        $this->using('select2', 22, array(
            'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.10/js/select2.min.js',
             /* ini load sesuai bahasa dari session */
            'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.10/js/i18n/id.js',
             /**/
            'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.10/css/select2.min.css',
            'https://cdn.jsdelivr.net/npm/@ttskch/select2-bootstrap4-theme@1.3.2/dist/select2-bootstrap4.min.css',
        ));
	    $this->using('generic');
		
		/* get layout n view prefix from config */
		$config = $this->getConfig();
		if (isset($config['attendance'])) {
			$config = $config['attendance'];
			$layout = isset($config['formlayout']) ? $config['formlayout'] : $layout;
			$prefix = isset($config['viewprefix']) ? $config['viewprefix'] : $prefix;
		}

		/* set layout and view */
		$this->layout($layout);
		$view = new \Zend\View\Model\ViewModel($variables);
		$view->setTemplate($prefix . '/' . strtolower($name));

		return $view;
	}

    # dashboard summary [BEGIN]

	public function summaryAction() {

		
        $this->using('bootbox',21,  array(
            'https://cdnjs.cloudflare.com/ajax/libs/bootbox.js/4.4.0/bootbox.min.js'
        ));
        $this->using('p-loading', 22);	
        $this->using('moment', 23, array(
            'https://cdnjs.cloudflare.com/ajax/libs/moment.js/2.29.1/moment.min.js'
        ));
        $this->using('jquery.validate', 34, array(
            'https://cdnjs.cloudflare.com/ajax/libs/jquery-validate/1.19.2/jquery.validate.min.js',
            'https://cdnjs.cloudflare.com/ajax/libs/jquery-validate/1.19.2/additional-methods.min.js'
        ));
		$this->using('air-datepicker', 24, array(
            'https://cdnjs.cloudflare.com/ajax/libs/air-datepicker/2.2.3/js/datepicker.min.js',
            'https://cdnjs.cloudflare.com/ajax/libs/air-datepicker/2.2.3/css/datepicker.min.css',
            'https://cdnjs.cloudflare.com/ajax/libs/air-datepicker/2.2.3/js/i18n/datepicker.en.min.js'
        ));

		$sessmgt = $this->getSession()->get();
		$sess = (isset($sessmgt['ref_id'])) ? true : false;
        $last_filter = $this->params()->fromQuery();

		$title = 'Attendance';
        $start_date = (isset($last_filter['start_date']) && !empty($last_filter['start_date'])) ? $last_filter['start_date'] : date("d F Y", mktime(0, 0, 0, date("m"), 1));
        $end_date = (isset($last_filter['end_date']) && !empty($last_filter['end_date'])) ? $last_filter['end_date'] : date('t F Y');
        $interval = (isset($last_filter['interval']) && !empty($last_filter['interval'])) ? $last_filter['interval'] : 2;
        $interval_recomendation = (isset($last_filter['interval_recomendation']) && !empty($last_filter['interval_recomendation'])) ? $last_filter['interval_recomendation'] : 2;

        $range = (isset($last_filter['range']) && !empty($last_filter['range'])) ? $last_filter['range'] : null;
        $range_recomendation = (isset($last_filter['range_recomendation']) && !empty($last_filter['range_recomendation'])) ? $last_filter['range_recomendation'] : null;

        //load data summary
        $sessionOwner       = $this->getSession()->owner();
        $employeeMap        = new Map(Map\Storage::factory($this->getDb(),$this->getConfig()));
        $employeeEntity     = new Employee\Entity(Employee\Entity\Storage::factory($this->getDb(),$this->getConfig()));
        $employeeMap->load($sessionOwner,Map\Storage::LOADBY_USER_ID);
        $employeeEntity->load($employeeMap->person_id,Employee\Entity\Storage::LOADBY_PERSON);
        $employeeId         = $employeeEntity->id;

        $filter['start_date'] = date("Y-m-d", strtotime($start_date));
        $filter['end_date'] = date("Y-m-d", strtotime($end_date));
        $filter['ref_id'] = $employeeId;
        $entity = new Entity(Entity\Storage::factory($this->getDb(), $this->getConfig()));
        $result = $entity->loadSummary($filter);
        $summary = $result->data;


        //load data attendance
        $entity     = new \Neuron\Attendance\Entity(\Neuron\Attendance\Entity\Storage::factory($this->getDb(),$this->getConfig()));
        $canAttend    = $entity->canAttend($sessmgt);
        $paramsAttendance['info'] = $canAttend->info;
        $paramsAttendance = array_merge($paramsAttendance, $canAttend->data);

         //load data shift
        $shift    = new \Neuron\Attendance\Timetable\Objects(\Neuron\Attendance\Timetable\Objects\Storage::factory($this->getDb(),$this->getConfig()));
        $resultShift     = $shift->listDataShift(['ref_id'=>$sessmgt['ref_id']]);
        $dataShift = $resultShift['data'];
        
		return $this->getView('attendance/summary', get_defined_vars());
	}

    public function summaryDataAction() {

		$sessmgt = $this->getSession()->get();

        $post = $this->getREST()->getParameters();

        //load data summary
        $filter['start_date'] = isset($post['start_date'])? date("Y-m-d", strtotime($post['start_date'])) : date("Y-m-d", mktime(0, 0, 0, date("m"), 1));
        $filter['end_date'] = isset($post['end_date'])? date("Y-m-d", strtotime($post['end_date'])) : date("Y-m-t");
        $filter['ref_id'] = $sessmgt['ref_id'];
        $entity = new Entity(Entity\Storage::factory($this->getDb(), $this->getConfig()));
        $result = $entity->loadSummary($filter);

        return $this->getREST()->getResponse($result);
	}

    public function howWellStatusAction() {
        $result = new \Neuron\Generic\Result();
        $session = $this->getSession();
		$sessmgt = $session->get();
        $sessOwner = $session->owner();
        $employeeMap = new Map(Map\Storage::factory($this->getDb(),$this->getConfig()));
        $employeeMap->load($sessOwner,Map\Storage::LOADBY_USER_ID);
        $employee   = $employeeMap->employee();

        $post = $this->getREST()->getParameters();
        if($post['interval'] == 1){
            $start_date = date('Y-m-d',strtotime('this week monday'));
            $end_date = date('Y-m-d',strtotime('this week sunday'));
        }elseif ($post['interval'] == 2){ 
            $start_date = date("Y-m-d", mktime(0, 0, 0, date("m"), 1));
            $end_date = date("Y-m-t");
        }elseif ($post['interval'] == 3){ 
            $start_date = date("Y-01-d", mktime(0, 0, 0, date("m"), 1));
            $end_date = date("Y-12-t");
        }elseif ($post['interval'] == 4){
            $range = explode(' - ',$post['range']);
            $start_date = date("Y-m-d", strtotime($range[0]));
            $end_date = date("Y-m-d", strtotime($range[1])) ;
        }


        //load data summary
        $filter['start_date'] = $start_date;
        $filter['end_date'] = $end_date;
        $filter['ref_id'] = $employee->id;
        $entity = new Entity(Entity\Storage::factory($this->getDb(), $this->getConfig()));
        $retval = $entity->loadSummary($filter);
        $dataRetval = $retval->data;
        $dataRetval['start_date'] = date("d F Y", strtotime($start_date));
        $dataRetval['end_date'] = date("d F Y", strtotime($end_date));

        $result->data = $dataRetval;
        return $this->getREST()->getResponse($result);
	}

    public function summaryRecommendationAction(){
        $result = new \Neuron\Generic\Result();
		$session = $this->getSession();
		$sessmgt = $session->get();
        $sessOwner = $session->owner();
        $employeeMap = new Map(Map\Storage::factory($this->getDb(),$this->getConfig()));
        $employeeMap->load($sessOwner,Map\Storage::LOADBY_USER_ID);
        $employee   = $employeeMap->employee();

        $post = $this->getREST()->getParameters();
        if($post['interval'] == 1){
            $start_date = date('Y-m-d',strtotime('this week monday'));
            $end_date = date('Y-m-d',strtotime('this week sunday'));
        }elseif ($post['interval'] == 2){ 
            $start_date = date("Y-m-d", mktime(0, 0, 0, date("m"), 1));
            $end_date = date("Y-m-t");
        }elseif ($post['interval'] == 3){ 
            $start_date = date("Y-01-d", mktime(0, 0, 0, date("m"), 1));
            $end_date = date("Y-12-31");
        }elseif ($post['interval'] == 4){
            $range = explode(' - ',$post['range']);
            $start_date = date("Y-m-d", strtotime($range[0]));
            $end_date = date("Y-m-d", strtotime($range[1])) ;
        }

        $filter['start_date'] = $start_date;
        $filter['end_date'] = $end_date;
        $filter['ref_id'] = $employee->id;
        $filter['user_id'] = $sessmgt['user_id'];
        // $entity = new Entity(Entity\Storage::factory($this->getDb(), $this->getConfig()));
        // $result = $entity->loadSummaryRecommendation($filter);
        $sApproval = \Neuron\Order\Entity\Storage::factory($this->getDb(), $this->getConfig());
        $mApproval = new \Neuron\Approval\Entities($sApproval);
        $where['where']['create_user_id'] = $filter['user_id'];
        $where['where']['order_status_id'] = 2;
        $where['where']['xn5'] = 2;
        $where['where']['sdate'] = $filter['start_date'];
        $where['where']['edate'] = $filter['end_date'];
        $retval = $mApproval->reportChart(null, \Neuron\Order\Entity\Storage::LIST_ALL, $where, true);


        $dataRetval['list'] = $retval->data;
        $dataRetval['start_date'] = date("d F Y", strtotime($start_date));
        $dataRetval['end_date'] = date("d F Y", strtotime($end_date));

        $result->data = $dataRetval;
        return $this->getREST()->getResponse($result);
    }

    public function summaryDetailAction() {

        $this->using('bootbox',21,  array(
            'https://cdnjs.cloudflare.com/ajax/libs/bootbox.js/4.4.0/bootbox.min.js'
        ));
		$this->using('p-loading', 22);
        $this->using('moment', 23, array(
            'https://cdnjs.cloudflare.com/ajax/libs/moment.js/2.29.1/moment.min.js'
        ));
        $this->using('jquery.validate', 31, array(
            'https://cdnjs.cloudflare.com/ajax/libs/jquery-validate/1.19.2/jquery.validate.min.js',
            'https://cdnjs.cloudflare.com/ajax/libs/jquery-validate/1.19.2/additional-methods.min.js'
        ));
        $this->using('air-datepicker', 24, array(
            'https://cdnjs.cloudflare.com/ajax/libs/air-datepicker/2.2.3/js/datepicker.min.js',
            'https://cdnjs.cloudflare.com/ajax/libs/air-datepicker/2.2.3/css/datepicker.min.css',
            'https://cdnjs.cloudflare.com/ajax/libs/air-datepicker/2.2.3/js/i18n/datepicker.en.min.js'
        ));

		$sessmgt = $this->getSession()->get();
		$sess = (isset($sessmgt['ref_id'])) ? true : false;

        $type_id = $this->params()->fromQuery('type');
        $filter  = $this->params()->fromQuery('filter');
        $delimiter = $this->translate("to");
        $range = explode($delimiter, $filter);

		$title = 'Attendance';
        if(count($range) > 1){
            $start_date = date("d F Y", strtotime($range[0]));
            $end_date = date('d F Y', strtotime($range[1]));
        }else{
            $start_date = date("d F Y", mktime(0, 0, 0, date("m"), 1));
            $end_date = date('t F Y');
        }

		return $this->getView('attendance/summary-detail', get_defined_vars());
	}

    public function dataSummaryDetailAction() {
        $result = new \Neuron\Generic\Result();
		$sessmgt = $this->getSession()->get();

        // penyesuaian dengan reenginering employee
        $userId = $this->getSession()->owner();
            
        $map 				= new Map(Map\Storage::factory($this->getDb(),$this->getConfig()));
        $map->load($userId,Map\Storage::LOADBY_USERID);
        $employee 			= $map->employee();


        $dataAddt = $this->getPostParams();
        $post = $this->getREST()->getParameters();

        if ($this->getREST()->getFormatter() instanceof Datatable) {
            $mode = Entity\Storage::LISTBY_KEYWORD;
        } else {
            $mode = Entity\Storage::LISTBY_PARAM; //$data['mode'];
        }

        //load data summary
        $dataAddt['start_date'] = isset($dataAddt['start_date'])? date("Y-m-d", strtotime($dataAddt['start_date'])) : date("Y-m-d", mktime(0, 0, 0, date("m"), 1));
        $dataAddt['end_date'] = isset($dataAddt['end_date'])? date("Y-m-d", strtotime($dataAddt['end_date'])) : date("Y-m-t");
        $dataAddt['ref_id'] = $employee->id;

        if(isset($dataAddt['search']['value'])){
            $post['params']['search']['value'] = $dataAddt['search']['value'];
        }

        if(isset($dataAddt['type_id']) && empty($dataAddt['type_id'])){
            unset($dataAddt['type_id']);
        }else{
            $post['params']['type_id'] = $dataAddt['type_id'];
        }
        $entities = new \Neuron\Attendance\Entities(Entity\Storage::factory($this->getDb(), $this->getConfig()));
        $result       = $entities->load(null,$mode,$dataAddt);
        if($result->code == 0){
            if(isset($dataAddt['draw'])){
                $resData    = array(
                    'draw' => $dataAddt['draw'],
                    'recordsTotal' => $result->data['countTotal'],
                    'recordsFiltered' => $result->data['countTotal'],
                    'data'  => $result->data['list']
                );
            }else{
                $resData    = array(
                    'draw' => $dataAddt['draw'],
                    'recordsTotal' => 0,
                    'recordsFiltered' => 0,
                    'data'  => array()
                );
            }
            return $this->getOutput(json_encode($resData));

        }

        return $this->getREST()->getResponse($result);

	}

    public function formAction() {

        $this->using('bootbox',21,  array(
            'https://cdnjs.cloudflare.com/ajax/libs/bootbox.js/4.4.0/bootbox.min.js'
        ));
		$this->using('p-loading', 22);
		$this->using('moment', 23, array(
            'https://cdnjs.cloudflare.com/ajax/libs/moment.js/2.29.1/moment.min.js'
        ));
		$this->using('jquery.validate', 31, array(
            'https://cdnjs.cloudflare.com/ajax/libs/jquery-validate/1.19.2/jquery.validate.min.js',
            'https://cdnjs.cloudflare.com/ajax/libs/jquery-validate/1.19.2/additional-methods.min.js'
        ));
        $this->using('air-datepicker', 24, array(
            'https://cdnjs.cloudflare.com/ajax/libs/air-datepicker/2.2.3/js/datepicker.min.js',
            'https://cdnjs.cloudflare.com/ajax/libs/air-datepicker/2.2.3/css/datepicker.min.css',
            'https://cdnjs.cloudflare.com/ajax/libs/air-datepicker/2.2.3/js/i18n/datepicker.en.min.js'
        ));

		$sessmgt    = $this->getSession()->get();
        $mapper     = $this->getMapper()->data;
        $employee   = $mapper['employee'];

		$guid = $this->getSession()->getId();
		$sess = (isset($sessmgt['ref_id'])) ? true : false;
		$title = 'Approval';
        $start_date = date("01 F Y", mktime(0, 0, 0, date("01"), 1));
        $end_date = date('31 F Y', mktime(0, 0, 0, date("12"), 1));

        /** 
         * Get all employee
         * ini dipake untuk list employee kalo administrator, seharusnya bisa pake ajax 
        */
        $params['order'] = 'first_name';
        $params['status_id'] = 1 ;

        $sEmployee = \Neuron\Employee\Entity\Storage::factory($this->getDb(), $this->getConfig());
		$employeeEntities = new \Neuron\Employee\Entities($sEmployee);
		$employeeEntities->load(null,Entity\Storage::LIST_ALL,$params);

        $module_attendance = true;
        $entity     = new \Neuron\Attendance\Entity(\Neuron\Attendance\Entity\Storage::factory($this->getDb(),$this->getConfig()));
        $result     = $entity->load(
            null,\Neuron\Attendance\Entity\Storage::LOADBY_ID,
            array(
                'ref_id'        => $employee->id,
                'start_date'    => date("Y-m-d")
            )
        );
        $dataAttendance = $entity->pull();

        $attendanceEntity   = new Entity(Entity\Storage::factory($this->getDb(),$this->getConfig()));
        $getParamsAttd      = $attendanceEntity->checkAttend();
        $paramsAttendance   = $getParamsAttd->data;
        
        $leaveTypes = [];

        if(class_exists("\Neuron\Leave\Model\ReportFacts")){
            $reportFacts 	= new \Neuron\Leave\Model\ReportFacts(\Neuron\Leave\Model\ReportFact\Storage::factory(
                $this->getDb(),
                $this->getConfig()
            ));
            $leaveTypes 	= $reportFacts->load(
                $sessmgt['ref_id'],
                \Neuron\Leave\Model\ReportFact\Storage::LISTBY_REFID
            )->data;
        }
        

        $types      	= new ApprovalSubtypes(OrderSubtype\Storage::factory($this->getDb(),$this->getConfig()));		
        $dataTypes     	= $types->load(null, OrderSubtype\Storage::LIST_ALL, array());
        if($dataTypes->code ==0){
            $arrType = array();
            if($dataTypes->data['list']){
                foreach ($dataTypes->data['list'] as $type) {
                    if($type['order_type_id'] == 2200){
                        $available = false;
                        foreach ($leaveTypes as $fact) {
                            if($type['order_subtype_id'] == $fact['order_subtype_id']){
                                $type['include_holiday'] = $fact['include_holiday'];
                                $type['type_code'] = $fact['type_code'];
                                $type['fact_id'] = $fact['leave_fact_id'];
                                $type['quota_remain'] = $fact['quota_remain'];
                                $type['quota'] = $fact['quota'];
                                $available = true;
                            }
                        }
                        if($available){
                            $arrType[]  = $type;    
                        }
                    }else{
                        $arrType[]  = $type;
                    }
                }
            }
        }else{
            $arrType = array();
        }
		return $this->getView('attendance/form', get_defined_vars());
	}

    # dashboard summary [END]

    # dashboard reporting HR [BEGIN]
    public function reportingHrAction() {

        $this->using('bootbox', 21);
		$this->using('p-loading', 22);
		$this->using('moment', 23);
		$this->usingjs('jquery.validate', 34);
        $this->using('air-datepicker', 24);

        $sessmgt = $this->getSession()->get();
        // seleksi hanya administrator yang dapat mengakses halaman ini
        $last_filter = $this->params()->fromQuery();

        $title = 'Dashboard Attendance';
        $start_date = (isset($last_filter['start_date']) && !empty($last_filter['start_date'])) ? $last_filter['start_date'] : date("d F Y", mktime(0, 0, 0, date("m"), 1));
        $end_date = (isset($last_filter['end_date']) && !empty($last_filter['end_date'])) ? $last_filter['end_date'] : date('t F Y');
        
        $interval = (isset($last_filter['interval']) && !empty($last_filter['interval'])) ? $last_filter['interval'] : 2;
        $interval_recomendation = (isset($last_filter['interval_recomendation']) && !empty($last_filter['interval_recomendation'])) ? $last_filter['interval_recomendation'] : 2;
        
        $interval_vacation = (isset($last_filter['interval_vacation']) && !empty($last_filter['interval_vacation'])) ? $last_filter['interval_vacation'] : 1;
        $interval_attfrom = (isset($last_filter['interval_attfrom']) && !empty($last_filter['interval_attfrom'])) ? $last_filter['interval_attfrom'] : 1;
        $interval_attornot = (isset($last_filter['interval_attornot']) && !empty($last_filter['interval_attornot'])) ? $last_filter['interval_attornot'] : 1;
        
        $interval_monitoring_att = (isset($last_filter['interval_monitoring_att']) && !empty($last_filter['interval_monitoring_att'])) ? $last_filter['interval_monitoring_att'] : 6;
        $interval_monitoring_pmt = (isset($last_filter['interval_monitoring_pmt']) && !empty($last_filter['interval_monitoring_pmt'])) ? $last_filter['interval_monitoring_pmt'] : 6;

        $range = (isset($last_filter['range']) && !empty($last_filter['range'])) ? $last_filter['range'] : null;
        $range_recomendation = (isset($last_filter['range_recomendation']) && !empty($last_filter['range_recomendation'])) ? $last_filter['range_recomendation'] : 'null';
        $range_attfrom = (isset($last_filter['range_attfrom']) && !empty($last_filter['range_attfrom'])) ? $last_filter['range_attfrom'] : 'null';
        $range_vacation = (isset($last_filter['range_vacation']) && !empty($last_filter['range_vacation'])) ? $last_filter['range_vacation'] : 'null';
        $range_attornot = (isset($last_filter['range_attornot']) && !empty($last_filter['range_attornot'])) ? $last_filter['range_attornot'] : 'null';

        //load data summary
        
        $filter['start_date'] = date("Y-m-d", strtotime($start_date));
        $filter['end_date'] = date("Y-m-d", strtotime($end_date));
        
        $filter['ref_id'] = $sessmgt['ref_id'];
        $entity = new Entity(Entity\Storage::factory($this->getDb(), $this->getConfig()));
        $result = $entity->loadSummaryHrReporting($filter);
        
        $summary = $result->data;

        return $this->getView('reporting/summary', get_defined_vars());
		
    }

    public function summaryDetailHrAction() {

        $this->using('bootbox',21,  array(
            'https://cdnjs.cloudflare.com/ajax/libs/bootbox.js/4.4.0/bootbox.min.js'
        ));
		$this->using('p-loading', 22);
        $this->using('moment', 23, array(
            'https://cdnjs.cloudflare.com/ajax/libs/moment.js/2.29.1/moment.min.js'
        ));
        $this->using('jquery.validate', 31, array(
            'https://cdnjs.cloudflare.com/ajax/libs/jquery-validate/1.19.2/jquery.validate.min.js',
            'https://cdnjs.cloudflare.com/ajax/libs/jquery-validate/1.19.2/additional-methods.min.js'
        ));
        $this->using('air-datepicker', 24, array(
            'https://cdnjs.cloudflare.com/ajax/libs/air-datepicker/2.2.3/js/datepicker.min.js',
            'https://cdnjs.cloudflare.com/ajax/libs/air-datepicker/2.2.3/css/datepicker.min.css',
            'https://cdnjs.cloudflare.com/ajax/libs/air-datepicker/2.2.3/js/i18n/datepicker.en.min.js'
        ));

		$sessmgt = $this->getSession()->get();
		$sess = (isset($sessmgt['ref_id'])) ? true : false;

        
        $type_id = $this->params()->fromQuery('type');
        $order_type_id = $this->params()->fromQuery('type_id');
        $order_subtype_id = $this->params()->fromQuery('subtype_id');
        if(is_array($order_subtype_id)){
            $order_subtype_id = json_encode($order_subtype_id);
        }
        $interval_recomendation = $this->params()->fromQuery('interval_recomendation');
        $interval_attfrom = $this->params()->fromQuery('interval_attfrom');
        $interval_attornot = $this->params()->fromQuery('interval_attornot');
        $table_type = $this->params()->fromQuery('table_type');
        $interval_monitoring_att = $this->params()->fromQuery('interval_monitoring_att');

        if ($this->params()->fromQuery('interval_monitoring_pmt')){
            $interval_monitoring_pmt = $this->params()->fromQuery('interval_monitoring_pmt');
        } else {
            $interval_monitoring_pmt = 999;
        }

        $filter  = $this->params()->fromQuery('filter');
        $delimiter = $this->translate("to");
        $range = explode($delimiter, $filter);

        // print_r($interval_attfrom);die('wadidaw');
        // detail recomendation
        if($interval_recomendation == 1){
            $start_date = date('d F Y');
            $end_date = date('d F Y');
        }elseif($interval_recomendation == 2){
            $start_date = date('d F Y',strtotime('this week monday'));
            $end_date = date('d F Y',strtotime('this week sunday'));
        }elseif ($interval_recomendation == 3){ 
            $start_date = date("d F Y", mktime(0, 0, 0, date("m"), 1));
            $end_date = date("t F Y");
        }elseif ($interval_recomendation == 4){ 
            $start_date = date("Y-01-d", mktime(0, 0, 0, date("m"), 1));
            $end_date = date("Y-12-31");
        }elseif ($interval_recomendation == 5){
            $range = explode(' - ',$this->params()->fromQuery('range_recomendation'));
            $end = strtr($range[1], '/', '-');
            $start_date = date("d F Y", strtotime($range[0]));
            $end_date = date("d F Y", strtotime($end));

        }
      
        // chart att from
        elseif($interval_attfrom == 1){
            $start_date = date('d F Y');
            $end_date = date('d F Y');
        }elseif($interval_attfrom == 2){
            $start_date = date('d F Y',strtotime('this week monday'));
            $end_date = date('d F Y',strtotime('this week sunday'));
        }elseif ($interval_attfrom == 3){ 
            $start_date = date("d F Y", mktime(0, 0, 0, date("m"), 1));
            $end_date = date("t F Y");
        }elseif ($interval_attfrom == 4){ 
            $start_date = date("Y-01-d", mktime(0, 0, 0, date("m"), 1));
            $end_date = date("Y-12-31");
        }elseif ($interval_attfrom == 5){
            $range = explode(' - ',$this->params()->fromQuery('range_attfrom'));
            $end = strtr($range[1], '/', '-');
            $start_date = date("d F Y", strtotime($range[0]));
            $end_date = date("d F Y", strtotime($end));

           
        }

        // chart att or not
        elseif($interval_attornot == 1){
            $start_date = date('d F Y');
            $end_date = date('d F Y');
        }elseif($interval_attornot == 2){
            $start_date = date('d F Y',strtotime('this week monday'));
            $end_date = date('d F Y',strtotime('this week sunday'));
        }elseif ($interval_attornot == 3){ 
            $start_date = date("d F Y", mktime(0, 0, 0, date("m"), 1));
            $end_date = date("t F Y");
        }elseif ($interval_attornot == 4){ 
            $start_date = date("Y-01-d", mktime(0, 0, 0, date("m"), 1));
            $end_date = date("Y-12-31");
        }elseif ($interval_attornot == 5){
            $range = explode(' - ',$this->params()->fromQuery('range_attornot'));
            $start = str_replace('/', '-', $range[0]);
            $end = str_replace('/', '-', $range[1]);

            $start_date = date("d F Y", strtotime($start));
            $end_date = date("d F Y", strtotime($end));
        }

        // detail monitoring attendance
        elseif($interval_monitoring_att == 3) {
            $start_dates = date("Y-01-d", mktime(0, 0, 0, date("m"), 1));
            $end_dates = date("Y-12-31");

            $start_date = date("d F Y",strtotime($start_dates));
            $end_date = date("d F Y",strtotime($end_dates));

        }elseif($interval_monitoring_att == 6) {
            $end_date = date('d F Y',mktime(0, 0, 0, date("m"), 1));
            $start_date = date("t F Y", strtotime("-6 months", strtotime($end_date)));
        }

        // detail monitoring pamantra
        elseif($interval_monitoring_pmt == 3) {
            $start_dates = date("Y-01-d", mktime(0, 0, 0, date("m"), 1));
            $end_dates = date("Y-12-31");

            $start_date = date("d F Y",strtotime($start_dates));
            $end_date = date("d F Y",strtotime($end_dates));

        }elseif($interval_monitoring_pmt == 6) {
            $end_date = date('d F Y',mktime(0, 0, 0, date("m"), 1));
            $start_date = date("t F Y", strtotime("-6 months", strtotime($end_date)));
        }

        
        elseif(count($range) > 1){
            $start_date = date("d F Y", strtotime($range[0]));
            $end_date = date('d F Y', strtotime($range[1]));
        }else{
            $start_date = date("d F Y", mktime(0, 0, 0, date("m"), 1));
            $end_date = date('t F Y');
        }

		$title = 'Attendance';
        
		return $this->getView('reporting/summary-detail-hr', get_defined_vars());
	}

    public function summaryRecommendationHrAction(){
        $result = new \Neuron\Generic\Result();
		$sessmgt = $this->getSession()->get();

        $post = $this->getREST()->getParameters();
        
        if($post['interval'] == 1){
            $start_date = date('Y-m-d');
            $end_date = date('Y-m-d');
        }elseif($post['interval'] == 2){
            $start_date = date('Y-m-d',strtotime('this week monday'));
            $end_date = date('Y-m-d',strtotime('this week sunday'));
        }elseif ($post['interval'] == 3){ 
            $start_date = date("Y-m-d", mktime(0, 0, 0, date("m"), 1));
            $end_date = date("Y-m-t");
        }elseif ($post['interval'] == 4){ 
            $start_date = date("Y-01-d", mktime(0, 0, 0, date("m"), 1));
            $end_date = date("Y-12-31");
        }elseif ($post['interval'] == 5){
            $range = explode(' - ',$post['range']);
            
            $start = strtr($range[0], '/', '-');
            $end = strtr($range[1], '/', '-');

            $start_date = date("Y-m-d", strtotime($start));
            $end_date = date("Y-m-d", strtotime($end));

        }

        $filter['start_date'] = $start_date;
        $filter['end_date'] = $end_date;

        
        // $filter['ref_id'] = $sessmgt['ref_id'];
        // $filter['user_id'] = $sessmgt['user_id'];
        // $entity = new Entity(Entity\Storage::factory($this->getDb(), $this->getConfig()));
        // $result = $entity->loadSummaryRecommendation($filter);
        $sApproval = \Neuron\Order\Entity\Storage::factory($this->getDb(), $this->getConfig());
        $mApproval = new \Neuron\Approval\Entities($sApproval);
        // $where['where']['create_user_id'] = $filter['user_id'];
        $where['where']['order_status_id'] = 2;
        $where['where']['xn5'] = 2;
        $where['where']['sdate'] = $filter['start_date'];
        $where['where']['edate'] = $filter['end_date'];
        $retval = $mApproval->reportChart(null, \Neuron\Order\Entity\Storage::LIST_ALL, $where, true);


        $dataRetval['list'] = $retval->data;
        $dataRetval['start_date'] = date("d F Y", strtotime($start_date));
        $dataRetval['end_date'] = date("d F Y", strtotime($end_date));

        $result->data = $dataRetval;
        return $this->getREST()->getResponse($result);
    }

    public function summaryHrDataAction() {

		$sessmgt = $this->getSession()->get();

        $post = $this->getREST()->getParameters();

        //load data summary
        $filter['start_date'] = isset($post['start_date'])? date("Y-m-d", strtotime($post['start_date'])) : date("Y-m-d", mktime(0, 0, 0, date("m"), 1));
        $filter['end_date'] = isset($post['end_date'])? date("Y-m-d", strtotime($post['end_date'])) : date("Y-m-t");
        // $filter['ref_id'] = $sessmgt['ref_id'];
        $entity = new Entity(Entity\Storage::factory($this->getDb(), $this->getConfig()));
        $result = $entity->loadSummaryHrReporting($filter);

        return $this->getREST()->getResponse($result);
	}

    public function attendFromAction() {
        $result = new \Neuron\Generic\Result();
		$sessmgt = $this->getSession()->get();

        $post = $this->getREST()->getParameters();
        if($post['interval'] == 1){
            $start_date = date('Y-m-d');
            $end_date = date('Y-m-d');
        }elseif($post['interval'] == 2){
            $start_date = date('Y-m-d',strtotime('this week monday'));
            $end_date = date('Y-m-d',strtotime('this week sunday'));
        }elseif ($post['interval'] == 3){ 
            $start_date = date("Y-m-d", mktime(0, 0, 0, date("m"), 1));
            $end_date = date("Y-m-t");
        }elseif ($post['interval'] == 4){ 
            $start_date = date("Y-01-d", mktime(0, 0, 0, date("m"), 1));
            $end_date = date("Y-12-t");
        }elseif ($post['interval'] == 5){
            $range = explode(' - ',$post['range_attfrom']);
            $end = strtr($range[1], '/', '-');
            $start_date = date("Y-m-d", strtotime($range[0]));
            $end_date = date("Y-m-d", strtotime($end));
        }


        //load data summary
        $filter['start_date'] = $start_date;
        $filter['end_date'] = $end_date;

        // print_r($filter);die();

        // $filter['ref_id'] = $sessmgt['ref_id'];
        $entity = new Entity(Entity\Storage::factory($this->getDb(), $this->getConfig()));
        $retval = $entity->loadSummaryHrReporting($filter);
        $dataRetval = $retval->data;
        $dataRetval['start_date'] = date("d F Y", strtotime($start_date));
        $dataRetval['end_date'] = date("d F Y", strtotime($end_date));

        $result->data = $dataRetval;
        
        return $this->getREST()->getResponse($result);
	}

    public function attendStatsAction() {
        $result = new \Neuron\Generic\Result();
		$sessmgt = $this->getSession()->get();

        $post = $this->getREST()->getParameters();
        if($post['interval'] == 1){
            $start_date = date('Y-m-d');
            $end_date = date('Y-m-d');
        }elseif($post['interval'] == 2){
            $start_date = date('Y-m-d',strtotime('this week monday'));
            $end_date = date('Y-m-d',strtotime('this week sunday'));
        }elseif ($post['interval'] == 3){ 
            $start_date = date("Y-m-d", mktime(0, 0, 0, date("m"), 1));
            $end_date = date("Y-m-t");
        }elseif ($post['interval'] == 4){ 
            $start_date = date("Y-01-d", mktime(0, 0, 0, date("m"), 1));
            $end_date = date("Y-12-t");
        }elseif ($post['interval'] == 5){
            $range = explode(' - ',$post['range_attornot']);
            $start = str_replace('/', '-', $range[0]);
            $end = str_replace('/', '-', $range[1]);

            $start_date = date("Y-m-d", strtotime($start));
            $end_date = date("Y-m-d", strtotime($end));
        }


        //load data summary
        $filter['start_date'] = $start_date;
        $filter['end_date'] = $end_date;


        // $filter['ref_id'] = $sessmgt['ref_id'];
        $entity = new Entity(Entity\Storage::factory($this->getDb(), $this->getConfig()));
        $retval = $entity->loadSummaryHrReporting($filter);
        $dataRetval = $retval->data;
        $dataRetval['start_date'] = date("d F Y", strtotime($start_date));
        $dataRetval['end_date'] = date("d F Y", strtotime($end_date));

        $result->data = $dataRetval;
        
        return $this->getREST()->getResponse($result);
	}

    public function attendanceStatusAction(){
        $result 				= new \Neuron\Generic\Result;
		$post 					= $this->getRequest()->getPost();
		$error 					= array();


		/* Fetch Attendance */
		$modelAttendance 		= new Attendance(Attendance\Storage::factory($this->getDb(),$this->getConfig()));
		$fetchAttendance		= $modelAttendance->countAttendance(null,Attendance\Storage::LIST_ALL,array(),intval($post['time_mode']));
		$totalAttendance 		= $fetchAttendance->data;

		if($fetchAttendance->code != 0){
			$error[]			= 'Error fetching `Attendance` Data.';
		}

		/* Fetch not attend */
		$attendance 			= new Attendance(Attendance\Storage::factory($this->getDb(),$this->getConfig()));
		$notAttend 				= $attendance->countNotAttend(date('Y-m-d'),date('Y-m-d'));

		$widgetData 			= array(
			'data' 				=> [
				[
					'label' 	=> "Attendance",
					'total' 	=> intval($totalAttendance),
					'color' 	=> "#89c4f4"
				],
				[
					'label' 	=> "No information",
					'total' 	=> intval($notAttend->data['total']),
					'color' 	=> "#e26a6a"
				]
			],
			'error' 				=> $error,
			'no_information_data'	=> $notAttend->data['list']
		);


		/* Fetch From Leave */
		if(class_exists('\Neuron\Leave\Model\LeaveMain')){
			$leaveData 		= 	$modelAttendance->fetchLeaveData($post['time_mode']);
			$mergeArray 	= array_merge($widgetData,$leaveData->data);
			$result->data 	= $mergeArray;
		}else{
			$result->data 	= $widgetData;
		}

		return $this->getREST()->getResponse($result);

    }

    public function testSumAction(){
        $last_filter = $this->params()->fromQuery();

         //load data summary
         $filter['start_date'] = date("Y-m-d", strtotime($start_date));
         $filter['end_date'] = date("Y-m-d", strtotime($end_date));
         $filter['ref_id'] = $sessmgt['ref_id'];
         $entity = new Entity(Entity\Storage::factory($this->getDb(), $this->getConfig()));
         $result = $entity->loadSummaryHrReporting($filter);
         $summary = $result->data;

         echo '<pre>',print_r($summary,1),'</pre>';

         die();
    }

    public function dataSummaryDetailHrAction1() {
        $result = new \Neuron\Generic\Result();
		// $sessmgt = $this->getSession()->get();
        $dataAddt = $this->getPostParams();
        $post = $this->getREST()->getParameters();

        if ($this->getREST()->getFormatter() instanceof Datatable) {
            $mode = Entity\Storage::LISTBY_KEYWORD;
        } else {
            $mode = Entity\Storage::LISTBY_PARAM; //$data['mode'];
        }

        //load data summary
        $dataAddt['start_date'] = isset($dataAddt['start_date'])? date("Y-m-d", strtotime($dataAddt['start_date'])) : date("Y-m-d", mktime(0, 0, 0, date("m"), 1));
        $dataAddt['end_date'] = isset($dataAddt['end_date'])? date("Y-m-d", strtotime($dataAddt['end_date'])) : date("Y-m-t");
        // $dataAddt['ref_id'] = $sessmgt['ref_id'];

        if(isset($dataAddt['search']['value'])){
            $post['params']['search']['value'] = $dataAddt['search']['value'];
        }

        if(isset($dataAddt['type_id']) && empty($dataAddt['type_id'])){
            unset($dataAddt['type_id']);
        }else{
            $post['params']['type_id'] = $dataAddt['type_id'];
        }
        $entities = new \Neuron\Attendance\Entities(Entity\Storage::factory($this->getDb(), $this->getConfig()));
        $result       = $entities->load(null,$mode,$dataAddt);
        if($result->code == 0){
            if(isset($dataAddt['draw'])){
                $resData    = array(
                    'draw' => $dataAddt['draw'],
                    'recordsTotal' => $result->data['countTotal'],
                    'recordsFiltered' => $result->data['countTotal'],
                    'data'  => $result->data['list']
                );
            }else{
                $resData    = array(
                    'draw' => $dataAddt['draw'],
                    'recordsTotal' => 0,
                    'recordsFiltered' => 0,
                    'data'  => array()
                );
            }
            return $this->getOutput(json_encode($resData));

        }

        return $this->getREST()->getResponse($result);
	}

    public function dataSummaryDetailHrAction() {
        $result = new \Neuron\Generic\Result();
		$sessmgt = $this->getSession()->get();
        $dataAddt = $this->getPostParams();
        $post = $this->getREST()->getParameters();

        if ($this->getREST()->getFormatter() instanceof Datatable) {
            $mode = Entity\Storage::LISTBY_KEYWORD;
        } else {
            $mode = Entity\Storage::LISTBY_PARAM; //$data['mode'];
        }

        //load data summary
        $dataAddt['start_date'] = isset($dataAddt['start_date'])? date("Y-m-d", strtotime($dataAddt['start_date'])) : date("Y-m-d", mktime(0, 0, 0, date("m"), 1));
        $dataAddt['end_date'] = isset($dataAddt['end_date'])? date("Y-m-d", strtotime($dataAddt['end_date'])) : date("Y-m-t");
        // $dataAddt['ref_id'] = $sessmgt['ref_id'];

        if(isset($dataAddt['search']['value'])){
            $post['params']['search']['value'] = $dataAddt['search']['value'];
        }

        if(isset($dataAddt['type_id']) && empty($dataAddt['type_id'])){
            unset($dataAddt['type_id']);
        }else{
            $post['params']['type_id'] = $dataAddt['type_id'];
        }
        $entities = new \Neuron\Attendance\Entities(Entity\Storage::factory($this->getDb(), $this->getConfig()));
        $result       = $entities->load(null,$mode,$dataAddt);
        if($result->code == 0){
            if(isset($dataAddt['draw'])){
                $resData    = array(
                    'draw' => $dataAddt['draw'],
                    'recordsTotal' => $result->data['countTotal'],
                    'recordsFiltered' => $result->data['countTotal'],
                    'data'  => $result->data['list']
                );
            }else{
                $resData    = array(
                    'draw' => $dataAddt['draw'],
                    'recordsTotal' => 0,
                    'recordsFiltered' => 0,
                    'data'  => array()
                );
            }
            return $this->getOutput(json_encode($resData));

        }

        return $this->getREST()->getResponse($result);
	}

    public function dataSummaryDetailAppAction() {
        $data = $this->getListParamaters(\Neuron\Order\Entity\Storage::LISTBY_PAGE);
        $datatable = ($data->code == $this::IS_DATATABLE)? true : false;
        
        /* daftarkan nama module,
           digunakan sebagai parameter module apa yang melakukan request ke module approval
        */
        $module = 'attendance';
        if(class_exists('Neuron\Approval\Entity')){
            if ($this->getREST()->getFormatter() instanceof Datatable) {
                $mode = \Neuron\Order\Entity\Storage::LISTBY_PAGE;
            } else {
                $mode = $this->getREST()->getCode();
            }

            if($datatable == true){
                $params = $data->data['params']['params'];
                $params['page'] = $data->data['params']['page'];
                $params['records'] = $data->data['params']['records'];
                $params['module'] = $module;
                if(isset($params['order'])){
                    $params['order']        = array(
                        'column'       => $post['columns'][ $post['order'][0]['column'] ]['data'],
                        'dir'          => $post['order'][0]['dir']
                    );
                }
            }else{
                $params['module'] = $module;
                $params = $data->data['params'];
            }
            
            $entities = new \Neuron\Approval\Entities(\Neuron\Order\Entity\Storage::factory($this->getDb(),$this->getConfig()));
            $result = $entities->load($data->data['keyword'], $data->data['mode'], $params);
        }
        return $this->getREST()->getResponse($result);
    }

    public function dataSummaryDetailPamantraAction() {
        $result = new \Neuron\Generic\Result();
        $post = $this->getREST()->getParameters();
        
        if(class_exists('Neuron\Payroll\Entity')){
            $storage = \Neuron\Payroll\Bonus\Storage::factory($this->getDb(), $this->getConfig());
            $entity = new \Neuron\Payroll\Bonuses($storage);
            $bonus = new \Neuron\Payroll\Bonus($storage);

            $mode = isset($post['mode'])? $post['mode']: \Neuron\Payroll\Entity\Storage::LIST_ALL;
            $keyword = @$post['keyword'];
            
            // if filter 3 and 6 monts filter
            if ($post['pamantra_range'] == 3) {
            
            
                $pamantra = $result->data['list'];
                $from = date("Y-01", mktime(0, 0, 0, date("m"), 1));

                $start = date('Y-M',strtotime($from));
                $end = date("Y-M", strtotime("+12 months", strtotime($start)));
                $starts = $start;
                $res = [];
                while (strtotime($start) < strtotime($end)) {
                
                    $params = array(
                        'filter_period'=> date('Y-m', strtotime($start)), 
                        'type_chart'=>'Full',
                    );

                    $pdata = $entity->load($keyword, $mode, $params);
                    $res[] = array(
                        "period" =>  $start,
                        "data" => $pdata->data['list']
                    );

                    // counter
                    $start = date ("Y-M", strtotime("+1 months", strtotime($start)));
                }

                $froms = date("Y-01", mktime(0, 0, 0, date("m"), 1));

                $starts = date('d F Y',strtotime($froms));
                $ends = date("t F Y", strtotime("+11 months", strtotime($starts)));
                $result->data = array(
                    "data"=>$res,
                    "end_date" => $ends,
                    "start_date" => $starts,
                );
            
            }else if ($post['pamantra_range'] == 6) {
                $pamantra = $result->data['list'];

                $end = date('Y-M');
                $start = date("Y-M", strtotime("-5 months", strtotime($end)));
                $starts = $start;
                $res = [];
                while (strtotime($start) <= strtotime($end)) {
                
                    $params = array(
                        'filter_period'=> date('Y-m', strtotime($start)), 
                        'type_chart'=>'Full',
                    );

                    $pdata = $entity->load($keyword, $mode, $params);
                    $res[] = array(
                        "period" =>  $start,
                        "data" => $pdata->data['list']
                    );

                    // counter
                    $start = date ("Y-M", strtotime("+1 months", strtotime($start)));
                }

                $ends = date('t F Y',mktime(0, 0, 0, date("m"), 1));
                $starts = date("d F Y", strtotime("-6 months", strtotime($ends)));
                $result->data = array(
                    "data"=>$res,
                    "end_date" => $ends,
                    "start_date" => $starts,
                );
            

            } else {
                $params = array(
                    'start_date' => date('Y-m-d',strtotime($post['start_date'])),
                    'end_date' => date('Y-m-d',strtotime($post['end_date'])),
                    // 'filter_period'=> '2020-01', 
                    'type_chart'=>'Full',
                );

                $result = $entity->load($keyword, $mode, $params);
            }
            return $this->getREST()->getResponse($result);
        }
       
       
    }

    public function dataSummaryMonitoringAttAction() {
        $result = new \Neuron\Generic\Result();
        $post = $this->getREST()->getParameters();

        // $post['interval'] = 6;

        // print_r($post);die();

        if($post['interval'] == 1){
            $start_date = date('Y-m-d');
            $end_date = date('Y-m-d');
        }elseif($post['interval'] == 2){
            $start_date = date('Y-m-d',strtotime('this week monday'));
            $end_date = date('Y-m-d',strtotime('this week sunday'));
        }elseif ($post['interval'] == 3){ 
            $start_date = date("Y-m-d", mktime(0, 0, 0, date("m"), 1));
            $end_date = date("Y-m-t");
        }elseif ($post['interval'] == 4){ 
            $start_date = date("Y-01-d", mktime(0, 0, 0, date("m"), 1));
            $end_date = date("Y-12-t");
        }elseif ($post['interval'] == 5){
            $range = explode(' - ',$post['range']);
            $start_date = date("Y-m-d", strtotime($range[0]));
            $end_date = date("Y-m-d", strtotime($range[1])) ;
        }elseif ($post['interval'] == 6){
            $start_date = date('Y-m-d');
        }

        $entity = new Entity(Entity\Storage::factory($this->getDb(), $this->getConfig()));

        if($post['interval'] == 3){
            $start = date("Y-01-d", mktime(0, 0, 0, date("m"), 1));
            $end = date("Y-12-t");
            $starts = $start;
            $res = [];

            $count_clockin = []; 
            $count_absent = []; 
            $count_period = [];
            while (strtotime($start) <= strtotime($end)) {
                $ends = date('Y-m-t',strtotime($start));

                $filter['start_date'] = $start;
                $filter['end_date'] = $ends;

        
                // $filter['ref_id'] = $sessmgt['ref_id'];
                $retval = $entity->loadSummaryAttendaceHr($filter);

                $count_clockin[] = $retval->data['count_clockin']; 
                $count_absent[] = $retval->data['count_absent']; 
                $count_period[] = date('Y-M',strtotime($start));

                $start = date ("Y-m-d", strtotime("+1 months", strtotime($start)));
            }

            $res = array(
                "count_clockin" => $count_clockin,
                "count_absent" => $count_absent,
                "count_period" => $count_period,
            );

            $starts = date("Y-01-d", mktime(0, 0, 0, date("m"), 1));
            $ends = date("Y-12-t");
            $result->data = array(
                "data"=>$res,
                "start_date" => date('d F Y',strtotime($starts)),
                "end_date" => date('d F Y',strtotime($ends)),
            );

        }elseif ($post['interval'] == 6){
            $end = date("Y-m-d", mktime(0, 0, 0, date("m"), 1));
            $start = date("Y-m-d", strtotime("-5 months", strtotime($end)));
            $starts = $start;
            $res = [];

            $count_clockin = []; 
            $count_absent = []; 
            $count_period = [];
            while (strtotime($start) <= strtotime($end)) {
                $ends = date('Y-m-t',strtotime($start));

                $filter['start_date'] = $start;
                $filter['end_date'] = $ends;

        
                // $filter['ref_id'] = $sessmgt['ref_id'];
                $retval = $entity->loadSummaryAttendaceHr($filter);

                $count_clockin[] = $retval->data['count_clockin']; 
                $count_absent[] = $retval->data['count_absent']; 
                $count_period[] = date('Y-M',strtotime($start));

                $start = date ("Y-m-d", strtotime("+1 months", strtotime($start)));
            }

            $res = array(
                "count_clockin" => $count_clockin,
                "count_absent" => $count_absent,
                "count_period" => $count_period,
            );

            $ends = date("Y-m-d", mktime(0, 0, 0, date("m"), 1));
            $starts = date("Y-m-d", strtotime("-5 months", strtotime($ends)));
            $result->data = array(
                "data"=>$res,
                "start_date" => date('d F Y',strtotime($starts)),
                "end_date" => date('t F Y',strtotime($ends)),
            );


        } else {
            $filter['start_date'] = $start_date;
            $filter['end_date'] = $end_date;
            $result = $entity->loadSummaryAttendaceHr($filter);
        }

        

        return $this->getREST()->getResponse($result);
    }
    # dashboard reporting HR [END]

    # get list type each employee recommendation
    public function formTypesAction() {
        $post = $this->getREST()->getParameters();
        
        $leaveTypes = [];
        
        if(class_exists("\Neuron\Leave\Model\ReportFacts")){
            $reportFacts    = new \Neuron\Leave\Model\ReportFacts(\Neuron\Leave\Model\ReportFact\Storage::factory(
                $this->getDb(),
                $this->getConfig()
            ));
            $leaveTypes     = $reportFacts->load(
                $post['ref_id'],
                \Neuron\Leave\Model\ReportFact\Storage::LISTBY_REFID
            )->data;
        }

        $types          = new ApprovalSubtypes(OrderSubtype\Storage::factory($this->getDb(),$this->getConfig()));       
        $dataTypes      = $types->load(null, OrderSubtype\Storage::LIST_ALL, array());
        if($dataTypes->code ==0){
            $arrType = array();
            if($dataTypes->data['list']){
                foreach ($dataTypes->data['list'] as $type) {
                    if($type['order_type_id'] == 2200){
                        $available = false;
                        foreach ($leaveTypes as $fact) {
                            if($type['order_subtype_id'] == $fact['order_subtype_id']){
                                $type['include_holiday'] = $fact['include_holiday'];
                                $type['type_code'] = $fact['type_code'];
                                $type['fact_id'] = $fact['leave_fact_id'];
                                $type['quota_remain'] = $fact['quota_remain'];
                                $type['quota'] = $fact['quota'];
                                $available = true;
                            }
                        }
                        if($available){
                            $arrType[]  = $type;    
                        }
                    }else{
                        $arrType[]  = $type;
                    }
                }
            }
        }else{
            $arrType = array();
        }

        $htmlView = new \Zend\View\Model\ViewModel();
        $htmlOutput = $htmlView
              ->setTerminal(true)
              ->setTemplate('neuron/attendance/v2_1/attendance/partial/form-types')
              ->setVariable('arrType', $arrType);
        return $htmlOutput;
    }
    #end get typ recommendation each employee
}