<?php
namespace Neuron\Attendance\Controller;

use Neuron\Attendance\Timetable\Group as GroupModel;
use Neuron\Attendance\Timetable\Groups;

use Neuron\Attendance\Timetable;
use Neuron\Generic\Result;

class Group extends Controller{
    protected $ref_code = 'Employee';

    protected function getView($name, $variables = array()) {
        /* default layout & view prefix */
        $layout = 'neuron/attendance/layout/form';
        $prefix = 'neuron/attendance/group';

        /* libraries */
        /* libraries */;
		if(! class_exists('Neuron\Application\Framework\Entity')) {
            $this->using('jquery', null)
                ->using('bootstrap', null)
                ->using('fontawesome', null)
                ->using('generic',null)
                ->using('select2',null)
				->using('theme',99);
        }

       

        /* get layout n view prefix from config */
        $config = $this->getConfig();
        if (isset($config['attendance'])) {
            $config = $config['attendance'];
            $layout = isset($config['formlayout']) ? $config['formlayout'] : $layout;
            $prefix = isset($config['viewprefix']) ? $config['viewprefix'] : $prefix;
        }

        /* set layout and view */
        $this->layout($layout);
        $view = new \Zend\View\Model\ViewModel($variables);
        $view->setTemplate($prefix . '/' . strtolower($name));

        return $view;
    }
	public function loadListAction(){

        try {
            /* get post */
            $post = $this->getRequest()->getPost();

            $filterby = @$this->params()->fromQuery('filterby');
            $filterquery = @$this->params()->fromQuery('filterquery');

            /* bootgrid or API */
            if(isset($post['rowCount']) && isset($post['current'])){
                $params = array(
                    'page' => $post['current'],
                    'records' => $post['rowCount'],
                    'sort' => @$post['sort'],
                );

                $keyword = @$post['searchPhrase'];
                $mode = GroupModel\Storage::LISTBY_PAGE;
                
                $filters = $post['filters'];
                if(!empty($filterquery)){
                    $params['filterby'] = $filterby;
                	$params['filterquery'] = $filterquery;
                	// $mode = GroupModel\Storage::LISTBY_DATE;
                }

                $bootgrid = true;
            }else{
                // $post = $this->getREST()->getParameters();
                $params = $post;
                $keyword = '';
                $mode = $this->getCode() ? $this->getCode() : null;
                $bootgrid = false;
            }

			$storage = GroupModel\Storage::factory($this->getDb(),$this->GetConfig());
            $groups= new Timetable\Groups($storage);

            if(isset($post['mode']) && $post['mode'] == 'LISTBY_PARENT'){
                $mode = GroupModel\Storage::LISTBY_PARENTID;
                $keyword = $post['id'];
            }


            $result = $groups->load($keyword, $mode, $params);

            /* bootgrid? */
            if ($bootgrid) {
                if ($result->code == $result::CODE_SUCCESS) {

                     // calc total 
                    $total = $result->data['countTotal'];;
                    $pager = $result->data['pager'];

                    $listData['rows'] = $result->data['list'];
                    $listData['total'] = $total;
                    $listData['current'] = intval($params['page']);
                    $listData['rowCount'] = $pager->getLimit(); //$roles->count();

                } else {

                    $page = intval($params['page']);
                    $records = intval($params['records']);

                    $listData['rows'] = array();
                    $listData['total'] = $page * $records;
                    $listData['current'] = $page;
                    $listData['rowCount'] = $records;
                }
                return $this->getOutput(json_encode($listData));
                
            }   
            
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            $result = new \Neuron\Generic\Result();
            $result->code = 500;
            $result->info = $ex->getMessage();
            return $this->getREST()->getResponse($result);
        }
        
        return $this->getREST()->getResponse($result);        
    }
    public function getOutput($json) {

        $response = $this->getResponse();
        $response->getHeaders()->addHeaderLine('Content-Type', 'application/json');
        $response->setContent($json);
        return $response;
    }
    public function saveGroupAction(){
        $storage = \Neuron\Attendance\Shift\Storage::factory($this->getDb(), $this->getConfig());
        $ctt = new \Neuron\Attendance\Shift($storage);
        
        $post = $this->getPostParams();

        if (!empty($post['group_id'])) {
            $update = true;
        } else {
            $update = false;
        }

        $retval = $ctt->saveGroup($post, $update);
		
		$requestPost = array(
			'guid' => '',
			'code' => '',
			'info' => '',
			'data' => $post
		); 
		
		$logparam['subject'] = 'ATTENDANCE GROUP SAVEGROUPACTION';
		$logparam['identity'] = '';
		$logparam['user'] = '';
		$logparam['description'] = 'Log proses attendance group saveGroupAction';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode(array('success' => $retval));

		$this->writeLogAction($logparam);
		
        echo json_encode(array('success' => $retval));
        return $this->getResponse();
    }   
    public function uploadGroupAction(){
        $result = new \Neuron\Generic\Result();
        $files = $this->getRequest()->getFiles();

        if(count($files) <= 0 ){
            $result->code = 401;
            $result->info = 'No file selected';
            return $this->getREST()->getResponse($result);
            exit();
        }

        $config = $this->getConfig();
        
        if(!isset($config['start_date'])){
            $result = new \Neuron\Generic\Result();
            $result->info = 'Start date not found in config. Please Contact your Administrator';
            $result->code = 2;
            return $this->getREST()->getResponse($result);
        }

        $uploadPath = getcwd() . '/public/uploads/excel';
        $uploadPath = $uploadPath . '\\' . date('Y/m/d');

        $uploadfile = true;
        if(!file_exists($uploadPath)){
            mkdir($uploadPath, 0755, true);
        }

        if(count($files['file']) > 0){
            foreach($files as $file){
                $fileType = $file['type']; 
                $filename = $uploadPath.'/'.$file['name'];
                // var_dump($filename);die();
                if($fileType != 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'){
                    $uploadfile = false;
                    $result->info = 'The selected file type is not correct. Please choose the correct file.';
                    $result->code = 2;
                    return $this->getREST()->getResponse($result);
                    break;
                    exit();
                }

                if(file_exists($filename)) $filename = substr($filename, 0, strrpos($filename, '.')) . ' Copy ' . date('His') . substr($filename, strrpos($filename, '.'));
            
                if(!move_uploaded_file($file['tmp_name'], $filename)){
                    $uploadfile = false;
                    break;
                }
            }
        }



        if($uploadfile){
            //convert excel to array 
            $xlsx = \SimpleXLSX\SimpleXLSX::parse($filename);
            $rows = $xlsx->rows();


            $modelGroup = new \Neuron\Attendance\Timetable\Group(\Neuron\Attendance\Timetable\Group\Storage::factory(
                $this->getDb(),
                $this->getConfig()
            ));

            $storageLoader = array(
                'config' => $this->getConfig(),
                'adapter' => $this->getDb());

            $result = $modelGroup->saveRows($rows,$storageLoader);

            if($result->code != 0){
                unlink($filename);
            }
        } else {
            unlink($filename);
            $result = new Result(0, 1, 'Upload attachment failed, please try again.');
        }
        return $this->getREST()->getResponse($result);
    }
    public function uploadGroupByRangeAction(){
        $result = new \Neuron\Generic\Result();
        $files = $this->getRequest()->getFiles();

        if(count($files) <= 0 ){
            $result->code = 401;
            $result->info = 'No file selected';
            return $this->getREST()->getResponse($result);
            exit();
        }

        $config = $this->getConfig();
        
        if(!isset($config['start_date'])){
            $result = new \Neuron\Generic\Result();
            $result->info = 'Start date not found in config. Please Contact your Administrator';
            $result->code = 2;
            return $this->getREST()->getResponse($result);
        }

        $uploadPath = getcwd() . '/public/uploads/excel';
        $uploadPath = $uploadPath . '\\' . date('Y/m/d');

        $uploadfile = true;
        if(!file_exists($uploadPath)){
            mkdir($uploadPath, 0755, true);
        }

        if(count($files['file']) > 0){
            foreach($files as $file){
                $fileType = $file['type']; 
                $filename = $uploadPath.'/'.$file['name'];
                // var_dump($filename);die();
                if($fileType != 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'){
                    $uploadfile = false;
                    $result->info = 'The selected file type is not correct. Please choose the correct file.';
                    $result->code = 2;
                    return $this->getREST()->getResponse($result);
                    break;
                    exit();
                }

                if(file_exists($filename)) $filename = substr($filename, 0, strrpos($filename, '.')) . ' Copy ' . date('His') . substr($filename, strrpos($filename, '.'));
            
                if(!move_uploaded_file($file['tmp_name'], $filename)){
                    $uploadfile = false;
                    break;
                }
            }
        }



        if($uploadfile){
            //convert excel to array 
            $xlsx = \SimpleXLSX\SimpleXLSX::parse($filename);
            $rows = $xlsx->rows();

            $modelGroup = new \Neuron\Attendance\Timetable\Group(\Neuron\Attendance\Timetable\Group\Storage::factory(
                $this->getDb(),
                $this->getConfig()
            ));

            $storageLoader = array(
                'config' => $this->getConfig(),
                'adapter' => $this->getDb());

            $result = $modelGroup->saveRowsByRange($rows,$storageLoader);

            if($result->code != 0){
                unlink($filename);
            }
        } else {
            unlink($filename);
            $result = new Result(0, 1, 'Upload attachment failed, please try again.');
        }
        return $this->getREST()->getResponse($result);
    }
    public function saveDuplicateAction(){
        $post = $this->getRequest()->getPost();
        $modelGroup = new \Neuron\Attendance\Timetable\Group(\Neuron\Attendance\Timetable\Group\Storage::factory(
            $this->getDb(),
            $this->getConfig()
        ));
        if($post['action'] == 'replace'){
            $result = $modelGroup->saveReplace($post);
        }else if($post['action'] == 'delete'){
            $result = $modelGroup->saveDelete($post);
        }
		
		$requestPost = array(
			'guid' => '',
			'code' => '',
			'info' => '',
			'data' => $post
		); 
		
		$responseResult = array(
			'code' => $result->code,
			'info' => $result->info,
			'data' => $result->data
		); 
		
		$logparam['subject'] = 'ATTENDANCE GROUP SAVEDUPLICATEACTION';
		$logparam['identity'] = '';
		$logparam['user'] = '';
		$logparam['description'] = 'Log proses attendance group saveDuplicateAction';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode($responseResult);

		$this->writeLogAction($logparam);
		
        return $this->getREST()->getResponse($result); 
    }
    public function testAction(){
        $this->using('p-loading',20);
        $this->using('jquery.bootgrid',21);
        $this->using('bootbox',22);
        return $this->getView('test',get_defined_vars());
    }
    public function listByParentAction(){
        $post = $this->getRequest()->getPost();

        // Ambil untuk where condition
        $storage = \Neuron\Attendance\Timetable\Storage::factory($this->getDb(),$this->getConfig());
        $timetables = new \Neuron\Attendance\Timetables($storage);
        $timetableList = $timetables->load($parent,$mode);
    }
    public function chooseableListAction(){
        // $testErrorHandler = new \asd\qwe;
        
        try {
            /* get post */
            $post = $this->getRequest()->getPost();

            $filterby = @$this->params()->fromQuery('filterby');
            $filterquery = @$this->params()->fromQuery('filterquery');

            /* bootgrid or API */
            if(isset($post['rowCount']) && isset($post['current'])){
                $params = array(
                    'page' => $post['current'],
                    'records' => $post['rowCount'],
                    'sort' => @$post['sort'],
                );

                $keyword = @$post['searchPhrase'];
                $mode = GroupModel\Storage::LISTBY_VALIDDATE;
                
                $filters = $post['filters'];
                if(!empty($filterquery)){
                    $params['filterby'] = $filterby;
                    $params['filterquery'] = $filterquery;
                    // $mode = GroupModel\Storage::LISTBY_DATE;
                }

                $bootgrid = true;
            }else{
                // $post = $this->getREST()->getParameters();
                $params = $post;
                $keyword = '';
                $mode = $this->getCode() ? $this->getCode() : null;
                $bootgrid = false;
            }

            $storage = GroupModel\Storage::factory($this->getDb(),$this->GetConfig());
            $groups= new Timetable\Groups($storage);

            if(isset($post['mode']) && $post['mode'] == 'LISTBY_PARENT'){
                $mode = GroupModel\Storage::LISTBY_PARENTID;
                $keyword = $post['id'];
            }


            $result = $groups->load($keyword, $mode, $params);

            /* bootgrid? */
            if ($bootgrid) {
                if ($result->code == $result::CODE_SUCCESS) {

                     // calc total 
                    $total = $result->data['countTotal'];;
                    $pager = $result->data['pager'];

                    $listData['rows'] = $result->data['list'];
                    $listData['total'] = $total;
                    $listData['current'] = intval($params['page']);
                    $listData['rowCount'] = $pager->getLimit(); //$roles->count();

                } else {

                    $page = intval($params['page']);
                    $records = intval($params['records']);

                    $listData['rows'] = array();
                    $listData['total'] = $page * $records;
                    $listData['current'] = $page;
                    $listData['rowCount'] = $records;
                }
                return $this->getOutput(json_encode($listData));
                
            }   
            
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            $result = new \Neuron\Generic\Result();
            $result->code = 500;
            $result->info = $ex->getMessage();
            return $this->getREST()->getResponse($result);
        }
        
        return $this->getREST()->getResponse($result);        
    }
    public function saveMemberToGroupAction(){
        $post = $this->getRequest()->getPost();

        $post['ref_code'] = $this->ref_code;
        $post['start_date'] = date('Y-m-d');
        $post['last_update_dtm'] = date('Y-m-d');
        $post['update_by'] = 1; 
        $storageMember = GroupModel\Member\Storage::factory($this->getDb(),$this->getConfig());
        $member = new GroupModel\Member($storageMember);
        $result = new Result();

        
        // CHECK IF GROUP ALREADY MAPPED
            $storageTimetable = GroupModel\Timetable\Storage::factory($this->getDb(),$this->getConfig());
            $groupTimetables = new GroupModel\Timetables($storageTimetable);
            $dataGroupTimetables = $groupTimetables->load($post['group_id'],GroupModel\Timetable\Storage::LISTBY_GROUPID)->data['list'];


        if($dataGroupTimetables && !isset($post['force'])){
            $result->code = 999;
            $result->info = 'This group already mapped to shift.';
            $result->data = array(
                'post' => $post);
        }else{
			if($post['ref_id']){
					
				$ref_id = explode(',', $post['ref_id']);
				$ref_name = explode(',', $post['ref_name']);
				$failed = array();
				
				foreach ($ref_id as $key => $value) {
					$post['ref_id'] = $value;
					
					foreach ($ref_name as $key => $values) {
						$post['ref_name'] = $values;
						
						$loadMember = $member->loadEx(null,GroupModel\Member\Storage::LOADBY_GROUP_REFID,$post)->data;
						if($loadMember){
							// $result->code =2 ;
							// $result->info = 'This person is already a member';
							array_push($failed, $post['ref_name']);
						}else{
							$result = $member->saveEx($post);
							if(isset($post['force'])){                   
								$storageObject = Timetable\Object\Storage::factory($this->getDb(),$this->getConfig());
								$object = new Timetable\Object($storageObject);

								foreach ($dataGroupTimetables as $data) {
									$params =array(
										'timetable_id' => $data['timetable_id'],
										'ref_code' => 'Group',
										'ref_id' => $post['ref_id'],
										'ref_name' => $post['ref_name'],
										'start_date' => $post['start_date'],
										'last_update' => date('Y-m-d H:i:s'),
										'update_by' => 1
									);
									$object->saveEx($params);
								}

							}
						}
						
					}
					if($failed){
						$result->info = "Employee".implode(',', $failed)."This person is already a member";
						$result->data = $failed;
					}
				}
			}
			
            
        }
		
		$requestPost = array(
			'guid' => '',
			'code' => '',
			'info' => '',
			'data' => $post
		); 
		
		$responseResult = array(
			'code' => $result->code,
			'info' => $result->info,
			'data' => $result->data
		); 
		
		$logparam['subject'] = 'ATTENDANCE GROUP SAVEMEMBERTOGROUPACTION';
		$logparam['identity'] = '';
		$logparam['user'] = '';
		$logparam['description'] = 'Log proses attendance group saveMemberToGroupAction';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode($responseResult);

		$this->writeLogAction($logparam);
		
        return $this->getREST()->getResponse($result);
    }
    public function loadGroupTableAction(){
        $post = $this->getPostParams();
        $storage = GroupModel\Storage::factory($this->getDb(),$this->getConfig());
        $groups = new Groups($storage);
        
        $result = $groups->loadDataTable($post);

        return $this->getOutput(json_encode($result));
    }  
}
