<?php
namespace Neuron\Attendance\Controller;

use Neuron\Attendance\Timetable\Group\Member as MemberModel;
use Neuron\Attendance\Timetable;


class Member extends Controller{
    protected $ref_code = 'Employee';

    protected function getView($name, $variables = array()) {
        /* default layout & view prefix */
        $layout = 'neuron/attendance/layout/form';
        $prefix = 'neuron/attendance/group';
        
        /* libraries */
        if(! class_exists('Neuron\Application\Framework\Entity')) {
            $this->using('jquery', null)
                ->using('bootstrap', null)
                ->using('fontawesome', null)
                ->using('generic',null)
                ->using('select2',null)
				->using('theme',99);
        }

        /* get layout n view prefix from config */
        $config = $this->getConfig();
        if (isset($config['attendance'])) {
            $config = $config['attendance'];
            $layout = isset($config['formlayout']) ? $config['formlayout'] : $layout;
            $prefix = isset($config['viewprefix']) ? $config['viewprefix'] : $prefix;
        }

        /* set layout and view */
        $this->layout($layout);
        $view = new \Zend\View\Model\ViewModel($variables);
        $view->setTemplate($prefix . '/' . strtolower($name));

        return $view;
    }
	public function loadListAction(){

        try {
            /* get post */
            $post = $this->getRequest()->getPost();
			
			$requestPost = array(
				'guid' => '',
				'code' => '',
				'info' => '',
				'data' => $post
			); 
			
            $filterby = @$this->params()->fromQuery('filterby');
            $filterquery = @$this->params()->fromQuery('filterquery');

            /* bootgrid or API */
            if(isset($post['rowCount']) && isset($post['current'])){
                $params = array(
                    'page' => $post['current'],
                    'records' => $post['rowCount'],
                    'sort' => @$post['sort'],
                );

                $keyword = @$post['searchPhrase'];
                $mode = MemberModel\Storage::LISTBY_PAGE;
                
                $filters = $post['filters'];
                if(!empty($filterquery)){
                    $params['filterby'] = $filterby;
                	$params['filterquery'] = $filterquery;
                	// $mode = MemberModel\Storage::LISTBY_DATE;
                }

                $bootgrid = true;
            }else{
                // $post = $this->getREST()->getParameters();
                $params = $post;
                $keyword = '';
                $mode = $this->getCode() ? $this->getCode() : null;
                $bootgrid = false;
            }

			$storage = MemberModel\Storage::factory($this->getDb(),$this->GetConfig());
            $groups= new Timetable\Group\Members($storage);

            if(isset($post['group_id'])){
                $mode = MemberModel\Storage::LISTBY_GROUPID;
                $keyword = $post['group_id'];
            }

            $result = $groups->load($keyword, $mode, $params);

            /* bootgrid? */
            if ($bootgrid) {
                if ($result->code == $result::CODE_SUCCESS) {

                     // calc total 
                    $total = $result->data['countTotal'];;
                    $pager = $result->data['pager'];

                    $listData['rows'] = $result->data['list'];
                    $listData['total'] = $total;
                    $listData['current'] = intval($params['page']);
                    $listData['rowCount'] = $pager->getLimit(); //$roles->count();

                } else {

                    $page = intval($params['page']);
                    $records = intval($params['records']);

                    $listData['rows'] = array();
                    $listData['total'] = $page * $records;
                    $listData['current'] = $page;
                    $listData['rowCount'] = $records;
                }
				
				$logparam['subject'] = 'ATTENDANCE MEMBER LOADLISTACTION';
				$logparam['identity'] = '';
				$logparam['user'] = '';
				$logparam['description'] = 'Log proses attendance member loadlistaction';
				$logparam['request'] = json_encode($requestPost);
				$logparam['response'] = json_encode($listData);

				$this->writeLogAction($logparam);
				
                return $this->getOutput(json_encode($listData));
                
            }   
            
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            $result = new \Neuron\Generic\Result();
            $result->code = 500;
            $result->info = $ex->getMessage();
			
			$responseResult = array(
				'code' => $result->code,
				'info' => $result->info,
				'data' => $result->data
			); 
			
			$logparam['subject'] = 'ATTENDANCE MEMBER LOADLISTACTION';
			$logparam['identity'] = '';
			$logparam['user'] = '';
			$logparam['description'] = 'Log proses attendance member loadlistaction';
			$logparam['request'] = json_encode($requestPost);
			$logparam['response'] = json_encode($responseResult);

			$this->writeLogAction($logparam);
			
            return $this->getREST()->getResponse($result);
        }
        
		$logparam['subject'] = 'ATTENDANCE MEMBER LOADLISTACTION';
		$logparam['identity'] = '';
		$logparam['user'] = '';
		$logparam['description'] = 'Log proses attendance member loadlistaction';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode($responseResult);

		$this->writeLogAction($logparam);
		
        return $this->getREST()->getResponse($result);        
    }
    public function loadByGroupIdAction_old(){

        try {
            /* get post */
            $post = $this->getRequest()->getPost();

            /* bootgrid or API */
            if(isset($post['rowCount']) && isset($post['current'])){
                $params = array(
                    'page' => $post['current'],
                    'records' => $post['rowCount'],
                    'sort' => @$post['sort'],
                );

                $keyword = @$post['searchPhrase'];
                $mode = MemberModel\Storage::LISTBY_PAGE;
                $bootgrid = true;
            }else{
                $params = $post;
                $keyword = '';
                $mode = $this->getCode() ? $this->getCode() : null;
                $bootgrid = false;
            }

            $storage = MemberModel\Storage::factory($this->getDb(),$this->GetConfig());
            $groups= new Timetable\Groups($storage);

            $result = $groups->load($keyword, $mode, $params);

            /* bootgrid? */
            if ($bootgrid) {
                if ($result->code == $result::CODE_SUCCESS) {

                     // calc total 
                    $total = $result->data['countTotal'];;
                    $pager = $result->data['pager'];

                    $listData['rows'] = $result->data['list'];
                    $listData['total'] = $total;
                    $listData['current'] = intval($params['page']);
                    $listData['rowCount'] = $pager->getLimit(); //$roles->count();

                } else {

                    $page = intval($params['page']);
                    $records = intval($params['records']);

                    $listData['rows'] = array();
                    $listData['total'] = $page * $records;
                    $listData['current'] = $page;
                    $listData['rowCount'] = $records;
                }
                return $this->getOutput(json_encode($listData));
                
            }   
            
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            $result = new \Neuron\Generic\Result();
            $result->code = 500;
            $result->info = $ex->getMessage();
            return $this->getREST()->getResponse($result);
        }
        
        return $this->getREST()->getResponse($result);            
    }
    public function deleteMemberFromGroupAction(){
        $post = $this->getRequest()->getPost();
        $storage = MemberModel\Storage::factory($this->getDb(),$this->getConfig());
        $member = new MemberModel($storage);
        $mode = MemberModel\Storage::DELETEBY_GROUP_REFID;
        $result = $member->delete(null,$mode,$post);
		
		$requestPost = array(
			'guid' => '',
			'code' => '',
			'info' => '',
			'data' => $post
		);
		
		$responseResult = array(
			'code' => $result->code,
			'info' => $result->info,
			'data' => $result->data
		); 
		
		$logparam['subject'] = 'ATTENDANCE MEMBER DELETEMEMBERFROMGROUPACTION';
		$logparam['identity'] = '';
		$logparam['user'] = '';
		$logparam['description'] = 'Log proses attendance member deleteMemberFromGroupAction';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode($responseResult);

		$this->writeLogAction($logparam);
		
        return $this->getREST()->getResponse($result);
    }
    public function loadByGroupIdAction(){
        $params = $this->getPostParams();
        $result = new \Neuron\Generic\Result;
        if(!isset($params['group_id'])){
            $result->code = 401;
            $result->info = 'Please provide group id';
        }else{
            $group_id = $params['group_id'];
            $mode = MemberModel\Storage::LISTBY_GROUPID;
            $members = new \Neuron\Attendance\Timetable\Group\Members(MemberModel\Storage::factory($this->getDb(),$this->getConfig()));
            $dataMember = $members->load($group_id,$mode);
            if($dataMember->code != 0){
                $result->data = array();
            }else{
                $result->data = $dataMember->data['list'];
            }
        }
		
		$requestPost = array(
			'guid' => '',
			'code' => '',
			'info' => '',
			'data' => $params
		);
		
		$responseResult = array(
			'code' => $result->code,
			'info' => $result->info,
			'data' => $result->data
		); 
		
		$logparam['subject'] = 'ATTENDANCE MEMBER LOADBYGROUPIDACTION';
		$logparam['identity'] = '';
		$logparam['user'] = '';
		$logparam['description'] = 'Log proses attendance member loadByGroupIdAction';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode($responseResult);

		$this->writeLogAction($logparam);
		
        return $this->getREST()->getResponse($result);
    }

}
