<?php
namespace Neuron\Attendance\Controller;

use Neuron\Attendance\v2\Timetable\Obj as ObjModel;
use Neuron\Attendance\v2\Timetable\Objs;
use Neuron\Attendance\Timetable\ObjectTimetable;
use Neuron\Attendance\Timetable\Objects;
use Neuron\Attendance\Shift;

class Obj extends Controller{
    protected $ref_code = 'EMPLOYEE';

    protected function getView($name, $variables = array()) {
        /* default layout & view prefix */
        $layout = 'neuron/attendance/layout/form';
        $prefix = 'neuron/attendance/object';

        /* libraries */
        if(! class_exists('Neuron\Application\Framework\Entity')) {
            $this->using('jquery', null)
                ->using('bootstrap', null)
                ->using('fontawesome', null)
                ->using('generic',null)
                ->using('select2',null)
				->using('theme',99);
        }

        /* get layout n view prefix from config */
        $config = $this->getConfig();
        if (isset($config['attendance'])) {
            $config = $config['attendance'];
            $layout = isset($config['formlayout']) ? $config['formlayout'] : $layout;
            $prefix = isset($config['viewprefix']) ? $config['viewprefix'] : $prefix;
        }

        /* set layout and view */
        $this->layout($layout);
        $view = new \Zend\View\Model\ViewModel($variables);
        $view->setTemplate($prefix . '/' . strtolower($name));

        return $view;
    }

    public function listAction(){
        $post       = $this->getRequest()->getQuery();

        $params     = array();
        if(isset($post['draw'])){
            $params     = array(
                'page'      => ($post['start'] + $post['length']) / $post['length'], 
                'records'   => $post['length'],
                'order'     => array(
                    'column'       => $post['columns'][ $post['order'][0]['column'] ]['data'],
                    'dir'           => $post['order'][0]['dir']
                ) 
            );
            $id         = $post['search']['value'];
            $dataTable  = true;
            $mode       = ObjModel\Storage::LOADBY_PAGE;
        }else{
            $id         = null;
            $dataTable  = false;
            $mode       = ObjModel\Storage::LIST_ALL;
        }
        
        if(isset($post['ref_id'])){
            $params['where']['ref_id'] = $post['ref_id'];
        }

        $objs      = new Objs(ObjModel\Storage::factory($this->getDb(),$this->getConfig()));
        $result     = $objs->load($id,$mode,$params);
        if($dataTable){
            if($result->code == 0){
                $res    = array(
                    'draw'  => $post['draw'],
                    'data'  => $result->data['list'],
                    'recordsFiltered' => intval($result->data['countTotal']),
                    'recordsTotal'  => intval($result->data['countTotal'])
                );
            }else{
                $res    = array(
                    'draw'  => $post['draw'],
                    'data'  => array(),
                    'recordsFiltered' => 0,
                    'recordsTotal'  => 0
                );
            }
            return $this->getOutput(json_encode($res));
        }

        return $this->getREST()->getResponse($result);
    }

    public function saveAction(){
        $post = $this->getPostParams();
        $session = $this->getSession();
        $sessionOwner = intval($session->owner());

    	$result = new \Neuron\Generic\Result();
    	$storage = \Neuron\Attendance\Shift\Storage::factory($this->getDb(), $this->getConfig());
        $ctt = new \Neuron\Attendance\Shift($storage);

        $timetableIds = $post['timetable_ids'];
        $params = array();
        $listError = array();

        if(empty($timetableIds)){
          	$result->code = 2;
        	$result->info = 'No timetable choosen. Please choose atleast one day.';
            $result->info = $this->translate($result->info);
            return $this->getREST()->getResponse($result);
        }

        $storage->beginTransaction();

        foreach ($timetableIds as $timetableId) {

            //load detail data shift baru
            $storageShift           = Shift\Storage::factory($this->getDb(),$this->getConfig());
            $shift                  = new Shift($storageShift);
            $newShift     = $shift->load($timetableId, Shift\Storage::LOADBY_TIMETABLE_ID);

            //load shift object yang masih aktif dan day_no nya sama seperti dengan shift baru
            $ObjectTimetable = new ObjectTimetable(Objects\Storage::factory($this->getDb(),$this->getConfig()));
            $filter['day_no'] = $newShift->data['day_no'];
            $filter['ref_id'] = $post['ref_id'];
            $filter['start_date'] = date('Y-m-d', strtotime($post['start_date']));
            $oldShift = $ObjectTimetable->loadEx(null,Objects\Storage::LOADBY_REFID_DAY_NO,$filter);
           
            $obj            = new ObjModel(ObjModel\Storage::factory($this->getDb(),$this->getConfig()));

            /* Parsing end date */
            $endDate        = null;
        	if($post['end_date']){
	        	$endDate    = date('Y-m-d',strtotime($post['end_date']));
        	}

			// $check = $ctt->isDuplicate(
			// 	$post['ref_id'], 
            //     $timetableId,
			// 	date('Y-m-d', strtotime($post['start_date'])), 
			// 	$endDate
			// );

            $params     = [
                "timetable_id"          => $timetableId,
                "ref_code"              => $post['ref_code'],
                "ref_id"                => $post['ref_id'],
                "start_date"            => date('Y-m-d',strtotime($post['start_date'])),
                "end_date"              => $endDate,
                "last_update"           => date('Y-m-d H:i:s'),
                "update_by"             => $sessionOwner,
            ];

			if ($oldShift->code != 1) {
				$listError[] = $params;
			}else{
                $obj->push($params);
                if(!$obj->save()){

                    $storage->rollback();

                    $result->info = "Failed saving object";
                    $result->code = $result->code;
                    $result->data = $params;
                    return $this->getREST()->getResponse($result);
                    break;
                }
			}		
		}

        $storage->commit();
		
    	$result->data = ['listError' => $listError];
        return $this->getREST()->getResponse($result);
    }
}
