<?php
namespace Neuron\Attendance\Controller;

use Neuron\Attendance\Timetable\Objects as ObjectModel;
use Neuron\Attendance\Timetable;
use Neuron\Person\Entity as PersonEntity;

//php PDF
use Mpdf\Mpdf;

class ObjectTimetable extends Controller{
    protected $ref_code = 'EMPLOYEE';

    protected function getView($name, $variables = array()) {
        /* default layout & view prefix */
        $layout = 'neuron/attendance/layout/form';
        $prefix = 'neuron/attendance/object';

        /* libraries */
        if(! class_exists('Neuron\Application\Framework\Entity')) {
            $this->using('jquery', null)
                ->using('bootstrap', null)
                ->using('fontawesome', null)
                ->using('generic',null)
                ->using('select2',null)
				->using('theme',99);
        }

        /* get layout n view prefix from config */
        $config = $this->getConfig();
        if (isset($config['attendance'])) {
            $config = $config['attendance'];
            $layout = isset($config['formlayout']) ? $config['formlayout'] : $layout;
            $prefix = isset($config['viewprefix']) ? $config['viewprefix'] : $prefix;
        }

        /* set layout and view */
        $this->layout($layout);
        $view = new \Zend\View\Model\ViewModel($variables);
        $view->setTemplate($prefix . '/' . strtolower($name));

        return $view;
    }

	public function loadListAction(){
        try {
            /* get post */
            $post = $this->getRequest()->getPost();
            
            /* bootgrid or API */
            if(isset($post['start']) && isset($post['length'])){
                $params = $post;

                $keyword = @$post['searchPhrase'];
                $mode = ObjectModel\Storage::LISTBY_PAGE;
                $datatable = true;
            }
            else{
                // $post = $this->getREST()->getParameters();
                $params = $post;
                $keyword = '';
                $mode = $this->getCode() ? $this->getCode() : null;
                $datatable = false;
            }
            
			$storage = ObjectModel\Storage::factory(
                $this->getDb(),
                $this->GetConfig()
            );
            $objects= new Timetable\Objects($storage);

            if(isset($post['mode']) && $post['mode'] == 'LISTBY_PARENT'){
                $mode = ObjectModel\Storage::LISTBY_PARENTID;
                $keyword = $post['id'];
            }

            /* Result Actual */
            $groupBy= ObjectModel\Storage::GROUPBY_NONE;
            $result = $objects->load($keyword, $mode, $params,$groupBy);
            
            /* Cari conflict */
            if(!isset($params['ref_id']) && !empty($result->data['data'])){
                $resData        = $result->data;
                $allObject      = $resData['data'];
                $newData        = array();

                $soMemoization  = array();
                
                foreach($allObject as $object){
                    $flagConflict   = false;
                    $loadParams     = array(
                        'ref_id'    => $object['ref_id'],
                        'draw'      => null,
                    );
                    $res            = $objects->load($keyword,$mode,$loadParams,$groupBy);
                    $shiftObject    = $res->data['data'];

                    $actualData     = $objects->load($keyword,$mode,array(
                        'ref_id'    => $object['ref_id'],
                        'id'        => $params['id'],
                        'draw'      => null
                    ));
                    $actualData     = $actualData->data['data'];
                    $conflictData   = array();
                    foreach($actualData as $so){
                        $flagConflict = false;
                        foreach($shiftObject as $so2){
                            if($so['timetable_object_id'] === $so2['timetable_object_id']){
                                continue;
                            }
                            /* Check yang dayno nya sama saja. */
                            if($so['day_no'] == $so2['day_no']){
                                /* Check start datenya & end_datenya conflict ga */
                                $soStartDate    = strtotime($so['start_date']);
                                $so2StartDate   = strtotime($so2['start_date']);
                                $isDateConflict = ($soStartDate < $so2StartDate) ? 
                                                $this->isDateConflict($so,$so2,'end_date','start_date') : 
                                                $this->isDateConflict($so2,$so,'end_date','start_date');

                                /* Kalo datenya conflict. sekarang cari timenya conflict atau tidak */
                                if($isDateConflict){
                                    $soOnDutyTime   = strtotime($so['on_duty_time']);
                                    $so2OnDutyTime  = strtotime($so2['on_duty_time']);
                                    $isTimeConflict = ($soOnDutyTime < $so2OnDutyTime) ? 
                                                    $this->isDateConflict($so,$so2,'off_duty_time','on_duty_time') : 
                                                    $this->isDateConflict($so2,$so,'off_duty_time','on_duty_time');
                                    if($isTimeConflict){
                                        $flagConflict   = true;

                                        /* load nama timetablenya untuk so1 dan so2 */
                                        $timetable      = new Timetable(Timetable\Storage::factory(
                                            $this->getDb(),$this->GetConfig()
                                        ));

                                        if(!isset($soMemoization[$so['timetable_id']])){
                                            $timetable->load($so['timetable_id']);
                                            $soMemoization[$so['timetable_id']] = $timetable->pull();    
                                            $timetable->clear();
                                        }
                                        if(!isset($soMemoization[$so2['timetable_id']])){
                                            $timetable->load($so2['timetable_id']);
                                            $soMemoization[$so2['timetable_id']] = $timetable->pull();    
                                            $timetable->clear();
                                        }
                                        $so['timetable_name']   = $soMemoization[$so['timetable_id']]['name'];
                                        $so2['timetable_name']   = $soMemoization[$so2['timetable_id']]['name'];

                                        $conflictData[]  = array(
                                            $so,
                                            $so2
                                        );
                                    }
                                }
                            }
                        }

                        if(count($conflictData)){
                            $flagConflict = true;

                            // Update ke employeenya
                            $personDetail   = new \Neuron\Person\Detail(
                                \Neuron\Person\Detail\Storage::factory(
                                    $this->getDb(),
                                    $this->getConfig()
                                )
                            );
                            $personDetail->load(
                                $object['ref_id'],
                                \Neuron\Person\Detail\Storage::LOADBY_PERSON
                            );
                            $dataDetail     = $personDetail->pull();
                            if($dataDetail['person_id'] == $object['ref_id']){
                                $personDetail->flag_timetable_overlap = 1;
                                if($personDetail->save(true)){
                                    $personDetail->clear();
                                }
                            }
                        } 

                        $so['isConflict']       = $flagConflict;
                        $so['conflictData']     = $conflictData;
                    }
                    $object['hasConflict']      = $flagConflict;
                    $object['conflictData']     = $conflictData;
                    $newData[]                  = $object;
                }
                $resData['data']        = $newData;
                $result->data           = $resData;
            }

            /* datatable? */
            if ($datatable) {
                if($result->code == $result::CODE_SUCCESS) {
                    $outputData = $result->data;
                    if(isset($params['ref_id'])){
                        $outputData['recordsTotal'] = count($outputData['data']);
                        $outputData['recordsFiltered'] = count($outputData['data']);
                    }
                    return $this->getOutput(json_encode($outputData));
                }else{
                    if(isset($post['draw'])){
                        $result         = array(
                            'draw' => $post['draw'],
                            'recordsTotal' => 0,
                            'recordsFiltered' => 0,
                            'data'  => array()
                        );
                        return $this->getOutput(json_encode($result));
                    }
                }
            }else{
                return $this->getREST()->getResponse($result);
            }
            
        } catch (\Exception $ex) {
            error_log(
                'Error at ' . $ex->getFile() . 
                ' line ' . $ex->getLine() . ': ' . $ex->getMessage()
            );
            $result = new \Neuron\Generic\Result();
            $result->code = 500;
            $result->info = $ex->getMessage();
            return $this->getREST()->getResponse($result);
        }
        
        return $this->getREST()->getResponse($result);        
    }

    public function terminateAction(){
        $post       = $this->getREST()->getParameters();
        $object     = new \Neuron\Attendance\Timetable\ObjectTimetable(
            \Neuron\Attendance\Timetable\Objects\Storage::factory(
                $this->getDb(),
                $this->getConfig()
            )
        );
        $result     = $object->massTerminate(
            $post['ref_id'],
            $post['parent_timetable_id']
        );
        $result->info = $this->translate($result->info);
        return $this->getREST()->getResponse($result);
    }

    /* digunakan untuk terminate yang conflict2 */
    public function terminateRowAction(){
        $result     = new \Neuron\Generic\Result;
        $post       = $this->getREST()->getParameters();
        $ttStorage  = ObjectModel\Storage::factory(
            $this->getDb(),
            $this->getConfig()
        );
        
        $timetableObject    = new Timetable\ObjectTimetable($ttStorage);

        $timetableObject->load($post['timetable_object_id']);
        $newEndDate         = date('Y-m-d',strtotime(date('Y-m-d').' -1 days '));
        $timetableObject->end_date      = $newEndDate;
        $timetableObject->last_update   = $ttStorage->now();
        $timetableObject->update_by     = $this->getSession()->owner();

        if($timetableObject->save(true)){
            $result->data   = $timetableObject->pull();
            $result->info   = "Success terminating shift";
        }else{
            $result->data   = $timetableObject->pull();
            $result->code   = 1;
            $result->info   = "Failed terminating shift";
        }

        $result->info   = $this->translate($result->info);
        return $this->getREST()->getResponse($result);
    }

    private function isDateConflict($data1,$data2,$endKey,$startKey){
        $hasConflict    = false;
        $data1End   = strtotime($data1[$endKey]);
        $data2Start = strtotime($data2[$startKey]);
        if($data1End > $data2Start){
            $hasConflict    = true;
        }
        return $hasConflict;
    }

    public function findConflict_(){
        /* Ambil semua data timetablenya untuk dibandingin */
        $parent_timetable_id        = null;
        if(isset($params['id'])){
            $parent_timetable_id    = $params['id'];
            unset($params['id']);
        }
        $resultAll = $objects->load($keyword, $mode, $params,$groupBy);

        /* Buat flag has conflict*/
        $actualData     = $result->data['data'];
        $allData        = $resultAll->data['data'];
        $outputData     = array();
        foreach($actualData as $ad){
            $hasConflict  = false;
            /**
             * Kalo actual end_Datenya null, karena hanya bakal berenti 
             * kalo diberhentiin, maka di set tahunnya 9999 
            */
            $endDateIsEmpty     = false;
            if(empty($ad['end_date'])){
                $endDateIsEmpty     = true;
                $ad['end_date'] = '9999-12-31';
            }

            $adStartDate        = strtotime($ad['start_date']);
            $adOnDuty           = strtotime($ad['on_duty_time']);
            $adOffDuty          = strtotime($ad['off_duty_time']);

            foreach($allData as $all){
                if($all['day_no'] != $ad['day_no']){
                    continue;
                }
                /* cari tau apakah tanggalnya conflict? */
                $allStartDate   = strtotime($all['start_date']);
                $allOnDuty      = strtotime($all['on_duty_time']);
                $allOffDuty     = strtotime($all['off_duty_time']);

                if($adStartDate >= $allStartDate){
                    if($all['day_no'] == $ad['day_no']){

                    }
                }
            }




            if($endDateIsEmpty){
                $ad['end_date']     = null;
            }
            $ad['has_conflict']   = (int)$hasConflict;
            $outputData[]           = $ad;
        }
    }

    public function getOutput($json) {

        $response = $this->getResponse();
        $response->getHeaders()
            ->addHeaderLine('Content-Type', 'application/json');
        $response->setContent($json);
        return $response;
    }   
    public function uploadObjectAction(){
        $result = new \Neuron\Generic\Result();
        $files = $this->getRequest()->getFiles();

        if(count($files) <= 0 ){
            $result->code = 401;
            $result->info = 'No file selected';
            return $this->getREST()->getResponse($result);
            exit();
        }

        $config = $this->getConfig();
        
        if(!isset($config['start_date'])){
            $result = new \Neuron\Generic\Result();
            $result->info = 'Start date not found in config. Please Contact your Administrator';
            $result->code = 2;
            return $this->getREST()->getResponse($result);
        }

        $uploadPath = getcwd() . '/public/uploads/excel';
        $uploadPath = $uploadPath . '\\' . date('Y/m/d');

        $uploadfile = true;
        if(!file_exists($uploadPath)){
            mkdir($uploadPath, 0755, true);
        }

        if(count($files['file']) > 0){
            foreach($files as $file){
                $fileType = $file['type']; 
                $filename = $uploadPath.'/'.$file['name'];
                // var_dump($filename);die();
                if($fileType != 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'){
                    $uploadfile = false;
                    $result->info = 'The selected file type is not correct. Please choose the correct file.';
                    $result->code = 2;
                    return $this->getREST()->getResponse($result);
                    break;
                    exit();
                }

                if(file_exists($filename)) $filename = substr($filename, 0, strrpos($filename, '.')) . ' Copy ' . date('His') . substr($filename, strrpos($filename, '.'));
            
                if(!move_uploaded_file($file['tmp_name'], $filename)){
                    $uploadfile = false;
                    break;
                }
            }
        }



        if($uploadfile){
            //convert excel to array 
            $xlsx = \SimpleXLSX\SimpleXLSX::parse($filename);
            $rows = $xlsx->rows();


            $modelObject = new \Neuron\Attendance\Timetable\Object(
                \Neuron\Attendance\Timetable\Object\Storage::factory(
                    $this->getDb(),
                    $this->getConfig()
                )
            );

            $storageLoader = array(
                'config' => $this->getConfig(),
                'adapter' => $this->getDb());

            $result = $modelObject->saveRows($rows,$storageLoader);

            if($result->code != 0){
                unlink($filename);
            }
        } else {
            unlink($filename);
            $result = new Result(
                0,
                1,
                'Upload attachment failed, please try again.'
            );
        }
        return $this->getREST()->getResponse($result);
    }
    public function uploadObjectByRangeAction(){
        $result = new \Neuron\Generic\Result();
        $files = $this->getRequest()->getFiles();

        if(count($files) <= 0 ){
            $result->code = 401;
            $result->info = 'No file selected';
            return $this->getREST()->getResponse($result);
            exit();
        }

        $config = $this->getConfig();
        
        if(!isset($config['start_date'])){
            $result = new \Neuron\Generic\Result();
            $result->info = 'Start date not found in config. Please Contact your Administrator';
            $result->code = 2;
            return $this->getREST()->getResponse($result);
        }

        $uploadPath = getcwd() . '/public/uploads/excel';
        $uploadPath = $uploadPath . '\\' . date('Y/m/d');

        $uploadfile = true;
        if(!file_exists($uploadPath)){
            mkdir($uploadPath, 0755, true);
        }

        if(count($files['file']) > 0){
            foreach($files as $file){
                $fileType = $file['type']; 
                $filename = $uploadPath.'/'.$file['name'];
                // var_dump($filename);die();
                if($fileType != 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'){
                    $uploadfile = false;
                    $result->info = 'The selected file type is not correct. Please choose the correct file.';
                    $result->code = 2;
                    return $this->getREST()->getResponse($result);
                    break;
                    exit();
                }

                if(file_exists($filename)) $filename = substr($filename, 0, strrpos($filename, '.')) . ' Copy ' . date('His') . substr($filename, strrpos($filename, '.'));
            
                if(!move_uploaded_file($file['tmp_name'], $filename)){
                    $uploadfile = false;
                    break;
                }
            }
        }

        if($uploadfile){
            //convert excel to array 
            $xlsx = \SimpleXLSX\SimpleXLSX::parse($filename);
            $rows = $xlsx->rows();

            $modelObject = new \Neuron\Attendance\Timetable\ObjectTimetable(\Neuron\Attendance\Timetable\Object\Storage::factory(
                $this->getDb(),
                $this->getConfig()
            ));

            $storageLoader = array(
                'config' => $this->getConfig(),
                'adapter' => $this->getDb());
            $result = $modelObject->saveRowsByRange($rows,$storageLoader);

            if($result->code != 0){
                unlink($filename);
            }
        } else {
            unlink($filename);
            $result = new Result(0, 1, 'Upload attachment failed, please try again.');
        }
        return $this->getREST()->getResponse($result);
    }
    public function saveDuplicateAction(){
        $post = $this->getRequest()->getPost();
        $modelObject = new \Neuron\Attendance\Timetable\ObjectTimetable(\Neuron\Attendance\Timetable\Object\Storage::factory(
            $this->getDb(),
            $this->getConfig()
        ));
        if($post['action'] == 'replace'){
            $result = $modelObject->saveReplace($post);
        }else if($post['action'] == 'delete'){
            $result = $modelObject->saveDelete($post);
        }
        return $this->getREST()->getResponse($result); 
    }
    public function testAction(){
        $this->using('p-loading',20);
        $this->using('jquery.bootgrid',21);
        $this->using('bootbox',22);
        return $this->getView('test',get_defined_vars());
    }
    public function listByParentAction(){
        $post = $this->getRequest()->getPost();

        // Ambil untuk where condition
        $storage = \Neuron\Attendance\Timetable\Storage::factory($this->getDb(),$this->getConfig());
        $timetables = new \Neuron\Attendance\Timetables($storage);
        $timetableList = $timetables->load($parent,$mode);
    }
    public function loadShiftToChangeAction(){
        $storage = \Neuron\Attendance\Shift\Storage::factory($this->getDb(), $this->getConfig());
        $ctt = new \Neuron\Attendance\Shift($storage);

        $post = $this->getPostParams();
        $dayno = date('w', strtotime($post['date']))+1;
        $sessmgt = $this->getSession()->get();
        $owntimetable = array();
        $arrShift = array();

        if($post['swap'] == 'true'){
            $dataShift = $ctt->loadShiftObject(null, date('Y-m-d', strtotime($post['date'])));
        }else{
            $dataShift = $ctt->loadShiftByDate(date('Y-m-d', strtotime($post['date'])), $dayno);
        }
        $own = $ctt->loadShiftObject($post['ref_id'], date('Y-m-d', strtotime($post['date'])));
        $defShift = array();
        if($own){
            foreach ($own as $val) {
                if($val['day_no']== $dayno){
                    array_push($defShift, $val);
                }
                array_push($owntimetable, $val['timetable_id']);
            }
        }
        // print_r($owntimetable); die();
        if($dataShift){
            foreach ($dataShift as $val) {
                // print_r($val);
                if($val['day_no'] == $dayno){
                    if($post['swap'] == 'true'){
                        if($val['ref_id'] != $post['ref_id'] && !in_array($val['timetable_id'], $owntimetable)){
                            array_push($arrShift, $val);
                        }
                    }else{
                        if(!in_array($val['timetable_id'], $owntimetable)){
                            array_push($arrShift, $val);
                        }
                        
                    }
                }
            }
        }
        echo json_encode(array('success' => true, 'listtarget'=> $arrShift, 'currshift'=> $defShift));
        return $this->getResponse();
    }
    public function saveChangeShiftAction(){
        $storage = \Neuron\Attendance\Shift\Storage::factory($this->getDb(), $this->getConfig());
        $cts = new \Neuron\Attendance\Shift($storage);

        $post = $this->getPostParams();
        $dayno = date('w', strtotime($post['date']));
        $sessmgt = $this->getSession()->get();
        $upChShift = false;
        if($post['tt_employee_shift_id']){
            $upChShift = true;
        }
        $chshift['tt_employee_shift_id'] = $post['tt_employee_shift_id'];
        $chshift['ref_code'] = $post['ref_code_modal'];
        $chshift['ref_id'] = $post['ref_id_modal'];
        $chshift['shift_change_date'] = date('Y-m-d', strtotime($post['shift_change_date']));
        $chshift['default_shift_id'] = $post['default_shift_id'];
        $chshift['change_shift_id'] =  $post['change_shift_id'];
        $chshift['request_id'] = null;
        $chshift['change_by'] = $sessmgt['ref_id'];
        $saveChShift = $cts->saveChangeShift($chshift, $upChShift);
        if($saveChShift){
            $chShift = $cts->loadChangeShift('tt_employee_shift_id', $saveChShift);
            if($chShift){
                foreach ($chShift as $value) {
                    $valShift =  $cts->load($value['change_shift_id']);
                    $oldShift = $cts->loadObjectTimetable($value['ref_id'], $value['shift_change_date'], 3);
                    // print_r($oldShift);
                    if($oldShift['success']){
                        if($oldShift['data']['ato_end_date'] == NULL || $oldShift['data']['ato_end_date'] > $value['shift_change_date']){
                            //update old shift date(format)
                            $updData['start_date'] = date('Y-m-d', strtotime($oldShift['data']['ato_start_date']));
                            $updData['end_date'] = date('Y-m-d', strtotime('-1 day', strtotime($value['shift_change_date'])));
                            $updData['timetable_object_id'] = $oldShift['data']['timetable_object_id'];
                            $shift1 = $cts->updateShiftFromChange($updData);
                            if(!$shift1){
                                echo json_encode(array('success' => false, 'message'=>'Error on update shift'));
                                return $this->getResponse();
                            }

                            //add new shift object
                            $new1['timetable_shift_id'] = $value['change_shift_id'];
                            $new1['timetable_id'] = $valShift['data']['timetable_id'];
                            $new1['ref_code'] = $value['ref_code'];
                            $new1['ref_id'] = $value['ref_id'];
                            $new1['ref_name'] = $oldShift['data']['ref_name'];
                            $new1['start_date'] = date('Y-m-d', strtotime($value['shift_change_date']));
                            $new1['end_date'] = date('Y-m-d', strtotime($value['shift_change_date']));
                            $new1['update_by'] = $value['change_by'];
                            // print_r($new1); die();
                            $savenew1 = $cts->saveObject($new1, false, false);

                            if(!$savenew1){
                                echo json_encode(array('success' => false, 'message'=>'Error on insert new shift'));
                                return $this->getResponse();
                            }

                            //add new2 shift object
                            $new2['timetable_shift_id'] = $oldShift['data']['timetable_shift_id'];
                            $new2['timetable_id'] = $oldShift['data']['timetable_id'];
                            $new2['ref_code'] = $oldShift['data']['ref_code'];
                            $new2['ref_id'] = $oldShift['data']['ref_id'];
                            $new2['ref_name'] = $oldShift['data']['ref_name'];
                            $new2['start_date'] = date('Y-m-d', strtotime('+1 day', strtotime($value['shift_change_date'])));
                            $new2['end_date'] = empty($oldShift['data']['ato_end_date'])? NULL :date('Y-m-d', strtotime($oldShift['data']['ato_end_date']));
                            $new2['update_by'] = $value['change_by'];
                            // print_r($new2);die();
                            $savenew2 = $cts->saveObject($new2, false, false);

                            if(!$savenew2){
                                echo json_encode(array('success' => false, 'message'=>'Error on insert new shift'));
                                return $this->getResponse();
                            }
                        }else if($oldShift['data']['ato_end_date'] = $value['shift_change_date']){
                            $updData['start_date'] = date('Y-m-d', strtotime($oldShift['data']['ato_start_date']));
                            $updData['end_date'] = date('Y-m-d', strtotime('-1 day', strtotime($value['shift_change_date'])));
                            $updData['timetable_object_id'] = $oldShift['data']['timetable_object_id'];
                            $Shift1 = $cts->updateShiftFromChange($updData);
                            if(!$Shift1){
                                echo json_encode(array('success' => false, 'message'=>'Error on insert new shift'));
                                return $this->getResponse();
                            }
                        }else{
                            $new1['timetable_shift_id'] = $value['change_shift_id'];
                            $new1['timetable_id'] = $valShift['data']['timetable_id'];
                            $new1['ref_code'] = $value['ref_code'];
                            $new1['ref_id'] = $value['ref_id'];
                            $new1['ref_name'] = $oldShift['data']['ref_name'];
                            $new1['start_date'] = date('Y-m-d', strtotime($value['shift_change_date']));
                            $new1['end_date'] = date('ref_id-m-d', strtotime($value['shift_change_date']));
                            $new1['update_by'] = $value['ref_id'];
                            $savenew1 = $cts->saveObject($new1, false, false);

                            if(!$savenew1){
                                echo json_encode(array('success' => false, 'message'=>'Error on insert new shift'));
                                return $this->getResponse();
                            }
                        }
                    }
                }
                echo json_encode(array('success' => true, 'message'=>'Success on insert new shift'));
                return $this->getResponse();
            }else{
                echo json_encode(array('success' => false, 'message'=>'Error on load data change shift'));
                return $this->getResponse();
            }
        }else{
            echo json_encode(array('success' => false, 'message'=>'Error on save change shift'));
            return $this->getResponse();
        }
    }
    public function filterListAction(){

        try {
            /* get post */
            $post = $this->getRequest()->getPost();


            $filterby = $this->params()->fromQuery('filterby');
            $filterquery = $this->params()->fromQuery('filterquery');

            /* bootgrid or API */
            if(isset($post['rowCount']) && isset($post['current'])){
                $params = array(
                    'page' => $post['current'],
                    'records' => $post['rowCount'],
                    'sort' => @$post['sort'],
                );

                // $keyword = @$post['searchPhrase'];
                $mode = ObjectModel\Storage::LISTBY_PAGE;
                
                $filters = $post['filters'];
                if(!empty($filterquery)){
                    $params['filterby'] = $filterby;
                    $params['filterquery'] = $filterquery;
                    // $mode = ObjectModel\Storage::LISTBY_DATE;
                }

                $bootgrid = true;
            }else{
                $post = $this->getREST()->getParameters();
                $params = $post;
                $keyword = '';
                $mode = $this->getCode() ? $this->getCode() : null;
                $bootgrid = false;
            }

            $storage = ObjectModel\Storage::factory($this->getDb(),$this->GetConfig());
            $objects= new Timetable\Objects($storage);




            $result = $objects->filterList($params);

            /* bootgrid? */
            if ($bootgrid) {
                if ($result->code == $result::CODE_SUCCESS) {

                     // calc total 
                    $total = $result->data['countTotal'];;
                    $pager = $result->data['pager'];

                    $listData['rows'] = $result->data['list'];
                    $listData['total'] = $total;
                    $listData['current'] = intval($params['page']);
                    $listData['rowCount'] = $pager->getLimit(); //$roles->count();

                } else {

                    $page = intval($params['page']);
                    $records = intval($params['records']);

                    $listData['rows'] = array();
                    $listData['total'] = $page * $records;
                    $listData['current'] = $page;
                    $listData['rowCount'] = $records;
                }
                return $this->getOutput(json_encode($listData));
                
            }   
            
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            $result = new \Neuron\Generic\Result();
            $result->code = 500;
            $result->info = $ex->getMessage();
            return $this->getREST()->getResponse($result);
        }
        
        return $this->getREST()->getResponse($result);        
    }

    public function loadListSetAdminAction(){
        $post       = $this->getRequest()->getPost();
        $objects    = new \Neuron\Attendance\Timetable\Objects(\Neuron\Attendance\Timetable\Objects\Storage::factory($this->getDb(),$this->getConfig()));
        $result     = $objects->listSetAdmin($post);
        return $this->getOutput(json_encode($result));
    }
    
    public function listchangeshiftAction(){
        try {
            $post = $this->getRequest()->getPost();
            $params = $post;

            if(isset($params['filterby'])){
                $filterby   = $params['filterby'];
                $filterquery = $params['filterquery'];
            }else{
                $filterby = @$this->params()->fromQuery('filterby');
                $filterquery = @$this->params()->fromQuery('filterquery');
            }

        
            if(!empty($filterquery)){
                $params['filterby'] = $filterby;
                $params['filterquery'] = $filterquery;

                if($params['filterby'] == 'start_date'){
                    $date   = $params['filterquery'];
                    $date   = date('Y-m-d',strtotime($params['filterquery']));
                    $params['filterquery'] = $date;
                }
                
            }else{
                $data = array(
                    'draw' => 0,
                    'recordsTotal' =>0,
                    'recordsFiltered' => 0,
                    'data' => array(),
                );
                return $this->getOutput(json_encode($data));
            }


            $storage = ObjectModel\Storage::factory($this->getDb(),$this->GetConfig());
            $objects= new Timetable\Objects($storage);

            $groupBy= ObjectModel\Storage::GROUPBY_NONE;
            $result = $objects->listChangeShift($params);
            
            if($result->code == $result::CODE_SUCCESS) {
                $outputData = $result->data;
                return $this->getOutput(json_encode($outputData));
            }else{
                return $this->getOutput(json_encode($result->data));
            }
        } catch (\Exception $e) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            $result = new \Neuron\Generic\Result();
            $result->code = 500;
            $result->info = $ex->getMessage();
            return $this->getREST()->getResponse($result);
        }
    }
    public function loadDatatableAction(){
        $post       = $this->getRequest()->getPost();
        $objects    = new ObjectModel(ObjectModel\Storage::factory($this->getDb(),$this->getConfig()));
        $result     = $objects->loadDatatable($post);

        echo json_encode($result);
        return $this->getResponse();
    }

    public function listDataShiftAction(){
        $post       = $this->getRequest()->getPost();
        $objects    = new \Neuron\Attendance\Timetable\Objects(\Neuron\Attendance\Timetable\Objects\Storage::factory($this->getDb(),$this->getConfig()));
        $result     = $objects->listDataShift($post);
        return $this->getOutput(json_encode($result));
    }

    public function DownloadDataShiftAction(){
        $post       = $this->getPostParams();

        $ref_id     = $post['id'];
        $objects    = new \Neuron\Attendance\Timetable\Objects(\Neuron\Attendance\Timetable\Objects\Storage::factory($this->getDb(),$this->getConfig()));
        $result     = $objects->listDataShift($ref_id);
        $sessmgt = $this->getSession()->get();

        $person 	= new PersonEntity(PersonEntity\Storage::factory($this->getDb(),$this->getConfig()));
        $person->load($ref_id);
        $dataPerson = $person->pull();

        $mpdf 	 = new Mpdf();
		$mpdf    = new \Mpdf\Mpdf();
        $content		.= "<table border=\"0\" cellspacing=\"0\" cellpadding=\"5\" width=\"100%\">";
        $content		.= 		"<tr>";
        $content		.= 		"<td width=\"30\" style=\"text-align: center; vertical-align: middle;\"><h3>" . $this->translate('Shift schedule')." ".$dataPerson['first_name']. " " . $dataPerson['last_name']." </h3></td>";
        $content		.= 		"</tr>";
        $content		.= "</table>";

        $content		.= 		"<br>";
        $content		.= 		"<br>";

        $content		.= "<table border=\"1\" cellspacing=\"0\" cellpadding=\"5\" width=\"100%\">";
        $content		.= 		"<tr>";
        $content		.= 		    "<th >" . $this->translate('Day Name') . "</th>";
        $content		.= 		    "<th >" . $this->translate('Shift Date') . "</th>";
        $content		.= 		"</tr>";
        $content		.= 		"<tr>";
        
        $datas = $result['data'];
        foreach ($datas as $shift) {
            $start  = ($shift['start_date'] == '') ? 'No information' : date('j F Y',strtotime($shift['start_date']));
            $end    = ($shift['end_date'] == '') ? 'No information' : date('j F Y',strtotime($shift['end_date']));
            
            // iterate day no
            switch ($shift['day_no']) {
                case 1:
                    $date = $this->translate('Sunday');
                  break;
                case 2:
                    $date = $this->translate('Monday');
                  break;
                case 3:
                    $date = $this->translate('Tuesday');
                  break;
                case 4:
                    $date = $this->translate('Wednesday');
                    break;
                case 5:
                    $date = $this->translate('Thursday');
                    break;
                case 6:
                    $date = $this->translate('Friday');
                    break;
                case 7:
                    $date = $this->translate('Saturday');
                    break;
                default :
                    $date = $this->translate('Not Found');
              }

            $content		.= 		"<tr>";
            $content		.= 		"<td>".$date."</td>";
            $content		.= 		"<td>".$start." - " .$end. "</td>";
            $content		.= 		"</tr>";
        }

        $content		.= 		"</tr>";
        $content		.= "</table>";

        $mpdf->WriteHTML($content);
        $mpdf->Output('Shift schadule-'.$dataPerson['first_name']. ' '. $dataPerson['last_name'].'.pdf', 'I');

    }

    public function deleteRowAction(){
        $result     = new \Neuron\Generic\Result;
        $post       = $this->getREST()->getParameters();
        $ttStorage  = ObjectModel\Storage::factory(
            $this->getDb(),
            $this->getConfig()
        );
        
        $timetableObject    = new Timetable\ObjectTimetable($ttStorage);

        $timetableObject->load($post['timetable_object_id']);

        if($timetableObject->id){
            $result = $timetableObject->delete($timetableObject->pull());
        }else{
            $result->data   = $timetableObject->pull();
            $result->code   = 1;
            $result->info   = "Failed terminating shift";
        }

        $result->info   = $this->translate($result->info);
        return $this->getREST()->getResponse($result);
    }
}
