<?php
namespace Neuron\Attendance\Controller;

use Zend\Form\Element\DateTime;
use Neuron\Attendance\Attendance;
use Neuron\Attendance\Entity;
use Neuron\Attendance\Reporting as AttendanceReporting;
use Neuron\Attendance\Request;
use Neuron\Attendance\Request\Setting\Type;
use Neuron\Attendance\Request\Type as RequestType;
use Neuron\Attendance\Request\Types;
use Neuron\Attendance\Requests;
use Neuron\Generic\Result;

/*require './vendor/PHPExcel/PHPExcel.php';
require './vendor/PHPExcel/PHPExcel/IOFactory.php';

use PHPExcel;
use PHPExcel_IOFactory;
use PHPExcel_Cell;
use PHPExcel_Cell_DataType;
use PHPExcel_Writer_Excel2007;
use PHPExcel_Style_Alignment;
use PHPExcel_Style_Fill;
use PHPExcel_Style_Border;
use PHPExcel_Worksheet_Drawing;
use PHPExcel_Settings;*/

class Reporting extends Controller {
	protected function getView($name, $variables = array()) {
		$layout = 'neuron/attendance/layout/form';
		$prefix = 'neuron/attendance/reporting';

		/* libraries */;
	    $this->using('font-awesome', 10, 'https://use.fontawesome.com/releases/v5.8.1/css/all.css');

		if(! class_exists('Neuron\Application\Framework\Entity')) {
	        $this->using(
		            'bootstrap-4',
		            null,
		            array(
		                'https://ajax.googleapis.com/ajax/libs/jquery/3.3.1/jquery.min.js',
		                'https://cdnjs.cloudflare.com/ajax/libs/popper.js/1.14.7/umd/popper.min.js',
		                'https://maxcdn.bootstrapcdn.com/bootstrap/4.3.1/css/bootstrap.min.css',
		                'https://maxcdn.bootstrapcdn.com/bootstrap/4.3.1/js/bootstrap.min.js'
		            )
		        )
		        ->using('toast',null)
		        ->using('bootbox',null);
		}

	    $this->using('toast');
		/* get layout n view prefix from config */
		$config = $this->getConfig();
		if (isset($config['attendance'])) {
			$config = $config['attendance'];
			$layout = isset($config['formlayout']) ? $config['formlayout'] : $layout;
			$prefix = isset($config['viewprefix']) ? $config['viewprefix'] : $prefix;
		}
		$sessmgt = $this->getSession()->get();
		$sess = (isset($sessmgt['ref_id'])) ? true : false;
		$variables['sessmgt'] = $sessmgt;
		$variables['sess'] = $sess;
		
		/* set layout and view */
		$this->layout($layout);
		$view = new \Zend\View\Model\ViewModel($variables);
		$view->setTemplate($prefix . '/' . strtolower($name));

		return $view;
	}
	public function manageAction(){
		$this->using('fontawesome',0);
		$this->using('chart.js',30,array(
			'https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.8.0/Chart.min.js',
			'https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.8.0/Chart.min.css'
		));

		$this->using('p-loading',29);

		$this->using('datatable', 20, array(
		    'https://cdn.datatables.net/1.10.19/js/jquery.dataTables.min.js',
		    'https://cdn.datatables.net/1.10.19/js/dataTables.bootstrap4.min.js',
		    'https://cdn.datatables.net/1.10.19/css/dataTables.bootstrap4.min.css'
		));
		$this->using('air-datepicker',21);

		$this->using('moment',22);

		$this->using('toast',32);

		$layout = $this->layout();

		$session 	= $this->getSession()->get();
		if(isset($session['ref_id'])){
			$ref_id 	= $session['ref_id'];
		}else{
			$ref_id 	= null;
		}

		if(isset($session['user_id'])){
			$user_id 	= $session['user_id'];
		}else{
			$user_id 	= null;
		}

		$layout->setVariables(['current' => $this->translate('Attendance Reporting')]);

		return $this->getView('reporting-chart',get_defined_vars());
	}
	public function loadReportAction(){

		$storage = \Neuron\Attendance\Reporting\Storage::factory($this->getDb(), $this->getConfig());
		$crep = new \Neuron\Attendance\Reporting($storage);

		$post = $this->getPostParams();

		$sessmgt = $this->getSession()->get();

		$sess = false;
		if(isset($sessmgt['ref_id'])) {
			$sess = true;
		}

		$result = new \Neuron\Generic\Result;
		if(!isset($sessmgt['ref_id']) || empty(@$sessmgt['ref_id'])){
			$result->code = 404;
			$result->info = 'Please set ref_id!';
			return $this->getREST()->getResponse($result);
		}
		
		if(@!$sessmgt['roles']['list'][0]['access_role_id']){
			$result->code = 405;
			$result->info = 'Please set access_role!';
			return $this->getREST()->getResponse($result);
		}

		if($access_role = @$sessmgt['roles']['list'][0]['access_role_id']){
			$page = $post['start'] + 1;

			$rowCount = $post['length'];
			$post['session'] = array('user_id'=>$sessmgt['ref_id'], 'access_role'=>$access_role);
			$result = $crep->reportAttendance($post);
			$output = array(
				'draw' => $post['draw'],
				'recordsTotal' => intval($result['total']),
				'recordsFiltered' => intval($result['total']),
				'data' => $result['rows']
			);
		}else{
			$output = array(
				'draw' => $post['draw'],
				'recordsTotal' => 0,
				'recordsFiltered' => 0,
				'data' => array()
			);
		}
		
		echo json_encode($output);
		return $this->getResponse();
	}
	public function downloadReportAction(){
		$post =$this->getPostParams();
		$storage = \Neuron\Attendance\Reporting\Storage::factory($this->getDb(), $this->getConfig());
		$crep = new \Neuron\Attendance\Reporting($storage);
		
		$sessmgt = $this->getSession()->get();
		$post['session'] = array('user_id'=>$sessmgt['ref_id'], 'access_role'=>$sessmgt['access_role']);;
		$post['current'] = 1;
		$post['rowCount'] = -1;
		$result = $crep->reportAttendance($post);
		// print_r($result['rows']); die(); 
		
		if($post['filter']==1){
			$filename = 'Reporting Attendance Weekly';
		}else if($post['filter']== 2){
			$month = date("F", strtotime(($post['month']!= '')?$post['month']:date('m')));
			$year = date("Y", strtotime(($post['year']!= '')?$post['year']:date('Y')));
			$filename = 'Reporting Attendance '.$month.' '.$year;
		}else{
			$year = date("Y", strtotime(($post['year']!= '')?$post['year']:date('Y')));
			$filename = 'Reporting Attendance'.$year;
		}


		$objPHPExcel = new PHPExcel;
		$sheet = $objPHPExcel->setActiveSheetIndex(0);
		if($sessmgt['access_role']== 1)
			$maxcol = $adjustedColumn = PHPExcel_Cell::stringFromColumnIndex(1);
		else{
			$maxcol = $adjustedColumn = PHPExcel_Cell::stringFromColumnIndex(4);
		}
		foreach (range('A', $maxcol) as $columnID) {
			$sheet->getColumnDimension($columnID)->setAutoSize(true);
		}

		$styleArray = array(
			'borders' => array(
				'allborders' => array(
					'style' => PHPExcel_Style_Border::BORDER_THIN
				)
			)
		);

		$sheet->getStyle('A1:'.$maxcol.'1')->getFill()
				->setFillType(PHPExcel_Style_Fill::FILL_SOLID)
				->getStartColor()->setRGB('FFFFFF');

		
		$objDrawing = new PHPExcel_Worksheet_Drawing();

		$sheet->mergeCells('A1:'.$maxcol.'1');
		$sheet->getStyle('A1')->getFont()->setBold(true);
		$sheet->getStyle('A1')->getFont()->setSize(16)->getColor()->setRGB('000000');
		$sheet->getStyle('A1')
				->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
		$sheet->SetCellValue('A1', strtoupper($filename));

		$sheet->getStyle('A2:'.$maxcol.'2')
				->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
		$sheet->getStyle('A2:'.$maxcol.'' . (count($result['rows']) + 2))->applyFromArray($styleArray);
		$sheet->getStyle('A2:'.$maxcol.'2')->getFont()->setBold(true);
		$sheet->getStyle('A2:'.$maxcol.'2')->getFont()->setSize(14);
		$sheet->getStyle('A2:'.$maxcol.'2')->getFont()->getColor()->setRGB('FF0000');
		$sheet->getStyle('A2:'.$maxcol.'2')->getFill()
				->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setRGB('FFFF00');

		$sheet->getStyle('A2:'.$maxcol.'2')->getAlignment()
				->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
				
		if($sessmgt['access_role']== 1){
			$sheet->SetCellValue('A2', 'Ref Id');
			$sheet->SetCellValue('B2', 'Attend');
		}else{
			$sheet->SetCellValue('A2', 'Ref Id');
			$sheet->SetCellValue('B2', 'Start Dtm');
			$sheet->SetCellValue('C2', 'Start Description');
			$sheet->SetCellValue('D2', 'End Dtm');
			$sheet->SetCellValue('E2', 'End Description');
		}


		$rowCount = 3;
		foreach ($result['rows'] as $arSubData) {
			$sheet->getStyle('A' . $rowCount . ':B' . $rowCount)
					->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_LEFT);
			$sheet->getStyle('C' . $rowCount)
					->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_RIGHT);
			$sheet->getStyle('D' . $rowCount . ':E' . $rowCount)
					->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_LEFT);
			$sheet->getStyle('F' . $rowCount . ':G' . $rowCount)
					->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
			if($sessmgt['access_role']== 1){
				$sheet->SetCellValue('A' . $rowCount, $arSubData['ref_id']);
				$sheet->SetCellValue('B' . $rowCount, $arSubData['total_attend']);
			}else{
				$sheet->SetCellValue('A'. $rowCount, $arSubData['ref_id']);
				$sheet->SetCellValue('B'. $rowCount, $arSubData['start_dtm']);
				$sheet->SetCellValue('C'. $rowCount, $arSubData['start_description']);
				$sheet->SetCellValue('D'. $rowCount, $arSubData['end_dtm']);
				$sheet->SetCellValue('E'. $rowCount, $arSubData['end_description']);
			}
			$rowCount++;
		}

		$objWriter = PHPExcel_IOFactory::createWriter($objPHPExcel, 'Excel2007');
		ob_end_clean();
		header('Content-Type: application/vnd.ms-excel');
		header('Content-Description: File Transfer');
		header('Content-Disposition: attachment; filename="' . $filename . '_' . date('YmdHis') . '.xlsx"');
		header('Content-Transfer-Encoding: binary');
		header('Expires: 0');
		header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
		header('Pragma: public');
		$objWriter->save('php://output', 'xlsx');

		return $this->getResponse();
	}
	public function reportingChartAction(){
		$this->using('fontawesome',0);
		$this->using('chart.js',30,array(
			'https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.8.0/Chart.min.js',
			'https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.8.0/Chart.min.css'
		));

		$this->using('p-loading',29);

		$this->using('datatable', 20, array(
		    'https://cdn.datatables.net/1.10.19/js/jquery.dataTables.min.js',
		    'https://cdn.datatables.net/1.10.19/js/dataTables.bootstrap4.min.js',
		    'https://cdn.datatables.net/1.10.19/css/dataTables.bootstrap4.min.css'
		));
		$this->using('air-datepicker',21);

		$this->using('moment',22);

		$this->using('toast',32);

		$layout = $this->layout();

		$session 	= $this->getSession()->get();
		if(isset($session['ref_id'])){
			$ref_id 	= $session['ref_id'];
		}else{
			$ref_id 	= null;
		}

		if(isset($session['user_id'])){
			$user_id 	= $session['user_id'];
		}else{
			$user_id 	= null;
		}

		$layout->setVariables(['current' => $this->translate('Attendance Reporting')]);

		return $this->getView('reporting-chart',get_defined_vars());
	}
	public function getReportingDataAction(){
		$post = $this->getPostParams()->toArray();
		$result = new \Neuron\Generic\Result;
		$data = new \stdClass;
		
		
		if(isset($post['ref_id']) && !empty($post['ref_id'])){
			$ref_id 	= $post['ref_id'];
		}else{
			$fetchRefId = $this->getRefId();
			if($fetchRefId->code == 0){
				$ref_id = $fetchRefId->data;
			}else{
				return $this->getREST()->getResponse($fetchRefId);
			}
		}

		// Load model attendance
		$attendance = new Attendance(\Neuron\Attendance\Attendance\Storage::factory($this->getDb(),$this->getConfig()));

		// parameter yang dibutuhkan untuk meload data attendance
		$loadParams = array(
			'type_id' => Attendance::REPORTING_TYPE_ATTENDANCE,
			'ref_id' => $ref_id,
			'start_dtm' => date('Y-m-d'),
		);

		/*
		* Time mode nya, misalkan weekly,monthly dll
		*/
		switch ($post['time_mode']) {
			case 0:
				$timeMode = Attendance\Storage::TIME_WEEKLY;
				break;
			case 1:
				$timeMode = Attendance\Storage::TIME_MONTHLY;
				break;
			case 2:
				$timeMode = Attendance\Storage::TIME_YEARLY;
				break;
			default: 
				$timeMode = Attendance\Storage::TIME_WEEKLY;
				break;
		}
		// get data attendance
			$dataAttendance = $attendance->loadList(null,Attendance\Storage::LISTBY_REF_ID,$loadParams,$timeMode);
			if($dataAttendance->code == 0){
				$data->attendance = $dataAttendance->data;
			}else{
				$data->attendance = array();
			}
		// TODO : get data absent
			$absent = array();
			$loadParamsAbsent = array(
				'ref_id' => $ref_id,
				'type_id' => Attendance::REPORTING_TYPE_ABSENT,
				'start_dtm' => date('Y-m-d'),
			);
			$dataAbsent = $attendance->loadList(null,Attendance\Storage::LISTBY_REF_ID,$loadParamsAbsent,$timeMode);
			if($dataAbsent->code == 0){
				$absent = $dataAbsent->data;
			}
			$data->absent = $absent;

		$result->data = $data;
		return $this->getREST()->getResponse($result);
	}
	public function getRequestDataAction(){
		$post = $this->getPostParams();
		$result = new \Neuron\Generic\Result;

		if(isset($post['ref_id'])){
			$ref_id 	= $post['ref_id'];
		}else{
			$fetchRefId = $this->getRefId();
			if($fetchRefId->code == 0){
				$ref_id = $fetchRefId->data;
			}else{
				return $this->getREST()->getResponse($fetchRefId);
			}
		}
		
		$request = new Requests(Request\Storage::factory($this->getDb(),$this->getConfig()));
		$time_mode = isset($post['time_mode']) ? $post['time_mode'] : Request\Storage::TIME_WEEKLY;
		$loadParam = array(
			'ref_id' => $ref_id,
			'start_dtm' => date('Y-m-d')
		);

		$dataRequest = $request->load(null,Request\Storage::LISTBY_REFID,$loadParam,$time_mode);
		$result = $dataRequest;

		$requestTypes 	= new Types(RequestType\Storage::factory(
			$this->getDb(),
			$this->getConfig()
		));
		$requestTypes->load();
		$chartData 		= array();
		foreach($requestTypes as $reqType){
			$reqTypeData = $reqType->pull();
			$reqTypeData['list'] = array();
			$chartData[] = $reqTypeData;
		}
		// echo '<pre>'; print_r($chartData); echo '</pre>';die();
		$resultData 	= $result->data['rows'];
		foreach($resultData as $resData){
			$requestTypeId 	= $resData['request_type'];
			foreach($chartData as $key => $cd){
				if($cd['type_id'] == $requestTypeId){
					$chartData[$key]['list'][] = $resData;
				}
			}
		}

		$resultData 				= $result->data;
		$resultData['chartData'] 	= $chartData;
		$result->data 				= $resultData; 
		
		return $this->getREST()->getResponse($result);
	}
	public function widgetDataAction(){
		$result 				= new \Neuron\Generic\Result;
		$post 					= $this->getRequest()->getPost();
		$error 					= array();


		/* Fetch Attendance */
		$modelAttendance 		= new Attendance(Attendance\Storage::factory($this->getDb(),$this->getConfig()));
		$fetchAttendance		= $modelAttendance->countAttendance(null,Attendance\Storage::LIST_ALL,array(),intval($post['time_mode']));
		$totalAttendance 		= $fetchAttendance->data;

		if($fetchAttendance->code != 0){
			$error[]			= 'Error fetching `Attendance` Data.';
		}

		/* Fetch not attend */
		$attendance 			= new Attendance(Attendance\Storage::factory($this->getDb(),$this->getConfig()));
		$notAttend 				= $attendance->countNotAttend(date('Y-m-d'),date('Y-m-d'));

		$widgetData 			= array(
			'data' 				=> [
				[
					'label' 	=> "Attendance",
					'total' 	=> intval($totalAttendance),
					'color' 	=> "#89c4f4"
				],
				[
					'label' 	=> "No information",
					'total' 	=> intval($notAttend->data['total']),
					'color' 	=> "#e26a6a"
				]
			],
			'error' 				=> $error,
			'no_information_data'	=> $notAttend->data['list']
		);


		/* Fetch From Leave */
		if(class_exists('\Neuron\Leave\Model\LeaveMain')){
			$leaveData 		= 	$modelAttendance->fetchLeaveData($post['time_mode']);
			$mergeArray 	= array_merge($widgetData,$leaveData->data);
			$result->data 	= $mergeArray;
		}else{
			$result->data 	= $widgetData;
		}

		return $this->getREST()->getResponse($result);

	}
	public function monthlyAction(){
		$this->using('moment',19);
		$this->using('p-loading',20);
		$this->using('air-datepicker',21);
		$this->using('chart.js',30,array(
			'https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.8.0/Chart.min.js',
			'https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.8.0/Chart.min.css'
		));

		$leaveExists 	= class_exists('Neuron\Leave\Model\Entity');

		/* Ambil List organization */
		$listOrg 			= array();
		if(class_exists('Neuron\Organization\Entities')){
			$organization 		= new \Neuron\Organization\Entities(\Neuron\Organization\Entity\Storage::factory(
				$this->getDb(),$this->getConfig()
			));

			/* Check user bisa liat semua organization atau tidak */
			$hasGrant  			= $this->checkGrants('attd_all_org_list');
			if($hasGrant){
				$result 			= $organization->load();
				if($result->code == 0){
					$data 		= $result->data['list'];
					$listOrg 	= $organization->buildTree($data,'id','parent_id');
				}
			}else{
				// Kalo gapunya grant, list organization di optionnya hanya yang di miliki aja.
				$session 			= $this->getSession()->get();
				$empJobs 			= new \Neuron\Employee\Jobs(
					\Neuron\Employee\Jobs\Storage::factory(
						$this->getDb(),
						$this->getConfig()
					)
				);
				$jobs 			= $empJobs->loadEmpJob($session['ref_id']);
				
				/* Yang baru kk */
				$result = $organization->load();
				if($result->code == 0){
					$organizations 		= $result->data['list'];
					$listOrg 	= $organization->buildTree($organizations,'id','parent_id');
					$ownedJobs 	= array();
					if($jobs){
						foreach($jobs as $job){
							if(
								(strtotime($job['end_date'] > strtotime(date('Y-m-d')))) ||
								empty($job['end_date'])
							){
								$ownedJobs[] 	= $job['organization_id'];
							}
						};
					}
					$listOrg 	= $this->listOrg($listOrg,$ownedJobs);
				}
			}
		}

		return $this->getView('monthly',array(
			"listOrg" 		=> $listOrg,
			'hasOrgGrant' 	=> $hasGrant,
			'leaveExists' 	=> $leaveExists
		));
	}
	public function monthlyAttendanceAction(){
		$post 		= $this->getRequest()->getPost();
		$start_date = date('Y-m-d',strtotime($post['start_date']));
		$end_date 	= date('Y-m-d',strtotime($post['end_date']));
		$ref_name 	= $post['ref_name'];
		$attendance = new Attendance(Attendance\Storage::factory(
			$this->getDb(),$this->getConfig()
		));

		if(isset($post['organization_id']) && !empty($post['organization_id'])){
			$filter['organization_id'] = $post['organization_id'];
		}
		$filter['ref_name'] 		= $ref_name;

		$result 	= $attendance->countReporting(
			Attendance::REPORTING_TYPE_ATTENDANCE,
			$start_date,
			$end_date,
			$filter
		);
		$result1 	= $attendance->countReporting(
			Attendance::REPORTING_TYPE_ABSENT,
			$start_date,
			$end_date,
			$filter
		);
		$totalAttendance 		= intval($result->data['total']);
		$totalAbsent			= intval($result1->data['total']);

		/* Ambil total not attend */
		
		$result 				= new Result();
		$result->data 			= array(
			$this->translate('Attendance') 		=> $totalAttendance,
			$this->translate('Absent')			=> $totalAbsent
		);
		return $this->getREST()->getResponse($result);
	}
	public function reportingTableAction(){
		$post 		= $this->getRequest()->getQuery();
		$reporting 	= new AttendanceReporting(AttendanceReporting\Storage::factory($this->getDb(),$this->getConfig()));

		$result 	= $reporting->reportingTable($post);
		
		return $this->getOutput(json_encode($result));
	}
	public function downloadAction(){
        $result = new Result();
		$post =  $this->getPostParams();
    
        $reporting 	= new AttendanceReporting(AttendanceReporting\Storage::factory($this->getDb(),$this->getConfig()));
		$download 	= $reporting->downloadReporting($post);
        return $this->getREST()->getResponse($download);
	}
	public function downloadDetailAction(){
		$post 			= $this->getRequest()->getQuery()->toArray();
		$reporting 		= new AttendanceReporting(AttendanceReporting\Storage::factory(
			$this->getDb(),
			$this->getConfig()
		));
		$result 		= $reporting->downloadDetails($post);
		$result->info   = $this->translate($result->info);
		return $this->getREST()->getResponse($result);
	}
	private function listOrg($datas = array(),$jobs = array()){
		$listOrg 	 = array();
		foreach($datas as $data){
			if(in_array($data['id'],$jobs)){
				array_push($listOrg,$data);
			}else if(isset($data['children']) && count($data['children'])){
				$findChild 	= $this->listOrg($data['children'],$jobs);
				if(count($findChild)){
					$listOrg = array_merge(
						$listOrg,
						$this->listOrg($data['children'],$jobs)
					);
				}
			}
		}
		return $listOrg;
	}
	public function detailLateAction() {
		
        $sessmgt = $this->getSession()->get();
        $post = $this->getPostParams();
          
		$reporting 	= new AttendanceReporting(AttendanceReporting\Storage::factory($this->getDb(),$this->getConfig()));
		$result 	= $reporting->detailLate($post);
        return $this->getOutput(json_encode($result));  
	}
	public function detailLeaveAction(){
		$post 				= $this->getRequest()->getQuery()->toArray();
		$orderEntities 		= new \Neuron\Order\Entities(
			\Neuron\Order\Entity\Storage::factory(
				$this->getDb(),
				$this->getConfig()
			)
		);

		$userId	= $post['ref_id'];
		
		if ($manager = \Neuron\Application\Framework\Event\Manager::get()) {
			$manager->events()->register('order_query',array($this,'onGetDetailLeave'),0,3);
		}

		$filter = [
			'where' 	=> [
				'create_user_id' 	=> $userId,
				'order_status_id' 	=> 2, # Status closed
			],
			'optional' 	=> [
				'start_date' 		=> $post['start_date'],
				'end_date' 			=> $post['end_date']
			]	
		];
		
		$start 	= $post['start'];
		$length = $post['length'];
		$page 	= $start + $length / $length;
		$filter['order']['column'] = $post['columns'][$post['order'][0]['column']]['data'];
		$filter['order']['dir'] = $post['order'][0]['dir'];
		$filter['page'] 	= $page;
		$filter['records'] = $post['length'];
		$orders 	= $orderEntities->load(null,\Neuron\Order\Entity\Storage::LISTBY_PAGE,$filter);

		if($orders->code == 0){
			$result = array(
				'draw' 	=> $post['draw'],
				'recordsTotal' 	=> $orders->data['countTotal'],
				'recordsFiltered' => $orders->data['countTotal'],
				'data' 	=> $orders->data['list']
			);
		}else{
			$result = array(
				'draw' 	=> $post['draw'],
				'recordsTotal' 	=> 0,
				'recordsFiltered' => 0,
				'data' 	=> []
			);
		}

		return $this->getOutput(json_encode($result));
	}

	public function onGetDetailLeave($query,$params,$caller){
		$params	= $params['optional'];
		$query->where->greaterThanOrEqualTo(new \Zend\Db\Sql\Expression('DATE(o.xd1)'),$params['start_date']);
		$query->where->lessThanorEqualTo(new \Zend\Db\Sql\Expression('DATE(o.xd1)'),$params['end_date']);
		$query->where([
			'o.order_type_id'	 => 2200
		]);
	}

	public function todayStatusDataAction(){
		$post 		= $this->getREST()->getParameters();
		$result 	= new \Neuron\Generic\Result;
		$entity 	= new Entity(Entity\Storage::factory(
			$this->getDb(),
			$this->getConfig()
		));
		$attendance = new Attendance(Attendance\Storage::factory(
			$this->getDb(),
			$this->getConfig()
		));

		$startDate 	= isset($post['start_date']) ?: date('Y-m-d'); 
		$endDate 	= isset($post['start_date']) ?: date('Y-m-d');

		$countAttend 		= $entity->countByType(Entity::TYPE_ATTENDANCE,$startDate,$endDate);
		// $countNoInf 		= $entity->countByType(Entity::TYPE_NOINF,$startDate,$endDate);
		$countNoInf 		= $attendance->countNotAttend($startDate,$endDate);
		if($countNoInf->code == 0){
			$countNoInf 	= count($countNoInf->data['list']);
		}

		$resultData 		= [
			"attend" 	=> $countAttend,
			"noinf" 	=> $countNoInf
		];
		
		$result->data 	= $resultData;
		$result->info   = $this->translate($result->info);
		return $this->getREST()->getResponse($result);
	}
}