<?php

namespace Neuron\Attendance\Controller;

use Neuron\Notification;
use Neuron\Employee\Entity as Employee;
use Neuron\Generic\Result;

use Neuron\Attendance\Entity;
use Neuron\Attendance\Attendance;
use Neuron\Attendance\Request as RequestM;
use Neuron\Attendance\Requests;
use Neuron\Attendance\Timetable\Changeshift;
use Neuron\Attendance\Request\Type;
use Neuron\Attendance\Request\Types;
use Neuron\Attendance\Request\Setting;
use Neuron\Attendance\Request\Setting\Detail;
use Neuron\Attendance\Request\Setting\Details;
use Neuron\Order\Entity as OrderEntity;
use Neuron\Addon\Order\SimpleFlow;
use Zend\Stdlib\ArrayUtils;
use Neuron\Attendance\Request\Status;
use Neuron\Attendance\Request\Statuses;

use Neuron\Actor\User;
use Neuron\Organization\Entity as OrganizationEntity;
use Neuron\Organization\Position;

class Request extends Controller
{

    const UPDATE_BY = '10000000';

    const SUCCESS_CODE = 0;
    const FAIL_CODE = 1;
    const CODE_FIELD_REQUIRED  = 400;
    const CODE_UNAUTHORIZED = 401;
    const CODE_SESSION_EXPIRED = 403;

    const REQUEST_TYPE_ATTENDANCE   = 1;
    const REQUEST_TYPE_OVERTIME     = 2;
    const REQUEST_TYPE_CHANGESHIFT  = 3;

    const ORDER_TYPE_ID_ATTENDANCE = 2100;
    const ORDER_TYPE_ATTENDANCE    = 2101;
    const ORDER_TYPE_OVERTIME      = 2102;
    const ORDER_TYPE_OVERTIME_WORKDAY      = 2110;
    const ORDER_TYPE_CHANGESHIFT   = 2103;

    const APPROVAL_STATUS_APPROVED = 2;
    const APPROVAL_STATUS_REJECTED = 3;

    protected function getView($name, $variables = array())
    {
        $layout = 'neuron/attendance/layout/form';
        $prefix = 'neuron/attendance/request';

        $this->using('font-awesome', 10, 'https://use.fontawesome.com/releases/v5.8.1/css/all.css');

        /* libraries */
        if (!class_exists('Neuron\Application\Framework\Entity')) {
            $this->using(
                'bootstrap-4',
                null,
                array(
                    'https://ajax.googleapis.com/ajax/libs/jquery/3.3.1/jquery.min.js',
                    'https://cdnjs.cloudflare.com/ajax/libs/popper.js/1.14.7/umd/popper.min.js',
                    'https://maxcdn.bootstrapcdn.com/bootstrap/4.3.1/css/bootstrap.min.css',
                    'https://maxcdn.bootstrapcdn.com/bootstrap/4.3.1/js/bootstrap.min.js'
                )
            )
                ->using('toast', null)
                ->using('bootbox', null);
        }

        $this->using('generic', 200);

        /* get layout n view prefix from config */
        $config = $this->getConfig();
        if (isset($config['attendance'])) {
            $config = $config['attendance'];
            $layout = isset($config['formlayout']) ? $config['formlayout'] : $layout;
            $prefix = isset($config['viewprefix']) ? $config['viewprefix'] : $prefix;
        }
        /* set layout and view */
        $this->layout($layout);
        $view = new \Zend\View\Model\ViewModel($variables);
        $view->setTemplate($prefix . '/' . strtolower($name));

        return $view;
    }
    public function ManageAction()
    {
        $query 		= $this->getRequest()->getQuery();
		if(isset($query['notif_id'])){
			$storage = InboxModel\Storage::factory($this->getDb(), $this->getConfig(),3,$this->getSession()->owner());
			$inbox = new InboxModel($storage);
			$inbox->read($query['notif_id']);
		}
		$sessmgt = $this->getSession()->get();
		$this->using('moment',19);
		$this->using('datatable', 20, array(
            'https://cdn.datatables.net/1.10.19/js/jquery.dataTables.min.js',
            'https://cdn.datatables.net/1.10.19/js/dataTables.bootstrap4.min.js',
            'https://cdn.datatables.net/1.10.19/css/dataTables.bootstrap4.min.css'
        ));
        $this->using('air-datepicker',21);
        $this->using('select2', 22, array(
		    'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.10/js/select2.min.js',
		     /* ini load sesuai bahasa dari session */
		    'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.10/js/i18n/id.js',
		     /**/
		    'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.10/css/select2.min.css',
		    'https://cdn.jsdelivr.net/npm/@ttskch/select2-bootstrap4-theme@1.3.2/dist/select2-bootstrap4.min.css',
		));
		$this->using('bootbox', 23);

		$this->using('generic',24);
        $this->usingjs('jquery.validate',25);
		$this->using('toast', 26);
		$this->using('ploading', 27, array(
			'https://rawgit.com/joseshiru/p-loading/master/src/css/p-loading.css',
            'https://rawgit.com/joseshiru/p-loading/master/src/js/p-loading.js',
        ));


        $types 		= new Types(Type\Storage::factory($this->getDb(),$this->getConfig()));
        $types 		= $types->load();

        $status 	= new Statuses(Status\Storage::factory($this->getDb(),$this->getConfig()));
        $status 	= $status->load();

		$usrRefId 	= isset($sessmgt["ref_id"]) ? $sessmgt['ref_id'] : null;
		
		$usrJobId = null;
		$usrOrgId = null;


		$config = $this->getConfig();
		$checkin_recomendation = $config['attendance']['checkin_recomendation'];

		$layout = $this->layout();
		$layout->setVariables(['current' => $this->translate('Manage Request')]);


        return $this->getView('manage', get_defined_vars());
    }
    public function loadParamAction()
    {
        $refstorage = \Neuron\Attendance\Attendance\Reference\Storage::factory($this->getDb(), $this->getConfig());
        $crefs = new \Neuron\Attendance\Attendance\References($refstorage);

        $sourstorage = \Neuron\Attendance\Attendance\Source\Storage::factory($this->getDb(), $this->getConfig());
        $csour = new \Neuron\Attendance\Attendance\Source($sourstorage);
        $refs = $crefs->listReference(1, 999);
        $sours = $csour->listSource(1, 999);

        echo json_encode(array('references' => $refs, 'source' => $sours));
        return $this->getResponse();
    }

    public function listAction()
    {
        //Check Current Status
        $this->CheckUpdatedStatusRequest();


        $postData = $this->getParameters();
        $session = $this->getSession();
        if ($postData) {
            $result = new \Neuron\Generic\Result();

            $username = isset($postData['username']) ? $postData['username'] : null;
            $token = isset($postData['token']) ? $postData['token'] : "";
            $page = isset($postData['page']) ? $postData['page'] : 1;
            $rowCount = isset($postData['row_count']) ? $postData['row_count'] : 10;
            try {
                if ($token == "") {
                    $result->code = self::CODE_FIELD_REQUIRED;
                    $result->info = 'token is null';
                    return $this->getREST()->getResponse($result);
                }

                $userPerson = $this->getUserPerson($username);

                $session->owner($userPerson['user']['user_id']);

                $sessid = $session->getId();
                if ($token != $sessid) {
                    $result->code = self::CODE_UNAUTHORIZED;
                    $result->info = 'Failed authenticate';
                    return $this->getREST()->getResponse($result);
                }

                $islogin = $this->isLoggedIn();
                if (!$islogin) {
                    $result->code = self::CODE_SESSION_EXPIRED;
                    $result->info = 'Session Expired';
                    return $this->getREST()->getResponse($result);
                }
                $storage = \Neuron\Attendance\Request\Storage::factory($this->getDb(), $this->getConfig());
                $catt = new \Neuron\Attendance\Request($storage);

                $postData['filter'] = array('searchField' => 'ref_id', 'searchValue' => '', 'ref_id' => $userPerson['person']['ref_id']);
                $data = $catt->listRequest($postData);
                $result->guid = self::SUCCESS_CODE;
                $result->code = self::SUCCESS_CODE;
                $result->info = 'OK';
                $result->data = $data['rows'];
                return $this->getREST()->getResponse($result);
            } catch (Exception $e) {
                $result->code = self::SUCCESS_CODE;
                $result->info = 'error';
                $result->data = array(
                    'message' => $ex->getMessage(),
                );
                return $this->getREST()->getResponse($result);
            }
        } else {
            $post = $this->getPostParams();

            $post["request_type"] = $this->params()->fromQuery('request_type');
            $post["start_dtm"] = $this->params()->fromQuery('start_dtm');
            $post["end_dtm"] = $this->params()->fromQuery('end_dtm');
            $post["status_id"] = $this->params()->fromQuery('status_id');

            $storage = \Neuron\Attendance\Request\Storage::factory($this->getDb(), $this->getConfig());
            $catt = new \Neuron\Attendance\Request($storage);
            $result = $catt->listRequest($post);

            echo json_encode($result);
            return $this->getResponse();
        }
    }

    public function saveAction(){
        $postData   = $this->getParameters();

		$savedNotif = array();
        $result     = new Result();

        $sessmgt    = $this->getSession()->get();
        $sess       = (count($sessmgt) > 0) ? true : false;
        $storageTs  = \Neuron\Attendance\Shift\Storage::factory($this->getDb(), $this->getConfig());
        $cts        = new \Neuron\Attendance\Shift($storageTs);

        $request    = new \Neuron\Approval\Entity(\Neuron\Order\Entity\Storage::factory($this->getDb(),$this->getConfig()));          

        $post       = $this->getPostParams();

        $requestPost = array(
			'guid' => '',
			'code' => '',
			'info' => '',
			'data' => $post
        ); 

        if ($post['start_dtm']!= null) {
            $start_dtm = date("Y-m-d H:i:s", strtotime($post['start_dtm']));
        } else {
            $start_dtm = null;
        }

        if ($post['end_dtm'] != null) {
            $end_dtm = date("Y-m-d H:i:s", strtotime($post['end_dtm']));
        } else {
            $end_dtm = null;
        }

        if ($post['request_type'] == 2103) {
            $start_dtm = date("Y-m-d H:i:s", strtotime($post['change_date']));
        }         

        if ($post['end_dtm'] != null AND $post['start_dtm'] != null) {
            $difference = strtotime(date('H:i:s', strtotime($end_dtm))) - strtotime(date('H:i:s', strtotime($start_dtm)));
            $hours      = floor($difference / 60 / 60);

            if ($hours <= 0) {
                $result->code       = 413;
				$result->info       = 'the end date cannot be smaller than the start date';
				
				$responseResult = array(
					'code' => $result->code,
					'info' => $result->info,
					'data' => $result->data
				); 

				$logparam['subject'] = 'ATTENDANCE REQUEST SAVEACTION';
				$logparam['identity'] = '';
				$logparam['user'] = '';
				$logparam['description'] = 'Log proses attendance request saveaction';
				$logparam['request'] = json_encode($requestPost);
				$logparam['response'] = json_encode($responseResult);

				$this->writeLogAction($logparam);

				$result->info = $this->translate($result->info);
				return $this->getREST()->getResponse($result);
            }
        }
        
        # Fetching Ref ID [BEGIN]
        $session_owner          = $this->getSession()->owner();
        if (class_exists('\Neuron\Employee\Entity')) {
            $map                = $this->getMapPerson(null);
            $ref_id             = isset($map['ref_id']) ? $map['ref_id'] : null;
        } else {
            $ref_id             = $session_owner;
        }
        # Fetching Ref ID [END]


        # tambah validasi sebelum
		if ($nafManager = \Neuron\Application\Framework\Event\Manager::get()) {
			if($resultEvent = $nafManager->events()->trigger('attendance_before_request', $post)){
                if($resultEvent->code != 0){
                    return $this->getREST()->getResponse($resultEvent);
                }
			}
		}
		
		# Validasi Lembur
		$storageRequest		= \Neuron\Attendance\Request\Storage::factory($this->getDb(), $this->getConfig());
        $requestValidation	= new RequestM($storageRequest);
		$validateRequest 	= $requestValidation->validateRequest($ref_id, $post['request_type'], $start_dtm, $end_dtm);
		
		if($validateRequest['out_statusCode'] == 0){
			# Cek timetable, apakah waktu request sesuai dengan yang diajukan [BEGIN]
			$cektimetable = $cts->loadObjectTimetable($ref_id, $start_dtm , $post['request_type']);
			if (!$cektimetable['success']) {
		  
				$result->code       = 412;
				$result->info       = $cektimetable['message'];
				
				$responseResult = array(
					'code' => $result->code,
					'info' => $result->info,
					'data' => $result->data
				); 

				$logparam['subject'] = 'ATTENDANCE REQUEST SAVEACTION';
				$logparam['identity'] = '';
				$logparam['user'] = '';
				$logparam['description'] = 'Log proses attendance request saveaction';
				$logparam['request'] = json_encode($requestPost);
				$logparam['response'] = json_encode($responseResult);

				$this->writeLogAction($logparam);

				$result->info = $this->translate($result->info);
				return $this->getREST()->getResponse($result);
			}
			# Cek timetable, apakah waktu request sesuai dengan yang diajukan [END]
			
			# Buat data untuk dikirim ke "Module Approval"
            $requesterDetail = array();
            # check user
            $employeeMap    = new \Neuron\Employee\Map(\Neuron\Employee\Map\Storage::factory($this->getDb(), $this->getConfig()));
			$employeeMap->load($post['ref_id'],\Neuron\Employee\Map\Storage::LOADBY_EMPID);
			$requesterMap = array_merge(
                $employeeMap->employee()->pull(),
                $employeeMap->pull(),
                $employeeMap->employee()->person()->pull(),
                $employeeMap->employee()->job()->pull(),
                $employeeMap->employee()->job()->ohp()->pull(),
                $employeeMap->employee()->job()->type()->pull()
            );

            # load requester detail

            # get organization
            $orgPosition 	= new Position(Position\Storage::factory($this->getDb(),$this->getConfig()));
            $org 			= new OrganizationEntity(OrganizationEntity\Storage::factory($this->getDb(),$this->getConfig()));
            $orgPosition->load($requesterMap['position_id']);
		    $org->load($requesterMap['organization_id']);
            
            $detailOrg = array_merge($orgPosition->pull(),$org->pull());

            $requesterDetail['position'] =   $detailOrg['title']; 
            $requesterDetail['organization'] =   $detailOrg['name']; 
            $requesterDetail['code'] =   $detailOrg['code']; 
            
            $user = new User(User\Storage::factory($this->getDb(), $this->getConfig()));
            $user->pictures->load($requesterMap['user_id'], \Neuron\Actor\User\Picture\Storage::LISTBY_USER_ID);
            $pic = $user->pictures->find('active', 1);
            if($user->pictures->count() > 0){
                if($pic = $user->pictures->find('active', 1)){
                    $picture = $pic->name;
                }else{
                    $picture =  $this->getPublicURL().'/actor/picture/default';
                }
            }else{
                $picture =  $this->getPublicURL().'/actor/picture/default';
            }
    
            if ($picture) {						
                if (!file_exists(getcwd() .'/public'. $picture)) {
                    $requesterDetail['avatar'] = $this->getPublicURL().'/actor/picture/default';
                } else {
                    $requesterDetail['avatar'] = $this->getPublicURL().$picture;
                }
            } else {
                $requesterDetail['avatar']     = $this->getPublicURL().'/actor/picture/default';
            }

            $requesterDetail['birth_date'] = $requesterMap['birth_date'];
            $requesterDetail['birth_place'] = $requesterMap['birth_place'];
            

			$approval_to = array();
			foreach ($post['approval_ref_id'] as $i => $val){
				$approval_to[] = $val;
			}
            
			$data = [
				 "order_type_id"        => self::ORDER_TYPE_ID_ATTENDANCE,
				 "order_subtype_id"     => $post['request_type'],
				 "order_desc"           => $post['request_message'],
				 "customer_desc"        => $requesterMap['first_name']." ".$requesterMap['last_name'],
				 "create_user_id"       => $requesterMap['user_id'],
				 "xd1"                  => $start_dtm,
				 "xd2"                  => $end_dtm,
                 "xs15"                  => $requesterDetail,
				 "flow"                 =>[
					"order_flow_type_id" => 1,
					"assigned_to_id"     => $approval_to,
					"remark_in"          => "Assign to user"
				]
			];
            // print_r($data);die('profile picture');
            
			# cek masing masing tipe request [BEGIN]
			if($post['request_type'] == self::ORDER_TYPE_OVERTIME OR $post['request_type'] == self::ORDER_TYPE_OVERTIME_WORKDAY){
				 # Validasi overtime ==> kalo request overtime harus ada attendance na dulu
				$cektimetable = $cts->loadObjectTimetable($ref_id, $start_dtm, $post['request_type']);
				if(!$cektimetable['success']){
					if($post['request_type'] == self::ORDER_TYPE_OVERTIME_WORKDAY){
						$start_date         = date("Y-m-d",strtotime($post['start_dtm']));
						$entity             = new Entity(Entity\Storage::factory($this->getDb(),$this->getConfig()));
						$entity->load($ref_id,Entity\Storage::LOADBY_ID,array('ref_id' => $ref_id, 'start_date' => $start_date));
						if(!$entity->id){
							$result->code   = 414;
							$result->info   = "You need to attend at to request overtime!";
							
							$responseResult = array(
								'code' => $result->code,
								'info' => $result->info,
								'data' => $result->data
							); 

							$logparam['subject'] = 'ATTENDANCE REQUEST SAVEACTION';
							$logparam['identity'] = '';
							$logparam['user'] = '';
							$logparam['description'] = 'Log proses attendance request saveaction';
							$logparam['request'] = json_encode($requestPost);
							$logparam['response'] = json_encode($responseResult);

							$this->writeLogAction($logparam);
							
							$result->info = $this->translate($result->info);
							return $this->getREST()->getResponse($result);
						}
					}
				} else {
					# Simpan data untuk dikirim ke "Module Approval"
					$retval = $request->create($data);
				}

			}
			else if ($post['request_type'] == self::ORDER_TYPE_CHANGESHIFT) {
				$changeShift = new Changeshift(Changeshift\Storage::factory($this->getDb(), $this->getConfig()));
			
				# cek duplicate shift [BEGIN]
				$parCh['ref_id']             = $sessmgt['ref_id'];
				$parCh['shift_change_date']  = date('Y-m-d', strtotime($start_dtm));
				$checkChShift                = $changeShift->loadEx($id = null ,Changeshift\Storage::LOADBY_REFID_CHANGEDATE, $parCh);
				
				if ($checkChShift->data) {
					return $this->getREST()->getResponse(new Result(0, 511, $this->translate('You have change shift request on this date ')));
				} else {
					$retval = $request->create($data);
					if (isset($post['swap']) && $post['swap'] == "on") {
		
						$dayno       = $cts->getDayNumber(date('Y-m-d', strtotime($post['change_date'])))->data;
						$datachshift = $cts->loadShiftObject($post['target_id'], null);
		
						if ($datachshift) {
							foreach ($datachshift as $val) {
								if ($val['day_no'] == $dayno) {
									$targetId = $val['timetable_shift_id'];
								}
							}
						}
		
						$chshift['tt_employee_shift_id'] = null;
						$chshift['ref_code'] = $post['ref_code'];
						$chshift['ref_id'] = $post['target_id'];
						$chshift['shift_change_date'] = date('Y-m-d', strtotime($start_dtm));
						$chshift['default_shift_id'] = $targetId;
						$chshift['change_shift_id'] =  $cektimetable['data']['timetable_shift_id'];
						$chshift['recommendation_id'] = $retval->data;
						$chshift['change_by'] = $sessmgt['ref_id'];
					} else {
		
						$targetId = $post['target_id'];
						$chshift['tt_employee_shift_id']    = null;
						$chshift['ref_code']                = $post['ref_code'];
						$chshift['ref_id']                  = $sessmgt['ref_id'];
						$chshift['shift_change_date']       = date('Y-m-d', strtotime($start_dtm));
						$chshift['default_shift_id']        = $cektimetable['data']['timetable_shift_id'];
						$chshift['change_shift_id']         = $targetId;
						$chshift['recommendation_id']       = $retval->data;
						$chshift['change_by']               = $sessmgt['ref_id'];
					}

					$saveChShift                        = $changeShift->saveEx($chshift);
					 
				}
				# cek duplicate shift [END]

			}
			else {
				 $retval = $request->create($data);
			}
			# cek masing masing tipe request [END]

			# Jika sukses lakukan pengecekan untuk action lanjutan
			$employeeMap    = new \Neuron\Employee\Map(\Neuron\Employee\Map\Storage::factory($this->getDb(), $this->getConfig()));
			$employeeMap->load($post['ref_id'],\Neuron\Employee\Map\Storage::LOADBY_EMPID);
			$requesterMap = array_merge($employeeMap->employee()->pull(), $employeeMap->pull(), $employeeMap->employee()->person()->pull());
			
			$targetId = null;
			if ($retval->code == 0) {

				# proses kirim notifikasi [BEGIN]
				if ($approval_to) {
					foreach ($approval_to as $appr){
						$employeeMap->load($appr,\Neuron\Employee\Map\Storage::LOADBY_EMPID);
						$empMap = array_merge($employeeMap->employee()->pull(), $employeeMap->pull(), $employeeMap->employee()->person()->pull());
						$userId     = $empMap['user_id'];
		
						# proses notivikasi [BEGIN]
						$requestTypeName            = "";
						switch ($post['request_type']) {
							case self::ORDER_TYPE_ATTENDANCE:
								$requestTypeName    = 'Clock in';
								break;
							case self::ORDER_TYPE_OVERTIME:
								$requestTypeName    = 'Overtime';
								break;
							case self::ORDER_TYPE_OVERTIME_WORKDAY:
								$requestTypeName    = 'Overtime Workday';
								break;
							case self::ORDER_TYPE_CHANGESHIFT:
								$requestTypeName    = "Change shift";
								break;
							default:
								$requestTypeName    = "Clock in";
								break;
						}

						$inboxStorage   = Notification\Inbox\Storage::factory(
							$this->getDb(),
							$this->getConfig(),
							Notification\Profile::TYPE_FCM,
							$userId
						);
						$inbox      = new Notification\Inbox($inboxStorage);
						$inbox->setPublicUrl($this->getPublicUrl());

						# check terlebihdahuu apakah sudah ada tabelnya atau belum
						$result = @$inbox->pushNotification(
							$userId,
							'REQUEST_NEED_APPROVAL',
							array(
								'name' => $requesterMap['first_name'] . ' ' . $requesterMap['last_name'],
								'request_type' => $requestTypeName,
							),
							array(
								'request_id'    => $retval->data,
								'approval_id'   => $retval->data,
								'assigned_to_id'   => $userId
							)
						);
						$notifId->$userId = $inbox->id;
						
						if ($result->code == 0) {
							$savedNotif[]   = $userId;
							
						}
						# proses notivikasi [END]
						
						$notifIdJSON = json_encode($notifId);
					}
					
					$data = [
						 "order_id"        		=> $retval->data,
						 "order_type_id"        => self::ORDER_TYPE_ID_ATTENDANCE,
						 "order_subtype_id"     => $post['request_type'],
						 "order_desc"           => $post['request_message'],
						 "customer_desc"        => $sessmgt['name'],
						 "create_user_id"       => $sessmgt['user_id'],
						 "xd1"                  => $start_dtm,
						 "xd2"                  => $end_dtm,
						 "xs2"                  => $notifIdJSON,
					];
					
					$result    = $request->saveNotifId($data);
				}
		
			}
			
			$result         = new Result();
			$result->data   = null;

			# create log
			$responseResult = array(
				'code' => $result->code,
				'info' => $result->info,
				'data' => $result->data
			); 

			$logparam['subject'] = 'ATTENDANCE REQUEST SAVEACTION';
			$logparam['identity'] = '';
			$logparam['user'] = '';
			$logparam['description'] = 'Log proses attendance request saveaction';
			$logparam['request'] = json_encode($requestPost);
			$logparam['response'] = json_encode($responseResult);

			$this->writeLogAction($logparam);
		}else{
			$result         = new Result();
			$result->code       = $validateRequest['out_statusCode'];
            $result->info       = $validateRequest['out_statusMsg'];
			
			$responseResult = array(
				'code' => $result->code,
				'info' => $result->info,
				'data' => null
			); 
		}

        $result->info = $this->translate($result->info);
        return $this->getREST()->getResponse($result);
    }

    public function saveApproveAction(){
        $post       = $this->getREST()->getParameters();
        if (!$post) {
            $post   = $this->getRequest()->getPost();
        }
        if(!isset($post['status_id']) && isset($post['status'])){
            $post['status_id'] = $post['status'];
        }
        
		$entity = new \Neuron\Approval\Entity(\Neuron\Order\Entity\Storage::factory($this->getDb(), $this->getConfig()));
		$param['order_id'] = $post['approval_id'];
		/* load and return */
		$result = $entity->loadEx($param, 3);
		$data = $result->data;
        
		//load data type
        $toOrder['user_id'] = $this->getSession()->owner();
        $toOrder['order_id'] = $data['order_id'];
        $toOrder['flow']['order_flow_status_id'] = intval($post['status_id']);
        $toOrder['flow']['remark_out'] = $post['reason'];
        $request   = new \Neuron\Approval\Entity(\Neuron\Order\Entity\Storage::factory($this->getDb(),$this->getConfig()));          
        $result    = $request->update(@$toOrder);
        
	    if($result->code == 0){
			$loadnewData = $entity->loadEx($data['order_id'], 3);
            $newData = $loadnewData->data;
            
			if($data['order_status_id'] != $newData['order_status_id']){
                //jika status berubah
				if($post['status_id'] == 2){
                    //jika approve
                    $template_code 	= 'REQUEST_APPROVE'; //template notif
                    
                    // Create attendance data. changeshift / overtime / attendance;
                    $requestType    = new \Neuron\Attendance\Request(\Neuron\Attendance\Request\Storage::factory(
                        $this->getDb(),
                        $this->getConfig()
                    ));
                    
                    $createRequestData = $requestType->createRequestTypeData($newData);
                    if($createRequestData->code != 0){
                        $result->info   = $this->translate($result->info);
                        return $this->getREST()->getResponse($result);
                    }
				}else{
					//jika reject
					$template_code 	= 'REQUEST_REJECT'; //template notif
				}
				
				## NOTIFICATION HERE
				# Kirim notifikasi kalo request di approve / di reject.
				$inboxStorage   = Notification\Inbox\Storage::factory(
					$this->getDb(),
					$this->getConfig(),
					Notification\Profile::TYPE_FCM,
					$newData['create_user_id']
				);
				$inbox      = new Notification\Inbox($inboxStorage);
				$inbox->setPublicUrl($this->getPublicUrl());

				# check terlebihdahuu apakah sudah ada tabelnya atau belum
				$res = @$inbox->pushNotification(
					// $userId,
					$newData['create_user_id'],
					$template_code ,
					array(
						'request_type' => $newData['order_type_name'],
					),
					array(
						'request_id' 	=> $newData['order_id'],
					)
				);
				
				$array = array(
                    "userId" => $userId,
                    "template_code" => $template_code,
                    "request_type" => $newData['order_status_name'],
                    "request_id" => $newData['order_id'],
				);
				
				if ($res->code == 0) {
					$savedNotif[]   = $userId;
				}
			}
		}

	    return $this->getREST()->getResponse($result);
    }

    public function saveApproveOldAction()
    {
        $post       = $this->getREST()->getParameters();
        if (!$post) {
            $post   = $this->getRequest()->getPost();
        }
        if (!isset($post['ref_id'])) {
            $user_id    = $this->getSession()->owner();
            $mapPerson  = $this->getMapPerson($user_id);
            $post['ref_id'] = $mapPerson['ref_id'];
        }
        $request    = new RequestM(RequestM\Storage::factory($this->getDb(), $this->getConfig()));
        $publicUrl  = $this->getPublicUrl();
        $result     = $request->saveApproval($post, $publicUrl);
        return $this->getREST()->getResponse($result);
    }

    public function updateAction(){
        $sessmgt   = $this->getSession()->get();
        $result    = new \Neuron\Generic\Result(); 
        $post      = $this->getREST()->getParameters();
        
        $data['user_id']  = $sessmgt['user_id'];
        $data['order_id'] = $post['order_id'];
        $data['flow']     = $post['flow'];

        $request   = new \Neuron\Approval\Entity(\Neuron\Order\Entity\Storage::factory($this->getDb(),$this->getConfig()));          
        $result   = $request->update(@$data);

        if ($result->code == 0) {

            # Check statusnya rejected / approved untuk notifikasi template.
            if($post['flow']['order_flow_status_id'] == self::APPROVAL_STATUS_APPROVED){
                $template_code 	= 'REQUEST_APPROVE';
            }else if($post['flow']['order_flow_status_id'] == self::APPROVAL_STATUS_REJECTED){
                $template_code 	= 'REQUEST_REJECT';
            } 

            # data requester
            $reqId = 7;

            # check user
            $employee       = new Employee(Employee\Storage::factory($this->getDb(), $this->getConfig()));
            $empMap         = $employee->loadMap($reqId, 'person_id');
            $userId         = $empMap['data']['user_id'];


            # Kirim notifikasi kalo request di approve / di reject.
            $inboxStorage   = Notification\Inbox\Storage::factory(
                $this->getDb(),
                $this->getConfig(),
                Notification\Profile::TYPE_FCM,
                $userId
            );
            $inbox      = new Notification\Inbox($inboxStorage);
            $inbox->setPublicUrl($this->getPublicUrl());

            # check terlebihdahuu apakah sudah ada tabelnya atau belum
            $result = @$inbox->pushNotification(
                $userId,
                $template_code ,
                array(
                    'request_type' => $post['order_subtype'],
                ),
                array(
                    'request_id' 	=> 7,
                )
            );

            if ($result->code == 0) {
                $savedNotif[]   = $userId;
            }

        }

        return $this->getREST()->getResponse($result);
    }

    public function saveOldAction(){
        $postData   = $this->getParameters();
		
		$savedNotif = array();
        $result     = new Result();

        $sessmgt    = $this->getSession()->get();
        $sess       = (count($sessmgt) > 0) ? true : false;

        $storage    = \Neuron\Attendance\Request\Storage::factory($this->getDb(), $this->getConfig());
        $creq       = new \Neuron\Attendance\Request($storage);
        $storageTs  = \Neuron\Attendance\Shift\Storage::factory($this->getDb(), $this->getConfig());
        $cts        = new \Neuron\Attendance\Shift($storageTs);
        $post       = $this->getPostParams();

		$requestPost = array(
			'guid' => '',
			'code' => '',
			'info' => '',
			'data' => $post
		); 
		

        /* Fetching Ref ID */
        $session_owner          = $this->getSession()->owner();
        if (class_exists('\Neuron\Employee\Entity')) {
            $map                = $this->getMapPerson($session_owner);
            $ref_id             = $map['ref_id'];
        } else {
            $ref_id             = $session_owner;
        }
        /* Fetching Ref ID */

        // Validating overtime ==> kalo request overtime harus ada attendance na dulu
        if($post['request_type'] == self::REQUEST_TYPE_OVERTIME){
            
            $cektimetable = $cts->loadObjectTimetable($ref_id, date('Y-m-d H:i:s',strtotime($post['start_dtm'])), $post['request_type']);
            if(!$cektimetable['success']){
                $start_date         = date("Y-m-d",strtotime($post['start_dtm']));
                $entity             = new Entity(Entity\Storage::factory($this->getDb(),$this->getConfig()));
                $entity->load($ref_id,Entity\Storage::LOADBY_ID,array('ref_id' => $ref_id, 'start_date' => $start_date));
                if(!$entity->id){
                    $result->code   = 414;
                    $result->info   = "You need to attend at $start_date to request overtime!";
					
					$responseResult = array(
						'code' => $result->code,
						'info' => $result->info,
						'data' => $result->data
					); 

					$logparam['subject'] = 'ATTENDANCE REQUEST SAVEACTION';
					$logparam['identity'] = '';
					$logparam['user'] = '';
					$logparam['description'] = 'Log proses attendance request saveaction';
					$logparam['request'] = json_encode($requestPost);
					$logparam['response'] = json_encode($responseResult);

					$this->writeLogAction($logparam);
					
                    return $this->getREST()->getResponse($result);
                }
            }

        }

        /*
        * Validating approval : 
        * validasi dari setting dulu. cocokkan dengan setting yang dipake
        * tidak bisa request ke diri sendiri --> Masih belum jelas.
        * tidak bisa request ke orang yang sama.
        */
        /* Load request type untuk ngambil setting_id nya */
        $requestType    = new Type(Type\Storage::factory($this->getDb(), $this->getConfig()));
        $load           = $requestType->load($post['request_type']);
        if ($load->code != 0) {
            $result = new Result(0, 511, $this->translate('Failed loading request setting'));
			
			$responseResult = array(
				'code' => $result->code,
				'info' => $result->info,
				'data' => $result->data
			); 

			$logparam['subject'] = 'ATTENDANCE REQUEST SAVEACTION';
			$logparam['identity'] = '';
			$logparam['user'] = '';
			$logparam['description'] = 'Log proses attendance request saveaction';
			$logparam['request'] = json_encode($requestPost);
			$logparam['response'] = json_encode($responseResult);

			$this->writeLogAction($logparam);
			
            return $this->getREST()->getResponse($result);
        }

        /* Ambil setting_detailnya */
        $setting_id     = $load->data['setting_id'];
        $detail         = new Details(Detail\Storage::factory($this->getDb(), $this->getConfig()));
        $listDetail     = $detail->load($setting_id, Detail\Storage::LISTBY_SETTING_ID, array('setting_id' => $setting_id));
        if ($listDetail->code != 0) {
            $result     = new Result(0, 512, $this->translate('Failed loading setting detail for this request type'));
			
			$responseResult = array(
				'code' => $result->code,
				'info' => $result->info,
				'data' => $result->data
			); 

			$logparam['subject'] = 'ATTENDANCE REQUEST SAVEACTION';
			$logparam['identity'] = '';
			$logparam['user'] = '';
			$logparam['description'] = 'Log proses attendance request saveaction';
			$logparam['request'] = json_encode($requestPost);
			$logparam['response'] = json_encode($responseResult);

			$this->writeLogAction($logparam);

			
            return $this->getREST()->getResponse($result);
        }

        $listDetail         = $listDetail->data['rows'];
        $requiredApproval   = count($listDetail);
        $approvalIsValid    = FALSE;
        $postedApproval     = $post['approval_ref_id'];
        $neededApproval     = array();

        /* Comparing start setting_detail & posted approval start here */
        foreach ($listDetail as $row) {
            if ($row['approval_type_id'] == 2) {
                $neededValue        = $row['type_value'];
                $approvalIsValid    = FALSE;

                /* Find if needed value exists in post */
                foreach ($postedApproval as $pa) {
                    if ($pa == $neededValue) {
                        $approvalIsValid    = TRUE;
                        break;
                    }
                }

                if (!$approvalIsValid) {
                    $neededApproval[] = array(
                        'value'         => $row['type_value'],
                        'description'   => $row['type_value_description']
                    );
                    break;
                }
            } else {
                $approvalIsValid            = TRUE;
            }
        }

        if (!$approvalIsValid) {
            $result         = new Result();
            $result->code   = 413;
            $result->info   = $this->translate("Specific approval required");
            $result->data   = $neededApproval;
			
			$responseResult = array(
				'code' => $result->code,
				'info' => $result->info,
				'data' => $result->data
			); 

			$logparam['subject'] = 'ATTENDANCE REQUEST SAVEACTION';
			$logparam['identity'] = '';
			$logparam['user'] = '';
			$logparam['description'] = 'Log proses attendance request saveaction';
			$logparam['request'] = json_encode($requestPost);
			$logparam['response'] = json_encode($responseResult);

			$this->writeLogAction($logparam);

			
            return $this->getREST()->getResponse($result);
        }
        /* Comparing start setting_detail & posted approval end here */

        /* Validating duplicate approval */
        $validApproval      = array();
        $duplicateApproval  = array();
        foreach ($postedApproval as $pa) {
            $find = FALSE;
            foreach ($validApproval as $va) {
                if ($va === $pa) {
                    $find = TRUE;
                }
            }
            if (!$find) {
                $validApproval[]        = $pa;
            } else {
                $duplicateApproval[]    = $pa;
            }
        }

        if (count($duplicateApproval) > 0) {
            $result         = new Result();
            $result->code   = 414;
            $result->info   = $this->translate("Cannot choose same approval at one request.");
            $result->data   = $duplicateApproval;
            return $this->getREST()->getResponse($result);
        }
        /* Validating duplicate approval */

        /* Validating approving to self */
        /* Kalo ada spesifik ke diri sendiri skip aja gausah di validasi */
        $bypass         = FALSE;
        foreach ($listDetail as $data) {
            if ($data['approval_type_id'] == 2 && $data['type_value'] === $ref_id) {
                $bypass = TRUE;
                break;
            }
        }

        if (!$bypass) {
            foreach ($validApproval as $data) {
                if ($data == $ref_id) {
                    $result         = new Result();
                    $result->code   = 414;
                    $result->info   = $this->translate("Cannot requesting approval to self.");
                    return $this->getREST()->getResponse($result);
                }
            }
        }
        /* Validating approving to self */

        $post['start_dtm'] = date('Y-m-d H:i:s', strtotime($post['start_dtm']));
        $post['end_dtm'] = ($post['end_dtm'] != "") ? date('Y-m-d H:i:s', strtotime($post['end_dtm'])) : null;

        if ($post['request_type'] == 3) {
            $post['start_dtm'] = date("Y-m-d H:i:s", strtotime($post['change_date']));
        } else if ($post['request_type'] == 1) {
            $attendance     = new Attendance(Attendance\Storage::factory($this->getDb(), $this->getConfig()));
            $checkParams    = array(
                'chckdate'  => date('Y-m-d', strtotime($post['start_dtm'])),
                'ref_id'    => $ref_id,
                'start_dtm' => null
            );
            $checkByDate    = $attendance->listAttendance($checkParams);
            if (count($checkByDate['data']) > 0) {
                return $this->getREST()->getResponse(new Result(0,411, $this->translate("You already attended at the day you choosen. Cannot duplicating attendance")));
            }
        }

        /* Cek timetable, apakah waktu request sesuai dengan yang diajukan */

        $cektimetable = $cts->loadObjectTimetable($ref_id, $post['start_dtm'], $post['request_type']);

        if (!$cektimetable['success']) {
            $result->code       = 412;
            $result->info       = $cektimetable['message'];
			
			$responseResult = array(
				'code' => $result->code,
				'info' => $result->info,
				'data' => $result->data
			); 

			$logparam['subject'] = 'ATTENDANCE REQUEST SAVEACTION';
			$logparam['identity'] = '';
			$logparam['user'] = '';
			$logparam['description'] = 'Log proses attendance request saveaction';
			$logparam['request'] = json_encode($requestPost);
			$logparam['response'] = json_encode($responseResult);

			$this->writeLogAction($logparam);
			
            return $this->getREST()->getResponse($result);
        }

        if (!empty($post['id'])) {
            $update = true;
        } else {
            $update = false;
            $request_date       = explode(' ', $post['start_dtm'])[0];
            $request_type       = $post['request_type'];

            $check  = $creq->checkDuplicateRec($request_type, $request_date, $ref_id)->data;

            if (count($check) > 0) {
                $result->code   = 413;
                $result->info   = $this->translate('Failed saving request. You have an unapproved request, or request on the same date already approved!');
                
				$responseResult = array(
					'code' => $result->code,
					'info' => $result->info,
					'data' => $result->data
				); 

				$logparam['subject'] = 'ATTENDANCE REQUEST SAVEACTION';
				$logparam['identity'] = '';
				$logparam['user'] = '';
				$logparam['description'] = 'Log proses attendance request saveaction';
				$logparam['request'] = json_encode($requestPost);
				$logparam['response'] = json_encode($responseResult);

				$this->writeLogAction($logparam);
				
				return $this->getREST()->getResponse($result);
            }
        }

        $creq->request_id = $post['id'];
        $creq->ref_code = $post['ref_code'];
        $creq->ref_id = $sessmgt['ref_id'];
        $creq->start_dtm = $post['start_dtm'];
        $creq->end_dtm = $post['end_dtm'];
        $creq->request_type = $post['request_type'];
        $creq->request_dtm = date("Y-m-d H:i:s");
        $creq->status_id = 1;
        $creq->request_message = $post['request_message'];

        $employee       = new Employee(Employee\Storage::factory($this->getDb(), $this->getConfig()));
        $requesterMap   = $employee->loadMap($ref_id, 'person_id');

        $retval = $creq->save($update);
        $request_id = $creq->request_id;

        $targetId = null;
        if ($retval) {
            if ($post['request_type'] == 3) {
                if (isset($post['swap']) && $post['swap'] == "on") {
                    $dayno       = $cts->getDayNumber(date('Y-m-d', strtotime($post['change_date'])))->data;
                    $datachshift = $cts->loadShiftObject($post['target_id'], null);

                    if ($datachshift) {
                        foreach ($datachshift as $val) {
                            if ($val['day_no'] == $dayno) {
                                $targetId = $val['timetable_shift_id'];
                            }
                        }
                    }

                    $chshift['tt_employee_shift_id'] = null;
                    $chshift['ref_code'] = $post['ref_code'];
                    $chshift['ref_id'] = $post['target_id'];
                    $chshift['shift_change_date'] = date('Y-m-d', strtotime($post['start_dtm']));
                    $chshift['default_shift_id'] = $targetId;
                    $chshift['change_shift_id'] =  $cektimetable['data']['timetable_shift_id'];
                    $chshift['recommendation_id'] = $request_id;
                    $chshift['change_by'] = $sessmgt['ref_id'];
                } else {
                    $targetId = $post['target_id'];

                    $chshift['tt_employee_shift_id']    = null;
                    $chshift['ref_code']                = $post['ref_code'];
                    $chshift['ref_id']                  = $sessmgt['ref_id'];
                    $chshift['shift_change_date']       = date('Y-m-d', strtotime($post['start_dtm']));
                    $chshift['default_shift_id']        = $cektimetable['data']['timetable_shift_id'];
                    $chshift['change_shift_id']         = $targetId;
                    $chshift['recommendation_id']       = $request_id;
                    $chshift['change_by']               = $sessmgt['ref_id'];
                }
                $changeShift                        = new Changeshift(Changeshift\Storage::factory($this->getDb(), $this->getConfig()));
                $saveChShift                        = $changeShift->saveEx($chshift);

                if ($saveChShift->code !== 0) {
                    $creq->delete($request_id);
                    return $this->getREST()->getResponse(new Result(0, 511, $this->translate('Failed saving change shift. request deleted.')));
                }
            }

            /* End Request Overtime */

            /* Saving Approval */
            $approval = $post['approval_ref_id'];
            $savedNotif = [];

            if ($approval) {
                foreach ($approval as $appr) {
                    $data = array();
                    $data['approval_id'] = null;
                    $data['request_id'] = $request_id;
                    $data['ref_code'] = "EMPLOYEE";
                    $data['ref_id'] = $appr;
                    $data['status_id'] = 1;
                    $data['approval_dtm'] = date("Y-m-d H:i:s");
                    $data['reason'] = null;

                    $saveAppr = $creq->saveApproval($data, false);

                    if (!$saveAppr) {
                        $result     = new Result();
                        $result->code = 414;
                        $result->info = 'Failed saving approval';
                        return $this->getREST()->getResponse($result);
                    }

                    $empMap     = $employee->loadMap($appr, 'person_id');
                    $userId     = $empMap['data']['user_id'];

                    $inboxStorage   = Notification\Inbox\Storage::factory(
                        $this->getDb(),
                        $this->getConfig(),
                        Notification\Profile::TYPE_FCM,
                        $userId
                    );
                    $inbox      = new Notification\Inbox($inboxStorage);
                    $inbox->setPublicUrl($this->getPublicUrl());

                    $requestTypeName            = "";
                    switch ($post['request_type']) {
                        case self::REQUEST_TYPE_ATTENDANCE:
                            $requestTypeName    = 'Attendance';
                            break;
                        case self::REQUEST_TYPE_OVERTIME:
                            $requestTypeName    = 'Overtime';
                            break;
                        case self::REQUEST_TYPE_CHANGESHIFT:
                            $requestTypeName    = "Change shift";
                            break;
                        default:
                            $requestTypeName    = "Attendance";
                            break;
                    }

                    /* Prepare dulu barangkali belum ada tablenya... */
                    $result = @$inbox->pushNotification(
                        $userId,
                        'REQUEST_NEED_APPROVAL',
                        array(
                            'name' => $requesterMap['data']['first_name'] . ' ' . $requesterMap['data']['last_name'],
                            'request_type' => $requestTypeName,
                        ),
                        array(
                            'request_id'    => $request_id,
                            'approval_id'   => $saveAppr->data
                        )
                    );

                    if ($result->code == 0) {
                        $savedNotif[]   = $userId;
                    }
                }
            }
        }
        $result         = new Result();
        $result->data   = $savedNotif;
		
		$responseResult = array(
			'code' => $result->code,
			'info' => $result->info,
			'data' => $result->data
		); 

		$logparam['subject'] = 'ATTENDANCE REQUEST SAVEACTION';
		$logparam['identity'] = '';
		$logparam['user'] = '';
		$logparam['description'] = 'Log proses attendance request saveaction';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode($responseResult);

		$this->writeLogAction($logparam);
		
        return $this->getREST()->getResponse($result);
    }

    public function CancelRequestAction()
    {
        $request = $this->getRequest();
        $params = $request->getPost();

        $storage = \Neuron\Attendance\Request\Storage::factory($this->getDb(), $this->getConfig());
        $modelAttendance = new \Neuron\Attendance\Request($storage);
        $data = $modelAttendance->loadRequest('request_id', $params['request_id']);

        foreach ($data as $row) {
            $attendance = $row;
            $attendance['status_id'] = 4;

            $model = new \Neuron\Attendance\Request($storage, $attendance);
            $attendance_id = $model->save(true);
        }

        $response = $this->getResponse();
        $response->getHeaders()->addHeaderLine('Content-Type', 'application/json');
        $response->setContent(json_encode($attendance_id));
        return $response;
    }

    public function deleteAction()
    {
        $storage = \Neuron\Attendance\Request\Storage::factory($this->getDb(), $this->getConfig());
        $creq = new \Neuron\Attendance\Request($storage);

        $post = $this->getPostParams();
        $delapp = $creq->deleteApprovalBy('request_id', $post['id']);
        if ($delapp) {
            $delreq = $creq->delete($post['id']);
            $success = false;
            if ($delreq) $success = true;
        } else {
            $success = false;
        }

        echo json_encode(array('success' => $success));
        return $this->getResponse();
    }

    public function loadApprovalAction()
    {
        $storage = \Neuron\Attendance\Request\Storage::factory($this->getDb(), $this->getConfig());
        $creq = new \Neuron\Attendance\Request($storage);

        $post = $this->getPostParams();
        $arrAppr = array();
        $dataAppr = $creq->loadApprovalBy('request_id', $post['id']);
        if ($dataAppr) {
            foreach ($dataAppr as $val) {
                array_push($arrAppr, $val);
            }
        }
        echo json_encode(array('success' => true, 'data' => $arrAppr));
        return $this->getResponse();
    }

    public function ListStatusApprovalAction()
    {
        $storage = \Neuron\Attendance\Request\Storage::factory($this->getDb(), $this->getConfig());
        $creq = new \Neuron\Attendance\Request($storage);

        $post = $this->getPostParams();
        $fromApi = $this->getDataFromApi('emp_all');
        $dataEmployee = $fromApi;
        $dataAppr = $creq->loadApprovalBy('request_id', $post['request_id']);
        $arrAppr = array();
        if ($dataAppr) {
            foreach ($dataAppr as $val) {
                $val['ref_name'] = '';
                if ($dataEmployee) {
                    foreach ($dataEmployee as $emp) {
                        if ($val['ref_id'] == $emp->id) {
                            $val['ref_name'] = $emp->first_name . ' ' . $emp->last_name;
                        }
                    }
                }
                array_push($arrAppr, $val);
            }
        }

        $retval = array(
            'draw' => $post['draw'],
            'recordsTotal' => count($arrAppr),
            'recordsFiltered' => count($arrAppr),
            'data' => $arrAppr,
        );

        echo json_encode($retval);
        return $this->getResponse();
    }

    public function deleteApprovalAction()
    {
        $storage = \Neuron\Attendance\Request\Storage::factory($this->getDb(), $this->getConfig());
        $creq = new \Neuron\Attendance\Request($storage);

        $post = $this->getPostParams();
        $delapp = $creq->deleteApprovalBy('approval_id', $post['id']);

        $success = false;
        if ($delapp) $success = true;

        echo json_encode(array('success' => $success));
        return $this->getResponse();
    }

    public function loadParamSettingAction()
    {
        $storage = \Neuron\Attendance\Request\Storage::factory($this->getDb(), $this->getConfig());
        $creq = new \Neuron\Attendance\Request($storage);

        $post = $this->getPostParams();
        $arrType = array();
        $dataType = $creq->loadApprovalType();
        if ($dataType) {
            foreach ($dataType as $val) {
                array_push($arrType, $val);
            }
        }

        $arrSettting = array();
        $dataSetting = $creq->loadSetting();
        if ($dataSetting) {
            foreach ($dataSetting as $val) {
                $val['id'] = $val['setting_id'];
                array_push($arrSettting, $val);
            }
        }

        echo json_encode(array('success' => true, 'listType' => $arrType, 'listSetting' => $arrSettting));
        return $this->getResponse();
    }
    public function saveSettingAction()
    {
        $storage = \Neuron\Attendance\Request\Storage::factory($this->getDb(), $this->getConfig());
        $creq = new \Neuron\Attendance\Request($storage);

        $post = $this->getPostParams();
        $retval = $creq->saveSetting($post);
        $success = false;
        if ($retval) $success = true;
        echo json_encode(array('success' => $success));
        return $this->getResponse();
    }
   
    public function loadShiftAction()
    {
        $storage = \Neuron\Attendance\Shift\Storage::factory($this->getDb(), $this->getConfig());
        $cs = new \Neuron\Attendance\Shift($storage);

        $post = $this->getPostParams();
        $dayno = $cs->getDayNumber($post['date'])->data;
        $sessmgt = $this->getSession()->get();
        $owntimetable = array();
        $arrShift = array();

        if ($post['swap'] == 'true') {
            $dataShift = $cs->loadShiftObject(null, date('Y-m-d', strtotime($post['date'])));
        } else {
            $dataShift = $cs->loadShiftByDate(date('Y-m-d', strtotime($post['date'])), $dayno);
        }

        if (class_exists('\Neuron\Employee\Entity')) {
            $map        = $this->getMapPerson();
            $ref_id     = $map['ref_id'];
        } else {
            $ref_id     = $sessmgt['ref_id'];
        }

        $own = $cs->loadShiftObject($ref_id, date('Y-m-d', strtotime($post['date'])));
        if ($own) {
            foreach ($own as $val) {
                array_push($owntimetable, $val['timetable_id']);
            }
        }

        if ($dataShift) {
            foreach ($dataShift as $val) {
                if ($val['day_no'] == $dayno) {
                    if ($post['swap'] == 'true') {
                        if ($val['ref_id'] != $ref_id && !in_array($val['timetable_id'], $owntimetable)) {
                            array_push($arrShift, $val);
                        }
                    } else {
                        if (!in_array($val['timetable_id'], $owntimetable)) {
                            array_push($arrShift, $val);
                        }
                    }
                }
            }
        }

        $result = array('success' => true, 'data' => $arrShift);
        if ($post['swap'] == 'true') {
            $result = $this->GetRefName($result);
        }

        echo json_encode($result);
        return $this->getResponse();
    }
    public function listApprovalAction()
    {
        $postData = $this->getParameters();
        $session = $this->getSession();
        if ($postData) {
            $result = new \Neuron\Generic\Result();

            $username = isset($postData['username']) ? $postData['username'] : null;
            $token = isset($postData['token']) ? $postData['token'] : "";
            $page = isset($postData['page']) ? $postData['page'] : 1;
            $rowCount = isset($postData['row_count']) ? $postData['row_count'] : 10;
            try {
                if ($token == "") {
                    $result->code = self::CODE_FIELD_REQUIRED;
                    $result->info = 'token is null';
                    return $this->getREST()->getResponse($result);
                }

                $userPerson = $this->getUserPerson($username);

                $session->owner($userPerson['user']['user_id']);

                $sessid = $session->getId();
                if ($token != $sessid) {
                    $result->code = self::CODE_UNAUTHORIZED;
                    $result->info = 'Failed authenticate';
                    return $this->getREST()->getResponse($result);
                }

                $islogin = $this->isLoggedIn();
                if (!$islogin) {
                    $result->code = self::CODE_SESSION_EXPIRED;
                    $result->info = 'Session Expired';
                    return $this->getREST()->getResponse($result);
                }
                $storage = \Neuron\Attendance\Request\Storage::factory($this->getDb(), $this->getConfig());
                $catt = new \Neuron\Attendance\Request($storage);

                $filter = array('searchField' => 'ref_id', 'searchValue' => '', 'ref_id' => $userPerson['person']['ref_id']);
                $data = $catt->listApproval($page, $rowCount, $filter, $userPerson['person']['ref_id']);
                $result->guid = self::SUCCESS_CODE;
                $result->code = self::SUCCESS_CODE;
                $result->info = 'OK';
                $result->data = $data['rows'];
                return $this->getREST()->getResponse($result);
            } catch (Exception $e) {
                $result->code = self::SUCCESS_CODE;
                $result->info = 'error';
                $result->data = array(
                    'message' => $ex->getMessage(),
                );
                return $this->getREST()->getResponse($result);
            }
        } else {
            $storage = \Neuron\Attendance\Request\Storage::factory($this->getDb(), $this->getConfig());
            $catt = new \Neuron\Attendance\Request($storage);

            $post = $this->getPostParams();
            $sessmgt = $this->getSession()->get();

            $page = $post['start'];
            $rowCount = $post['length'];
            $post['ftrsearch'] = isset($post['ftrsearch']) ? $post['ftrsearch'] : 'ref_id';
            $filter = $post['search']['value'];
            $result = $catt->listApproval($page, $rowCount, $filter, @$sessmgt['ref_id'], $post);

            $rows = array();
            if ($result['rows']) {
                foreach ($result['rows'] as $row) {
                    if ($row['status_id'] != 4) {
                        $rows[]     = $row;
                    }
                }
            }

            $output = array(
                'draw' => $post['draw'],
                'recordsTotal' => intval($result['total']),
                'recordsFiltered' => intval($result['total']),
                'data' => $rows
            );

            $output = $this->GetRefName($output);

            return $this->getOutput(json_encode($output));
        }
    }
    public function testAction()
    {
        $output = $this->CheckUpdatedStatusRequest();

        $response = $this->getResponse();
        $response->getHeaders()->addHeaderLine('Content-Type', 'application/json');
        $response->setContent(json_encode($output));
        return $response;
    }
    private function CheckUpdatedStatusRequest($params = false)
    {
        $storageAtt = \Neuron\Attendance\Attendance\Storage::factory($this->getDb(), $this->getConfig());
        $catt = new \Neuron\Attendance\Attendance($storageAtt);

        $storageAtt = \Neuron\Attendance\Shift\Storage::factory($this->getDb(), $this->getConfig());
        $cs = new \Neuron\Attendance\Shift($storageAtt);

        $return = false;
        $storage = \Neuron\Attendance\Request\Storage::factory($this->getDb(), $this->getConfig());
        $model = new \Neuron\Attendance\Request($storage);
        $result = $model->listRequest($params);
        $data = $result['data'];

        $a = 0;
        $r = 0;
        $output = "";

        if ($data != null) {
            foreach ($data as $row) {
                if ($row['total_approve'] >= $row['min_approved']) {
                    $inputParams = array(
                        'request_id' => $row['request_id'],
                        'request_type' => $row['request_type'],
                        'request_dtm' => $row['request_dtm'],
                        'ref_code' => $row["ref_code"],
                        'ref_id' => $row["ref_id"],
                        'start_dtm' => date('Y-m-d H:i:s', strtotime($row['start_dtm'])),
                        'end_dtm' => date('Y-m-d H:i:s', strtotime($row['end_dtm'])),
                        'request_message' => $row['request_message'],
                        'status_id' => 2,
                    );

                    $model = new \Neuron\Attendance\Request($storage, $inputParams);
                    $retval = $model->save(true);

                    if ($retval) {
                        $a++;

                        if ($row['request_type'] == 3) {
                            // change shift
                            $chShift = $cs->loadChangeShift('request_id', $row['request_id']);
                            if ($chShift) {
                                foreach ($chShift as $value) {
                                    $valShift =  $cs->load($value['change_shift_id']);
                                    $oldShift = $cs->loadObjectTimetable($value['ref_id'], $value['shift_change_date'], $row['request_type']);
                                    if ($oldShift['success']) {
                                        if ($oldShift['data']['ato_end_date'] > $value['shift_change_date']) {
                                            $updData['start_date'] = date('Y-m-d', strtotime($oldShift['data']['ato_start_date']));
                                            $updData['end_date'] = date('Y-m-d', strtotime('-1 day', strtotime($value['shift_change_date'])));
                                            $updData['timetable_object_id'] = $oldShift['data']['timetable_object_id'];
                                            $shift1 = $cs->updateShiftFromChange($updData);

                                            //add new shift object
                                            $new1['timetable_shift_id'] = $value['change_shift_id'];
                                            $new1['timetable_id'] = $valShift['data']['timetable_id'];
                                            $new1['ref_code'] = $value['ref_code'];
                                            $new1['ref_id'] = $value['ref_id'];
                                            $new1['ref_name'] = $oldShift['data']['ref_name'];
                                            $new1['start_date'] = date('Y-m-d', strtotime($value['shift_change_date']));
                                            $new1['end_date'] = date('Y-m-d', strtotime($value['shift_change_date']));
                                            $new1['update_by'] = $value['change_by'];
                                            // print_r($new1); die();
                                            $savenew1 = $cs->saveObject($new1, false, false);

                                            //add new2 shift object
                                            $new2['timetable_shift_id'] = $oldShift['data']['timetable_shift_id'];
                                            $new2['timetable_id'] = $oldShift['data']['timetable_id'];
                                            $new2['ref_code'] = $oldShift['data']['ref_code'];
                                            $new2['ref_id'] = $oldShift['data']['ref_id'];
                                            $new2['ref_name'] = $oldShift['data']['ref_name'];
                                            $new2['start_date'] = date('Y-m-d', strtotime('+1 day', strtotime($value['shift_change_date'])));
                                            $new2['end_date'] = date('Y-m-d', strtotime($oldShift['data']['ato_end_date']));
                                            $new2['update_by'] = $value['change_by'];
                                            // print_r($new2);die();
                                            $savenew2 = $cs->saveObject($new2, false, false);
                                        } else if ($oldShift['data']['ato_end_date'] = $value['shift_change_date']) {
                                            $updData['start_date'] = date('Y-m-d', strtotime($oldShift['data']['ato_start_date']));
                                            $updData['end_date'] = date('Y-m-d', strtotime('-1 day', strtotime($value['shift_change_date'])));
                                            $updData['timetable_object_id'] = $oldShift['data']['timetable_object_id'];
                                            $Shift1 = $cs->updateShiftFromChange($updData);
                                        } else {
                                            $new1['timetable_shift_id'] = $value['change_shift_id'];
                                            $new1['timetable_id'] = $valShift['data']['timetable_id'];
                                            $new1['ref_code'] = $value['ref_code'];
                                            $new1['ref_id'] = $value['ref_id'];
                                            $new1['ref_name'] = $oldShift['data']['ref_name'];
                                            $new1['start_date'] = date('Y-m-d', strtotime($value['shift_change_date']));
                                            $new1['end_date'] = date('ref_id-m-d', strtotime($value['shift_change_date']));
                                            $new1['update_by'] = $value['ref_id'];
                                            $savenew1 = $cs->saveObject($new1, false, false);
                                        }
                                    }
                                }
                            }
                        } else {
                            if ($row['request_type'] == 2) {
                                // overtime
                                $catt->type_id = 2;
                                $catt->start_dtm = $row['start_dtm'];
                                $catt->end_dtm = $row['end_dtm'];
                            } elseif ($row['request_type'] == 1) {
                                // request type
                                $catt->type_id = 1;
                                $catt->end_dtm = $row['start_dtm'];
                            }

                            $catt->start_dtm = $row['start_dtm'];
                            $catt->source_dtm = $row['request_dtm'];
                            $catt->start_description = $row['request_message'];
                            $catt->end_description = $row['request_message'];
                            $catt->ref_code = $row['ref_code'];
                            $catt->ref_id = $row['ref_id'];
                            $catt->source_type_code = 'REQUEST';
                            $catt->source_reference = $row['request_id'];
                            $saveAttd = $catt->save(false);
                        }
                    }
                } elseif ($row['total_reject'] >= $row['min_rejected']) {
                    $inputParams = array(
                        'request_id' => $row['request_id'],
                        'request_type' => $row['request_type'],
                        'request_dtm' => $row['request_dtm'],
                        'ref_code' => $row["ref_code"],
                        'ref_id' => $row["ref_id"],
                        'start_dtm' => date('Y-m-d H:i:s', strtotime($row['start_dtm'])),
                        'end_dtm' => date('Y-m-d H:i:s', strtotime($row['end_dtm'])),
                        'request_message' => $row['request_message'],
                        'status_id' => 3,
                    );

                    $model = new \Neuron\Attendance\Request($storage, $inputParams);
                    $retval = $model->save(true);

                    if ($retval) {
                        $r++;
                    }
                }
            }

            if ($a > 0) {
                $output .= "[Total Approved = " . $a . "]";
            }
            if ($r > 0) {
                $output .= "[Total Rejected = " . $r . "]";
            }
        }

        return $output;
    }
    public function SettingAction()
    {
        $this->using('datatable', 30);
        $this->using('bootbox', 21);
        $this->using('p-loading', 22);
        $this->using('select2', 23);
        $this->using('select2-bootstrap', 24);
        $this->using('jquery.validate', 25);

        $sessmgt = $this->getSession()->get();
        $sess = (isset($sessmgt['ref_id'])) ? true : false;

        $title = $this->translate('Approval Setting');

        $layout = $this->layout();
        $this->getSession();
        $layout->setVariables(['current' => $this->translate('Approval Setting')]);
        $types  = new Setting\Types(Setting\Type\Storage::factory($this->getDb(), $this->getConfig()));
        $types  = $types->load()->data['rows'];
        return $this->getView('setting/list', get_defined_vars());
    }
    public function SettingListAction()
    {
        $request = $this->getRequest();
        $post = $request->getPost();

        $storage = \Neuron\Attendance\Request\Setting\Storage::factory($this->getDb(), $this->getConfig());
        $model = new \Neuron\Attendance\Request\Setting($storage);
        $result = $model->listdata($post);

        $response = $this->getResponse();
        $response->getHeaders()->addHeaderLine('Content-Type', 'application/json');
        $response->setContent(json_encode($result->data));
        return $response;
    }
    public function DeployedSettingAction()
    {
        $request = $this->getRequest();
        $params = $request->getPost();
        $storage = \Neuron\Attendance\Request\Setting\Storage::factory($this->getDb(), $this->getConfig());
        $model = new \Neuron\Attendance\Request\Setting($storage);
        $result = $model->CheckDeployedSetting($params);

        $response = $this->getResponse();
        $response->getHeaders()->addHeaderLine('Content-Type', 'application/json');
        $response->setContent(json_encode($result));
        return $response;
    }
    public function ApprovalTypeListAction()
    {

        $term = $this->params()->fromQuery('term');
        $storage = \Neuron\Attendance\Request\Setting\Detail\Storage::factory($this->getDb(), $this->getConfig());
        $approvalType = new \Neuron\Attendance\Request\Setting\Detail($storage);
        $result = $approvalType->approvalTypeDdl($term);

        $response = $this->getResponse();
        $response->getHeaders()->addHeaderLine('Content-Type', 'application/json');
        $response->setContent(json_encode($result));
        return $response;
    }
    public function IsNameExistAction()
    {
        $setting_name = $this->params()->fromPost('setting_name');
        $storage = \Neuron\Attendance\Request\Setting\Storage::factory($this->getDb(), $this->getConfig());

        $model = new \Neuron\Attendance\Request\Setting($storage);
        $result = $model->searchByName($setting_name);

        $return = ($result) ? false : true;

        $response = $this->getResponse();
        $response->getHeaders()->addHeaderLine('Content-Type', 'application/json');
        $response->setContent(json_encode($return));
        return $response;
    }
    public function LoadTypeListAction()
    {
        $request = $this->getRequest();
        $post = $request->getPost();

        $storage = \Neuron\Attendance\AttdRequestType\Storage::factory($this->getDb(), $this->getConfig());
        $model = new \Neuron\Attendance\AttdRequestType($storage);
        $result = $model->listdata($post);

        $response = $this->getResponse();
        $response->getHeaders()->addHeaderLine('Content-Type', 'application/json');
        $response->setContent(json_encode($result->data));
        return $response;
    }
    public function ChangeSettingAction()
    {
        $request = $this->getRequest();
        $params = $request->getPost();

        $storage = \Neuron\Attendance\AttdRequestType\Storage::factory($this->getDb(), $this->getConfig());
        $status = false;
        $retval = false;
        $output = [];

        if ($request->isPost()) {
            $update = (isset($params['type_id'])) ? true : false;

            $mType = new \Neuron\Attendance\AttdRequestType($storage);
            $data = $mType->searchType($params);

            $data['type_id'] = intval($data['type_id']);
            $data['setting_id'] = intval($params['setting_id']);

            $cexp = new \Neuron\Attendance\AttdRequestType($storage, $data);
            $retval = $cexp->save($update);

            $status = ($retval) ? true : false;
        }

        $response = $this->getResponse();
        $response->getHeaders()->addHeaderLine('Content-Type', 'application/json');
        $response->setContent(json_encode($retval, $status));
        return $response;
    }
    public function DeleteSettingAction()
    {
        $post = $this->getPostParams();

        $storage = \Neuron\Attendance\Request\Setting\Detail\Storage::factory($this->getDb(), $this->getConfig());
        $model = new \Neuron\Attendance\Request\Setting\Detail($storage);
        $model->delete($post['id']);

        $storage = \Neuron\Attendance\Request\Setting\Storage::factory($this->getDb(), $this->getConfig());
        $model = new \Neuron\Attendance\Request\Setting($storage);
        $retval = $model->delete($post['id']);

        $success = false;
        if ($retval) $success = true;

        $response = $this->getResponse();
        $response->getHeaders()->addHeaderLine('Content-Type', 'application/json');
        $response->setContent(json_encode($success));
        return $response;
    }
    public function GetNumApprovalAction()
    {
        $post = $this->getPostParams()->toArray();

        $storage    = Type\Storage::factory($this->getDb(), $this->getConfig());
        $type       = new Type($storage);
        $result     = $type->load($post['type_id']);

        $setting_id = $result->data['setting_id'];

        $storageDetail = Detail\Storage::factory($this->getDb(), $this->getConfig());
        $details    = new Details($storageDetail);
        $result  = $details->load($setting_id, Detail\Storage::LISTBY_SETTING_ID);

        return $this->getREST()->getResponse($result);
    }
    public function ListStatusAction()
    {
        $term = $this->params()->fromQuery('term');

        $storage = \Neuron\Attendance\Request\Storage::factory($this->getDb(), $this->getConfig());
        $model = new \Neuron\Attendance\Request($storage);
        $result = $model->liststatus($term);

        $response = $this->getResponse();
        $response->getHeaders()->addHeaderLine('Content-Type', 'application/json');
        $response->setContent(json_encode($result));
        return $response;
    }
    public function DeleteSettingDetailAction()
    {
        $storage = \Neuron\Attendance\Request\Setting\Detail\Storage::factory($this->getDb(), $this->getConfig());
        $csetdet = new \Neuron\Attendance\Request\Setting\Detail($storage);


        $post = $this->getPostParams();
        $dataToDel = $csetdet->loadDetail($post['id']);
        if ($dataToDel) {
            $retval = $csetdet->delete($post['id']);
            if (!$retval) {
                echo json_encode(array('success' => false, 'message' => 'Error on delete setting detail'));
                return $this->getResponse();
            }
            $dataup = $csetdet->loadSeqUp($dataToDel['setting_id'], $dataToDel['seq']);
            if ($dataup) {
                foreach ($dataup as $seqUp) {
                    $changeSeq = $csetdet->saveChange(array(
                        'detail_id' => $seqUp['detail_id'],
                        'setting_id' => $seqUp['setting_id'],
                        'seq' => $seqUp['seq'] - 1,
                    ));
                    if (!$changeSeq) {
                        echo json_encode(array('success' => false, 'message' => 'Error on reorder seq'));
                        return $this->getResponse();
                    }
                }
            }
        }

        $response = $this->getResponse();
        $response->getHeaders()->addHeaderLine('Content-Type', 'application/json');
        $response->setContent(json_encode(true));
        return $response;
    }
    public function loadListAction(){
        $post   = $this->getRequest()->getPost();
        $sessmgt    = $this->getSession()->get();

        $user_id    = $sessmgt['user_id'];
        $mapPerson  = $this->getMapPerson($user_id);
        $post['ref_id'] = $mapPerson['ref_id'];
        $requests   = new Requests(RequestM\Storage::factory($this->getDb(), $this->getConfig()));

        $rs         = $requests->load(null, RequestM\Storage::LISTBY_REFID, $post, RequestM\Storage::TIME_NONE);
        
        if (isset($post['draw'])) {

            if ($rs->code == 0) {
                $result     = array(
                    'draw'          => $post['draw'],
                    'recordsTotal'  => $rs->data['countTotal'],
                    'recordsFiltered' => $rs->data['countTotal'],
                    'data'  => $rs->data['rows']
                );
            } else {
                $result     = array(
                    'draw'          => $post['draw'],
                    'recordsTotal'  => 0,
                    'recordsFiltered' => 0,
                    'data'  => array()
                );
            }
            echo json_encode($result);
            return $this->getResponse();
        } else {
            return $this->getREST()->getResponse($rs);
        }
    }
    public function loadAction()
    {
        $result     = new \Neuron\Generic\Result;
        $post   = $this->getRequest()->getPost();
        if (!isset($post['request_id'])) {
            $result->code = 401;
            $result->info = 'invalid request id';
            return $this->getREST()->getResponse($result);
        }
        $request_id     = $post['request_id'];
        $requestStorage = RequestM\Storage::factory($this->getDb(), $this->getConfig());
        $request = new RequestM($requestStorage);
        $result = $request->load($request_id, RequestM\Storage::LOADBY_ID, array(), TRUE);

        return $this->getREST()->getResponse($result);
    }
    public function typeAction()
    {
        $type = new Types(Type\Storage::factory($this->getDb(), $this->getConfig()));
        $types = $type->load();
        return $this->getREST()->getResponse($types);
    }

    public function cancelAction(){
        $params     = $this->getREST()->getParameters();
        $attendanceRequest = new RequestM(RequestM\Storage::factory(
            $this->getDb(),
            $this->getConfig()
        ));
        $requestId  = $params['order_id'];
        $result     = $attendanceRequest->cancel($requestId);
        
        $result->info   = $this->translate($result->info);
        return $this->getREST()->getResponse($result);
    }

    private function getAddonConfig(){
        /* local config cache */
        $naf    = \Neuron\Application\Framework\Entity::get();
        $vendorDir  = $naf->base()->getVenDir();
        $file = join(DIRECTORY_SEPARATOR,[
            $vendorDir,
            'Addon',
            'Order',
            'SimpleFlow',
            'Config.php'
        ]);
        
        if(file_exists($file)){
            $config = new \Zend\Config\Config(include $file, true);
            if (isset($config->simpleflow)){
                $config = $config->simpleflow;
            }else {
                $config = new \Zend\Config\Config([], true);
            }
            $cfg = $config->toArray();
        } else {
            $cfg = [];
        }

        $base = $naf->base()->getModuleConfig();
        $_config = ArrayUtils::merge($cfg, $base);
        return $_config;
    }
}
