<?php
namespace Neuron\Attendance\Controller;

use Neuron\Attendance\Request\Setting as SettingModel;
use Neuron\Attendance\Request\Settings;
use Neuron\Generic\Result;

class Setting extends Controller{
    public function saveAction(){
        $result = new Result();
        $post   = $this->getRequest()->getPost()->toArray();
        $update = FALSE;
        $setting = new SettingModel(SettingModel\Storage::factory($this->getDb(),$this->getConfig()));
        if(isset($post['dataSetting']['setting_id']) && !empty($post['dataSetting']['setting_id'])){
            $update = TRUE;
        }


        // Check duplicate
        if(!$update){
            $loadSetting = $setting->load($post['dataSetting']['setting_name'],SettingModel\Storage::LOADBY_NAME,$post);
            if($loadSetting->code == 0){
                $result->code = 413;
                $result->info = "Duplicate setting name";
                return $this->getREST()->getResponse($result);
            }
        }

        // Validasi approval
        $dataSetting        = $post['dataSetting'];
        $numToApprove       = $dataSetting['num_to_approve'];
        $min_approved       = $dataSetting['min_approved'];
        $min_rejected       = $dataSetting['min_rejected'];
        if($min_approved > $numToApprove){
            $result->code = 413;
            $result->info = "Minimal approve should be equal to or less than Total Approval";
            return $this->getREST()->getResponse($result);
        }
        if($min_rejected > $numToApprove){
            $result->code = 413;
            $result->info = "Minimal reject should be equal to or less than Total Approval";
            return $this->getREST()->getResponse($result);
        }
        
        $setting->push($post['dataSetting']);
        if($update){
            $settingDetail = new SettingModel\Detail(SettingModel\Detail\Storage::factory($this->getDb(),$this->getConfig()));
            $settingDetail->delete($setting->setting_id,SettingModel\Detail\Storage::DELETEBY_SETTING_ID);
        }
        $savedDetailID = array();
        if($setting->save($update)){
            $setting_id = $setting->setting_id;
            $seq = 0;
            foreach ($post['dataDetail'] as $sd) {
                $seq++;
                $settingDetail = new SettingModel\Detail(SettingModel\Detail\Storage::factory($this->getDb(),$this->getConfig()));
                $settingDetail->setting_id          = $setting_id;
                $settingDetail->seq                 = $seq;
                $settingDetail->approval_type_id    = $sd['approval_type_id'];
                $settingDetail->type_value          = isset($sd['type_value']) ? $sd['type_value'] : null;
                if($settingDetail->save(FALSE)){
                    $savedDetailID[]    = $settingDetail->setting_id;
                }
            }
            $result->data = array(
                'setting_id' => $setting->setting_id,
                'savedDetail' => $savedDetailID
            );
            $result->info = "Success saving setting";
        }else{
            $result->code = 401;
            $result->info = "Failed saving setting";
        }

        return $this->getREST()->getResponse($result);
    }
    public function removeAction(){
        $post       = $this->getREST()->getParameters();
        if(!$post){
            $post   = $this->getRequest()->getPost();
        }
        
        $setting    = new SettingModel(SettingModel\Storage::factory($this->getDb(),$this->getConfig()));
        $result     = $setting->remove($post['setting_id']);

        return $this->getREST()->getResponse($result);
    }
    public function loadListAction(){
        $post       = $this->getREST()->getParameters();
        if (!$post) {
            $post   = $this->getRequest()->getPost();
        }

        $settings   = new Settings(SettingModel\Storage::factory($this->getDb(),$this->getConfig()));
        $result     = $settings->load(null,SettingModel\Storage::LIST_ALL,$post);

        if(isset($post['draw'])){
            $result     = array(
                'draw' => $post['draw'],
                'data' => $result->data->rows,
                'recordsTotal' => $result->data->countTotal,
                'recordsFiltered' => $result->data->countTotal
            );
            return $this->getOutput(json_encode($result));
            return $this->getResponse();
        }else{
            return $this->getREST()->getResult($result);
        }
        return false;
    }
    public function loadAction(){
        $post   = $this->getRequest()->getPost()->toArray();
        if(isset($post['setting_id'])){
            $id = $post['setting_id'];
            $mode = SettingModel\Storage::LOADBY_ID;
        }
        $setting    = new SettingModel(SettingModel\Storage::factory($this->getDb(),$this->getConfig()));
        $result    = $setting->load($id,$mode,$post);

        return $this->getREST()->getResponse($result);
    }
}
