<?php
namespace Neuron\Attendance\Controller;

use Neuron\Attendance\Shift as ShiftModel;
use Neuron\Attendance\Timetable;

class Shift extends Controller{

	public function loadListAction(){

        try {
            /* get post */
            $post = $this->getRequest()->getPost();

            /* bootgrid or API */
            if(isset($post['rowCount']) && isset($post['current'])){
                $params = array(
                    'page' => $post['current'],
                    'records' => $post['rowCount'],
                    'sort' => @$post['sort'],
                );

                $keyword = @$post['searchPhrase'];
                $mode = ShiftModel\Storage::LISTBY_PAGE;

                $bootgrid = true;
            }else{
                $post = $this->getREST()->getParameters();
                $params = $post;
                $keyword = '';
                $mode = $this->getCode() ? $this->getCode() : null;
                $bootgrid = false;
            }

			$storage = ShiftModel\Storage::factory($this->getDb(),$this->GetConfig());
            $objects= new \Neuron\Attendance\Shifts($storage);

            $result = $objects->load($keyword, $mode, $params);

            /* bootgrid? */
            if ($bootgrid) {
                if ($result->code == $result::CODE_SUCCESS) {

                     // calc total 
                    $total = $result->data['countTotal'];;
                    $pager = $result->data['pager'];

                    $listData['rows'] = $result->data['list'];
                    $listData['total'] = $total;
                    $listData['current'] = intval($params['page']);
                    $listData['rowCount'] = $pager->getLimit(); //$roles->count();

                } else {

                    $page = intval($params['page']);
                    $records = intval($params['records']);

                    $listData['rows'] = array();
                    $listData['total'] = $page * $records;
                    $listData['current'] = $page;
                    $listData['rowCount'] = $records;
                }
                return $this->getOutput(json_encode($listData));
                
            }   
            
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            $result = new \Neuron\Generic\Result();
            $result->code = 500;
            $result->info = $ex->getMessage();
            return $this->getREST()->getResponse($result);
        }
        
        return $this->getREST()->getResponse($result);        
    }

    public function saveAction(){
        $post = $this->getPostParams()->toArray();

		$post['start_date'] = date('Y-m-d', strtotime($post['start_date']));
		if(strlen($post['end_date']) <= 0){
			$post['end_date'] = null;
		}else{
			$post['end_date'] = date('Y-m-d', strtotime($post['end_date']));
		}
		
		$requestPost = array(
			'guid' => '',
			'code' => '',
			'info' => '',
			'data' => $post
		); 
		
		$timetable = new \Neuron\Attendance\Timetable(\Neuron\Attendance\Timetable\Storage::factory($this->getDb(),$this->Getconfig()));
		$shift = new \Neuron\Attendance\Shift(\Neuron\Attendance\Shift\Storage::factory($this->getDb(),$this->Getconfig()));

		$dataDuplicate = array();

		$dayNo = $post['day_no'];	
		foreach ($dayNo as $dayNo) {
			if(isset($post['timetable_id']) && !empty($post['timetable_id'])){
				$update = true;
			}else{
				$update = false;
			}

			if(!$update){
				$existcode = $timetable->searchByCode($post['code'].'_'.$dayNo);
				if($existcode){
					echo json_encode(array('success'=>false, 'message'=> 'Code already exists', 'dataDuplicate'=> $dataDuplicate));
					return $this->getResponse();
                }
                
                /* Cari apakah pada parent_id yang mau di tambahkan, sudah ada harinya? */
                if(isset($post['parent_timetable_id']) && !empty($post['parent_timetable_id'])){
                    $parentId   = $post['parent_timetable_id'];
                    $timetable = new \Neuron\Attendance\Timetable(\Neuron\Attendance\Timetable\Storage::factory($this->getDb(),$this->Getconfig()));
                    $timetable->load($parentId,Timetable\Storage::LOADBY_PARENT_DAYNO,array('day_no' => $dayNo));
                    if($timetable->timetable_id){
                        echo json_encode(array('success'=>false, 'message'=> $this->translate("Failed saving data. Data with the same day already exist"), 'dataDuplicate'=> $timetable->pull()));
                        return $this->getResponse();
                    }
                }


			}else{
				$timetable->load($post['timetable_id']);
				if($post['code'].'_'.$dayNo != $timetable->code){
					$existcode = $timetable->searchByCode($post['code'].'_'.$dayNo);
					if($existcode){
						echo json_encode(array('success'=>false, 'message'=> 'Code already exists', 'dataDuplicate'=> $dataDuplicate));
						return $this->getResponse();
					}
				}
			}

			$timetable_code = $post['code'].'_'.$dayNo;

			$dataTimetable = array(
				'timetable_id' => $post['timetable_id'],
				'code' => $timetable_code,
				'parent_timetable_id' => $post['parent_timetable_id'],
				'time_mode' => $post['time_mode'],
				'name' => $post['name'],
			);
			$timetable->clear();
			$timetable->push($dataTimetable);

			$timetable_id = $timetable->save($update);

			// save timetable shift
			$dataShift = array(
				'timetable_shift_id' => $post['timetable_shift_id'],
				'timetable_id' => $timetable_id,
				'day_no' => $dayNo,
				'is_workday' => $post['is_workday'],
				'on_duty_time' => $post['on_duty_time'],
				'off_duty_time' => $post['off_duty_time'],
				'in_beginning_time' => $post['in_beginning_time'],
				'in_ending_time' => $post['in_ending_time'],
				'out_beginning_time' => $post['out_beginning_time'],
				'out_ending_time' => $post['out_ending_time'],
				'late_time' => $post['late_time'],
				'leave_early_time' => $post['leave_early_time'],
				'overtime_start_time' => $post['overtime_start_time'],
				'start_date' => $post['start_date'],
				'end_date' => $post['end_date']
			);
			$shift->clear();
			$shift->push($dataShift);
			$saveShift = $shift->save($update);
        }
		
		$logparam['subject'] = 'ATTENDANCE SHIFT SAVEACTION';
		$logparam['identity'] = '';
		$logparam['user'] = '';
		$logparam['description'] = 'Log proses attendance shift saveaction';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode(array('success'=>true, 'message'=> 'shift saved', 'dataDuplicate'=> $dataDuplicate));

		$this->writeLogAction($logparam);
		
        $result =  json_encode(array('success'=>true, 'message'=> 'shift saved', 'dataDuplicate'=> $dataDuplicate));
        return $this->getOutput($result);
	}

    public function getOutput($json) {

        $response = $this->getResponse();
        $response->getHeaders()->addHeaderLine('Content-Type', 'application/json');
        $response->setContent($json);
        return $response;
    }

    public function fetchActiveShiftAction(){
        $result = new \Neuron\Generic\Result;
        $post   = $this->getRequest()->getPost();
        $shift  = new ShiftModel(ShiftModel\Storage::factory($this->getDb(),$this->getConfig()));

        $session    = $this->getSession()->get();
        if(!isset($session['ref_id'])){
            $result->code = 411;
            $result->info = "Invalid ref_id";
            return $this->getREST()->getResponse($result);
        }
        $ref_id     = intval($session['ref_id']);

        $data   = $shift->loadObjectTimetable($ref_id,date('Y-m-d',strtotime($post['start_dtm'])),$post['req_type']);

        if(array_key_exists('data',$data)  && count($data['data']) > 0){
            $result->data   = $data['data'];
        }else{
            $result->code = 411;
            $result->info = "Shift not found";
        }

        return $this->getREST()->getResponse($result);
    }

    
}
