<?php
namespace Neuron\Attendance\Controller;

use Neuron\Attendance\Test as TestModel;
use Neuron\Application\Framework as NAF;
use Neuron\Generic\Result;
use Neuron\Attendance\Entity;

class Test extends Controller{
    protected function getView($name, $variables = array()) {

		/* default layout & view prefix */
		$layout = 'neuron/attendance/layout/form';
        $prefix = 'neuron/attendance/test';
        
		/* libraries */;
        $this->using('font-awesome', 10, 'https://use.fontawesome.com/releases/v5.8.1/css/all.css');

        $this->using('select2', 22, array(
            'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.10/js/select2.min.js',
             /* ini load sesuai bahasa dari session */
            'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.10/js/i18n/id.js',
             /**/
            'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.10/css/select2.min.css',
            'https://cdn.jsdelivr.net/npm/@ttskch/select2-bootstrap4-theme@1.3.2/dist/select2-bootstrap4.min.css',
        ));

        $this->using('toast');
	    $this->using('generic');
		$this->using('fontawesome', null);
		
		/* get layout n view prefix from config */
		$config = $this->getConfig();
		if (isset($config['attendance'])) {
			$config = $config['attendance'];
			$layout = isset($config['formlayout']) ? $config['formlayout'] : $layout;
			$prefix = isset($config['viewprefix']) ? $config['viewprefix'] : $prefix;
		}

		/* set layout and view */
		$this->layout($layout);
		$view = new \Zend\View\Model\ViewModel($variables);
		$view->setTemplate($prefix . '/' . strtolower($name));

		return $view;
    }
    
    public function indexAction(){
        return $this->getView('index');
    }

    public function uploadAction(){
        NAF\Entity::publish();
        \Neuron\Application\Framework\Entity::publish();
        $post       = $this->getRequest()->getFiles()->toArray();
        
        /* Load file excelnya */
        $spreadsheet 	= \PhpOffice\PhpSpreadsheet\IOFactory::load($post['excel']['tmp_name']);
        
        // $rows  = yang menyimpen row dari spreadsheets
        $rows 			= [];
        $rowIterator 	= $spreadsheet->getActiveSheet()->getRowIterator();
        // Isi $rows dengan rows dari spreadsheet yang di upload, karena dari spreadsheet bentuknya ga langsung array
        foreach($rowIterator as $row){
            $cellIterator = $row->getCellIterator();
            foreach ($cellIterator as $cell) {
                if($cell->getCalculatedValue()){
                    $rows[$row->getRowIndex()][$cell->getColumn()] = $cell->getFormattedValue();
                }
            }
        }
        unset($rows[1]);
        $rows 			= array_values($rows);

        $testModel      = new TestModel($this->getDb(),$this->getConfig());
        $result         = $testModel->executeRow($rows);


        $naf            = NAF\Entity::get();
        $logPath        = join(DIRECTORY_SEPARATOR,array(
            $naf->base()->getRunDir(),
            'temp/attendance-test-log.txt'
        ));
        $file = fopen($logPath,'a');
        fwrite($file,'=========== '.date('Y-m-d H:i:s')."=============\n");
        fwrite($file,print_r($result,true));
        fwrite($file,"=========== END LOGGING ============\n");
        fclose($file);
        return $this->getREST()->getResponse($result);
        
    }

    public function clearAction(){
        /* $hardCoded password to clear data : */
        $deletePassword = "jangandihapus";
        $request        = $this->getRequest()->getPost();
        $password       = $request->password;
        if($password === $deletePassword){
            $testModel  = new TestModel(
                $this->getDb(),
                $this->getConfig()
            );
            $result     = $testModel->clearData();
        }else{
            $result         = new Result();
            $result->code   = 1;
            $result->info   = "Invalid Password"; 
        }

        $result->info = $this->translate($result->info);
        return $this->getREST()->getResponse($result);
    }

    public function holidayAttendAction(){
        NAF\Entity::publish();

        $entity         = new Entity(Entity\Storage::factory(
            $this->getDb(),
            $this->getConfig()
        ));
        $entityData     = array(
            "type_id"               => Entity::TYPE_ATTENDANCE,
            "start_dtm"             => date('Y-m-d H:i:s',strtotime('2020-08-15 08:00:00')),
            'start_time'            => date('H:i:s',strtotime('08:00:00')),
            "start_description"     => "Test Attendance Hari Libur",
            "end_dtm"               => null,
            "end_description"       => null,
            "ref_code"              => "EMPLOYEE",
            "ref_id"                => 9,
            "source_type_code"      => "WEB",
            "source_reference"      => 1,
            "source_dtm"            => date('Y-m-d H:i:s'),
            "overtime_start_dtm"    => null,
            "overtime_end_dtm"      => null,
            "longitude"             => null,
            "latitude"              => null
        );

        $result    = $entity->saveEx($entityData);
        $result->info = $this->translate($result->info);
        return $this->getREST()->getResponse($result);
    }

    public function overtimeAction(){
        NAF\Entity::publish();
        $entity         = new Entity(Entity\Storage::factory(
            $this->getDb(),
            $this->getConfig()
        ));
        $dataOvertime    = array( 
            "type_id"               => Entity::TYPE_OVERTIME,
            "start_dtm"             => "2020-08-25 19:00:00",
            "request_message"       => "Just a test bro",     
            "end_dtm"               => "2020-08-24 20:00:00",
            "ref_code"              => "EMPLOYEE",
            "ref_id"                => 9,
            "source_reference"      => "REQUEST"
        );
        $result         = $entity->saveEx($dataOvertime);
        $result->info = $this->translate($result->info);
        return $this->getREST()->getResponse($result);
    }
}