<?php 
namespace Neuron\Attendance;

use Neuron\Generic\Entity\Map;
use Neuron\Generic\Result;

class Detail extends \Neuron\Generic\Entity\Mapper {
    protected $_storage;
    protected $_loaded = false;
    const CONFIG_CODE = 'ATTENDANCE';
    const PULL_ALL = 10;

	public function __construct(\Neuron\Attendance\Detail\Storage\Skeleton $storage, $data= array()) {
			
        $this->_storage = $storage;

		parent::__construct(
	        $data, 
	        array(
	            new Map('attd_detail_id'),
	            new Map('attendance_id'),
	            new Map('timetable_id'),
	            new Map('attd_in_out'),
                new Map('ip_address'), 
                new Map('geocode_location'),
                
	        ), 
	        parent::CASE_INSENSITIVE
	    );
    }

    public function load($id, $mode = \Neuron\Attendance\Detail\Storage::LOADBY_ID){
        /* load dari storage */
        if ($data = $this->_storage->load($id, $mode)) {

            /* load sukses, set data properti class dari hasil query */
            $this->push($data);

            /* set loaded */
            $this->_loaded = true;
            return true;

        } else {
            $this->_loaded = false;
        }
        return false;
    }
    public function create($data = array())
    {   
        $result = new Result();
        $update = false;
        if(!empty($data['attd_detail_id'])){
            $update = true;
        }
        $data['ip_address'] = $this->get_client_ip();

        $param = $this->pull();
        foreach($data as $k => $v){
            if(array_key_exists($k, $param)){
                $param[$k] = $v;
            }
        }
        $this->push($param);
        if($this->save($update)){
            $result->info = 'Success save detail';
            $result->data = $this->attd_detail_id;
        }else{
            $result->code  = self::CONFIG_CODE."111";
            $result->info = 'Failed, please check data';
            $result->data = null;
        }
        return $result;
    }

    public function terminate($id)
    {
        $result = new Result();
        if($this->_storage->terminate($id)){
            $result->info = 'Success delete detail';
            $result->data = $id;
        }else{
            $result->code  = self::CONFIG_CODE."112";
            $result->info = 'Failed delete detail';
            $result->data = null;
        }
        return $result;
    }
    
    public function save($update = false)
    {
        /* simpan via storage */
        if ($id = $this->_storage->save($this, $update)) {

            /* simpan sukses, ambil id yg tersimpan, return true */
            $this->attd_detail_id = $id;
            return true;
        }
        return false;
    }

    function get_client_ip() {
        $ipaddress = '';
        if (getenv('HTTP_CLIENT_IP'))
            $ipaddress = getenv('HTTP_CLIENT_IP');
        else if(getenv('HTTP_X_FORWARDED_FOR'))
            $ipaddress = getenv('HTTP_X_FORWARDED_FOR');
        else if(getenv('HTTP_X_FORWARDED'))
            $ipaddress = getenv('HTTP_X_FORWARDED');
        else if(getenv('HTTP_FORWARDED_FOR'))
            $ipaddress = getenv('HTTP_FORWARDED_FOR');
        else if(getenv('HTTP_FORWARDED'))
           $ipaddress = getenv('HTTP_FORWARDED');
        else if(getenv('REMOTE_ADDR'))
            $ipaddress = getenv('REMOTE_ADDR');
        else
            $ipaddress = 'UNKNOWN';
        return $ipaddress;
    }

}