<?php 

namespace Neuron\Attendance\Detail\Storage;
use Zend\Db\Sql\Where;
use Zend\Db\Sql\Predicate\Expression;

class Mysql extends \Neuron\Db\Storage implements Skeleton {
    const CONFIG_CODE = 'ATTENDANCE';

    public function __construct(\Zend\Db\Adapter\Adapter $adapter, $config = array()) {

        /* construct parent --> Neuron\Db\Storage */
        parent::__construct($adapter, $config);
    }

    public function load($id, $mode = \Neuron\Attendance\Detail\Storage::LOADBY_ID) {
        
        /** kalau modif fungsi ini, silakan modif juga versi oracle dan versi postgre nya yah*/
        
        try {
            $select = $this->select()
                ->from(array('a' => $this->_tables['attd_details']));
            if($mode == \Neuron\Attendance\Detail\Storage::LOADBY_ID){
                $select->where($this->__(array('a.attd_detail_id' => $id)));
            }else if($mode == \Neuron\Attendance\Detail\Storage::LOADBY_ATTENDANCE){
                $select->where($this->__(array('a.attendance_id' => $id)));
            } 
            
            return $this->fetchRow($select);
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }

    public function loadList($ids, $mode = \Neuron\Attendance\Detail\Storage::LIST_ALL, $params = null) {
        /* base select */
        $select = $this->select()
        ->from(array('a' => $this->_tables['attd_details']));

        /* loadby page */
        if ($mode == \Neuron\Attendance\Detail\Storage::LISTBY_PAGE) {
            /* has keyword? */
            if (!empty($ids)) {
                $select->where->nest()
                    ->like($this->_('description', 'a.'), '%' . trim($ids) . '%')
                ->unnest();
            }
            /* get pager */
            $pager = \Neuron\Db\Storage\Pager::sanitize($params);
            $select->limit($pager->getLimit())->offset($pager->getOffset())->cache(0);
            
        } else {
            
            $pager = new \Neuron\Db\Storage\Pager(0, 999999);
            switch ($mode) {
                case \Neuron\Attendance\Detail\Storage::LISTBY_TIMETABLE:
                    $select->where(array($this->_('timetable_id') => $ids));
                    break;
                case \Neuron\Attendance\Detail\Storage::LISTBY_ATTENDANCE_ID:
                    $select->where(array($this->_('attendance_id') => $ids));
                    break;
            }
        }
        
        try {
            if($rows = $this->fetchAll($select)){
                $result = new \stdClass();
                $result->list = $rows;
                $result->pager = $pager;
                $result->countTotal = $rows->total;
                return $result;
            }
            return false;
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }

    public function save(\Neuron\Attendance\Detail $period,$update = false)
    {
        try {
            /* mode update? */
            if ($update == true) {

                $update = $this->update()
                        ->table($this->_tables['attd_details'])
                        ->set($this->__($period->pull()))
                        ->where($this->__(array('attd_detail_id' => $period->attd_detail_id)));
                if ($this->execute($update)) {

                    /* update sukses, return user id */
                    return $period->attd_detail_id;
                }
            } else {
                $period->attd_detail_id = null ;
                $insert = $this->insert()
                        ->into($this->_tables['attd_details'])
                        ->autoincrement($this->_('attd_detail_id'))
                        ->values($this->__($period->pull()));
                
                if ($id = $this->execute($insert)) {

                    /* insert sukses, return user id */
                    return $id;
                }
            }

            /* default gagal */
            return false;
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }   
    }

    public function terminate($id)
    {
        try {
            $delete = $this->delete()
                ->from($this->_tables['attd_details'])
                ->where($this->__(array('attd_detail_id' => $id)));
                
            if($this->execute($delete)){
                return $id;
            }
            return false;
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }
}