<?php
namespace Neuron\Attendance;

use Neuron\Generic\Result;
use Neuron\Attendance\Entity;
use Neuron\Attendance\Entity\Storage;
use Neuron\Attendance\Shift;

class Entities extends \Neuron\Generic\Set{
    protected $_storage;
    public function __construct(\Neuron\Attendance\Entity\Storage\Skeleton $storage){
        $this->_storage     = $storage;
    }
    
    public function load($id = null, $mode = Storage::LIST_ALL,$params = array()){
        $type_id  = array_key_exists('type_id',$params) ? $params['type_id'] : 1;
        if($params['type_id'] == 9 || $params['type_id'] == 4 || $params['type_id'] == 7){
            $params['type_id'] = 1;
        }

        if($type_id == 9){
            $mode = Storage::LISTBY_EARLY;
        }
        
        $result     = new Result();
        $list   = array();
        if ($rows = $this->_storage->loadList($id, $mode, $params)) {
            $lists   = $rows->rows;
           
            $this->clear();
            /* Push items */
            foreach($lists as $row){
                $row = array_change_key_case($row);

                unset($row['b_rownum']);

                $entity     = new Entity($this->_storage);
                @$entity->push($row);
                $this->add($entity);

                $shift                  = new Shift(Shift\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));

                $params['start_date']     = date('Y-m-d',strtotime($row['start_dtm']));
                $params['start_dtm']      = date('Y-m-d H:i:s',strtotime($row['start_dtm']));
                $params['day_no']         = $shift->getDayNumber($params['start_date'])->data;
                $params['ref_id']         = $row['ref_id'];

                // Check apakah user memiliki shift yang aktif di tanggal yang di request
                $loadShift              = $shift->load(null,Shift\Storage::LOADBY_BETWEENDATE_REF_ID_DAYNO,$params);
                
                $start_time = date('H:i:s', strtotime($row['start_dtm']));
                $end_time = date('H:i:s', strtotime($row['end_dtm']));
                if($type_id == 4){
                    if($loadShift->code == 0){
                        $dutytime   = $loadShift->data['on_duty_time'];
                        $latetime   = $loadShift->data['late_time'];

                        $totalLate =  floor((strtotime($start_time)- strtotime($dutytime))/60)- $latetime;
                        if($totalLate != 0 && $totalLate > 0){
                            $row['total_late'] =  (floor((strtotime($start_time)- strtotime($dutytime))/60)- $latetime);
                            $list[]     = $row;
                        }
 
                    }

                }elseif($type_id == 9){
                    // early attendance
                    $dutytime   = $loadShift->data['on_duty_time'];
                    if(strtotime($start_time) < strtotime($dutytime)){
                        $list[]     = $row;
                    }

            

                }elseif ($type_id == 2){
                    if($loadShift->code == 0){
                        $dutytime = $loadShift->data['on_duty_time'];
                        $latetime = $loadShift->data['late_time'];
                        if((CEIL((strtotime($row['end_dtm'])- strtotime($row['start_dtm']))/3600)) > 0){
                            $row['total_overtime'] = (CEIL((strtotime($row['end_dtm'])- strtotime($row['start_dtm']))/3600));
                        }
                    }
                    $list[]     = $row;
                }else{
                    $list[]     = $row;
                }
            }
        }
        if (count($list) > 0) {
            $result->data = array(
                'list' => $list,
                'countTotal' => ($rows->countTotal)? $rows->countTotal : count($list),
                // 'pager' => $rows->pager
              );
          } else {
              $result->code = 1;
              $result->info = 'No data attendance found!';
          }

        
        return $result;
    }
    
}