<?php
namespace Neuron\Attendance;

use Neuron\Generic\Entity\Map;
use Neuron\Generic\Result;
use Neuron\Attendance\Timetable;
use Neuron\Attendance\Timetable\ObjectTimetable;
use Neuron\Attendance\Timetable\Objects;
use Neuron\Attendance\Shift;
use Neuron\Application\Framework as NAF;
use Neuron\Person\Entities;
use Neuron\Person\Entity as PersonEntity;
use Neuron\Employee\Entity as EmpEntity;
use Neuron\Employee\Entities as EmpEntities;
use Neuron\Calendar\Event;
use Neuron\Calendar\Events;
use Neuron\Approval\Subtypes as ApprovalSubtypes;
use Neuron\Attendance\Traits\Mapper;
use Neuron\Employee\Map as EmployeeMap;
use Neuron\Employee\Detail;
use Neuron\Attendance\Traits\Model;
// use Neuron\Approval\Entities as Approval;
use Neuron\Order\Subtype as OrderSubtype;

class Entity extends \Neuron\Generic\Entity\Mapper{
    use Model,Mapper;

    protected $_storage;
    protected $_loaded      = FALSE;
    const TYPE_ATTENDANCE   = 1;
    const TYPE_OVERTIME     = 2;
    const TYPE_NOINF        = 3;
    const TYPE_ATTENDANCE_OUT_YESTERDAY   = 4;
    
    CONST TEST_ENVIRONMENT  = FALSE;
    const HOLIDAY           = 0;
    const WORKDAY           = 1;
    const ERROR_PREFIX      = 'ATTENDANCE_ENTITY-';

    const ATTEND_TYPE_IN                = 0;
    const ATTEND_TYPE_OUT               = 1;
    CONST ATTEND_TYPE_NOT_IN_RANGE_IN   = 2;
    CONST ATTEND_TYPE_NOT_IN_RANGE_OUT  = 3;
    CONST ATTEND_TYPE_ALREADY_IN        = 4;
    CONST ATTEND_TYPE_ALREADY_OUT       = 5;



    public function __construct(\Neuron\Attendance\Entity\Storage\Skeleton $storage,$data = array()){
        $this->_storage     = $storage;
        parent::__construct(
            $data,
            array(
                new Map('id'),
                new Map('type_id'),
                new Map('start_dtm'),
                new Map('start_description'),
                new Map('end_dtm'),
                new Map('end_description'),
                new Map('ref_code'),
                new Map('ref_id'),
                new Map('source_type_code'),
                new Map('source_reference'),
                new Map('source_dtm'),
                new Map('overtime_start_dtm'),
                new Map('overtime_end_dtm'),
                new Map('longitude'),
                new Map('latitude'),
                new Map("day_no",null,true),
                new Map('start_date',null,true),
                new Map('start_time',null,true),
                new Map('geocode_location',null,true)
            ),
            parent::CASE_INSENSITIVE
        );
    }

    public function employee(){
        $employee   = new \Neuron\Employee\Entity(\Neuron\Employee\Entity\Storage::factory($this->getDb(),$this->getConfig()));
        $employee->load($this->ref_id);
        return $employee;
    }

    public function load($id = null, $mode = Entity\Storage::LOADBY_ID,$params = array()){
        $res        = $this->_storage->load($id,$mode,$params);
        if($res){
            $this->push($res);
            $this->_loaded = TRUE;
            return true;
        }
        return false;
    }

    public function save($update = FALSE){
        if($id = $this->_storage->save($this,$update)){
            $this->id = $id;
            return true;
        }else{
            return false;
        }
    }

    public function updateAttdOutKemarin($data){
        $shift                  = new Shift(Shift\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));

        $data['start_date']     = date('Y-m-d',strtotime($data['start_dtm']));
        $data['start_dtm']      = date('Y-m-d H:i:s',strtotime($data['start_dtm']));
        $data['day_no']         = $shift->getDayNumber($data['start_date'])->data;

        $date  = new \DateTime($data['start_date']);
        $interval = new \DateInterval('P1D');
        $date->sub($interval); 
        
        $dataKemarin['start_date'] = $date->format('Y-m-d');
        $dataKemarin['day_no'] = $shift->getDayNumber($date->format('Y-m-d'))->data;
        $dataKemarin['ref_id'] = $data['ref_id'];
        $dataKemarin['ref_code'] = $data['ref_code'];
        $dataKemarin['source_type_code'] = $data['source_type_code'];
        $dataKemarin['source_reference'] = $data['source_reference'];
        $dataKemarin['source_dtm'] = $data['source_dtm'];
        
        // Check apakah user memiliki shift yang aktif di tanggal yang di request kemarin
        $loadShift              = $shift->load(null,Shift\Storage::LOADBY_BETWEENDATE_REF_ID_DAYNO,$dataKemarin);

        // $shiftData  = $loadShift->data;
        if($loadShift->code == 0){
            $shiftData  = $loadShift->data;

            $mulai = new \DateTime(date('H:i:s', strtotime($shiftData['on_duty_time'])));
            $akhir = new \DateTime(date('H:i:s', strtotime($shiftData['off_duty_time'])));

            if($shiftData['is_workday'] == self::WORKDAY && $mulai > $akhir){
                $this->load(
                    null,Entity\Storage::LOADBY_ID,
                    array(
                        'ref_id'        => $data['ref_id'],
                        'start_date'    => date("Y-m-d",strtotime($dataKemarin['start_date'])),
                        'type_id'       => array($data['type_id'],3),
                    )
                );

                //cek data attendance hari itu, jika ada
                if($this->_loaded){
                    $now    = $data['start_time'];
                    $timeOut= $shift->isWithinTimeRange($now, $shiftData['out_beginning_time'], $shiftData['out_ending_time']);

                    if($timeOut){
                        $dataKemarin['id'] = $this->id;
                        $dataKemarin['type_id'] = self::TYPE_ATTENDANCE_OUT_YESTERDAY;
                        $dataKemarin['start_dtm'] = $this->start_dtm;
                        $dataKemarin['end_dtm'] = $data['start_dtm'];
                        $dataKemarin['end_description'] = "Leaving... May tomorrow be better!";
                        $dataKemarin['start_description'] = $this->start_description ?? "Start attendance and ready to work!";
                        $dataKemarin['overtime_start_dtm'] = $this->overtime_start_dtm;
                        $dataKemarin['overtime_end_dtm'] = $this->overtime_end_dtm;
                        
                        return $dataKemarin;
                    }
                }
            }
        }
        
        return false;
    }

    public function saveEx($data = array()){
        $naf            = \Neuron\Application\Framework\Entity::get();
        $session        = $naf->session();
        $lang           = $session->get('lang');
         //translator
         $translator     = new \Zend\I18n\Translator\Translator();
         if(file_exists(__DIR__ .'/Resource/Language/'.$lang.'.php')){
             $translator->addTranslationFile("phparray",__DIR__ .'/Resource/Language/'.$lang.'.php');
         }

        $result         = new \Neuron\Generic\Result;

        if(!isset($data['start_dtm']) || strlen($data['start_dtm']) <= 0 ){
            return new Result(0,411,'Start date time Required!');
        }
        if(!isset($data['ref_id']) || strlen($data['ref_id']) <= 0 ){
            return new Result(0,412,'ref_id required!');
        }
        if(!isset($data['type_id'])){
            return new Result(0,413,'Type ID Required');
        }
        if(!isset($data['source_reference'])){
            return new Result(0,414,'source_reference Required');
        }
        if(!isset($data['source_dtm'])){
            $data['source_dtm'] = date("Y-m-d H:i:s");
        }


        // Check hari libur calendar, 
        // dilakukan pengecekan kalo typenya attendance
        if($data['type_id'] == self::TYPE_ATTENDANCE){

            if(class_exists('Neuron\Calendar\Event')){
                $events             = new Events(Event\Storage::factory(
                    $this->_storage->getAdapter(),
                    $this->_storage->getConfig()
                ));
            }

            // Load hari liburnya
            $loadEvents   = $events->load(
                self::HOLIDAY, # 0 = hari libur
                Event\Storage::LISTBY_WORKDAY,
                array(
                    'start_date'    => date('Y-m-d',strtotime($data['start_dtm'])),
                    'end_date'      => date('Y-m-d',strtotime($data['start_dtm'])),
                    'date_between'  => true
                )
            );
            if($loadEvents->code == 0){
                // Kalo ada hari liburnya langsung aja cancel
                $listEvents     = $loadEvents->data['list'];
                if(count($listEvents) > 0){
                    return new Result(0,415,"Cannot attend at holiday.");
                }
            }
        }

        $fileevidence = isset($data['file_evidence'])? $data['file_evidence']:null;
        $geocode_location = (isset($data['geocode_location']) && !empty($data['geocode_location']) && $data['geocode_location'] != '')? $data['geocode_location']:null;
		unset($data['file_evidence']);
        unset($data['geocode_location']);
        
        $shift                  = new Shift(Shift\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));

        $data['start_date']     = date('Y-m-d',strtotime($data['start_dtm']));
        $data['start_dtm']      = date('Y-m-d H:i:s',strtotime($data['start_dtm']));
        $data['day_no']         = $shift->getDayNumber($data['start_date'])->data;
        
        // Check apakah user memiliki shift yang aktif di tanggal yang di request
        $loadShift              = $shift->load(null,Shift\Storage::LOADBY_BETWEENDATE_REF_ID_DAYNO,$data);
        
        // Validasi, kalo typenya = attendance & workday, maka gabisa attend
        $isForYesterday = false;
        if($data['type_id'] == self::TYPE_ATTENDANCE){
            $shiftData  = $loadShift->data;
            $checkYesterdayNight = $this->updateAttdOutKemarin($data);
            if($shiftData['is_workday'] == self::HOLIDAY){
                if(!$checkYesterdayNight){//Check Kemarin apakah ada shift lintas hari
                    return new Result(0,"415-2","Cannot attend at holiday.");
                }
                $isForYesterday = true;
                $data = $checkYesterdayNight;
            }
        }
        
        if($loadShift->code != 0 && !$isForYesterday) return $loadShift;
        
        if($data['type_id'] === self::TYPE_ATTENDANCE){
            if(isset($data['start_time'])){
                $now    = $data['start_time'];
            }else{
                $now    = date("H:i:s");
            }

            // Check if already attended today
            $this->load(
                null,Entity\Storage::LOADBY_ID,
                array(
                    'ref_id'        => $data['ref_id'],
                    'start_date'    => date("Y-m-d",strtotime($data['start_dtm'])),
                    'type_id'       => array($data['type_id'],3),
                )
            );
            //cek data attendance hari itu, jika ada
            if($this->_loaded){
                /**
                 * jika data yang di load ada, dan type nya != 3 dan waktu berada diantara jam pulang, 
                 * maka dimasukan ke absen pulang (blok if,yang $timeOut true)
                 * 
                 * Jika data yang diload ada, dan type nya != 3, dan waktu berada diantara jam masuk,
                 * maka akan dibandingkan, jika waktu data yang terbaru lebih rendah, maka akan merubah 
                 * data attendance yang sebelumnya 
                 * 
                 * tapi kalo data yang di load  type_id nya == 3, maka dianggap absen masuk, rubah datanya
                 *
                 * Jika diabsenkan oleh ADMIN dan employee sudah melakukan absen sebelumnya maka, proses absen dibatalkan
                */ 

                if(isset($data['source_type_code'])){
                    if($data['source_type_code'] == 'ADMIN') {
                        return new Result(0,401,$translator->translate('This employee has done absent today'));
                    } else {

                        if($this->type_id != 3){
                            // menentukan, ini request untuk jam pulang atau jam keluar?
                            $timeOut            = $shift->isWithinTimeRange($now,$shift->out_beginning_time,$shift->out_ending_time);
                            $timeIn             = $shift->isWithinTimeRange($now,$shift->in_beginning_time,$shift->in_ending_time);
                            
                            $data['id'] = $this->id;
                            if($timeOut){
                                
                                //cek apakah sudah absen keluar atau belum
                                if($this->end_dtm){                  
                                    return new Result(0,402,$translator->translate('You already attended out at ').$this->end_dtm);
                                }
                                
                                // jika ada Check apakah sudah waktu pulang atau belum
                                $isWithinRange          = $shift->isWithinTimeRange($now,$shift->out_beginning_time,$shift->out_ending_time);
                                if(!$isWithinRange){
                                    return new Result(
                                        0,
                                        402,
                                        $translator->translate('You cannot attendance out at this time : ').date('d M Y H:m:s',strtotime($this->start_dtm))
                                    );
                                }
                                
                                $data['id'] = $this->id;
                                $data['end_dtm'] = $data['start_dtm'];
                                $data['start_dtm'] = $this->start_dtm;
                                $data['end_description'] = (isset($data['start_description']) && !empty($data['start_description']) ) ? $data['start_description'] : "Leaving... May tomorrow be better!";
                                $data['start_description'] = $this->start_description;
                                $data['overtime_start_dtm'] = $this->overtime_start_dtm;
                                $data['overtime_end_dtm'] = $this->overtime_end_dtm;
                            }else if($timeIn){
                                
                                if($data['source_reference']){
                                    $currentStartDtm    = $this->start_dtm;
                                    $submittedStartDtm  = $data['start_dtm'];
                                    /**
                                     * Jika waktu mulai data yang disubmit < data yang tersimpan
                                     * maka update datanya */
                                    if(strtotime($submittedStartDtm) < strtotime($currentStartDtm)){
                                        $data['start_dtm']  = $submittedStartDtm;
                                        $data['start_description'] = $data['start_description'] ?? "Start attendance and ready to work!";
                                    }
                                }else{
                                    return new Result(0,414,'You already attended at '.date('d M Y H:i:s',strtotime($this->start_dtm)));
                                }
                            }else{
                                //Ini utk attd dluar range tidak di save jadinya
                                return new Result(0,404,$translator->translate('Time is out of range'));
                            }
                        }else{
                            
                            $data['id'] = $this->id;
                            $data['start_dtm'] = $data['start_dtm'];
                            $data['end_dtm'] = $this->end_dtm;
                            $data['start_description'] = $data['start_description'] ?? "Start attendance and ready to work!";
                            $data['end_description'] = $this->start_description;
                            $data['overtime_start_dtm'] = $this->overtime_start_dtm;
                            $data['overtime_end_dtm'] = $this->overtime_end_dtm;
                        }
                        $update = true;
                    }
                } else {
                    return new Result(0,404,$translator->translate('Source_type_code not found'));
                }
                
            }else{
                // Check apakah waktu yang dipilih berada diantara jam masuk & jam keluar
                $isWithinRange          = $shift->isWithinTimeRange($now,$shift->in_beginning_time,$shift->in_ending_time);
                $data['start_description'] = $data['start_description'] ?? "Start attendance and ready to work!";
                
                if(!$isWithinRange){
                    return new Result(0,401,$translator->translate('Your Attendance time is invalid. Your current active shift start at ').$shift->in_beginning_time.$translator->translate(' and end at ').$shift->in_ending_time);
                }
                $update = false;
            }
            
            $this->clear();
            $this->push($data);

            $id = $this->save($update);
            
            if($id){
                if($fileevidence){
                    $dataAttendace = $this->pull();
                    $dataAttendace['file'] = $fileevidence;
                    $dataAttendace['id'] = $this->id;
                    $dataAttendace['action_type'] = 1;
                    $dataAttendace['action_type_name'] = 'attendance';
                    $storageEvidence = \Neuron\Attendance\Evidence\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig());
                    $modelEvidence  = new \Neuron\Attendance\Evidence($storageEvidence);
                    $modelEvidence->create($dataAttendace);
                }
                
                $dataDetail['attendance_id'] = $this->id;
                $dataDetail['timetable_id'] = $loadShift->data['timetable_id'];
                $dataDetail['geocode_location'] = $geocode_location;
                if($update){
                    $dataDetail['attd_in_out'] = 2; 
                }else{
                    $dataDetail['attd_in_out'] = 1; 
                }
                
                $storageDetail = \Neuron\Attendance\Detail\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig());
                $modelDetail  = new \Neuron\Attendance\Detail($storageDetail);
                $saveDetail = $modelDetail->create($dataDetail);
                
                if($update && $timeOut){
                    $result->info = 'Success saving attendance out';
                }else{
                    $result->info = 'Success saving attendance in';
                }

                //insert timeline
                if(class_exists('\Neuron\Addon\Employee\Timeline\Entity')){
                    if($update){
                        $time = $this->end_dtm;
                        $title = 'Mulai Pulang';
                        $summary = $data['end_description'];
                    }else{
                        $time = $this->end_dtm;
                        $title = 'Mulai masuk';
                        $summary = $data['start_description'];
                    }

                    
                    // $requesterMap['data'] = array_merge($employeeMap->employee()->pull(), $employeeMap->pull());
                    
                    // $employee 	= new \Neuron\Employee\Entity(\Neuron\Employee\Entity\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
                    // $loadMap 	= $employee->loadMap($this->ref_id,'person_id');

                    $employeeMap    = new \Neuron\Employee\Map(\Neuron\Employee\Map\Storage::factory($this->getDb(), $this->getConfig()));
                    $employeeMap->load($this->ref_id,\Neuron\Employee\Map\Storage::LOADBY_EMPID);

                    $userId 	= null;
                    if($employeeMap->user_id){
                        $userId 	= $employeeMap->user_id;
                    }

                    $timelineStorage  = \Neuron\Feed\Entity\Storage::factory($this->_storage->getAdapter(), $this->_storage->getConfig());
                    $timeline = new \Neuron\Addon\Employee\Timeline\Entity($timelineStorage);
                    $timelineData = [
                        "feed_group_id" => 1,
                        "title"         => $title,
                        "summary"       => $summary,
                        "thumbnail"     => null,
                        "url"           => 'attendance/attendance/save',
                        "keywords"      => null,
                        "publish_dtm"   => $time,
                        "reference_code"   => 'USER',
                        "reference_id"   => $userId,
                    ];
                    $timeline->create($timelineData);
                }
            }else{
                $result->code = 500;
                if($update){
                    $result->info = "Failed saving attendance out";
                }else{
                    $result->info = "Failed saving attendance in";
                }
            }


        }else if($data['type_id'] === self::TYPE_OVERTIME){

            if(strtotime($data['end_dtm']) < strtotime($data['start_dtm'])){
                $result->code 					= 502;
                $result->info 					= 'Overtime End Date time cannot lower than Start date time!';
                return $result;
            }

            $this->type_id 				    = 2;
            $this->start_dtm			 	= $data['start_dtm'];
            $this->end_dtm 				    = $data['end_dtm'];
            $this->overtime_start_dtm 	    = $data['start_dtm'];
            $this->overtime_end_dtm 		= $data['end_dtm'];
            $this->start_description 		= $data['order_desc'];
            $this->ref_code 				= "EMPLOYEE";
            $this->ref_id 				    = $data['ref_id'];
            $this->source_type_code 		= "REQUEST";
            $this->source_reference 		= $data['ref_id'];
            $this->source_dtm 			    = date('Y-m-d');

            $save 								= $this->save(FALSE);
            if(!$save){
                $result->code 					= 501;
                $result->info 					= 'Failed saving overtime';
            }else{
                //insert timeline
                if(class_exists('\Neuron\Addon\Employee\Timeline\Entity')){
                    
                    $time = $data['start_dtm'];
                    $title = 'Mulai Lembur';
                    $summary = $data['request_message'];

                    // $employee 	= new \Neuron\Employee\Entity(\Neuron\Employee\Entity\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
                    // $loadMap 	= $employee->loadMap($this->ref_id,'person_id');
                    
                    // $userId 	= null;
                    // if($loadMap['code'] == 200){
                    //     $map 		= $loadMap['data'];
                    //     $userId 	= $map['user_id'];
                    // }
                    
                    $employeeMap    = new \Neuron\Employee\Map(\Neuron\Employee\Map\Storage::factory($this->getDb(), $this->getConfig()));
                    $employeeMap->load($this->ref_id,\Neuron\Employee\Map\Storage::LOADBY_EMPID);

                    $userId 	= null;
                    if($employeeMap->user_id){
                        $userId 	= $employeeMap->user_id;
                    }

                    $timelineStorage  = \Neuron\Feed\Entity\Storage::factory($this->_storage->getAdapter(), $this->_storage->getConfig());
                    $timeline = new \Neuron\Addon\Employee\Timeline\Entity($timelineStorage);
                    $timelineData = [
                        "feed_group_id" => 1,
                        "title"         => $title,
                        "summary"       => $summary,
                        "thumbnail"     => null,
                        "url"           => 'attendance/attendance/save',
                        "keywords"      => null,
                        "publish_dtm"   => $time,
                        "reference_code"   => 'USER',
                        "reference_id"   => $userId,
                    ];
                    $timeline->create($timelineData);
                }
                
                $result->data 					= $this->id;
                $result->info                   = "Success saving overtime";
            }
        }else if($data['type_id'] === self::TYPE_ATTENDANCE_OUT_YESTERDAY){
            $data['type_id'] = self::TYPE_ATTENDANCE;
            $update = true;
            $this->clear();
            $this->push($data);

            $id = $this->save($update);
            
            if($id){
                if($fileevidence){
                    $dataAttendace = $this->pull();
                    $dataAttendace['file'] = $fileevidence;
                    $dataAttendace['id'] = $this->id;
                    $dataAttendace['action_type'] = 1;
                    $dataAttendace['action_type_name'] = 'attendance';
                    $storageEvidence = \Neuron\Attendance\Evidence\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig());
                    $modelEvidence  = new \Neuron\Attendance\Evidence($storageEvidence);
                    $modelEvidence->create($dataAttendace);
                }
                
                $dataDetail['attendance_id'] = $this->id;
                $dataDetail['timetable_id'] = $loadShift->data['timetable_id'];
                $dataDetail['geocode_location'] = $geocode_location;
                $dataDetail['attd_in_out'] = 2; 
                
                $storageDetail = \Neuron\Attendance\Detail\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig());
                $modelDetail  = new \Neuron\Attendance\Detail($storageDetail);
                $saveDetail = $modelDetail->create($dataDetail);
                
                $result->info = 'Success saving attendance out';

                //insert timeline
                if(class_exists('\Neuron\Addon\Employee\Timeline\Entity')){
                    $time = $this->end_dtm;
                    $title = 'Mulai Pulang';
                    $summary = $data['end_description'];

                    $employeeMap    = new \Neuron\Employee\Map(\Neuron\Employee\Map\Storage::factory($this->getDb(), $this->getConfig()));
                    $employeeMap->load($this->ref_id,\Neuron\Employee\Map\Storage::LOADBY_EMPID);

                    $userId     = null;
                    if($employeeMap->user_id){
                        $userId     = $employeeMap->user_id;
                    }

                    $timelineStorage  = \Neuron\Feed\Entity\Storage::factory($this->_storage->getAdapter(), $this->_storage->getConfig());
                    $timeline = new \Neuron\Addon\Employee\Timeline\Entity($timelineStorage);
                    $timelineData = [
                        "feed_group_id" => 1,
                        "title"         => $title,
                        "summary"       => $summary,
                        "thumbnail"     => null,
                        "url"           => 'attendance/attendance/save',
                        "keywords"      => null,
                        "publish_dtm"   => $time,
                        "reference_code"   => 'USER',
                        "reference_id"   => $userId,
                    ];
                    $timeline->create($timelineData);
                }
            }else{
                $result->code = 500;
                $result->info = "Failed saving attendance out";
            }
        }

        return $result;
    }

    public function locationList(){
        if(class_exists('Neuron\Organization\Controller\Organization')){
            $organizations      = new \Neuron\Organization\Entities(
                \Neuron\Organization\Entity\Storage::factory(
                    $this->_storage->getAdapter(),$this->_storage->getConfig()
                )
            );
            $result            = $organizations->load(null, \Neuron\Organization\Entity\Storage::LIST_ROOT_PARENT);
            if($result->code == 0){
                $orgList        = $result->data['list'];
                
                $addresses                  = array();
    
                foreach($orgList as $org){
                    $contactEntities    = new \Neuron\Contact\Entities(
                        \Neuron\Contact\Entity\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig())
                    );
                    
                    $listContact        = $contactEntities->listContact(0,99999,array(
                        'ref_code'      => 'ORGANIZATION',
                        'searchField'   => 'ref_id',
                        'searchValue'   => $org['id']
                    ));
                    if($listContact['rows']){
                        $address              = $this->findOrgAddress($listContact['rows'], $org['name']);
                        if($address){
                            // $address['org_name']    = $org['name'];
                            // $addresses[] = $address;
                            $addresses = array_merge($addresses, $address);
                        }
                    }
                }
                $result->data   = $addresses;
                return $result;
            }else{
                return new Result(0,self::ERROR_PREFIX.'1',"Failed loading organiaztion list");
            }
        }else{
            return new Result(0,self::ERROR_PREFIX.'1',"Failed loading organiaztion list");
        }
       
    }

    private function findOrgAddress($listContact = array(), $org_name = null){
        $retval = array();
        foreach($listContact as $contact){
            // cari di detail yang type nya 1
            $contactDetail  = new \Neuron\Contact\Detail(
                \Neuron\Contact\Detail\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig())
            );

            $details = $contactDetail->load($contact['id']);
            if($details){
                foreach($details as $detail){
                    if($detail['type_id'] == 1){
                        if($org_name != null){
                            $detail['org_name'] = $org_name;
                        }
                        array_push($retval, $detail);
                        // return $detail;
                    }
                }
            }
        }
        return $retval;

    }

    public function todayStatus($post = array()){
        $result         = new Result();
        
        // Ambil dulu list employeenya 
        $personEntities     = new Entities(PersonEntity\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));

        $post['draw']   = 1;
        $post['order'][0]['column'] = 0;
        $post['order'][0]['dir']    = 'asc';
        $post['columns'][0]['data'] = 'id';
        $post['columns'][0]['name'] = 'id';
        $post['start']              = 0;
        $post['length']             = 10;
        $personList             = $personEntities->load(null,PersonEntity\Storage::LIST_ALL,$post);
        if($personList->code !== 0){
            return $personList;
        }
        $persons                = $personList->data['rows'];
        foreach($persons as $key => $person){
            $searchParams               = array();
            $searchParams['ref_id']     = $person['id'];
            $searchParams['start_date'] = date("Y-m-d");
            if($this->load(null,Entity\Storage::LOADBY_ID,$searchParams)){
                $persons[$key]['today_status']  = 1;
            }else{
                $persons[$key]['today_status']  = 0;
            }
        }
        $result                 = new Result();
        $result->data           = $persons;
        return $result;
    }

    public function countByType($type = 1,$startDate,$endDate,$refId = null){
        $getTotal   = $this->_storage->countTotalByType($type,$startDate,$endDate,$refId);
        return $getTotal;
    }

    public function todayStatusEmployees($date,$name,$org_id,$status,$sort,$length,$offset){
        $result     = new \Neuron\Generic\Result;
        if(!class_exists('Neuron\Employee\Entity')){
            $result->info   = "No employee module ";
            $result->code   = 1;
            return $result;
        }

        $personEntities           = new \Neuron\Person\Entities(\Neuron\Person\Entity\Storage::factory(
            $this->_storage->getAdapter(),
            $this->_storage->getConfig()
        ));
		
		$empEntities           	  = new \Neuron\Employee\Entities(\Neuron\Employee\Entity\Storage::factory(
            $this->_storage->getAdapter(),
            $this->_storage->getConfig()
        ));
		
        $personStatusId     = 1; # Status ID Active saja
		$employeesParams    = [
			'status_id'     => $personStatusId,
			'draw'          => 1,
			'length'        => $length,
			'start'         => $offset,
			'columns'       => [
				0       => [
					'data'  => "id"
				],
				1       => [
					'data'  => "first_name"
				],
				2       => [
					'data'  => "start_dtm"
				],
				3       => [
					'data'  => "type_id"
				],
			],
			'organization_id' => $org_id,
			'start_date' => date('Y-m-d'),
		];
		
		if($sort == 'name'){ # Sorting berdasarkan nama							
			$page    	= ($employeesParams['start'] + $employeesParams['length']) / $employeesParams['length'];
			$rowCount   = $employeesParams['length'];
			$employeesParams['page'] = $page;
			$employeesParams['records'] = $rowCount;			
		}else if($sort == 'date'){ # Sorting berdasarkan tanggal
			$arrayOrder = array( 0           => [
									'column'    => 0,
									'dir'       => 'asc'
								]);
			$employeesParams['order'] = $arrayOrder;
		
			$arrayOrderDate = array( 0           => [
									'column'    => 2,
									'dir'       => 'desc'
								]);
			$employeesParams['orderDate'] = $arrayOrderDate;
		}else{
			$arrayOrder = array( 0           => [
									'column'    => 0,
									'dir'       => 'asc'
								]);
			$employeesParams['order'] = $arrayOrder;
			
			$arrayOrderDate = array( 0           => [
									'column'    => 2,
									'dir'       => 'asc'
								]);
			$employeesParams['orderDate'] = $arrayOrderDate;
			
			$arrayOrderType = array( 0           => [
									'column'    => 3,
									'dir'       => 'asc'
								]);
			$employeesParams['orderType'] = $arrayOrderType;
		}
		
		if(empty($name) && empty($org_id) && empty($sort) && ($status == 0)){
			/*Load all data employee sort by name*/
			$page    	= ($employeesParams['start'] + $employeesParams['length']) / $employeesParams['length'];
			$rowCount   = $employeesParams['length'];
			$employeesParams['page'] = $page;
			$employeesParams['records'] = $rowCount;
			
			if($manager = NAF\Event\Manager::get()) {
				$manager->events()->register('employee_before_loadlist',array($this,'eventListenerOrderEmployee'),0,1);
			}
			
			$load = $empEntities->load($name, \Neuron\Employee\Entity\Storage::LISTBY_PAGE, $employeesParams);
			
			if($load->code != 0){
				$result->info   = "No employees";
				$result->code   = 2;
				return $result;
			}
			$employees      = $load->data['list'];
			$empIds         = [];
			$newEmployees   = [];				

			foreach($employees as $employee){
				$empIds[]   = $employee['id'];
				//load data employee
				$emp 		= new EmpEntity(EmpEntity\Storage::factory($this->getDb(),$this->getConfig()));
				$emp		->load($employee['id']);
				$dataEmp	= $emp->pull();
				
				//load data person
				$person 	= new PersonEntity(PersonEntity\Storage::factory($this->getDb(),$this->getConfig()));
				$person		->load($employee['person_id']);
				$dataPerson = $person->pull();

				//load data person detail
				$detail 	= new Detail(Detail\Storage::factory($this->getDb(),$this->getConfig()));
				$detail		->load($employee['id'],\Neuron\Employee\Detail\Storage::LOADBY_EMPID);
				$dataDetail =  $detail->pull();
				
				$dataEmployee = array_merge($dataPerson, $dataDetail, $dataEmp);
				
				if($dataEmployee['pic_official']){
					if(!file_exists(getcwd() .'/public'. $dataEmployee['pic_official'])){
						$dataEmployee['pic_official'] = '/asset/images/upload/avatar/user.png';
					}
				}else{
					$dataEmployee['pic_official'] = '/asset/images/upload/avatar/user.png';
				}

				$newEmployees[$employee['id']] = $dataEmployee;
				$employeesParams['id'] = $employee['id'];
			}   
			
			//print_r($newEmployees);die();
			
			$attendances    = $this->_storage->loadIn("ref_id",$empIds,$date);
			//print_r($attendances);
			$rows           = [];
			foreach($newEmployees as $employee){
				$searchAttendance   = $this->searchAttendance($attendances,$employee['id']);
					
				if(empty($searchAttendance) || $searchAttendance['type_id'] == 3){							
					$leave     = new \Neuron\Approval\Entities(\Neuron\Order\Entity\Storage::factory(
						$this->_storage->getAdapter(),
						$this->_storage->getConfig()
					));
					
					$personMap  = new \Neuron\Person\Map(\Neuron\Person\Map\Storage::factory(
						$this->_storage->getAdapter(),
						$this->_storage->getConfig()
					));
					$personMap->load($employee['id'],\Neuron\Person\Map\Storage::LOADBY_PERSON_ID);
					$userId 	= $personMap->user_id;
					
					$where['where']['create_user_id'] = $userId;
					$where['where']['order_type_id'] = 2200;
					$where['where']['order_status_id'] = 2;
					$where['where']['xn5'] = 2;
					$where['where']['sdate'] = $date;
					$where['where']['edate'] = $date;
					
					$searchAttendance = $leave->load(null, \Neuron\Order\Entity\Storage::LIST_ALL, $where, true);
					
					if($searchAttendance->code == 0){
						$searchAttendances = $searchAttendance->data;
						$searchAttendance = $this->searchLeave($searchAttendances,$userId['user_id']);
					}else{
						$searchAttendance = array();
					}
				}
				
				$rows[] = [
					'employees'     => $employee,
					'attendance'    => $searchAttendance
				];
			}
			
		}else{
			if($name){ # FILTER BY NAME
				$employeesParams['keyword'] = $name;
			
				$load = $empEntities->load($name, \Neuron\Employee\Entity\Storage::LISTBY_PAGE, $employeesParams);

				if($load->code != 0){
					$result->info   = "No employees";
					$result->code   = 2;
					return $result;
				}
				$employees      = $load->data['list'];
				$empIds         = [];
				$newEmployees   = [];

				foreach($employees as $employee){
					$empIds[]   = $employee['id'];
					//load data employee
					$emp 		= new EmpEntity(EmpEntity\Storage::factory($this->getDb(),$this->getConfig()));
					$emp		->load($employee['id']);
					$dataEmp	= $emp->pull();
					
					//load data person
					$person 	= new PersonEntity(PersonEntity\Storage::factory($this->getDb(),$this->getConfig()));
					$person		->load($dataEmp['person_id']);
					$dataPerson = $person->pull();

					//load data person detail
					$detail 	= new Detail(Detail\Storage::factory($this->getDb(),$this->getConfig()));
					$detail		->load($employee['id'],\Neuron\Employee\Detail\Storage::LOADBY_EMPID);
					$dataDetail =  $detail->pull();
					
					$dataEmployee = array_merge($dataPerson, $dataDetail, $dataEmp);
					
					if($dataEmployee['pic_official']){
						if(!file_exists(getcwd() .'/public'. $dataEmployee['pic_official'])){
							$dataEmployee['pic_official'] = '/asset/images/upload/avatar/user.png';
						}
					}else{
						$dataEmployee['pic_official'] = '/asset/images/upload/avatar/user.png';
					}
					
					$newEmployees[$dataEmployee['id']] = $dataEmployee;
					$employeesParams['id'] = $dataEmployee['id'];
				}
				
				$attendances    = $this->_storage->loadIn("ref_id",$empIds,$date);
				
				$rows           = [];
				foreach($newEmployees as $employee){
					$searchAttendance   = $this->searchAttendance($attendances,$employee['id']);

					if(empty($searchAttendance) || $searchAttendance['type_id'] == 3){				
						$leave     = new \Neuron\Approval\Entities(\Neuron\Order\Entity\Storage::factory(
							$this->_storage->getAdapter(),
							$this->_storage->getConfig()
						));
						
						$personMap  = new \Neuron\Person\Map(\Neuron\Person\Map\Storage::factory(
							$this->_storage->getAdapter(),
							$this->_storage->getConfig()
						));
						$personMap->load($employee['id'],\Neuron\Person\Map\Storage::LOADBY_PERSON_ID);
						$userId 	= $personMap->user_id;
						
						$where['where']['create_user_id'] = $userId;
						$where['where']['order_type_id'] = 2200;
						$where['where']['order_status_id'] = 2;
						$where['where']['xn5'] = 2;
						$where['where']['sdate'] = $date;
						$where['where']['edate'] = $date;
						
						$searchAttendance = $leave->load(null, \Neuron\Order\Entity\Storage::LIST_ALL, $where, true);
						
						if($searchAttendance->code == 0){
							$searchAttendance = $searchAttendance->data[0];
						}else{
							$searchAttendance = array();
						}
					}
					
					$rows[] = [
						'employees'     => $employee,
						'attendance'    => $searchAttendance
					];
				}
			}
			
			if($org_id){ # FILTER BY ORG
				$page    	= ($employeesParams['start'] + $employeesParams['length']) / $employeesParams['length'];
				$rowCount   = $employeesParams['length'];
				$employeesParams['page'] = $page;
				$employeesParams['records'] = $rowCount;	
			
				if($manager = NAF\Event\Manager::get()) {
					$manager->events()->register('employee_before_loadlist',function($select, $params){
						//echo '</pre>'; print_r($params); echo '</pre>';die();
						$_tables 	= $this->getConfig()['tables'];
						/* Join to person to get first_name*/
						$select->join(
							['ej' => $_tables['emp_jobs']],
							'ej.employee_id = e.person_id',
							[
								'ohp_id'
							],
							\Neuron\Db\Sql\Select::JOIN_RIGHT
						);
						/* Right Join ke OHP Supaya valid data ohpnya ada. */
						$select->join(
							['ohp' => $_tables['org_has_position']],
							'ohp.id = ej.ohp_id',
							[
								'position_id'
							],
							\Neuron\Db\Sql\Select::JOIN_RIGHT
						);
						
						$select->where([
							'ej.job_type_id' => 1
						]);
						
						$select->where([
							'ohp.organization_id' => $params['organization_id']
						]);
						
						$nest = $select->where->nest();
						$nest->isNull('ej.end_date')->or;
						$nest->greaterThan('ej.end_date',new \Zend\Db\Sql\Expression("NOW()"));
						$nest->unnest();
						
						//echo '</pre>'; print_r($select); echo '</pre>';die();
					},0,2);
				}
					
				$load = $empEntities->load($name, \Neuron\Employee\Entity\Storage::LISTBY_PAGE, $employeesParams);
				
				if($load->code != 0){
					$result->info   = "No employees";
					$result->code   = 2;
					return $result;
				}
				$employees      = $load->data['list'];
				$empIds         = [];
				$newEmployees   = [];

				foreach($employees as $employee){
					$empIds[]   = $employee['id'];
					//load data employee
					$emp 		= new EmpEntity(EmpEntity\Storage::factory($this->getDb(),$this->getConfig()));
					$emp		->load($employee['id']);
					$dataEmp	= $emp->pull();
					
					//load data person
					$person 	= new PersonEntity(PersonEntity\Storage::factory($this->getDb(),$this->getConfig()));
					$person		->load($employee['id']);
					$dataPerson = $person->pull();

					//load data person detail
					$detail 	= new Detail(Detail\Storage::factory($this->getDb(),$this->getConfig()));
					$detail		->load($employee['id'],\Neuron\Employee\Detail\Storage::LOADBY_EMPID);
					$dataDetail =  $detail->pull();
					
					$dataEmployee = array_merge($dataPerson, $dataDetail, $dataEmp);
					
					if($dataEmployee['pic_official']){
						if(!file_exists(getcwd() .'/public'. $dataEmployee['pic_official'])){
							$dataEmployee['pic_official'] = '/asset/images/upload/avatar/user.png';
						}
					}else{
						$dataEmployee['pic_official'] = '/asset/images/upload/avatar/user.png';
					}

					$newEmployees[$employee['id']] = $dataEmployee;
					$employeesParams['id'] = $employee['id'];
				}   

				$attendances    = $this->_storage->loadIn("ref_id",$empIds,$date);
				$rows           = [];
				foreach($newEmployees as $employee){
					$searchAttendance   = $this->searchAttendance($attendances,$employee['id']);

					if(empty($searchAttendance) || $searchAttendance['type_id'] == 3){				
						$leave     = new \Neuron\Approval\Entities(\Neuron\Order\Entity\Storage::factory(
							$this->_storage->getAdapter(),
							$this->_storage->getConfig()
						));
						
						$personMap  = new \Neuron\Person\Map(\Neuron\Person\Map\Storage::factory(
							$this->_storage->getAdapter(),
							$this->_storage->getConfig()
						));
						$personMap->load($employee['id'],\Neuron\Person\Map\Storage::LOADBY_PERSON_ID);
						$userId 	= $personMap->user_id;
						
						$where['where']['create_user_id'] = $userId;
						$where['where']['order_type_id'] = 2200;
						$where['where']['order_status_id'] = 2;
						$where['where']['xn5'] = 2;
						$where['where']['sdate'] = $date;
						$where['where']['edate'] = $date;
						
						$searchAttendance = $leave->load(null, \Neuron\Order\Entity\Storage::LIST_ALL, $where, true);
												
						//print_r($searchAttendance);
						if($searchAttendance->code == 0){
							$searchAttendance = $searchAttendance->data[0];
						}else{
							$searchAttendance = array();
						}
					}
					
					$rows[] = [
						'employees'     => $employee,
						'attendance'    => $searchAttendance
					];
				}
			}			
			
			if($status){ # FILTER BY STATUS
				if($status == 1){ # Attendance
					$attendanceByDate    = $this->_storage->loadInToday("ref_id",$employeesParams,$date);

					foreach($attendanceByDate as $attd){								
						//load data employee
						$emp 		= new EmpEntity(EmpEntity\Storage::factory($this->getDb(),$this->getConfig()));
						$emp		->load($attd['ref_id']);
						$dataEmp	= $emp->pull();
						
						//load data person
						$person 	= new PersonEntity(PersonEntity\Storage::factory($this->getDb(),$this->getConfig()));
						$person		->load($attd['ref_id']);
						$dataPerson = $person->pull();

						//load data person detail
						$detail 	= new Detail(Detail\Storage::factory($this->getDb(),$this->getConfig()));
						$detail		->load($attd['ref_id'],\Neuron\Employee\Detail\Storage::LOADBY_EMPID);
						$dataDetail =  $detail->pull();
						
						$dataEmployee = array_merge($dataPerson, $dataDetail, $dataEmp);
						
						if($dataEmployee['pic_official']){
							if(!file_exists(getcwd() .'/public'. $dataEmployee['pic_official'])){
								$dataEmployee['pic_official'] = '/asset/images/upload/avatar/user.png';
							}
						}else{
							$dataEmployee['pic_official'] = '/asset/images/upload/avatar/user.png';
						}
						//print_r($dataEmployee);die();

						$searchAttendance   = $this->searchAttendance($attendanceByDate,$attd['ref_id']);	

						$rows[] = [
							'employees'     => $dataEmployee,
							'attendance'    => $searchAttendance
						];
					}
				}elseif($status == 2){ # LEAVE
					$leave     = new \Neuron\Approval\Entities(\Neuron\Order\Entity\Storage::factory(
						$this->_storage->getAdapter(),
						$this->_storage->getConfig()
					));
					
					$where['where']['order_type_id'] = 2200;
					$where['where']['order_status_id'] = 2;
					$where['where']['xn5'] = 2;
					$where['where']['sdate'] = $date;
					$where['where']['edate'] = $date;

					$searchAttendance = $leave->load(null, \Neuron\Order\Entity\Storage::LIST_ALL, $where, true);
					//print_r($searchAttendance);die();
					
					if($searchAttendance->code == 0){
						$searchAttendances = $searchAttendance->data;
						
						foreach($searchAttendances as $attd){	
							$personMap  = new \Neuron\Person\Map(\Neuron\Person\Map\Storage::factory(
								$this->_storage->getAdapter(),
								$this->_storage->getConfig()
							));
							$personMap->load($attd['create_user_id'],\Neuron\Person\Map\Storage::LOADBY_USER_ID);
							$map        = $personMap->pull();
							if($map['person_id']){
								$user_id = $map['person_id'];
							}
							
							//load data employee
							$emp 		= new EmpEntity(EmpEntity\Storage::factory($this->getDb(),$this->getConfig()));
							$emp		->load($map['person_id']);
							$dataEmp	= $emp->pull();
							
							//load data person
							$person 	= new PersonEntity(PersonEntity\Storage::factory($this->getDb(),$this->getConfig()));
							$person		->load($map['person_id']);
							$dataPerson = $person->pull();

							//load data person detail
							$detail 	= new Detail(Detail\Storage::factory($this->getDb(),$this->getConfig()));
							$detail		->load($map['person_id'],\Neuron\Employee\Detail\Storage::LOADBY_EMPID);
							$dataDetail =  $detail->pull();
							
							$dataEmployee = array_merge($dataPerson, $dataDetail, $dataEmp);
							
							if($dataEmployee['pic_official']){
								if(!file_exists(getcwd() .'/public'. $dataEmployee['pic_official'])){
									$dataEmployee['pic_official'] = '/asset/images/upload/avatar/user.png';
								}
							}else{
								$dataEmployee['pic_official'] = '/asset/images/upload/avatar/user.png';
							}

							$searchAttendance   = $this->searchLeave($searchAttendances,$attd['create_user_id']);

							$rows[] = [
								'employees'     => $dataEmployee,
								'attendance'    => $searchAttendance
							];
						}
					}
				}else{ # NO INFORMATION
					$load = $this->_storage->loadNotAttend($date, $date, $employeesParams);
				
					if(empty($load)){
						$result->info   = "No employees";
						$result->code   = 2;
						return $result;
					}
					
					$employees      = $load;
					$empIds         = [];
					$newEmployees   = [];

					foreach($employees as $employee){
						$empIds[]   = $employee['id'];
						
						//load data employee
						$emp 		= new EmpEntity(EmpEntity\Storage::factory($this->getDb(),$this->getConfig()));
						$emp		->load($employee['id']);
						$dataEmp	= $emp->pull();
						
						//load data person
						$person 	= new PersonEntity(PersonEntity\Storage::factory($this->getDb(),$this->getConfig()));
						$person		->load($employee['id']);
						$dataPerson = $person->pull();

						//load data person detail
						$detail 	= new Detail(Detail\Storage::factory($this->getDb(),$this->getConfig()));
						$detail		->load($employee['id'],\Neuron\Employee\Detail\Storage::LOADBY_EMPID);
						$dataDetail =  $detail->pull();
						
						$dataEmployee = array_merge($dataPerson, $dataDetail, $dataEmp);
						
						if($dataEmployee['pic_official']){
							if(!file_exists(getcwd() .'/public'. $dataEmployee['pic_official'])){
								$dataEmployee['pic_official'] = '/asset/images/upload/avatar/user.png';
							}
						}else{
							$dataEmployee['pic_official'] = '/asset/images/upload/avatar/user.png';
						}

						$newEmployees[$employee['id']] = $dataEmployee;
						$employeesParams['id'] = $employee['id'];
					}
					
					$attendances    = $this->_storage->loadIn("ref_id",$empIds,$date);	
		   
					foreach($newEmployees as $employee){
						$searchAttendance   = $this->searchAttendance($attendances,$employee['id']);
						
						if(empty($searchAttendance) || $searchAttendance['type_id'] == 3){							
							$leave     = new \Neuron\Approval\Entities(\Neuron\Order\Entity\Storage::factory(
								$this->_storage->getAdapter(),
								$this->_storage->getConfig()
							));
							
							$personMap  = new \Neuron\Person\Map(\Neuron\Person\Map\Storage::factory(
								$this->_storage->getAdapter(),
								$this->_storage->getConfig()
							));
							$personMap->load($employee['id'],\Neuron\Person\Map\Storage::LOADBY_PERSON_ID);
							$userId 	= $personMap->user_id;
							
							$where['where']['create_user_id'] = $userId;
							$where['where']['order_type_id'] = 2200;
							$where['where']['order_status_id'] = 2;
							$where['where']['xn5'] = 2;
							$where['where']['sdate'] = $date;
							$where['where']['edate'] = $date;
							
							$searchAttendance = $leave->load(null, \Neuron\Order\Entity\Storage::LIST_ALL, $where, true);
							
							if($searchAttendance->code == 0){
								$searchAttendances = $searchAttendance->data;
								$searchAttendance = $this->searchLeave($searchAttendances,$userId);
							}else{
								$searchAttendance = array();
							}
						}
						
						$rows[] = [
							'employees'     => $employee,
							'attendance'    => $searchAttendance
						];
					}
				}
				
			}
			
			if($sort){ # FILTER BY SORTING
				if($sort == 'date'){
					$attendanceByDate    = $this->_storage->loadInToday("ref_id",$employeesParams,$date);
					
					foreach($attendanceByDate as $attd){		
						//load data employee
						$emp 		= new EmpEntity(EmpEntity\Storage::factory($this->getDb(),$this->getConfig()));
						$emp		->load($attd['ref_id']);
						$dataEmp	= $emp->pull();
						
						//load data person
						$person 	= new PersonEntity(PersonEntity\Storage::factory($this->getDb(),$this->getConfig()));
						$person		->load($attd['ref_id']);
						$dataPerson = $person->pull();

						//load data person detail
						$detail 	= new Detail(Detail\Storage::factory($this->getDb(),$this->getConfig()));
						$detail		->load($attd['ref_id'],\Neuron\Employee\Detail\Storage::LOADBY_EMPID);
						$dataDetail =  $detail->pull();
						
						$dataEmployee = array_merge($dataPerson, $dataDetail, $dataEmp);
						
						if($dataEmployee['pic_official']){
							if(!file_exists(getcwd() .'/public'. $dataEmployee['pic_official'])){
								$dataEmployee['pic_official'] = '/asset/images/upload/avatar/user.png';
							}
						}else{
							$dataEmployee['pic_official'] = '/asset/images/upload/avatar/user.png';
						}

						$searchAttendance   = $this->searchAttendance($attendanceByDate,$attd['ref_id']);	
						
						$rows[] = [
							'employees'     => $dataEmployee,
							'attendance'    => $searchAttendance
						];
					}
				}else{
					if($manager = NAF\Event\Manager::get()) {
						$manager->events()->register('employee_before_loadlist',array($this,'eventListenerOrderEmployee'),0,1);
					}
					
					$load = $empEntities->load($name, \Neuron\Employee\Entity\Storage::LISTBY_PAGE, $employeesParams);
					
					if($load->code != 0){
						$result->info   = "No employees";
						$result->code   = 2;
						return $result;
					}
					$employees      = $load->data['list'];
					$empIds         = [];
					$newEmployees   = [];				

					foreach($employees as $employee){
						$empIds[]   = $employee['id'];
						//load data employee
						$emp 		= new EmpEntity(EmpEntity\Storage::factory($this->getDb(),$this->getConfig()));
						$emp		->load($employee['id']);
						$dataEmp	= $emp->pull();
						
						//load data person
						$person 	= new PersonEntity(PersonEntity\Storage::factory($this->getDb(),$this->getConfig()));
						$person		->load($employee['id']);
						$dataPerson = $person->pull();

						//load data person detail
						$detail 	= new Detail(Detail\Storage::factory($this->getDb(),$this->getConfig()));
						$detail		->load($employee['id'],\Neuron\Employee\Detail\Storage::LOADBY_EMPID);
						$dataDetail =  $detail->pull();
						
						$dataEmployee = array_merge($dataPerson, $dataDetail, $dataEmp);
						
						if($dataEmployee['pic_official']){
							if(!file_exists(getcwd() .'/public'. $dataEmployee['pic_official'])){
								$dataEmployee['pic_official'] = '/asset/images/upload/avatar/user.png';
							}
						}else{
							$dataEmployee['pic_official'] = '/asset/images/upload/avatar/user.png';
						}

						$newEmployees[$employee['id']] = $dataEmployee;
						$employeesParams['id'] = $employee['id'];
					}   
					
					//print_r($newEmployees);die();
					
					$attendances    = $this->_storage->loadIn("ref_id",$empIds,$date);
					//print_r($attendances);
					$rows           = [];
					foreach($newEmployees as $employee){
						$searchAttendance   = $this->searchAttendance($attendances,$employee['id']);
							
						if(empty($searchAttendance) || $searchAttendance['type_id'] == 3){							
							$leave     = new \Neuron\Approval\Entities(\Neuron\Order\Entity\Storage::factory(
								$this->_storage->getAdapter(),
								$this->_storage->getConfig()
							));
							
							$personMap  = new \Neuron\Person\Map(\Neuron\Person\Map\Storage::factory(
								$this->_storage->getAdapter(),
								$this->_storage->getConfig()
							));
							$personMap->load($employee['id'],\Neuron\Person\Map\Storage::LOADBY_PERSON_ID);
							$userId 	= $personMap->user_id;
							
							$where['where']['create_user_id'] = $userId;
							$where['where']['order_type_id'] = 2200;
							$where['where']['order_status_id'] = 2;
							$where['where']['xn5'] = 2;
							$where['where']['sdate'] = $date;
							$where['where']['edate'] = $date;
							
							$searchAttendance = $leave->load(null, \Neuron\Order\Entity\Storage::LIST_ALL, $where, true);
							
							if($searchAttendance->code == 0){
								$searchAttendances = $searchAttendance->data;
								$searchAttendance = $this->searchLeave($searchAttendances,$userId['user_id']);
							}else{
								$searchAttendance = array();
							}
						}
						
						$rows[] = [
							'employees'     => $employee,
							'attendance'    => $searchAttendance
						];
					}
				}
			}
		}
		
        $result->data   = $rows;
        return $result;
    }

    public function searchAttendance($lists,$ref_id){
        $found = [];
        foreach($lists as $row){
            if($row['ref_id'] == $ref_id){
                $found = $row;
            }
        }
        return $found;
    }
	
	public function searchLeave($lists,$ref_id){
        $found = [];
        foreach($lists as $row){
            if($row['create_user_id'] == $ref_id){
                $found = $row;
            }
        }
        return $found;
    }


    public function loadSummary($filter = [])
    {
        $result         = new Result();
        $data = $this->_storage->loadSummary($filter);

        $count_late = 0;
        $count_clockin = 0;
        $count_absent = 0;
        $count_overtime = 0;

        if($data){
            foreach ($data as $row) {
                $shift                  = new Shift(Shift\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));

                $params['start_date']     = date('Y-m-d',strtotime($row['start_dtm']));
                $params['start_dtm']      = date('Y-m-d H:i:s',strtotime($row['start_dtm']));
                $params['day_no']         = $shift->getDayNumber($params['start_date'])->data;
                $params['ref_id']         = $row['ref_id'];

                // Check apakah user memiliki shift yang aktif di tanggal yang di request
                $loadShift              = $shift->load(null,Shift\Storage::LOADBY_BETWEENDATE_REF_ID_DAYNO,$params);
                
                $start_time = date('H:i:s', strtotime($row['start_dtm']));
                $end_time = date('H:i:s', strtotime($row['end_dtm']));

                if($row['type_id'] == 1){
                    //clockin
                    $count_clockin = $count_clockin + 1;

                    if($loadShift->code == 0){
                        $dutytime = $loadShift->data['on_duty_time'];
                        $latetime = $loadShift->data['late_time'];
                        if((floor((strtotime($start_time)- strtotime($dutytime))/60)- $latetime) > 0){
                            $count_late = $count_late + (floor((strtotime($start_time)- strtotime($dutytime))/60)- $latetime);
                        }
                    }

                }elseif ($row['type_id'] ==2) {
                    //overtime
                    if($loadShift->code == 0){
                        $dutytime = $loadShift->data['on_duty_time'];
                        $latetime = $loadShift->data['late_time'];
                        if((CEIL((strtotime($row['end_dtm'])- strtotime($row['start_dtm']))/3600)) > 0){
                            $count_overtime = $count_overtime + (CEIL((strtotime($row['end_dtm'])- strtotime($row['start_dtm']))/3600));
                        }
                    }
                }elseif($row['type_id'] ==3) {
                    //absent
                    $count_absent = $count_absent + 1;
                }
            }
        }
        $result->data = array('count_late'=>$count_late, 'count_clockin'=>$count_clockin, 'count_absent'=>$count_absent, 'count_overtime'=>$count_overtime);

        return $result;
    }

    public function loadSummaryRecommendation($filter = [])
    {
        $result         = new Result();

        $sub  = new ApprovalSubtypes(OrderSubtype\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
        $result = $sub->load(OrderSubtype\Storage::LIST_ALL);
        $lists  = $result->data['list'];
        $rows = [];
        foreach($lists as $row){

            if($row['order_type_id'] == 2100 || $row['order_type_id'] == 2200){
                $sApproval = \Neuron\Order\Entity\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig());
                $mApproval = new \Neuron\Approval\Entities($sApproval);
                if(isset($filter['user_id']) && !empty($filter['user_id'])){
                    $where['where']['create_user_id'] = $filter['user_id'];  
                }
                $where['where']['order_type_id'] = $row['order_type_id'];
                $where['where']['order_subtype_id'] = $row['order_subtype_id'];
                $where['where']['order_status_id'] = 2;
                $where['where']['xn5'] = 2;
                $where['where']['sdate'] = $filter['start_date'];
                $where['where']['edate'] = $filter['end_date'];
                $retval = $mApproval->load(null, \Neuron\Order\Entity\Storage::LIST_ALL, $where, true);

                $rows[$row['order_subtype_id']]['label'] = $row['name'];
                $rows[$row['order_subtype_id']]['counter'] = ($retval->data)? count($retval->data):0;
                $rows[$row['order_subtype_id']]['order_type_id'] = $row['order_type_id'];

          
            }
        }
     

        $result->data = $rows;
        return $result;
    }

    public function canAttend($sessmgt)
    {
        $naf            = \Neuron\Application\Framework\Entity::get();
        $result         = new Result();

        $session        = $naf->session();
        $sessOwner      = $session->owner();
        $employeeMap    = new EmployeeMap(EmployeeMap\Storage::factory($this->getDb(),$this->getConfig()));
        $employeeMap->load($sessOwner,EmployeeMap\Storage::LOADBY_USER_ID);
        $employee       = $employeeMap->employee();
        $employeeId     = $employee->id;

        //load data attendance
        $this->load(
            null,
            \Neuron\Attendance\Entity\Storage::LOADBY_ID,
            array(
                'ref_id'        => $employeeId,
                'start_date'    => date("Y-m-d")
            )
        );

        $shift                  = new \Neuron\Attendance\Shift(\Neuron\Attendance\Shift\Storage::factory(
            $this->_storage->getAdapter(),$this->_storage->getConfig()
        ));
        $data['ref_id']         = $employeeId;
        $data['start_date']     = date('Y-m-d');
        $data['start_dtm']      = date('Y-m-d H:i:s');
        $data['day_no']         = $shift->getDayNumber($data['start_date'])->data;
        $shift->load(null,\Neuron\Attendance\Shift\Storage::LOADBY_BETWEENDATE_REF_ID_DAYNO,$data);
        $canAttend              = true;
        $info                   = "Can attend";
        $attendType             = self::ATTEND_TYPE_IN;
        $autoPop                = false;
        
        $isWorkday  = $shift->is_workday;
        if($isWorkday){
            if($this->id && $this->type_id != 3){
                // udah ada absennya 
                $isWithinRange          = $shift->isWithinTimeRange(date('H:i:s'),$shift->out_beginning_time,$shift->out_ending_time);
                if(!$isWithinRange){
                    // Belum jam pulang, berarti gabisa absen
                    $canAttend  = false;
                    $info       = "Anda sudah melakukan absen. tidak dapat melakukan absen pulang pada saat ini.";
                    $attendType = self::ATTEND_TYPE_NOT_IN_RANGE_OUT;
                }else{
                    if($this->end_dtm != null){
                        $canAttend  = false;
                        $info       = "Anda sudah keluar.";
                        $attendType = self::ATTEND_TYPE_ALREADY_OUT;
                    }else{
                        $attendType = self::ATTEND_TYPE_OUT;
                    }
                }
            }else{
                /* Cari dulu, hari ini libur ga */
                $events             = new \Neuron\Calendar\Events(\Neuron\Calendar\Event\Storage::factory(
                    $this->_storage->getAdapter(),$this->_storage->getConfig()
                ));
                $loadEvents   = $events->load(
                    0, # 0 = hari libur
                    \Neuron\Calendar\Event\Storage::LISTBY_WORKDAY,
                    array(
                        'start_date'    => date('Y-m-d',strtotime($data['start_dtm'])),
                        'end_date'      => date('Y-m-d',strtotime($data['start_dtm'])),
                        'date_between'  => true
                    )
                );
                if($loadEvents->code == 0){
                    $data   = $loadEvents->data;
                    $eventData = $data['list'][0];
                    $canAttend = false;
                    $attendType = self::ATTEND_TYPE_NOT_IN_RANGE_IN;
                    $info       = "Tidak dapat hadir pada hari libur";
                }else{
                    // Check kalo melewati batas waktu absen, gabisa absen;
                    $canAttend      = $shift->isWithinTimeRange(date("H:i"),$shift->in_beginning_time,$shift->in_ending_time);
                    $attendType     = ($canAttend) ? self::ATTEND_TYPE_IN : self::ATTEND_TYPE_NOT_IN_RANGE_IN;
                    $info           = ($canAttend) ? "OK" : "Tidak bisa masuk saat ini. Jam di mulai pada" .' '. $shift->in_beginning_time .' '. "Sampai" .' '.$shift->in_ending_time;
                    $autoPop        = ($canAttend) ? true : false;
                }

            }
        }else{
            $info       = "Tidak dapat hadir pada hari libur"; 
            $canAttend  = false;
        }
        $result->info = $info;
        $result->data = [
            'can_attend'    => $canAttend,
            'attendance'    => $this->pull(),
            'shift'         => $shift->pull(),
            'attend_type'   => $attendType,
            'autoRedirect'  => $autoPop,
        ];
        
        return $result;
    }

    # HR reporting data [BEGIN]
    public function loadSummaryHrReporting($filter = [])
    {
        $result         = new Result();
        // attendance summary
        $aparams = array(
            'start_date' => $filter['start_date'],
            'end_date' => $filter['end_date']
        );

        $data = $this->_storage->loadSummary($aparams);

        $count_late = 0;

        $count_clockin = 0;
        $count_absent = 0;
       

        $count_recomendation = 0;
        $count_pamantra = 0;
        $count_overtime = 0;
        $count_clockin_earlier = 0;

       
        $count_clockin_office = 0;
        $count_clockin_nonoffice = 0;
       
        if($data){
          
            // recomendation summary
            $recomen = $this->loadSummaryRecommendation($filter);

            foreach ($recomen->data as $key => $val) {
                $count_recomendation  +=$val['counter'];
            }

            // pamantra summary
            if(class_exists('Neuron\Payroll\Entity')){
            
                $ppamantra = array(
                    'start_date' => date('Y-m-d',strtotime($filter['start_date'])),
                    'end_date' => date('Y-m-d',strtotime($filter['end_date'])),
                    // 'filter_period'=> '2020-01', 
                    'type_chart'=>'Full',
                );
                $pamantra = $this->loadSummaryPamantra($ppamantra);
                //  echo "<pre>" ,print_r($pamantra,1), "</pre>";die();
    
                $count_pamantra +=  count($pamantra->data['list']);
            }


            foreach ($data as $row) {

                $shift                  = new Shift(Shift\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
                $params['start_date']     = date('Y-m-d',strtotime($row['start_dtm']));
                $params['start_dtm']      = date('Y-m-d H:i:s',strtotime($row['start_dtm']));
                $params['day_no']         = $shift->getDayNumber($params['start_date'])->data;
                $params['ref_id']         = $row['ref_id'];

                // Check apakah user memiliki shift yang aktif di tanggal yang di request
                $loadShift              = $shift->load(null,Shift\Storage::LOADBY_BETWEENDATE_REF_ID_DAYNO,$params);
 
                $start_time = date('H:i:s', strtotime($row['start_dtm']));
                $end_time = date('H:i:s', strtotime($row['end_dtm']));

                if($row['type_id'] == 1){
                    //clockin
                    $count_clockin = $count_clockin + 1;
                    if($loadShift->code == 0){
                        $dutytime = $loadShift->data['on_duty_time'];
                        $latetime = $loadShift->data['late_time'];
                        // late
                        if((floor((strtotime($start_time)- strtotime($dutytime))/60)- $latetime) > 0){
                            $count_late = $count_late + (floor((strtotime($start_time)- strtotime($dutytime))/60)- $latetime);
                        }
                        // earlier
                        if(strtotime($start_time) < strtotime($dutytime)){
                            $count_clockin_earlier = $count_clockin_earlier + 1;
                        }
                        
                        // FP attendance
                        if($row['source_type_code'] == 'FP'){
                            $count_clockin_office = $count_clockin_office + 1;
                        } else {
                            $count_clockin_nonoffice =  $count_clockin_nonoffice + 1;
                        }

                    }

                }elseif ($row['type_id'] ==2) {
                    //overtime
                    if($loadShift->code == 0){
                        $dutytime = $loadShift->data['on_duty_time'];
                        $latetime = $loadShift->data['late_time'];
                        if((CEIL((strtotime($row['end_dtm'])- strtotime($row['start_dtm']))/3600)) > 0){
                            $count_overtime = $count_overtime + (CEIL((strtotime($row['end_dtm'])- strtotime($row['start_dtm']))/3600));
                        }
                    }
                }elseif($row['type_id'] ==3) {
                    //absent
                    $count_absent = $count_absent + 1;
                }
            
            }


        }

        // $result->data = array('count_late'=>$count_late, 'count_clockin'=>$count_clockin, 'count_absent'=>$count_absent, 'count_overtime'=>$count_overtime);
        $result->data = array(
            'count_recomendation'=>$count_recomendation,
            'count_pamantra' => $count_pamantra,
            'count_clockin_earlier' => $count_clockin_earlier,
            'count_overtime'=>$count_overtime,
            'count_clockin'=>$count_clockin, 
            'count_absent'=>$count_absent,
   
            'count_clockin_office'=>$count_clockin_office,
            'count_clockin_nonoffice'=>$count_clockin_nonoffice,
 
        );
        // echo '<pre>',print_r($result,1),'</pre>';
        return $result;
    }

    public function loadSummaryPamantra($params){
        if(class_exists('Neuron\Payroll\Entity')){
            $storage = \Neuron\Payroll\Bonus\Storage::factory($this->_storage->getAdapter(), $this->_storage->getConfig());
            $entity = new \Neuron\Payroll\Bonuses($storage);
            // $bonus = new \Neuron\Payroll\Bonus($storage);

            $mode = isset($post['mode'])? $post['mode']: \Neuron\Payroll\Entity\Storage::LIST_ALL;
            $result = $entity->load(null, $mode, $params);

        return $result;
        }
    }

    public function loadSummaryAttendaceHr($params = []){
        $result         = new Result();
        // attendance summary
        $aparams = array(
            'start_date' => $params['start_date'],
            'end_date' => $params['end_date']
        );

        $data = $this->_storage->loadSummary($aparams);

        $count_late = 0;

        $count_clockin = 0;
        $count_absent = 0;
       
        $count_overtime = 0;
        $count_clockin_earlier = 0;

        $count_clockin_web = 0;
        $count_clockin_fp = 0;
        $count_clockin_request = 0;
        $count_clockin_api = 0;
        $count_clockin_admin = 0;

        if($data){
          

            foreach ($data as $row) {

                $shift                  = new Shift(Shift\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
                $params['start_date']     = date('Y-m-d',strtotime($row['start_dtm']));
                $params['start_dtm']      = date('Y-m-d H:i:s',strtotime($row['start_dtm']));
                $params['day_no']         = $shift->getDayNumber($params['start_date'])->data;
                $params['ref_id']         = $row['ref_id'];

                // Check apakah user memiliki shift yang aktif di tanggal yang di request
                $loadShift              = $shift->load(null,Shift\Storage::LOADBY_BETWEENDATE_REF_ID_DAYNO,$params);

                $start_time = date('H:i:s', strtotime($row['start_dtm']));
                $end_time = date('H:i:s', strtotime($row['end_dtm']));

                if($row['type_id'] == 1){
                    //clockin
                    $count_clockin = $count_clockin + 1;
                    if($loadShift->code == 0){
                        $dutytime = $loadShift->data['on_duty_time'];
                        $latetime = $loadShift->data['late_time'];
                        // late
                        if((floor((strtotime($start_time)- strtotime($dutytime))/60)- $latetime) > 0){
                            $count_late = $count_late + (floor((strtotime($start_time)- strtotime($dutytime))/60)- $latetime);
                        }
                        // earlier
                        if(strtotime($start_time) < strtotime($dutytime)){
                            $count_clockin_earlier = $count_clockin_earlier + 1;
                            //  echo '<pre>',print_r($row,1),'</pre>';
                        }
                        // WEB attendance
                        if($row['source_type_code'] == 'WEB'){
                            $count_clockin_web = $count_clockin_web + 1;
                        }
                        // FP attendance
                        if($row['source_type_code'] == 'FP'){
                            $count_clockin_fp = $count_clockin_fp + 1;
                        }
                        // REQUEST attendance
                        if($row['source_type_code'] == 'REQUEST'){
                            $count_clockin_request = $count_clockin_request + 1;
                        }

                        // API attendance 
                        if($row['source_type_code'] == 'API'){
                            $count_clockin_api = $count_clockin_api + 1;
                        }

                        // BY ADMIN attendance 
                        if($row['source_type_code'] == 'ADMIN'){
                            $count_clockin_admin = $count_clockin_admin + 1;
                        }
                    }

                }elseif ($row['type_id'] ==2) {
                    //overtime
                    if($loadShift->code == 0){
                        $dutytime = $loadShift->data['on_duty_time'];
                        $latetime = $loadShift->data['late_time'];
                        if((CEIL((strtotime($row['end_dtm'])- strtotime($row['start_dtm']))/3600)) > 0){
                            $count_overtime = $count_overtime + (CEIL((strtotime($row['end_dtm'])- strtotime($row['start_dtm']))/3600));
                        }
                    }
                }elseif($row['type_id'] ==3) {
                    //absent
                    $count_absent = $count_absent + 1;
                }
            
            }
            

        }

        // $result->data = array('count_late'=>$count_late, 'count_clockin'=>$count_clockin, 'count_absent'=>$count_absent, 'count_overtime'=>$count_overtime);
        $result->data = array(
           
            'count_clockin_earlier' => $count_clockin_earlier,
            'count_overtime'=>$count_overtime,
            'count_clockin'=>$count_clockin, 
            'count_absent'=>$count_absent,
            'count_clockin_web' => $count_clockin_web,
            'count_clockin_fp'=>$count_clockin_fp,
            'count_clockin_request'=>$count_clockin_request,
            'count_clockin_api'=>$count_clockin_api,
            'count_clockin_admin'=>$count_clockin_admin,
        );
        // echo '<pre>',print_r($result,1),'</pre>';
        return $result;
    }

    # HR reporting data [END]

    /**
     * checkAttendAction
     *
     * @return \Neuron\Generic\Result
     */
    public function checkAttend(){
        $result     = new \Neuron\Generic\Result;
        $attendanceEntity = new Entity(Entity\Storage::factory(
            $this->getDb(),
            $this->getConfig()
        ));

        if(class_exists('Neuron\Employee\Entity')){
            $mapper     = $this->getMapper();
            $employee   = $mapper->data['employee'];
            $map        = $mapper->data['map'];
            $userId     = $map->user_id;
            $refId      = $employee->id;
        }else{
            $naf        = NAF\Entity::get();
            $session    = $naf->session();
            $userId     = $session->owner();
            $refId      = $userId;
        }

        /**
         * Check ke module lain menggunakan event
         * barangkali module lain ada yang ga mengijinkan absen
         * misalkan ada data cuti pada hari ini berarti gabisa absen.
         * https://git.neuron.id/library/attendance/-/issues/391#note_251099
         */
        if ($nafManager = \Neuron\Application\Framework\Event\Manager::get()) {
			$checkBarrier = $nafManager->events()->trigger(
                'attendance_allow_attend',
                [
                    'start_dtm' => date('Y-m-d'),
                    'user_id'   => $userId,
                    'ref_id'    => $refId
                ]
            );

            if($checkBarrier->code != 0){
                $result->data = [  
                    'can_attend'    => false,
                    'attendance'    => $this->pull(),
                    'shift'         => null,
                    'attend_type'   => 1,
                    'auto_pop'      => false,
                    'info'          => $checkBarrier->info,
                    'barrier'       => $checkBarrier->data
                ];
                $result->info       = $checkBarrier->info;
                return $result;
            }
		}
        /* End Checking module lain */

        $attendanceEntity->load(
            null,Entity\Storage::LOADBY_ID,
            array(
                'ref_id'        => $refId,
                'start_date'    => date("Y-m-d")
            )
        );
        $shift                  = new Shift(Shift\Storage::factory($this->getDb(),$this->getConfig()));
        $data['ref_id']         = $refId;
        $data['start_date']     = date('Y-m-d');
        $data['start_dtm']      = date('Y-m-d H:i:s');
        $data['day_no']         = $shift->getDayNumber($data['start_date'])->data;
        $shift->load(null,Shift\Storage::LOADBY_BETWEENDATE_REF_ID_DAYNO,$data);
        $canAttend              = true;
        $info                   = "Can attend";
        $attendType             = self::ATTEND_TYPE_IN;
        $autoPop                = false;
        
        $isWorkday  = $shift->is_workday;
        if($isWorkday){
            if($attendanceEntity->id && $attendanceEntity->type_id != 3){
                // udah ada absennya 
                $isWithinRange          = $shift->isWithinTimeRange(date('H:i:s'),$shift->out_beginning_time,$shift->out_ending_time);
                if(!$isWithinRange){
                    // Belum jam pulang, berarti gabisa absen
                    $canAttend  = false;
                    $info       = "You're already attended. you cannot clock out at this time.";
                    $attendType = self::ATTEND_TYPE_NOT_IN_RANGE_OUT;
                }else{
                    if($attendanceEntity->end_dtm != null){
                        $canAttend  = false;
                        $info       = "You're already clock out.";
                        $attendType = self::ATTEND_TYPE_ALREADY_OUT;
                    }else{
                        $attendType = self::ATTEND_TYPE_OUT;
                    }
                }
            }else{
                /* Cari dulu, hari ini libur ga */
                $events             = new Events(Event\Storage::factory(
                    $this->getDb(),
                    $this->getConfig()
                ));
                $loadEvents   = $events->load(
                    0, # 0 = hari libur
                    Event\Storage::LISTBY_WORKDAY,
                    array(
                        'start_date'    => date('Y-m-d',strtotime($data['start_dtm'])),
                        'end_date'      => date('Y-m-d',strtotime($data['start_dtm'])),
                        'date_between'  => true
                    )
                );
                if($loadEvents->code == 0){
                    $data   = $loadEvents->data;
                    $eventData = $data['list'][0];
                    $canAttend = false;
                    $attendType = self::ATTEND_TYPE_NOT_IN_RANGE_IN;
                    $info       = "Cannot attend at holiday. [ ".$eventData['event_title']." ]";
                }else{
                    // Check kalo melewati batas waktu absen, gabisa absen;
                    $canAttend      = $shift->isWithinTimeRange(date("H:i"),$shift->in_beginning_time,$shift->in_ending_time);
                    $attendType     = ($canAttend) ? self::ATTEND_TYPE_IN : self::ATTEND_TYPE_NOT_IN_RANGE_IN;
                    $info           = ($canAttend) ? "OK" : "Cannot clock in at this time. Clock in start at " . $shift->in_beginning_time .' Until '.$shift->in_ending_time;
                    $autoPop        = ($canAttend) ? true : false;
                }

            }
        }else{
            $info       = "Cannot Attend at holiday"; 
            $canAttend  = false;
        }

        $result->data = [
            'can_attend'    => $canAttend,
            'attendance'    => $attendanceEntity->pull(),
            'shift'         => $shift->pull(),
            'attend_type'   => $attendType,
            'auto_pop'      => $autoPop,
            'info'          => $info
        ];
        $result->info       = $info;
        return $result;
    }
	
	public function eventListenerOrderEmployee(\Neuron\Db\Sql\Select $select){
		$_tables 	= $this->getConfig()['tables'];
		/* Join to person to get first_name*/
		$select->order(array('p.first_name ASC'));
	}

	public function delete($ref_id, $start_dtm) {
        return $this->_storage->remove($ref_id, $start_dtm);
    }
}