<?php
namespace Neuron\Attendance\Entity\Storage;

use Exception;
use Neuron\Attendance\Entity;
use Zend\Db\Sql\Predicate\Expression;

class Mysql extends \Neuron\Db\Storage implements Skeleton{
    const TYPE_ID_ATTENDANCE    = 1;
    const TYPE_ID_OVERTIME      = 2;

    public function loadList($id = null, $mode = Entity\Storage::LIST_ALL,$params = array()){

        $select         = $this->select();
        $select->from(array(
            'attd' => $this->_tables['attendance']
        ));
        $select->join(array(
            'emp' => $this->_tables['employee']),
            'emp.id = attd.ref_id',
            array('code','status_id')
        );
        $select->join(array(
            'per' => $this->_tables['person']),
            'emp.person_id = per.id',
             array(
                'ref_name' => new Expression("CONCAT(per.first_name,' ',per.last_name)")
             )
        );

        if($id){
            $nest = $select->where->nest();
            $nest->like($this->_('start_description','.attd.'),'%'.$id.'%')->or;
            $nest->like($this->_('end_description','.attd.'),'%'.$id.'%')->or;
            $nest->unnest();
        }
		
        if(isset($params['draw'])){
            $select->limit(intval($params['length']));
            $select->offset(intval($params['start']));
            if(isset($params['search']['value']) && !empty($params['search']['value'])){
                $select->where->nest()
                    ->like('start_description', '%'.$params['search']['value'].'%')
                    ->or
                    ->like('per.first_name', '%'.$params['search']['value'].'%')
                    ->or
                ->unnest();
            }

            $orderColumn    = $params['columns'][$params['order'][0]['column']]['data'];
            $dir            = $params['order'][0]['dir'];

            if($orderColumn == 'end_time' || $orderColumn == 'start_time'){
                switch ($orderColumn) {
                    case 'end_time':
                        $select->order(new Expression("TIME_FORMAT(end_dtm,'%h:%i:%s') ".$dir));
                        break;
                    case 'start_time':
                        $select->order(new Expression("TIME_FORMAT(start_dtm,'%h:%i:%s') ".$dir));
                }
            }else{
                $select->order($orderColumn.' '.$dir);
            }

        }

        if(isset($params['ref_id'])) $select->where(array('ref_id' => $params['ref_id']));

        if(isset($params['type_id'])) $select->where(array('attd.type_id' => $params['type_id']));

        if(isset($params['start_date'])){
            $select->where("DATE(start_dtm) >= '".$params['start_date']."'");
        }

        if(isset($params['end_date'])){
            $select->where("DATE(start_dtm) <= '".$params['end_date']."'");
        }

        if($mode != Entity\Storage::LIST_ALL){
            switch($mode){
                case Entity\Storage::LISTBY_SOURCE:
                    $select->where(array(
                        $this->_('source_type_code','attd.') => $id
                    ));
                break;
            }
        }

        try{
		// echo str_replace('"','',$select->getSqlString()).'<br />'; die();

            $result = new \stdClass;
            $data   = $this->fetchAll($select);
            if($data){
                $selectC = $select;
                $selectC->reset('limit')->reset('offset');
                $selectC->columns(array('countTotal' => new Expression('COUNT(*)')));
                $countTotal = $this->fetchRow($selectC)['countTotal'];
    
                $result->rows   = $data;
                $result->countTotal = $countTotal;
            }else{
                $result->rows   = array();
                $result->countTotal = 0;
            }
            
            return $result;

        }catch(\Exception $ex){
            return false;
            error_log('Error ::'.$ex->getMEssage());
        }
    }
    
    public function save(Entity $entity, $update = FALSE){
        if($update){
            $update     = $this->update();
            $update->table($this->_tables['attendance']);
            $update->where($this->__(array('id'=> $entity->id)));
            $update->set($this->__($entity->pull()));
            try{
                $rs = $this->execute($update);
                if($rs){
                    return $entity->id;
                }else{
                    return false;
                }
            }catch(\Exception $ex){
                return false;
            }
        }else{
            $entity->id = 0;
            $insert = $this->insert();
            $insert->into($this->_tables['attendance']);
            $insert->values($this->__($entity->pull()));
            try{
                $rs = $this->execute($insert);
                if($rs){
                    $entity->id = $rs;
                    return $rs;
                }else{
                    return false;
                }
            }catch(\Exception $ex){
                return false;
            }
        }
    }

    public function load($id = null, $mode = Entity\Storage::LOADBY_ID,$params = array()){
        $select         = $this->select();
        $select->from($this->_tables['attendance']);
        
        if(isset($params['ref_id']) && strlen($params['ref_id']) > 0){
            $select->where(array('ref_id' => $params['ref_id']));
        }
        if(isset($params['start_date']) && strlen($params['start_date']) > 0){
            $select->where("DATE(start_dtm) = '".$params['start_date']."'");
        }
        if(isset($params['start_dtm']) && strlen($params['start_dtm']) > 0){
            $select->where(array('start_dtm' => $params['start_dtm']));
        }
        if(isset($params['type_id'])){
            if(is_array($params['type_id'])){
                $select->where("type_id IN (".implode(',',$params['type_id']).")");
            }else{
                $select->where(array('type_id' => $params['type_id']));
            }
        }

        try {
            $fetch = $this->fetchRow($select);
            if($fetch){
                return $fetch;
            }else{
                return false;
            }
        } catch (\Exception $ex) {
            return false;
        }
        
    }

    public function countTotalByType($type,$startDate,$endDate,$refId = null){
        $select     = $this->select()->from($this->_tables['attendance']);

        if($startDate){
            $select->where->greaterThanOrEqualTo(new Expression("DATE(start_dtm)"),$startDate);
        }
        if($endDate){
            $select->where->lessThanOrEqualTo(new Expression("DATE(start_dtm)"),$endDate);
        }

        $select->columns([
            'countTotal' => new Expression("COUNT(*)")
        ]);

        $select->where([
            'type_id' => $type
        ]);

        if($refId){
            $select->where([
                'ref_id'    => $refId
            ]);
        }
		
		//echo str_replace('"','',$select->getSqlString()).'<br />'; 

        $data   = $this->fetchRow($select);
        return $data['countTotal'];
    }

    public function loadIn($column = null,$values = array(),$date = null){

        if(!is_array($values)){
            return false;
        }
		
        $select     = $this->select()->from($this->_tables['attendance']);
        $select->where->in($column,$values);
        if($date){
            $select->where("DATE(start_dtm) = '".$date."'");
        }
		
		// echo str_replace('"','',$select->getSqlString()).'<br />'; 

        try{
            $data   = $this->fetchAll($select);
            $rows   =[];
            if($data){
                foreach($data as $data){
                    $rows[] = $data;
                }
            }
            return $rows;
        }catch(\Exception $ex){
            error_log("ERROR ::". $ex->getMessage().' File '.$ex->getFile(),' Line : '.$ex->getLine());
            return false;
        }
    }
	
	public function loadInToday($column = null,$values = array(),$date = null){

        if(!is_array($values)){
            return false;
        }
		
        $select     = $this->select()->from($this->_tables['attendance']);
		
        if($date){
            $select->where("DATE(start_dtm) = '".$date."' AND type_id <> 2");
        }
				
		if(isset($values['draw'])){
			$select->limit(intval($values['length']));
            $select->offset(intval($values['start']));
			
			if(date('Y-m-d') == $date){
				$orderColumn	= $values['columns'][$values['orderDate'][0]['column']]['data'];
				$dir            = $values['orderDate'][0]['dir'];
				
				$select->order($orderColumn.' '.$dir);
			}else{
				$orderColumn1   = $values['columns'][$values['orderType'][0]['column']]['data'];
				$orderColumn2   = $values['columns'][$values['orderDate'][0]['column']]['data'];
				$dir            = $values['orderType'][0]['dir'];
				
				$select->order([$orderColumn1 => $dir, $orderColumn2 => $dir]);
			}
			
		}
		
		//echo str_replace('"','',$select->getSqlString()).'<br />'; 
		
        try{
            $data   = $this->fetchAll($select);
            $rows   =[];
            if($data){
                foreach($data as $data){
                    $rows[] = $data;
                }
            }
            return $rows;
        }catch(\Exception $ex){
            error_log("ERROR ::". $ex->getMessage().' File '.$ex->getFile(),' Line : '.$ex->getLine());
		}
	}

    public function loadSummary($filter = [])
    {
        $select     = $this->select()->from($this->_tables['attendance']);
        $select->where->greaterThanOrEqualTo(new Expression("DATE(start_dtm)"),$filter['start_date']);
        $select->where->lessThanOrEqualTo(new Expression("DATE(start_dtm)"),$filter['end_date']);
        if(isset($filter['ref_id']) && !empty($filter['ref_id'])){
            $select->where(array('ref_id' => $filter['ref_id']));
        }
        try {
            $fetch = $this->fetchAll($select);
            if($fetch){
                return $fetch;
            }else{
                return false;
            }
        } catch (\Exception $ex) {
            return false;
        }
    }
	
	public function loadNotAttend($start_date,$end_date,$params = null){
        $select         = $this->select();
        $select->from($this->__([
            'p'     => $this->_tables['person']
        ]));

		$select->join(array(
            'emp' => $this->_tables['employee']),
            'emp.person_id = p.id'
        );
		
		if(date('Y-m-d') == $start_date){
			$select->where(new Expression("p.id NOT IN (
            SELECT
                attendance.ref_id AS ref_id
            FROM
                attendance
            WHERE
                DATE(start_dtm) ='".$start_date."')
			"));
		}else{
			$select->where(new Expression("(p.id NOT IN (
            SELECT
                attendance.ref_id AS ref_id
            FROM
                attendance
            WHERE
                DATE(start_dtm) ='".$start_date."')
			OR p.id IN (
			SELECT
                attendance.ref_id AS ref_id
            FROM
                attendance
            WHERE
                DATE(start_dtm) ='".$start_date."'
				AND attendance.type_id = 3)
			)"));
		}
        
        $select->where($this->__([
            'emp.status_id' => 1
        ]));
		
		if(isset($params['draw'])){
			$select->limit(intval($params['length']));
            $select->offset(intval($params['start']));
		}
		
		//echo str_replace('"','',$select->getSqlString()).'<br />'; 
        
		try{
            $data   = $this->fetchAll($select);
            $rows   =[];
            if($data){
                foreach($data as $data){
                    $rows[] = $data;
                }
            }
            return $rows;
        }catch(\Exception $ex){
            error_log("ERROR ::". $ex->getMessage().' File '.$ex->getFile(),' Line : '.$ex->getLine());
		}
    }
	
	public function remove($ref_id, $start_dtm) {
		$delete = 	$this->delete()
					->from($this->_tables['attendance'])
					->where(array(	'type_id' => 3, 
									'ref_id' => $ref_id,
									'DATE(start_dtm) = "'. $start_dtm .'"'
								));
								
		return $this->execute($delete);
	}
}