<?php
namespace Neuron\Attendance;

use Neuron\Generic\Entity\Map;
use Neuron\Generic\Result;

class Evidence extends \Neuron\Generic\Entity\Mapper{
    protected $_storage;
    protected $_loaded      = FALSE;
    const TYPE_ATTENDANCE   = 1;
    const TYPE_OVERTIME     = 2;
    CONST TEST_ENVIRONMENT  = FALSE;
    const ERROR_PREFIX      = 'ATTENDANCE_EVIDENCE-';

    public function __construct(\Neuron\Attendance\Evidence\Storage\Skeleton $storage,$data = array()){
        $this->_storage     = $storage;
        parent::__construct(
            $data,
            array(
                new Map('attd_evidence_id'),
                new Map('attendance_id'),
                new Map('action_type'),
                new Map('filename'),
            ),
            parent::CASE_INSENSITIVE
        );
    }

    public function load($id, $mode = \Neuron\Attendance\Evidence\Storage::LOADBY_ID) {
        if ($data = $this->_storage->load($id, $mode)) {
            $this->push($data);
            return true;
        }
        return false;
    }

    public function save($update = false){
        if ($id = $this->_storage->save($this, $update)) {
            $this->attd_evidence_id = $id;
            return true;
        }
        return false;
    }

    public function create($data = array())
    {
        $result = new Result();
        if(empty($data)){
            $result->code = 1;
            $result->info = 'File Cannot NULL';
            return $result;
        }
        if($data['file']){
            $fileloc = getcwd().'/public/attendance/evidence/';
            $filename = $data['action_type_name'].'_'.$data['ref_id'].'_'.time().'.jpeg';
            $saveImage = $this->convertBase64ToImage($data['file'], $filename); //Save Image
            if($saveImage['status']){
                //Update DB
                if(file_exists($fileloc.$saveImage['file'])){
                    $this->attd_evidence_id = null;
                    $this->attendance_id = $data['id'];
                    $this->action_type = $data['action_type'];
                    $this->filename = $saveImage['file'];
                    $this->save(false);
                }else{
                   return false;
                }
            }
        }
    }

    public function convertBase64ToImage($photo = null, $filename = null) {
        $status = false;
        $file = null;
        $message = 'File Empty';
        $fileloc = getcwd().'/public/attendance/evidence/';
        if(!file_exists($fileloc)) @mkdir($fileloc, 0775, true);
        
        if (!empty($photo)) {
            $photo = str_replace('data:image/png;base64,', '', $photo);
            $photo = str_replace(' ', '+', $photo);
            $photo = str_replace('data:image/jpeg;base64,', '', $photo);
            $photo = str_replace('data:image/gif;base64,', '', $photo);
            $entry = base64_decode($photo);
            $image = imagecreatefromstring($entry);

            $directory = $fileloc . $filename;
            $saveImage = imagejpeg($image, $directory);
            // $saveImage = file_put_contents($directory, $entry);

            imagedestroy($image);

            if ($saveImage) {
                $status = true;
                $file = $filename;
                $message = 'File saved!';
            } else {
                $status = false;
                $file = null;
                $message = 'File not saved!';
            }
        }

        return array('status' => $status, 'file' => $file, 'message' => $message);
    }

}