<?php 

namespace Neuron\Attendance\Evidence\Storage;
use Zend\Db\Sql\Where;
use Zend\Db\Sql\Predicate\Expression;

class Mysql extends \Neuron\Db\Storage implements Skeleton {
    const CONFIG_CODE = 'Attendance';

    public function __construct(\Zend\Db\Adapter\Adapter $adapter, $config = array()) {

        /* construct parent --> Neuron\Db\Storage */
        parent::__construct($adapter, $config);
    }

    public function load($id, $mode = \Neuron\Attendance\Evidence\Storage::LOADBY_ID) {
        
        /** kalau modif fungsi ini, silakan modif juga versi oracle dan versi postgre nya yah*/
        
        try {
            $select = $this->select()
                ->from(array('pc' => $this->_tables['attd_evidence']));
            if($mode == \Neuron\Attendance\Evidence\Storage::LOADBY_ID){
                $select->where($this->__(array('pc.attd_evidence_id' => $id)));
            }else if($mode == \Neuron\Attendance\Evidence\Storage::LOADBY_ATTENDANCE){
                $select->where($this->__(array('pc.attendance_id' => $id)));
            } 
            
            return $this->fetchRow($select);
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }

    public function loadList($ids, $mode = \Neuron\Attendance\Evidence\Storage::LIST_ALL, $params = null) {
        /* base select */
        $select = $this->select()
        ->from(array('a' => $this->_tables['attd_evidence']));

        /* loadby page */
        if ($mode == \Neuron\Attendance\Evidence\Storage::LISTBY_PAGE) {

            if(isset($params['deep']) && $params['deep']){
                $select->join(array('b'=> $this->_tables['payroll']), 'a.payroll_id=b.payroll_id', array('period','ref_code','ref_id', 'status', 'total_amount'), 'left');
                $select->join(array('c'=>$this->_tables['payroll_component']), 'a.component_id=c.component_id', array('component_name'=> 'name', 'credit_type', 'component_parent_id'), 'left');
                // $select->group(array($this->_('ref_id', 'b.')));
            }

            if($params['filter_period']){
                $select->where(array('b.period'=> $params['filter_period']));
            }

            /* has keyword? */
            if (!empty($ids)) {
                $select->where->nest()
                    ->like($this->_('description', 'a.'), '%' . trim($ids) . '%')
                ->unnest();
            }
            /* get pager */
            $pager = \Neuron\Db\Storage\Pager::sanitize($params);
            $select->limit($pager->getLimit())->offset($pager->getOffset())->cache(0);
            
        } else {
            $pager = new \Neuron\Db\Storage\Pager(0, 999999);
            switch ($mode) {
                case \Neuron\Attendance\Evidence\Storage::LISTBY_GROUP_PERIOD:
                    /* count total */
                    $select->columns(array(
                        'total_sum' => new \Zend\Db\Sql\Expression('sum(total_amount)'),
                        'period'=>'period',
                    ));
                    $select->where(array('SUBSTR(a.period, 1, 4) = "'.$params['filter_year'].'"'));
                    $select->group(array($this->_('period', 'a.')));
                    /* limit */
                    break;
                case \Neuron\Attendance\Evidence\Storage::LISTBY_PARENT:
                    $select->where(array($this->_('parent_id') => $ids));
                    break;
                case \Neuron\Attendance\Evidence\Storage::LISTBY_PAYROLL:
                    $select->join(array('b'=> $this->_tables['payroll']), 'a.payroll_id=b.payroll_id', array('period','ref_code','ref_id', 'status', 'total_amount'), 'left');
                    $select->join(array('c'=>$this->_tables['payroll_component']), 'a.component_id=c.component_id', array('component_name'=> 'name', 'credit_type', 'component_parent_id'), 'left');
                    $select->where(array($this->_('payroll_id', 'b.') => $ids));
                    
                    break;
                case \Neuron\Attendance\Evidence\Storage::LISTBY_GROUP_COMPONENT:
                    $select->join(array('b'=> $this->_tables['payroll']), 'a.payroll_id=b.payroll_id', array('period','ref_code','ref_id', 'status'), 'left');
                    $select->join(array('c'=>$this->_tables['payroll_component']), 'a.component_id=c.component_id', array('component_name'=> 'name', 'credit_type', 'component_parent_id'), 'left');

                    if($params['filter_period']){
                        $select->where(array('b.period'=> $params['filter_period']));
                    }
                    $select->group(array('a.component_id'));
                    break;
                case \Neuron\Attendance\Evidence\Storage::LISTBY_GROUP_REF:
                        $select->group(array($this->_('ref_id', 'b.')));
                    break;
                default:
            }

            if(isset($params['deep']) && $params['deep']){
                $select->join(array('b'=> $this->_tables['payroll']), 'a.payroll_id=b.payroll_id', array('period','ref_code','ref_id', 'status', 'total_amount'), 'left');
                $select->join(array('c'=>$this->_tables['payroll_component']), 'a.component_id=c.component_id', array('component_name'=> 'name', 'credit_type', 'component_parent_id'), 'left');
            }

            if(isset($params['filter_period']) && $params['filter_period']){
                $select->where(array('b.period'=> $params['filter_period']));
            }

            if(isset($params['payroll_id'])) {
                $select->where(array('a.payroll_id'=> $params['payroll_id']));
            }
        }
        
        try {
            // echo str_replace('"','',$select->getSqlString()).'<br />';  die();
            
            if($rows = $this->fetchAll($select)){
                $result = new \stdClass();
                $result->list = $rows;
                $result->pager = $pager;
                $result->countTotal = $rows->total;
                return $result;
            }
            return false;
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }

    public function save(\Neuron\Attendance\Evidence $period,$update = false)
    {
        try {
            /* mode update? */
            if ($update == true) {

                $update = $this->update()
                        ->table($this->_tables['attd_evidence'])
                        ->set($this->__($period->pull()))
                        ->where($this->__(array('attd_evidence_id' => $period->attd_evidence_id)));
                if ($this->execute($update)) {

                    /* update sukses, return user id */
                    return $period->attd_evidence_id;
                }
            } else {
                $period->attd_evidence_id = null ;
                $insert = $this->insert()
                        ->into($this->_tables['attd_evidence'])
                        ->autoincrement($this->_('attd_evidence_id'))
                        ->values($this->__($period->pull()));
                
                if ($id = $this->execute($insert)) {

                    /* insert sukses, return user id */
                    return $id;
                }
            }

            /* default gagal */
            return false;
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }   
    }

    public function terminate($id)
    {
        try {
            $delete = $this->delete()
                ->from($this->_tables['attd_evidence'])
                ->where($this->__(array('attd_evidence_id' => $id)));
                
            if($this->execute($delete)){
                return $id;
            }
            return false;
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }
}