<?php

namespace Neuron\Attendance;

use Neuron\Application\Framework as NAF;
use Neuron\Application\Framework\Hook\Manager;
use Neuron\Attendance\Controller\Profile;
use Neuron\Attendance\Controller\Widget\Employee\Manage;
use Neuron\Generic\Result;
use Neuron\Notification\Action;
use Neuron\Notification\Inbox;
use Neuron\Notification\Inboxes;
use Neuron\Notification\Template;
use Neuron\Notification\Template\Content;
use Neuron\Mvc\Layout as MVCLayout;
use Neuron\Attendance\Traits\Mapper;
use Neuron\Addon\Order\SimpleFlow\Flow;
use Neuron\Addon\Order\SimpleFlow\Flows;

class Module
{
    use Mapper;
    const DEFAULT_PERMISSION = 4;
    const ROLE_USER_CODE = 'user';

    const SUBTYPE_SPJ_INTOWN  = 2107;
    const SUBTYPE_SPJ_OUTOWN  = 2108;
    const SUBTYPE_SPJ_TRANSPORT  = 2109;

    public function __construct(){
        $manager = Manager::get();
        $manager->events()->register('naf_module_install', array($this, 'onModuleInstall'));
        $manager->events()->register('naf_module_uninstall', array($this, 'onModuleUninstall'));
        $manager->events()->register('naf_module_update', array($this, 'onModuleUpdate'), null, 2);
        $manager->events()->register('attd_check_geoloc', array($this, 'checkGeolocation'), null, 1);
        $manager->events()->register('attd_check_evidence', array($this, 'checkFlagEvidence'), null, 1);
        $manager->events()->register('naf_home_ready', array($this, 'onBelumAbsen'), 1, 3);
        // $manager->events()->register('theme_notif_init', array($this, 'displayModal'));
        // $manager->events()->register('theme_navbar_menu', array($this, 'buttonNavbarAttendance'));

        //event untuk execute action setelah approval
        $manager->events()->register('approval_after_approved', array($this, 'onApprovalApproved'), 0, 1);
        $manager->events()->register('approval_after_rejected', array($this, 'onApprovalRejected'), 0, 1);
        $manager->events()->register('leave_before_request', array($this, 'leaveBeforeRequest'), 0, 1);


        /* Untuk nambahin view ke employee/manage */
        $manager->events()->register('employee_manage_after_init', array($this, 'addModalTimetable'), 0);
        $manager->events()->register('actor_profile_widget', array($this, 'summaryAttendance'), 1,1);
    }

    function addModalTimetable(){
        $naf                = NAF\Entity::get();
        $config             = $naf->base()->getModuleConfig();
        $manageController   = new Manage($config);

        $viewModalTimetable = $manageController->modalTimetableAction();

        $controller         = $naf->getController();
        $resolver = $controller->getEvent()
            ->getApplication()
            ->getServiceManager()
            ->get('Zend\View\Resolver\TemplateMapResolver');

        $renderer = $controller->getEvent()
            ->getApplication()
            ->getServiceManager()
            ->get('Zend\View\Renderer\PhpRenderer');
        $renderer->setResolver($resolver);

        echo $renderer->render($viewModalTimetable);
    }

    public function leaveBeforeRequest($data = []){
        $result         = new Result();
        $naf            = NAF\Entity::get(new \Neuron\Mvc\Controller());
        $config         = $naf->base()->getModuleConfig();

        $naf            = NAF\Entity::get(new \Neuron\Mvc\Controller(($config)));
        $session        = $naf->session();
        $adapter        = $naf->getDb();
        $lang           = $session->get('lang');

        $translator     = new \Zend\I18n\Translator\Translator();
        if(file_exists(__DIR__ .'/Resource/Language/'.$lang.'.php')){
            $translator->addTranslationFile("phparray",__DIR__ .'/Resource/Language/'.$lang.'.php');
        }

        if($data['request_type'] != self::SUBTYPE_SPJ_INTOWN && $data['request_type'] != self::SUBTYPE_SPJ_OUTOWN && $data['request_type'] != self::SUBTYPE_SPJ_TRANSPORT){
            $attendances    = new Entities(Entity\Storage::factory($adapter,$config));
            $getAttd        = $attendances->load(null,Entity\Storage::LIST_ALL,[
                'ref_id'        => $data['ref_id'],
                'start_date'    => date('Y-m-d',strtotime($data['start_dtm'])),
                'end_date'    => date( 'Y-m-d', strtotime( $data['end_dtm'] ) )
            ]);

            if($getAttd->code == 0 && array_key_exists('list',$getAttd->data)){
                $attds  = $getAttd->data['list'];
                if(count($attds)){
                    $dates  = [];
                    foreach($attds as $attd){
                        $dates[] = date('Y-m-d',strtotime($attd['start_dtm']));
                    }
                    $result->code = 1;
                    $result->info = $translator->translate("Attendance data found in this date : ").implode(', ' ,$dates).$translator->translate(' Continuing this request will replacing the data.');
                }
            }
        }
        return $result;
    }

    public function summaryAttendance($user){
        $naf                = NAF\Entity::get();
        $config             = $naf->base()->getModuleConfig();
        $profileController  = new Profile($config);

        $widgetSummaryAttendance = $profileController->summaryAttendanceAction($user);

        $controller         = $naf->getController();
        $resolver = $controller->getEvent()
            ->getApplication()
            ->getServiceManager()
            ->get('Zend\View\Resolver\TemplateMapResolver');

        $renderer = $controller->getEvent()
            ->getApplication()
            ->getServiceManager()
            ->get('Zend\View\Renderer\PhpRenderer');
        $renderer->setResolver($resolver);

        echo $renderer->render($widgetSummaryAttendance);
    }

    public function onModuleInstall()
    {

        error_log('========================== onModuleInstall Function ======================================');

        /* get NAF entity */
        if ($naf = NAF\Entity::get()) {

            /* get dml file */
            if (strtolower($naf->database()->getType()) == 'oracle') {
                $file = __DIR__ . '/Resource/Db/Oci8.php';
            } else {
                $file = __DIR__ . '/Resource/Db/Mysql.php';
            }

            /* include it */
            $sqls = include($file);
            if (is_array($sqls)) {
                foreach ($sqls as $sql) {
                    $naf->database()->execute($sql);
                }
            }

            // $naf->module()->add('employee','master');

            /* register menu for this library 
            $naf->menu()->add('mnu_attd', null, 'Attendance', null, "Attendance");
            $naf->menu()->add('mnu_attd_manage', '/attendance/attendance/manage', 'Manage Attendance', 'mnu_attd', "Attendance");
            $naf->menu()->add('mnu_attd_request', '/attendance/form/request', 'Manage Request', 'mnu_attd', "Attendance");
            $naf->menu()->add('mnu_attd_set_by_admin', '/attendance/attendance/set-by-admin', 'Manual Attendance', 'mnu_attd', "Attendance");
            $naf->menu()->add('mnu_attd_timeshift_manage', '/attendance/timetable/manage', 'Manage Timeshift', 'mnu_attd', 'Attendance');
            $naf->menu()->add('mnu_attd_timeshift_change_shift', '/attendance/timetable/change-shift', 'Change Shift', 'mnu_attd', 'Attendance');
            $naf->menu()->add('mnu_attd_my_report', '/attendance/reporting/reporting-chart', 'My Report', 'mnu_attd', 'Attendance');
            $naf->menu()->add('mnu_attd_setting', '/approval/setting/', 'Approval Setting', 'mnu_attd', 'Attendance');
            $naf->menu()->add('mnu_attd_emp_report', '/attendance/reporting/monthly', 'Employee Report', 'mnu_attd', 'Attendance'); */
			
			/* Register Menu To Attendance */
			$naf->menu()->add('mnu_attd', null, 'Attendance', null, 'Attendance');
			$naf->menu()->add('mnu_attd_dashboard_summary', 'attendance/dashboard/summary', 'Dashboard Attendance', 'mnu_attd', 'Attendance');
			$naf->menu()->add('mnu_attd_timeshift_manage', '/attendance/timetable/manage', 'Manage Timeshift', 'mnu_attd', 'Attendance');
            $naf->menu()->add('mnu_attd_timeshift_change_shift', '/attendance/timetable/change-shift', 'Change Shift', 'mnu_attd', 'Attendance');
            $naf->menu()->add('mnu_attd_dashboard_summary_hr', '/attendance/dashboard/reporting-hr', 'Dashboard Attendance HR', 'mnu_reporting', 'Employee');
			
            /* Register Menu To Reporting */
			$naf->menu()->add('mnu_attd_my_report', '/attendance/reporting/reporting-chart', 'Reporting Attendance', 'mnu_reporting', 'Employee');            
            $naf->menu()->add('mnu_attd_emp_report', '/attendance/reporting/monthly', 'Employee Report', 'mnu_reporting', 'Employee');

            /* Create acl attd_all_org_list */
            $accessEntity       = new \Neuron\Access\Entity(\Neuron\Access\Entity\Storage::factory(
                $naf->getDb(),
                $naf->base()->getModuleConfig()
            ));
            $accessEntity->register('attd_all_org_list', 3, "View all organization", "Permission to list all organization", 'mnu_attd', null, 'Attendance');

            /* Grant to attd org list */
            $grant          = new \Neuron\Access\Grant(\Neuron\Access\Grant\Storage::factory(
                $naf->getDb(),
                $naf->base()->getModuleConfig()
            ));

            # Grant attd org list to admin
            $grant->register(1, 'attd_all_org_list', 4, 1, date('Y-m-d H:i:s'), '9999-09-09 00:00:00.0');
            $grant->register(1, 'mnu_reporting', 4, 1, date('Y-m-d H:i:s'), '9999-09-09 00:00:00.0');
            $grant->register(1, 'mnu_attd_dashboard_summary_hr', 4, 1, date('Y-m-d H:i:s'), '9999-09-09 00:00:00.0');

            $this->createNotifTemplate();
            $this->grantUser();
        }
    } //omi

    public function onModuleUninstall()
    {

        /* get NAF entity */
        if ($naf = NAF\Entity::get()) {

            /* deregister menu for this library 
            $naf->menu()->remove('mnu_attd');
            $naf->menu()->remove('mnu_attd_manage');
            $naf->menu()->remove('mnu_attd_request');
            $naf->menu()->remove('mnu_attd_set_by_admin');
            $naf->menu()->remove('mnu_attd_timeshift_manage');
            $naf->menu()->remove('mnu_attd_timeshift_change_shift');
            $naf->menu()->remove('mnu_attd_reporting');
            $naf->menu()->remove('mnu_attd_setting');
            $naf->menu()->remove('mnu_attd_my_report');
            $naf->menu()->remove('mnu_attd_emp_report'); */
			
			$naf->menu()->remove('mnu_attd');
            $naf->menu()->remove('mnu_attd_dashboard_summary');
            $naf->menu()->remove('mnu_attd_dashboard_summary_hr');
            $naf->menu()->remove('mnu_attd_timeshift_manage');
            $naf->menu()->remove('mnu_attd_timeshift_change_shift');			
            $naf->menu()->remove('mnu_attd_request');
            $naf->menu()->remove('mnu_attd_my_report');
            $naf->menu()->remove('mnu_attd_emp_report');

            // check kalo ada organisasi, karena geolocationnya jadi active, jadi harus update ke versi 1.5.9
            $config     = $naf->base()->getInstalledModule('internal');
            if (isset($config['organization'])) {
                $orgConfig     = $config['organization'];
                $installedVersion = $orgConfig['version'];
                $compareVersion     = version_compare($installedVersion, 'v1.5.9');
                if ($compareVersion === -1) {
                    $module     = new \Neuron\Application\Framework\Setup\Module();
                    $result     = $module->install(
                        \Neuron\Application\Framework\Setup\Module::MODULE_INTERNAL,
                        'organization',
                        'v1.5.8'
                    );
                }
            }
        }
    } //omu

    public function onModuleUpdate($new, $old)
    {
        if ($naf = NAF\Entity::get()) {
            /* deregister menu for this library
            $naf->menu()->remove('mnu_attd');
            $naf->menu()->remove('mnu_attd_manage');
            $naf->menu()->remove('mnu_attd_request');
            $naf->menu()->remove('mnu_attd_set_by_admin');
            $naf->menu()->remove('mnu_attd_timeshift_manage');
            $naf->menu()->remove('mnu_attd_timeshift_change_shift');
            $naf->menu()->remove('mnu_attd_reporting');
            $naf->menu()->remove('mnu_attd_setting');
            $naf->menu()->remove('mnu_attd_monthly_reporting');
            $naf->menu()->remove('mnu_attd_my_report');
            $naf->menu()->remove('mnu_attd_emp_report');

            $naf->menu()->add('mnu_attd', null, 'Attendance', null, "Attendance");
            $naf->menu()->add('mnu_attd_manage', '/attendance/attendance/manage', 'Manage Attendance', 'mnu_attd', "Attendance");
            $naf->menu()->add('mnu_attd_request', '/attendance/form/request', 'Manage Request', 'mnu_attd', "Attendance");
            $naf->menu()->add('mnu_attd_set_by_admin', '/attendance/attendance/set-by-admin', 'Manual Attendance', 'mnu_attd', "Attendance");
            $naf->menu()->add('mnu_attd_timeshift_manage', '/attendance/timetable/manage', 'Manage Timeshift', 'mnu_attd', 'Attendance');
            $naf->menu()->add('mnu_attd_timeshift_change_shift', '/attendance/timetable/change-shift', 'Change Shift', 'mnu_attd', 'Attendance');
            $naf->menu()->add('mnu_attd_my_report', '/attendance/reporting/reporting-chart', 'My Report', 'mnu_attd', 'Attendance');
            $naf->menu()->add('mnu_attd_setting', '/approval/setting/', 'Approval Setting', 'mnu_attd', 'Attendance');
            $naf->menu()->add('mnu_attd_emp_report', '/attendance/reporting/monthly', 'Employee Report', 'mnu_attd', 'Attendance'); */
			
			$naf->menu()->remove('mnu_attd_dashboard_summary');
			$naf->menu()->remove('mnu_attd_dashboard_summary_hr');
            $naf->menu()->remove('mnu_attd_timeshift_manage');
            $naf->menu()->remove('mnu_attd_timeshift_change_shift');			
            $naf->menu()->remove('mnu_attd_request');
            $naf->menu()->remove('mnu_attd_my_report');
            $naf->menu()->remove('mnu_attd_emp_report');
			
			/* Register Menu To Attendance */
			$naf->menu()->add('mnu_attd', null, 'Attendance', null, 'Attendance');
			$naf->menu()->add('mnu_attd_dashboard_summary', 'attendance/dashboard/summary', 'Dashboard Attendance', 'mnu_attd', 'Attendance');
			$naf->menu()->add('mnu_attd_timeshift_manage', '/attendance/timetable/manage', 'Manage Timeshift', 'mnu_attd', 'Attendance');
            $naf->menu()->add('mnu_attd_timeshift_change_shift', '/attendance/timetable/change-shift', 'Change Shift', 'mnu_attd', 'Attendance');
            $naf->menu()->add('mnu_attd_dashboard_summary_hr', '/attendance/dashboard/reporting-hr', 'Dashboard Attendance HR', 'mnu_reporting', 'Employee');
			
            /* Register Menu To Approval */
			//$naf->menu()->add('mnu_attd_request', '/attendance/form/request', 'Manage Request', 'mnu_approval_request', 'Approval');
			
			/* Register Menu To Reporting */
			$naf->menu()->add('mnu_attd_my_report', '/attendance/reporting/reporting-chart', 'Reporting Attendance', 'mnu_reporting', 'Employee');            
            $naf->menu()->add('mnu_attd_emp_report', '/attendance/reporting/monthly', 'Employee Report', 'mnu_reporting', 'Employee');


            /* Create acl attd_all_org_list */
            $accessEntity       = new \Neuron\Access\Entity(\Neuron\Access\Entity\Storage::factory(
                $naf->getDb(),
                $naf->base()->getModuleConfig()
            ));
            $accessEntity->register('attd_all_org_list', 3, "View all organization", "Permission to list all organization", 'mnu_attd', null, 'Attendance');

            /* Grant to attd org list */
            $grant          = new \Neuron\Access\Grant(\Neuron\Access\Grant\Storage::factory(
                $naf->getDb(),
                $naf->base()->getModuleConfig()
            ));

            # Grant attd org list to admin
            $grant->register(1, 'attd_all_org_list', 4, 1, date('Y-m-d H:i:s'), '9999-09-09 00:00:00.0');

            // check kalo ada organisasi, karena geolocationnya jadi active, jadi harus update ke versi 1.5.9
            $config     = $naf->base()->getInstalledModule('internal');
            if (isset($config['organization'])) {
                $orgConfig     = $config['organization'];
                $installedVersion = $orgConfig['version'];
                $compareVersion     = version_compare($installedVersion, 'v1.5.8');
                if ($compareVersion === -1) {
                    $module     = new \Neuron\Application\Framework\Setup\Module();
                    $result     = $module->install(
                        \Neuron\Application\Framework\Setup\Module::MODULE_INTERNAL,
                        'organization',
                        'v1.5.8'
                    );
                }
            }

            # Grant dashboard hr to admin
            // $accessEntity->register('cal_event_all_person',3,"View all person","View calendar of all person",'cal_event',null,'Calendar');
            $grant->register(1, 'mnu_attd_dashboard_summary_hr', 4, 3, date('Y-m-d H:i:s'), '9999-09-09 00:00:00.0');

            $this->updateTemplate();
            $this->grantUser();

            if (version_compare($old, 'v1.5.24') < 0) {
                $sql     = include(__DIR__ . '/Resource/Db/v1.5.24/Mysql.php');
                error_log('attendance compare ' . $old . ' is lower than v1.5.24');
                $rs     = array();
                foreach ($sql as $sql) {
                    $rs[] = $naf->database()->execute($sql);
                }
            }

            if (version_compare($old, 'v1.5.26') < 0) {
                $sql     = include(__DIR__ . '/Resource/Db/v1.5.26/Mysql.php');
                error_log('attendance compare ' . $old . ' is lower than v1.5.26');
                $rs     = array();
                foreach ($sql as $sql) {
                    $rs[] = $naf->database()->execute($sql);
                }
            }

            if (version_compare($old, 'v1.5.38') < 0) {
                $sql     = include(__DIR__ . '/Resource/Db/v1.5.38/Mysql.php');
                error_log('attendance compare ' . $old . ' is lower than v1.5.38');
                $rs     = array();
                foreach ($sql as $sql) {
                    $rs[] = $naf->database()->execute($sql);
                }
            }

            if (version_compare($old, 'v1.5.52') < 0) {
                $sql     = include(__DIR__ . '/Resource/Db/v1.5.52/Mysql.php');
                error_log('attendance compare ' . $old . ' is lower than v1.5.52');
                $rs     = array();
                foreach ($sql as $sql) {
                    $rs[] = $naf->database()->execute($sql);
                }
            }

            if (version_compare($old, 'v1.5.53') < 0) {
                $sql     = include(__DIR__ . '/Resource/Db/v1.5.53/Mysql.php');
                error_log('attendance compare ' . $old . ' is lower than v1.5.53');
                $rs     = array();
                foreach ($sql as $sql) {
                    $rs[] = $naf->database()->execute($sql);
                }
            }

            if (version_compare($old, 'v1.5.89') == -1) {
                error_log('executing query');
                $sql     = include(__DIR__ . '/Resource/Db/v1.5.89/Mysql.php');

                foreach ($sql as $sql) {
                    $rs[] = $naf->database()->execute($sql);
                }
            }

            if (version_compare($old, 'v1.5.101') == -1 || $new == '303-changeshift') {
                error_log('executing query removing foreign key');
                $sql     = include(__DIR__ . '/Resource/Db/v1.5.101/Mysql.php');
                foreach ($sql as $sql) {
                    $rs[] = $naf->database()->execute($sql);
                }
            }

            if (version_compare($old, 'v1.5.106') == -1 || $new == 'widget-today-status') {
                error_log("Grant today status widget");
                $this->grantTodayStatusWidget();
            }

            if (version_compare($old, 'v1.5.108') == -1 || $new == '331-standarisasi-api' || $new = 'master') {
                $sql     = include(__DIR__ . '/Resource/Db/v1.5.108/Mysql.php');
                foreach ($sql as $sql) {
                    $rs[] = $naf->database()->execute($sql);
                }
            }

            if (version_compare($old, 'v2.0.27') == -1 || $new == '394-upload-shift' || $new = 'master') {
                $sql     = include(__DIR__ . '/Resource/Db/v2.0.27/Mysql.php');
                foreach ($sql as $sql) {
                    $rs[] = $naf->database()->execute($sql);
                }
            }
        }
    }

    public function checkGeolocation($ref_id  = null){
        if (!class_exists('Neuron\Organization\Entity')) {
            $geolocation        = array(
                'is_active'     => false
            );
            return $geolocation;
        }


        if ($naf    = \Neuron\Application\Framework\Entity::get()) {
            $config             = $naf->base()->getModuleConfig();

            if (!isset($config['attendance'])) {
                $geolocation        = array(
                    'is_active'     => false
                );
                return $geolocation;
            }

            $attendanceConfig    = $config['attendance'];
            if (isset($attendanceConfig['geolocation'])) {

                $geolocation            = $attendanceConfig['geolocation'];
            } else {

                $geolocation        = array(
                    'is_active'     => false
                );
            }
            if ($geolocation['is_active'] == 1) {
                $db = $naf->getDb();
                $config = $naf->base()->getModuleConfig();
                
                if ($ref_id) {
                    $employee = new \Neuron\Employee\Entity(\Neuron\Employee\Entity\Storage::factory($db,$config));
                    $employee->load($ref_id);
                    $detail     = $employee->detail();
                    $geolocation        = array(
                        'is_active'     => boolval($detail->flag_geolocation),
                        'min_distance'  => $attendanceConfig['geolocation']['min_distance'] ?? 0
                    );
                }
            }

            return $geolocation;
        } else {

            return array(
                'is_active'     => false
            );;
        }
    }

    /**
     * untuk check apakah user memiliki flag_attd_evidence atau tidak
     * digunakan di wAttendance
     */
    public function checkFlagEvidence($ref_id = null)
    {
        if ($naf = \Neuron\Application\Framework\Entity::get()) {
            $sPersonMap  = \Neuron\Person\Entity\Storage::factory($naf->getDb(), $naf->base()->getModuleConfig());
            $mPersonMap = new \Neuron\Person\Entity($sPersonMap);
            $user_id = $naf->user()->id;
            $dataPerson = $mPersonMap->loadMap($user_id, 'user_id');
            $ref_id = @$dataPerson['ref_id'];
            if ($ref_id) {
                $sPersonDetail  = \Neuron\Employee\Detail\Storage::factory($naf->getDb(), $naf->base()->getModuleConfig());
                $mPersonDetail = new \Neuron\Employee\Detail($sPersonDetail);
                $mPersonDetail->load($ref_id, \Neuron\Employee\Detail\Storage::LOADBY_EMPID);
                return array('flag_attd_evidence' => $mPersonDetail->flag_attd_evidence);
            }
        }
    }

    public function displayModal()
    {
        if ($naf = NAF\Entity::get()) {
            $mAttendance = new \Neuron\Attendance\View();
            $mAttendance->displayModal();
            $mAttendance->requestDetail();
            return true;
        }
    }

    public function updateTemplate()
    {
        $result                 = new Result();
        if ($naf = NAF\Entity::get()) {
            $tables             = $naf->base()->getModuleConfig()['tables'];
            $db                 = new \Neuron\Db\Storage($naf->getDb(), $naf->base()->getModuleConfig());

            /* ============ DELETE REQUEST_APPROVE ================ */
            // Ambil seluruh template yang memiliki code 'REQUEST_APPROVE'
            $select             = $db->select()->from($tables['notification_templates']);
            $select->where(array('template_code' => 'REQUEST_APPROVE'));
            $templates          = $db->fetchAll($select);

            $template_ids   = array();

            if ($templates) {
                foreach ($templates as $template) {
                    $template_ids[] = $template['notif_template_id'];
                }
            }

            // Delete semua not_content yang memiliki template_id diatas;
            $where              = implode(",", $template_ids);

            // Delete notification_contents
            $delete             = $db->delete()->from($tables['notification_contents']);
            $delete->where("notif_template_id in ($where)");

            if ($deleteContent = $db->execute($delete)) {
                $delete             = $db->delete()->from($tables['notification_templates']);
                $delete->where("notif_template_id in ($where)");
                $db->execute($delete);
            }

            /* ============ DELETE REQUEST_REJECT ================ */
            // Ambil seluruh template yang memiliki code 'REQUEST_APPROVE'
            $select             = $db->select()->from($tables['notification_templates']);
            $select->where(array('template_code' => 'REQUEST_REJECT'));
            $templates          = $db->fetchAll($select);

            $template_ids   = array();

            if ($templates) {
                foreach ($templates as $template) {
                    $template_ids[] = $template['notif_template_id'];
                }
            }

            // Delete semua not_content yang memiliki template_id diatas;
            $where              = implode(",", $template_ids);

            // Delete notification_contents
            $delete             = $db->delete()->from($tables['notification_contents']);
            $delete->where("notif_template_id in ($where)");

            if ($deleteContent = $db->execute($delete)) {
                $delete             = $db->delete()->from($tables['notification_templates']);
                $delete->where("notif_template_id in ($where)");
                $db->execute($delete);
            }


            /* ============= CREATE NEW TEMPLATE ========== */
            $result     = $this->createNotifTemplate();
            return $result;
        }
    }

    public function createNotifTemplate()
    {
        if ($naf     = NAF\Entity::get()) {

            /* =====================CREATE REQUEST_APPROVE TEMPLATE=================== */
            $newTemplate        = array(
                "notif_profile_id" => 1,
                "template_code" => "REQUEST_APPROVE",
                "template_name" => "Attendance Request Approve",
                "description" => "Notifikasi ketika request di approve dikirim ke Requester",
                "icon" => null
            );
            $template           = new Template(Template\Storage::factory($naf->getDb(), $naf->base()->getModuleConfig()));
            $result             = $template->saveEx($newTemplate);
            if ($result->code != 0) {
                return $result;
            }

            // Create action
            $action             = new Action(Action\Storage::factory($naf->getDb(), $naf->base()->getModuleConfig()));
            $result             = $action->saveEx(array(
                'notif_action_code'         => 'ATTENDANCE_DETAIL_REQ',
                'notif_action_type_id'      => 3,
                'reference'                 => "openDetailRequest",
                'data'                      => null,
                'remark'                    => "Detail Request"
            ));
            if ($result->code != 0) {
                $template->delete($template->id);
                return $result;
            }

            // Create Content;
            $content            = new Content(Content\Storage::factory($naf->getDb(), $naf->base()->getModuleConfig()));
            $result             = $content->saveEx(array(
                "notif_template_id" => $template->id,
                "subject" => "Your %request_type% request has been approved",
                "content" => "Your %request_type% request has been approved",
                "notif_action_code" => "ATTENDANCE_DETAIL_REQ"
            ));

            /* =====================CREATE REQUEST_REJECT TEMPLATE=================== */
            $newTemplate        = array(
                "notif_profile_id" => 1,
                "template_code" => "REQUEST_REJECT",
                "template_name" => "Attendance Request Rejected",
                "description" => "Notifikasi ketika request di reject dikirim ke Requester",
                "icon" => null
            );
            $template           = new Template(Template\Storage::factory($naf->getDb(), $naf->base()->getModuleConfig()));
            $result             = $template->saveEx($newTemplate);
            if ($result->code != 0) {
                return $result;
            }

            // Create action
            $action             = new Action(Action\Storage::factory($naf->getDb(), $naf->base()->getModuleConfig()));
            $result             = $action->saveEx(array(
                'notif_action_code'         => 'ATTENDANCE_DETAIL_REQ',
                'notif_action_type_id'      => 3,
                'reference'                 => "openDetailRequest",
                'data'                      => null,
                'remark'                    => "Detail Request"
            ));
            if ($result->code != 0) {
                $template->delete($template->id);
                return $result;
            }

            // Create Content;
            $content            = new Content(Content\Storage::factory($naf->getDb(), $naf->base()->getModuleConfig()));
            $result             = $content->saveEx(array(
                "notif_template_id" => $template->id,
                "subject" => "Your %request_type% request has been rejected",
                "content" => "Your %request_type% request has been rejected",
                "notif_action_code" => "ATTENDANCE_DETAIL_REQ"
            ));

            return $result;
        }
    }

    public function grantUser()
    {
        $result = new Result();
        if ($naf = NAF\Entity::get()) {

            $storage = \Neuron\Access\Grant\Storage::factory($naf->getDb(), $naf->base()->getModuleConfig());
            $grant = new \Neuron\Access\Grant($storage);

            $storage = \Neuron\Access\Role\Storage::factory($naf->getDb(), $naf->base()->getModuleConfig());
            $role = new \Neuron\Access\Role($storage);
            $role->load("user",  \Neuron\Access\Role\Storage::LOADBY_CODE);

            $result = $grant->register(
                \Neuron\Access\Grant\Type\Storage::ROLE,
                'mnu_attd',
                self::DEFAULT_PERMISSION,
                $role->id,
                $storage->now(),
                null
            );
			
			$result = $grant->register(
                \Neuron\Access\Grant\Type\Storage::ROLE,
                'mnu_reporting',
                self::DEFAULT_PERMISSION,
                $role->id,
                $storage->now(),
                null
            );

            $result = $grant->register(
                \Neuron\Access\Grant\Type\Storage::ROLE,
                'mnu_attd_dashboard_summary',
                self::DEFAULT_PERMISSION,
                $role->id,
                $storage->now(),
                null
            );

            $result = $grant->register(
                \Neuron\Access\Grant\Type\Storage::ROLE,
                'mnu_attd_my_report',
                self::DEFAULT_PERMISSION,
                $role->id,
                $storage->now(),
                null
            );

            return $result;
        }
    }

    public function grantTodayStatusWidget()
    {
        // Creating widget object
        $naf    = NAF\Entity::get();
        $storage = \Neuron\Access\Entity\Storage::factory($naf->getDb(), $naf->base()->getModuleConfig());
        $accessEntity = new \Neuron\Access\Entity($storage);
        $result = $accessEntity->register(
            "widget_widget-attd-today-status",
            4,
            "Today Status",
            "Today Status",
            null,
            NULL,
            "Attendance"
        );

        /* Granting Widget object to user & admin */
        $storage = \Neuron\Access\Grant\Storage::factory($naf->getDb(), $naf->base()->getModuleConfig());
        $grant = new \Neuron\Access\Grant($storage);
        $storage = \Neuron\Access\Role\Storage::factory($naf->getDb(), $naf->base()->getModuleConfig());
        $role = new \Neuron\Access\Role($storage);
        $role->load("admin",  \Neuron\Access\Role\Storage::LOADBY_CODE);
        //grant ke role admin
        $grant->register(
            \Neuron\Access\Grant\Type\Storage::ROLE,
            "widget_widget-attd-today-status",
            self::DEFAULT_PERMISSION,
            $role->id,
            $storage->now(),
            null
        );
        $role->clear();

        $role->load("user",  \Neuron\Access\Role\Storage::LOADBY_CODE);
        //grant ke role user
        $grant->register(
            \Neuron\Access\Grant\Type\Storage::ROLE,
            "widget_widget-attd-today-status",
            self::DEFAULT_PERMISSION,
            $role->id,
            $storage->now(),
            null
        );
    }

    public function buttonNavbarAttendance()
    {
        echo '
        <li class="nav-item" data-toggle="tooltip" data-placement="bottom" title="" data-original-title="Attendance">
            <a class="nav-link waves-effect waves-light" href="#" onclick="openAttendance()"><i class="fas fa-calendar"></i></a>
        </li>';
    }

    public function modalAttendance()
    {
        $naf        = NAF\Entity::get();
        
        $controller = $naf->getController();
        $controller->using('jquery.validate.attendanceModal',[
            'https://cdnjs.cloudflare.com/ajax/libs/jquery-validate/1.19.2/jquery.validate.min.js'
        ],MVCLayout\Library::ON_FOOTER);
        $resolver = $controller
            ->getEvent()
            ->getApplication()
            ->getServiceManager()
            ->get('Zend\View\Resolver\TemplateMapResolver');

        //tambah mapping view ke resolver
        $viewTemplate = 'neuron/attendance/attendance/widget-attendance';

        // $resolver->add($viewTemplate,  __DIR__.'/View/Attendance/wAttendance.phtml');
        /** script js nya disimpen di file aja biar bisa di translate kalau ada kata2.*/

        //bikin view baru + set template sesuai resolver
        $view = new \Zend\View\Model\ViewModel();
        $view->setTemplate($viewTemplate);

        if ($lang = $naf->session()->get('lang')) {
            $lang = $lang;
        } else {
            $lang = $naf->user()->language;
        }

        //set variable ke view
        $view->setVariables([
            'lang' => $lang
        ]);

        //render view
        $renderer = $naf->getController()
            ->getEvent()
            ->getApplication()
            ->getServiceManager()
            ->get('Zend\View\Renderer\PhpRenderer');
        $renderer->setResolver($resolver);
        echo $renderer->render($view);
    }

    public function onApprovalApproved($dataRequest){
        if ($naf = NAF\Entity::get()) {
            if($dataRequest['order_type_id'] == 2100){
                $result = new Result();

                $template_code 	= 'REQUEST_APPROVE'; //template notif
                    
                // Create attendance data. changeshift / overtime / attendance;
                $requestType    = new \Neuron\Attendance\Request(\Neuron\Attendance\Request\Storage::factory(
                    $naf->getDb(), $naf->base()->getModuleConfig()
                ));
                
                $createRequestData = $requestType->createRequestTypeData($dataRequest);
                if($createRequestData->code != 0){
                    $result->info   = $naf->getController()->translate($result->info);
                    return $result;
                }

                ## NOTIFICATION HERE
				# Kirim notifikasi kalo request di approve / di reject.
				$inboxStorage   = \Neuron\Notification\Inbox\Storage::factory(
					$naf->getDb(), $naf->base()->getModuleConfig(),
					\Neuron\Notification\Profile::TYPE_FCM,
					$dataRequest['create_user_id']
				);
				$inbox      = new \Neuron\Notification\Inbox($inboxStorage);
				$inbox->setPublicUrl($naf->getController()->getPublicUrl());

				# check terlebihdahuu apakah sudah ada tabelnya atau belum
				$res = @$inbox->pushNotification(
					// $userId,
					$dataRequest['create_user_id'],
					$template_code ,
					array(
						'request_type' => $dataRequest['order_type_name'],
					),
					array(
						'request_id' 	=> $dataRequest['order_id'],
					)
				);
				
				$array = array(
                    "userId" => $dataRequest['create_user_id'],
                    "template_code" => $template_code,
                    "request_type" => $dataRequest['order_status_name'],
                    "request_id" => $dataRequest['order_id'],
				);

                return $result;
            }else if($dataRequest['order_type_id'] == 2200){
                if($dataRequest['order_subtype_id'] != self::SUBTYPE_SPJ_INTOWN && $dataRequest['order_subtype_id'] != self::SUBTYPE_SPJ_OUTOWN && $dataRequest['order_subtype_id'] != self::SUBTYPE_SPJ_TRANSPORT){
                    $refId              = $dataRequest['create_user_id'];
                    if(class_exists('\Neuron\Employee\Entity')){
                        $map            = new \Neuron\Employee\Map(\Neuron\Employee\Map\Storage::factory($naf->getDb(),$naf->base()->getModuleConfig()));
                        $map->load($refId,\Neuron\Employee\Map\Storage::LOADBY_USER_ID);
                        if($map->user_map_id){
                            $refId      = $map->person_id;
                        }
                    }
                    // Kalo ada leave di approve check punya attendance ga
                    $attendances        = new Entities(Entity\Storage::factory($naf->getDb(),$naf->base()->getModuleConfig()));
                    $attendances->load(null,Attendance\Storage::LOADBY_ID,[
                        'ref_id'        => $refId,
                        'start_date'    => date('Y-m-d',strtotime($dataRequest['xd1'])),
                        'end_date'      => date('Y-m-d',strtotime($dataRequest['xd2'])),
                    ]);
                    $updated            = [];
                    foreach($attendances as $attendance){
                        $attendance->type_id    = 4;
                        if($attendance->save(true)){
                            $updated[] = $attendance->pull();
                        }
                    }
                }
            }
        }
    }

    public function onApprovalRejected($dataRequest){
        if ($naf = NAF\Entity::get()) {
            if($dataRequest['order_type_id'] == 2100){
				
				if($dataRequest['xn5'] == 3){					
					foreach($dataRequest['flow'] as $dataFlow){						
						if($dataFlow['order_flow_task_code'] == 'approval'){
							$generic		= new \Neuron\Attendance\Generic;
							$addonConfig	= Generic::getAddonConfig();
							$flowStorage 	= new \Neuron\Addon\Order\SimpleFlow\Flow(\Neuron\Addon\Order\SimpleFlow\Flow\Storage::factory($naf->getDb(), $addonConfig));
							
							if(empty($dataFlow['remark_out'])){
								$dataFlow['remark_out'] = 'Autoreject';
							}
							
							if(empty($dataFlow['processed_by_id'])){
								$dataFlow['processed_by_id'] = $dataFlow['assigned_to_id'];
							}
							
							if(empty($dataFlow['update_dtm'])){
								$dataFlow['update_dtm'] = date("Y-m-d H:m:s");
							}
							
							$flowStorage->orderId				= $dataRequest['order_id'];
							$flowStorage->id 					= $dataFlow['order_flow_id'];
							$flowStorage->statusId			 	= 3;
							$flowStorage->typeId			 	= 1;
							$flowStorage->taskCode			 	= "approval";
							$flowStorage->remarkOut           	= $dataFlow['remark_out'];
							$flowStorage->processor		     	= $dataFlow['processed_by_id'];
							$flowStorage->assignee		      	= $dataFlow['assigned_to_id'];
							$flowStorage->update_dtm			= $dataFlow['update_dtm'];
							
							$saveFlow 		= $flowStorage->save(true, \Neuron\Addon\Order\SimpleFlow\Flow\Storage::SAVEBY_ID);	
						}
					}
				}
				
                $result = new Result();
                $template_code 	= 'REQUEST_REJECT'; //template notif

                ## NOTIFICATION HERE
				# Kirim notifikasi kalo request di approve / di reject.
				$inboxStorage   = \Neuron\Notification\Inbox\Storage::factory(
					$naf->getDb(), $naf->base()->getModuleConfig(),
					\Neuron\Notification\Profile::TYPE_FCM,
					$dataRequest['create_user_id']
				);
				$inbox      = new \Neuron\Notification\Inbox($inboxStorage);
				$inbox->setPublicUrl($naf->getController()->getPublicUrl());

				# check terlebihdahuu apakah sudah ada tabelnya atau belum
				$res = @$inbox->pushNotification(
					// $userId,
					$dataRequest['create_user_id'],
					$template_code ,
					array(
						'request_type' => $dataRequest['order_type_name'],
					),
					array(
						'request_id' 	=> $dataRequest['order_id'],
					)
				);
				
				$array = array(
                    "userId" => $dataRequest['create_user_id'],
                    "template_code" => $template_code,
                    "request_type" => $dataRequest['order_status_name'],
                    "request_id" => $dataRequest['order_id'],
				);
                return $result;
            }
        }
    }

    public function onBelumAbsen($view, $controller, $inside) {
        if ($naf = NAF\Entity::get()) {
            /* Sudah login? */
            if ($inside) {
                /* Cek absen */
                $user_id = $naf->session()->owner();
                $mapper     = $this->getMapper();
                $employee   = $mapper->data['employee'];
                $entity     = new \Neuron\Attendance\Entity(\Neuron\Attendance\Entity\Storage::factory($naf->getDb(), $naf->base()->getModuleConfig()));
                $canAttend    = $entity->canAttend(['ref_id'=>  $employee->id]);
                if ($canAttend->data['autoRedirect']) {
                    header('Location: ' . $controller->getPublicURL() . '/attendance/dashboard/form');
                    die();
                } 
            }
        }
    }
}
