<?php
namespace Neuron\Attendance;

use Neuron\Generic\Result;
use Neuron\Person\Entities;
use Neuron\Person\Entity;
use Neuron\Attendance\Entity as AttendanceEntity;
use Neuron\Attendance\Entities as AttendanceEntities;

use Neuron\Leave\Model\Entity as LeaveEntity;
use Neuron\Leave\Model\Entities as LeaveEntities;

use Neuron\Leave\Model\Type as TypeEntity;
use Neuron\Leave\Model\Types as TypesEntities;

use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\IOFactory;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;
use PhpOffice\PhpSpreadsheet\Cell\DataValidation;
use PhpOffice\PhpSpreadsheet\Worksheet\Worksheet;
use PhpOffice\PhpSpreadsheet\Cell\Coordinate;
use Neuron\Attendance\Generic;
use Neuron\Attendance\Traits\Model;
use PhpOffice\PhpSpreadsheet\Style\Alignment;
use PhpOffice\PhpSpreadsheet\Style\Color;
use PhpOffice\PhpSpreadsheet\Style\Fill;
use Zend\Mvc\Router\Http\Part;

class Reporting extends \Neuron\Generic\Entity\Mapper {
	use Model;

	protected $_storage;

	const WEEKLY 	 =0;
	const MONTHLY 	 =1;
	CONST ANNUALLY 	 =2;

	public function __construct(\Neuron\Attendance\Reporting\Storage\Skeleton $storage, $data= array()) {
			
		$this->_storage = $storage;

		parent::__construct(
	        $data, 
	        array(), 
	        parent::CASE_INSENSITIVE
	    );
	}

	public function reportAttendance($params = array()) {
		if($params['filter']['type']== 1){
			$ts = strtotime(date('y-m-d'));
			$start = (date('w', $ts) == 0) ? $ts : strtotime('last sunday', $ts);
			$params['start_date'] = date('Y-m-d', $start);
			$params['end_date'] = date('Y-m-d', strtotime('next saturday', $start));
		}else if($params['filter']['type']== 2){
			$year = ($params['year'] && $params['year'] != '')?  $params['year']: date('Y');
			$month = ($params['month'] && $params['month'] != '')? $params['month']: date('m');
			$start = date($year.'-'.$month.'-01');
			$end = date($year.'-'.$month.'-t');
			$params['start_date'] = date('Y-m-d', strtotime($start));
			$params['end_date'] = date('Y-m-d', strtotime($end));
		}else{
			$year = ($params['year'] && $params['year'] != '')?  $params['year']: date('Y');
			$start = date($year.'-01-01');
			$end = date($year.'-12-t');
			$params['start_date'] = date('Y-m-d', strtotime($start));
			$params['end_date'] = date('Y-m-d', strtotime($end));
		}
		$result = $this->_storage->reportAttendance($params);
		$arrref = array();
		if ($result) {
			foreach ($result as $data) {
				$data['id'] = intval($data['id']);
				$data['total_attend'] = isset($data['total_attend'])?intval($data['total_attend']):intval(0);
				array_push($arrref, $data);
			}
		}

		$total = $this->_storage->getTotalRows();
		// $total = 0;
		$page =$params['current'];
		$rowCount = $params['rowCount'];
		$retval = array(
			'current' => $page,
			'rowCount' => $rowCount,
			'rows' => $arrref,
			'total' => $total
		);

		return $retval;
	}

	public function reportingTable($post){
		$leaveExists = class_exists('Neuron\Leave\Model\Entity');

		$persons 	= new Entities(Entity\Storage::factory($this->_storage->getAdapter(), $this->_storage->getConfig()));
		
		$search 	= $post['search']['value'];


		$page 		= ($post['start'] + $post['length']) / $post['length'];
		$records 	= $post['length'];
		$nameSearch = $post['name'] ?? null;
		$loadEmpParams = [
			'page' 	=> $page,
			'records' => $records,
			'status_id' => 1, # Status Aktif,
			'org_id' => $post['organization_id'] ?? null,
			'keyword' 	=> $nameSearch ?? $post['search']['value']
		];

		$employees 	= new \Neuron\Employee\Entities(\Neuron\Employee\Entity\Storage::factory($this->getDb(),$this->getConfig()));
		$listEmployees = $employees->load($post['search']['value'],\Neuron\Employee\Entity\Storage::LISTBY_PAGE,$loadEmpParams);
		$employeeIds 	= [];
		$personIds		= [];
		foreach($employees as $employee){
			$employeeIds[] = $employee->id;
			$personIds[] = $employee->person_id;
		}

		// getCountTotal
		$totalAttendances 	= $this->countTotal(1,$post['start_date'],$post['end_date'],$employeeIds);
		$totalOvertimeDay	= $this->countTotal(2,$post['start_date'],$post['end_date'],$employeeIds);
		$totalAbsent 		= $this->countTotal(3,$post['start_date'],$post['end_date'],$employeeIds);
		$totalOvertime 		= $this->getTotalOvertime($post['start_date'],$post['end_date'],$employeeIds);
		if($leaveExists){
			$totalLeave 		= $this->getTotalLeave($post['start_date'],$post['end_date'],$personIds);
		}
		$totalAttendanceInvalids 	= $this->getTotalLogInvalid(1,$post['start_date'],$post['end_date'],$personIds);
		
		// satu lagi, ngeload data terlambat harus dikeluarin dari looping
		$resultData = array();
		foreach($employees as $employee){
			$totalLate			= $this->getTotalLate($post['start_date'],$post['end_date'],$employee->id)->data;
			$person 			= $employee->person();
			$tempResult 	= array(
				'ref_id' 			=> $employee->id,
				'ref_name' 			=> $person->first_name.' '.$person->last_name,
				'code' 				=> $employee->code,
				'total_overtime' 	=> array_key_exists($employee->id,$totalOvertime) ? $totalOvertime[$employee->id] : 0, 
				'total_late'		=> $totalLate['totalMinuteLate'],
				'total_late_day'	=> $totalLate['totalDayLate'],
				'total_attend'		=> array_key_exists($employee->id,$totalAttendances) ? intval($totalAttendances[$employee->id]) : 0,
				'total_overtime_day'=> array_key_exists($employee->id,$totalOvertimeDay) ? intval($totalOvertimeDay[$employee->id]) : 0,
				'total_absent' 		=> array_key_exists($employee->id,$totalAbsent) ? intval($totalAbsent[$employee->id]) : 0,
				'total_invalid_attd'=> array_key_exists($employee->person_id,$totalAttendanceInvalids) ? intval($totalAttendanceInvalids[$employee->person_id]) : 0
			);

			if($leaveExists){
				if(isset($totalLeave[$employee->person_id])){
					$tempResult['total_leave'] = $totalLeave[$employee->person_id];
				}else{
					$tempResult['total_leave'] = 0;
				}
			}

			//User_ID
			$map 			= new \Neuron\Person\Map(
				\Neuron\Person\Map\Storage::factory(
					$this->_storage->getAdapter(),
					$this->_storage->getConfig()
				)
			);
			$map->load($employee->person_id,\Neuron\Person\Map\Storage::LOADBY_PERSON_ID);
			$tempResult['user_id'] = $map->user_id;

			$resultData[] 	= $tempResult;
		}

		$dataTableResult 		= array(
			'draw' 				=> $post['draw'],
			'recordsTotal' 		=> $listEmployees->data['countTotal'],
			'recordsFiltered' 	=> $listEmployees->data['countTotal'],
			'data' 				=> $resultData
		);

		return $dataTableResult;
	}

	public function countTotal($type,$start_date,$end_date,$personIds){
		$lists = $this->_storage->countTotal($type,$start_date,$end_date,$personIds);
		$data = [];
		if($lists){
			foreach ($lists as $list){
				$data[$list['ref_id']] = $list['countTotal'];
			}
		}
		return $data;
	}
	public function getTotalOvertime($start_date,$end_date,$personIds){
		$data 	= [];

		$listMinutes 		= $this->_storage->totalOvertime($start_date,$end_date,$personIds);
		if($listMinutes){
			foreach($listMinutes as $row){
				$datetime1 = date_create($row['overtime_start_dtm']);
				$datetime2 = date_create($row['overtime_end_dtm']);
				$interval = date_diff($datetime1, $datetime2)->format('%h');
				if(isset($data[$row['ref_id']])){
					$data[$row['ref_id']] += $interval;
				}else{
					$data[$row['ref_id']] = $interval;
				}
			}
		}
		return $data;
	}
	public function downloadReporting($post = array()){
		//init spreadsheet
        $spreadsheet = new Spreadsheet();
        $worksheet = new Worksheet();
		$spreadsheet->disconnectWorksheets();


		//init header spreadsheet
		$header = array(
			'Kode',
			'Nama',
			'Jumlah Kehadiran',
			'Jumlah Tidak Hadir',
			'Jumlah Lembur'
		);

		//set Header
        $cell = $worksheet->getCellByColumnAndRow(count($header) + 1, 1);
        if ($header) {
            foreach ($header as $key => $value) {
				$cell = $worksheet->getCellByColumnAndRow($key + 1, 1);
				$worksheet->getColumnDimensionByColumn($key + 1)->setAutoSize(true);
                $worksheet->setCellValueByColumnAndRow($key + 1, 1, $value);
            }
        }

		
		$persons 	= new Entities(Entity\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
		$search 	= $post['filter'];

		$loadPersonParams = array(
			'search' => array(
				'value' => $post['search']['value'],
			),
		);

		$personList = $persons->load($search,Entity\Storage::LIST_ALL,$loadPersonParams);
		
		$resultData = array();
		
		if($personList->code == 0){
			if($personList->data['countTotal']){
				$persons = $personList->data['rows'];
				foreach($persons as $key => $person){
					$totalAttendance 	= $this->countTotal(1,$post['start_date'],$post['end_date'],array('ref_id' => $person['id']));
					$totalAbsent 	= $this->countTotal(3,$post['start_date'],$post['end_date'],array('ref_id' => $person['id']));
					$totalOvertime 		= intval($this->getTotalOvertime($post['start_date'],$post['end_date'],$person['id']));

					$resultData[] 	= array(
						'ref_name' 			=> $person['first_name'].' '.$person['last_name'],
						'code' 				=> $person['code'],
						'total_attend'		=> $totalAttendance->data['total'],
						'total_overtime' 	=> $totalOvertime
					);
					$worksheet->setCellValueByColumnAndRow(1, $key + 2, $person['code']);
					$worksheet->setCellValueByColumnAndRow(2, $key + 2, $person['first_name'].' '.$person['last_name']);
					$worksheet->setCellValueByColumnAndRow(3, $key + 2, $totalAttendance->data['total']);
					$worksheet->setCellValueByColumnAndRow(4, $key + 2, $totalAbsent->data['total']);
					$worksheet->setCellValueByColumnAndRow(5, $key + 2, $totalOvertime);
				}
			}
		}

		$spreadsheet->addSheet($worksheet, 1);

		$sheet = $spreadsheet->getActiveSheet();
		$spreadsheet->getActiveSheet()->getStyle('A:B')->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_LEFT);
		$spreadsheet->getActiveSheet()->getStyle('C:E')->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_RIGHT);

        $writer = IOFactory::createWriter($spreadsheet, "Xlsx"); //Xls is also possible
        $filename = 'Reporting-Attendance.xlsx';

        header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
        header('Content-Disposition: attachment; filename="' . $filename);
        header('Cache-Control: max-age=0');
        ob_end_clean();
        $writer->save('php://output', 'xlsx');
        exit();
	}
	public function downloadDetails($post = array()){
		$spreadsheet 		= new Spreadsheet();
		$worksheet 			= new Worksheet();
		$listDate 	= Generic::dateToArray($post['start_date'],$post['end_date']);
		$countDayBetweenDate 	= count($listDate);

		$totalColumn 		= $countDayBetweenDate + 9;

		/* Create Header */
			$sheet = $spreadsheet->getActiveSheet();
			$sheet->setCellValue('A1', 'No')->getColumnDimension('A')->setAutoSize(true);
			$sheet->setCellValue('B1', 'NIK')->getColumnDimension('B')->setAutoSize(true);
			$sheet->setCellValue('C1', 'Nama Karyawan')->getColumnDimension('C')->setAutoSize(true);
			$sheet->setCellValue('D1', 'Persentase')->getColumnDimension('D')->setAutoSize(true);
			$sheet->mergeCells('A1:A2');
			$sheet->mergeCells('B1:B2');
			$sheet->mergeCells('C1:C2');
			$sheet->mergeCells('D1:D2');
			
			/* Bikin periode */
			$sheet->mergeCellsByColumnAndRow(5,1,$countDayBetweenDate +4 ,1);
			$sheet->setCellValue('E1','PERIODE');

			/* Looping Tanggal untuk dibawah periode */
			$startingColumn 		= 5; /* Mulai dari E untuk periodnya */
			$dateColumnIndex 		= [];
			foreach($listDate as $date){
				$dateColumnIndex[$date] 	= $startingColumn;
				$sheet->setCellValueByColumnAndRow($startingColumn,2,$date)
					->getColumnDimensionByColumn($startingColumn)
					->setAutoSize(true);
				$startingColumn++;
			}
			$columnTotalAttend 	= $countDayBetweenDate+5;
			$columnLate 		= $countDayBetweenDate+6;
			$columnTotalHour 	= $countDayBetweenDate+7;
			$columnTotalLeave 	= $countDayBetweenDate+8;
			$columnTotalHoliday	= $countDayBetweenDate+9;

			/* Isi text untuk headernya */
			$sheet->setCellValueByColumnAndRow($columnTotalAttend,1,'Masuk (Hari)');
			$sheet->setCellValueByColumnAndRow($columnLate,1,'Keterlambatan (Menit)');
			$sheet->setCellValueByColumnAndRow($columnTotalHour,1,'Total Jam Kerja (Jam)');
			$sheet->setCellValueByColumnAndRow($columnTotalLeave,1,'Cuti (Hari)');
			$sheet->setCellValueByColumnAndRow($columnTotalHoliday,1,'Libur (Hari)');

			/* merge column nya kebawah */
			$sheet->mergeCellsByColumnAndRow($columnTotalAttend,1,$columnTotalAttend,2);
			$sheet->mergeCellsByColumnAndRow($columnLate,1,$columnLate,2);
			$sheet->mergeCellsByColumnAndRow($columnTotalHour,1,$columnTotalHour,2);
			$sheet->mergeCellsByColumnAndRow($columnTotalLeave,1,$columnTotalLeave,2);
			$sheet->mergeCellsByColumnAndRow($columnTotalHoliday,1,$columnTotalHoliday,2);

			/* set autowidth ya kk */
			$sheet->getColumnDimensionByColumn($columnTotalAttend)->setAutoSize(1==1); /* biar fancy aja dikasih 1 */					
			$sheet->getColumnDimensionByColumn($columnLate)->setAutoSize(1==1); /* biar fancy aja dikasih 1 */
			$sheet->getColumnDimensionByColumn($columnTotalHour)->setAutoSize(1==1); /* biar fancy aja dikasih 1 */
			$sheet->getColumnDimensionByColumn($columnTotalLeave)->setAutoSize(1==1); /* biar fancy aja dikasih 1 */
			$sheet->getColumnDimensionByColumn($columnTotalHoliday)->setAutoSize(1==1); /* biar fancy aja dikasih 1 */

		
		/* styling header cell background */
			$headerStyle = $sheet->getStyleByColumnAndRow(1,1,$totalColumn,2);
			$styleArray = [
				'font' => [
					'bold' => true,
				],
				'alignment' => [
					'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
					'vertical' 	=> Alignment::VERTICAL_CENTER
				],
				'borders' => [
					'allBorders' => [
						'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
						'color' => ['argb' => Color::COLOR_BLACK],
					],
				],
				'fill' => [
					'fillType' => Fill::FILL_SOLID,
					'startColor' => [
						'argb' => Color::COLOR_GREEN,
					],
				],
			];
			$headerStyle->applyFromArray($styleArray);
			$sheet->getStyle('B2:B9999')->getAlignment()->setHorizontal('left');
		
		/* Fill data */

			$persons 		= new \Neuron\Person\Entities(\Neuron\Person\Entity\Storage::factory(
				$this->_storage->getAdapter(),
				$this->_storage->getConfig()
			));

			$loadEmpParams = [
				'status_id' => 1, # Status Aktif,
				'org_id' => $post['organization_id'] ?? null, 
				'keyword' 	=> $post['search'] ?? null
			];

			$employees 		= new \Neuron\Employee\Entities(\Neuron\Employee\Entity\Storage::factory($this->getDb(),$this->getConfig()));
			$listEmployee 	= $employees->load(null,\Neuron\Employee\Entity\Storage::LIST_ALL,$loadEmpParams);

			/* Get Leave Types */
			$typeEntities 	= new TypesEntities(TypeEntity\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
			$listTypes 		= $typeEntities->load(null,TypeEntity\Storage::LIST_ALL,$loadAttendanceParams);
			
			if($listEmployee->code == 0){
				$no  			= 1;
				$startingRow 	= 3;
				$indexRow 		= [];
				foreach($employees as $employee){
					$person 	= $employee->person();

					if($employee->status_id != 1){
						continue;
					}
					
					$totalLate 				= 0;
					$totalWorkingHour 		= 0;
					$totalHoliday 			= 0;
					$indexRow[$employee->code] = $startingRow;
					$sheet->setCellValueByColumnAndRow(1,$startingRow,$no); /* Column nomor */
					$sheet->setCellValueByColumnAndRow(2,$startingRow,$employee->code); /* Column NIK */
					$sheet->setCellValueByColumnAndRow(3,$startingRow,$person->first_name.' '.$person->last_name); /* Column Nama */

					/* Set no information dulu semua kolom */
					foreach($dateColumnIndex as $dci){
						$sheet->setCellValueByColumnAndRow($dci,$startingRow,'No Data')
							->getStyleByColumnAndRow($dci,$startingRow)
							->getFill()
							->setFillType(Fill::FILL_SOLID)
							->getStartColor()
							->setARGB(Color::COLOR_YELLOW);
					}

					/* Get Attendance */
					$attendanceEntities 	= new AttendanceEntities(AttendanceEntity\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
					$loadAttendanceParams 	= array('start_date' => $post['start_date'],'end_date' => $post['end_date'],'ref_id' => $employee->id);
					$listAttendance 		= $attendanceEntities->load(null,Entity\Storage::LIST_ALL,$loadAttendanceParams);

					$leaveEntities 	= new LeaveEntities(LeaveEntity\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
					$listLeave = $leaveEntities->load(null,Entity\Storage::LIST_ALL,$loadAttendanceParams);

					$objects    = new \Neuron\Attendance\Timetable\Objects(\Neuron\Attendance\Timetable\Objects\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
        			$listShift     = $objects->listData($employee->id, \Neuron\Attendance\Timetable\Objects\Storage::LISTBY_REF_ID, $loadAttendanceParams);
        			
					if($listAttendance->code == 0){
						$attendances 			= $listAttendance->data['list'];
						$shifts 			= $listShift->data['list'];

						if(!empty($shifts)){
							foreach($shifts as $key => $employeeShift){
								$defColor = COLOR::COLOR_WHITE;
								if($employeeShift['is_workday'] == 0 ){
									$date = date('Y-m-d',strtotime($employeeShift['start_date']));
									$combineTime = $employeeShift['name'];
									
									$defColor = 'f57382';
									$totalHoliday++;
									$column 	= $dateColumnIndex[$date];
									
									$sheet->setCellValueByColumnAndRow($column,$startingRow,$combineTime)->getStyleByColumnAndRow($column,$startingRow)
									->getFill()
									->setFillType(Fill::FILL_SOLID)
									->getStartColor()
									->setARGB($defColor);
								}
							}
						}


						foreach($attendances as $key => $attendance){
							
							if(intval($attendance['type_id']) != 1){
								unset($attendances[$key]);
								continue;
							}

							/* Masukin per tanggal */
							$date 		= date('Y-m-d',strtotime($attendance['start_dtm']));
							$startTime 	= date('H:i',strtotime($attendance['start_dtm']));
							/* Cek dulu attendancenya ada end_dtm nya atau engga*/
							if($attendance['end_dtm']){
								$endTime 	= date('H:i',strtotime($attendance['end_dtm']));
								$combineTime = $startTime.' - '.$endTime;
							}else{
								/* kalau gaada end_dtmnya di excel dis et (No Information) untuk jam pulang */
								$endTime 	= date('H:i',strtotime($attendance['start_dtm']));
								$combineTime = $startTime.' - (No Information)';
							}

							$defColor = COLOR::COLOR_WHITE;
							/*if($leaves){
								foreach($leaves as $leave){
									$date = date('Y-m-d',strtotime($leave['start_dtm']));
									$combineTime = $leave['type_code'];

									$findTypeId = array_search($combineTime, array_column($listTypes->data, 'code'));
									if($findTypeId){
										$defColor = $listTypes->data[$findTypeId]['color_code'];
									}
								}
							}*/

							/*if(!$combineTime && $shifts){
								foreach($shifts as $employeeShift){
									if($employeeShift['is_workday'] == 0 ){
										$date = date('Y-m-d',strtotime($employeeShift['start_date']));
										$combineTime = $employeeShift['name'];

										$defColor = 'f48174';
									}
								}
							}*/
							
							$column 	= $dateColumnIndex[$date];
							
							$sheet->setCellValueByColumnAndRow($column,$startingRow,$combineTime)->getStyleByColumnAndRow($column,$startingRow)
							->getFill()
							->setFillType(Fill::FILL_SOLID)
							->getStartColor()
							->setARGB($defColor);

							/* Hitung terlambat  */
								/* Ambil shift pada hari attendance */
								$shift 					= new Shift(Shift\Storage::factory(
									$this->_storage->getAdapter(),
									$this->_storage->getConfig()
								));
								$loadShiftParam 		= array();
								$data['start_date']     = date('Y-m-d',strtotime($attendance['start_dtm']));
								$data['start_dtm']      = date('Y-m-d H:i:s',strtotime($attendance['start_dtm']));
								$data['day_no']         = $shift->getDayNumber($data['start_date'])->data;
								$data['ref_id'] 		= $attendance['ref_id'];
								$data['ref_code'] 		= "EMPLOYEE";
								$loadShift              = $shift->load(null,Shift\Storage::LOADBY_BETWEENDATE_REF_ID_DAYNO,$data);
								$shiftStartDtm 			= $loadShift->data['on_duty_time'];
								$lateTime 				= $loadShift->data['late_time'];
								$is_workday 			= $loadShift->data['is_workday'];
								$shiftStartDtm 			= date("H:i:s",strtotime($shiftStartDtm .'+'.$lateTime.' minutes' ));
								
								if($loadShift->code == 0){
									/* kalo workday nya == 0 gausah dihitung */
									if($is_workday){
										
										/* Hitung terlambat dengan actual attend time */
										$actualAttendTime 		= strtotime(date("H:i:s",strtotime($attendance['start_dtm'])));
										$shiftStartTime 		= strtotime(date("H:i:s",strtotime($shiftStartDtm)));
										
										if(!$shiftStartTime){
											$shiftStartTime 	= strtotime(date("H:i:s",strtotime($attendance['start_dtm'])));
										}
										
										$countLate 				= ($actualAttendTime - $shiftStartTime) / 60;
										if($countLate > 0){
											$totalLate 			+= ceil($countLate);
										}
									}
								}

							/* hitung total jam kerja */
							if($attendance['end_dtm']){
								/* hanya dihitung kalo end_dtmnya ada */
								$attdStartTime 			= strtotime(date("H:i:s",strtotime($attendance['start_dtm'])));
								$attdEndTime 			= strtotime(date("H:i:s",strtotime($attendance['end_dtm'])));
								$countTotalWorkingHour 	= ceil((($attdEndTime - $attdStartTime) / 60) / 60);
								if($countTotalWorkingHour > 0){
									$totalWorkingHour 	+= $countTotalWorkingHour;
								}
							}
						}

						$persentaseAttendance 	= (count($attendances) / $countDayBetweenDate) * 100;
						$persentaseAttendance 	= number_format($persentaseAttendance, 2, '.', '');

						$sheet->setCellValueByColumnAndRow(4,$startingRow,$persentaseAttendance.'%');
						$sheet->setCellValueByColumnAndRow($columnTotalAttend,$startingRow,count($attendances));

						$sheet->setCellValueByColumnAndRow($columnLate,$startingRow,$totalLate);
						$sheet->setCellValueByColumnAndRow($columnTotalHour,$startingRow,$totalWorkingHour);
						$sheet->setCellValueByColumnAndRow($columnTotalHoliday,$startingRow,$totalHoliday);

					}

					$leaves 			= $listLeave;
					if(!empty($leaves)){
						foreach($leaves as $key => $leave){
							$begin = new \DateTime($leave['xd1']);
							$end = new \DateTime($leave['xd2']);
							$difference = $begin->diff($end);
							$end->setTime(0,0,1);     // new line
							
							$interval = \DateInterval::createFromDateString('1 day');
							$period = new \DatePeriod($begin, $interval, $end);
							
							foreach ($period as $dt) {
								$date = $dt->format("Y-m-d");
								$defColor = COLOR::COLOR_WHITE;
								// $findTypeId = array_search($combineTime, array_column($listTypes->data, 'code'));
								// if($findTypeId){
									// 	$defColor = $listTypes->data[$findTypeId]['color_code'];
									// 	$defColor = trim(str_replace('#','',$defColor));
									// }
									
								if($listTypes){
									foreach($listTypes->data as $type){
										if($type['order_subtype_id'] == $leave['order_subtype_id']){
											$combineTime = $type['code'];
											$defColor = $type['color_code'];
											$defColor = trim(str_replace('#','',$defColor));
										}
									}
								}

								$column 	= $dateColumnIndex[$date];
								$sheet->setCellValueByColumnAndRow($column,$startingRow,$combineTime)->getStyleByColumnAndRow($column,$startingRow)
								->getFill()
								->setFillType(Fill::FILL_SOLID)
								->getStartColor()
								->setARGB($defColor);
							}
						}

						$sheet->setCellValueByColumnAndRow($columnTotalLeave,$startingRow,count($leaves));
					}
					
					
					$no++;

					$startingRow++;
				}
			}
		
		/* Save to file */
			$writer = IOFactory::createWriter($spreadsheet, "Xlsx"); //Xls is also possible
			$filename = 'Attendance-detail.xlsx';
			

			header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
			header('Content-Disposition: attachment; filename="' . $filename);
			header('Cache-Control: max-age=0');
			ob_end_clean();
			$writer->save('php://output', 'xlsx');
			exit();
		
	}
	public function getTotalLate($start_date,$end_date,$ref_id){
		
		$result 				= new \Neuron\Generic\Result;
		$totalLate 				= 0;
		/* Get Attendance */
		$attendanceEntities 	= new AttendanceEntities(AttendanceEntity\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));

		$loadAttendanceParams 	= array(
			'start_date' => $start_date,
			'end_date' => $end_date,
			'ref_id' => $ref_id,
			'type_id' => AttendanceEntity::TYPE_ATTENDANCE
		);

		$listAttendance 		= $attendanceEntities->load(null,Entity\Storage::LIST_ALL,$loadAttendanceParams);
		$totalDayLate 			= 0;
		if($listAttendance->code == 0){
			$attendances 			= array_key_exists('rows',$listAttendance->data) ? $listAttendance->data['rows'] : [];
			foreach($attendances as $attendance){
				/* Hitung terlambat  */
				/* Ambil shift pada hari attendance */
				$shift 					= new Shift(Shift\Storage::factory(
					$this->_storage->getAdapter(),
					$this->_storage->getConfig()
				));
				$loadShiftParam 		= array();
				$data['start_date']     = date('Y-m-d',strtotime($attendance['start_dtm']));
				$data['start_dtm']      = date('Y-m-d H:i:s',strtotime($attendance['start_dtm']));
				$data['day_no']         = $shift->getDayNumber($data['start_date'])->data;
				$data['ref_id'] 		= $attendance['ref_id'];
				$data['ref_code'] 		= "EMPLOYEE";
				$loadShift              = $shift->load(null,Shift\Storage::LOADBY_BETWEENDATE_REF_ID_DAYNO,$data);
				$shiftStartDtm 			= $loadShift->data['on_duty_time'];
				$lateTime 				= $loadShift->data['late_time'];
				$is_workday 			= $loadShift->data['is_workday'];
				$shiftStartDtm 			= date("H:i:s",strtotime($shiftStartDtm .'+'.$lateTime.' minutes' ));
				
				if($loadShift->code == 0){

					/* kalo workday nya == 0 gausah dihitung */
					if($is_workday){
						/* Hitung terlambat dengan actual attend time */

						$actualAttendTime 		= strtotime(date("H:i:s",strtotime($attendance['start_dtm'])));
						$shiftStartTime 		= strtotime(date("H:i:s",strtotime($shiftStartDtm)));

						if(!$shiftStartTime){
							$shiftStartTime 	= strtotime(date("H:i:s",strtotime($attendance['start_dtm'])));
						}
						
						$countLate 				= ($actualAttendTime - $shiftStartTime) / 60;
						if($countLate > 0){
							$totalLate 			+= ceil($countLate);
							$totalDayLate++;
						}
					}
				}

			}
		}
		$resultData 	= array(
			'totalMinuteLate' 	=> $totalLate,
			'totalDayLate' 		=> $totalDayLate
		);
		$result->data 	= $resultData;
		return $result;
	}
	public function detailLate($params){
		$storage = \Neuron\Attendance\Attendance\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig());
        $catt = new \Neuron\Attendance\Attendance($storage);
		$result = $catt->listAttendance($params);
		$newData = array();
		if($result['data']){
			$listData = $result['data'];
			foreach ($listData as $attendance) {
				$totalLate = 0 ;
				/* Hitung terlambat  */
				/* Ambil shift pada hari attendance */
				$shift 					= new Shift(Shift\Storage::factory(
					$this->_storage->getAdapter(),
					$this->_storage->getConfig()
				));
				$loadShiftParam 		= array();
				$data['start_date']     = date('Y-m-d',strtotime($attendance['start_dtm']));
				$data['start_dtm']      = date('Y-m-d H:i:s',strtotime($attendance['start_dtm']));
				$data['day_no']         = $shift->getDayNumber($data['start_date'])->data;
				$data['ref_id'] 		= $attendance['ref_id'];
				$data['ref_code'] 		= "EMPLOYEE";
				$loadShift              = $shift->load(null,Shift\Storage::LOADBY_BETWEENDATE_REF_ID_DAYNO,$data);
				$shiftStartDtm 			= $loadShift->data['on_duty_time'];
				$shiftEndDtm 			= $loadShift->data['off_duty_time'];
				$lateTime 				= $loadShift->data['late_time'];
				$is_workday 			= $loadShift->data['is_workday'];
				$shiftStartDtm 			= date("H:i:s",strtotime($shiftStartDtm .'+'.$lateTime.' minutes' ));
				
				if($loadShift->code == 0){
					/* kalo workday nya == 0 gausah dihitung */
					if($is_workday){
						/* Hitung terlambat dengan actual attend time */
						$actualAttendTime 		= strtotime(date("H:i:s",strtotime($attendance['start_dtm'])));
						$shiftStartTime 		= strtotime(date("H:i:s",strtotime($shiftStartDtm)));

						if(!$shiftStartTime){
							$shiftStartTime 	= strtotime(date("H:i:s",strtotime($attendance['start_dtm'])));
						}
						
						$countLate 				= ($actualAttendTime - $shiftStartTime) / 60;

						if($countLate > 0){
							$totalLate 			+= ceil($countLate);
						}
					}
				}
				$attendance['late'] = $totalLate;
				$attendance['shift'] =  $loadShift->data['on_duty_time'] .' - '.  $loadShift->data['off_duty_time'];
				$attendance['real_start_dtm'] = $attendance['start_dtm'];
				$attendance['time'] = date("H:i:s",strtotime($attendance['start_dtm']));
				$attendance['start_dtm'] = date("Y-m-d",strtotime($attendance['start_dtm']));
				array_push($newData, $attendance);
			}
		}

		$result['data'] = $newData;
        return $result;
	}
	public function getTotalLeave($start_date,$end_date,$personIds){
		$userIds 		= [];
		$returnData 	= [];
		
		foreach($personIds as $personId){
			$map 			= new \Neuron\Person\Map(
				\Neuron\Person\Map\Storage::factory(
					$this->_storage->getAdapter(),
					$this->_storage->getConfig()
				)
			);
			$map->load($personId,\Neuron\Person\Map\Storage::LOADBY_PERSON_ID);
			$userIds[$personId] = $map->user_id;
		}

		$orderEntities 	= new \Neuron\Order\Entities(
			\Neuron\Order\Entity\Storage::factory(
				$this->_storage->getAdapter(),
				$this->_storage->getConfig()
			)
		);
		if ($manager = \Neuron\Application\Framework\Event\Manager::get()) {
			$manager->events()->register('order_query',array($this,'onGetTotalLeave'),0,3);
		}
		$filter = [
			'optional' 	=> [
				'create_user_id' 	=> $userIds,
				'create_dtm' 		=> $start_date,
				'end_dtm' 			=> $end_date
			]
		];

		$orders 	= $orderEntities->load(2200,\Neuron\Order\Entity\Storage::LISTBY_TYPE,$filter);

		if($orders->code == 0){
			$orders 	= $orders->data;
			foreach($orders as $order){
				// Cari person_id nya
				$orderCreateUserId 	= $order['create_user_id'];
				$personId 	= null;
				foreach($userIds as $pId => $userId){
					if($orderCreateUserId == $userId){
						$personId = $pId;
						break;
					}
				}

				if(isset($returnData[$personId])){
					$returnData[$personId] += 1;
				}else{
					$returnData[$personId] = 1;
				}
			}
		}

		return $returnData;
		
	}

	public function onGetTotalLeave($query,$params,$caller){
		$params 	= $params['optional'];
		$query->where->in("create_user_id",$params['create_user_id']);
		$query->where->greaterThanOrEqualTo(new \Zend\Db\Sql\Expression("DATE(o.xd1)"),$params['create_dtm']);
		$query->where->lessThanOrEqualTo(new \Zend\Db\Sql\Expression("DATE(o.xd1)"),$params['end_dtm']);
		$query->where(['o.order_status_id' => 2]);
	}

	public function getTotalLogInvalid($type,$start_date,$end_date,$personIds){
		$userIds 		= [];
		$returnData 	= [];

		foreach($personIds as $personId){
			$map 			= new \Neuron\Person\Map(
				\Neuron\Person\Map\Storage::factory(
					$this->_storage->getAdapter(),
					$this->_storage->getConfig()
				)
			);
			$map->load($personId,\Neuron\Person\Map\Storage::LOADBY_PERSON_ID);
			$userIds[$personId] = $map->user_id;
		}
		
		$fplogs 	= new \Neuron\Fingerprint\FpLogs(\Neuron\Fingerprint\FpLog\Storage::factory($this->getDb(),$this->getConfig()));

		$lists = $fplogs->countTotal($type,$start_date,$end_date,$userIds);
		if($lists){
			foreach ($lists as $list){
				$personId 	= null;
				foreach($userIds as $pId => $userId){
					if($list['user_id'] == $userId){
						$personId = $pId;
						
						if(!isset($returnData[$personId])){
							$returnData[$personId] = $list['countTotal'];
						}
					}
				}
			}
		}
		
		return $returnData;
		
	}
}