<?php

namespace Neuron\Attendance\Reporting\Storage;
use Zend\Db\Sql\Predicate\Expression;

class Mysql extends \Neuron\Db\Storage implements Skeleton {

	public function getTotalRows() {
		try {

			$select = $this->select()
							->columns(array('total' => $this->expression('FOUND_ROWS()')));
														 
			return $this->fetchValue($select);  

		} catch (\Exception $ex) {
			return false;
		}              
	}  

    public function reportAttendance($params = null) {
		try {
			$select = $this->select()
						->from(array('at'=>$this->_tables['attendance']));

			if($params['rowCount'] != -1){
				$offset= ($params['start']) * $params['length'];
				$select->limit(intval($params['length']))
						->offset(intval($offset));
			}
			
			if($params['session']['access_role'] == 1){
				$select->join(array('ta'=>$this->expression('(SELECT COUNT(*) AS total_attend, ref_id FROM attendance where DATE_FORMAT(start_dtm, \'%Y-%m-%d\') BETWEEN DATE_FORMAT("'.$params['start_date'].'", \'%Y-%m-%d\') AND DATE_FORMAT("'.$params['end_date'].'", \'%Y-%m-%d\')  GROUP BY(ref_id))')), 'ta.ref_id = at.ref_id', array('total_attend'), 'left');
				$select->group('at.ref_id');
			}else{
				$select->where(array('at.ref_id'=> $params['session']['user_id'], 'DATE_FORMAT(at.start_dtm, \'%Y-%m-%d\') BETWEEN DATE_FORMAT("'.$params['start_date'].'", \'%Y-%m-%d\') AND DATE_FORMAT("'.$params['end_date'].'", \'%Y-%m-%d\')'));
			}
			$fetchAll = $this->fetchAll($select);
			return $fetchAll; 
		} catch (\Exception $e) {
			return false;
		}
	}

	public function countTotal($type,$start_date,$end_date,$personIds){
		$select         = $this->select();
        $select->from($this->_tables['attendance']);
		$select->where(array('type_id' => $type));
		
        $select->where->greaterThanOrEqualTo(new Expression("DATE(start_dtm)"),$start_date);
		$select->where->lessThanOrEqualTo(new Expression('DATE(start_dtm)'),$end_date);

		$select->where->in('ref_id',$personIds);

		$select->group('ref_id');

        $select->columns(
			array(
				'countTotal' => new Expression("COUNT(*)"),
				'ref_id'
			)
		);
		
        try{
			$data   = $this->fetchAll($select);
            if($data){
                return $data;
            }else{
                return false;
            }
        }catch(\Exception $ex){
            error_log("ERROR ::". $ex->getMessage().' File '.$ex->getFile(),' Line : '.$ex->getLine());
            return false;
        }
	}
	
	public function totalOvertime($start_date,$end_date,$personIds){
		$select 		= $this->select()->from($this->_tables['attendance']);

		// $select->columns( array(
		// 	'totalMinutes' 	=> new Expression('(TIMESTAMPDIFF(minute,start_dtm,end_dtm))')
		// ));

		$select->where->greaterThanOrEqualTo(new Expression("DATE(start_dtm)"),$start_date);
        $nest   = $select->where->nest();
        $nest->lessThanOrEqualTo(new Expression('DATE(end_dtm)'),$end_date)->or;
        $nest->isNull('end_dtm');
		$nest->unnest();
		
		$select->where->in('ref_id',$personIds);
		$select->where(array('type_id' => 2));
		
		try{
			$data   = $this->fetchAll($select);
            if($data){
                return $data;
            }else{
                return false;
            }
        }catch(\Exception $ex){
            error_log("ERROR ::". $ex->getMessage().' File '.$ex->getFile(),' Line : '.$ex->getLine());
            return false;
        }
	}

	public function getTotalLeave($start_date,$end_date,$personIds){
		$select 	= $this->select();
		$select->from($this->_tables['leave']);
		$select->where->in('ref_id',$personIds);
		$select->where->greaterThanOrEqualTo(new Expression("date(start_dtm)"),$start_date);
		$select->where->lessThanOrEqualTo(new Expression("date(start_dtm)"),$end_date);
		$select->where(['status_id' => 2]);
		
		try {
			$result = $this->fetchAll($select);
			return $result;
		} catch (\Throwable $th) {
			return false;
		}
	}
}