<?php
namespace Neuron\Attendance;

use Neuron\Generic\Entity\Map;
use Neuron\Attendance\Request as requestModel;
use Neuron\Generic\Result;
use Neuron\Attendance\Entity;
use Neuron\Attendance\Attendance;
use Neuron\Attendance\Request\Approval;
use Neuron\Attendance\Request\Approvals;
use Neuron\Attendance\Request\Type;
use Neuron\Attendance\Request\Types;
use Neuron\Attendance\Request\Setting;
use Neuron\Attendance\Request\Setting\Detail;
use Neuron\Attendance\Request\Setting\Details;
use Neuron\Attendance\Timetable;
use Neuron\Attendance\Timetable\Changeshift;
use Neuron\Notification;
use Neuron\Employee;
use Neuron\Notification\Inbox\Cache;
use Neuron\Order\Entity as OrderEntity;
use Neuron\Addon\Order\SimpleFlow;
use Zend\Stdlib\ArrayUtils;

class Request extends \Neuron\Generic\Entity\Mapper {
	CONST REQUEST_TYPE_ATTENDANCE 	= 2101;
	CONST REQUEST_TYPE_OVERTIME	 	= 2102;
	CONST REQUEST_TYPE_CHANGE_SHIFT = 2103;

	CONST DEFAULT_REF_CODE 			= 'EMPLOYEE';
	CONST TEST_ENVIRONMENT 			= true;

	const REQ_TYPE_ATTENDANCE	= "Attendance";
	const REQ_TYPE_OVERTIME	= "Overtime";
	const REQ_TYPE_CHSHIFT	= "Change Shift";

	protected $_storage;

	public function __construct(\Neuron\Attendance\Request\Storage\Skeleton $storage, $data= array()) {
		
		$this->_storage = $storage;

		parent::__construct(
			$data, 
			array(
				new Map('request_id'),
				new Map('ref_code'),
				new Map('ref_id'),
				new Map('start_dtm'),
				new Map('end_dtm'),
				new Map('request_type'),
				new Map('request_dtm'),
				new Map('status_id'),
				new Map('request_message'),
				new Map('request_type_name',null,true),
				new Map('status_name',null,true),
				new Map('ref_name',null,true)
			), 
			parent::CASE_INSENSITIVE
		);
	}

	/* Diguakan untuk save request */
	public function saveEx($data = array(),$publicURL = null){
		
		$result 	= new Result;

		/* Validating Approval Required */
		if(!isset($data['approval']) || count($data['approval']) <= 0){
			$result->code 	= 400;
			$result->info 	= "Approval Required";
			return $result;
		}

		$fileevidence = isset($data['file_evidence'])? $data['file_evidence']:null;
		unset($data['file_evidence']);

		/* Approval Exist ? Check setting */
			$storageType 	= Type\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig());
			$type 			= new Type($storageType);
			$typeData 		= $type->load($data['request_type'])->data;
			$settingID 		= $typeData['setting_id'];

			/* Load setting untuk ambil data request_setting 
			* untuk membandingkan jumlah data approval yang di post
			* dan data yang dibutuhkan 
			*/
			$storageSetting = Setting\Storage::factory($this->_storage->getAdapter(), $this->_storage->getConfig());
			$setting 		= new Setting($storageSetting);
			$dataSetting 	= $setting->load($settingID);
			if(!$dataSetting->data){
				$result->code 	= 500;
				$result->info 	= "Failed loading setting";
				return $result;
			}

			/* Bandingkan dengan jumlah approval yang dibutuhkan */
			$numToApprove 	= intval($dataSetting->data->setting['num_to_approve']);
			if(count($data['approval']) < $numToApprove){
				$result->code 	= 401;
				$result->info 	= "Minimum Approval not passed";
				return $result;
			}
		// Validating Approval

		/* validating user_id */
			if(!isset($data['user_id']) || strlen($data['user_id']) <= 0){
				$result->code 	= 402;
				$result->info 	= "Invalid user id!";
				return $result;
			}
			/* get ref_id */
			if(class_exists('\Neuron\Employee\Entity')){
				$storageEmployee 	= Employee\Entity\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig());
				$employee 			= new Employee\Entity($storageEmployee);
				$mapEmployee 		= $employee->loadMap($data['user_id'],'user_id');
				if($mapEmployee['data']){
					$ref_id 		= $mapEmployee['data']['person_id'];
				}else{
					$result->code 	= 501;
					$result->info 	= "failed loading person map";
					return $result;
				}
				unset($data['user_id']);
			}else{
				/* Block else ini digunakan kalau ga pake modul employee NAF */
				$ref_id 		= null;
				unset($data['user_id']);
			}
		/* validating user_id */


		/* Validating approving to self */
            /* Kalo ada spesifik ke diri sendiri skip aja gausah di validasi */
			$bypass         = FALSE;

			$detail         = new Details(Detail\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
			$listDetail     = $detail->load($settingID,Detail\Storage::LISTBY_SETTING_ID,array('setting_id' => $settingID))->data['rows'];

            foreach ($listDetail as $ld) {
                if($ld['approval_type_id'] == 2 && $ld['type_value'] === $ref_id){
                    $bypass = TRUE;
                    break;
                }
            }

            if(!$bypass){
                foreach ($data['approval'] as $appr) {
                    if($appr == $ref_id){
                        $result         = new Result();
                        $result->code   = 414;
						$result->info   = "Cannot requesting approval to self.";
						return $result;
                    }
                }
            }
		/* Validating approving to self */

		// Validating Duplicate Attendance
			if ($data['request_type'] == self::REQUEST_TYPE_ATTENDANCE) {
				$attendance     = new Attendance(Attendance\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
				$checkParams    = array(
					'chckdate'  => date('Y-m-d',strtotime($data['start_dtm'])),
					'ref_id'    => $ref_id,
					'start_dtm' => null
				);
				$checkByDate    = $attendance->listAttendance($checkParams);
				if(count($checkByDate['data']) > 0 ){
					$data = $checkByDate['data'][0];
					return new Result(0,411,"You already attended at ".$data['start_dtm'].".");
				}
			}
		// Validating Duplicate Attendance

		// ambil timetable
		$storageTs  = \Neuron\Attendance\Shift\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig());
		$cts        = new \Neuron\Attendance\Shift($storageTs);
		$cektimetable = $cts->loadObjectTimetable($ref_id, date('Y-m-d H:i:s',strtotime($data['start_dtm'])), $data['request_type']);
		
		if(!$cektimetable['success']){
			$result->code 	= 3;
			$result->info 	= "Invalid date / Timetable doesnt exists";
			return $result;
		}

		// Validating duplicate request
			$request_date       = explode(' ', $data['start_dtm'])[0];
			$request_type       = $data['request_type'];

			$check  = $this->checkDuplicateRec($request_type,$request_date,$ref_id)->data;

			if(count($check) > 0){
				$result->code   = 413;
				$result->info   = 'Failed saving request. You have an unapproved request, or request on the same date already approved!';
				return $result;
			}
		// Validating duplicate request

		$approval 		= $data['approval'];
		unset($data['approval']);
		
		if(!isset($data['ref_code']) || strlen($data['ref_code']) <= 0){
			$data['ref_code'] = self::DEFAULT_REF_CODE;
		}

		$data['start_dtm'] = !empty($data['start_dtm'])? date('Y-m-d H:i:s', strtotime($data['start_dtm'])) : date("Y-m-d H:i:s");
		$data['end_dtm'] = !empty($data['end_dtm'])? date('Y-m-d H:i:s', strtotime($data['end_dtm'])) : null;
		$data['ref_id'] 		= $ref_id;
		$data['request_dtm']	= date("Y-m-d H:i:s");
		/* Set default ke requested ajah */
		$data['status_id'] 	 	= Approval::APPROVAL_STATUS_REQUESTED;
		@$this->push($data);

		/* Save to table request */
		if(!$this->save(FALSE)){
			$result->code 		= 502;
			$result->info 		= "FAILED SAVING REQUEST";
			return $result;
		}

		/* Save Approval */
		$storageApproval 	= Approval\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig());
		$savedApproval 		= 0;
		foreach ($approval as $rowApproval ) {
			$approval 			= new Approval($storageApproval);
			$dataApproval 		= array(
				'request_id' 	=> $this->request_id,
				'ref_code' 		=> $data['ref_code'],
				'ref_id' 		=> $rowApproval,
				'status_id' 	=> Approval::APPROVAL_STATUS_REQUESTED,
				'approval_dtm' 	=> date("Y-m-d H:i:s"),
				'request_type' 	=> $data['request_type']
			);
			$saveApproval 		= $approval->saveEx($dataApproval,$publicURL);
			if($saveApproval->code == 0){
				$savedApproval++;
			}
		}

		/* Save ke table chshift */
		if ($data['request_type'] == 3) {
			$changeShift                        = new Changeshift(Changeshift\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
			$targetId 							= $data['target_id'];

			$chshift['tt_employee_shift_id']    = null;
			$chshift['ref_code']                = "EMPLOYEE";
			$chshift['ref_id']                  = $ref_id;
			$chshift['shift_change_date']       = date('Y-m-d', strtotime($data['start_dtm']));
			$chshift['default_shift_id']        = $cektimetable['data']['timetable_shift_id'];
			$chshift['change_shift_id']         = $data['target'];
			$chshift['recommendation_id']       = $this->request_id;
			$chshift['change_by']               = $ref_id;

			$saveChShift                        = $changeShift->saveEx($chshift);
		}

		//save evidence jika ada
		if($fileevidence){
			$dataAttendace = $this->pull();
			$dataAttendace['file'] = $fileevidence;
			$dataAttendace['id'] = $this->request_id;
			$dataAttendace['action_type'] = 2;
			$dataAttendace['action_type_name'] = 'request';
			$storageEvidence = \Neuron\Attendance\Evidence\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig());
			$modelEvidence  = new \Neuron\Attendance\Evidence($storageEvidence);
			$modelEvidence->create($dataAttendace);
		}
		$result->info 	= "Success creating request";
		$result->data 	= array(
			'savedApproval' 	=> $savedApproval,
			'request_id' 		=> $this->request_id
		);

		return $result;
	}

	public function listRequest($params=false) {
		try {
			$result = $this->_storage->listRequest($params);

			if(isset($params['filter'])) {
				$arrref = [];
				if ($result) {
					foreach ($result as $data) {
						if($data['request_type'] == self::REQUEST_TYPE_ATTENDANCE){
							$data['request_type_name'] = 'Attendance';
						}else if($data['request_type'] == self::REQUEST_TYPE_OVERTIME){
							$data['request_type_name'] = 'Overtime';
						}else{
							$data['request_type_name'] = 'Change Shift';
						}
						$data['id'] = intval($data['request_id']);
						array_push($arrref, $data);
					}
				}

				$total = $this->_storage->getTotalRows();
				
				$retval = array(
					'current' => $page,
					'rowCount' => $rowCount,
					'rows' => $arrref,
					'total' => $total
				);

				return $retval;
			}
			else {

				$draw = (isset($params['draw']) ? $params['draw']:null);
				
				$retval = array(
					'draw' => $draw,
					'recordsTotal' => $result['total'],
					'recordsFiltered' => $result['filtered'],
					'data' => $result['data'],
				);

				return $retval;
			}
		} catch (\Exception $e) {
			return false;
		}
	}

	public function load($id = null, $mode = requestModel\Storage::LOADBY_ID,$params = array(),$rest = FALSE){
		$result 	= new Result();

		$data 		= $this->_storage->load($id,$mode,$params);
		if($data){
			$result->data 	= $data;
			$this->push($data);
		}else{
			$result->data 	= array();
		}
		
		if($rest){
			return $result;
		}
		return $data;
	}

	public function save($update = false){
		if ($id = $this->_storage->save($this, $update)) {
			$this->request_id = $id;
			return true;
		}
		return false;
	}

	public function saveApproval($post,$publicUrl = null){
		$result 		= new Result;
		$approval 		= new Approval(Approval\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
	
		/*
		* block if digunakan untuk mengupdate status approval
		* Block else digunakan untuk menyimpan approval baru.
		*/
		if(!empty($post['approval_id'])){
			/* Saving Approval */
			$approval->load($post['approval_id']);

			$approval->status_id 	= $post['status_id'];
			$approval->reason 		= $post['reason'];
			$approval->approval_dtm = date('Y-m-d H:i:s');

			if($approval->save(TRUE)){
				$result->info = "Success approve request";
				if($post['status_id'] ==3 ){
					$result->info = "Success reject request";
				}
			}else{
				$result->info = "Failed approve request";
				if($post['status_id'] ==3 ){
					$result->info = "Failed reject request";
				}
			}

			/* Load current request */
			$request 		= $this->load($approval->request_id);

			/*
			* $createRow
			* digunakan untuk ngecheck
			* request ini sudah pernah dibuat row untuk attd/overtime/changeshiftnya atau belum.
			* dilihat dari status sebelum dilakukan aksi, apabila status masih requested
			* berarti belum pernah dibuat row.
			* apabila status sudah approved berarti pernah dibuat row.
			*/
			$createRow 		= ($request['status_id'] == 1) ? TRUE : FALSE;
			
			/* Kalo statusnya masih request / sudah approve, masih bisa di declined */
			if($request['status_id'] == Approval::APPROVAL_STATUS_REQUESTED || $request['status_id'] == Approval::APPROVAL_STATUS_APPROVED){
				$requestType 		= new Type(Type\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
				$setting 			= new Setting(Setting\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));

				$request_type 		= $request['request_type'];
				$loadRequestType 	= $requestType->load($request_type)->data;
				$setting_id 		= $loadRequestType['setting_id'];

				$loadSetting 		= $setting->load($setting_id)->data;
				$approvalStatus 	= $approval->getApprovalStatus($approval->request_id)->data;
				/* Check if Approved */
				$needUpdate 		= FALSE;

				if($approvalStatus['total_approve'] >= $loadSetting->setting['min_approved']){
					/* Approved */
					$needUpdate 	= TRUE;
					$this->status_id 	= $approval::APPROVAL_STATUS_APPROVED;
				}

				/*Check if rejected*/
				if($approvalStatus['total_reject'] >= $loadSetting->setting['min_rejected']){
					/*Rejected*/
					$needUpdate 		= TRUE;
					$this->status_id 	= $approval::APPROVAL_STATUS_REJECTED;
				}
				
				if($needUpdate){
					/*Update request status*/
					//if(!self::TEST_ENVIRONMENT){
						$save = $this->save(TRUE);
					//}

					$employee 	= new Employee\Entity(Employee\Entity\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
					$loadMap 	= $employee->loadMap($this->ref_id,'person_id');
					$userId 	= null;
					if($loadMap['code'] == 200){
						$map 		= $loadMap['data'];
						$userId 	= $map['user_id'];
					}

					/* Check statusnya rejected / approved untuk notifikasi template. */
					if($this->status_id == $approval::APPROVAL_STATUS_APPROVED){
						$template_code 	= 'REQUEST_APPROVE';
					}else if($this->status_id == $approval::APPROVAL_STATUS_REJECTED){
						$template_code 	= 'REQUEST_REJECT';
					}

					/* Kirim notifikasi kalo request di approve / di reject. */
					
					if(!self::TEST_ENVIRONMENT){

						$pushNotif = $this->pushNotif(
							intval($userId),
							$template_code,
							array(
								'request_type' => $loadRequestType['type_name'],
							),
							array(
								'request_id' 	=> $approval->request_id,
							), 
							$publicUrl
						);
					}

					/* Create Attendance */
					if($createRow && $this->status_id == $approval::APPROVAL_STATUS_APPROVED){
						if($request['request_type'] == self::REQUEST_TYPE_ATTENDANCE){
							$entity = new Entity(Entity\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
							$data  	= array();

							$data['type_id'] 				= Entity::TYPE_ATTENDANCE;
							$data['start_dtm'] 				= $request['start_dtm'];
							$data['start_description'] 		= $request['request_message'];
							$data['end_dtm'] 				= $request['end_dtm'];
							$data['end_description'] 		= null;
							$data['ref_code'] 				= $request['ref_code'];
							$data['ref_id'] 				= $request['ref_id'];
							$data['source_type_code'] 		= 'REQUEST';
							$data['source_reference'] 		= $request['ref_id'];
							$data['source_dtm'] 			= date("Y-m-d H:i:s");
							$data['overtime_start_dtm'] 	= null;
							$data['overtime_end_dtm'] 		= null;
							$data['longitude'] 				= null;
							$data['latitude']				= null;
							$data['start_time'] 			= explode(' ',$request['start_dtm'])[1];

							$result 						= $entity->saveEx($data);
						}else if($request['request_type'] == self::REQUEST_TYPE_OVERTIME){
							$request['type_id'] = Entity::TYPE_OVERTIME;
							$request['start_time'] = explode(" ",$request['start_dtm'])[1];
							$request['source_reference'] = $request['ref_id'];
							$entity = new Entity(Entity\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
							$result = $entity->saveEx($request);
						}else if($request['request_type'] == self::REQUEST_TYPE_CHANGE_SHIFT){
							$storage  			= \Neuron\Attendance\Timetable\Objects\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig());
							$objectTimetable 	= new \Neuron\Attendance\Timetable\ObjectTimetable($storage);

							$storageChangeshift = Timetable\Changeshift\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig());
							$changeShift 		= new Timetable\Changeshift($storageChangeshift);

							// Load Changeshift by request ID
							$changeShift->load($this->request_id,Timetable\Changeshift\Storage::LOADBY_REC_ID);
							$result 		= $objectTimetable->changeShift($changeShift);
							
						}
					}
				}
			}
		}else{
			$approval->approval_id 	= null;
			$approval->request_id 	= $post['request_id'];
			$approval->ref_code 	= $post['ref_code'];
			$approval->ref_id 		= $post['ref_id'];
			$approval->status_id 	= $post['status_id'];
			$approval->approval_dtm = $post['approval_dtm'];
			$approval->reason 		= $post['reason'];

			if($approval->save(FALSE)){
				$result->data 		= $approval->approval_id;
				$result->info 		= "Success saving approval";
			}else{
				$result->info 		= "Failed saving approval";
			}
		}
		
		return $result;

	}
	public function delete($id) {
		return $this->_storage->remove($id);
	}
	public function loadApprovalBy($by, $id){
		return $this->_storage->loadApprovalBy($by, $id);
	}
	public function deleteApprovalBy($by, $id){
		return $this->_storage->removeApprovalBy($by, $id);
	}
	public function loadApprovalType(){
		return $this->_storage->loadApprovalType();
	}
	public function loadSetting($params=false){
		return $this->_storage->loadSetting($params);
	}
	public function saveSetting($data){
		if(!empty($data['id'])){    
			$update = true;
		}else{
			$update = false;
		}
		$data['end_date'] = null;
		$saveSetting = $this->_storage->saveSetting($data, $update);
		if($saveSetting){
			if(!empty($data['detail_id'])){
				$updet = true;
			}else{
				$updet = false;
			}
			$data['seq'] = 1;
			$data['setting_id'] = $saveSetting;
			$savedetail = $this->_storage->saveSettingDetail($data, $updet);
			if($savedetail){
				return true;
			}else{
				return true;
			}
		}else{
			return false;
		}
	}
	public function saveApprove($post){
		return $this->_storage->saveApprove($post);
	}
	public function updateRequest($post){
		return $this->_storage->updateRequest($post);   
	}
	public function checkStatusApproval($id){
		$approved = 0;
		$rejected = 0;
		$onrequest = 0;

		$dataApproval = $this->loadApprovalBy('request_id', $id);
		$arrApproval = array();
		if($dataApproval){
			foreach ($dataApproval as $val) {
				array_push($arrApproval, $val);
				if($val['status_id']== self::REQUEST_TYPE_ATTENDANCE){
					$onrequest +=1;
				}else if($val['status_id']==self::REQUEST_TYPE_OVERTIME){
					$approved +=1;
				}else if($val['status_id']==self::REQUEST_TYPE_CHANGE_SHIFT){
					$rejected +=1;
				}

			}
		}

		return array('approved'=>$approved, 'rejected'=>$rejected, 'onrequest', $onrequest);
	}
	public function loadRequest($by,$id){
		$data = [];
		$dataAll = $this->_storage->loadRequest($by,$id);
		if($dataAll){
			foreach ($dataAll as $val) {
				array_push($data, $val);
			}
		}
		return $data;
	}
	public function listApproval($page = 1, $rowCount = 10, $filter = null ,$ref_id = null,$post = array()) {

		$result = $this->_storage->listApproval($page, $rowCount, $filter, $ref_id,$post);
		$arrref = array();
		if ($result) {
			foreach ($result as $data) {
				if($data['request_type'] == self::REQUEST_TYPE_ATTENDANCE){
					$data['request_type_name'] = 'Attendance';
				}else if($data['request_type'] == self::REQUEST_TYPE_OVERTIME){
					$data['request_type_name'] = 'Overtime';
				}else{
					$data['request_type_name'] = 'Change Shift';
				}
				$data['id'] = intval($data['approval_id']);
				array_push($arrref, $data);
			}
		}

		$total = $this->_storage->getTotalRows();
		
		$retval = array(
			'current' => $page,
			'rowCount' => $rowCount,
			'rows' => $arrref,
			'total' => $total
		);

		return $retval;
	}
	public function loadApproval($appr_id = null){
		return $this->_storage->loadApproval($appr_id);
	}
	public function loadList($mode = requestModel\Storage::LISTBY_ID,$params = array(),$time_mode = requestModel\Storage::TIME_WEEKLY){
		$result = new \Neuron\Generic\Result;
		$data = $this->_storage->loadList($mode,$params,$time_mode);
		if(!$data){
			$resultData = array();
		}else{
			$resultData = array();
			foreach ($data as $data) {
				$resultData[] = $data;
			}
		}
		$result->data = $resultData;
		return $result;
	}
	public function liststatus($term = null){
		return $this->_storage->liststatus($term);
	}
	public function checkDuplicateRec($type_id,$date,$ref_id){
		$result 		= new \Neuron\Generic\Result;
		$load 			= $this->_storage->loadRecByTypeDate($type_id,$date,$ref_id);
		$data = array();
		if($load){
			foreach ($load as $row) {
				$data[] 	= $row;
			}
		}
		$result->data = $data;
		return $result;
	}

	public function pushNotif($userId, $template_code, $temp_var = array(), $additional = array(), $publicUrl ){
        $cache = new Cache(Cache\Storage::factory($this->_storage->getAdapter(), $this->_storage->getConfig()));
        if (!$cache->load($userId, Cache\Storage::LOADBY_SUBJECT_TYPE, array('type_id' => 3))) {
            $cache->subject = $userId;
            $cache->typeId = 3;
            $cache->token = NULL;
            $prepare = $cache->prepare();
        }
        /* Load model Notification\Inbox. Fungsi pushNotification dari sini */
        $inboxStorage   = Notification\Inbox\Storage::factory(
            $this->_storage->getAdapter(),
            $this->_storage->getConfig(),
            Notification\Profile::TYPE_FCM,
            $userId
        );
        $inbox      = new Notification\Inbox($inboxStorage);
        $inbox->setPublicUrl($publicUrl);

        $result = $inbox->pushNotification(
            $userId,
            $template_code,
            $temp_var,
            $additional
        );
        return $result;
	}
	
	public function updateDataAttendance($params = array(), $post = array(), $publicUrl = null)
	{
		$approval 		= new Approval(Approval\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
		//bagian code dari saveApproval yang di pisah

		$employee 	= new Employee\Entity(Employee\Entity\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
		$loadMap 	= $employee->loadMap($params['create_user_id'],'user_id');
		$ref_id 	= null;
		if($loadMap['code'] == 200){
			$map 		= $loadMap['data'];
			$ref_id		= $map['person_id'];
		}

		/* Check statusnya rejected / approved untuk notifikasi template. */
		if($post['status_id'] == $approval::APPROVAL_STATUS_APPROVED){
			$template_code 	= 'REQUEST_APPROVE';
		}else if($post['status_id'] == $approval::APPROVAL_STATUS_REJECTED){
			$template_code 	= 'REQUEST_REJECT';
		}

		/* Kirim notifikasi kalo request di approve / di reject. */
		
		if(!self::TEST_ENVIRONMENT){

			$pushNotif = $this->pushNotif(
				intval($params['create_user_id']),
				$template_code,
				array(
					'request_type' => $params['order_subtype_name'],
				),
				array(
					'request_id' 	=> $params['order_id'],
				), 
				$publicUrl
			);
		}

		/* Create Attendance */
		if($post['status_id'] == $approval::APPROVAL_STATUS_APPROVED){
			if($params['order_subtype_id'] == self::REQUEST_TYPE_ATTENDANCE){
				$entity = new Entity(Entity\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
				$data  	= array();

				$data['type_id'] 				= Entity::TYPE_ATTENDANCE;
				$data['start_dtm'] 				= $params['xd1'];
				$data['start_description'] 		= $params['order_desc'];
				$data['end_dtm'] 				= $params['xd2'];
				$data['end_description'] 		= null;
				$data['ref_code'] 				= self::DEFAULT_REF_CODE;
				$data['ref_id'] 				= $ref_id;
				$data['source_type_code'] 		= 'REQUEST';
				$data['source_reference'] 		= $ref_id;
				$data['source_dtm'] 			= date("Y-m-d H:i:s");
				$data['overtime_start_dtm'] 	= null;
				$data['overtime_end_dtm'] 		= null;
				$data['longitude'] 				= null;
				$data['latitude']				= null;
				$data['start_time'] 			= explode(' ',$params['xd1'])[1];

				$result 						= $entity->saveEx($data);
			}else if($params['order_subtype_id'] == self::REQUEST_TYPE_OVERTIME){

				$data['type_id'] 				= Entity::TYPE_OVERTIME;
				$data['start_dtm'] 				= $params['xd1'];
				$data['start_description'] 		= $params['order_desc'];
				$data['end_dtm'] 				= $params['xd2'];
				$data['end_description'] 		= null;
				$data['ref_code'] 				= self::DEFAULT_REF_CODE;
				$data['ref_id'] 				= $ref_id;
				$data['source_type_code'] 		= 'REQUEST';
				$data['source_reference'] 		= $ref_id;
				$data['source_dtm'] 			= date("Y-m-d H:i:s");
				$data['start_time'] 			= explode(' ',$params['xd1'])[1];

				$entity = new Entity(Entity\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
				$result = $entity->saveEx($data);
			}else if($params['order_subtype_id'] == self::REQUEST_TYPE_CHANGE_SHIFT){
				$storage  			= \Neuron\Attendance\Timetable\Objects\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig());
				$objectTimetable 	= new \Neuron\Attendance\Timetable\ObjectTimetable($storage);

				$storageChangeshift = Timetable\Changeshift\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig());
				$changeShift 		= new Timetable\Changeshift($storageChangeshift);

				// Load Changeshift by request ID
				$changeShift->load($params['order_id'],Timetable\Changeshift\Storage::LOADBY_REC_ID);
				$result 		= $objectTimetable->changeShift($changeShift);
				
			}
		}

		return $result;
	}

	public function createRequestTypeData($orderData){
		
		$orderSubtype = [
			'REQUEST_TYPE_ATTENDANCE' 		=> 2101,
			'REQUEST_TYPE_OVERTIME' 		=> 2102,
			'REQUEST_TYPE_OVERTIME_WORKDAY' => 2110,
			'REQUEST_TYPE_CHANGE_SHIFT' 	=> 2103
		];

		// Cari map
		$employeeMap 	= new \Neuron\Employee\Map(\Neuron\Employee\Map\Storage::factory(
			$this->_storage->getAdapter(),
			$this->_storage->getConfig()
		));
		$employeeMap->load($orderData['create_user_id'],\Neuron\Employee\Map\Storage::LOADBY_USER_ID);
		$ref_id 	= $employeeMap->employee()->id;
		$orderData['ref_id'] = $ref_id;


		if($orderData['order_subtype_id'] == $orderSubtype['REQUEST_TYPE_ATTENDANCE']){
			$entity = new Entity(Entity\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
			$data  	= array();

			$data['type_id'] 				= Entity::TYPE_ATTENDANCE;
			$data['start_dtm'] 				= $orderData['xd1'];
			$data['start_description'] 		= $orderData['order_desc'];
			$data['end_dtm'] 				= $orderData['xd2']; 
			$data['end_description'] 		= null;
			$data['ref_code'] 				= self::DEFAULT_REF_CODE;
			$data['ref_id'] 				= $orderData['ref_id'];
			$data['source_type_code'] 		= 'REQUEST';
			$data['source_reference'] 		= $orderData['order_id'];
			$data['source_dtm'] 			= date("Y-m-d H:i:s");
			$data['overtime_start_dtm'] 	= null;
			$data['overtime_end_dtm'] 		= null;
			$data['longitude'] 				= null;
			$data['latitude']				= null;
			$data['start_time'] 			= explode(' ',$orderData['xd1'])[1];

			$result 						= $entity->saveEx($data);

		}else if($orderData['order_subtype_id'] == $orderSubtype['REQUEST_TYPE_OVERTIME'] || $orderData['order_subtype_id'] == $orderSubtype['REQUEST_TYPE_OVERTIME_WORKDAY']){
			$orderData['type_id'] = Entity::TYPE_OVERTIME;
			$orderData['start_time'] = explode(" ",$orderData['xd1'])[1];
			$orderData['source_reference'] = $orderData['ref_id'];
			$orderData['start_dtm'] 	= $orderData['xd1'];
			$orderData['start_description'] 		= $orderData['order_desc'];
			$orderData['end_dtm'] 	= $orderData['xd2'];
			$entity = new Entity(Entity\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
			$result = $entity->saveEx($orderData);
		}else if($orderData['order_subtype_id'] == $orderSubtype['REQUEST_TYPE_CHANGE_SHIFT']){
			$storage  			= \Neuron\Attendance\Timetable\Objects\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig());
			$objectTimetable 	= new \Neuron\Attendance\Timetable\ObjectTimetable($storage);

			$storageChangeshift = Timetable\Changeshift\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig());
			$changeShift 		= new Timetable\Changeshift($storageChangeshift);

			// Load Changeshift by request ID
			$changeShift->load($orderData['order_id'],Timetable\Changeshift\Storage::LOADBY_REC_ID);
			// Create changeshift
			$result 		= $objectTimetable->changeShift($changeShift);
		}
		return $result;
	}

	public function cancel($requestId){
		$params 	= [
			'order_id' 	=> $requestId,
			'order_status_id' => 2, # Cancel mungkin / rejected gatau nyontek dari cancel yang web
			'xn5' => 4 # Cancel mungkin / rejected gatau nyontek dari cancel yang web
		];
        /* Update Entity */
		$entity = new OrderEntity(OrderEntity\Storage::factory($this->_storage->getAdapter(), $this->_storage->getConfig()));
        $result = $entity->update($params);
        
        $resultArray    = [];
        if($result->code == 0){
            /* Load order with flow */
            $load = $entity->loadEx([
                'order_id' => $params['order_id'],
                'flow'     => 'list'
            ],OrderEntity\Storage::LOADBY_ID);
            
            if($load->code == 0){   
                $data   = $load->data;
                if($data['flow']){
                    $flows  = $data['flow'];
                    /* Update Order Flow */
					$resultFlows    = [];
					$generic 		= new \Neuron\Attendance\Generic;
					$addonConfig    = Generic::getAddonConfig();
                    foreach($flows as $flow){
                        if($flow['order_flow_task_code'] == 'approval'){
							$flowModel  = new SimpleFlow\Flow(SimpleFlow\Flow\Storage::factory($this->_storage->getAdapter(),$addonConfig));
                            $flow['order_flow_status_id'] = 3;
                            $flow['remark_out'] = "Canceled by requester";
                            $flowModel->push($flow);
                            $save = $flowModel->save(true);
                            $resultFlows[] = $save;
                        }
                    }
                    $resultArray['flow'] = $resultFlows;
                }
            }
        }

        $result     = new \Neuron\Generic\Result();
		$result->data = $resultArray;
		return $result;
    }
	
	public function validateRequest($ref_id, $request_type, $start_dtm, $end_dtm){
		return $this->_storage->validateRequest($ref_id, $request_type, $start_dtm, $end_dtm);
	}

}
