<?php
namespace Neuron\Attendance\Request;

use Neuron\Generic\Result;
use Neuron\Generic\Entity\Map;
use Neuron\Generic\Entity\Mapper;
use Neuron\Attendance\Request;
use Neuron\Notification;
/**
 * 
 */
class Approval extends Mapper{
    CONST APPROVAL_STATUS_REQUESTED 	= 1;
    CONST APPROVAL_STATUS_APPROVED 		= 2;
    CONST APPROVAL_STATUS_REJECTED 		= 3;
    CONST APPROVAL_STATUS_CANCELED 		= 4;

	protected $_storage;

	function __construct(Approval\Storage\Skeleton $storage,$data = array()){
		$this->_storage = $storage;
		parent::__construct(
			$data,
			array(
				new Map('approval_id'),
				new Map('request_id'),
				new Map('ref_code'),
				new Map('ref_id'),
				new Map('status_id'),
				new Map('approval_status_id',null,true),
				new Map('approval_status_name',null,true),
				new Map('approval_dtm'),
				new Map('reason'),
				new Map('ref_name',null,true),
				new Map('start_dtm',null,true),
				new Map('end_dtm',null,true),
				new Map('request_type',null,true),
				new Map('request_dtm',null,true),
				new Map('request_message',null,true),
				new Map('request_type_name',null,true),
				new Map('status_id_name',null,true),
				new Map('request_status_id',null,true),
				new Map('requester_ref_id',null,true),
				new Map('requester_ref_name',null,true),
				new Map('avatar',null,true)
			),
			parent::CASE_INSENSITIVE
		);
	}
	public function save($update = FALSE){
		$save 		= $this->_storage->save($this,$update);
		if($save){
			$this->approval_id = $save;
			return $save;
		}else{
			return false;
		}
	}
	public function saveEx($data 	= array(),$publicURL = null){
		
		$result 		= new Result();
		$this->push($data);
		if(!$this->save(FALSE)){
			$result->code 	= 500;
			$result->info 	= "failed saving approval";
			return $result;
		}


		/* Send Notification */
		$requestTypeName            = "";
		switch ($data['request_type']) {
			case Request::REQUEST_TYPE_ATTENDANCE:
				$requestTypeName    = 'Attendance';
				break;
			case Request::REQUEST_TYPE_OVERTIME:
				$requestTypeName    = 'Overtime';
				break;
			case Request::REQUEST_TYPE_CHANGE_SHIFT:
				$requestTypeName    = "Change shift";
				break;
			default:
				$requestTypeName    = "Attendance";
				break;
		}

		/* get user_id yang akan dikirim notification*/
		if(class_exists('\Neuron\Employee\Entity')){
			$storageEmp		= \Neuron\Employee\Entity\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig());
			$employee 		= new \Neuron\Employee\Entity($storageEmp);
			$map 			= $employee->loadMap($data['ref_id'],'person_id');
			if($map['data']){
				$user_id 	= $map['data']['user_id'];
			}else{
				$result->code 	= 501;
				$result->info 	= "Failed sending notification while loading approval map.";
				return $result;
			}
		}else{
			/* Block else digunakan untuk mengambil user_id namun tidak menggunakan modul employee */
			$user_id 		= $data['ref_id'];
		}

		/* Jika tidak di set ambil publicURL dari \Framework\Entity */
		if(!$publicURL){
			if(class_exists('\Neuron\Application\Framework\Entity')){
				$naf 	= \Neuron\Application\Framework\Entity::get();
				$publicURL 	= $naf->layout()->getHomeURL();
			}
		}

		/* Validasi public URL */
		if(!$publicURL){
			$result->code 	= 400;
			$result->info 	= "Failed sending notification. publicURL Cannot be null";
			return $result;
		}

		$inboxStorage   = Notification\Inbox\Storage::factory(
			$this->_storage->getAdapter(),
			$this->_storage->getConfig(),
			Notification\Profile::TYPE_FCM,
			$user_id
		);

		$inbox      = new Notification\Inbox($inboxStorage);
		$inbox->setPublicUrl($publicURL);

		/* Load Request */
		$requestStorage 	= Request\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig());
		$request 			= new Request($requestStorage);
		$dataRequest 		= $request->load($this->request_id);
		$requesterRefID 	= $dataRequest['ref_id'];

		if (class_exists('\Neuron\Employee\Entity')) {
			$storageEmp		= \Neuron\Employee\Entity\Storage::factory($this->_storage->getAdapter(), $this->_storage->getConfig());
			$employee 		= new \Neuron\Employee\Entity($storageEmp);
			$requesterMap   = $employee->loadMap($requesterRefID, 'person_id');
			$requester_name = $requesterMap['data']['first_name'] . ' ' . $requesterMap['data']['last_name'];
		} else {
			$requester_name 	= null;
		}
		
		$result = $inbox->pushNotification(
			$user_id,
			'REQUEST_NEED_APPROVAL',
			array(
				'name' => $requester_name,
				'request_type' => $requestTypeName,
			)
		);
		
		return $result;
	}
	public function load($id = null, $mode = Approval\Storage::LOADBY_ID, $params = array()){
		$result 			= new Result();
		$data 				= array();
		$dataApproval 		= $this->_storage->load($id,$mode,$params);
		$dataApproval['approval_status_id'] 	= $dataApproval['status_id'];

		$this->push($dataApproval);
		
		$request 							= new Request(Request\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
		$dataRequest 						= $request->load($this->request_id);
		$dataRequest['request_status_name']	= $dataRequest['status_name'];
		$dataRequest['request_status_id']	= $dataRequest['status_id'];


		
		$data 			= array_merge($dataApproval,$dataRequest);
		
		$result->data = $data;

		return $result;
	}
	public function getApprovalStatus($request_id){
		$result 		= new Result;
		$approvals 		= new Approvals($this->_storage);
		$dataApprovals 	= $approvals->load(null,Approval\Storage::LISTBY_REQUEST_ID,array('request_id' => $request_id));
		$totalRequested = 0;
		$totalApproved 	= 0;
		$totalRejected 	= 0;
		$totalCanceled 	= 0;

		if($dataApprovals->data['rows']){
			foreach ($dataApprovals->data['rows'] as $data) {
				switch ($data['status_id']) {
					case self::APPROVAL_STATUS_REQUESTED : 
						$totalRequested++;
						break;
					case self::APPROVAL_STATUS_APPROVED : 
						$totalApproved++;
						break;
					case self::APPROVAL_STATUS_REJECTED : 
						$totalRejected++;
						break;
					case self::APPROVAL_STATUS_CANCELED : 
						$totalCanceled++;
						break;
				}
			}
		}

		$result->data = array(
			'total_approve' 	=> $totalApproved,
			'total_reject' 		=> $totalRejected,
			'total_requested'	=> $totalRequested,
			'total_canceled' 	=> $totalCanceled
		);

		return $result;
	}
	public function countApproval_OLD($params = array()){
		$result 	= new Result();

		if(!isset($params['ref_id'])){
			$result->code = 400;
			$result->info = "ref_id required!";
			return $result;
		}

		$data 	= $this->_storage->countApproval($params);
		$rows 	= array();
		foreach ($data as $row ) {
            unset($row["request_id"]);                                          
            unset($row["ref_code"]);;
            unset($row["ref_id"]);            
            unset($row["start_dtm"]);     
            unset($row["end_dtm"]);              
            unset($row["request_type"]);  
            unset($row["request_dtm"]);  
            unset($row["status_id"]);                                 
            unset($row["request_message"]);      
            unset($row["type_id"]);
            unset($row["setting_id"]);                                    
			$rows[] = $row;
		}
		$result->data = $rows;
		return $result;
	}

	public function countApproval($params = array()){
		$result = new Result();

		$rows = array();
		$requestedApprovals = $this->_storage->countApproval($params);
		foreach ($requestedApprovals as $ra) {
			$typeName 	= $ra['type_name'];
			$count 		= $ra['count'];
			$rows[$ra['type_id']] 	= array(
				'type_id'		=> $ra['type_id'],
				'count' 		=> $count,
				'type_name' 	=> $typeName,
			);
		}

		$pastApproval = $this->_storage->countApproval($params,0);

		foreach ($pastApproval as $pa) {

			if( !isset($rows[$pa['type_id']]['type_name']) ){
				$rows[$pa['type_id']]['count'] = "0";
				$rows[$pa['type_id']]['type_name'] = $pa['type_name'];	
			}
			$rows[$pa['type_id']]['type_id'] = $pa['type_id'];
			
			$rows[$pa['type_id']]['pastCount'] = $pa['count'];
		}

		$newRows = array();
		foreach ($rows as $data ) {
			$newRows[] = $data;
		}

		$result->data = $newRows;

		return $result;

	} 
}