<?php
namespace Neuron\Attendance\Request\Approval\Storage;

use Neuron\Attendance\Request\Approval;
use Zend\Db\Sql\Expression;

class Mysql extends \Neuron\Db\Storage implements Skeleton{

	public function loadList($id = null, $mode = Approval\Storage::LIST_ALL,$params = array() ){
		$select 		= $this->select();
		$select->from(array('ara' => $this->_tables['attd_request_approval']));
		
		$select->join(
			array('emp' => $this->_tables['person']),
			'ara.ref_id = emp.id', 
			array(
				'ref_name' => new \Zend\Db\Sql\Expression("CONCAT(emp.first_name,' ',emp.last_name)"),
			)
		);

		$select->join(
			array(
				'r' => $this->_tables['attd_request'],
			),
			'ara.request_id = r.request_id',
			array('request_type','requester_ref_id' => 'ref_id','request_dtm')
		);

		$select->join(
			array(
				'empr' => $this->_tables['person']
			),
			'empr.id = r.ref_id',
			array(
				'requester_ref_name' => new \Zend\Db\Sql\Expression("CONCAT(empr.first_name,' ',empr.last_name)"),
			)
		);

		$select->join(
			array('empd' => $this->_tables['person_detail']),
			'empr.id = empd.person_id',
			array('avatar' => 'pic_official')
		);

		$select->join(
			array(
				'rt' => $this->_tables['attd_request_type'],
			),
			'r.request_type = rt.type_id',
			array('request_type_name' => 'type_name')
		);

		$select->join(
			array(
				'ars' => $this->_tables['attd_request_status']
			),
			'ara.status_id = ars.status_id',
			array('status_id_name' => 'name')
		);

		if($mode != Approval\Storage::LIST_ALL){
			switch ($mode) {
				case Approval\Storage::LISTBY_REQUEST_ID:
					$select->where(array($this->_('r.request_id') => $params['request_id']));
					break;
				case Approval\Storage::LISTBY_REF_ID:
					$select->where(array($this->_('ara.ref_id') => $params['ref_id']));
					break;
				case Approval\Storage::LISTBY_REF_ID_STATUS_ID:
					$select->where(array($this->_('ara.ref_id') => $params['ref_id'], $this->_('r.status_id') => $params['status_id']));
					break;

			}
		}

		if(isset($params['length']) && $params['length'] > 0){
			$select->limit(intval($params['length']));
			if(isset($params['start'])){
				$select->offset(intval($params['start']));
			}
		}

		if(isset($params['order'])){
			$columnName 	= $params['columns'][ $params['order'][0]['column'] ]['data'];
			if($columnName == 'ref_id'){
				$columnName = 'ara.ref_id';
			}
			$dir 			= $params['order'][0]['dir'];
			$select->order($columnName.' '.$dir);
		}

		if(isset($params['search']['value']) && strlen($params['search']['value']) > 0){
			$nest 	= $select->where->nest();
			
			$nest->like(    new Expression("CONCAT(empr.first_name,' ',empr.last_name)") , new Expression('\'%' . $params['search']['value'] . '%\'') )->or;
			$nest->like(    new Expression("rt.type_name") , new Expression('\'%' . $params['search']['value'] . '%\'') )->or;
			$nest->like(    new Expression("r.request_dtm") , new Expression('\'%' . $params['search']['value'] . '%\'') )->or;
			$nest->like(    new Expression("ars.name") , new Expression('\'%' . $params['search']['value'] . '%\'') )->or;

			$nest->unnest();

		}
		$where = [];

		if(isset($params['request_type']) && $params['request_type'] != "") {
			$where['r.request_type'] = $params['request_type'];
		}
		if(isset($params['status_id']) && $params['status_id'] != "") {
			if(isset($params['notEquals']) && $params['notEquals'] == 1) {
				$where[] = 'ara.status_id != '. $params['status_id'];
			}else{
				// print_r('dsadas'); die();
				$where['ara.status_id'] = $params['status_id'];
			}
		}
		// print_r($where); die();
		
		$select->where($where);
		// echo str_replace('"','',$select->getSqlString()).'<br />';  
		
		try {
			
			$result 	= new \stdClass;
			$rs 		= $this->fetchAll($select);
			
			if($rs){
				$result->list 	= $rs;
				
				$joins = $select->joins->getJoins();
				$select->reset(\Zend\Db\Sql\Select::JOINS);
				
				foreach ($joins as $key => $join) {
					//re add join without cols
					$join['columns'] = [];
					$type = $join["type"];
					$select->join($join["name"], $join["on"], [], $type);
				
				}
				
				$select->reset('limit')->reset('offset');
				$select->reset(\Zend\Db\Sql\Select::ORDER);
				$select->columns(array(
					'countTotal' 	=> new \Zend\Db\Sql\Expression('COUNT(ara.ref_id)')
				));
				// echo str_replace('"','',$select->getSqlString()).'<br />';  die();
		
				$dataCount = $this->fetchRow($select);
				// print_r($dataCount);die();
				$result->countTotal = 0;
				
				if(isset($dataCount['countTotal'])){
					$result->countTotal = ($dataCount['countTotal'] > 0) ? $dataCount['countTotal']:0;
				}
				
			}else{
				$result->list 	= array();
				$result->countTotal 	= 0;
			}
			
			return $result;
		} catch (\Exception $e) {
			return false;
		}
	}
	public function load($id = null, $mode = Approval\Storage::LOADBY_ID,$params = array()){
		$select 		= $this->select();
		$select->from(array('ara' => $this->_tables['attd_request_approval']));
		$select->join(array('ars' => $this->_tables['attd_request_status']),'ars.status_id = ara.status_id',array('approval_status_name' => 'name'));

		switch ($mode) {
			case Approval\Storage::LOADBY_ID:
				$select->where(array('ara.approval_id' => $id));
				break;
		}

		try {
			$data = $this->fetchRow($select);
			return $data;
		} catch (Exception $e) {
			error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
			return false;
		}
	}
	public function save(Approval $approval, $update = FALSE){
		if($update){
			$update 	= $this->update();
			$update->table($this->_tables['attd_request_approval']);
			$update->set($this->__($approval->pull()));
			$update->where(array($this->_('approval_id') => $approval->approval_id));

			try {
				$save 	= $this->execute($update);
				if($save){
					return $approval->approval_id;
				}
			} catch (Exception $e) {
				error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
				return false;
			}
		}else{
			$approval->approval_id = 0;
			$insert = $this->insert();
			$insert->into($this->_tables['attd_request_approval']);
			$insert->values($this->__($approval->pull()));

			try {
				$save 	= $this->execute($insert);
				return $save;
			} catch (Exception $e) {
				error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
				return false;
			}
		}
	}
	public function countApproval($params = array(),$status = 1){
		$select 	= $this->select();
		$select->from(array(
			'ara' => $this->_tables['attd_request_approval']
		));
		$select->join(array('ar' => $this->_tables['attd_request']),'ar.request_id = ara.request_id');
		$select->join(array('art' => $this->_tables['attd_request_type']),'art.type_id = ar.request_type');

		$select->columns(array('count' => new Expression('COUNT(*)'),'type_name' => new Expression('art.type_name')));
		$select->group(array('ar.request_type'));

		$select->where(
			array(
				'ara.ref_id' => $params['ref_id']
			)
		);

		if($status == 1){
			$select->where(array('ara.status_id' => $status));
		}else{
			$select->where('ara.status_id != 1');
		}
		
		try {
			$res = $this->fetchAll($select);
			return $res;
		} catch (\Exception $th) {
			return false;
		}
	}

	public function loadCountByType($mode = Approval\Storage::LIST_ALL,$params = array() ){
		$select 		= $this->select();
		$select->columns(array('countTotal' => new \Zend\Db\Sql\Expression("COUNT(*)")));
		$select->from(array('ara' => $this->_tables['attd_request_approval']));
		
		$select->join(
			array('emp' => $this->_tables['person']),
			'ara.ref_id = emp.id', 
			array(
				'ref_name' => new \Zend\Db\Sql\Expression("CONCAT(emp.first_name,' ',emp.last_name)"),
			)
		);

		$select->join(
			array(
				'r' => $this->_tables['attd_request'],
			),
			'ara.request_id = r.request_id',
			array('request_type','requester_ref_id' => 'ref_id','request_dtm')
		);

		$select->join(
			array(
				'empr' => $this->_tables['person']
			),
			'empr.id = r.ref_id',
			array(
				'requester_ref_name' => new \Zend\Db\Sql\Expression("CONCAT(empr.first_name,' ',empr.last_name)"),
			)
		);

		$select->join(
			array('empd' => $this->_tables['person_detail']),
			'empr.id = empd.person_id',
			array('avatar' => 'pic_official')
		);

		$select->join(
			array(
				'rt' => $this->_tables['attd_request_type'],
			),
			'r.request_type = rt.type_id',
			array('request_type_name' => 'type_name')
		);

		$select->join(
			array(
				'ars' => $this->_tables['attd_request_status']
			),
			'ara.status_id = ars.status_id',
			array('status_id_name' => 'name')
		);

		if($mode != Approval\Storage::LIST_ALL){
			switch ($mode) {
				case Approval\Storage::LISTBY_REQUEST_ID:
					$select->where(array($this->_('r.request_id') => $params['request_id']));
					break;
				case Approval\Storage::LISTBY_REF_ID:
					$select->where(array($this->_('ara.ref_id') => $params['ref_id']));
					break;
				case Approval\Storage::LISTBY_REF_ID_STATUS_ID:
					$select->where(array($this->_('ara.ref_id') => $params['ref_id'], $this->_('ara.status_id') => $params['status_id']));
					break;

			}
		}

		if(isset($params['length']) && $params['length'] > 0){
			$select->limit(intval($params['length']));
			if(isset($params['start'])){
				$select->offset(intval($params['start']));
			}
		}

		if(isset($params['order'])){
			$columnName 	= $params['columns'][ $params['order'][0]['column'] ]['data'];
			if($columnName == 'ref_id'){
				$columnName = 'ara.ref_id';
			}
			$dir 			= $params['order'][0]['dir'];
			$select->order($columnName.' '.$dir);
		}

		if(isset($params['search']['value']) && strlen($params['search']['value']) > 0){
			$nest 	= $select->where->nest();
			
			$nest->like(    new Expression("CONCAT(emp.first_name,' ',emp.last_name)") , new Expression('\'%' . $params['search']['value'] . '%\'') )->or;
			$nest->like(    new Expression("rt.type_name") , new Expression('\'%' . $params['search']['value'] . '%\'') )->or;
			$nest->like(    new Expression("r.request_dtm") , new Expression('\'%' . $params['search']['value'] . '%\'') )->or;
			$nest->like(    new Expression("ars.name") , new Expression('\'%' . $params['search']['value'] . '%\'') )->or;

			$nest->unnest();

		}
		$where = [];

		if(isset($params['request_type']) && $params['request_type'] != "") {
			$where['r.request_type'] = $params['request_type'];
		}
		if(isset($params['status_id']) && $params['status_id'] != "") {
			if(isset($params['notEquals']) && $params['notEquals'] == 1) {
				$where[] = 'ara.status_id != '. $params['status_id'];
			}else{
				$where['ara.status_id'] = $params['status_id'];
			}
		}
		
		$select->where($where); 
		
		try {
			$data       = $this->fetchValue($select);
            return $data;
		} catch (\Exception $e) {
			return false;
		}
	}

}
