<?php
namespace Neuron\Attendance\Request;

use Neuron\Generic\Entity\Map;
use Neuron\Generic\Result;
use Neuron\Attendance\Request\Setting\Detail;
use Neuron\Attendance\Request\Setting\Details;


class Setting extends \Neuron\Generic\Entity\Mapper {
    
    protected $_storage;
    
    public function __construct(\Neuron\Attendance\Request\Setting\Storage\Skeleton $storage, $data = array()) {

        $this->_storage = $storage;

        parent::__construct(
            $data, 
            array(
                new Map('setting_id'),
                new Map('setting_name'),
                new Map('num_to_approve'),
                new Map('min_approved'),
                new Map('min_rejected')
            ),
            parent::CASE_INSENSITIVE
        );
    }

    public function listdata($params) {

        $data = $this->_storage->listdata($params,$this);
        $preparedResult = array(
            'draw' => $params['draw'],
            'recordsTotal' => $data['dataCount'],
            'recordsFiltered' => $data['dataFilterCount'],
            'data' => $data['dataAll']
        );

        $result = new \Neuron\Generic\Result();
        $result->data = $preparedResult;

        return $result;
        
    }
	
    public function load($id = null, $mode = \Neuron\Attendance\Request\Setting\Storage::LOADBY_ID,$params = array()) {
        $result = new Result();

        if ($data = $this->_storage->load($id,$mode,$params)) {
            $this->push($data);

            $details = new Details(Detail\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
            $details = $details->load($this->setting_id,Detail\Storage::LISTBY_SETTING_ID);

            $res    = new \stdClass;
            $res->setting = $this->pull();
            $res->detail  = $details->data['rows'];

            $result->data = $res;
            $result->info = "Setting found";

        }else{
            $result->code = 400;
            $result->info = "Failed loading data";
        }

        return $result;
    }

    public function save($update = false) {
        if ($id = $this->_storage->save($this, $update)) {
            $this->setting_id = $id;
            return true;
        }
        return false;
    }

    public function searchByName($setting_name) {
        return $this->_storage->searchByName($setting_name);
    }

    public function CheckDeployedSetting($params) {
        return $this->_storage->CheckDeployedSetting($params);
    }

    public function remove($setting_id){
        $result     = new Result();
        $this->setting_id   = $setting_id;

        /* Delete dulu setting detailnya */
        $detail         = new Setting\Detail(Setting\Detail\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
        $deleteDetail   = $detail->delete($setting_id,Setting\Detail\Storage::DELETEBY_SETTING_ID); 

        if(!$deleteDetail){
            $result->code   = 502;
            $result->info   = "Error deleting detail";    
        }else{
            if($this->_storage->remove($this)){
                $result->data   = $this->setting_id;
            }else{
                $result->code   = 501;
                $result->info   = "Error deleting data";
            }
        }

        
        return $result;
    }
}
