<?php

namespace Neuron\Attendance\Request\Setting\Detail\Storage;

use Neuron\Attendance\Request\Setting\Detail;
use Zend\Db\Sql\Predicate\Expression;

class Mysql extends \Neuron\Db\Storage implements Skeleton {

	public function listdata($params = null) {
		$select = $this->select()
			->from(array('detail' => $this->_tables['attd_request_setting_detail']))
			->join(array('appv' => $this->_tables['attd_request_setting_approval_type']), 'detail.approval_type_id = appv.approval_type_id', array('name'), \Zend\Db\Sql\Select::JOIN_LEFT);
		
		if(isset($params['setting_id'])) {
			$select->where(array('detail.setting_id' => intval($params['setting_id'])));
		}
		if(isset($params['setting_approval_id'])) {
			$select->where(array('detail.setting_id' => intval($params['setting_approval_id'])));
		}

		/*Check Datatable*/
		if(isset($params['draw'])){
			$offset = ($params->start/$params->length) * $params->length;
			$select->limit(intval($params->length));
			$select->offset(intval($offset));
		}

		// echo str_replace('"','',$select->getSqlString())."<br />";
		
		$data = $this->fetchAll($select);


		$dataAll = [];

		if($data) {
			foreach($data as $row) {
				if($row['approval_type_id']==2){
					if(class_exists('Neuron\Employee\Controller\Employee')){
						$row['type_value_name'] = $this->fetchValue($this->select()->from($this->_tables['person'])->columns(array('type_value_name'=>$this->expression('CONCAT(first_name, " ", last_name)')))->where(array('id'=>$row['type_value'])));
					}
				}else if($row['approval_type_id']==4){
					if(class_exists('Neuron\Organization\Controller\Organization')){
						$row['type_value_name'] = $this->fetchValue($this->select()->from($this->_tables['organization'])->columns(array('name'))->where(array('id'=>$row['type_value'])));
					}
				}else{
					$row['type_value_name'] = '';
				}
				array_push($dataAll, $row);
			}
		}
		// print_r($dataAll); die();
		$select->columns(array('total_count' => new \Zend\Db\Sql\Expression('COUNT(*)')));
		$dataCount = $this->fetchRow($select);
		           
		return array(
			'dataAll' => $dataAll,
			'dataCount' => $dataCount,
		);

	}

	public function load($id, $setting_id = false) {
		$select = $this->select()
			->from(array('detail' => $this->_tables['attd_request_setting_detail']))
			->where(array('detail.detail_id' => $id))
			->join(array('appv' => $this->_tables['attd_request_setting_approval_type']), 'detail.approval_type_id = appv.approval_type_id', array('name'), \Zend\Db\Sql\Select::JOIN_LEFT);

		if($setting_id) {
			$select->where(array('detail.setting_id' => $setting_id));
		}
		
		$result = $this->fetchRow($select);
		return $result;
	}

	public function save(\Neuron\Attendance\Request\Setting\Detail $model, $update = false) {
		if ($update) {
			$update = $this->update()
				->table($this->_tables['attd_request_setting_detail'])
				->set($this->__($model->pull()))
				->where($this->__(array('detail_id' =>$model->detail_id ,'setting_id' => $model->setting_id)));

			if ($this->execute($update)) {
				return $model->detail_id;
			}
		}
		else {
			$insert = $this->insert()
				->into($this->_tables['attd_request_setting_detail'])
				->values($this->__($model->pull()));
			$save = $this->execute($insert);
			if ($save) {
				$model->detail_id = $save;
				return $model->detail_id;
			}
		}

		return false;
	}

	public function saveChange($param) {
		$update = $this->update()
			->table($this->_tables['attd_request_setting_detail'])
			->set($this->__(array('seq' =>$param['seq'])))
			->where($this->__(array('detail_id' =>$param['detail_id'] ,'setting_id' => $param['setting_approval_id'])));

		// echo str_replace('"','',$update->getSqlString())."<br />";   exit;

		return $this->execute($update);
	}

	public function remove($id,$mode = SettingDetail\Storage::DELETEBY_ID,$params = array()){
		$delete 	= $this->delete();
		$delete->from($this->_tables['attd_request_setting_detail']);
		
		switch ($mode) {
			case Detail\Storage::DELETEBY_ID:
				$delete->where($this->__(array('detail_id' => $id)));
				break;
			case Detail\Storage::DELETEBY_SETTING_ID:
				$delete->where($this->__(array('setting_id' => $id)));
				break;
		}

		try {
			if($this->execute($delete)){
				return $id;
			}else{
				return false;
			}
		} catch (\Throwable $th) {
			return false;
		}

	}

	public function approvalTypeDdl($term = null) {
		$select = $this->select()
			->columns(['id' => 'approval_type_id','text' => 'name'])
			->from($this->_tables['attd_request_setting_approval_type']);

		if($term) {
			$select->where->like('name', '%'.$term.'%');
		}

		// echo str_replace('"','',$select->getSqlString())."<br />";   exit;

		$data = $this->fetchAll($select);

		$result = [];

		if($data) {
			foreach($data as $key => $values) {
				$result[$key] = $values;
				// array_push($result, $row);
			}
		}

		return $result;
	}

	public function searchExisting($setting_id, $seq) {
		$select = $this->select()
			->from(array('detail' => $this->_tables['attd_request_setting_detail']))
			->where(array('detail.setting_id' => $setting_id,'detail.seq' => $seq));

		$result = $this->fetchRow($select);
		return $result;
	}

	public function loadDetail($id){
		try {
			$select = $this->select()
							->from($this->_tables['attd_request_setting_detail'])
							->where(array('detail_id'=> $id));
			return $this->fetchRow($select);
		} catch (Exception $e) {
			return false;
		}
	}

	public function loadSeqUp($setting, $seq){
		try {
			$select = $this->select()
							->from($this->_tables['attd_request_setting_detail'])
							->where(array('setting_id'=>$setting, 'seq > '. $seq));
			return $this->fetchAll($select);
		} catch (Exception $e) {
			return false;
		}
	}

	public function loadList($id = null,$mode = Detail\Storage::LIST_ALL,$params = array()){
		$select 	= $this->select();
		$select->from($this->_tables['attd_request_setting_detail']);

		if($mode != Detail\Storage::LIST_ALL){
			switch ($mode) {
				case Detail\Storage::LISTBY_SETTING_ID:
					$select->where(array($this->_('setting_id') => $id));
					break;
			}
		}

		try {
			$result = new \stdClass;
			$data 	= $this->fetchAll($select);
			if($data){
				$result->rows 	= $data;

				$selectC 		= $select;
				$selectC->reset('limit')->reset('offset');
				$selectC->columns(array('countTotal' => new Expression('COUNT(*)')));

				$result->countTotal = $this->fetchRow($selectC)['countTotal'];

				return $result;
			}
			return false;
		} catch (\Exception $th) {
			return false;
		}
	}
}
