<?php

namespace Neuron\Attendance\Request\Setting\Storage;

use Zend\Db\Sql\Where;
use Zend\Db\Sql\Predicate\Expression;
use Neuron\Attendance\Request\Setting;

class Mysql extends \Neuron\Db\Storage implements Skeleton {

	public function listdata($params = null,$attdSetting) {
		$searchable = false;

		try {
			$select = $this->select()->from(array('ls' => $this->_tables['attd_request_setting']));

			if(isset($params['draw'])){
				$i = $params['order'][0]['column'];
				$dir = $params['order'][0]['dir'];
				$select->order(array($params['columns'][$i]['data'] . " " . $dir));
		
				$offset = ($params->start/$params->length) * $params->length;
				$select->limit(intval($params->length));
				$select->offset(intval($offset));

				// search
				if($params['search']['value']){
					$searchable = true;
					$columnList = $attdSetting->pull();
					$nest = $select->where->nest();
					foreach ($columnList as $key => $value) {
						$nest->like(new Expression('LOWER(ls.'.$key.')'),new Expression('\'%'.$params['search']['value'].'%\''));
						$nest->or;
	 				}
					// $select->where->like($where,\Zend\Db\Sql\Where::COMBINED_BY_OR);
					$nest->unnest();
				}
			}
			
			// echo str_replace('"','',$select->getSqlString())."<br />";
			$data = $this->fetchAll($select);

			$dataAll = [];
			if($data){
				foreach($data as $row) {
					$row['total_detail']  = $this->fetchValue($this->select()->columns(array('total'=>$this->expression('count(*)')))->from($this->_tables['attd_request_setting_detail'])->where(array('setting_id'=>$row['setting_id'])));
					if($row['num_to_approve'] > $row['total_detail']){
						$row['status'] = 3;
					}
					array_push($dataAll, $row);
				}
			}
			
			$selectC = $this->select()->from(['ls' => $this->_tables['attd_request_setting']]);
			$selectC->columns(['total_count' => new \Zend\Db\Sql\Expression('COUNT(*)')]);
			$dataCount = $this->fetchRow($selectC);

			$total = ($dataCount['total_count'] > 0) ? $dataCount['total_count']:0;

			$filtered = $total;
			if($searchable){
				$selectF = $this->select()->from(['ls' => $this->_tables['attd_request_setting']]);
				$selectF->columns(['total_count' => new \Zend\Db\Sql\Expression('COUNT(*)')]);
				if(!empty($params['search']['value'])) {
					$columnList = $attdSetting->pull();
					$nest = $selectF->where->nest();
					foreach ($columnList as $key => $value) {
						$nest->like(new Expression('LOWER(ls.'.$key.')'),new Expression('\'%'.$params['search']['value'].'%\''));
						$nest->or;
	 				}
					$nest->unnest();
	 				// $selectF->where($where,\Zend\Db\Sql\Where::COMBINED_BY_OR);
				}

				$dataFiltered = $this->fetchRow($selectF);
				$filtered = ($dataFiltered['total_count'] > 0) ? $dataFiltered['total_count']:0;
			}

			return array(
				'dataAll' => $dataAll,
				'dataCount' => $total,
				'dataFilterCount' => $filtered
			);

		} catch (\Exception $ex) {
			error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
			return false;
		}
	}

	public function load($id = null, $mode = Setting\Storage::LOADBY_ID,$params = array()) {

		try {
			$select = $this->select()
				->from($this->_tables['attd_request_setting']);

			switch ($mode) {
				case Setting\Storage::LOADBY_ID:
					$select->where(array($this->_('setting_id') => $id));
					break;
				case Setting\Storage::LOADBY_NAME:
					$select->where(array($this->_('setting_name') => $id));
			}
			$result = $this->fetchRow($select);
			return $result;
		} catch (\Exception $e) {
			return false;
		}
		
	}

	public function searchByName($setting_name) {
		$select = $this->select()
			->from($this->_tables['attd_request_setting'])
			->where(array('setting_name' => $setting_name));

		$result = $this->fetchRow($select);
		return $result;
	}

	public function CheckDeployedSetting($params) {

		$select = $this->select()->from(['t' => $this->_tables['attd_request_type']]);

		if(isset($params['id'])) {
			$select->where(array('t.setting_id' => $params['id']));
		}

		$data = $this->fetchAll($select);

		$dataAll = [];
		if($data) {
			foreach($data as $row) {
				array_push($dataAll, $row);
			}
		}

		$output['dataAll'] = $dataAll;

		return $output;
	}

	public function save(\Neuron\Attendance\Request\Setting $model, $update = false) {
		try {
			if ($update) {
				$update = $this->update()
					->table($this->_tables['attd_request_setting'])
					->set($this->__($model->pull()))
					->where($this->__(array('setting_id' => $model->setting_id)));

				if ($this->execute($update)) {
					return $model->setting_id;
				}
			}
			else {
				$model->setting_id 	= 0;
				$insert = $this->insert()
				->into($this->_tables['attd_request_setting'])
				->values($this->__($model->pull()));
				
				$save = $this->execute($insert);
				if ($save) {
					$model->setting_id = $save;
					return $model->setting_id;
				}
			}	
			return false;
		} catch (\Exception $e) {
			return false;
		}
		

	}

	public function remove(Setting $setting){
		$delete 	= $this->delete();
		$delete->from($this->_tables['attd_request_setting']);
		$delete->where(array('setting_id' => $setting->setting_id));
		
		try {
			if($this->execute($delete)){
				return $setting->setting_id;
			}else{
				return false;
			}
		} catch (\Exception $ex) {
			return false;
		}
		
	}
	
	public function loadList($id = null, $mode = Setting\Storage::LIST_ALL,$params = array()){
		$select 	= $this->select();
		$select->from(array('rs' =>$this->_tables['attd_request_setting']));

		if(isset($params['order'])){
			$columnName 	= $params['columns'][$params['order'][0]['column']]['data'];
			$dir 			= $params['order'][0]['dir'];
			
			$select->order($columnName.' '.$dir);
		};

		if(isset($params['search']['value']) && strlen($params['search']['value']) > 0){
			$nest = $select->where->nest();

			$nest->like(new Expression('LOWER(rs.setting_name)'), 	new Expression('\'%' . $params['search']['value'] . '%\''))->or;
			$nest->like(new Expression('LOWER(rs.num_to_approve)'), new Expression('\'%' . $params['search']['value'] . '%\''))->or;
			$nest->like(new Expression('LOWER(rs.min_approved)'), 	new Expression('\'%' . $params['search']['value'] . '%\''))->or;
			$nest->like(new Expression('LOWER(rs.min_rejected)'), 	new Expression('\'%' . $params['search']['value'] . '%\''))->or;

			$nest->unnest();
		}

		if(isset($params['length'])){
			$select->limit(intval($params['length']));
		}
		if(isset($params['start'])){
			$select->offset(intval($params['start']));
		}

		try {
			$result 	= new \stdClass;
			
			$data 	= $this->fetchAll($select);
			if($data){
				$rows 	= array();
				foreach ($data as $data ) {
					$rows[] 	= $data;
				}

				$select->reset('limit')->reset('offset');
				$select->columns(array('countTotal' => new Expression("COUNT(*)")));
				$countTotal = $this->fetchRow($select)['countTotal'];

				$result->rows 	= $rows;
				$result->countTotal = $countTotal;
			}else{
				$result->rows 	= array();
				$result->countTotal = 0;
			}

			return $result;
		} catch (\Exception $th) {
			return false;
		}

	}
}
