<?php

namespace Neuron\Attendance\Request\Storage;
use Neuron\Attendance\Request;
use Neuron\Db\Sql\Select;
use Zend\Db\Adapter\Adapter;
use Zend\Db\Sql\Where;
use Zend\Db\Sql\Predicate\Expression;
use \Neuron\Db\Parameter;

class Mysql extends \Neuron\Db\Storage implements Skeleton {

	public function getTotalRows() {
		try {

			$select = $this->select()
							->columns(array('total' => $this->expression('FOUND_ROWS()')));
														 
			return $this->fetchValue($select);  

		} catch (\Exception $ex) {
			return false;
		}              
    }

	public function listRequest($params=false) {
		try {
            $select=$this->select()->from(array('ar' => $this->_tables['attd_request']));

            $select->join(array('ars'=> $this->_tables['attd_request_status']), 'ar.status_id=ars.status_id', array('status_name'=>'name'));
            $select->join(array('art' => $this->_tables['attd_request_type']), 'ar.request_type = art.type_id', array('request_type_name'=>'type_name'));
			$select->join(array('arstg' => $this->_tables['attd_request_setting']), 'art.setting_id = arstg.setting_id', array('num_to_approve','min_approved','min_rejected'));
			$select->join(array('arst' => $this->_tables['attd_request_status']), 'ar.status_id = arst.status_id', array('status_id_name'=>'name'));

            if(isset($params['filter'])) {
                $filter = $params['filter'];
                $page = $params['page'];
                $order = $params['order'];
                $rowCount = $params['row_count'];

                $offset = $page;

                $select->limit(intval($rowCount));
                $select->offset(intval($offset));


                if($filter['ref_id']) {
                    $select->where(array('ar.ref_id'=> $filter['ref_id']));
                    if($filter['searchValue']){
                        $searchValue = $filter['searchValue'];
                        $whereString = "(
                            lower(ar.request_message) like '%".strtolower($searchValue)."%'
                        )";
                        $select->where($whereString);
                    }
                }

                // ordering 
                if($order){
                    if($order['columns'] == 'id'){
                        $order['columns'] = 'request_id';
                    }else if($order['columns'] == 'request_type_name'){
                        $order['columns'] = 'request_type';
                    }
                    $select->order($order['columns'].' '.$order['dir']);
                }

                // echo str_replace('"','',$select->getSqlString())."<br />";           
                // echo('$select=> '.str_replace('"','',$select->getSqlString()));die();     
                return $this->fetchAll($select);  

            }
            else {
                $searchable = false;
                $where = [];
                
                $arrColumn = [];
                $arrColumn[] = "*";
                $arrColumn['total_approve'] = new Expression('IFNULL((SELECT COUNT(*) FROM attd_request_approval AS ara WHERE ara.request_id = ar.request_id AND ara.status_id = 2 GROUP BY ara.request_id), 0)');
                $arrColumn['total_reject'] = new Expression('IFNULL((SELECT COUNT(*) FROM attd_request_approval AS ara WHERE ara.request_id = ar.request_id AND ara.status_id IN (3,4,5,6) GROUP BY ara.request_id), 0)');
                
                if(isset($params['draw'])) {
                    if(isset($params['columns'])) {
                        
                        foreach ($params['columns'] as $col) {
                            if($col['data'] != null) {
                                if($col['name'] != null) {
                                    $arrColumn[$col['data']] = $col['name'];
                                }
                                else {
                                    $arrColumn[$col['data']] = $col['data'];
                                }
                            }
                        }                        
                    }
    
                    $offset = ($params->start/$params->length) * $params->length;
                    $select->limit(intval($params->length));
                    $select->offset(intval($offset));
            
                    $i = $params['order'][0]['column'];
                    $dir = $params['order'][0]['dir'];
                    $select->order(array($params['columns'][$i]['data'] . " " . $dir));
                }

                $select->columns($arrColumn,false);
                if(isset($params['request_id'])){
                    $where['ar.request_id'] = $params['request_id'];
                }
                if(isset($params['ref_id'])) {
                    $where['ar.ref_id'] = $params['ref_id'];
                }
                if(isset($params['request_type']) && $params['request_type'] != "") {
                    $searchable = true;
                    $where['ar.request_type'] = $params['request_type'];
                }
                if(isset($params['status_id']) && $params['status_id'] != "") {
                    $searchable = true;
                    $where['ar.status_id'] = $params['status_id'];
                }
                if($params['start_dtm'] != null && $params['end_dtm'] != "") {
                    $searchable = true;
                    $where[] = new Expression('(DATE(ar.request_dtm) BETWEEN  "'.$params['start_dtm'].'" AND "'.$params['end_dtm'].'")');
                }

                $select->where($where);

                // echo str_replace('"','',$select->getSqlString())."<br />";
                $data = $this->fetchAll($select);
                $dataAll = [];

                if($data) {
                    foreach($data as $row) {
                        array_push($dataAll, $row);
                    }
                }

                $selectC = $this->select()->from(['ar' => $this->_tables['attd_request']]);
                $selectC->columns(['total_count' => new \Zend\Db\Sql\Expression('COUNT(*)')]);
                $selectC->join(array('ars'=> $this->_tables['attd_request_status']), 'ar.status_id=ars.status_id', array('status_name'=>'name'));
                $selectC->join(array('art' => $this->_tables['attd_request_type']), 'ar.request_type = art.type_id', array('type_name'));
                $selectC->join(array('arstg' => $this->_tables['attd_request_setting']), 'art.setting_id = arstg.setting_id', array('num_to_approve','min_approved','min_rejected'));
                $selectC->join(array('arst' => $this->_tables['attd_request_status']), 'ar.status_id = arst.status_id', array('status'=>'name'));
                $selectC->where($where);
                $dataCount = $this->fetchRow($selectC);
                
                $total = ($dataCount['total_count'] > 0) ? $dataCount['total_count']:0;
                $filtered = $total;

                if($searchable) {
                    $selectF = $this->select()->from(['ar' => $this->_tables['attd_request']]);
                    $selectF->columns(['total_count' => new \Zend\Db\Sql\Expression('COUNT(*)')]);
                    $selectF->join(array('ars'=> $this->_tables['attd_request_status']), 'ar.status_id=ars.status_id', array('status_name'=>'name'));
                    $selectF->join(array('art' => $this->_tables['attd_request_type']), 'ar.request_type = art.type_id', array('type_name'));
                    $selectF->join(array('arstg' => $this->_tables['attd_request_setting']), 'art.setting_id = arstg.setting_id', array('num_to_approve','min_approved','min_rejected'));
                    $selectF->join(array('arst' => $this->_tables['attd_request_status']), 'ar.status_id = arst.status_id', array('status'=>'name'));

                    $selectF->where($where);
                    $dataFiltered = $this->fetchRow($selectF);
                    $filtered = ($dataFiltered['total_count'] > 0) ? $dataFiltered['total_count']:0;
                }

                $result = ['data' => $dataAll, 'total' => $total, 'filtered' => $filtered];
                return $result;
            }

		} catch (\Exception $ex) {
			return false;
		}       
	}

	public function save(\Neuron\Attendance\Request $req, $update = false){
		try{
            if ($update) {
                $update = $this->update()
                            ->table($this->_tables['attd_request'])
                            ->set($this->__($req->pull()))
                            ->where($this->__(array('request_id' => $req->request_id)));

                // echo str_replace('"','',$update->getSqlString()).'<br />'; die();
                if ($this->execute($update)) {
                    return $req->request_id;
                }
              
            } else {
                $req->request_id    = 0;
                $insert = $this->insert()
                            ->into($this->_tables['attd_request'])
                            ->values($this->__($req->pull()));
                // echo str_replace('"','',$insert->getSqlString()).'<br />'; die();
                $save = $this->execute($insert);
                if ($save) {
                    $req->request_id = $save;
                    return $req->request_id;
                }
            }
        }catch(\Exception $e){
            return $e->getMessage();
        }
	}

    public function saveApproval($data, $update = false){
        try{
            if ($update) {
                $update = $this->update()
                            ->table($this->_tables['attd_request_approval'])
                            ->set($this->__(array(
                                                'request_id'=>$data['request_id'],
                                                'ref_code'=>$data['ref_code'],
                                                'ref_id'=>$data['ref_id'],
                                                'status_id'=>$data['status_id'],
                                                'approval_dtm'=>$data['approval_dtm'],
                                                'reason'=>$data['reason'])))
                            ->where($this->__(array('approval_id' => $data['approval_id'])));

                // echo str_replace('"','',$update->getSqlString()).'<br />'; die();
                if ($this->execute($update)) {
                    return $data['approval_id'];
                }
              
            } else {
                $insert = $this->insert()
                            ->into($this->_tables['attd_request_approval'])
                            ->values($this->__($data));
                // echo str_replace('"','',$insert->getSqlString()).'<br />'; die();
                $save = $this->execute($insert);
                if ($save) { 
                    return $save;
                }
            }
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }
	
	public function remove($id) {
        try {
            $delete = $this->delete()
                    ->from($this->_tables['attd_request'])
                    ->where(array('request_id' => $id));
            return $this->execute($delete);
        }catch (\Exception $ex) {
            return false;
        }  
    }

    public function loadApprovalBy($by, $id) {
        try {

            $select = $this->select()
                            ->from(array('ar' => $this->_tables['attd_request_approval']))
                            ->where(array($by => $id));

            $select->join(
                array('ats' => $this->_tables['attd_request_status']), 
                'ar.status_id = ats.status_id', 
                array('*'), 
                \Zend\Db\Sql\Select::JOIN_LEFT
            );
            
            //echo str_replace('"','',$select->getSqlString())."<br />";                
            return $this->fetchAll($select);

        } catch (\Exception $ex) {
            return false;
        }       
    }	

    public function removeApprovalBy($by, $id){
        try {
            $delete = $this->delete()
                    ->from($this->_tables['attd_request_approval'])
                    ->where(array($by => $id));
            return $this->execute($delete);
        }catch (\Exception $ex) {
            return false;
        }  
    }

    public function loadApprovalType(){
        try {

            $select = $this->select()
                            ->from(array('arsat' => $this->_tables['attd_request_setting_approval_type']));
            
            //echo str_replace('"','',$select->getSqlString())."<br />";                
            return $this->fetchAll($select);  

        } catch (\Exception $ex) {
            return false;
        }  
    }

    public function loadSetting($params){
        try {

            $select = $this->select()
                            ->from(array('arsd' => $this->_tables['attd_request_setting_detail']))
                            ->join(array('ars'=> $this->_tables['attd_request_setting']), 'arsd.setting_id=ars.setting_id', array('*'),  \Zend\Db\Sql\Select::JOIN_LEFT);
            $select->where($params);
            
            // echo str_replace('"','',$select->getSqlString())."<br />";
            return $this->fetchAll($select);  

        } catch (\Exception $ex) {
            return false;
        }
    }

    public function saveSetting($data, $update = false){
        try{
            if ($update) {
                $update = $this->update()
                            ->table($this->_tables['attd_request_setting'])
                            ->set($this->__(array(
                                                'num_to_approve'=>$data['num_to_approve'],
                                                'min_approved'=>$data['min_approved'],
                                                'min_rejected'=>$data['min_rejected'],
                                                'start_date'=>$data['start_date'],
                                                'end_date'=>$data['end_date'])))
                            ->where($this->__(array('setting_id' => $data['id'])));

                // echo str_replace('"','',$update->getSqlString()).'<br />'; die();
                if ($this->execute($update)) {
                    return $data['id'];
                }
              
            } else {
                $insert = $this->insert()
                            ->into($this->_tables['attd_request_setting'])
                            ->values($this->__(array(
                                                    'num_to_approve'=>$data['num_to_approve'],
                                                    'min_approved'=>$data['min_approved'],
                                                    'min_rejected'=>$data['min_rejected'],
                                                    'start_date'=>$data['start_date'],
                                                    'end_date'=>$data['end_date'])));
                // echo str_replace('"','',$insert->getSqlString()).'<br />'; die();
                $save = $this->execute($insert);
                if ($save) { 
                    return $save;
                }
            }
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }

    public function saveSettingDetail($data, $update= false){
        try{
            if ($update) {
                $update = $this->update()
                            ->table($this->_tables['attd_request_setting_detail'])
                            ->set($this->__(array(
                                                'setting_id'=>$data['setting_id'],
                                                'seq'=>$data['seq'],
                                                'approval_type_id'=>$data['approval_type_id'],
                                                'type_value'=>$data['type_value'])))
                            ->where($this->__(array('detail_id' => $data['detail_id'])));

                // echo str_replace('"','',$update->getSqlString()).'<br />'; die();
                if ($this->execute($update)) {
                    return $data['setting_id'];
                }
              
            } else {
                $insert = $this->insert()
                            ->into($this->_tables['attd_request_setting_detail'])
                            ->values($this->__(array(
                                                    'setting_id'=>$data['setting_id'],
                                                    'seq'=>$data['seq'],
                                                    'approval_type_id'=>$data['approval_type_id'],
                                                    'type_value'=>$data['type_value'])));
                // echo str_replace('"','',$insert->getSqlString()).'<br />'; die();
                $save = $this->execute($insert);
                if ($save) { 
                    return $save;
                }
            }
        }catch(\Exception $e){
            return false;
        }
    }

    public function saveApprove($data){
        try{
            $update = $this->update()
                        ->table($this->_tables['attd_request_approval'])
                        ->set($this->__(array(
                                            'status_id'=>$data['status_id'],
                                            'approval_dtm'=> $this->expression('now()'),
                                            'reason'=>$data['reason'])))
                        ->where($this->__(array('approval_id' => $data['approval_id'])));

            // echo str_replace('"','',$update->getSqlString()).'<br />'; die();
            if ($this->execute($update)) {
                return $data['approval_id'];
            }else{
                return false;
            }
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }

    public function updateRequest($data){
        try{
            $update = $this->update()
                        ->table($this->_tables['attd_request'])
                        ->set($this->__(array('status_id'=> $data['status_id'])))
                        ->where($this->__(array('request_id' => $data['request_id'])));

            // echo str_replace('"','',$update->getSqlString()).'<br />'; die();
            if ($this->execute($update)) {
                return $data['request_id'];
            }else{
                return false;
            }
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }

    public function loadRequest($by, $id){
        try {
            $select = $this->select()
                            ->from(array('ar' => $this->_tables['attd_request']))
                            ->where(array($by=>$id));
            // $select->join(array('ars'=> $this->_tables['attd_request_setting']), 'arsd.setting_id=ars.setting_id', array('*'),  \Zend\Db\Sql\Select::JOIN_LEFT);
            
            // echo str_replace('"','',$select->getSqlString())."<br />";                
            return $this->fetchAll($select);

        } catch (\Exception $ex) {
            return false;
        }
    }


    public function listApproval($page = 1, $rowCount = 10, $filter = array(), $ref_id = null,$post = array()) {
        try {
            $offset = $page;

            $select = $this->select()
                            ->quantifier($this->expression('SQL_CALC_FOUND_ROWS'))
                            ->from(array('ara'=> $this->_tables['attd_request_approval']))
                            ->join(array('ar' => $this->_tables['attd_request']), 'ara.request_id=ar.request_id', array('*'),  \Zend\Db\Sql\Select::JOIN_LEFT)
                            ->join(array('ars'=> $this->_tables['attd_request_status']), 'ar.status_id=ars.status_id', array('status_id_name'=>'name'),  \Zend\Db\Sql\Select::JOIN_LEFT)
                            ->where(array('ara.status_id'=>1))
                            ->limit(intval($rowCount))
                            ->offset(intval($offset));
            if($ref_id){
                $select->where(array('ara.ref_id'=>$ref_id));
            }
            if ($filter) {
                $select->where(
                    array(
                        'ref_id like "%' . $filter . '%"',
                        'message like "%' . $filter . '%"',
                        'request_type like "%' . $filter . '%"',
                        'status like "%' . $filter . '%"',
                    ),\Zend\Db\Sql\Where::COMBINED_BY_OR
                );
            }     

            if($post){
                $columnName = $post['columns'][$post['order'][0]['column']]['data'];
                $dir = $post['order'][0]['dir'];
                // var_dump($columnName);die();
                switch ($columnName) {
                    case 'ref_id':
                        $columnName = 'ar.ref_id';
                        break;
                    case 'start_date':
                        $columnName = 'ar.start_date';
                        break;
                    case 'request_type_name':
                        $columnName = 'request_type';
                        break;
                    case 'end_date':
                        $columnName = 'ar.end_date';
                        break;
                    case 'ref_name':
                        $columnName = 'ar.ref_id';
                        break;
                    default:
                        $columnName;
                        break;
                }

                $select->reset('order');
                if($columnName == 'ref_name'){
                    $columnName = 'ara.ref_id';
                }

                $select->order($columnName.' '.$dir);
            }
            // echo('$select=> '.str_replace('"','',$select->getSqlString()));die();
            return $this->fetchAll($select);  

        } catch (\Exception $ex) {
            return false;
        }       
    }

    public function loadApproval($appr_id){
        try {
            $select = $this->select()
                        ->columns(array('approval_id','appr_ref_code'=>'ref_code', 'appr_ref_id'=>'ref_id', 'status_id', 'approval_dtm', 'reason'))
                        ->from(array('ara'=> $this->_tables['attd_request_approval']))
                        ->join(array('ar' => $this->_tables['attd_request']), 'ara.request_id=ar.request_id', array('*'),  \Zend\Db\Sql\Select::JOIN_LEFT)
                        ->join(array('ars'=> $this->_tables['attd_request_status']), 'ar.status_id=ars.status_id', array('status_id_name'=>'name'),  \Zend\Db\Sql\Select::JOIN_LEFT);
            if($appr_id){
                $select->where(array('ara.approval_id'=> $appr_id));
            }
            // echo str_replace('"','',$select->getSqlString())."<br />"; 
            return $this->fetchRow($select);
        } catch (Exception $e) {
            return false;
        }
    }

    public function loadList($mode = Request\Storage::LIST_ALL ,$params = array(),$timeMode = Request\Storage::TIME_WEEKLY){
        $select = $this->select()->from(array('request' => $this->_tables['attd_request']));
        $select->join(array('art' => $this->_tables['attd_request_type']),'art.type_id = request.request_type',array('request_type_name' => 'type_name'));
        $select->join(array('ars' => $this->_tables['attd_request_status']),'ars.status_id = request.status_id',array('status_id_name' => 'name'));
        
        if($mode != Request\Storage::LIST_ALL){
            switch ($mode) {
                case Request\Storage::LISTBY_REFID:
                    $select->where(array('request.ref_id' => $params['ref_id']));
                    break;
            }
        }

        if(isset($params['draw'])){
            $select->limit(intval($params['length']));
            $select->offset(intval($params['start']));


            $orderColumn    = $params['columns'][$params['order'][0]['column']]['data'];
            $dir            = $params['order'][0]['dir'];

            $select->order($orderColumn.' '.$dir);

            if(strlen($params['search']['value'] > 0)){
                $select->where("(
                    LOWER(start_dtm)          LIKE '%".strtolower($params['search']['value'])."%' OR
                    LOWER(end_dtm)            LIKE '%".strtolower($params['search']['value'])."%' OR
                    LOWER(request_dtm)        LIKE '%".strtolower($params['search']['value'])."%' OR
                    LOWER(request_message)    LIKE '%".strtolower($params['search']['value'])."%' OR
                    LOWER(request_id)         LIKE '%".strtolower($params['search']['value'])."%'

                )");
            }
        }
        
        /*Filtering*/
        if(isset($params['request_type']) && !empty($params['request_type'])){
            $select->where(array('request.request_type' => $params['request_type']));
        }

        if(isset($params['status_id']) && !empty($params['status_id'])){
            if(isset($params['notEquals']) && !empty($params['notEquals'])) {
                $select->where(array('request.status_id != '.$params['notEquals']));
            }else{
                $select->where(array('request.status_id' => $params['status_id']));
            }
        }

        if(isset($params['start_date']) && !empty($params['start_date'])){
            $select->where('DATE(request.start_dtm) >= "'.date('Y-m-d',strtotime($params['start_date'])).'"');
        }

        if(isset($params['end_date']) && !empty($params['end_date'])){
            $select->where('DATE(request.start_dtm) <= "'.date('Y-m-d',strtotime($params['end_date'])).'"');
        } 

        // defining time mode 
        if($timeMode != Request\Storage::TIME_NONE){
            switch ($timeMode) {
                case Request\Storage::TIME_WEEKLY:
                    //fetch data in this feek between monday & friday;
                        $thisMonday = date('Y-m-d',strtotime('this week monday'));
                        $thisFriday = date('Y-m-d',strtotime('this week friday'));
                    $select->where(
                        "start_dtm >= '".$thisMonday."' AND start_dtm <= '".$thisFriday."' "
                    );
                    break;
                case Request\Storage::TIME_MONTHLY:
                    $select->where(
                        "month(start_dtm) = '".date('m')."'"
                    );
                    break;
                case Request\Storage::TIME_YEARLY:
                    $select->where(
                        "year(start_dtm) = '".date('Y')."'"
                    );
                    break;
            }
        }
        try {

            $rs         = new \stdClass;
            // echo('$select=> '.str_replace('"','',$select->getSqlString()));die();
            $data       = $this->fetchAll($select);
            $rs->data   = $data;
            $selectC    = $select;
            $selectC->reset('limit')->reset('offset');
            $selectC->reset(Select::COLUMNS);
            $selectC->reset(Select::JOINS);
            $selectC->reset(Select::ORDER);

            $selectC->columns(array('countTotal' => new \Zend\Db\Sql\Expression("COUNT(*)")));
            // echo str_replace('"','',$selectC->getSqlString());die();
            $countTotal     = $this->fetchRow($selectC)['countTotal'];
            $rs->countTotal = $countTotal;
            return $rs;

        } catch (\Exception $e) {
            return false;
        }
    }

    public function liststatus($term = false){
        try {
            $select = $this->select()
                ->columns(['id' => 'status_id','text' => 'name'])
                ->from(array('ars'=> $this->_tables['attd_request_status']));
            
            if($term){
                $select->where->like('ars.name', '%'.$term.'%');
            }
            
            $data = $this->fetchAll($select);

            $result = [];

            if($data) {
                foreach($data as $key => $values) {
                    $result[$key] = $values;
                }
            }

            return $result;
        } catch (Exception $e) {
            return false;
        }
    }

    public function loadRecByTypeDate($type_id,$date,$ref_id){
        $select         = $this->select();
        $select->from(array(
            'ar'    => $this->_tables['attd_request']
        ));

        $select->where('status_id in (1)');
        $select->where('DATE(start_dtm) = "'.$date.'"');
        $select->where(array('request_type' => $type_id));
        $select->where(array('ref_id' => $ref_id));

        try {
            $data       = $this->fetchAll($select);
        } catch (\Exception $e) {
            return false;
        }
        return $data;
    }

    public function load($id = null, $mode = Request\Storage::LOADBY_ID,$params = array()){
        $select     = $this->select();
        $select->from(array('r' => $this->_tables['attd_request']));

        $select->join(
            array('rt' => $this->_tables['attd_request_type']), 
            'r.request_type = rt.type_id', 
            array('request_type_name' => 'type_name')
        );

        $select->join(
            array('rs' => $this->_tables['attd_request_status']), 
            'r.status_id = rs.status_id', 
            array('status_name' => 'name')
        );

        if(class_exists('\Neuron\Employee\Entity')){
            $select->join(
                array('emp' => $this->_tables['person']),
                'emp.id = r.ref_id',
                array('ref_name' => new Expression("CONCAT(first_name, ' ',last_name)"))
            );
        }

        switch ($mode) {
            case Request\Storage::LOADBY_ID:
                $select->where(array($this->_('request_id') => $id));
                break;
        }

        try {
            $row = $this->fetchRow($select); 
            return $row;
        } catch (Exception $e) {
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
            return false;
        }

    }

    public function loadCountByType($mode = Request\Storage::LIST_ALL ,$params = array(),$timeMode = Request\Storage::TIME_WEEKLY){
        $select = $this->select();
        $select->columns(array('countTotal' => new \Zend\Db\Sql\Expression("COUNT(*)")));
        $select->from(array('request' => $this->_tables['attd_request']));
        $select->join(array('art' => $this->_tables['attd_request_type']),'art.type_id = request.request_type',array('request_type_name' => 'type_name'));
        $select->join(array('ars' => $this->_tables['attd_request_status']),'ars.status_id = request.status_id',array('status_id_name' => 'name'));
        $select->where(array('request.ref_id' => $params['ref_id']));
        
        if(isset($params['draw'])){
            $select->limit(intval($params['length']));
            $select->offset(intval($params['start']));


            $orderColumn    = $params['columns'][$params['order'][0]['column']]['data'];
            $dir            = $params['order'][0]['dir'];

            $select->order($orderColumn.' '.$dir);

            if(strlen($params['search']['value'] > 0)){
                $select->where("(
                    LOWER(start_dtm)          LIKE '%".strtolower($params['search']['value'])."%' OR
                    LOWER(end_dtm)            LIKE '%".strtolower($params['search']['value'])."%' OR
                    LOWER(request_dtm)        LIKE '%".strtolower($params['search']['value'])."%' OR
                    LOWER(request_message)    LIKE '%".strtolower($params['search']['value'])."%' OR
                    LOWER(request_id)         LIKE '%".strtolower($params['search']['value'])."%'

                )");
            }
        }
        
        /*Filtering*/
        if(isset($params['request_type']) && !empty($params['request_type'])){
            $select->where(array('request.request_type' => $params['request_type']));
        }

        if(isset($params['status_id']) && !empty($params['status_id'])){
            if(isset($params['status_id_not']) && !empty($params['status_id_not'])){
                $select->where(array('request.status_id != '.$params['status_id']));
            }else{
                $select->where(array('request.status_id' => $params['status_id']));
            }
        }

        if(isset($params['start_date']) && !empty($params['start_date'])){
            $select->where('DATE(request.start_dtm) >= "'.date('Y-m-d',strtotime($params['start_date'])).'"');
        }

        if(isset($params['end_date']) && !empty($params['end_date'])){
            $select->where('DATE(request.start_dtm) <= "'.date('Y-m-d',strtotime($params['end_date'])).'"');
        } 

        // defining time mode 
        if($timeMode != Request\Storage::TIME_NONE){
            switch ($timeMode) {
                case Request\Storage::TIME_WEEKLY:
                    //fetch data in this feek between monday & friday;
                        $thisMonday = date('Y-m-d',strtotime('this week monday'));
                        $thisFriday = date('Y-m-d',strtotime('this week friday'));
                    $select->where(
                        "start_dtm >= '".$thisMonday."' AND start_dtm <= '".$thisFriday."' "
                    );
                    break;
                case Request\Storage::TIME_MONTHLY:
                    $select->where(
                        "month(start_dtm) = '".date('m')."'"
                    );
                    break;
                case Request\Storage::TIME_YEARLY:
                    $select->where(
                        "year(start_dtm) = '".date('Y')."'"
                    );
                    break;
            }
        }
        try {
            // echo('$select=> '.str_replace('"','',$select->getSqlString()));die();
            $data       = $this->fetchValue($select);
            return $data;

        } catch (Exception $e) {
            return false;
        }
    }
	
	public function validateRequest($ref_id, $request_type, $start_dtm, $end_dtm){
	
		$proc = $this->procedure()->call('recommendationValidation')
		->parameters(array(
			'in_employeeId' => new \Neuron\Db\Parameter($ref_id, \Neuron\Db\Parameter::TYPE_STRING, 32, \Neuron\Db\Parameter::INPUT),
			'in_orderSubTypeId' => new \Neuron\Db\Parameter($request_type, \Neuron\Db\Parameter::TYPE_INTEGER, 10, \Neuron\Db\Parameter::INPUT),
			'in_startDtm' => new \Neuron\Db\Parameter($start_dtm, \Neuron\Db\Parameter::TYPE_STRING, 10, \Neuron\Db\Parameter::INPUT),
			'in_endDtm' => new \Neuron\Db\Parameter($end_dtm, \Neuron\Db\Parameter::TYPE_STRING, 10, \Neuron\Db\Parameter::INPUT),
			'out_statusCode' => new \Neuron\Db\Parameter(null, \Neuron\Db\Parameter::TYPE_INTEGER, 2, \Neuron\Db\Parameter::OUTPUT),
			'out_statusMsg' => new \Neuron\Db\Parameter(null, \Neuron\Db\Parameter::TYPE_STRING, 200, \Neuron\Db\Parameter::OUTPUT),
		));
		//print_r($this->debug(1));
		if($result = $this->execute($proc)){
		//print_r($this->getLastError());die();
			$result = array(
				'out_statusCode' => intval($this->getOutput('out_statusCode')),
				'out_statusMsg' => $this->getOutput('out_statusMsg'),
			);
		}

		return $result;

	}
}