<?php
namespace Neuron\Attendance;

use Neuron\Generic\Entity\Map;
use Neuron\Generic\Result;
use Neuron\Attendance\Attendance;
use Neuron\Attendance\Timetable\ObjectTimetable;
use Neuron\Calendar\Event;
use Neuron\Calendar\Events;

class Shift extends \Neuron\Generic\Entity\Mapper {
    
    const SUNDAY    = 1;
    const MONDAY    = 2;
    const TUESDAY   = 3;
    const WEDNESDAY = 4;
    const THURSDAY  = 5;
    const FRIDAY    = 6;
    const SATURDAY  = 7;

    const ORDER_TYPE_ATTENDANCE    = 2101;
    const ORDER_TYPE_OVERTIME      = 2102;
    const ORDER_TYPE_CHANGESHIFT   = 2103;
    const ORDER_TYPE_OVERTIME_WORKDAY      = 2110;

    protected $_storage;

    public function __construct(\Neuron\Attendance\Shift\Storage\Skeleton $storage, $data = array()) {

        $this->_storage = $storage;

        parent::__construct(
            $data, 
            array(
                new Map('timetable_shift_id'),
                new Map('timetable_id'),
                new Map('day_no'),
                new Map('is_workday'),
                new Map('on_duty_time'),
                new Map('off_duty_time'),
                new Map('in_beginning_time'),
                new Map('in_ending_time'),
                new Map('out_beginning_time'),
                new Map('out_ending_time'),
                new Map('late_time'),
                new Map('leave_early_time'),
                new Map('overtime_start_time'),
                new Map('start_date'),
                new Map('end_date'),
                new Map('timetable_object_id',null,true),
                new Map('ref_code',null,true),
                new Map('ref_id',null,true),
                new Map('update_by',null,true),
                new Map('last_update',null,true)
            ),
            parent::CASE_INSENSITIVE
        );
    }
    public function listdata($params){
        $data = $this->_storage->listdata($params);
        $rowCount = ($params['rowCount'] > 0) ? $params['rowCount'] : $data['dataCount']['total_count'];

        $rows       = $data['dataAll'];
        foreach($rows as $key => $row){
            $rows[$key]['day_name'] = $this->getDayName($row['day_no']);
        }

        $preparedResult = array(
            'current' => $params['page'],
            'rowCount' => $rowCount,
            'total' => $data['dataCount']['total_count'],
            'rows' => $rows,
        );

        $result = new \Neuron\Generic\Result();
        $result->data = $preparedResult;

        return $result;
    }
    public function load($id = null, $mode = Shift\Storage::LOADBY_ID, $params = array()) {
        $result     = new Result();
        if ($data = $this->_storage->load($id, $mode,$params)) {
            $result->data   = $data;
            $this->push($data);
        }else{
            $result->code = 401;
            $result->info = "Shift not found";
        }
        return $result;
    }
    public function save($update = false) {
        if ($timetable_shift_id = $this->_storage->save($this, $update)) {
            $this->timetable_shift_id = $timetable_shift_id;
            return intval($this->timetable_shift_id);
        }
        return false;
    }
    public function replace(){
         if ($timetable_shift_id = $this->_storage->replace($this)) {
            $this->timetable_shift_id = $timetable_shift_id;
            return intval($this->timetable_shift_id);
        }
        return false;
    }
    public function delete($timetable_shift_id) {
        return $this->_storage->remove($timetable_shift_id);
    }
    public function searchByShiftDay($param) {
        return $this->_storage->searchByShiftDay($param);
    }
    public function listMisc($id){
        return $this->_storage->listMisc($id);
    }
    public function saveMisc($data, $update){
        $result     = new \Neuron\Generic\Result;
        if(!$update){
            $loadMisc   = $this->_storage->loadMisc($data['timetable_shift_id'],$data['start_time'],$data['end_time']);
        }
        if($loadMisc){
            $result->code = 401;
            $result->info   = "Cannot add same shift misc with duplicate Start time or End time";
        }else{
            $value = array(
                'shift_misc_id'=> $data['shift_misc_id'],
                'timetable_shift_id'=>$data['timetable_shift_id'],
                'misc_name'=>$data['misc_name'],
                'start_time'=>$data['start_time'],
                'end_time'=>$data['end_time']
            );
            $result->data =  $this->_storage->saveMisc($value, $update);
        }

        return $result;
    }

    public function deleteMisc($id){
        return $this->_storage->deleteMisc($id);
    }
    public function listGroup($id = null){
        return $this->_storage->listGroup($id);
    }
    public function saveGroup($post, $update = false){
        $data   = array(
            'group_timeshift_id'    => $post['group_timeshift_id'],
            'group_id'              => $post['group_id'],
            'name'                  => $post['name'],
            'ref_code'              => $post['ref_code'],
            'description'           => $post['description'],
            'start_date'            => date("Y-m-d",strtotime($post['start_date'])),
        );

        if ($id = $this->_storage->saveGroup($data, $update)) {
            return $id;
        }
        return false;
    }
    public function deleteGroup($id) {
        return $this->_storage->removeGroup($id);
    }
    public function listObject($data){
        return $this->_storage->listObject($data);
    }
    public function saveObject($data, $update = false, $asMember = false){
         if ($id = $this->_storage->saveObject($data, $update, $asMember)) {
            return true;
         }
         return false;
    }
    public function deleteObject($data) {
        return $this->_storage->removeObject($data);
    }
    public function saveShiftObjectTable($data, $update = false) {
        return $this->_storage->saveShiftObjectTable($data, $update);
    }
    public function checkObjectByDate($ref_id, $timetable_id, $startDate, $endDate) {
        return $this->_storage->checkObjectByDate($ref_id, $timetable_id, $startDate, $endDate);
    }
    
    public function checkGroupByDate($group_id, $timetable_id, $startDate, $endDate) {
        return $this->_storage->checkGroupByDate($group_id, $timetable_id, $startDate, $endDate);
    }
    public function saveShiftGroupTable($data, $update = false) {
        return $this->_storage->saveShiftGroupTable($data, $update);
    }
    public function loadObjectTimetable($ref_id, $start_date, $reqtype){
        $start  = explode(" ", $start_date);
        $date   = $start[0];
        $getDayNo  =  $this->getDayNumber($date);
        if($getDayNo->code !== 0){
            return $getDayNo;
        }
        $dayno      = $getDayNo->data;

        $dataTimetable = $this->_storage->loadObjectTimetable($ref_id, date('Y-m-d', strtotime($start[0])), $dayno);
		
        
        if(!$dataTimetable){
            
            return  array(
                'success'=>false, 
                'message'=> 'does not have timetable, please mapping this ref id to timeshift first'
            );

        }else{
            if($reqtype== self::ORDER_TYPE_ATTENDANCE){
                $requestTime            = date('His',strtotime($start[1]));
                $inBeginningTime        = date('His',strtotime($dataTimetable['in_beginning_time']));
                $inEndingTime           = date('His',strtotime($dataTimetable['in_ending_time']));
                $outBeginningTime       = date('His',strtotime($dataTimetable['out_beginning_time']));
                $outEndingTime          = date('His',strtotime($dataTimetable['out_ending_time']));

                $isTimeIn           = $this->isWithinTimeRange($requestTime,$inBeginningTime,$inEndingTime);
                $isTimeOut          = $this->isWithinTimeRange($requestTime,$inEndingTime,$outEndingTime);
                

                $res        = array();
                
                if($isTimeIn || $isTimeOut){
                    if($isTimeIn){
                        $res    = array(
                            'success' => true,
                            'message' => 'can attendance in',
                            'code' => 1,
                            'data' => $dataTimetable
                        ); 
                    }else if($isTimeOut){
                        /* Check if already attended or not*/
                        $attendance         = new Attendance(Attendance\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
                        $paramsCheck        = array(
                            'ref_id'        => $ref_id,
                            'chckdate'      => date('Y-m-d',strtotime($start_date)),
                            'start_dtm'     => null,
                        );

                        $listAttd               = $attendance->listAttendance($paramsCheck);
                        $countAttd              = count($listAttd['data']);


                        if($isTimeOut && $countAttd > 0){
                            $res = array(
                                'success' => true,
                                'message' => 'can attendance out',
                                'code' => 2,
                                'data' => $dataTimetable
                            );
                        }else{
                            $res        = array(
                                'success' => false,
                                'message' => 'Sorry, your attendance time invalid'
                            );
                        }
                    }
                }else{
                    $res        = array(
                        'success' => false,
                        'message' => 'Sorry, your attendance time invalid'
                    );
                }

                return $res;


            }else if($reqtype==self::ORDER_TYPE_OVERTIME OR $reqtype==self::ORDER_TYPE_OVERTIME_WORKDAY){
                $attendIn   = array('id' => '');
                if(class_exists("\Neuron\Attendance\Entity")){
                    $entity   = new \Neuron\Attendance\Entity(\Neuron\Attendance\Entity\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
                    $entity->load(
                        null,\Neuron\Attendance\Entity\Storage::LOADBY_ID,
                        array(
                            'ref_id'        => $ref_id,
                            'start_date'    => date("Y-m-d")
                        )
                    );

                    $attendIn = $entity->pull();
                }
                				
                if($dataTimetable['is_workday'] == 1){
                    // check hari libur calendar
                    /* Cari dulu, hari ini libur ga */
                    $events             = new Events(Event\Storage::factory(
                        $this->_storage->getAdapter(),
                        $this->_storage->getConfig()
                    ));
                    $loadEvents   = $events->load(
                        0, # 0 = hari libur
                        Event\Storage::LISTBY_WORKDAY,
                        array(
                            'start_date'    => date('Y-m-d',strtotime($date)),
                            'end_date'      => date('Y-m-d',strtotime($date)),
                            'date_between'  => true
                        )
                    );
					
					// print_r($loadEvents);die();

                    if($loadEvents->code == 0){
                        // Holiday bisa request lembur jam berapa aja..
                        return  array('success'=>true, 'message'=>'can request', 'data'=>$dataTimetable,'is_holiday' => true); 
                    }else{
                        if(!is_null($attendIn['id'])){
                            if(date('H:i:s', strtotime($start[1])) <= $dataTimetable['on_duty_time'] || date('H:i:s', strtotime($start[1])) >= $dataTimetable['overtime_start_time']){
                                return  array('success'=>true, 'message'=>'can request', 'data'=>$dataTimetable); 
                            }else{
                                return  array('success'=>false, 'message'=>'Sorry, your overtime time invalid');
                            }
                        } else {
                            return array('success'  => false, 'message' => 'You have not been entrance today!');
                        }
                    }
                }else{
                    if(!is_null($attendIn['id'])){
                        return  array('success'=>true, 'message'=>'can overtime');
                    } else {
                        return array('success'  => false, 'message' => 'You have not been entrance today!');
                    }
                }
            }else{
                // Holiday bisa request lembur jam berapa aja..
                return  array('success'=>true, 'message'=>'can request', 'data'=>$dataTimetable,'is_holiday' => true); 
            }
        }
    }

    /**
     * @return true kalo duplicate / overlap
     * @return false kalo ga duplikat / overlap
     * */ 
    public function isDuplicate($ref_id, $timetable_id, $startDate, $endDate) {
        // cari timetable yang sama sudah ada atau belum di ref_id ini.
        $isExists = $this->_storage->isDuplicate($ref_id, $timetable_id, $startDate, $endDate);
        if($isExists) {
            return $isExists;
        }
        
        // cari day_no yang sama, terus bandingkan jam nya overlap atau tidak.
        # Cari day_no timetable yang mau ditambahkan
        $this->load($timetable_id,Shift\Storage::LOADBY_TIMETABLE_ID);
        $dayNo  = $this->day_no;

        # cari day_no yang sama yang telah ditambahkan ke ref_id
        $objectTimetable    = new ObjectTimetable(
            \Neuron\Attendance\Timetable\Objects\Storage::factory(
                $this->_storage->getAdapter(),
                $this->_storage->getConfig()
            )
        );
        $loadObject = $objectTimetable->load(
            $ref_id,
            \Neuron\Attendance\Timetable\Objects\Storage::LOADBY_REFID_DAY_NO,
            [
                'day_no'        => $dayNo,
                'ref_id'        => $ref_id,
                'start_date'    => $startDate,
                'end_date'      => $endDate
            ]
        );

        if($loadObject){
            // Object dengan day_no dan ref_id yang sama sudah ada. bandingkan jam nya
            # $savedShift Detail shift yang udah pernah disimpan.
            $savedShift     = $loadObject;
            # $newShift detail Shift yang mau di simpan
            $newShift       = $this->pull();
            
            # Bandingkan
            $savedShiftStart    = strtotime($savedShift['in_beginning_time']); # A
            $savedShiftEnd      = strtotime($savedShift['out_beginning_time']); # A
            $newShiftStart      = strtotime($newShift['in_beginning_time']); # B
            $newShiftEnd        = strtotime($newShift['out_beginning_time']); # B

            $isOverlap          = (
                ($savedShiftStart <= $newShiftStart && $savedShiftEnd <= $newShiftEnd) ||
                ($newShiftStart <= $savedShiftStart && $newShiftEnd <= $savedShiftEnd)
            );

            return $isOverlap;
        }
        return false;
    }
    public function isWithinTimeRange($time,$start, $end){
        $now        = $time;
        /* time frame rolls over midnight */
        if ($start > $end) {
            /* if current time is past start time or before end time */
            if ($now >= $start || $now < $end) {
                return true;
            }
        }
        /* else time frame is within same day check if we are between start and end */
        else if ($now >= $start && $now <= $end) {
            return true;
        }
        
        return false;
    }
    
    public function loadGroupTimetable($timetable_id){
        return $this->_storage->loadGroupTimetable($timetable_id);
    }
    public function checkTimetable($code){
        return $this->_storage->checkTimetable($code);
    }
    public function checkObjectMember($ref_id){
        return $this->_storage->checkObjectMember($ref_id);
    }
    public function deleteShiftGroupTable($datagt){
        return $this->_storage->deleteShiftGroupTable($datagt);
    }
    public function deleteObjectByRef($data) {
        return $this->_storage->removeObjectByRef($data);
    }
    public function loadShiftByDate($date, $dayno){
        return $this->_storage->loadShiftByDate($date, $dayno);
    }

    public function saveChangeShift($data, $update = false){
        return $this->_storage->saveChangeShift($data, $update);
    }

    public function loadChangeShift($field= 'tt_employee_shift_id', $id){
        return $this->_storage->loadChangeShift($field, $id);
    }

    public function updateShiftFromChange($data){
        return $this->_storage->updateShiftFromChange($data);
    }
    public function loadShiftByDayNo($dayNo){
        if($data = $this->_storage->loadShiftByDayNo($dayNo)){
            $this->push($data);
            return true;
        }else{
            return false;
        }
    }

    public function getDayNo($dayName){
        switch ($dayName) {
            case 'Mon':
                return self::MONDAY;
                break;
            case 'Tue':
                return self::TUESDAY;
                break;
            case 'Wed':
                return self::WEDNESDAY;
                break;
            case 'Thu':
                return self::THURSDAY;
                break;
            case 'Fri':
                return self::FRIDAY;
                break;
            case 'Sat':
                return self::SATURDAY;
                break;
            case 'Sun':
                return self::SUNDAY;
                break;
        }
    }
    public function loadShiftObject($ref_id = null, $date= null){
        return $this->_storage->loadShiftObject($ref_id, $date);
    }
    public function listActiveShift($id = null, $date= null, $dayno = null,$excludeParent = null){
        $result = new \Neuron\Generic\Result();
        $list = array();
        if($rows = $this->_storage->listActiveShift($id, $date, $dayno,$excludeParent)){
            if($rows->countTotal > 0){
                foreach ($rows->list as $row) {
                    array_push($list , $row);
                }
            }
        }

        if(count($list) > 0){
            $result->data = array(
                'list' => $list,
                'countTotal' => $rows->countTotal,
                'pager' => $rows->pager);
        }else{
            $result->code = 1;
            $result->info = 'Agents load none';
        }

        return $result;
    }
    public function loadRow($id = null, $mode = Shift\Attendance\Storage::LOADBY_ID,$params = array()){
        $result     = new Result();
        $data       = $this->_storage->loadRow($id,$mode,$params);
        if($data){
            $this->push($data);
            $result->data       = $data;
        }else{
            $result->data       = array();
        }
        return $result;
    }
    public function getDayNumber($date = null){
        $result         = new Result();
        if(!$date){
            return new Result(0,411,"Invalid date");
        }

        /* Dummy */
        $dayName    = date('D',strtotime($date));

        switch ($dayName) {
            case 'Sun':
                $result->data   = self::SUNDAY;
                break;
            case 'Mon':
                $result->data   = self::MONDAY;
                break;
            case 'Tue':
                $result->data   = self::TUESDAY;
                break;
            case 'Wed':
                $result->data   = self::WEDNESDAY;
                break;
            case 'Thu':
                $result->data   = self::THURSDAY;
                break;
            case 'Fri':
                $result->data   = self::FRIDAY;
                break;
            case 'Sat':
                $result->data   = self::SATURDAY;
                break;
        }

        return $result;



    }
    public function getDayName($dayNo){
        switch ($dayNo) {
            case self::SUNDAY    :
                return "Sunday";
                break;
            case self::MONDAY    :
                return "Monday";
                break;
            case self::TUESDAY   :
                return "Tuesday";
                break;
            case self::WEDNESDAY :
                return "Wednesday";
                break;
            case self::THURSDAY  :
                return "Thursday";
                break;
            case self::FRIDAY    :
                return "Friday";
                break;
            case self::SATURDAY  :
                return "Saturday";
                break;
        }
    }
}