<?php

namespace Neuron\Attendance\Shift\Storage;
use Zend\Db\Sql\Predicate\Expression;
use Neuron\Attendance\Shift;

class Mysql extends \Neuron\Db\Storage implements Skeleton {
    public function listdata($params = null) {
        $select = $this->select()
            ->from(array('at' => $this->_tables['attd_timetable']))
            ->join(array('ats'=> $this->_tables['attd_timetable_shift']), 'at.timetable_id=ats.timetable_id', array('*'), 'left')
            ->where(array('(ats.end_date is null or ats.end_date >= now())'))
            ->order(array('at.code','ats.day_no'));

        if($params['id']){
            $select->where(array('parent_timetable_id'=> $params['id']));
        }

        if($params['draw']){
            $select->limit(intval($params['length']));
            $select->offset(intval($params['start']));

            // order
            $column = $params['order'][0]['column'];
            $dir = $params['order'][0]['dir'];
            $columnName = $params['columns'][$column]['data'];
            if($column != 0){
                $select->reset('order');
                $select->order($columnName.' '.$dir);
            }

            // search
            if(isset($params['search']['value']) && !empty($params['search']['value'])){
                $searchValue = $params['search']['value'];
    
                $whereString = "(
                    lower(at.code) LIKE '%".strtolower($searchValue)."%' OR
                    lower(at.name) LIKE '%".strtolower($searchValue)."%' 
                )";
                $select->where($whereString);
            }

            if(isset($params['day_no']) && !empty($params['day_no'])){
                $select->where(array('ats.day_no' => $params['day_no']));
            }
        }
        // echo('$select=> '.str_replace('"','',$select->getSqlString()));die();
        $data = $this->fetchAll($select);

        $dataAll = [];

        if($data) {
            foreach($data as $row) {
                array_push($dataAll, $row);
            }
        }
        
        $selectC        = $select;
        $selectC->reset('joins');
        $selectC->join(array('ats'=> $this->_tables['attd_timetable_shift']), 'at.timetable_id=ats.timetable_id', array(), 'left');
        $selectC->reset('limit')->reset('offset');
        $selectC->columns(array(
            'total_count'   => new Expression("COUNT(*)")
        ));
        $dataCount = $this->fetchRow($selectC); 

        return array(
            'dataAll' => $dataAll,
            'dataCount' => $dataCount,
        );
    }

    public function load($id = null, $mode = Shift\Storage::LOADBY_ID,$params = array()) {
        $select = $this->select()
            ->from(array('lt' => $this->_tables['attd_timetable_shift']));
        switch($mode){
            case Shift\Storage::LOADBY_ID:
                $select->where(array(
                    $this->_('timetable_shift_id') => $id
                ));
                break;
            case Shift\Storage::LOADBY_BETWEENDATE_REF_ID_DAYNO:
                $select->join(array('ato' => $this->_tables['attd_timetable_object']),'ato.timetable_id = lt.timetable_id','*');
                $select->where('ato.start_date <= "'.$params['start_date'].'"');
                $select->where('(ato.end_date >= "'.$params['start_date'].'" OR ato.end_date IS NULL)');
                $select->where(array('ato.ref_id' => $params['ref_id'],'lt.day_no' => $params['day_no']));
                break;
            case Shift\Storage::LOADBY_BETWEENDATE:
                $select->join(array('ato' => $this->_tables['attd_timetable_object']),'ato.timetable_id = lt.timetable_id','*');
                $select->where([new Expression("(DATE(ato.start_date) BETWEEN  '".$params['start_date']."' AND '".$params['end_date']."')")]);
                
                break;
            case Shift\Storage::LOADBY_TIMETABLE_ID:
                $select->where([
                    'timetable_id'  => $id
                ]);
                break;
        }
        if(isset($params['is_workday']) && !empty($params['is_workday'])){
            $select->where(array(
                $this->_('is_workday','lt.')    => $params['is_workday']
            ));
        }

        if(isset($params['ref_code']) && !empty($params['ref_code'])){
            $select->where(array(
                $this->_('ref_code','ato.')    => $params['ref_code']
            ));
        }
        
        $result = $this->fetchRow($select);
        return $result;
    }

    public function save(\Neuron\Attendance\Shift $model, $update = false) {
        if ($update) {
            $update = $this->update()
                ->table($this->_tables['attd_timetable_shift'])
                ->set($this->__($model->pull()))
                ->where($this->__(array('timetable_shift_id' => $model->timetable_shift_id)));

            if ($this->execute($update)) {
                return $model->timetable_shift_id;
            }
        }
        else {
            $model->timetable_shift_id = 0;
            $insert = $this->insert()
                ->into($this->_tables['attd_timetable_shift'])
                ->autoincrement($this->_('timetable_shift_id'))
                ->values($this->__($model->pull()));

            $save = $this->execute($insert);
            if ($save) {
                $model->timetable_shift_id = $save;
                return $model->timetable_shift_id;
            }
        }

        return false;
    }

    public function remove($id) {
        try {
            $deleteTO = $this->update()
                    ->table($this->_tables['attd_timetable_object'])
                    ->set($this->__(array('end_date'=> $this->expression('date(now() - INTERVAL 1 DAY)'))))
                    ->where(array('timetable_id' => $id));
            if($this->execute($deleteTO)){  
                $deleteTG = $this->update()
                        ->table($this->_tables['attd_object_group_timetable'])
                        ->set($this->__(array('end_date'=> $this->expression('date(now() - INTERVAL 1 DAY)'))))
                        ->where(array('timetable_id' => $id));
                // echo str_replace('"','',$deleteTG->getSqlString()).'<br />'; die();
                if($this->execute($deleteTG)){  
                    $deleteTT = $this->update()
                            ->table($this->_tables['attd_timetable_shift'])
                            ->set($this->__(array('end_date'=> $this->expression('date(now() - INTERVAL 1 DAY)'))))
                            ->where(array('timetable_id' => $id));
                    return $this->execute($deleteTT);
                }else{
                    return false;
                }
            }else{
                return false;
            }
        }catch (\Exception $ex) {
            return false;
        }  
    }

    public function searchByShiftDay($param) {
        $select = $this->select()
            ->from($this->_tables['attd_timetable_shift'])
            ->where(array('timetable_id' => $param['timetable_id'], 'day_no' => $param['day_no']));
        // echo str_replace('"','',$select->getSqlString())."<br />"; die();
        $result = $this->fetchRow($select);
        return $result;
    }

    public function replace(\Neuron\Attendance\Shift $model) {
        try {
            $select = $this->select()
                        ->columns(array('timetable_shift_id'))
                        ->from($this->_tables['attd_timetable_shift'])
                        ->where(array('timetable_id'=>$model->timetable_id, 'day_no'=>$model->day_no));
            $id = $this->fetchValue($select);
            $model->timetable_shift_id = $id;
            
            $update = $this->update()
                ->table($this->_tables['attd_timetable_shift'])
                ->set($this->__($model->pull()))
                ->where($this->__(array('timetable_id'=>$model->timetable_id, 'day_no'=>$model->day_no)));

            if ($this->execute($update)) {
                return $id;
            }
            return false;
        } catch (Exception $e) {
            return false;
        }
        
    }

    public function loadMisc($timetable_id, $start_date, $end_date){
        $select         = $this->select();
        $select->from($this->_tables['attd_timetable_shiftmisc']);
        $select->where(array(
            'timetable_shift_id'  => $timetable_id,
        ));

        $select->where("(start_time = '".$start_date."' OR end_time = '".$end_date."' )");

        $data   = $this->fetchRow($select);
        return $data;
    }

    public function listMisc($id){
        try {
            $select = $this->select()
                        ->from($this->_tables['attd_timetable_shiftmisc'])
                        ->where(array('timetable_shift_id'=> $id));
            return $this->fetchAll($select);
        } catch (Exception $e) {
            return false;
        }
        
    }

    public function saveMisc($data, $update){
        try {
            if($update){
                $update = $this->update()
                    ->table($this->_tables['attd_timetable_shiftmisc'])
                    ->set($this->__($data))
                    ->where($this->__(array('shift_misc_id' => $data['shift_misc_id'])));
                    // echo str_replace('"','',$update->getSqlString())."<br />"; die();
                if ($this->execute($update)) {
                    return $data['shift_misc_id'];
                }
            }else{
                $data['shift_misc_id']  = 0;
                $insert = $this->insert()
                    ->into($this->_tables['attd_timetable_shiftmisc'])
                    ->values($this->__($data))
                    ->autoincrement('shift_misc_id');
                $save = $this->execute($insert);
                if ($save) {
                    return $save;
                }
            }
            return false;
        } catch (Exception $e) {
            return false;
        }
        
    }

    public function deleteMisc($id){
        try {
            $deleteSM = $this->delete()
                    ->from($this->_tables['attd_timetable_shiftmisc'])
                    ->where(array('shift_misc_id' => $id));
            return $this->execute($deleteSM);
        } catch (Exception $e) {
            return false;
        }
    }

    public function listGroup($id= null){
        try {
            $select = $this->select()
                            ->from(array('aog' => $this->_tables['attd_object_group']))
                            ->join(array('aogt'=> $this->_tables['attd_object_group_timetable']), $this->expression('aog.group_id=aogt.object_group_id AND aogt.applied_status=1'), array('*'), 'left')
                            ->where(array('aog.end_date' => null))                
                            ->order('aogt.applied_status desc, aog.name');
            if($id){
                $select->where(array('aog.group_id'=>$id));
            }
                // echo str_replace('"','',$select->getSqlString()).'<br />'; die();
            return $this->fetchAll($select);                

        } catch (\Exception $ex) {
            return false;
        }  
    }

    public function saveGroup($data, $update = false) {
        try{
            if ($update) {
                $update = $this->update()
                            ->table($this->_tables['attd_object_group'])
                            ->set($this->__(array(
                                            'name'=> $data['name'],
                                            'ref_code'=>$data['ref_code'],
                                            'description'=>$data['description'],
                                            'start_date'=>$data['start_date'],
                                            'end_date'=>NULL)))
                            ->where($this->__(array('group_id' => $data->group_id)));

                // echo str_replace('"','',$update->getSqlString()).'<br />'; die();
                if ($id = $this->execute($update)) {
                    return $id;
                }
                
            } else {

                $insert = $this->insert()
                            ->into($this->_tables['attd_object_group'])
                            ->values($this->__(array(
                                            'group_id'  => 0,
                                            'name'=> $data['name'],
                                            'ref_code'=>$data['ref_code'],
                                            'description'=>$data['description'],
                                            'start_date'=>$data['start_date'],
                                            'end_date'=>NULL)));
                // echo('$insert=> '.str_replace('"','',$insert->getSqlString()));die();
                $save = $this->execute($insert);
                if ($save) {
                    return $save;
                }
            }
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }

    public function removeGroup($id) {
        $delete = $this->update()
                        ->table($this->_tables['attd_object_group'])
                        ->set($this->__(array(
                                            'end_date'=>$this->expression('now()'))))
                        ->where(array('group_id' => $id));
                    
        return $this->execute($delete);
        }

    public function listObject($data){
        try {
            if($data['by']=='timetable'){
                $select = $this->select()
                                ->from(array('ato' => $this->_tables['attd_timetable_object']))
                                ->join(array('aogm'=> $this->_tables['attd_object_group_member']), 'ato.ref_id=aogm.ref_id', array('group_id'=>'group_id', 'group_member_id'=>'group_member_id'), 'left')
                                ->join(array('aog'=> $this->_tables['attd_object_group']), 'aogm.group_id=aog.group_id', array('group_name'=>'name'), 'left')
                                ->join(array('at'=> $this->_tables['attd_timetable']), 'ato.timetable_id=at.timetable_id', array('shift_name'=>'name'), 'left')
                                ->join(array('ats'=> $this->_tables['attd_timetable_shift']), 'at.timetable_id=ats.timetable_id', array(), 'left')
                                ->where(array('at.timetable_id'=>$data['id'], '(ato.end_date IS NULL OR ato.end_date >= CURDATE())'))
                                ->order(array('ato.ref_id'));
            }else if($data['by']=='group'){
                $select = $this->select()
                                ->from(array('aogm' => $this->_tables['attd_object_group_member']))
                                ->join(array('aog'=> $this->_tables['attd_object_group']), 'aogm.group_id=aog.group_id', array('group_name'=>'name'), 'left')
                                ->where(array('aogm.group_id'=>$data['id'], '(aogm.end_date IS NULL OR aogm.end_date >= CURDATE())'))
                                ->order(array('aogm.ref_id'));
            }else if($data['by']=='timetableObj'){
                $select = $this->select()
                                ->from(array('ato' => $this->_tables['attd_timetable_object']))
                                ->join(array('aogm'=> $this->_tables['attd_object_group_member']), 'ato.ref_id=aogm.ref_id', array('group_id'=>'group_id', 'group_member_id'=>'group_member_id'), 'left')
                                ->join(array('aog'=> $this->_tables['attd_object_group']), 'aogm.group_id=aog.group_id', array('group_name'=>'name'), 'left')
                                ->join(array('ats'=> $this->_tables['attd_timetable_shift']), 'ato.timetable_shift_id=ats.timetable_shift_id', array(), 'left')
                                ->join(array('at'=> $this->_tables['attd_timetable']), 'at.timetable_id=ats.timetable_id', array('shift_name'=>'name'), 'left')
                                ->where(array('ato.timetable_object_id'=>$data['id'], '(ato.end_date IS NULL OR ato.end_date >= CURDATE())'))
                                ->order(array('ato.ref_id'));
            }else if($data['by']=='groupObj'){
                $select = $this->select()
                                ->from(array('aogm' => $this->_tables['attd_object_group_member']))
                                ->join(array('aog'=> $this->_tables['attd_object_group']), 'aogm.group_id=aog.group_id', array('group_name'=>'name'), 'left')
                                ->where(array('aogm.group_member_id'=>$data['id'], '(aogm.end_date IS NULL OR aogm.end_date >= CURDATE())'))
                                ->order(array('aogm.ref_id'));
            }
            // echo str_replace('"','',$select->getSqlString()).'<br />'; die();
            return $this->fetchAll($select);                
        } catch (\Exception $ex) {
            // return false;
            return $e->getMessage();
        }  
    }

    public function saveObject($data, $update = false, $asMember = false) {
        try{
            if($data['end_date']==''){
                $data['end_date'] = NULL;
            }
            if ($update) {
                if($asMember){
                    $update = $this->update()
                            ->table($this->_tables['attd_object_group_member'])
                            ->set($this->__(array(
                                            'group_id'=> $data['group_id'],
                                            'ref_code'=>$data['ref_code'],
                                            'ref_id'=>$data['ref_id'],
                                            'start_date'=>$data['start_date'],
                                            'end_date'=>$data['end_date'],
                                            'last_update_dtm'=>$this->expression('now()'),
                                            'update_by'=>$data['update_by'])))
                            ->where($this->__(array('group_member_id' => $data['group_member_id'])));    
                }else{
                    $update = $this->update()
                                ->table($this->_tables['attd_timetable_object'])
                                ->set($this->__(array(
                                                'timetable_id'=> $data['timetable_id'],
                                                'ref_code'=>$data['ref_code'],
                                                'ref_id'=>$data['ref_id'],
                                                'start_date'=>$data['start_date'],
                                                'end_date'=>$data['end_date'],
                                                'last_update'=>$this->expression('now()'),
                                                'update_by'=>$data['update_by'])))
                                ->where($this->__(array('timetable_object_id' => $data['timetable_object_id'])));
                }

                // echo str_replace('"','',$update->getSqlString()).'<br />'; die();
                if ($id = $this->execute($update)) {
                    return $id;
                }
                
            } else {
                if($asMember){
                    $insert = $this->insert()
                            ->into($this->_tables['attd_object_group_member'])
                            ->values($this->__(array(
                                            'group_id'=> $data['group_id'],
                                            'ref_code'=>$data['ref_code'],
                                            'ref_id'=>$data['ref_id'],
                                            'start_date'=>$data['start_date'],
                                            'end_date'=>$data['end_date'],
                                            'last_update_dtm'=>$this->expression('now()'),
                                            'update_by'=>$data['update_by'])));    
                }else{
                    $insert = $this->insert()
                                ->into($this->_tables['attd_timetable_object'])
                                ->values($this->__(array(
                                            'timetable_id'=> $data['timetable_id'],
                                            'ref_code'=>$data['ref_code'],
                                            'ref_id'=>$data['ref_id'],
                                            'start_date'=>$data['start_date'],
                                            'end_date'=>$data['end_date'],
                                            'last_update'=>$this->expression('now()'),
                                            'update_by'=>$data['update_by'])));
                }
                // echo str_replace('"','',$insert->getSqlString()).'<br />';
                $save = $this->execute($insert);
                if ($save) {
                    return $save;
                }
            }
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }

    public function removeObject($data) {
        try{
            if($data['by']=='groupObj'){
                $update = $this->update()
                        ->table($this->_tables['attd_object_group_member'])
                        ->set($this->__(array(
                                        'end_date'=>$this->expression('now()'),
                                        'last_update_dtm'=>$this->expression('now()'),
                                        'update_by'=>$data['update_by'])))
                        ->where($this->__(array('group_member_id' => $data['id'])));    
            }else{
                $update = $this->update()
                            ->table($this->_tables['attd_timetable_object'])
                            ->set($this->__(array(
                                            'end_date'=>$this->expression('now() - INTERVAL 1 DAY'),
                                            'last_update'=>$this->expression('now()'),
                                            'update_by'=>$data['update_by'])))
                            ->where($this->__(array('timetable_object_id' => $data['id'])));
            }
            if ($id = $this->execute($update)) {
                return $id;
            }
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }

    public function removeObjectByRef($data) {
        try{
            $update = $this->update()
                        ->table($this->_tables['attd_timetable_object'])
                        ->set($this->__(array(
                                        'end_date'=>$this->expression('date(now() - INTERVAL 1 DAY)'),
                                        'last_update'=>$this->expression('now()'),
                                        'update_by'=>$data['update_by'])))
                        ->where($this->__(array('ref_id' => $data['ref_id'], 'timetable_id'=>$data['timetable_id'])));
            

            // echo str_replace('"','',$update->getSqlString()).'<br />'; die();
            if ($id = $this->execute($update)) {
                return $id;
            }
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }

    public function checkObjectByDate($ref_id, $timetable_id, $start_date, $end_date) {

        try {
            $select = $this->select()
                ->from(array('ato' => $this->_tables['attd_timetable_object']))
                ->where(
                    array(
                        'timetable_id' => $timetable_id, 
                        'ref_id' => $ref_id, 
                        'start_date >= \''.$start_date.'\'', 
                    )
                );
                
            if($end_date){
                $select->where(array('end_date' => $end_date));
            }
            
            return $this->fetchRow($select);
        } catch (Exception $e) {
            return false;
        } 
    }

    public function saveShiftObjectTable($data, $update = false) {
        if ($update == true) {
            $update = $this->update()
                    ->table($this->_tables['attd_timetable_object'])
                    ->set($this->__(array(
                                        'timetable_id'=>$data['timetable_id'],
                                        'ref_code'=>$data['ref_code'],
                                        'ref_id'=>$data['ref_id'],
                                        'start_date'=>$data['start_date'],
                                        'end_date'=>$data['end_date'],
                                        'last_update_dtm'=>$this->expression('now()'),
                                        'update_by'=>$data['update_by'])))
                    ->where($this->__(array('timetable_object_id' => $data['timetable_object_id'])));

            // die(str_replace('"','',$update->getSqlString()));
            if ($this->execute($update)) {
                return $data['object_id'];
            }
        } else {
            $insert = $this->insert()
                    ->into($this->_tables['attd_timetable_object'])
                    ->values($this->__(array(
                                            'timetable_id'=> $data['timetable_id'],
                                            'ref_code'=>$data['ref_code'],
                                            'ref_id'=>$data['ref_id'],
                                            'start_date'=>$data['start_date'],
                                            'end_date'=>$data['end_date'],
                                            'last_update'=>$this->expression('now()'),
                                            'update_by'=>$data['update_by'])));
            // die(str_replace('"','',$insert->getSqlString()));
            $save = $this->execute($insert);
            if ($save) {
                return $save;
            }
        }
    }

    public function checkGroupByDate($group_id, $timetable_id, $start_date, $end_date) {
        try {
            $select = $this->select()
                ->from(array('aogt' => $this->_tables['attd_object_group_timetable']))
                ->where(
                    array(
                        // 'timetable_id' => $timetable_id, 
                        'object_group_id' => $group_id, 
                        'timetable_id' => $timetable_id, 
                        'start_date <= \''.$start_date.'\'', 
                        'end_date >= \''.$end_date.'\''
                    )
                );

            // echo str_replace('"','',$select->getSqlString())."<br />"; die();
            return $this->fetchRow($select);
        } catch (Exception $e) {
            return false;
        }
    }

    public function saveShiftGroupTable($data, $update = false) {
        try {
            if ($update == true) {
                $update = $this->update()
                        ->table($this->_tables['attd_object_group_timetable'])
                        ->set($this->__(array(
                                            'object_group_id'=>$data['object_group_id'],
                                            'timetable_id'=>$data['timetable_id'],
                                            'start_date'=>$data['start_date'],
                                            'end_date'=>$data['end_date'],
                                            'applied_status'=>$data['applied_status'],
                                            'applied_on'=>$this->expression('now()'),
                                            'applied_by'=>$data['applied_by'])))
                        ->where($this->__(array('object_id' => $data['object_id'])));

                // die(str_replace('"','',$update->getSqlString()));
                if ($this->execute($update)) {
                    return $data['id'];
                }
            } else {
                $insert = $this->insert()
                        ->into($this->_tables['attd_object_group_timetable'])
                        ->values($this->__(array(
                                                'object_group_id'=>$data['object_group_id'],
                                                'timetable_id'=>$data['timetable_id'],
                                                'start_date'=>$data['start_date'],
                                                'end_date'=>$data['end_date'],
                                                'applied_status'=>$data['applied_status'],
                                                'applied_on'=>$this->expression('now()'),
                                                'applied_by'=>$data['applied_by'])));
                // die(str_replace('"','',$insert->getSqlString()));
                $save = $this->execute($insert);
                if ($save) {
                    return $save;
                }
            } 
        } catch (Exception $e) {
            return false;
        }
    }

    public function loadObjectTimetable($ref_id, $start_date, $dayno = null){
        try {
            $select = $this->select()
                ->columns(array('timetable_object_id', 'timetable_id','ref_code', 'ref_id', 'ato_start_date'=> 'start_date', 'ato_end_date'=>'end_date', 'last_update', 'update_by'))
                ->from(array('ato' => $this->_tables['attd_timetable_object']))
                ->join(array('at' => $this->_tables['attd_timetable']), 'ato.timetable_id=at.timetable_id', array('*'), 'left')
                ->join(array('ats' => $this->_tables['attd_timetable_shift']), 'at.timetable_id=ats.timetable_id', array('*'), 'left')
                ->where(
                    array( 
                        'ato.ref_id' => $ref_id, 
                        'ato.start_date <= \''.$start_date.'\'', 
                        '(ato.end_date is null or ato.end_date >= \''.$start_date.'\')'
                    )
                );
            if($dayno){
                $select->where(array('ats.day_no'=>$dayno));
            }
            // echo str_replace('"','',$select->getSqlString())."<br />"; die();
            
            $result = $this->fetchRow($select);
            return $result;
        } catch (\Exception $e) {
            return false;
        }
    }
    
    public function checkTimetable($code) {
        try {
            $select = $this->select()
                ->from(array('at' => $this->_tables['attd_timetable']))
                ->where(
                    array(
                        // 'timetable_id' => $timetable_id, 
                        'code' => $code
                    )
                );

            // echo str_replace('"','',$select->getSqlString())."<br />"; die();
            return $this->fetchRow($select);
        } catch (Exception $e) {
            return false;
        } 
    }

    public function checkObjectMember($ref_id){
        try {
            $select = $this->select()
                ->from(array('aogm' => $this->_tables['attd_object_group_member']))
                ->where(
                    array(
                        // 'timetable_id' => $timetable_id, 
                        'ref_id' => $ref_id
                    )
                );

            // echo str_replace('"','',$select->getSqlString())."<br />"; die();
            return $this->fetchRow($select);
        } catch (Exception $e) {
            return false;
        } 
    }

    public function loadGroupTimetable($timetable_id){
        try {
            $select = $this->select()
                ->from(array('aogt' => $this->_tables['attd_object_group_timetable']))
                ->join(array('aog' => $this->_tables['attd_object_group']), 'aogt.object_group_id=aog.group_id', array('*'), 'left')
                ->where(array('aogt.timetable_id'=> $timetable_id));
                // echo str_replace('"','',$select->getSqlString())."<br />"; die();
            return $this->fetchAll($select);
        } catch (Exception $e) {
            return false;
        }
    }

    public function deleteShiftGroupTable($data){
        try{
            $update = $this->update()
                        ->table($this->_tables['attd_object_group_timetable'])
                        ->set($this->__(array(
                                        'end_date'=>$this->expression('date(now() - INTERVAL 1 DAY)'),
                                        'applied_status'=>0,
                                        'applied_on'=>$this->expression('now()'),
                                        'applied_by'=>$data['applied_by'])))
                        ->where($this->__(array('id' => $data['id'])));
            

            // echo str_replace('"','',$update->getSqlString()).'<br />'; die();
            if ($id = $this->execute($update)) {
                return $id;
            }
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }
    
    public function loadShiftByDate($date, $dayno){
        try {
            $select = $this->select()
                            ->from(array('at'=>$this->_tables['attd_timetable']))
                            ->join(array('ats'=>$this->_tables['attd_timetable_shift']), 'at.timetable_id = ats.timetable_id', array('*'), 'left')
                            ->where(array('at.parent_timetable_id IS NOT NULL','ats.start_date <= \''.$date.'\'', '(ats.end_date is null or ats.end_date >= \''.$date.'\')', 'ats.day_no'=>$dayno ));
            // echo str_replace('"','',$select->getSqlString()).'<br />'; die();
            return $this->fetchAll($select);
        } catch (\Exception $e) {
            return false;
        }
    }

    public function loadShiftObject($ref_id, $date){
        try {
            $select = $this->select()
                            ->from(array('ato'=>$this->_tables['attd_timetable_object']))
                            ->join(array('at'=>$this->_tables['attd_timetable']), 'ato.timetable_id=at.timetable_id', array("*"), 'left')
                            ->join(array('ats'=>$this->_tables['attd_timetable_shift']), 'ato.timetable_id = ats.timetable_id', array('*'), 'left');
            if($ref_id){
                $select->where(array('ato.ref_id' => $ref_id));
            }

            if($date){
                $select->where(array('ato.start_date <= \''.$date.'\'', '(ato.end_date is null or ato.end_date >= \''.$date.'\')' ));
            }
            
            // echo str_replace('"', '', $select->getSqlString()); die();
            return $this->fetchAll($select);
        } catch (Exception $e) {
            return false;
        }
    }

    public function saveChangeShift($data, $update){
        try {
            if ($update == true) {
                $update = $this->update()
                        ->table($this->_tables['attd_timetable_object_chshift'])
                        ->set($this->__(array(
                                            'ref_code'=>$data['ref_code'],
                                            'ref_id'=>$data['ref_id'],
                                            'shift_change_date'=>$data['shift_change_date'],
                                            'default_shift_id'=>$data['default_shift_id'],
                                            'change_shift_id'=>$data['change_shift_id'],
                                            'recommendation_id'=>$data['recommendation_id'],
                                            'change_by'=>$data['change_by'])))
                        ->where($this->__(array('tt_employee_shift_id' => $data['tt_employee_shift_id'])));

                // die(str_replace('"','',$update->getSqlString()));
                if ($this->execute($update)) {
                    return $data['tt_employee_shift_id'];
                }
            } else {
                $insert = $this->insert()
                        ->into($this->_tables['attd_timetable_object_chshift'])
                        ->values($this->__(array(
                                            'ref_code'=>$data['ref_code'],
                                            'ref_id'=>$data['ref_id'],
                                            'shift_change_date'=>$data['shift_change_date'],
                                            'default_shift_id'=>$data['default_shift_id'],
                                            'change_shift_id'=>$data['change_shift_id'],
                                            'recommendation_id'=>$data['recommendation_id'],
                                            'change_by'=>$data['change_by'])));
                // die(str_replace('"','',$insert->getSqlString()));
                $save = $this->execute($insert);
                if ($save) {
                    return $save;
                }
            } 
        } catch (Exception $e) {
            return false;
        }
    }

    public function loadChangeShift($field, $id){
        try {
            $select = $this->select()
                            ->from(array('atoc'=> $this->_tables['attd_timetable_object_chshift']))
                            ->where(array($field => $id));
            return $this->fetchAll($select);
        } catch (Exception $e) {
            return false;
        }
    }

    public function updateShiftFromChange($data){
        try {
            $update = $this->update()
                    ->table($this->_tables['attd_timetable_object'])
                    ->set($this->__(array(
                                        'start_date'=>$data['start_date'],
                                        'end_date'=>$data['end_date'])))
                    ->where($this->__(array('timetable_object_id' => $data['timetable_object_id'])));

            // die(str_replace('"','',$update->getSqlString()));
            if ($this->execute($update)) {
                return $data['timetable_object_id'];
            }
        } catch (Exception $e) {
            return false;
        }
    }

    public function loadShiftByDayNo($dayNo){
        $select = $this->select();
        $select->from($this->_tables['attd_timetable_shift']);
        $select->where($this->__(array('day_no' => $dayNo)));
        try{
            return $this->fetchRow($select);
        }catch(\Exception $ex){
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
            return false;
        }
    }

    public function isDuplicate($ref_id, $timetable_id, $startDate, $endDate){
        try {
            $select = $this->select()
                ->from(array('ato' => $this->_tables['attd_timetable_object']))
                ->where(
                    array(
                        'timetable_id' => $timetable_id,
                        'ref_id' => $ref_id
                    )
                );
            
            $select->where(
                array(
                    new Expression("(end_date >= '$startDate' or end_date is NULL)")
                )
            );
            
            return $this->fetchRow($select);
        } catch (Exception $e) {
            return false;
        } 
    }
    
    public function loadList($id = null, $mode = Shift\Storage::LIST_ALL,$params = array()){
        // Base select
        $select = $this->select();
        $select->from(
            array('to' => $this->_tables['attd_timetable_shift'])
        );

        if($mode == \Neuron\Attendance\Shift\Storage::LISTBY_PAGE){
            $pager = new \Neuron\Db\Storage\Pager($params);
            $select->limit($pager->getLimit(),$pager->getOffset())->cache(0);
        }else{
            $pager = new \Neuron\Db\Storage\Pager(1,99999999);
            switch ($mode) {
                case \Neuron\Attendance\Shift\Storage::LISTBY_ID:
                    $select->where($this->__(array('ticket_agent_id' => $id)));
                    break;
                case \Neuron\Attendance\Shift\Storage::LISTBY_DAYNO:
                    $select->where($this->__(['day_no' => $id]));
                    break;
            }
        }

        try{
            if($rows = $this->fetchAll($select)){
                $result = new \stdClass();
                $result->list = $rows;
                $result->pager = $pager;
                $result->countTotal = $rows->total;
                return $result;
            }
            return false;
        }catch(\Exception $ex){
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
            return false;
        }
    }

    public function listActiveShift($id = null, $date = null, $dayno = NULL,$excludeParent = null){
        try {
            $select = $this->select();
            $select->from(
                array('ats' => $this->_tables['attd_timetable_shift'])
            );
            $select->join(array('at'=>$this->_tables['attd_timetable']), 'ats.timetable_id=at.timetable_id', array('code', 'parent_timetable_id','time_mode', 'name'), 'left');

            if($id){
                $select->where(array('timetable_shift_id'=>$id));
            }

            if($date){
                $select->where(
                    new Expression(
                        '((\''.$date.'\' between ats.start_date and ats.end_date) OR ats.start_date >= '.$date.')'
                    )
                );
            }

            if ($dayno) {
                $select->where(array('day_no'=> $dayno));
            }

            if($excludeParent){
                $select->where(
                    new Expression("at.parent_timetable_id != $excludeParent")
                );
            }

            $pager = new \Neuron\Db\Storage\Pager(1,99999999);

            if($rows = $this->fetchAll($select)){
                $result = new \stdClass();
                $result->list = $rows;
                $result->pager = $pager;
                $result->countTotal = $rows->total;
                return $result;
            }
            return false;
        } catch (\Exception $e) {
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
            return false;
        }
    }
    public function loadRow($id = null, $mode = Shift\Storage::LOADBY_ID,$params = array()){
        $select         = $this->select();
        $select->from($this->_tables['attd_timetable_shift']);

        switch ($mode) {
            case Shift\Storage::LOADBY_ID:
                $select->where($this->__(array('timetable_shift_id' => $id)));
                break;
            case Shift\Storage::LOADBY_TIMETABLE_ID:
                $select->where($this->__(array('timetable_shift_id' => $id)));
                break;
        }

        try {
            $data       = $this->fetchRow($select);
            return $data;
        } catch (Exception $e) {
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
            return false;
        }
    }
}