<?php
namespace Neuron\Attendance;

use Neuron\Generic\Entity\Map;
use Neuron\Generic\Result;
use Neuron\Attendance;
use Neuron\Attendance\Shift;
use Neuron\Fingerprint;
use Neuron\Person;
use Neuron\Person\Map as PersonMap;
use Neuron\Application\Framework\Entity as NAF;

class Test {

    const COLUMN_SOURCE     = "A";
    const COLUMN_EMP_ID     = "B";
    const COLUMN_ATTD_DATE  = "C";
    const COLUMN_OUT_DATE   = "D";
    const COLUMN_TYPE_REC   = "E";
    CONST COLUMN_REC1       = "F";
    const COLUMN_REC2       = "G";
    const COLUMN_TARGET     = "H";
    const COLUMN_COORDINATE = "I";
    const COLUMN_LOCATION   = "J";

    const SOURCE_TYPE_FP        = "fp";
    const SOURCE_TYPE_WEB       = "web";
    const SOURCE_TYPE_MOBILE    = "mobile";
    const SOURCE_TYPE_REC       = "rec";

    const REC_TYPE_ATTENDANCE   = 'att';
    const REC_TYPE_CS_PERSON    = 'csp';
    const REC_TYPE_CS_SHIFT     = "css";
    const REC_TYPE_OVERTIME    = "ot";
    protected $_db;
    protected $_config;

    public function __construct($db,$config){
        $this->_db = $db;
        $this->_config = $config;
    }

    public static function hello() {

        return 'Attendance Test!';
    }

    public function executeRow($data = array()){
        $result         = new \Neuron\Generic\Result();
        $resultData     = array();
        foreach($data as $key => $row){
            switch ($row[self::COLUMN_SOURCE]) {
                case self::SOURCE_TYPE_FP:
                    $resultData[]       = array('excelRowNumber' => $key+2,'executeResult' => $this->insertFP($row)->toArray());
                    break;
                case self::SOURCE_TYPE_WEB:
                    $resultData[]       = array('excelRowNumber' => $key+2,'executeResult' => $this->insertWeb($row)->toArray());
                    break;
                case self::SOURCE_TYPE_MOBILE :
                    $resultData[]       = array('excelRowNumber' => $key+2,'executeResult' => $this->insertMobile($row)->toArray());
                    break;   
                case self::SOURCE_TYPE_REC :
                    $resultData[]       = array('excelRowNumber' => $key+2,'executeResult' => $this->insertRecomendation($row)->toArray());
                    break;
                default :
                    $resultData[]       = array('excelRowNumber' => $key+2,'executeResult' => $this->insertWeb($row)->toArray());
                    break;
            }
        }

        $result->data   = $resultData;
        return $result;
    }

    private function insertFP($row = array()){

        $result         = new Result();
        $resultData     = [];

        if(!class_exists('Neuron\Fingerprint\FpLog')){
            $result->code   = 2;
            $result->info   = "Module fingerprint doesnt exists";
            $result->data   = $row;
            return $result;
        }

        /* Ambil Pin berdasarkan data yang di input */
        $employee_id        = $row[self::COLUMN_EMP_ID];
        $userMap            = new PersonMap(PersonMap\Storage::factory(
            $this->_db,
            $this->_config
        ));
        $userMap->load($employee_id,PersonMap\Storage::LOADBY_PERSON_ID);
        if(!$userMap->user_map_id){
            $result->info   = "User map not found";
            $result->data   = $row;
            return $result;
        }

        // Insert FP Log IN;
        if(!empty($row[self::COLUMN_ATTD_DATE])){
            $fpLog      = new Fingerprint\FpLog(
                Fingerprint\FpLog\Storage::factory(
                    $this->_db,
                    $this->_config
                )
            );
    
            $dataAttendance = array();
            $dataAttendance['pin']              = $userMap->user_id;
            $dataAttendance['log_dtm']          = date("Y-m-d H:i:s",strtotime($row[self::COLUMN_ATTD_DATE]));
            $dataAttendance['machine_id']       = 1;
            $dataAttendance['downloaded_dtm']   =  date('Y-m-d H:i:s');
            $dataAttendance['processed']        = 0;
            $dataAttendance['process_status']   = 0;

            $resultData['fp_in']                =  $fpLog->saveEx($dataAttendance);
        }

        // Insert FP Log Out;
        if(!empty($row[self::COLUMN_OUT_DATE])){
            $fpLog      = new Fingerprint\FpLog(
                Fingerprint\FpLog\Storage::factory(
                    $this->_db,
                    $this->_config
                )
            );

            $dataAttendance = array();
            $dataAttendance['pin']              = $userMap->user_id;
            $dataAttendance['log_dtm']          = date("Y-m-d H:i:s",strtotime($row[self::COLUMN_OUT_DATE]));
            $dataAttendance['machine_id']       = 1;
            $dataAttendance['downloaded_dtm']   = date('Y-m-d H:i:s');
            $dataAttendance['processed']        = 0;
            $dataAttendance['process_status']   = 0;
    
            $resultData['fp_in']                =  $fpLog->saveEx($dataAttendance);
        }

        $result->data   = array(
            'row'           => $row,
            'resultData'    => $resultData
        );


        return $result;
    }

    private function insertWeb($row = array()){
        $result         = new Result();

        $resData        = [];

        // Insert attendance in kalo ada datanya..
        if(!empty($row[self::COLUMN_ATTD_DATE])){
            $dataAttendance     = array(
                'start_dtm'     => $row[self::COLUMN_ATTD_DATE],
                'ref_id'        => $row[self::COLUMN_EMP_ID],
                'type_id'       => Entity::TYPE_ATTENDANCE,
                'source_reference' => 1,
                'start_time'    => date("H:i:s",strtotime($row[self::COLUMN_ATTD_DATE])),
                'start_description' => "[Attendance In] Generated by test. source : web",
                "ref_code"      => "EMPLOYEE",
                "source_type_code" => "WEB",
                "source_reference" => 1,
                'geocode_location' => (!empty($row[self::COLUMN_LOCATION])) ? $row[self::COLUMN_LOCATION] : null,
            );
            $entity         = new Entity(Entity\Storage::factory(
                $this->_db,
                $this->_config
            ));
            $resultAttendance   = $entity->saveEx($dataAttendance);
            $resData['attendance_in'] = $resultAttendance->toArray();
        }

        // insert attendance out kalo ada datanya.
        if(!empty($row[self::COLUMN_OUT_DATE])){
            $dataAttendance     = array(
                'start_dtm'     => $row[self::COLUMN_OUT_DATE],
                'ref_id'        => $row[self::COLUMN_EMP_ID],
                'type_id'       => Entity::TYPE_ATTENDANCE,
                'source_reference' => 1,
                'start_time'    => date("H:i:s",strtotime($row[self::COLUMN_OUT_DATE])),
                'start_description' => "[Attendance Out] Generated by test. source : web",
                "ref_code"      => "EMPLOYEE",
                "source_type_code" => "WEB",
                "source_reference" => 1,
                'geocode_location' => (!empty($row[self::COLUMN_LOCATION])) ? $row[self::COLUMN_LOCATION] : null,
            );
            $entity         = new Entity(Entity\Storage::factory(
                $this->_db,
                $this->_config
            ));
            $resultAttendance   = $entity->saveEx($dataAttendance);
            $resData['attendance_out'] = $resultAttendance->toArray();
        }

        $result->data   = array('row' => $row,'result '=> $resData);
        return $result;
    }

    private function insertMobile($row = array()){
        $result         = new Result();
        $resData        = [];

        $entity         = new Entity(Entity\Storage::factory(
            $this->_db,
            $this->_config
        ));
        $latitude               = null;
        $longitude              = null;

        if(!empty($row[self::COLUMN_COORDINATE])){
            $coordinate         = explode(",",$row[self::COLUMN_COORDINATE]);
            $latitude           = trim($coordinate[0]);
            $longitude          = trim($coordinate[1]);
        }

        if(!empty($row[self::COLUMN_ATTD_DATE])){
            $dataAttendance     = array(
                'start_dtm'     => $row[self::COLUMN_ATTD_DATE],
                'ref_id'        => $row[self::COLUMN_EMP_ID],
                'type_id'       => Entity::TYPE_ATTENDANCE,
                'source_reference' => 1,
                'start_time'    => date("H:i:s",strtotime($row[self::COLUMN_ATTD_DATE])),
                'start_description' => "[Attendance In] Generated by test. source : mobile",
                "ref_code"      => "EMPLOYEE",
                "source_type_code" => "WEB",
                "source_reference" => "MOBILE",
                'longitude'     => $longitude,
                'latitude'      => $latitude,
                'geocode_location' => (!empty($row[self::COLUMN_LOCATION])) ? $row[self::COLUMN_LOCATION] : null,
            );
    
            $resultAttendance       = $entity->saveEx($dataAttendance);
            $resData['mobile_in']   = $resultAttendance->toArray();
        }

        if(!empty($row[self::COLUMN_OUT_DATE])){
            $dataAttendance     = array(
                'start_dtm'     => $row[self::COLUMN_OUT_DATE],
                'ref_id'        => $row[self::COLUMN_EMP_ID],
                'type_id'       => Entity::TYPE_ATTENDANCE,
                'source_reference' => 1,
                'start_time'    => date("H:i:s",strtotime($row[self::COLUMN_OUT_DATE])),
                'start_description' => "[Attendance Out] Generated by test. source : mobile",
                "ref_code"      => "EMPLOYEE",
                "source_type_code" => "WEB",
                "source_reference" => "MOBILE",
                'geocode_location' => (!empty($row[self::COLUMN_LOCATION])) ? $row[self::COLUMN_LOCATION] : null,
            );

            $resultAttendance   = $entity->saveEx($dataAttendance);
            $resData['mobile_out']   = $resultAttendance->toArray();
        }

        $result->data       = array('row' => $row, 'result' => $resData);

        return $result;
    }

    private function insertRecomendation($row){
        $result         = new Result();

        $userMap                      = new Person\Map(Person\Map\Storage::factory(
            $this->_db,
            $this->_config
        ));
        $loadUserMap                  = $userMap->load(
            $row[self::COLUMN_EMP_ID],
            Person\Map\Storage::LOADBY_PERSON_ID
        );
        if(!$userMap->user_id){
            $result->code       = 1;
            $result->info       = "User map of this person not found";
            $result->data       = $row;
            return $result;
        }

        $row[self::COLUMN_EMP_ID] = $userMap->user_id;
        
        $request        = new Request(
            Request\Storage::factory(
                $this->_db,
                $this->_config
            )
        );
        /* jadikan approvalnya array */
        $approval   = array();
        if(!empty($row[self::COLUMN_REC1])){
            $approval[] = $row[self::COLUMN_REC1];
        }
        if(!empty($row[self::COLUMN_REC2])){
            $approval[] = $row[self::COLUMN_REC2];
        }

        // pilih type requestnya apa.
        if($row[self::COLUMN_TYPE_REC] == self::REC_TYPE_ATTENDANCE){
            $recomendationData  = array(
                'approval'          => $approval,
                'request_type'      => Request::REQUEST_TYPE_ATTENDANCE,
                'user_id'           => $row[self::COLUMN_EMP_ID],
                'start_dtm'         => date('Y-m-d H:i:s',strtotime($row[self::COLUMN_ATTD_DATE])),
                'ref_code'          => "EMPLOYEE",
                "request_message"   => "This [Attendance] request generated by test."
            );
            $result     = $request->saveEx($recomendationData);
        }else if($row[self::COLUMN_TYPE_REC] == self::REC_TYPE_OVERTIME){
            $recomendationData  = array(
                'approval'          => $approval,
                'request_type'      => Request::REQUEST_TYPE_OVERTIME,
                'user_id'           => $row[self::COLUMN_EMP_ID],
                'start_dtm'         => date('Y-m-d H:i:s',strtotime($row[self::COLUMN_ATTD_DATE])),
                'end_dtm'           => date("Y-m-d H:i:s",strtotime($row[self::COLUMN_OUT_DATE])),
                'ref_code'          => "EMPLOYEE",
                "request_message"   => "This [Overtime] request generated by test."
            );
            $result     = $request->saveEx($recomendationData);
            
        }else if($row[self::COLUMN_TYPE_REC] == self::REC_TYPE_CS_SHIFT){
            $recomendationData  = array(
                'approval'          => $approval,
                'request_type'      => Request::REQUEST_TYPE_CHANGE_SHIFT,
                'user_id'           => $row[self::COLUMN_EMP_ID],
                'start_dtm'         => date('Y-m-d H:i:s',strtotime($row[self::COLUMN_ATTD_DATE])),
                'ref_code'          => "EMPLOYEE",
                "request_message"   => "This [Changeshift (Target person)] request generated by test.",
                'target_id'         => $row[self::COLUMN_TARGET],
                'target'            => $row[self::COLUMN_TARGET],
            );
            $result     = $request->saveEx($recomendationData);
        }else if($row[self::COLUMN_TYPE_REC] == self::REC_TYPE_CS_PERSON){
            $ref_id                 = $row[self::COLUMN_TARGET];

            $shift  = new Shift(Shift\Storage::factory(
                        $this->_db,
                        $this->_config
                    )
                );

            $data['start_date']     = date('Y-m-d',strtotime($row[self::COLUMN_ATTD_DATE]));
            $data['start_dtm']      = date('Y-m-d H:i:s',strtotime($row[self::COLUMN_ATTD_DATE]));
            $data['day_no']         = $shift->getDayNumber($data['start_date'])->data;
            $data['ref_id']         = $ref_id;

            // Check apakah user memiliki shift yang aktif di tanggal yang di request
            $loadShift              = $shift->load(null,Shift\Storage::LOADBY_BETWEENDATE_REF_ID_DAYNO,$data);
            if($loadShift->code != 0){
                $result->code       = 1;
                $result->info       = "Shift not found";
                $result->data       = $data;
                return $result;
            }
            $dataShift              = $loadShift->data;

            $recomendationData  = array(
                'approval'          => $approval,
                'request_type'      => Request::REQUEST_TYPE_CHANGE_SHIFT,
                'user_id'           => $row[self::COLUMN_EMP_ID],
                'start_dtm'         => date('Y-m-d H:i:s',strtotime($row[self::COLUMN_ATTD_DATE])),
                'ref_code'          => "EMPLOYEE",
                "request_message"   => "This [Changeshift Person] request generated by test.",
                'target_id'            => $dataShift['timetable_id'],
                'target'            => $dataShift['timetable_id'],
            );
            $result     = $request->saveEx($recomendationData);
        }
        
        return $result;
    }

    public function clearData(){
        $result         = new Result();

        $naf            = NAF::get(); 
        $query          = [
            "DELETE FROM attd_timetable_object_chshift",
            "DELETE FROM attd_details",
            "DELETE FROM attd_request_approval",
            "DELETE FROM attd_request",
            "DELETE FROM attd_evidence",
            "DELETE FROM attendance",
            "DELETE FROM fp_log_data"
        ];
        $queryResult = [];

        foreach($query as $sql){
            $queryResult[] = (bool)$naf->database()->execute($sql);
        }
        $result->data   = $queryResult;

        return $result;
    }

}