<?php
namespace Neuron\Attendance;

use Neuron\Generic\Entity\Map;
use Neuron\Generic\Result;
use Neuron\Attendance\Timetable as TimetableModel;

use Neuron\Attendance\Timetable\Storage;

use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\IOFactory;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;
use PhpOffice\PhpSpreadsheet\Cell\DataValidation;
use PhpOffice\PhpSpreadsheet\Worksheet\Worksheet;
use PhpOffice\PhpSpreadsheet\Cell\Coordinate;
use PhpOffice\PhpSpreadsheet\Style\NumberFormat;
use PhpOffice\PhpSpreadsheet\Cell\DataType;
use PhpOffice\PhpSpreadsheet\Worksheet\Drawing;


class Timetable extends \Neuron\Generic\Entity\Mapper {
    CONST ERROR_PREFIX = 'ATTD-M-TIMETABLE-';
    protected $_storage;
    protected $_loaded;

    public function __construct(Storage\Skeleton $storage, $data = array()) {

        $this->_storage = $storage;
        
        parent::__construct(
            $data, 
            array(
                new Map('timetable_id'),
                new Map('code'),
                new Map('parent_timetable_id'),
                new Map('time_mode'),
                new Map('name'),
                new Map('description'),
                new Map('day_no',null,false),
            ),
            parent::CASE_INSENSITIVE
        );
    }

    public function listdata($params){
        $data = $this->_storage->listdata($params);
        if(isset($params['draw'])){
            // returnnya bentuk datatable
            $result = array(
                'draw' => $params['draw'],
                'recordsTotal' => $data['dataCount']['total_count'],
                'recordsFiltered' => $data['dataCount']['total_count'],
                'data' => $data['dataAll']
            );
        }else{
            // return bentuk API
            $result = new \Neuron\Generic\Result;
            $result->data = $data;
        }
        return $result;
    }
    public function load($id = null, $mode = Timetable\Storage::LOADBY_ID, $params = array()) {
        $result             = new Result();

        if ($data = $this->_storage->load($id,$mode,$params)) {
            $this->push($data);
            $this->_loaded  = TRUE;
            $result->data   = $data;
            $result->info   = "Success loading data";
        }else{
            $this->_loaded  = FALSE;
            $result->data   = array();
            $result->info   = "no data loaded";
        }

        return $result;
    }
    public function save($update = false) {
        if ($timetable_id = $this->_storage->save($this, $update)) {
            $this->timetable_id = $timetable_id;
            return intval($this->timetable_id);
        }
        return false;
    }
    public function delete($timetable_id) {
        return $this->_storage->removeTimetable($timetable_id);
    }
    public function searchByCode($code) {
        return $this->_storage->searchByCode($code);
    }
    public function loadTimetable($timetable_id){
        return $this->_storage->loadTimetable($timetable_id);
    }

    public function loadChild($parent_id){
        $data = $this->_storage->loadChild($parent_id);
        $dataAll = array();
        if($data){
            foreach ($data as $data) {
                $dataAll[] = $data;
            }
        }
        return $dataAll;
    }

    public function saveEx($data = array(), $db){
        $result     = new Result();

        //PARAMS
        $timetableData  = $data['timetable'];
        $shiftData  = $data['shift'];
        $breaks     = $data['breakTimes'];

        $this->push($timetableData); // PUSH TIMETABLE

        if($this->timetable_id){
            $update     = TRUE;
        }else{
            $update     = FALSE;
        }

        // Validating duplicate code. only when creating new row
        if(!$update){
            $data = $this->load($this->code,TimetableModel\Storage::LOADBY_CODE);
            if($this->timetable_id){
                $result->code   = self::ERROR_PREFIX.'1';
                $result->info   = "Duplicate timetable code";
                return $result;
            }
        }

        try {
            $connection = $this->_storage->getAdapter()->getDriver()->getConnection()->beginTransaction(); 
            //Save TimeTable
            $saveTimetable = $this->save($update);
            $shiftData['parent_timetable_id']   = $saveTimetable;
            
            //SAVE SHIFT
            $shiftData['start_date'] = date('Y-m-d', strtotime($shiftData['start_date']));
            if(strlen($shiftData['end_date']) <= 0){
                $shiftData['end_date'] = null;
            }else{
                $shiftData['end_date'] = date('Y-m-d', strtotime($shiftData['end_date']));
            }

            $timetable = new \Neuron\Attendance\Timetable(\Neuron\Attendance\Timetable\Storage::factory(
                $this->_storage->getAdapter(),
                $this->_storage->getConfig()
            ));

            $shift = new \Neuron\Attendance\Shift(\Neuron\Attendance\Shift\Storage::factory(
                $this->_storage->getAdapter(),
                $this->_storage->getConfig()
            ));

            $dataDuplicate = array();

            $dayNo = $shiftData['day_no'];

            foreach ($dayNo as $dayNo) {
                //Save TimeTable
                $timetable_code = $timetableData['code'].'_'.$dayNo;

                $dataTimetable = array(
                    'timetable_id'          => null,
                    'code'                  => $timetable_code,
                    'parent_timetable_id'   => $saveTimetable,
                    'time_mode'             => 'WEEKLY',
                    'name'                  => $shiftData['name'],
                );
                $timetable->clear();
                $timetable->push($dataTimetable);

                $timetable_id = $timetable->save($update);

                $shiftData['day_no']                = $dayNo;
                $shiftData['timetable_id']          = $timetable_id;
                $shift->clear();
                $shift->push($shiftData);
                
                $saveShift = $shift->save(false);

                //Save Breaks
                if($breaks){
                    foreach ($breaks as $key => $break) {
                        if($break['break_start_time'] && $break['break_end_time'] && $break['misc_name']){
                            $dataBreaks = [
                                'start_time'            => $break['break_start_time'],
                                'end_time'              => $break['break_end_time'],
                                'misc_name'             => $break['misc_name'],
                                'shift_misc_id'         => null,
                                'timetable_shift_id'    => $saveShift
                            ];
                            
                            $shift->saveMisc($dataBreaks, false);
                        }
                    }
                }
            }
            
            $connection->commit();
            $result->info       = "Success creating new timetable!";
        } catch (\Exception $e) {
            if ($connection instanceof \Zend\Db\Adapter\Driver\ConnectionInterface) {
                $connection->rollback();
            }
            
            /* Other error handling */
            $result->code       = self::ERROR_PREFIX.'2';
            $result->info       = "Failed saving timetable!";
        }

        return $result;
    }

    public function updateEx($data = array(), $db){
        $result     = new Result();
        $this->push($data);
        if($this->timetable_id){
            $update     = TRUE;
        }else{
            $update     = FALSE;
        }

        // Validating duplicate code. only when creating new row
        if(!$update){
            $data = $this->load($this->code,TimetableModel\Storage::LOADBY_CODE);
            if($this->timetable_id){
                $result->code   = self::ERROR_PREFIX.'1';
                $result->info   = "Duplicate timetable code";
                return $result;
            }
        }

        try {
            $connection = $db->getDriver()->getConnection()->beginTransaction(); 
            //Save TimeTable
            $timetable = $this->save($update);

            $connection->commit();
            $result->info       = "Success updating new timetable!";
        } catch (\Exception $e) {
            if ($connection instanceof \Zend\Db\Adapter\Driver\ConnectionInterface) {
                $connection->rollback();
            }
            
            /* Other error handling */
            $result->code       = self::ERROR_PREFIX.'2';
            $result->info       = "Failed saving timetable!";
        }

        return $result;
    }

    public function downloadFormat($data = array())
    {

        $hari = array(
            'Mon' => 'Sen',
            'Tue' => 'Sel',
            'Wed' => 'Rab',
            'Thu' => 'Kam',
            'Fri' => 'Jum',
            'Sat' => 'Sab',
            'Sun' => 'Min'
        );
        $spreadsheet = new Spreadsheet();
        $worksheet = new Worksheet();
        $spreadsheet->disconnectWorksheets();
        $spreadsheet->addSheet($worksheet, 0);

        $begin = new \DateTime($data['start_date']);
        $end = new \DateTime($data['end_date']);
        $difference = $begin->diff($end);
        $end->setTime(0,0,1);     // new line
        
        $interval = \DateInterval::createFromDateString('1 day');
        $period = new \DatePeriod($begin, $interval, $end);
        $kolom = Coordinate::stringFromColumnIndex($difference->days+3); //get kulom by index

        
        
        $worksheet->mergeCells('A1:A3');
        $worksheet->mergeCells('B1:B3');
        $worksheet->mergeCells('C1:C3');
        $worksheet->mergeCells('D1:'.$kolom.'1');

        $worksheet->setCellValueByColumnAndRow(1, 1, "NO");
        $worksheet->setCellValueByColumnAndRow(2, 1, "NIK");
        $worksheet->setCellValueByColumnAndRow(3, 1, "NAMA");
        $worksheet->setCellValueByColumnAndRow(4, 1, 'Upload '.$difference->days .' days');

        $worksheet->getStyle('A1:A3')->getAlignment()->setHorizontal('center');
        $worksheet->getStyle('A1:A3')->getAlignment()->setVertical('center');
        $worksheet->getStyle('B1:B3')->getAlignment()->setHorizontal('center');
        $worksheet->getStyle('B1:B3')->getAlignment()->setVertical('center');
        $worksheet->getStyle('C1:C3')->getAlignment()->setHorizontal('center');
        $worksheet->getStyle('C1:C3')->getAlignment()->setVertical('center');

        $worksheet->getStyle('D1:'.$kolom.'1')->getAlignment()->setHorizontal('center');

        //
        // $worksheet->getColumnDimensionByColumn(1)->setAutoSize(true);
        // $worksheet->getColumnDimensionByColumn(2)->setAutoSize(true);
        // $worksheet->getColumnDimensionByColumn(3)->setAutoSize(true);
        // die('here');
        $worksheet->getStyle('A1:'.$kolom.'3')->getAlignment()->setWrapText(true);
        $totalColumn    = 3;
        foreach ($period as $idx => $date) {
            $totalColumn++;
            $col = Coordinate::stringFromColumnIndex($idx + 4); //get kulom by index
            $worksheet->getColumnDimensionByColumn($idx + 4)->setAutoSize(true);
            $worksheet->getStyle($col.'3')->getAlignment()->setHorizontal('center');
            $worksheet->getStyle($col.'2')->getAlignment()->setHorizontal('center');
            $worksheet->setCellValueByColumnAndRow($idx + 4, 2, $date->format('D'));
            $worksheet->setCellValueByColumnAndRow($idx + 4, 3, $date->format('d-M'));
        }     
        
        $timetables         = json_decode($data['timetables']);
        $timetableFormula   = '"'.implode(', ',$timetables).'"';
        $maxColumn          = Coordinate::stringFromColumnIndex($totalColumn); //get kulom by index
        
        // loop row excelnya
        for($i = 4; $i <= 100; $i++){
            $startColumn = "D";
            while($startColumn != $maxColumn){
                $validation = $worksheet->getCell($startColumn.$i)->getDataValidation();
                $validation->setType(\PhpOffice\PhpSpreadsheet\Cell\DataValidation::TYPE_LIST);
                $validation->setFormula1($timetableFormula);
                $validation->setAllowBlank(false);
                $validation->setShowDropDown(true);
                $startColumn++;
            }
        }

        $writer = IOFactory::createWriter($spreadsheet, "Xlsx"); //Xls is also possible
        $filename = 'Format shift-' . $data['start_date'] .' - '. $data['end_date']. '.xlsx';

        header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
        header('Content-Disposition: attachment; filename="' . $filename);
        header('Cache-Control: max-age=0');
        ob_end_clean();
        $writer->save('php://output', 'xlsx');
        exit();
    }
}