<?php
namespace Neuron\Attendance\Timetable;

use Neuron\Generic\Result;
use Neuron\Generic\Entity\Mapper;
use Neuron\Generic\Entity\Map;

class Changeshift Extends Mapper{
    protected $_storage;
    protected $_loaded;
    const UPDATE_BY     = 1000000;

    public function __construct(Changeshift\Storage\Skeleton $storage){
        $this->_storage = $storage;
        parent::__construct(
            array(),
            array(
                new Map('tt_employee_shift_id','id'),
                new Map('ref_code'),
                new Map('ref_id'),
                new Map('shift_change_date'),
                new Map('default_shift_id'),
                new Map('change_shift_id'),
                new Map('recommendation_id'),
                new Map('change_by'),
            ),parent::CASE_INSENSITIVE
        );
    }
    
    public function save($update = false){
        if($id = $this->_storage->save($this,$update)){
            return $id;
        }else{
            return false;
        }
        return false;
    }

    public function saveEx($data){
        $result = new Result();

        /* Load untuk mencegah duplikat pada tanggal yang sama. */
        $this->load(null,Changeshift\Storage::LOADBY_REFID_CHANGEDATE,$data);
        if($this->_loaded){
            return new Result(0,406,'Duplicate changeshift.');
        }

        if($data['change_shift_id'] === $data['default_shift_id']){
            $result->code = 200;
            $result->info = 'Tidak dapat berpindah ke shift yang sama.';
        }

        if(empty($data['change_shift_id']) || !isset($data['change_shift_id'])){
            $result->code = 502;
            $result->info = 'Changeshift required';
            return $result;

            exit();
        }

        if(isset($data['tt_employee_shift_id']) && !empty($data['tt_employee_shift_id']) ){
            $update = true;
            $this->load($data['tt_employee_shift_id'],Object\Storage::LOADBY_ID);
        }else{
            if($data['change_by']){
                $changeBy   = $data['change_by'];
            }else{
                $changeBy   = self::UPDATE_BY;
            }
            $update = false;
            $this->id                   = 0;
            $this->ref_code             = $data['ref_code'];    
            $this->ref_id               = $data['ref_id'];
            $this->shift_change_date    = date('Y-m-d',strtotime($data['shift_change_date']));
            $this->default_shift_id     = $data['default_shift_id'];
            $this->change_shift_id      = $data['change_shift_id'];
            $this->recommendation_id    = isset($data['recommendation_id']) ? $data['recommendation_id'] : null;
            $this->change_by            = $changeBy;
        }

        $param = $this->pull();
        
        if($id = $this->save($update)){
            $result->data = $id;
        }else{
            $result->code = 1;
            $result->info = 'Failed Saving Changeshift';
        }
        return $result;
    }
    
    public function load($id = null, $mode = Changeshift\Storage::LOADBY_ID,$params = array()){
        if($data = $this->_storage->load($id,$mode,$params)){
            $this->_loaded = true;
            $this->push($data);
        }else{
            $this->_loaded = false;
        }
        return false;
    }

    public function loadEx($id = null, $mode = Changeshift\Storage::LOADBY_ID,$params = array()){
        $result = new Result();
        $this->load($id,$mode,$params);
        if($this->_loaded){
            $result->data = $this->pull();
        }else{
            $result->code = 1;
            $result->info = 'Failed Loading data';
        }
        return $result;
    }

    public function delete($data,$mode = Changeshift\Storage::DELETEBY_ID){
        $result = new Result();
        $id = @$data['tt_employee_shift_id'];

        if($id = $this->_storage->deleteRow($id,$mode)){
            $result->data = $id;
        }else{
            $result->code = 1;
            $result->info = 'Failed deleting Changeshift';
        }
        return $result;
    }
}