<?php
namespace Neuron\Attendance\Timetable\Group\Member\Storage;

use Neuron\Attendance\Timetable\Group;
use Zend\Db\Sql\Where;
use Zend\Db\Sql\Expression;

class Mysql extends \Neuron\Db\Storage implements Skeleton {
    
    public function loadList($id = null, $mode = Group\Member\Storage::LIST_ALL,$params = array()){
        // Base select
        $select = $this->select();
        $select->from(
            array('ta' => $this->_tables['attd_object_group_member'])
        );


        if(class_exists('\Neuron\Employee\Entity')){
            $select->join(array('emp' => $this->_tables['person']),'emp.id = ta.ref_id',array('ref_name' => new Expression("CONCAT(first_name,' ',last_name)")));
        }


        if($mode != Group\Member\Storage::LIST_ALL){
            switch ($mode) {
                case Group\Member\Storage::LISTBY_ID:
                    $select->where($this->__(array('ticket_group_id' => $id)));
                    break;
                case Group\Member\Storage::LISTBY_GROUPID:
                    $select->where($this->__(array('group_id' => $id)));
                    break;
            }
        }

        if(isset($params['draw'])){
            $select->limit(intval($params['length']));
            $select->offset(intval($params['start']));

            if(strlen($params['search']['value']) > 0){
                $nest   = $select->where->nest();

                if (class_exists('\Neuron\Employee\Entity')) {
                    $nest->like(    new Expression("CONCAT(first_name,' ',last_name)") , new Expression('\'%' . $params['search']['value'] . '%\'') )->or;
                }

                $nest->like(    new Expression("ta.group_member_id") , new Expression('\'%' . $params['search']['value'] . '%\'') )->or;
                $nest->like(    new Expression("ta.ref_id") , new Expression('\'%' . $params['search']['value'] . '%\'') )->or;
                $nest->like(    new Expression("ta.start_date") , new Expression('\'%' . $params['search']['value'] . '%\'') )->or;

                $nest->unnest();
            }

            $columnName     = $params['columns'][$params['order'][0]['column']]['data'];
            if($columnName == 'ref_name'){
                $columnName = 'ref_id';
            }
            $dir            = $params['order'][0]['dir'];

            $select->order($columnName.' '.$dir);

        }



        try{
            if($rows = $this->fetchAll($select)){
                $result = new \stdClass();

                $result->list = $rows;

                $selectC    = $select;
                $selectC->reset('limit');
                $selectC->reset('offset');
                $selectC->reset('joins');
                if(class_exists('\Neuron\Employee\Entity')){
                    $select->join(array('emp' => $this->_tables['person']),'emp.id = ta.ref_id', array(), 'left');
                }
                $selectC->columns(array('countTotal' => new \Zend\Db\Sql\Expression("COUNT(*)")));
                
                $countTotal     = $this->fetchRow($selectC)['countTotal'];

                $result->countTotal = $countTotal;

                return $result;
            }

            return false;
        }catch(\Exception $ex){
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
            return false;
        }
    }
    public function save(Group\Member $member,$update = false){
        try{
            if($update){
                $update = $this->update();
                $update->table($this->_tables['attd_object_group_member']);
                $update->where($this->__(array('group_member_id' => $member->id)));
                $update->set($this->__($member->pull()));
                
                if($this->execute($update)){
                    return $member->id;
                }
            }else{
                $insert = $this->insert();
                $insert->into($this->_tables['attd_object_group_member']);
                $insert->autoincrement($this->__('group_member_id'));
                $insert->values($this->__($member->pull()));
                // echo('$insert=> '.str_replace('"','',$insert->getSqlString()));die();
                if($id = $this->execute($insert)){
                    return $id;
                }
            }
        }catch(\Exception $ex){
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine().' : '.$ex->getMessage());
            return false;
        }
    }
    public function load($id = null, $mode = Group\Storage::LOADBY_ID , $params = array()){
        $select = $this->select();
        $select->from($this->_tables['attd_object_group_member']);

        switch ($mode) {
            case Group\Storage::LOADBY_ID:
                $select->where($this->__(array('ticket_group_id' => $id)));
                break;
            case Group\Member\Storage::LOADBY_BETWEENDATE:
                // $select->where->greaterThanOrEqualTo('start_date',$params['start_date']);
                // $select->where->lessThanOrEqualTo('end_date',$params['end_date']);
                $select->where($this->__(array(
                    'start_date >=\'' . $params['start_date'].' \' ',
                    'end_date <=\'' . $params['end_date'].' \' ',
                )));

                $select->where($this->__(array('ref_id' => $params['ref_id'])));
                break;
            case Group\Member\Storage::LOADBY_GROUP_REFID:
                $select->where($this->__(array('ref_id' => $params['ref_id'])));
                $select->where($this->__(array('group_id' => $params['group_id'])));
                break;
        }

        try{
            // echo('$select=> '.str_replace('"','',$select->getSqlString()));die();
            $data = $this->fetchRow($select);
            return $data;
        }catch(\Exception $ex){
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
            return false;
        }
    }
    public function deleteRow($id = null ,$mode = Group\Storage::DELETEBY_ID,$params){
        $delete = $this->delete();
        $delete->from($this->_tables['attd_object_group_member']);
        switch ($mode) {
            case Group\Member\Storage::DELETEBY_ID:
                $delete->where($this->__(array('group_member_id' => $id)));
                break;
            case Group\Member\Storage::DELETEBY_GROUP_REFID:
                $delete->where($this->__(array('group_id' => $params['group_id'])));
                $delete->where($this->__(array('ref_id' => $params['ref_id'])));
                break;
        }
        try{
            // echo('$delete=> '.str_replace('"','',$delete->getSqlString()));die();
            if($this->execute($delete)){
                return true;
            }
        }catch(\Exception $ex){
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
            return false;
        }
    }
    public function loadGroupByParam($params) {

        $select = $this->select();
        $select->from(['ta' => $this->_tables['attd_object_group_member']]);
        $select->join(['ts' => $this->_tables['attd_timetable_shift']], 'ta.timetable_id= ts.timetable_id', array('*'));
        $select->join(['tt' => $this->_tables['attd_timetable']], 'ta.timetable_id= tt.timetable_id', array('*'));

        if(isset($params)) {
            foreach($params as $key => $val) {
                 $select->where(array($key => $val));
            }
        }

        // echo str_replace('"','',$select->getSqlString())."<br />"; exit;

        $result = $this->fetchRow($select);
        return $result;
    }
    

}