<?php
namespace Neuron\Attendance\Timetable\Group\Storage;

use Neuron\Attendance\Timetable\Group;
use Zend\Db\Sql\Where;

class Mysql extends \Neuron\Db\Storage implements Skeleton {
    
    public function loadList($id = null, $mode = Group\Storage::LIST_ALL,$params = array()){
        // Base select
        $select = $this->select();
        $select->from(
            array('ta' => $this->_tables['attd_object_group'])
        );
        if($mode == Group\Storage::LISTBY_PAGE){
            $pager = new \Neuron\Db\Storage\Pager($params);
            $select->limit($pager->getLimit(),$pager->getOffset())->cache(0);

        }else{
            $pager = new \Neuron\Db\Storage\Pager(1,99999999);
            if(isset($params['page']) && isset($params['records'])){
                $pager = new \Neuron\Db\Storage\Pager($params);
                $select->limit($pager->getLimit(),$pager->getOffset())->cache(0);
            }
            switch ($mode) {
                case Group\Storage::LISTBY_ID:
                    $select->where($this->__(array('ticket_group_id' => $id)));
                    break;
                case Group\Storage::LISTBY_DATE:
                    $select->where(
                        array(
                            'start_date >= "'.$params['date'].'"',
                            'end_date <= "'.$params['date'].'"'
                        )
                    );
                    break;
                case Group\Storage::LISTBY_PARENTID:
                    $select->where($this->__(array('parent_timetable_id' => $id)));
                    break;
                case Group\Storage::LISTBY_VALIDDATE:
                    $select->where($this->__(array(new \Zend\Db\Sql\Predicate\IsNull('end_date'))));
                    break;
            }
        }
        
        // echo str_replace('"','',$select->getSqlString())."<br />"; exit;

        try{
            // echo('$select=> '.str_replace('"','',$select->getSqlString()));die();
            if($rows = $this->fetchAll($select)){
                $result = new \stdClass();
                $result->list = $rows;
                $result->pager = $pager;
                $result->countTotal = $rows->total;
                return $result;
            }

            return false;
        }catch(\Exception $ex){
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
            return false;
        }
    }
    public function save(Group $group,$update = false){

        try{
            if($update){
                $update = $this->update();
                $update->table($this->_tables['attd_object_group']);
                $update->where($this->__(array('timetable_group_id' => $group->id)));
                $update->set($this->__($group->pull()));
                if($this->execute($update)){
                    return $group->id;
                }
            }else{
                $insert = $this->insert();
                $insert->into($this->_tables['attd_object_group']);
                $insert->autoincrement($this->__('timetable_group_id'));
                $insert->values($this->__($group->pull()));
                // echo('$insert=> '.str_replace('"','',$insert->getSqlString()));die();
                if($id = $this->execute($insert)){
                    return $id;
                }
            }
        }catch(\Exception $ex){
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine().' : '.$ex->getMessage());
            return false;
        }
    }
    public function load($id = null, $mode = Group\Storage::LOADBY_ID , $params = array()){
        $select = $this->select();
        $select->from($this->_tables['attd_object_group']);

        switch ($mode) {
            case Group\Storage::LOADBY_ID:
                $select->where($this->__(array('ticket_group_id' => $id)));
                break;
            case Group\Storage::LOADBY_BETWEENDATE:
                // $select->where->greaterThanOrEqualTo('start_date',$params['start_date']);
                // $select->where->lessThanOrEqualTo('end_date',$params['end_date']);
                $select->where($this->__(array(
                    'start_date >=\'' . $params['start_date'].' \' ',
                    'end_date <=\'' . $params['end_date'].' \' ',
                )));

                $select->where($this->__(array('ref_id' => $params['ref_id'])));
                break;
            case Group\Storage::LOADBY_NAME:
                $select->where(array('name' => $id));
                break;
        }
        
        try{
            $data = $this->fetchRow($select);
            return $data;
        }catch(\Exception $ex){
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
            return false;
        }
    }
    public function deleteRow($id = null ,$mode = Group\Storage::DELETEBY_ID){
        $delete = $this->delete();
        $delete->from($this->_tables['attd_object_group']);
        switch ($mode) {
            case Group\Storage::DELETEBY_ID:
                $delete->where($this->__(array('timetable_group_id' => $id)));
                break;
        }
        try{
            if($this->execute($delete)){
                return $id;
            }
        }catch(\Exception $ex){
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
            return false;
        }
    }
    public function loadGroupByParam($params) {

        $select = $this->select();
        $select->from(['ta' => $this->_tables['attd_object_group']]);
        $select->join(['ts' => $this->_tables['attd_timetable_shift']], 'ta.timetable_id= ts.timetable_id', array('*'));
        $select->join(['tt' => $this->_tables['attd_timetable']], 'ta.timetable_id= tt.timetable_id', array('*'));

        if(isset($params)) {
            foreach($params as $key => $val) {
                 $select->where(array($key => $val));
            }
        }

        // echo str_replace('"','',$select->getSqlString())."<br />"; exit;

        $result = $this->fetchRow($select);
        return $result;
    }
    public function saveMemberToGroup($params){
        $insert = $this->insert();
        $insert->into($this->_tables['attd_object_group_member']);
        $insert->autoincrement($this->__('group_member_id'));
        $insert->values('');
        echo('$insert=> '.str_replace('"','',$insert->getSqlString()));die();
        $this->execute($insert);
    }
    public function loadDataTable($post){
        $select = $this->select();
        $select->from($this->_tables{'attd_object_group'});
        $select->limit(intval($post['length']));
        $select->offset(intval($post['start']));
        $select->where('(end_date <= NOW() OR end_date IS NULL)');

        // ordering
            $column = $post['columns'][$post['order'][0]['column']]['data'];
            $dir = $post['order'][0]['dir'];
            $select->order($column.' '.$dir);
        // search
            if($post['search']['value']){
                $select->where(' 
                    (
                        lower(group_id) like \'%'.strtolower($post['search']['value']).'%\' OR
                        lower(name) like \'%'.strtolower($post['search']['value']).'%\' OR
                        lower(description) like \'%'.strtolower($post['search']['value']).'%\'
                    )
                ');
            }
        // echo('$select=> '.str_replace('"','',$select->getSqlString()));die();
        $rows = $this->fetchAll($select);
        $countTotal     = $select;
        $countTotal->reset('limit')->reset('offset');
        $countTotal->columns(array('countTotal' => new \Zend\Db\Sql\Expression("COUNT(*)")));
        $countRows = $this->fetchRow($countTotal)['countTotal'];

        $result = array(
            'rows' => $rows,
            'countRows' => $countRows
        );
        return $result;
    }
    public function getTotalRows(){
        $select = $this->select();
        $select->columns(array('total_count' => new \Zend\Db\Sql\Expression('COUNT(*)')));
        $select->from($this->_tables['attd_object_group']);
        $totalRows = $this->fetchRow($select);
        return $totalRows['total_count'];
    }
    

}