<?php
namespace Neuron\Attendance\Timetable\Group;

use Neuron\Generic\Result;
use Neuron\Generic\Entity\Mapper;
use Neuron\Generic\Entity\Map;

class Timetable Extends Mapper{
    protected $_storage;
    protected $_loaded;
    protected $ref_code = 'Employee';
    protected $_user_id;

    public function __construct(Timetable\Storage\Skeleton $storage){
        $this->_storage = $storage;
        parent::__construct(
            array(),
            array(
                new Map('id'),
                new Map('object_group_id'),
                new Map('timetable_id'),
                new Map('start_date'),
                new Map('end_date'),
                new Map('applied_status'),
                new Map('applied_on'),
                new Map('applied_by')
            ),parent::CASE_INSENSITIVE
        );
    }  
    public function save($update = false){
        if($id = $this->_storage->save($this,$update)){
            return $id;
        }else{
            return false;
        }
        return false;
    }
    public function saveEx($data){
        $result = new Result();

        if(isset($data['timetable_timetable_id']) && !empty($data['timetable_timetable_id']) ){
            $update = true;
            $this->load($data['timetable_timetable_id'],Timetable\Storage::LOADBY_ID);
        }else{
            $update = false;
        }
        $param = $this->pull();
        foreach ($data as $key => $value) {
            if(array_key_exists($key, $param)){
                $param[$key] = $value;
            }
        }
        $this->push($param);
        if($id = $this->save($update)){
            $result->data = $id;
        }else{
            $result->code = 1;
            $result->info = 'Failed Saving Timetable';
        }
        return $result;
    }
    public function load($id = null, $mode = Timetable\Storage::LOADBY_ID,$params = array()){
        if($data = $this->_storage->load($id,$mode,$params)){
            $this->_loaded = true;
            $this->push($data);
        }else{
            $this->_loaded = false;
        }
        return false;
    }
    public function loadEx($id = null, $mode = Timetable\Storage::LOADBY_ID){
        $result = new Result();
        if($this->load($id,$mode)){
            $result->data = $this->pull();
        }else{
            $result->code = 1;
            $result->info = 'Failed Loading data';
        }
        return $result;
    }
    public function delete($data,$mode = Timetable\Storage::DELETEBY_ID){
        $result = new Result();
        $id = @$data['timetable_timetable_id'];

        if($id = $this->_storage->deleteRow($id,$mode)){
            $result->data = $id;
        }else{
            $result->code = 1;
            $result->info = 'Failed deleting agent';
        }
        return $result;
    }
    public function saveRows($rows,$storageLoader){
        $result = new Result();

        $headerRow = $rows[0];
        $dayRow = $rows[1];
        $dateRow = $rows[2];
        
        $dateRange2 = array();
        
        for($i = 4;$i < count($headerRow); $i++){
            if(!empty($headerRow[$i])){
                $dateRange2 = array(
                    'year' => date('Y', strtotime($headerRow[$i])),
                    'month' => date('m', strtotime($headerRow[$i]))
                );
            }
        }
        
        if( !isset($dateRange2['year']) || !isset($dateRange2['month']) ){
            $result->code = 300;
            $result->info = 'Please upload the correct file.';
            return $result;
        }

        $dateRange1 = array(
            'year' => date('Y', strtotime($rows[0][3])),
            'month' => date('m', strtotime($rows[0][3]))
        );

        unset($rows[0]);
        unset($rows[1]);
        unset($rows[2]);

        /* Ngambil Jumlah hari dalam 1 bulan $dayCount */
            $config = $storageLoader['config'];
            $adapter = $storageLoader['adapter'];

            $startDate = abs($config['start_date']);
            $lastDate = $startDate-1;

            $date1 = $dateRange1['year'].'-'.$dateRange1['month'].'-'.$startDate;
            $date2 = $dateRange2['year'].'-'.$dateRange2['month'].'-'.$lastDate;

            $date = round(
                strtotime($date2) - strtotime($date1)
            );

            $dayCount = round($date / (60 * 60 * 24));

        // Load Model
        $modelShift = new \Neuron\Attendance\Shift(\Neuron\Attendance\Shift\Storage::factory($adapter,$config));

        $list = array();

        // Cari user ID yang login untuk kolom update_by;

        if(class_exists('Neuron\Application\Framework\Entity')){
            $this->_session = \Neuron\Application\Framework\Entity::get($this)->session();
            $user_id = $this->_session->get('user_id');
        } else {
            $user_id = 0;
        }


        /*Data Timetable yang valid dimasukin sini*/
        $dataTimetable = array();
        $listErrorCode = array();
        $existDuplicateData = array();
        $incomingDuplicateData = array();

        foreach ($rows as $key => $value) {
            $shift = array();
            for($i=3;$i<=($dayCount+3);$i++){

                // Ambil bulan dan tahun berdasarkan header

                if($headerRow[$i] != null){
                    $timetableMonth = explode(' ',$headerRow[$i])[0];
                    $timetableMonth = date('m', strtotime($timetableMonth));
                    $timetableYear = explode(' ',$headerRow[$i])[1];
                }
                
                $day_no = $modelShift->getDayNo($dayRow[$i]);

                $storage = \Neuron\Attendance\Timetable\Storage::factory($adapter,$config);
                $modelTimetable = new \Neuron\Attendance\Timetable($storage);
                $dataTimetable = $modelTimetable->searchByCode($value[$i]);

                // cek kalo di kolomnya string(0) gausah di cek. dianggap kosong ga dimasukin. 
                if(!empty($value[$i])){
                    // Cek code yang dimasukkin di kolom ada di table `attd_timetable` apa tidak.
                    if(empty($dataTimetable)){
                        if(!in_array($value[$i], $listErrorCode)){
                            $listErrorCode[] = $value[$i];
                        }
                    }else{
                        $timetable_id = $dataTimetable['timetable_id']; 
                        $timetableDate = $dateRow[$i];
                        $timetableStartDate = $timetableYear.'-'.$timetableMonth.'-'.$timetableDate;

                        // Check if the date range exist or not.
                            $params = array(
                                'start_date' => $timetableStartDate,
                                'end_date' => $timetableStartDate,
                                'ref_id' => $value[2]);
                            $this->load(null,Timetable\Storage::LOADBY_BETWEENDATE,$params);
                            $checkDuplicate = $this->pull();
                        $params = array(
                            'timetable_id' => $timetable_id,
                            'ref_name' => $value[1],
                            'ref_code' => $this->ref_code,
                            'ref_id' => $value[2],
                            'start_date' => $timetableStartDate,
                            'end_date' => $timetableStartDate,
                            'last_update' => date('Y-m-d H:i:s'),
                            'update_by' => $user_id
                        );

                        // Duplicate Handler
                        if($this->_loaded){ 
                            $existDuplicateData[] = $checkDuplicate;
                            $incomingDuplicateData[] = $params;
                        }else{
                            $dataTimetable[] = $params;
                        }

                    }
                }
            }
        }

        if(count($listErrorCode) > 0){
            $result->code = 333;
            $timeTableCode = '';
            foreach ($listErrorCode as $data) {
                $timeTableCode .= '<li>'.$data.'</li>';
            }
            $result->info = 'This Following Timetable Code(s) is not exist <ul>'.$timeTableCode.'</ul>';
            return $result;
        }

        if(count($dataTimetable) >= 0 && count($existDuplicateData) > 0){
            $result->code = 11;
            $result->data = array(
                'validData' => $dataTimetable,
                'existDuplicateData' => $existDuplicateData,
                'incomingDuplicateData' => $incomingDuplicateData,
                'allData' => array_merge($dataTimetable,$incomingDuplicateData));
            $result->info = 'The uploaded data contain Overlaping or Duplicate data.';
            return $result;
            exit;
        }else if(count($dataTimetable) > 0){
            if(count($existDuplicateData) > 0){
                $result->code = 1;
                $result->info = "Duplicate Data.";
                $result->data = array(
                    'existDuplicateData' => $existDuplicateData,
                    'incomingDuplicateData' => $incomingDuplicateData,
                    'validData' => $dataTimetable);

                return $result;
            }else{
                foreach ($dataTimetable as $data) {
                    $this->saveEx($data);
                }
            }
        }else{
            $result->code = 11;
            $result->data = array(
                'validData' => $dataTimetable,
                'existDuplicateData' => $existDuplicateData,
                'incomingDuplicateData' => $incomingDuplicateData);
            $result->info = 'No data saved. Please make sure you have filled the sheet at least 1 row before uploading.';
            return $result;
            exit;
        }

        $result->data = $dataTimetable;

        return $result;
    }
    public function saveReplace($rows){
        $result = new Result();
        foreach ($rows['allData'] as $data) {
            $params = array(
                'start_date' => $data['start_date'],
                'end_date' => $data['end_date'],
                'ref_id' => $data['ref_id']);
            $this->load(null,Timetable\Storage::LOADBY_BETWEENDATE,$params);
            $checkDuplicate = $this->pull();
            if($this->_loaded){
                $this->end_date = date('Y-m-d');
                $update = $this->save(true);
            }
            $this->id = null;
            $this->saveEx($data);    
        }
        return $result;
    }
    public function saveDelete($rows){
        $result = new Result();
        foreach ($rows['allData'] as $data) {
            $params = array(
                'start_date' => $data['start_date'],
                'end_date' => $data['end_date'],
                'ref_id' => $data['ref_id']);
            $this->load(null,Timetable\Storage::LOADBY_BETWEENDATE,$params);
            $checkDuplicate = $this->pull();
            if($this->_loaded){
                $this->end_date = date('Y-m-d');
                $delete = $this->delete($checkDuplicate);
            }
            $this->id = null;
            $this->saveEx($data);    
        }
        return $result;
    }
    public function saveRowsByRange($rows,$storageLoader){
        $result = new Result();
        $adapter = $storageLoader['adapter'];
        $config = $storageLoader['config'];

        // Invalid File Handler 
        if($rows[0][0] != 'No.' || $rows[0][1] != 'Name' || $rows[0][2] != 'NIK' || $rows[0][3] != 'Code'){

            $result->code = 111;
            $result->info = 'Invalid File';
            return $result;
        }else{
            unset($rows[0]);
            unset($rows[1]);

            // Get User ID
                if(class_exists('Neuron\Application\Framework\Entity')){
                    $this->_session = \Neuron\Application\Framework\Entity::get($this)->session();
                    $user_id = $this->_session->get('user_id');
                } else {
                    $user_id = 0;
                }

            // array definition
                $listErrorCode = array();
                $listIncomingDuplicate = array();
                $listExistingDuplicate = array();
                $uploadedData = array();

            foreach ($rows as $data) {
                $start_date = strtotime($data[4]);
                $end_date = strtotime($data[5]);
                

                // Loading Model
                    $storage = \Neuron\Attendance\Timetable\Storage::factory($adapter,$config);
                    $modelTimetable = new \Neuron\Attendance\Timetable($storage);

                // Posted data
                    $name = $data[1];
                    $ref_id = $data[2];
                    $code = $data[3];
                    $start_date = date('Y-m-d',$start_date);
                    $end_date = date('Y-m-d',$end_date);

                // Check Code 
                    $dataTimetable = $modelTimetable->searchByCode($code);
                    if($dataTimetable == null){
                        // Check Code -> Fail
                        if(!in_array($code, $listErrorCode)){
                            $listErrorCode[] = $code;
                        }
                    }else{
                        $params = array(
                            'timetable_id' => $dataTimetable['timetable_id'],
                            'ref_code' => $this->ref_code,
                            'ref_id' => $ref_id,
                            'ref_name' => $name,
                            'start_date' => $start_date,
                            'end_date' => $end_date,
                            'last_update' => date('Y-m-d H:i:s'),
                            'update_by' => $user_id
                        );

                        $this->load(null,Timetable\Storage::LOADBY_BETWEENDATE,$params);
                        $checkDuplicate = $this->pull();
                        if($this->_loaded){
                            // `$this->_loaded == true` means duplicate
                            $listExistingDuplicate[] = $checkDuplicate;
                            $listIncomingDuplicate[] = $params;
                            $uploadedData[] = $params;
                        }else{
                            $uploadedData[] = $params;
                        }
                    }
            }


            if(count($listErrorCode) > 0){
            // Error code result;
                $timeTableCode = '';
                foreach ($listErrorCode as $data) {
                    $timeTableCode .= '<li>'.$data.'</li>';
                }
                $result->code = 3;
                $result->info = 'This Following Timetable Code(s) is not exist <ul>'.$timeTableCode.'</ul>';
                return $result;

            }else if(count($listIncomingDuplicate) > 0 || count($listExistingDuplicate) > 0){
            // Duplicate Result

                $result->code = 11;
                $result->data = array(
                    'existDuplicateData' => $listExistingDuplicate,
                    'incomingDuplicateData' => $listIncomingDuplicate,
                    'allData' => $uploadedData);
                $result->info = 'The uploaded data contain Overlaping or Duplicate data.';
                return $result;

            }else{
            // Valid data result
                foreach ($uploadedData as $data) {
                    $this->saveEx($data);
                }
                $result->data = $uploadedData;
            }

            return $result;
        }
    }
    public function loadTimetableByParam($params){
        return $this->_storage->loadTimetableByParam($params);
    }
    public function saveGroup($data){
        if($id = $this->_storage->saveGroup($data)){
            return $id;
        }else{
            return false;
        }
        return false;
    }
    public function saveExGroup($data){
        $result = new Result();

        if($id = $this->saveGroup($data)){
            $result->data = $id;
        }else{
            $result->code = 1;
            $result->info = 'Failed Saving Timetable Grouup';
        }
        return $result;
    }
}