<?php
namespace Neuron\Attendance\Timetable\Group\Timetable\Storage;

use Neuron\Attendance\Timetable\Group;
use Zend\Db\Sql\Where;

class Mysql extends \Neuron\Db\Storage implements Skeleton {
    
    public function loadList($id = null, $mode = Group\Timetable\Storage::LIST_ALL,$params = array()){
        // var_dump($group);die();
        // Base select
        $select = $this->select();
        $select->from(
            $this->_tables['attd_object_group_timetable']
        );


        

        if($mode == Group\Timetable\Storage::LISTBY_PAGE){
            $pager = new \Neuron\Db\Storage\Pager($params);
            $select->limit($pager->getLimit(),$pager->getOffset())->cache(0);
        }else{
            $pager = new \Neuron\Db\Storage\Pager(1,99999999);
            if(isset($params['page']) && isset($params['records'])){
                $pager = new \Neuron\Db\Storage\Pager($params);
                $select->limit($pager->getLimit(),$pager->getOffset())->cache(0);
            }
            switch ($mode) {
                case Group\Timetable\Storage::LISTBY_ID:
                    $select->where($this->__(array('id' => $id)));
                    break;
                case Group\Timetable\Storage::LISTBY_GROUPID:
                    $select->where($this->__(array('object_group_id' => $id)));
                    break;
            }
        }
        
        try{
            // echo('$select=> '.str_replace('"','',$select->getSqlString()));die();
            if($rows = $this->fetchAll($select)){
                $result = new \stdClass();
                $result->list = $rows;
                $result->pager = $pager;
                $result->countTotal = $rows->total;
                return $result;
            }

            return false;
        }catch(\Exception $ex){
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
            return false;
        }
    }

    public function save(Group\Timetable $timetable,$update = false){

        try{
            if($update){
                $update = $this->update();
                $update->table($this->_tables['attd_timetable_timetable']);
                $update->where($this->__(array('timetable_timetable_id' => $timetable->id)));
                $update->set($this->__($timetable->pull()));
                if($this->execute($update)){
                    return $timetable->id;
                }
            }else{
                $insert = $this->insert();
                $insert->into($this->_tables['attd_timetable_timetable']);
                $insert->autoincrement($this->__('timetable_timetable_id'));
                $insert->values($this->__($timetable->pull()));
                // echo('$insert=> '.str_replace('"','',$insert->getSqlString()));die();
                if($id = $this->execute($insert)){
                    return $id;
                }
            }
        }catch(\Exception $ex){
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine().' : '.$ex->getMessage());
            return false;
        }
    }
    public function load($id = null, $mode = Timetable\Storage::LOADBY_ID , $params = array()){
        $select = $this->select();
        $select->from($this->_tables['attd_timetable_timetable']);

        switch ($mode) {
            case Timetable\Storage::LOADBY_ID:
                $select->where($this->__(array('ticket_timetable_id' => $id)));
                break;
            case Timetable\Storage::LOADBY_BETWEENDATE:
                // $select->where->greaterThanOrEqualTo('start_date',$params['start_date']);
                // $select->where->lessThanOrEqualTo('end_date',$params['end_date']);
                $select->where($this->__(array(
                    'start_date >=\'' . $params['start_date'].' \' ',
                    'end_date <=\'' . $params['end_date'].' \' ',
                )));

                $select->where($this->__(array('ref_id' => $params['ref_id'])));
                break;
        }

        try{
            $data = $this->fetchRow($select);
            return $data;
        }catch(\Exception $ex){
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
            return false;
        }
    }
    public function deleteRow($id = null ,$mode = Timetable\Storage::DELETEBY_ID){
        $delete = $this->delete();
        $delete->from($this->_tables['attd_timetable_timetable']);
        switch ($mode) {
            case Timetable\Storage::DELETEBY_ID:
                $delete->where($this->__(array('timetable_timetable_id' => $id)));
                break;
        }
        try{
            if($this->execute($delete)){
                return $id;
            }
        }catch(\Exception $ex){
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
            return false;
        }
    }
    public function loadTimetableByParam($params) {

        $select = $this->select();
        $select->from(['ta' => $this->_tables['attd_timetable_timetable']]);
        $select->join(['ts' => $this->_tables['attd_timetable_shift']], 'ta.timetable_id= ts.timetable_id', array('*'));
        $select->join(['tt' => $this->_tables['attd_timetable']], 'ta.timetable_id= tt.timetable_id', array('*'));

        if(isset($params)) {
            foreach($params as $key => $val) {
                 $select->where(array($key => $val));
            }
        }

        // echo str_replace('"','',$select->getSqlString())."<br />"; exit;

        $result = $this->fetchRow($select);
        return $result;
    }
    public function filterList($params = array()){
        // Base select
        $select = $this->select();
        $select->from(
            array('ta' => $this->_tables['attd_timetable_timetable'])
        );
        $select->join(array('ts' => $this->_tables['attd_timetable_shift']), 'ta.timetable_id= ts.timetable_id', array('day_no','is_workday','on_duty_time','off_duty_time', 'shift_start_date'=>'start_date', 'shift_end_date'=>'end_date'),'left');
        $select->join(array('tt' => $this->_tables['attd_timetable']), 'ta.timetable_id= tt.timetable_id', array('*'),'left');

        $select->where(array('ta.'.$params['filterby'].' like "%'. $params['filterquery'].'%"'));
        if($params['filterby']== 'ref_id' || $params['filterby'] == 'ref_name'){
            $select->group('ta.start_date');
        }

        $pager = new \Neuron\Db\Storage\Pager($params);
        $select->limit($pager->getLimit(),$pager->getOffset())->cache(0);
        // echo str_replace('"','',$select->getSqlString())."<br />"; exit;

        try{
            // echo('$select=> '.str_replace('"','',$select->getSqlString()));die();
            if($rows = $this->fetchAll($select)){
                $result = new \stdClass();
                $result->list = $rows;
                $result->pager = $pager;
                $result->countTotal = $rows->total;
                return $result;
            }

            return false;
        }catch(\Exception $ex){
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
            return false;
        }
    }
    public function saveGroup($data){
        try{
            $insert = $this->insert();
            $insert->into($this->_tables['attd_object_group_timetable']);
            $insert->autoincrement($this->__('id'));
            $insert->values($this->__($data));
            // echo('$insert=> '.str_replace('"','',$insert->getSqlString()));die();
            if($id = $this->execute($insert)){
                return $id;
            }
        }catch(\Exception $ex){
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine().' : '.$ex->getMessage());
            return false;
        }
    }
    

}