<?php
namespace Neuron\Attendance\Timetable;

use Neuron\Generic\Result;
use Neuron\Generic\Entity\Mapper;
use Neuron\Generic\Entity\Map;
use Neuron\Attendance\Timetable;
use Neuron\Attendance\Shift;

class ObjectTimetable Extends Mapper{
    protected $_storage;
    protected $_loaded;
    protected $ref_code = 'Employee';
    protected $_user_id;

    public function __construct(Objects\Storage\Skeleton $storage){
        $this->_storage = $storage;
        parent::__construct(
            array(),
            array(
                new Map('timetable_object_id','id'),
                new Map('timetable_id'),
                new Map('ref_code'),
                new Map('ref_id'),
                new Map('start_date'),
                new Map('end_date'),
                new Map('last_update'),
                new Map('update_by'),
                new Map('description',null,true),
                new Map('timetable_shift_id',null,true),
                new Map('day_no',null,true),
                new Map('is_workday',null,true),
                new Map('on_duty_time',null,true),
                new Map('off_duty_time',null,true),
                new Map('in_beginning_time',null,true),
                new Map('in_ending_time',null,true),
                new Map('out_ending_time',null,true),
                new Map('out_beginning_time',null,true),
                new Map('late_time',null,true),
                new Map('leave_early_time',null,true),
                new Map('overtime_start_time',null,true),

                new Map('code',null,true),
                new Map('parent_timetable_id',null,true),
                new Map('name',null,true),
                new Map('time_mode',null,true),
            ),parent::CASE_INSENSITIVE
        );
    }  
    public function save($update = false){
        if($id = $this->_storage->save($this,$update)){
            return $id;
        }else{
            return false;
        }
        return false;
    }
    public function saveEx($data){
        $result = new Result();
        
        if(isset($data['timetable_object_id']) && !empty($data['timetable_object_id']) ){
            $update = true;
            $this->load($data['timetable_object_id'],Objects\Storage::LOADBY_ID);
        }else{
            $update = false;
        }
        $param = $this->pull();
        foreach ($data as $key => $value) {
            if(array_key_exists($key, $param)){
                $param[$key] = $value;
            }
        }
        $this->push($param);
        if($id = $this->save($update)){
            $result->data = $id;
        }else{
            $result->code = 1;
            $result->info = 'Failed Saving Object';
        }
        return $result;
    }
    public function load($id = null, $mode = Objects\Storage::LOADBY_ID,$params = array()){
        if($data = $this->_storage->load($id,$mode,$params)){
            $this->_loaded = true;
            @$this->push($data);
            return $data;
        }else{
            $this->_loaded = false;
        }
        return false;
    }
    public function loadEx($id = null, $mode = Objects\Storage::LOADBY_ID,$params = array()){
        $result = new Result();
        $this->load($id,$mode,$params);
        if($this->_loaded){
            $result->data = $this->pull();
        }else{
            $result->code = 1;
            $result->info = 'Failed Loading data';
        }
        return $result;
    }
    public function delete($data,$mode = Objects\Storage::DELETEBY_ID){
        $result = new Result();
        $id = @$data['timetable_object_id'];

        if($id = $this->_storage->deleteRow($id,$mode)){
            $result->data = $id;
        }else{
            $result->code = 1;
            $result->info = 'Failed deleting agent';
        }
        return $result;
    }
    public function saveRows($rows,$storageLoader){
        $result = new Result();

        $headerRow = $rows[0];
        $dayRow = $rows[1];
        $dateRow = $rows[2];
        
        $dateRange2 = array();
        
        for($i = 4;$i < count($headerRow); $i++){
            if(!empty($headerRow[$i])){
                $dateRange2 = array(
                    'year' => date('Y', strtotime($headerRow[$i])),
                    'month' => date('m', strtotime($headerRow[$i]))
                );
            }
        }
        
        if( !isset($dateRange2['year']) || !isset($dateRange2['month']) ){
            $result->code = 300;
            $result->info = 'Please upload the correct file.';
            return $result;
        }

        $dateRange1 = array(
            'year' => date('Y', strtotime($rows[0][3])),
            'month' => date('m', strtotime($rows[0][3]))
        );

        unset($rows[0]);
        unset($rows[1]);
        unset($rows[2]);

        /* Ngambil Jumlah hari dalam 1 bulan $dayCount */
            $config = $storageLoader['config'];
            $adapter = $storageLoader['adapter'];

            $startDate = abs($config['start_date']);
            $lastDate = $startDate-1;

            $date1 = $dateRange1['year'].'-'.$dateRange1['month'].'-'.$startDate;
            $date2 = $dateRange2['year'].'-'.$dateRange2['month'].'-'.$lastDate;

            $date = round(
                strtotime($date2) - strtotime($date1)
            );

            $dayCount = round($date / (60 * 60 * 24));

        // Load Model
        $modelShift = new \Neuron\Attendance\Shift(\Neuron\Attendance\Shift\Storage::factory($adapter,$config));

        $list = array();

        // Cari user ID yang login untuk kolom update_by;

        if(class_exists('Neuron\Application\Framework\Entity')){
            $this->_session = \Neuron\Application\Framework\Entity::get($this)->session();
            $user_id = $this->_session->get('user_id');
        } else {
            $user_id = 0;
        }


        /*Data Object yang valid dimasukin sini*/
        $dataObject = array();
        $listErrorCode = array();
        $existDuplicateData = array();
        $incomingDuplicateData = array();

        foreach ($rows as $key => $value) {
            $shift = array();
            for($i=3;$i<=($dayCount+3);$i++){

                // Ambil bulan dan tahun berdasarkan header

                if($headerRow[$i] != null){
                    $objectMonth = explode(' ',$headerRow[$i])[0];
                    $objectMonth = date('m', strtotime($objectMonth));
                    $objectYear = explode(' ',$headerRow[$i])[1];
                }
                
                $day_no = $modelShift->getDayNo($dayRow[$i]);

                $storage = \Neuron\Attendance\Timetable\Storage::factory($adapter,$config);
                $modelTimetable = new \Neuron\Attendance\Timetable($storage);
                $dataTimetable = $modelTimetable->searchByCode($value[$i]);

                // cek kalo di kolomnya string(0) gausah di cek. dianggap kosong ga dimasukin. 
                if(!empty($value[$i])){
                    // Cek code yang dimasukkin di kolom ada di table `attd_timetable` apa tidak.
                    if(empty($dataTimetable)){
                        if(!in_array($value[$i], $listErrorCode)){
                            $listErrorCode[] = $value[$i];
                        }
                    }else{
                        $timetable_id = $dataTimetable['timetable_id']; 
                        $objectDate = $dateRow[$i];
                        $objectStartDate = $objectYear.'-'.$objectMonth.'-'.$objectDate;

                        // Check if the date range exist or not.
                            $params = array(
                                'start_date' => $objectStartDate,
                                'end_date' => $objectStartDate,
                                'ref_id' => $value[2]);
                            $this->load(null,Objects\Storage::LOADBY_BETWEENDATE,$params);
                            $checkDuplicate = $this->pull();
                        $params = array(
                            'timetable_id' => $timetable_id,
                            'ref_name' => $value[1],
                            'ref_code' => $this->ref_code,
                            'ref_id' => $value[2],
                            'start_date' => $objectStartDate,
                            'end_date' => $objectStartDate,
                            'last_update' => date('Y-m-d H:i:s'),
                            'update_by' => $user_id
                        );

                        // Duplicate Handler
                        if($this->_loaded){ 
                            $existDuplicateData[] = $checkDuplicate;
                            $incomingDuplicateData[] = $params;
                        }else{
                            $dataObject[] = $params;
                        }

                    }
                }
            }
        }

        if(count($listErrorCode) > 0){
            $result->code = 333;
            $timeTableCode = '';
            foreach ($listErrorCode as $data) {
                $timeTableCode .= '<li>'.$data.'</li>';
            }
            $result->info = 'This Following Timetable Code(s) is not exist <ul>'.$timeTableCode.'</ul>';
            return $result;
        }

        if(count($dataObject) >= 0 && count($existDuplicateData) > 0){
            $result->code = 11;
            $result->data = array(
                'validData' => $dataObject,
                'existDuplicateData' => $existDuplicateData,
                'incomingDuplicateData' => $incomingDuplicateData,
                'allData' => array_merge($dataObject,$incomingDuplicateData));
            $result->info = 'The uploaded data contain Overlaping or Duplicate data.';
            return $result;
            exit;
        }else if(count($dataObject) > 0){
            if(count($existDuplicateData) > 0){
                $result->code = 1;
                $result->info = "Duplicate Data.";
                $result->data = array(
                    'existDuplicateData' => $existDuplicateData,
                    'incomingDuplicateData' => $incomingDuplicateData,
                    'validData' => $dataObject);

                return $result;
            }else{
                foreach ($dataObject as $data) {
                    $this->saveEx($data);
                }
            }
        }else{
            $result->code = 11;
            $result->data = array(
                'validData' => $dataObject,
                'existDuplicateData' => $existDuplicateData,
                'incomingDuplicateData' => $incomingDuplicateData);
            $result->info = 'No data saved. Please make sure you have filled the sheet at least 1 row before uploading.';
            return $result;
            exit;
        }

        $result->data = $dataObject;

        return $result;
    }
    public function saveReplace($rows){
        $result = new Result();
        foreach ($rows['allData'] as $data) {
            $params = array(
                'start_date' => $data['start_date'],
                'end_date' => $data['end_date'],
                'ref_id' => $data['ref_id']);
            $this->load(null,Objects\Storage::LOADBY_BETWEENDATE,$params);
            $checkDuplicate = $this->pull();
            if($this->_loaded){
                $this->end_date = date('Y-m-d');
                $update = $this->save(true);
            }
            $this->id = null;
            $this->saveEx($data);    
        }
        return $result;
    }
    public function saveDelete($rows){
        $result = new Result();
        foreach ($rows['allData'] as $data) {
            $params = array(
                'start_date' => $data['start_date'],
                'end_date' => $data['end_date'],
                'ref_id' => $data['ref_id']);
            $this->load(null,Objects\Storage::LOADBY_BETWEENDATE,$params);
            $checkDuplicate = $this->pull();
            if($this->_loaded){
                $this->end_date = date('Y-m-d');
                $delete = $this->delete($checkDuplicate);
            }
            $this->id = null;
            $this->saveEx($data);    
        }
        return $result;
    }
    public function saveRowsByRange($rows,$storageLoader){
        $result = new Result();
        $adapter = $storageLoader['adapter'];
        $config = $storageLoader['config'];

        // Invalid File Handler 
        if($rows[0][0] != 'No.' || $rows[0][1] != 'Name' || $rows[0][2] != 'NIK' || $rows[0][3] != 'Code'){

            $result->code = 111;
            $result->info = 'Invalid File';
            return $result;

        }else{
            unset($rows[0]);
            unset($rows[1]);

            // Get User ID
                if(class_exists('Neuron\Application\Framework\Entity')){
                    $this->_session = \Neuron\Application\Framework\Entity::get($this)->session();
                    $user_id = $this->_session->get('user_id');
                } else {
                    $user_id = 10000000000;
                }

            // array definition
                $listErrorCode = array();
                $listIncomingDuplicate = array();
                $listExistingDuplicate = array();
                $uploadedData = array();

            foreach ($rows as $data) {
                $start_date = strtotime($data[4]);
                $end_date = strtotime($data[5]);
                

                // Loading Model
                    $storage = \Neuron\Attendance\Timetable\Storage::factory($adapter,$config);
                    $modelTimetable = new \Neuron\Attendance\Timetable($storage);

                // Posted data
                    $name = $data[1];
                    $ref_id = $data[2];
                    $code = $data[3];
                    $start_date = date('Y-m-d',$start_date);
                    $end_date = date('Y-m-d',$end_date);

                // Check Code 
                    $dataTimetable = $modelTimetable->searchByCode($code);
                    if($dataTimetable == null){
                        // Check Code -> Fail
                        if(!in_array($code, $listErrorCode)){
                            $listErrorCode[] = $code;
                        }
                    }else{
                        $params = array(
                            'timetable_id' => $dataTimetable['timetable_id'],
                            'ref_code' => $this->ref_code,
                            'ref_id' => $ref_id,
                            'ref_name' => $name,
                            'start_date' => $start_date,
                            'end_date' => $end_date,
                            'last_update' => date('Y-m-d H:i:s'),
                            'update_by' => $user_id
                        );

                        $this->load(null,Objects\Storage::LOADBY_BETWEENDATE,$params);
                        $checkDuplicate = $this->pull();
                        if($this->_loaded){
                            // `$this->_loaded == true` means duplicate
                            $listExistingDuplicate[] = $checkDuplicate;
                            $listIncomingDuplicate[] = $params;
                            $uploadedData[] = $params;
                        }else{
                            $uploadedData[] = $params;
                        }
                    }
            }


            if(count($listErrorCode) > 0){
            // Error code result;
                $timeTableCode = '';
                foreach ($listErrorCode as $data) {
                    $timeTableCode .= '<li>'.$data.'</li>';
                }
                $result->code = 3;
                $result->info = 'This Following Timetable Code(s) is not exist <ul>'.$timeTableCode.'</ul>';
                return $result;

            }else if(count($listIncomingDuplicate) > 0 || count($listExistingDuplicate) > 0){
            // Duplicate Result

                $result->code = 11;
                $result->data = array(
                    'existDuplicateData' => $listExistingDuplicate,
                    'incomingDuplicateData' => $listIncomingDuplicate,
                    'allData' => $uploadedData);
                $result->info = 'The uploaded data contain Overlaping or Duplicate data.';
                return $result;

            }else{
            // Valid data result
                foreach ($uploadedData as $data) {
                    $this->saveEx($data);
                }
                $result->data = $uploadedData;
            }

            return $result;
        }
    }
    public function loadObjectByParam($params){
        return $this->_storage->loadObjectByParam($params);
    }
    public function saveGroup($data){
        if($id = $this->_storage->saveGroup($data)){
            return $id;
        }else{
            return false;
        }
        return false;
    }
    public function saveExGroup($data){
        $result = new Result();

        if($id = $this->saveGroup($data)){
            $result->data = $id;
        }else{
            $result->code = 1;
            $result->info = 'Failed Saving Object Grouup';
        }
        return $result;
    }
    public function massTerminate($ref_id,$parent_timetable_id){
        $result         = new Result();
        $objects        = new Objects($this->_storage);

        /**
         * Load dulu semua objectsnya berdasarkan ref_id 
         * dan parent_timetable_id nya
         */
        $loadParams     = array(
            'draw'      => null,
            'id'        => $parent_timetable_id, /* di mysqlnya key id padahal isinya seharusnya parent_timetable_id */
            'ref_id'    => $ref_id
        );
        $loadObject     = $objects->load(null,Objects\Storage::LISTBY_PAGE,$loadParams);
        $terminated     = array();
        if($loadObject->code == 0){
            $objects    = $loadObject->data['data'];
            foreach($objects as $object){
                $this->push($object);

                /* Update end datenya jadi tudey dan save lagih */
                $this->end_date     = date('Y-m-d',strtotime( date('Y-m-d').'- 1 days' ));
                
                if($this->save(true)){
                    $terminated[]   = $this->pull();
                }

                $this->clear();
            }
        }

        $result->data   = $terminated;
        $result->info   = "Success terminating data";
        return $result;            
    }

    public function changeShift(Changeshift $changeShift){
        $result         = new Result;

        
        if(!$changeShift->id){
            return new Result(0,401,'Invalid changeshift data.');
        }
        $requesterRefID         = $changeShift->ref_id;
        $defaultShiftID         = $changeShift->default_shift_id;
        $changeShiftID          = $changeShift->change_shift_id;

        $storageShift           = Shift\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig());
        $shift                  = new Shift($storageShift);
        $defaultTimetableID     = $shift->load($defaultShiftID)->data['timetable_id'];
        $changeTimetableID      = $shift->load($changeShiftID)->data['timetable_id'];

        /* Load active timetable*/
        $this->load($defaultTimetableID,Objects\Storage::LOADBY_TIMETABLE_ID_REF_ID,array('timetable_id' => $defaultTimetableID,'ref_id' => $requesterRefID));
        /* assign to variable for recreate */
        $activeTimetable            = $this->pull();

        /* Deactive current Active Shift */
        $this->end_date         = date('Y-m-d', strtotime($changeShift->shift_change_date . "-1 days"));
        $this->last_update      = date('Y-m-d H:i:s');
        $oldShift               = $this->save(true);

        /* Insert Change Shift Records */
        $this->id               = null;
        $this->timetable_id     = $changeTimetableID;
        $this->ref_code         = "EMPLOYEE";
        $this->ref_id           = $requesterRefID;
        $this->start_date       = $changeShift->shift_change_date;
        $this->end_date         = $changeShift->shift_change_date;
        $this->last_update      = date('Y-m-d H:i:s');
        $this->update_by        = $changeShift->change_by;
        $changedShift           = $this->save(FALSE);

        /* Set active default records, but change start_dtm to shift_change_date + 1 day for reactive later. */
        $this->id                       = null;
        $this->start_date               = date('Y-m-d', strtotime($changeShift->shift_change_date . "+1 days"));
        $this->timetable_id             = $activeTimetable['timetable_id'];
        $this->last_update              = date('Y-m-d H:i:s');
        $this->update_by                = $changeShift->change_by;
        $this->end_date                 = $activeTimetable['end_date'];
        $this->ref_code                 = $activeTimetable['ref_code'];
        $this->ref_id                   = $activeTimetable['ref_id'];
        $reinsertShift                  = $this->save(FALSE);

        //insert timeline
        if(class_exists('\Neuron\Addon\Employee\Timeline\Entity')){

            $time = $changeShift->shift_change_date;
            $title = 'Mulai Pindah Shift';
            $summary = 'Mulai Pindah Shift';

            $employee 	= new \Neuron\Employee\Entity(\Neuron\Employee\Entity\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig()));
            $loadMap 	= $employee->loadMap($this->ref_id,'person_id');
            $userId 	= null;
            if($loadMap['code'] == 200){
                $map 		= $loadMap['data'];
                $userId 	= $map['user_id'];
            }

            $timelineStorage  = \Neuron\Feed\Entity\Storage::factory($this->_storage->getAdapter(), $this->_storage->getConfig());
            $timeline = new \Neuron\Addon\Employee\Timeline\Entity($timelineStorage);
            $timelineData = [
                "feed_group_id" => 1,
                "title"         => $title,
                "summary"       => $summary,
                "thumbnail"     => null,
                "url"           => 'attendance/request/saveapprove',
                "keywords"      => null,
                "publish_dtm"   => $time,
                "reference_code"   => 'USER',
                "reference_id"   => $userId,
            ];
            $timeline->create($timelineData);
        }
        
        $result->data       = array(
            'oldShift'      => $oldShift,
            'changeShift'   => $changedShift,
            'reinsertShift' => $reinsertShift
        );
        
        return $result;

    }
}