<?php
namespace Neuron\Attendance\Timetable;


use Neuron\Generic\Result;
use Neuron\Employee;
use Neuron\Attendance\Shift;
use Neuron\Attendance\Shifts;
use Neuron\Attendance\Timetable as TimetableModel;
use Throwable;


class Objects extends \Neuron\Generic\Set{
	protected $_storage;

	public function __construct(Objects\Storage\Skeleton $storage){
		$this->_storage = $storage;
	}
	public function load($id = null, $mode = Objects\Storage::LIST_ALL ,$params = array(),$group = array()){
		$result = new Result();
		
		if($rows = $this->_storage->loadList($id,$mode,$params,$group)){
			if(count($rows['data']) > 0){
				$shifts = [];
				foreach($rows['data'] as $shift){
					//load detail data shift baru
					$storageTimetable           = TimetableModel\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig());
					$modelTimetable                  = new TimetableModel($storageTimetable);
					$modelTimetable->load($shift['timetable_id']);
					$shift['name'] = $modelTimetable->name;
					$shifts[] = $shift;
				}
				$rows['data'] = $shifts;
				$result->data = array(
					'draw' => $params['draw'],
					'recordsTotal' => $rows['total'],
					'recordsFiltered' => $rows['total'],
					'data' => $rows['data'],
				);
				
			}else{
				$result->code = 1;
				$result->info = 'Shift Objects load none';
			}
		}else{
			$result->data = array(
				'draw' => $params['draw'],
				'recordsTotal' => 0,
				'recordsFiltered' => 0,
				'data' => array(),
			);
		}
		return $result;
	}
	public function filterList($params = array()){
		$result = new Result();
		$list = array();

		if($rows = $this->_storage->filterList($params)){
			$this->clear();
			if($rows->countTotal > 0){
				foreach ($rows->list as $row) {
					array_push($list, $row);
				}
			}
		}

		if(count($list) > 0){
			$result->data = array(
				'list' => $list,
				'countTotal' => $rows->countTotal,
				'pager' => $rows->pager);
		}else{
			$result->code = 1;
			$result->info = 'Shift Objects load none';
		}
		return $result;
	}
	public function addRefName($rows){
		$config['tables'] = $this->_storage->getTables();
		if(class_exists('Neuron\Employee\Entity')){
			$employee = new Employee\Entity(Employee\Entity\Storage::factory(
				$this->_storage->getAdapter(), array('tables' => $this->_storage->getTables())	
			));
            $dataEmp = $employee->detailEmp();
            $employee = $dataEmp['data'];
		}else{
			$employee = array();
		}

		// looping passed rows
		$list = array();
		foreach ($rows['data'] as $row) {
			// find the match ref_id with employee
				foreach ($employee as $empRow) {
					if($empRow['id'] == $row['ref_id']){
						$ref_name = $empRow['first_name'].' '.$empRow['last_name'];
						break;
					}else{
						$ref_name = 'References ID not found.';
					}
				}
				$row['ref_name'] = $ref_name;

			// manual grouping
				$addToList = true;
				foreach ($list as $rowList) {
					$listRefId = $rowList['ref_id'];
					$rowRefId = $row['ref_id'];
					if($listRefId === $rowRefId){
						$addToList = false;
						break;
					}
				}
				if($addToList){
					$list[] = $row;
				}
		}
		return $list;
	}
	public function listSetAdmin_old($params = array()){
		$result = new Result();
		$rows = $this->_storage->listSetAdmin($params);
		if($rows){
			$result->data 	= $rows;
		}else{
			$result->code  	= 501;
			$result->info 	= "failed to load data";
		}

		return $result;
	}
	public function listChangeShift($params = array()){
		$result = new Result();
		$rows = $this->_storage->listChangeShift($params);
		if($rows['recordsTotal'] > 0){
			$result->code = 0;
			$result->data = array(
				'draw' => $params['draw'],
				'recordsTotal' => $rows['recordsTotal'],
				'recordsFiltered' => $rows['recordsFiltered'],
				'data' => $rows['data'],
			);
		}else{
			$result->code = 1;
			$result->info = 'Shift Objects load none';
			$result->data = array(
				'draw' => $params['draw'],
				'recordsTotal' => 0,
				'recordsFiltered' => 0,
				'data' => array(),
			);
		}
		return $result;
	}
	public function loadDatatable($post){
		$rs = $this->_storage->loadDatatable($post);
		$data 	= $rs->data;
		$newData 	= array();

		if($data){
			foreach ($data as $row) {
				$row['ref_name'] 	= 'Name';
				$newData[] 			= $row;
			}
		}else{
			$newData 	= $data;
		}

		$result 		= array(
			'recordsTotal' 	=> $rs->countTotal,
			'recordsFiltered' => $rs->countTotal,
			'data' 	=> $newData,
			'draw' 	=> $post['draw'],
		);

		return $result;

	}
	public function listSetAdmin($post){
		$storageShift 		= Shift\Storage::factory($this->_storage->getAdapter(),$this->_storage->getConfig());
		$shift 				= new Shift($storageShift);
		$shifts 			= new Shifts($storageShift);

		$dayNumber 			= $shift->getDayNumber(date('Y-m-d'))->data;
		$shiftList 			= $shifts->load($dayNumber,Shift\Storage::LISTBY_DAYNO)->data;
		
		$timetable_id 		= array();
		
		if(isset($shiftList['list'])){
			$shiftList 		= $shiftList['list'];
			if($shiftList){
				foreach ($shiftList as $shift) {
					$timetable_id[] 	= $shift['timetable_id'];
				}
			}
		}

		$listObject 		= $this->_storage->listSetAdmin($timetable_id,$post);
		$lists 				= array();
		if($listObject->data){
			foreach ($listObject->data as $object) {
				$lists[] = $object;
			}
		}

		$result 			= array(
			'draw' 			=> $post['draw'],
			'data' 			=> $lists,
			'recordsTotal' 	=> $listObject->countTotal,
			'recordsFiltered' => $listObject->countTotal,
		);

		return $result;

	}
	
	public function listData($ids = '', $mode = Objects\Storage::LIST_ALL, $params = array()){
        $result = new Result();
        try {
            /* clear list */
            $this->clear();

            if ($rs = $this->_storage->listData($ids, $mode, $params)) {
                if ($rows = $rs->list) {
                    $rows = $rs->list;
                    $list = [];
                    foreach ($rows as $row) {
                        /* for issue oracle */
                        $row = array_change_key_case($row);
                        unset($row['b_rownum']);
                        $list[] = $row;
                    }
                    $result->data = [
                        'pager' => $rs->pager,
                        'list' => $list,
                        'countTotal' => $rs->countTotal,
                    ];
                } else {
                    $result->code = 2;
                    $result->info = 'load_none';
                }
            } else {
                $result->code = 1;
                $result->info = 'load_not_found';
            }
            return $result;
        } catch (Throwable $throwable) {
            error_log('Error at ' . $throwable->getFile() . ' line ' . $throwable->getLine() . ': ' . $throwable->getMessage());
            $result->code = 500;
            $result->info = 'error';
            return $result;
        }
	}
	
	public function listDataShift($post){
		$listObject 		= $this->_storage->listDataShift($post);
		$lists 				= array();
		if($listObject->data){
			foreach ($listObject->data as $object) {
				$lists[] = $object;
			}
		}

		$result 			= array(
			'draw' 			=> @$post['draw'],
			'data' 			=> $lists,
		);

		return $result;

	}

}