<?php
namespace Neuron\Attendance\Timetable\Objects\Storage;

use Neuron\Attendance\Timetable\Objects;
use Zend\Db\Sql\Predicate\Expression;
use Neuron\Attendance\Timetable\ObjectTimetable;
use Neuron\Db\Sql\Select;
use Zend\Db\Sql\Where;
use Neuron\Db\Storage\Pager;


class Mysql extends \Neuron\Db\Storage implements Skeleton {

    const EMP_STATUS_ACTIVE = 1;
    
    public function loadList($id = null, $mode = Object\Storage::LIST_ALL,$params = array(),$group = Object\Storage::GROUPBY_NONE){
        // Base select
        $select = $this->select();
        $select->from(array('ta' => $this->_tables['attd_timetable_object'] ));
        
        $select->where(new Expression('(CURDATE() <= ta.end_date OR ta.end_date IS NULL)'));
        
        $searchable = false;
        
        if(isset($params['draw'])) {
            $filtered = 0;
            $arrColumn = [];

            /* Kalo class employee exists langsung dijoin aja */
            if(class_exists('\Neuron\Employee\Entity')){
                $select->join(array('employee' => $this->_tables['employee']),'employee.id = ta.ref_id',array());
                $select->join(array('emp' => $this->_tables['person']),'emp.id = employee.person_id',array('ref_name' => new Expression("CONCAT(first_name,' ',COALESCE(`last_name`,''))")));
            }

            if(isset($params['ref_id'])) {
                
                foreach ($params['columns'] as $col) {
                    if($col['data'] != null && $col['name'] != null) {
                        $arrColumn[$col['data']] = $col['name'];
                    }
                }

                $i = $params['order'][0]['column'];
                $dir = $params['order'][0]['dir'];
                $select->order(array($params['columns'][$i]['name'] . " " . $dir));
            }else {
                $select->join(array('tt' => new Expression('(SELECT s.timetable_id, s.`parent_timetable_id` FROM attd_timetable as s WHERE s.parent_timetable_id = '.$params['id'].')')), 'ta.timetable_id= tt.timetable_id', array());
                $select->join(array('ts' => new Expression('(SELECT * FROM attd_timetable_shift AS ss WHERE CURDATE() <= ss.end_date OR ss.end_date is NULL)')), 'ta.timetable_id= ts.timetable_id', array());
                
                $arrColumn['ref_id'] = new \Zend\Db\Sql\Expression("DISTINCT ta.ref_id");
            }
            
            $arrColumn['ref_code'] = 'ta.ref_code';

            
			if(!empty($params['search']['value'])) {
				$searchable = true;
                $nest = $select->where->nest();
                

                if(class_exists('\Neuron\Employee\Entity')){
                    $nest->like(new Expression("CONCAT(first_name,' ',last_name)"),new Expression('\'%'.$params['search']['value'].'%\''));
                }

                $nest->unnest();

                if($searchable){
                    $selectF = $this->select()->from(['ta' => $this->_tables['attd_timetable_object']]);
                    $selectF->columns(['total_count' => new \Zend\Db\Sql\Expression('COUNT(DISTINCT ta.ref_id)')]);
                    
                    $selectF->where(new Expression('CURDATE() <= ta.end_date'));
                    $selectF->join(array('tt' => new Expression('(SELECT s.timetable_id, s.`parent_timetable_id` FROM attd_timetable as s WHERE s.parent_timetable_id = '.$params['id'].')')), 'ta.timetable_id= tt.timetable_id', array());
                    $selectF->join(array('ts' => new Expression('(SELECT * FROM attd_timetable_shift AS ss WHERE CURDATE() <= ss.end_date OR ss.end_date IS NULL)')), 'ta.timetable_id= ts.timetable_id', array());
                    
                    if(!empty($params['search']['value'])) {
                        $searchable = true;
                        $nestC = $selectF->where->nest();
                        foreach ($params['columns'] as $col) {
                            if($col['data'] != null) {
                                if($col['searchable'] === "true") {
                                    $nestC->like(new Expression('LOWER('.$col['name'].')'),new Expression('\'%'.$params['search']['value'].'%\''));
                                    $nestC->or;
                                }
                            }
                        }
                        $nestC->unnest();
                    }
        
                    $dataFiltered = $this->fetchRow($selectF);
                    $filtered = ($dataFiltered['total_count'] > 0) ? $dataFiltered['total_count']:0;
                }
            }

            $columnName     = $params['columns'][$params['order'][0]['column']]['data'];

            if($columnName == 'ref_name'){
                $columnName = 'ref_id';
            } 
            $dir            = $params['order'][0]['dir'];

            $select->order($columnName.' '.$dir);

            $select->columns($arrColumn,false);

            $offset = ($params->start/$params->length) * $params->length;
            $select->limit(intval($params->length));
            $select->offset(intval($offset));
        }
        
        if(isset($params['ref_id'])) {
            if(isset($params['id'])){
                $select->join(
                    array(
                        'tt' => new Expression('(SELECT s.timetable_id, s.`parent_timetable_id` FROM attd_timetable as s WHERE s.parent_timetable_id = '.$params['id'].')')), 
                        'ta.timetable_id= tt.timetable_id', 
                        array('*')
                    );
            }
            $select->join(
                array(
                    'ts' => new Expression('(SELECT * FROM attd_timetable_shift AS ss WHERE CURDATE() <= ss.end_date OR ss.end_date IS NULL)')
                ), 
                'ta.timetable_id = ts.timetable_id', 
                array('timetable_id','day_no','on_duty_time','off_duty_time')
            );
            
            $select->where(['ta.ref_id = '.$params['ref_id']]);
            $select->order(array("ta.start_date ASC"));
        }

        try{
            if($rows = $this->fetchAll($select)) {
                
                $dataAll = [];
                if($rows) {
                    foreach($rows as $row) {
                        array_push($dataAll, $row);
                    }
                }

                $selectC = $select;
                $selectC->reset('limit')->reset('offset')->reset('joins');

                if(class_exists('\Neuron\Employee\Entity')){
                    $select->join(array('employee' => $this->_tables['employee']),'employee.id = ta.ref_id',array());
                    $selectC->join(array('emp' => $this->_tables['person']),'emp.id = employee.person_id',array());
                }
                $arrColumn = [];
                if(isset($params['ref_id'])) {
                    
                    if(isset($params['columns'])){
						
						foreach ($params['columns'] as $col) {
							if($col['data'] != null && $col['name'] != null) {
								$arrColumn[$col['data']] = $col['name'];
							}
						}
		
						$i = $params['order'][0]['column'];
						$dir = $params['order'][0]['dir'];
						$select->order(array($params['columns'][$i]['name'] . " " . $dir));
							
					}
                }
                else {
                    $selectC->join(array('tt' => new Expression('(SELECT s.timetable_id, s.`parent_timetable_id` FROM attd_timetable as s WHERE s.parent_timetable_id = '.$params['id'].')')), 'ta.timetable_id= tt.timetable_id', array());
                    $selectC->join(array('ts' => new Expression('(SELECT * FROM attd_timetable_shift AS ss WHERE CURDATE() <= ss.end_date OR ss.end_date is NULL)')), 'ta.timetable_id= ts.timetable_id', array());
                    
                    $arrColumn['ref_id'] = new \Zend\Db\Sql\Expression("DISTINCT ta.ref_id");
                }

                $selectC->columns(['total_count' => new \Zend\Db\Sql\Expression('COUNT(DISTINCT ta.ref_id)')]);
                
                $dataCount = $this->fetchRow($selectC);
                
                $total = ($dataCount['total_count'] > 0) ? $dataCount['total_count']:0;
                
                $result = ['data' => $dataAll, 'total' => $total, 'filtered' => $rows->total];
                
                return $result;
            }

            return false;
        }catch(\Exception $ex){
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
            return false;
        }
    }
    public function save(ObjectTimetable $object,$update = false){
        try{
            if($update){
                $update = $this->update();
                $update->table($this->_tables['attd_timetable_object']);
                $update->where($this->__(array('timetable_object_id' => $object->id)));
                $update->set($this->__($object->pull()));
                if($this->execute($update)){
                    return $object->id;
                }
            }else{
                $object->id = 0;
                $insert = $this->insert();
                $insert->into($this->_tables['attd_timetable_object']);
                $insert->autoincrement($this->__('timetable_object_id'));
                $insert->values($this->__($object->pull()));
                // echo('$insert=> '.str_replace('"','',$insert->getSqlString()));die();
                if($id = $this->execute($insert)){
                    return $id;
                }
            }
        }catch(\Exception $ex){
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine().' : '.$ex->getMessage());
            return false;
        }
    }
    public function load($id = null, $mode = \Neuron\Attendance\Timetable\Objects\Storage::LOADBY_ID , $params = array()){
        $select = $this->select();
        $select->from($this->_tables['attd_timetable_object']);

        $select->join(
            array('tt' => $this->_tables['attd_timetable']), 
            'attd_timetable_object.timetable_id = tt.timetable_id', 
            array('*')
        );
        switch ($mode) {
            case \Neuron\Attendance\Timetable\Objects\Storage::LOADBY_ID:
                $select->where($this->__(array('timetable_object_id' => $id)));
                break;
            case \Neuron\Attendance\Timetable\Objects\Storage::LOADBY_BETWEENDATE:
                $select->where($this->__(array(
                    'start_date >=\'' . $params['start_date'].' \' ',
                    'end_date <=\'' . $params['end_date'].' \' ',
                )));

                $select->where($this->__(array('ref_id' => $params['ref_id'])));
                break;
            case \Neuron\Attendance\Timetable\Objects\Storage::LOADBY_BETWEENDATE_REF_ID:
                $select->where("'".$params['start_date']."'".' BETWEEN attd_timetable_object.start_date and attd_timetable_object.end_date ');
                $select->where($this->__(array('ref_id' => $params['ref_id'])));
                break;
            case \Neuron\Attendance\Timetable\Objects\Storage::LOADBY_TIMETABLE_ID:
                $select->where($this->__(array('attd_timetable_object.timetable_id' => $id)));
                break;
            case \Neuron\Attendance\Timetable\Objects\Storage::LOADBY_TIMETABLE_ID_REF_ID:
                $select->where($this->__(array(
                    'attd_timetable_object.timetable_id'    => $params['timetable_id'],
                    'attd_timetable_object.ref_id'          => $params['ref_id']
                )));
                break;
            case \Neuron\Attendance\Timetable\Objects\Storage::LOADBY_REFID_DAY_NO:
                $select->join(
                    ['ats'=> $this->_tables['attd_timetable_shift']],
                    'ats.timetable_id = attd_timetable_object.timetable_id'
                );
                $select->where([
                    'ref_id'        => $params['ref_id'],
                    'day_no'        => $params['day_no']
                ]);
                if($params['end_date']){
                    $select->where->lessThanOrEqualTo($this->_('start_date','attd_timetable_object.'),$params['start_date']);
                    $select->where->nest()
                    ->greaterThanOrEqualTo($this->_('end_date','attd_timetable_object.'),$params['end_date'])->or
                    ->isNull($this->_("end_date",'attd_timetable_object.'))->unnest();
                }else{
                    $select->where->between(new Expression('DATE("'.$params['start_date'].'")'), new Expression('DATE(attd_timetable_object.start_date)'), new Expression('DATE(attd_timetable_object.end_date)'));
                }
                break;
                
        }

        try{
            $data = $this->fetchRow($select);
            return $data;
        }catch(\Exception $ex){
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
            return false;
        }
    }
    public function deleteRow($id = null ,$mode = ObjectTimetable\Storage::DELETEBY_ID){
        $delete = $this->delete();
        $delete->from($this->_tables['attd_timetable_object']);
        switch ($mode) {
            case \Neuron\Attendance\Timetable\Objects\Storage::DELETEBY_ID:
                $delete->where($this->__(array('timetable_object_id' => $id)));
                break;
        }
        try{
            if($this->execute($delete)){
                return $id;
            }
        }catch(\Exception $ex){
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
            return false;
        }
    }
    public function loadObjectByParam($params) {

        $select = $this->select();
        $select->from(['ta' => $this->_tables['attd_timetable_object']]);
        $select->join(['ts' => $this->_tables['attd_timetable_shift']], 'ta.timetable_id= ts.timetable_id', array('*'));
        $select->join(['tt' => $this->_tables['attd_timetable']], 'ta.timetable_id= tt.timetable_id', array('*'));

        if(isset($params)) {
            foreach($params as $key => $val) {
                 $select->where(array($key => $val));
            }
        }

        // echo str_replace('"','',$select->getSqlString())."<br />"; exit;

        $result = $this->fetchRow($select);
        return $result;
    }
    public function filterList($params = array()){
        // Base select
        $select = $this->select();
        $select->from(
            array('ta' => $this->_tables['attd_timetable_object'])
        );
        $select->join(array('ts' => $this->_tables['attd_timetable_shift']), 'ta.timetable_id= ts.timetable_id', array('day_no','is_workday','on_duty_time','off_duty_time', 'shift_start_date'=>'start_date', 'shift_end_date'=>'end_date'),'left');
        $select->join(array('tt' => $this->_tables['attd_timetable']), 'ta.timetable_id= tt.timetable_id', array('*'),'left');

        $select->where(array('ta.'.$params['filterby'].' like "%'. $params['filterquery'].'%"'));
        if($params['filterby']== 'ref_id' || $params['filterby'] == 'ref_name'){
            $select->group('ta.start_date');
        }

        $pager = new \Neuron\Db\Storage\Pager($params);
        $select->limit($pager->getLimit(),$pager->getOffset())->cache(0);
        // echo str_replace('"','',$select->getSqlString())."<br />"; exit;

        try{
            // echo('$select=> '.str_replace('"','',$select->getSqlString()));die();
            if($rows = $this->fetchAll($select)){
                $result = new \stdClass();
                $result->list = $rows;
                $result->pager = $pager;
                $result->countTotal = $rows->total;
                return $result;
            }

            return false;
        }catch(\Exception $ex){
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
            return false;
        }
    }
    public function saveGroup($data){
        try{
            $insert = $this->insert();
            $insert->into($this->_tables['attd_object_group_timetable']);
            $insert->autoincrement($this->__('id'));
            $insert->values($this->__($data));
            // echo('$insert=> '.str_replace('"','',$insert->getSqlString()));die();
            if($id = $this->execute($insert)){
                return $id;
            }
        }catch(\Exception $ex){
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine().' : '.$ex->getMessage());
            return false;
        }
    }
    public function listChangeShift($params){
        try {
            $select  = $this->select()
            ->from(array('ato'=>$this->_tables['attd_timetable_object']))
            ->join(array('att'=>$this->_tables['attd_timetable']), 'ato.timetable_id=att.timetable_id', array('*'), 'left')
            ->join(array('ats'=>$this->_tables['attd_timetable_shift']), 'att.timetable_id=ats.timetable_id', array('timetable_shift_id', 'day_no', 'is_workday', 'on_duty_time', 'off_duty_time'), 'left');
            if($params['filterquery']){
                if($params['filterby'] == 'start_date'){
                    $dayno = date('w', strtotime($params['filterquery'])); 
                    $select->where(
                        array(
                            $this->_('day_no','ats.') => $dayno+1
                        )
                    );
                    $select->where->lessThanOrEqualTo($this->_('start_date','ato.'),date("Y-m-d",strtotime($params['filterquery'])));

                    $nest   = $select->where->nest();
                        $nest->isNull(
                            $this->_('end_date','ato.')
                        )->or
                        ->greaterThanOrEqualTo(
                            $this->_('end_date','ato.'),
                            date("Y-m-d",strtotime($params['filterquery']))
                        );
                    $nest->unnest();
                }
            }
            
            /* Kalo class employee exists langsung join aja untuk dapetin ref_name nya supaya ga query 2x */
            if(class_exists('Neuron\Employee\Entity')){
                $select->join(
                    array('emp' => $this->_tables['employee']),
                    'emp.id = ato.ref_id',
                    array('employee_code' => 'code')
                )->join(
                    array('p' => $this->_tables['person']),
                    'emp.person_id = p.id',
                    array('ref_name' => new Expression("CONCAT(p.first_name,' ',p.last_name)"))
                );

                $select->where([
                    $this->_('status_id','emp.') => self::EMP_STATUS_ACTIVE
                ]);

                /* Untuk advance query berdasarkan name employee */
                if(isset($params['ref_name']) && strlen($params['ref_name']) > 0){
                    $select->where(
                        new Expression("CONCAT(p.first_name,' ',p.last_name) LIKE '%".$params['ref_name']."%'")
                    );
                }
            }
            

            $i = $params['order'][0]['column'];
			$dir = $params['order'][0]['dir'];
            $select->order(array($params['columns'][$i]['data'] . " " . $dir));

            $select->limit(intval($params['length']));
            $select->offset(intval($params['start']));
            
            $data = $this->fetchAll($select);
            
            $arr = array();
            $arrRefId  = array();
            if($data){
                foreach ($data as $val) {
                    $dayno = date('w', strtotime($params['filterquery']));
                    $arr[]            = $val;
                    if(!in_array($val['ref_id'], $arrRefId) && $val['day_no'] == $dayno+1){
                        $arrRefId[]     = $val['ref_id'];
                    }
                }
            }
            
            $selectCount    = $select;
            $selectCount->reset('limit')->reset('offset');
            $selectCount->reset(Select::COLUMNS);
            $selectCount->reset(Select::JOINS);
            $selectCount->from(array('ato'=>$this->_tables['attd_timetable_object']))
            ->join(array('att'=>$this->_tables['attd_timetable']), 'ato.timetable_id=att.timetable_id', array(), 'left')
            ->join(array('ats'=>$this->_tables['attd_timetable_shift']), 'att.timetable_id=ats.timetable_id', array(), 'left');
            $select->join(
                array('emp' => $this->_tables['employee']),
                'emp.id = ato.ref_id',
                array()
            )->join(
                array('p' => $this->_tables['person']),
                'emp.person_id = p.id',
                array()
            );

            /* Untuk advance query berdasarkan name employee */
            if(isset($params['ref_name']) && strlen($params['ref_name']) > 0){
                $select->where(
                    new Expression("CONCAT(p.first_name,' ',p.last_name) LIKE '%".$params['ref_name']."%'")
                );
            }
            
            $selectCount->reset(Select::ORDER);
            $selectCount->columns(array(
               'countTotal' => new Expression('COUNT(*)')
            ));

            $countTotal = $this->fetchRow($selectCount)['countTotal'];

            $retval = array('data'=>$arr, 'draw'=>@$params['draw'], 'recordsTotal'=>$countTotal, 'recordsFiltered'=>$countTotal);

            return $retval;
        } catch (\Exception $e) {
            return false;
        }
    }
    public function loadDatatable($params){
        $select         = $this->select();
        $select->from($this->_tables['attd_timetable_object']);
        $select->join(
            array('at' => $this->_tables['attd_timetable']), 
            'attd_timetable_object.timetable_id = at.timetable_id', 
            array('*')
        );

        if(isset($params['id'])){
            $timetable_id   = $params['id'];
            $select->where(array($this->_('parent_timetable_id') => $timetable_id));
        }

        $columns        = $params['columns'][$params['order'][0]['column']]['data'];
        $dir            = $params['order'][0]['dir'];

        // $select->order($columns.' '.$dir);

        $select->limit(intval($params['length']));
        $select->offset(intval($params['start']));

        $select->group($this->_('ref_id'));
        
        try {

            $fetch      = $this->fetchAll($select);

            if($fetch){
                $countTotal     = $select;
                $countTotal->reset('limit')->reset('offset');
                $countTotal->columns(array('countTotal' => new \Zend\Db\Sql\Expression("COUNT(*)")));
                $countTotal     = $this->fetchRow($countTotal)['countTotal'];

                $data           = array();
                foreach ($fetch as $row) {
                    $data[]     = $row;
                }

            }else{
                $countTotal     = 0;
                $data           = array();
            }

            $result         = new \stdClass;
            $result->data   = $data;
            $result->countTotal = intval($countTotal);
            return $result;
        } catch (Exception $e) {
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
            return false;
        }

    }

    public function listSetAdmin($timetable_ids,$params = array()){
        try {
            $timetable_id   = implode(',',$timetable_ids);
            $select         = $this->select();
            $select->from(array("ato" => $this->_tables['attd_timetable_object']));
            $select->where(new Expression("ato.timetable_id IN (".$timetable_id.")"));
            $select->where([
                $this->_('status_id','empl.') => 1
            ]);
            // $select->where("ato.start_date <= '".date("Y-m-d")."'");
            
            $select->where->lessThanOrEqualTo($this->_('start_date','ato.'),$this->now());

            $nest = $select->where->nest();
                $nest->greaterThanOrEqualTo($this->_('end_date','ato.'),$this->now())->or
                ->isNull($this->_('end_date','ato.'));
            $nest->unnest();
            // $select->where("(ato.end_date >= '".date("Y-m-d")."' OR ato.end_date IS NULL)");


            $select ->join(
                array('empl'    => $this->_tables['employee']),
                'ato.ref_id = empl.id',
                array(),
                'left'
            );

            $select->join(
                array('emp' => $this->_tables['person']),
                'empl.person_id = emp.id',
                array(
                    'ref_name' => new Expression('CONCAT(emp.first_name, " ",emp.last_name)')
                )
            );

            if(isset($params['draw'])){

                $select->limit(intval($params['length']));
                $select->offset(intval($params['start']));

                $columnName     = $params['columns'][ $params['order'][0]['column'] ]['data'];
                $dir            = $params['order'][0]['dir'];

                if($columnName == 'name'){
                    $columnName = 'ref_name';
                }

                $select->order($columnName.' '.$dir);

                $searchValue    = $params['search']['value'];   

                if(isset($searchValue) && !empty($searchValue)){
                    $nest = $select->where->nest();
                    $nest->like(new Expression("CONCAT(emp.first_name,' ',emp.last_name)"),'%'.$searchValue.'%')->or;
                    $nest->like("empl.code",'%'.$searchValue.'%');
                    $nest->unnest();
                }
            }
            // echo @str_replace('"','', $select->getSqlString());die();
            $data           = $this->fetchAll($select);

            
            $selectC            = $select;
            // $selectC->reset('limit')->reset('offset')->reset('columns')->reset('joins')->reset('order');
            // $select->join(
            //     array('emp' => $this->_tables['person']),
            //     'ato.ref_id = emp.id',
            //     array()
            // );
            // $selectC->columns(array(
            //     'countTotal'    => new Expression("COUNT(*)")
            // ));
            $selectC->reset('limit')->reset('offset');
			$joins = $selectC->joins->getJoins();
			$selectC->reset(\Zend\Db\Sql\Select::JOINS);
			foreach ($joins as $key => $join) {
			  //re add join without cols
			  $join['columns'] = [];
			  $type = $join["type"];
			  $selectC->join($join["name"], $join["on"], [], $type);
			}

			$selectC->reset(Select::ORDER);
			
			$selectC->columns(array('countTotal' => new Expression("COUNT(*)")));
            // echo str_replace('"','',$select->getSqlString());die();
            $countTotal     = $this->fetchRow($selectC)['countTotal'];
            if(!$countTotal){
                $countTotal     = 0;
            }
            

            $result                 = new \stdClass;
            $result->data           = $data;
            $result->countTotal     = $countTotal;
            
            return $result;
        } catch (\Throwable $th) {
            var_dump($th->getMessage());die('');
        }
    }

    /* list data */
    public function listData($ids, $mode = Storage::LIST_ALL, $params = null){
        /* base select */
        $select = $this->select();
        $select->from(
            array(
                'ato'   => $this->_tables['attd_timetable_object']
            )
        );
        $select->join(
            array(
                'at'    => $this->_tables['attd_timetable']
            ),
            'at.timetable_id = ato.timetable_id'
        );
        $select->join(
            array(
                'ats'   => $this->_tables['attd_timetable_shift'],
            ),
            'ats.timetable_id = at.timetable_id',
            array('timetable_shift_id','day_no', 'is_workday', 'on_duty_time', 'off_duty_time', 'in_beginning_time', 'in_ending_time', 'out_beginning_time', 'out_ending_time', 'late_time', 'leave_early_time', 'overtime_start_time', 'ats_start_date' => 'start_date', 'ats_end_date' => 'end_date')
        );

        /* loadby page */
        if ($mode === Objects\Storage::LISTBY_PAGE) {
            /* get pager */
            $pager = Pager::sanitize($params);
            $select->limit($pager->getLimit())->offset($pager->getOffset())->cache(0);

            /* has keyword? */
            if (!empty($ids)) {
                /* Query where disini */
            }

            $select->order($params['order']['column'].' '.$params['order']['dir']);
        } else{
            $pager = new \Neuron\Db\Storage\Pager(1, 999999);

            switch($mode){
                case Objects\Storage::LISTBY_REF_ID:
                    $select->where(array(
                        $this->_('ref_id','ato.') => $ids
                    ));
                break;
                case Objects\Storage::LISTBY_TIMETABLEID:
                    $select->where(array(
                        $this->_('timetable_id','ato.') => $ids
                    ));
                break;
            }
            
            /* Logic kalo ada start_date dan end_datenya di array $params*/
            if(isset($params['start_date']) && !empty($params['start_date'])){
                $nest   = $select->where->nest();
                    $nest->greaterThanOrEqualTo(
                        $this->_('start_date','ato.'),
                        $params['start_date']
                    )->and;
                    $secondNest     = $nest->nest();
                        $secondNest->lessThanOrEqualTo(
                            $this->_('end_date','ato.'),
                            $params['end_date']
                        )->or->isNull($this->_('end_date','ato.'));
                    $secondNest->unnest();
                $nest->unnest();
            }

        }
        // echo(str_replace('"','',$select->getSqlString()));die();
        try {
            $rows = $this->fetchAll($select);
            $result = new \stdClass();
            $result->list = $rows;
            $result->pager = $pager;
            $total = isset($rows->total) ? $rows->total : 0;
            $result->countTotal = $total;
            return $result;
        } catch (\Throwable $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }

    public function  listDataShift($params){
        $select  = $this->select()
            ->from(array('ato'=>$this->_tables['attd_timetable_object']))
            ->join(array('ats'=>$this->_tables['attd_timetable_shift']), 'ato.timetable_id=ats.timetable_id', array('day_no','on_duty_time', 'off_duty_time'), 'left')
            ->where(array($this->_('ato.ref_id') => $params['ref_id']));

        if(isset($params['start_date']) && $params['start_date']){
            $select->where->greaterThanOrEqualTo($this->_('start_date','ato.'),$params['start_date']);
        }
        
        $data           = $this->fetchAll($select);
         //echo(str_replace('"','',$select->getSqlString()));die();
        $result                 = new \stdClass;
        $result->data           = $data;        
        return $result;
    }

}