<?php

namespace Neuron\Attendance\Timetable\Storage;
use Zend\Db\Sql\Predicate\Expression;
use Neuron\Attendance\Timetable;

class Mysql extends \Neuron\Db\Storage implements Skeleton {

	public function listdata($params = null) {
		$select = $this->select()
			->from(array('l' => $this->_tables['attd_timetable']))
			->where(array('l.parent_timetable_id is null'));

		// dilimit kalo asalnya dari datatable
		if(isset($params['draw'])){	
			$select->limit(intval($params['length']));
			$select->offset(intval($params['start']));

			// searching
				if($params['search']['value']){
					// searching disini
					$select->where(
						'l.parent_timetable_id is NULL AND 
						(
							LOWER(code) LIKE \'%'.strtolower($params['search']['value']).'%\' OR
							LOWER(name) LIKE \'%'.strtolower($params['search']['value']).'%\'
						)'
					);
					// echo('$select=> '.str_replace('"','',$select->getSqlString()));die();
				}

			// ordering
				$column = $params['order'][0]['column'];
				$columnName = $params['columns'][$column]['data'];
				$dir = $params['order'][0]['dir'];

				$select->order($columnName.' '.$dir);
		}

		$data = $this->fetchAll($select);

		$dataAll = [];

		if($data) {
			foreach($data as $row) {
				array_push($dataAll, $row);
			}
		}

		$selectC 	= $select;
		$selectC->reset('limit')->reset('offset');
		$selectC->columns(array('total_count' => new \Zend\Db\Sql\Expression('COUNT(*)')));
		$dataCount = $this->fetchRow($selectC);

		return array(
			'dataAll' => $dataAll,
			'dataCount' => $dataCount,
		);
	}

	public function load($id = null, $mode = Timetable\Storage::LOADBY_ID,$params = array()) {
		$select = $this->select()
			->from(array('lt' => $this->_tables['attd_timetable']));

		switch ($mode) {
			case Timetable\Storage::LOADBY_ID:
				$select->where(array($this->_('timetable_id')  => $id));
				break;
			case Timetable\Storage::LOADBY_NAME:
				$select->where(array($this->_('name') => $id));
				break;
			case Timetable\Storage::LOADBY_CODE:
				$select->where(array($this->_('code') => $id));
				break;
			case Timetable\Storage::LOADBY_PARENT_DAYNO:
				$select->where(array($this->_('parent_timetable_id') => $id));
				$select->where(new Expression("code LIKE '%".$params['day_no']."%'"));
			break;
		}

		try {
			
			$result = $this->fetchRow($select);
			return $result;
		} catch (\Exception $ex) {
			return false;
			error_log('Error occured from :'.$ex->getFile().' Line : '.$ex->getLine().' :: '.$ex->getMessage());
		}
	}

	public function save(\Neuron\Attendance\Timetable $model, $update = false) {
		$model->time_mode 	= "WEEKLY";
		$data = $model->pull();

		unset($data['day_no']);

		if ($update) {
			$update = $this->update()
				->table($this->_tables['attd_timetable'])
				->set($this->__($data))
				->where($this->__(array('timetable_id' => $model->timetable_id)));

			if ($this->execute($update)) {
				return $model->timetable_id;
			}
		}
		else {
			$data['timetable_id']	= null;
			
			$insert = $this->insert()
				->into($this->_tables['attd_timetable'])
				->values($this->__($data));

			$save = $this->execute($insert);
			
			if ($save) {
				$model->timetable_id = $save;
				return $model->timetable_id;
			}
		}

		return false;
	}

	public function searchByCode($code) {
		$select = $this->select()
			->from($this->_tables['attd_timetable'])
			->where(array('code' => $code));

		// echo $select->getSqlString() die();
		$result = $this->fetchRow($select);
		// print_r($result); die();
		return $result;
	}

	public function removeTimetable($timetable_id) {

        try {
			/*Delete timetable object*/
			$deleteObject 	= $this->delete()->from($this->_tables['attd_timetable_object']);
			$deleteObject->where(array('timetable_id' => $timetable_id));
			$this->execute($deleteObject);
			// echo('$deleteObject=> '.str_replace('"','',$deleteObject->getSqlString()));die();


			$deleteTT = $this->delete()
			->from($this->_tables['attd_timetable'])
			->where(array('timetable_id' => $timetable_id));

			return $this->execute($deleteTT);
        	
        }catch (Exception $ex) {
            return false;
        }  
    }

    public function loadTimetable($timetable_id){
    	try {
	    	$select = $this->select()
	    				->from($this->_tables['attd_timetable'])
	    				->where(array('timetable_id'=>$timetable_id));
			return $this->fetchRow($select);
    	} catch (Exception $e) {
    		return false;
    	}

    }

    public function loadList($id = null, $mode = \Neuron\Attendance\Timetable\Storage::LIST_ALL,$params = array()){
        // Base select
        $select = $this->select();
        $select->from(
            array('t' => $this->_tables['attd_timetable'])
		);

		$select->join(
			array(
				'tj' => new Expression('(SELECT* FROM attd_timetable_shift AS ats WHERE CURDATE() <= ats.end_date  OR ats.end_date is null)')
			), 
			't.timetable_id = tj.timetable_id', 
			array('*'),
			'LEFT'
		);
		
        if($mode == \Neuron\Attendance\Timetable\Storage::LISTBY_PAGE){
            $pager = new \Neuron\Db\Storage\Pager($params);
            $select->limit($pager->getLimit(),$pager->getOffset())->cache(0);
        }else{
            $pager = new \Neuron\Db\Storage\Pager(1,99999999);
            if(isset($params['page']) && isset($params['records'])){
                $pager = new \Neuron\Db\Storage\Pager($params);
                $select->limit($pager->getLimit(),$pager->getOffset())->cache(0);
            }
            switch ($mode) {
                case \Neuron\Attendance\Timetable\Storage::LISTBY_ID:
                    $select->where($this->__(array('timetable_id' => $id)));
                    break;
                case \Neuron\Attendance\Timetable\Storage::LISTBY_PARENT:
                    $select->where($this->__(array('parent_timetable_id' => $id)));
                    break;
            }
        }

        try{
			// echo @str_replace('"','', $select->getSqlString());die();
            if($rows = $this->fetchAll($select)){
                $result = new \stdClass();
                $result->list = $rows;
                $result->pager = $pager;
                $result->countTotal = $rows->total;
                return $result;
            }
            return false;
        }catch(\Exception $ex){
            error_log('Error at : '.$ex->getFile().' Line : '.$ex->getLine(). ' : '. $ex->getMessage());
            return false;
        }
    }
    public function loadChild($parent_id){
    	try {
	    	$select = $this->select()
	    				->from($this->_tables['attd_timetable'])
	    				->where(array('parent_timetable_id'=>$parent_id));
	    	// echo('$select=> '.str_replace('"','',$select->getSqlString()));die();
			return $this->fetchAll($select);
    	} catch (Exception $e) {
    		return false;
    	}
    }
}