<?php 
namespace Neuron\Attendance\Timetable;

use Neuron\Generic\Entity\Map;
use Neuron\Generic\Result;
use Neuron\Attendance\Timetable\Upload\Detail as mTempDetail;

class Upload extends \Neuron\Generic\Entity\Mapper {
    protected $_storage;
    protected $_loaded = false;

    const ERROR_PREFIX = 'Attendance';
    const DUMMY_SESSOWNER = 10000;

	public function __construct(\Neuron\Attendance\Timetable\Upload\Storage\Skeleton $storage, $data= array()) {
			
        $this->_storage = $storage;

		parent::__construct(
	        $data, 
	        array(
                new Map('upload_file_shift_id'),
                new Map('file_name'),
                new Map('file_path'),
                new Map('total_data'),
                new Map('total_success'),
                new Map('total_failed'),
                new Map('status'),
                new Map('upload_dtm'),
                new Map('upload_by'),
                new Map('failed', null, true), 
                new Map('success', null, true), 
                
	        ), 
	        parent::CASE_INSENSITIVE
	    );
    }

    public function load($id, $mode = \Neuron\Attendance\Timetable\Upload\Storage::LOADBY_ID){
        /* load dari storage */
        if ($data = $this->_storage->load($id, $mode)) {

            /* load sukses, set data properti class dari hasil query */
            $this->push($data);

            /* set loaded */
            $this->_loaded = true;
            return true;

        } else {
            $this->_loaded = false;
        }
        return false;
    }

    public function save($update = false)
    {
        /* simpan via storage */
        if ($id = $this->_storage->save($this, $update)) {

            /* simpan sukses, ambil id yg tersimpan, return true */
            $this->upload_file_shift_id = $id;
            return true;
        }
        return false;
    }

    public function create($update)
    {
        $result = new Result();
        if($this->save($update)){
            $result->info = 'Success save upload file';
            $result->data = $this->upload_file_shift_id;
        }else{
            $result->code  = self::ERROR_PREFIX."311";
            $result->info = 'Failed save upload file';
            $result->data = null;
        }
        return $result;
    }

    public function remove($id)
    {
        $result = new Result();
        if($this->_storage->remove($id)){
            $result->info = 'Success delete data upload';
            $result->data = $id;
        }else{
            $result->code  = self::ERROR_PREFIX."312";
            $result->info = 'Failed terminate tax config';
            $result->data = null;
        }
        return $result;
    }

    public function upload($post,$sessOwner){
		$result 		= new Result();
		if(!isset($post['file'])){
			$result->code 	= self::ERROR_PREFIX.'1';
            $result->info 	= "No file Uploaded!";
            return $result;
        }
        
        $files 			= $post['file'];
        if(class_exists('Neuron\Application\Framework\Entity')){
            $naf        = \Neuron\Application\Framework\Entity::get();
            $sessOwner  = $naf->session()->owner();
        }else{
            $sessOwner  = self::DUMMY_SESSOWNER;
        }

		/* Validating File type */
		$type 			= $files['type'];
		if($type !== 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'){
            /* Kalo bukan bawaan dari format gagal */
			$result->code 	= self::ERROR_PREFIX.'2';
			$result->info 	= 'Invalid file format';
		}else{
            /* Move to /temp folder */
            $files['name'] 	= str_replace(" ",'_',$files['name']);
            
            $fileloc = getcwd().'/public/attendance/uploadshift/';
            if(!file_exists($fileloc)) @mkdir($fileloc, 0775, true);
            
            $fileName 	= $fileloc.date('Ymdhis').'_'.$files['name'];
			try {
                $upload 	= move_uploaded_file($files['tmp_name'], $fileName);
				if(!$upload){
					$result->code = self::ERROR_PREFIX.'3';
					$result->info = 'Failed Uploading file';
				}else{
                    /* Load file excelnya */
					$spreadsheet 	= \PhpOffice\PhpSpreadsheet\IOFactory::load($fileName);

					// $rows  = yang menyimpen row dari spreadsheets
					$rows 			= [];
					$rowIterator 	= $spreadsheet->getActiveSheet()->getRowIterator();
					
					// Isi $rows dengan rows dari spreadsheet yang di upload, karena dari spreadsheet bentuknya ga langsung array
					foreach($rowIterator as $row){
						$cellIterator = $row->getCellIterator();
						foreach ($cellIterator as $cell) {
							if($cell->getCalculatedValue()){
								$rows[$row->getRowIndex()][$cell->getColumn()] = $cell->getFormattedValue();
							}
						}
                    }

                    //get data tanggalnya
                    $headerExcel = $rows[3];

                    // get data valuenya
                    $newArray = $this->valueExcel($rows, 3);

                    $this->file_name = $files['name'];
                    $this->file_path = $fileloc;
                    $this->total_data =  count($newArray);
                    $this->total_success = 0;
                    $this->total_failed = 0;
                    $this->status = 0;
                    $this->upload_dtm = $this->_storage->now();
                    $this->upload_by = $sessOwner;

                    $saveUpload = $this->create(false);

                    if($saveUpload->code != 0){
                        return $saveUpload;
                    }

                    if(count($rows) <= 0){
                        $result->code   = self::ERROR_PREFIX.'4';
                        $result->info   = "No data uploaded. No data in sheets.";
                        return $result;
                    }

					// Upload data melalui modul person
					$personToSave 	= [];
					$listFailed 	= array();
                    $notEmptyValidator  = new \Zend\Validator\NotEmpty();

					foreach ($newArray as $key => $row) {
                        // Validate row digunakan untuk memastikan setiap baris memiliki row `a` s/d `j
                        if($headerExcel){
                            foreach ($headerExcel as $key2 => $value) {
                                if($row[$key2] != null  && $row[$key2] != ''){
                                    $mTempDetail = new mTempDetail(mTempDetail\Storage::factory($this->_storage->getAdapter(), $this->_storage->getConfig()));
    
                                    $mTempDetail->upload_file_shift_id = $saveUpload->data;
                                    $mTempDetail->employee_code = trim($row['B']);
                                    $mTempDetail->name = $row['C'];
                                    $mTempDetail->timetable_code = trim($row[$key2]);
                                    $mTempDetail->date = date('Y-m-d', strtotime($value));
                                    $mTempDetail->status = 0;
                                    $mTempDetail->schedule_start = $this->_storage->now();
                                    $mTempDetail->schedule_end = null;
                                    
                                    $saveTempDetails = $mTempDetail->create(false);
                                    if($saveTempDetails->code != 0){
                                        //delete data detail yang sudah di insert
                                        $mTempDetail->remove($saveUpload->data,  \Neuron\Attendance\Timetable\Upload\Detail\Storage::DELETEBY_UPLOAD);
    
                                        //delete data payroll yang sudah di insert
                                        $this->remove($saveUpload->data);
                                        return $saveTempDetails;
                                    }
                                }
                            }
                        }			
					}
				}

				$result->code 	= 0;
				$result->data 	= $listFailed;
                
                
                return $result;
			} catch (\Exception $ex) {
                // throw $ex;
                // die;
                
				error_log("Error ::".$ex->getMessage().' file :: '.$ex->getFile().' line :: '.$ex->getLine());
				$result->code = 'EMP-2';
				$result->info = 'Failed Uploading file';
			}
		}

		return $result;
    }

    function valueExcel($array, $index)
    {
        $count = count($array);
        foreach ($array as $key => $row) {
            unset($array[$key]);
            if($key === $index) break;
        }
        $newArray = array_values($array);
        return $newArray;
    }
}