<?php 

namespace Neuron\Attendance\Timetable\Upload\Detail\Storage;
use Zend\Db\Sql\Predicate\Expression;
class Mysql extends \Neuron\Db\Storage implements Skeleton {
    const CONFIG_CODE = 'ATTENDANCE';

    public function __construct(\Zend\Db\Adapter\Adapter $adapter, $config = array()) {

        /* construct parent --> Neuron\Db\Storage */
        parent::__construct($adapter, $config);
    }

    public function load($id, $mode = \Neuron\Attendance\Timetable\Upload\Detail\Storage::LOADBY_ID) {
        
        /** kalau modif fungsi ini, silakan modif juga versi oracle dan versi postgre nya yah*/
        
        try {
            $select = $this->select()
                ->from(array('pc' => $this->_tables['attd_upload_shift_detail']));
            if($mode == \Neuron\Attendance\Timetable\Upload\Detail\Storage::LOADBY_ID){
                $select->where($this->__(array('pc.detail_id' => $id)));
            } 
            return $this->fetchRow($select);
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }

    public function loadList($ids, $mode = \Neuron\Attendance\Timetable\Upload\Detail\Storage::LIST_ALL, $params = null) {
        /* base select */
        $select = $this->select()
        ->from(array('pc' => $this->_tables['attd_upload_shift_detail']));
        
        if(isset($params['filter_status']) && !empty($params['filter_status'])){
            if($params['filter_status'] == -2){
                $select->where(new Expression('pc.status IN (3,4,5)'));
            }else{
                $select->where(array('status'=> $params['filter_status']));
            }
        }
        
        if(isset($params['upload_file_shift_id']) && ($params['upload_file_shift_id'] != '' || $params['upload_file_shift_id']  != null )){
            $select->where(array('upload_file_shift_id'=> $params['upload_file_shift_id']));
        }
        /* loadby page */
        if ($mode == \Neuron\Attendance\Timetable\Upload\Detail\Storage::LISTBY_PAGE) {
            /* order */
            if(array_key_exists('order', $params) && array_key_exists('columns', $params)){
                foreach($params['order'] as $order){
                    $columnName = $params['columns'][$order['column']]['data'];
                    $colIdentity = $this->_($columnName, 'pc.');
                    $select->order($colIdentity . ' ' . $order['dir']);
                }
            }

            /* has keyword? */
            if (!empty($ids)) {
                $select->where->nest()
                    ->like($this->_('name', 'pc.'), '%' . trim($ids) . '%')
                ->unnest();
            }
            /* get pager */
            $pager = \Neuron\Db\Storage\Pager::sanitize($params);
            $select->limit($pager->getLimit())->offset($pager->getOffset())->cache(0);

        } else {
            $pager = new \Neuron\Db\Storage\Pager(0, 999999);

           

        }
        try {
            // echo str_replace('"','',$select->getSqlString()).'<br />';  
            
            if($rows = $this->fetchAll($select)){
                $result = new \stdClass();
                $result->list = $rows;
                $result->pager = $pager;
                $result->countTotal = $rows->total;
                return $result;
            }
            return false;
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }

    public function save(\Neuron\Attendance\Timetable\Upload\Detail $period,$update = false)
    {
        try {
            /* mode update? */
            if ($update == true) {

                $update = $this->update()
                        ->table($this->_tables['attd_upload_shift_detail'])
                        ->set($this->__($period->pull()))
                        ->where($this->__(array('detail_id' => $period->detail_id)));
                if ($this->execute($update)) {

                    /* update sukses, return user id */
                    return $period->detail_id;
                }
            } else {
                $period->detail_id = null ;
                $insert = $this->insert()
                        ->into($this->_tables['attd_upload_shift_detail'])
                        ->autoincrement($this->_('detail_id'))
                        ->values($this->__($period->pull()));
                
                if ($id = $this->execute($insert)) {

                    /* insert sukses, return user id */
                    return $id;
                }
            }

            /* default gagal */
            return false;
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }   
    }

    public function terminate($id)
    {
        try {
            $update = $this->update()
            ->table($this->_tables['attd_upload_shift_detail'])
            ->set(array('end_date'=> new \Zend\Db\Sql\Predicate\Expression('NOW()'),'last_update_dtm'=> new \Zend\Db\Sql\Predicate\Expression('NOW()')))
            ->where($this->__(array('detail_id' => $id)));
            
            if ($this->execute($update)) {
                /* update sukses, return instance id */
                return $id;
            }
            return false;
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }

    public function remove($id, $mode = \Neuron\Attendance\Timetable\Upload\Detail\Storage::DELETEBY_ID)
    {
        try {
            $delete = $this->delete()
            ->from($this->_tables['attd_upload_shift_detail']);
            if($mode == \Neuron\Attendance\Timetable\Upload\Detail\Storage::DELETEBY_ID){
                $delete->where($this->__(array('detail_id' => $id)));
            }else if($mode == \Neuron\Attendance\Timetable\Upload\Detail\Storage::DELETEBY_UPLOAD){
                $delete->where($this->__(array('upload_file_shift_id' => $id)));
            }
            
            if($this->execute($delete)){
                return $id;
            }
            return false;
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }

    public function count($params = array())
    {
        $select = $this->select()
        ->columns(array(new Expression('count(*)')))
        ->from(array('pc' => $this->_tables['attd_upload_shift_detail']));

        if(isset($params['filter_status']) && !empty($params['filter_status'])){  
            $select->where(array('status'=> $params['filter_status']));
        }

        if(isset($params['upload_file_shift_id']) && ($params['upload_file_shift_id'] != '' || $params['upload_file_shift_id']  != null )){
            $select->where(array('upload_file_shift_id'=> $params['upload_file_shift_id']));
        }

        try {
            return $this->fetchValue($select);
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }
    
}