<?php 

namespace Neuron\Attendance\Timetable\Upload\Storage;

class Mysql extends \Neuron\Db\Storage implements Skeleton {
    const CONFIG_CODE = 'PAYROLL';

    public function __construct(\Zend\Db\Adapter\Adapter $adapter, $config = array()) {

        /* construct parent --> Neuron\Db\Storage */
        parent::__construct($adapter, $config);
    }

    public function load($id, $mode = \Neuron\Attendance\Timetable\Upload\Storage::LOADBY_ID) {
        
        /** kalau modif fungsi ini, silakan modif juga versi oracle dan versi postgre nya yah*/
        try {
            $select = $this->select()
                ->from(array('pc' => $this->_tables['attd_upload_shift']));
            if($mode == \Neuron\Attendance\Timetable\Upload\Storage::LOADBY_ID){
                $select->where($this->__(array('pc.upload_file_shift_id' => $id)));
            } 
            return $this->fetchRow($select);
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }

    public function loadList($ids, $mode = \Neuron\Attendance\Timetable\Upload\Storage::LIST_ALL, $params = null) {
        /* base select */
        $select = $this->select()
        ->from(array('pc' => $this->_tables['attd_upload_shift']));
        
        if(isset($params['filter_status']) && !empty($params['filter_status'])){
            $select->where(array('status'=> $params['filter_status']));
        }

        /* loadby page */
        if ($mode == \Neuron\Attendance\Timetable\Upload\Storage::LISTBY_PAGE) {
            /* order */
            if(array_key_exists('order', $params) && array_key_exists('columns', $params)){
                if(is_array($params['order'])){
                    foreach($params['order'] as $order){
                        $columnName = $params['columns'][$order['column']]['data'];
                        $colIdentity = $this->_($columnName, 'pc.');
                    }
                }
            }
            // $select->order($colIdentity . ' ' . $order['dir']);
            
            /* has keyword? */
            if (!empty($ids)) {
                $select->where->nest()
                    ->like($this->_('file_name', 'pc.'), '%' . trim($ids) . '%')
                ->unnest();
            }
            /* get pager */
            $pager = \Neuron\Db\Storage\Pager::sanitize($params);
            $select->limit($pager->getLimit())->offset($pager->getOffset())->cache(0);

        } else {
            $pager = new \Neuron\Db\Storage\Pager(0, 999999);

            switch ($mode) {
                case \Neuron\Attendance\Timetable\Upload\Storage::LISTBY_PARENT:
                    $select->where(array($this->_('parent_id') => $ids));
                    break;
                case \Neuron\Attendance\Timetable\Upload\Storage::LISTBY_TYPE:
                    $select->where(array($this->_('period_type_id') => $ids));
                    break;
                default:
            }

        }
        $select->order('upload_dtm DESC');
        try {
            // echo str_replace('"','',$select->getSqlString()).'<br />';  die();
            
            if($rows = $this->fetchAll($select)){
                $result = new \stdClass();
                $result->list = $rows;
                $result->pager = $pager;
                $result->countTotal = $rows->total;
                return $result;
            }
            return false;
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }

    public function save(\Neuron\Attendance\Timetable\Upload $period, $update = false)
    {
        try {
            /* mode update? */
            if ($update == true) {

                $update = $this->update()
                        ->table($this->_tables['attd_upload_shift'])
                        ->set($this->__($period->pull()))
                        ->where($this->__(array('upload_file_shift_id' => $period->upload_file_shift_id)));
                if ($this->execute($update)) {

                    /* update sukses, return user id */
                    return $period->upload_file_shift_id;
                }
            } else {
                $period->upload_file_shift_id = null;
                $insert = $this->insert()
                        ->into($this->_tables['attd_upload_shift'])
                        ->autoincrement($this->_('upload_file_shift_id'))
                        ->values($this->__($period->pull()));
                
                if ($id = $this->execute($insert)) {

                    /* insert sukses, return user id */
                    return $id;
                }
            }

            /* default gagal */
            return false;
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }   
    }

    public function terminate($id)
    {
        try {
            $update = $this->update()
            ->table($this->_tables['attd_upload_shift'])
            ->set(array('end_date'=> new \Zend\Db\Sql\Predicate\Expression('NOW()'),'last_update_dtm'=> new \Zend\Db\Sql\Predicate\Expression('NOW()')))
            ->where($this->__(array('upload_file_shift_id' => $id)));
            
            if ($this->execute($update)) {
                /* update sukses, return instance id */
                return $id;
            }
            return false;
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }

    public function remove($id)
    {
        try {
            $delete = $this->delete()
            ->from($this->_tables['attd_upload_shift'])
            ->where($this->__(array('upload_file_shift_id' => $id)));
            
            if($this->execute($delete)){
                return $id;
            }
            return false;
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }
}