<?php
namespace Neuron\Attendance\v2\Timetable\Obj\Storage;

use Neuron\Db\Storage\Pager;
use Neuron\Attendance\v2\Timetable\Obj;
use Neuron\Attendance\v2\Timetable\Obj\Storage;
use Zend\Db\Adapter\Adapter;
use Zend\Stdlib\ArrayUtils;
use Throwable;
use Neuron\Db\Sql\Filsort;


class Mysql extends \Neuron\Db\Storage implements Skeleton
{
    public function load($id, $mode = Storage::LOADBY_ID)
    {
        try {
            $select = $this->select()
                ->from($this->_tables['attd_timetable_object']);

            switch ($mode) {
                case Storage::LOADBY_ID:
                    $select->where($this->__(['' => $id]));
                    break;
                default:
                    $select->where($this->__(['timetable_object_id' => $id]));
            }

            return $this->fetchRow($select);
        } catch (Throwable $throwable) {
            error_log('Error at ' . $throwable->getFile() . ' line ' . $throwable->getLine() . ': ' . $throwable->getMessage());
            return false;
        }
    }

    public function save(Obj $model, $update = false){
        try {
            /* mode update? */
            if ($update === true) {
                $update = $this->update()
                    ->table($this->_tables['attd_timetable_object'])
                    ->set($this->__($model->pull()))
                    ->where($this->__(['model_id' => $model->id]));
                if ($this->execute($update)) {
                    /* update sukses, return model id */
                    return $model->id;
                }
            } else {
                $insert = $this->insert()
                    ->into($this->_tables['attd_timetable_object'])
                    ->autoincrement($this->_('timetable_object_id'))
                    ->values($this->__($model->pull()));

                if ($id = $this->execute($insert)) {
                    /* insert sukses, return model id */
                    return $id;
                }
            }

            /* default gagal */
            return false;
        } catch (Throwable $throwable) {
            error_log('Error at ' . $throwable->getFile() . ' line ' . $throwable->getLine() . ': ' . $throwable->getMessage());
            return false;
        }
    }

    public function loadList($ids, $mode = Storage::LIST_ALL, $params = null){
        
        $filter = Filsort::sanitizeFilter($params);
        
        /* base select */
        $select = $this->select()
            ->from(['ato' => $this->_tables['attd_timetable_object']]);

        // add join query, dibuat fungsi biar bisa dipake sama load satuan
        $select = $this->addDefaultJoin($select);

        // tambhain default load yang aktif aja.
        $select->where->lessThanOrEqualTo($this->_('start_date','ato.'),date('Y-m-d'));
        $select->where->nest()
        ->greaterThanOrEqualTo($this->_("end_date",'ato.'),date('Y-m-d'))->or
        ->isNull($this->_("end_date",'ato.'))->unnest();
        
        
        /* loadby page */
        if ($mode === Storage::LOADBY_PAGE) {
            /* get pager */
            $pager = Pager::sanitize($params);
            $select->limit($pager->getLimit())->offset($pager->getOffset())->cache(0);

            /* has keyword? */
            if (!empty($ids)) {
                /* Todo :: search query */
            }

        } else {
            $pager = new \Neuron\Db\Storage\Pager(1, 999999);

            switch ($mode) {
                case Storage::LISTBY_REFID:
                    $select->where([$this->_('ref_id', 'ato.') => $ids]);
                    break;
            }
        }

        $select->filter(['ato.' => $this->__($filter)]);

        try {
            // echo @str_replace('"','', $select->getSqlString());die();
            $rows = $this->fetchAll($select);
            $result = new \stdClass();
            $result->list = $rows;
            $result->pager = $pager;
            $total = isset($rows->total) ? $rows->total : 0;
            $result->countTotal = $total;
            return $result;
        } catch (\Throwable $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }

    private function addDefaultJoin($select){
        $select->join(
            ['at' => $this->_tables['attd_timetable']],
            'ato.timetable_id = at.timetable_id',
            array(
                'at_timetable_id' => 'timetable_id',
                'at_code' => 'code',
                'at_parent_timetable_id' => 'parent_timetable_id',
                'at_time_mode' => 'time_mode',
                'at_name' => 'name',
                'at_description' => 'description',
            )
        );

        $select->join(
            ['ats' => $this->_tables['attd_timetable_shift']],
            'ats.timetable_id = at.timetable_id',
            array(
                "ats_timetable_shift_id" => "timetable_shift_id",
                "ats_timetable_id" => "timetable_id",
                "ats_day_no" => "day_no",
                "ats_is_workday" => "is_workday",
                "ats_on_duty_time" => "on_duty_time",
                "ats_off_duty_time" => "off_duty_time",
                "ats_in_beginning_time" => "in_beginning_time",
                "ats_in_ending_time" => "in_ending_time",
                "ats_out_beginning_time" => "out_beginning_time",
                "ats_out_ending_time" => "out_ending_time",
                "ats_late_time" => "late_time",
                "ats_leave_early_time" => "leave_early_time",
                "ats_overtime_start_time" => "overtime_start_time",
                "ats_start_date" => "start_date",
                "ats_end_date" => "end_date",
            )
        );

        return $select;
    }
}
